// Global Header file of "NucleoSynth-v3.9" **********************************
// digital Audio Synthesizer based on STM32 Nucleo F446
// Author and (c)2019-22 Wolfgang Schemmert

// The copyright claim and GPL-3 License is restricted to this file and the
// depending files
// "main.c", "main.h, "comm.c", "panel.c", "filter.c", "dacfreq.c","synthi.c",
// "drums.c" and "usb-otg-4611.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org

// Contact: www.midi-and-more.de
// A detailed hardware diy construction manual and a user operating manual
// is provided at this website

// for the IIR filter algorithms used here, see detailled description:
// www.IowaHills.com/A4IIRBilinearTransform.html
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

// ##############################################################

#define CPU_446
//System CLock is 168MHz
#define HARDWARE_VERSION 3	//2022: redesign Nucleo baseboard and operator panel
#define REVNUM 9

//#define MINHW  //***********************************************************
//special mode for minimal hardware configuration with unmodified Nucleo module
//serial communication takes place via ST-Link/USB
//USART2 (PortA2,A3) is activated additionally, always 115.2 kBaud
//USART2 replaces USB-CDC communication, is always COM
//USART1 always MIDI IN
//F446 Kernel USB is OFF

#ifndef MINHW
//only one option allowed !!
#define CUT_OFF//modif.Nucleo:ST-LINK cut off,8MHz crystal Osc,PA2,3->Morpho
//#define ORIG  //special version:unmodified Nucleo,but HW with operator panel
				  //RS-232 communication takes place via ST-Link/USB
				  //USART2 (PortA2,A3) is activated, always 115.2 kBaud
				  //USART1 is always MIDI IN
				  //STM32-F446 USB can be configured as virtual COM or MIDI
#endif
//  **************************************************************************

#define SECTOR7 (0x08060000)

//the device will appear as "STMicroelectronics Virtual Com Port"
#define DEFAULT_VIDH	0x04
#define DEFAULT_VIDL	0x83
#define DEFAULT_VID		((DEFAULT_VIDH << 8) | DEFAULT_VIDL)
#define DEFAULT_PIDH	0x57
#define DEFAULT_PIDL	0x40
#define DEFAULT_PID		((DEFAULT_PIDH << 8) | DEFAULT_PIDL)

#define N_SOUNDS    60  //number of user sound sets. Must be a multiple of 4
#define N_USERWAVES  4  //number of uploadable user waves
#define N_VOICES    10
#define N_FILTER    5   //number of Filter structures (clones not counted here)
#define N_RUN_FILT  14  //Filters 5..13 = runtime clones of #0

#define N_ABASIC	11
				//index 0 = Osc1
				//index 1 = Osc2
				//index 2 = Noise Envelope
				//index 3 = Osc1+2 common Freq Mod
				//index 4 = Lfo1 Freq Mod
				//index 5 = Lfo2 Freq Mod
				//index 6 = Bass
				//index 7 = Treble
				//index 8 = Gain Envelope
				//index 9 = F0 Mid Freq
				//index 10 = F0 Peak
				//index 11++ = FreeAdsr
#define N_AFREE		8
#define N_ASTRUCT   19	//=N_ABASIC+N_AFREE = number of ADSR structures
				//FreeAdsr #0 ->index 11 = Vibrato
				//FreeAdsr #1 ->index 12 = Chorus
				//FreeAdsr #2 ->index 13 = Osc2 extra Freq Mod
				//FreeAdsr #3 ->index 14 = F0 Qfactor
				//FreeAdsr #4 ->index 15 = F2(Noise) Mid Freq
				//FreeAdsr #5 ->index 16 = Dry level
				//FreeAdsr #6 ->index 17 = Delay level
				//FreeAdsr #7 ->index 18 = Feedback level

#define A_FACTOR	16		//number of ADSR refresh loops per 10ms step
							//must be EVEN for correct results
#define L_FACTOR	320		//ADSR timing >1sec(loops per/s 200ms timing raster)
#define SECFACTOR	1600	//number of ADSR loops per 1 second
#define SYSFREQ		1600	//CPU SysTick frequency
#define SECDIV		16 		//=SYSFREQ/100 makes some calculations easier

#define LEVEL0      0x0020  //cross zero threshold to be modified quickly

#define EOX     0xF7
#define RUNNINGSTATE        //uncomment if not wanted
//works only on MIDI msgs which are transmitted from MIDI OUT
//Running State is always supported with MIDI msgs received at MIDI IN

#define LFF 	10	//for Macintosh OS:simply exchange values of CRR and LFF
#define CRR		13	//then New_Line will be interpreted correctly
#define ESC		27
#define SPACE	32

#define CDC_CLASS   0
#define MIDI_CLASS  1

#define MAXCHORUS 10000		//=0x2710 =302*33.075
#define BUFLEN 3400 //3072 //2048
#define USB_TXBUFLEN  BUFLEN
#define USB_RXBUFLEN  BUFLEN
// if changing buffer sizes: the SRAM .bss size must be below 128500 bytes,
// else CPU reset while upload of big files has been observed
// reason probably heap or stack defacto bigger than assumed by compiler

#define OSCI_DAC		//drive test pins for debugging only
#define OSCI_ADSR

#define mask0  1
#define mask1  (1<<1)
#define mask2  (1<<2)
#define mask3  (1<<3)
#define mask4  (1<<4)
#define mask5  (1<<5)
#define mask6  (1<<6)
#define mask7  (1<<7)
#define mask8  (1<<8)
#define mask9  (1<<9)
#define mask10 (1<<10)
#define mask11 (1<<11)
#define mask12 (1<<12)
#define mask13 (1<<13)
#define mask14 (1<<14)
#define mask15 (1<<15)
#define mask16 (1<<16)
#define mask17 (1<<17)
#define mask18 (1<<18)
#define mask19 (1<<19)
#define mask20 (1<<20)
#define mask21 (1<<21)
#define mask22 (1<<22)
#define mask23 (1<<23)
#define mask24 (1<<24)
#define mask25 (1<<25)
#define mask26 (1<<26)
#define mask27 (1<<27)
#define mask28 (1<<28)
#define mask29 (1<<29)
#define mask30 (1<<30)
#define mask31 (1<<31)

#endif  //GLOBALDEFINES

// ########################################################
//History:
//2022: Hardware Rev.3: redesign Nucleo baseboard and operator panel
//2022: Firmware Rev.8: complete redesign of firmware 7, 2019
//released 21.Nov2022
//Firmware Rev.9
//--------------
//04Dec22	AllNotesOff MIDI command implemented
//09Dec22	Conflict with General MIDI: Parameter set by PGM Change removed
//			Parameter configuration using MIDI Control Change Msgs reorganized
//19Dec22   bugfix comm.c: uploadSoundSet()
//22Dec22	bugfix synthi.c: ADSR10
//11Jan23	bugfix main.c, init Usart DIPSW
//11Jan23	bugfix panel.c controlSystem(void), USART MIDI/COM selection
//released 11.Jan2023
//--------------
