// Header file of of "NucleoSynth-v3.9" **************************************
// digital Audio Synthesizer based on STM32 Nucleo F446
// Author and (c)2019-22 Wolfgang Schemmert

// The copyright claim and GPL-3 License is restricted to this file and the
// depending files
// "main.c", "comm.c, "panel.c", "filter.c", "dacfreq.c", "synthi.c","drums.c",
// "usb-otg-4611.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org

// Contact: www.midi-and-more.de
// A detailed hardware diy construction manual and a user operating manual
// is provided at this website

// for the IIR filter algorithms used here, see detailled description:
// www.IowaHills.com/A4IIRBilinearTransform.html

// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

//from comm.c
extern uint32_t gcInstrVolPreset[N_SOUNDS];
extern uint32_t gcActualDirectory;
extern void activateSoundSet(void);
extern uint32_t  CMD_Interpreter(uint32_t);
extern void  USB_Midi_Scanner(uint32_t);
extern void  USART_Midi_Scanner(uint32_t);
extern void updateDisplay(void);

//from filter.c
extern struct IIRPack {
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
uint32_t QFACTOR;           //bandwidth or 100*zeta(1-100)
uint32_t PEAK;
} gfIIRPack[N_SOUNDS*N_FILTER];
//bits 16-31 contain Serial, bits0-15 contain Parallel filter parameters

extern struct{  //each 9words
float A1;
float A2;
float B0;
float B1;
float B2;
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
float QFMOD;                //precalculated during initFilterParams()
float FPEAKMOD;             //precalculated during initFilterParams()
}gfRuntimeIIR[N_FILTER]; //each 9word

extern uint32_t gfFilterType, gfFilterNum, gfFlexMode;
extern uint32_t gfStageMask[N_SOUNDS]; //bits 0,1,2,3: 1=filter active
//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bit 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)

extern uint32_t gfRuntimeMask, gfFilterOffset;
extern uint32_t gfBasTreb, gfBasTrebPreset[N_SOUNDS];
extern uint32_t gfGainPreset[N_SOUNDS];
extern uint32_t gfBaseFreq2, gfF0mod, gfF0modLfo2, gfF0modPreset[N_SOUNDS];
extern float gfF0ModBase;	//= 1. + (F0 freq deviation by Lfo modulation)
extern float gfFilterFreq0, gfF0TempQF;
extern void tempInitFilter(uint32_t, uint32_t);

//from panel.c
extern const uint8_t gpEmptyStr[];
extern uint32_t gpConfigPanel, gpPlusState,gpModDelay,gpPanelMode,gpNoServPanel;
extern uint32_t gpDisplayCount, gpBCursorPos;
extern void initControlPanel(void);
extern void controlPanel(void);
extern void displayString(uint32_t, const uint8_t *);
extern void placeCursor(uint32_t);
extern void display_OFF(void);
extern void display_ON(void);
extern void initDisplay(void);
extern void writeLcdData(uint32_t);
extern void writeLcdInstr(uint32_t);
extern void evalEncoder(void);

//from synthi.c
extern uint32_t gsFreeAdsr, gsFreeAdsrPreset[N_SOUNDS];
extern const int16_t gsSawtooth[1024];
extern const int16_t gsExpSin[1024];
extern int16_t gsWaveLfo1[4000], gsWaveLfo2[4000], gsWaveLfo3[4000];
extern int16_t gsUserWave0[1024], gsUserWave1[1024];
extern int16_t gsUserWave2[1024], gsUserWave3[1024];

extern uint32_t gsWAVEFORM_1, gsOsc1ModLfo2, gsWAVEFORM_1Preset[N_SOUNDS];
extern uint32_t gsWAVEFORM_2, gsOsc2ModLfo2, gsWAVEFORM_2Preset[N_SOUNDS];
extern int32_t gsTRANS, gsTRANSPreset[N_SOUNDS];
extern int32_t gsTRANS_2, gsTRANS_2Preset[N_SOUNDS];
extern int32_t gsDETUNE, gsDETUNEPreset[N_SOUNDS];
extern int32_t gsDETUNE_2, gsDETUNE_2Preset[N_SOUNDS];
extern uint32_t gsGLIDE, gsGLIDEPreset[N_SOUNDS];
extern uint32_t gsMOD_1, gsOsc1PWM, gsMOD_1Preset[N_SOUNDS];
extern uint32_t gsMOD_2, gsOsc2PWM, gsMOD_2Preset[N_SOUNDS];
extern int32_t gsPWM1Edge0, gsPWM2Edge0, gsPWM1Hi, gsPWM1Lo, gsPWM2Hi, gsPWM2Lo;
extern float gsNewPWM1EdgeMod, gsNewPWM2EdgeMod;
extern uint32_t gsRangeFS1, gsSlopeFS1, gsRangeFS2, gsSlopeFS2;
extern uint32_t gsFreeSquare1[N_SOUNDS], gsFreeSquare2[N_SOUNDS];
									//bits31-16= gsRangeFS,bits15-0=gsSlopeFS

extern uint32_t gsPhaseF0Pk, gsPhaseF0QF;
extern uint32_t gsPhase0, gsPhase0X, gsPhase1, gsPhase1X, gsPhase2, gsPhase2X;
extern uint32_t gsPhase3, gsPhase3X, gsPhase4, gsPhase4X, gsPhase5, gsPhase5X;
extern uint32_t gsPhase6, gsPhase6X, gsPhase7, gsPhase7X, gsPhase8, gsPhase8X;
extern uint32_t gsPhase9, gsPhase9X;
extern uint32_t gsPhaseF0M, gsPhaseC1, gsPhaseC2,gsPhaseC3,gsPhaseC4,gsPhaseC5;
extern uint32_t gsPhaseC6, gsPhaseC7, gsPhaseC8, gsPhaseC9;
extern float gsTrackShiftF0, gsTrackShiftC1, gsTrackShiftC2, gsTrackShiftC3;
extern float gsTrackShiftC4, gsTrackShiftC5, gsTrackShiftC6;
extern float gsTrackShiftC7, gsTrackShiftC8, gsTrackShiftC9;
extern uint32_t gsBaseFreqF0, gsBaseFreqC1, gsBaseFreqC2, gsBaseFreqC3;
extern uint32_t gsBaseFreqC4, gsBaseFreqC5, gsBaseFreqC6;
extern uint32_t gsBaseFreqC7, gsBaseFreqC8, gsBaseFreqC9;

extern float gsStep0, gsStep0X, gsStep1, gsStep1X;
extern float gsStep2, gsStep2X, gsStep3, gsStep3X;
extern float gsStep4, gsStep4X, gsStep5, gsStep5X;
extern float gsStep6, gsStep6X, gsStep7, gsStep7X;
extern float gsStep8, gsStep8X, gsStep9, gsStep9X;
extern float gsRunStep0, gsRunStep0X, gsRunStep1, gsRunStep1X;
extern float gsRunStep2, gsRunStep2X, gsRunStep3, gsRunStep3X;
extern float gsRunStep4, gsRunStep4X, gsRunStep5, gsRunStep5X;
extern float gsRunStep6, gsRunStep6X, gsRunStep7, gsRunStep7X;
extern float gsRunStep8, gsRunStep8X, gsRunStep9, gsRunStep9X;

struct ADSR {			//stored in Flash preset
uint32_t ATDECSUS;		//bits31..24:Attack.bits23..16:Decay,bits15..0:Sustain
uint32_t REFIXVEL;		//bits23..16:Release,bits15..8:Fixvol,bits7..0:Velocity
};
struct ADSR gsADSR[N_SOUNDS*N_ASTRUCT];
//sets of compacted each 7 ADSR structures
//Attack, Decay and Relase times up to 1sec in 10 ms scale,big values compressed
//Sustain bit7 set=hold, else 1x mode

extern uint32_t gsAdsrOffset, gsAdsrIndex, gsNoteFlags, gsNoteCount;
extern uint32_t gsMonoPoly;  //default=0 =polyphonic, 1=monophonic
extern uint32_t gsO2Sync;//(if set,Osc2 wave start is sync.with Osc1 wave start)
extern int32_t gsValueFMO12, gsValueFMO2, gsValueFLfo, gsValueVib;
extern int32_t gsValueFLB, gsValueChor;
extern uint32_t gsGlideHold;
extern uint32_t gsKeytrack0Preset[N_SOUNDS];
extern uint32_t gsPWM1, gsPWM1Preset[N_SOUNDS];

extern int16_t calcTrapez(uint32_t, uint32_t, uint32_t);
extern int16_t calcTri(uint32_t, uint32_t);
extern void triggerFilter0(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC1(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC2(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC3(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC4(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC5(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC6(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC7(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC8(uint32_t, uint32_t, uint32_t);
extern void triggerFilterC9(uint32_t, uint32_t, uint32_t);
extern void ADSR_Process(void);

//from dacfreq.c
extern uint32_t gaRoute0N, gaRoutePreset[N_SOUNDS];
extern uint32_t gaNoise;            //in main.c seed for xorshift32 algorithm
extern void initTimer6(void);
extern void initDAC(void);

//from drums.c
extern uint32_t gdKickVolPreset[N_SOUNDS], gdHiTomVolPreset[N_SOUNDS];
extern uint32_t gdSnareVolPreset[N_SOUNDS], gdRimVolPreset[N_SOUNDS];
extern uint32_t gdClapVolPreset[N_SOUNDS], gdHihatVolPreset[N_SOUNDS];
extern uint32_t gdCymbalVolPreset[N_SOUNDS], gdBellVolPreset[N_SOUNDS];
extern uint32_t gdDrumsVolPreset[N_SOUNDS];

//from usb-otg-4611.c
#ifdef MINHW
extern uint8_t gUSB_RXBUF[USB_RXBUFLEN];
extern volatile uint8_t *gUSB_TXHEAD, *gUSB_TXTAIL, *gUSB_TXBUFTOP;
extern uint8_t gUSB_TXBUF[USB_TXBUFLEN];
extern volatile uint8_t *gUSB_RXHEAD, *gUSB_RXTAIL, *gUSB_RXBUFTOP;
#endif
extern uint32_t guDeviceClass, guConfigured;
extern void stopUSB(void);
extern void initUSB(void);
extern void sendUsbByte(uint8_t);
extern uint32_t pullUsbRxBuf(void);
extern void enCodeMidi(uint8_t);

// ************************************
// if changing buffer sizes: the SRAM .BSS size must be below 128500 bytes,
// else CPU reset while upload of big files has been observed
// reason unknown, no compiler problem messages

uint8_t gUSART1_RXBUF[BUFLEN + 4];
uint8_t *gUSART1_RXBUFTOP= &(gUSART1_RXBUF[BUFLEN]);
uint8_t *gUSART1_RXHEAD=0,*gUSART1_RXTAIL=0;

#ifndef STD
uint8_t gUSART2_RXBUF[BUFLEN + 4];
uint8_t *gUSART2_RXBUFTOP= &(gUSART2_RXBUF[BUFLEN]);
uint8_t *gUSART2_RXHEAD=0,*gUSART2_RXTAIL=0;
#endif

const uint8_t gBootStr[] =      {"NucleoSynth F446"};
const uint8_t gRevStr[] =       {"Rev:            "};

const uint8_t gNoUsbVddStr[] =  {" no_USB_Power   "};
const uint8_t gNoUsbConnStr[] = {" noUSB_Interface"};
const uint8_t gUsbUnplugStr[] = {" USB_unplugged  "};
const uint8_t gComStr[] =  {"COM"};
const uint8_t gMidiStr[] =  {"MID"};
const uint8_t gErrorStr[] = {"Upload Error"};

uint32_t gSYSCOUNT=0;
uint32_t gDELAYCOUNT=0, gTIMEOUT=0, gLEDCOUNT=0;

int32_t gTest1, gTest2, gTest3, gTest4, gTest5;		//for debugging only
float gTestF;										//for debugging only

uint32_t gVid, gPid, gUsartCOM, gChannel, gDrumsChannel, gVUsb;

uint32_t * gFlashPtr;
uint32_t gSoundSet;

uint32_t gVolume, gVolumePreset[N_SOUNDS];
int32_t gVolumeFactor, gVibratoVolume;
uint32_t gVibrato, gVibratoLfo1, gVibratoPreset[N_SOUNDS], gNTrigger[N_SOUNDS];
//gVibratoLfo1 is a short phrase for gVibratoPreset,bit7, see DELAYFLAGS,bit0 too
uint32_t gChorusWidth, gChorusLfo3, gChorusPreset[N_SOUNDS];
//gChorusLfo3 is a short phrase for gVibratoPreset,bit7, see DELAYFLAGS,bit3 too
int32_t gChorus;
uint32_t gDelayTime, gDelayTimePreset[N_SOUNDS];         //DelayTime 100us units
                // packed specially:bits 8-15=hundreds+tens,bits0-7=ones+tenths
uint32_t gDelayOffset;              //DelayTime scaled Timer6 frequency (*=3.31)
int32_t gDelayLevelFactor, gDelayLevelPreset[N_SOUNDS];
//special conversion between gDelayLevel and gDelayLevelPreset
//gDelayLevelPreset contains 7bit level and bit7 set for phase inversion
//gDelayLevelFactor ranges -127 ... + 127 best fitness in panel.c and dacfreq.c
int32_t gFeedbackFactor, gFeedbackPreset[N_SOUNDS];
//special conversion between gFeedbackFactor and gFeedbackPreset
//gFeedbackPreset contains 7bit level and bit7 set for phase inversion
//gFeedbackFactor ranges -127 ... + 127 for best fitness in panel.c and dacfreq.c

uint32_t gLfoNum;
uint32_t gLfo1Ptr, gLfo1Freq, gLfo1FreqPreset[N_SOUNDS];
uint32_t gLfo1Type, gLfo1Wave, gLfo1Slope, gLfo1WavePreset[N_SOUNDS];
uint32_t gLfo2Ptr, gLfo2Freq, gLfo2FreqPreset[N_SOUNDS];
uint32_t gLfo2Type, gLfo2Wave, gLfo2Slope, gLfo2WavePreset[N_SOUNDS];
uint32_t gLfo3Ptr, gLfo3Freq, gLfo3FreqPreset[N_SOUNDS];
uint32_t gLfo3Type, gLfo3Wave, gLfo3Slope, gLfo3WavePreset[N_SOUNDS];
float gModCent1, gModCent2, gOldModCent1, gOldModCent2;

//attenuation as "per thousand of input"
//Though values are inacceptably high (must be divided by 8)
//this data format is used for compatibility with other work.
const int32_t gDbTable[53] = {0,11,12,14,                  //index 0...3
                           16,18,20,22,25,28,               //index 4...9
                           31,35,39,44,50,56,               //index 10..
                           63,70,79,88,99,111,              //index 16..
                           125,140,157,177,198,222,         //index 22..
                           250,281,315,353,396,445,         //index 28..
                           500,561,630,707,793,889,         //index 34..
                           1000,1122,1259,1413,1585,1778,   //index 40..
                           2000,2244,2518,2826,3170,3556,   //index 46..
                           4000} ; //index 52


void initControlPanel(void);
void servSystem(void);
void LowFreqProcess(void);
void reInitUSB(void);
void LED_Off(void);
void msDelay(uint32_t);
void wait100us(void);
void wait6us(void);
void wait3us(void);
void wait1us(void);

void popFlashSounds(void);
void pop1Sound(uint32_t);
uint32_t pushFlashSounds(void);
uint32_t push1Sound(uint32_t);
void eraseFlashSounds(void);

//USART
void initUsart1(void);
void USART1_IRQHandler(void);
#ifndef STD
void initUsart2(void);
uint32_t pullUsart2RxBuf(void);
void USART2_IRQHandler(void);
#endif
void sendB(uint8_t);
void sendM(uint8_t);
void sendX8(uint8_t);
void sendNibble(uint8_t);
void sendX16(uint16_t);
void sendX32(uint32_t);
void sendL1000(uint32_t);
void sendLS32(uint32_t);
void sendS32(uint32_t);
void sendNS32(uint32_t);
void sendString(const uint8_t *);
uint32_t pullUsart1RxBuf(void);
uint32_t pullByte(void);

uint32_t pullTimedByte(void);
int32_t getDeciNum(uint32_t);
float getFloat(void);

#ifdef DEBUG
uint8_t gDBGBUF[4098];
uint8_t *gDBGHEAD=gDBGBUF,*gDBGBUFTOP=&(gDBGBUF[4096]);

void initSDBG(void);
void sdbgB(uint8_t);
void sdbgX8(uint8_t);
void sdbgNibble(uint8_t);
void sdbgX16(uint16_t);
void sdbgX32(uint32_t);
void sendDebug(void);
#endif

