// DAC output generation specific routines of "NucleoSynth-v3.9" **************
// digital Audio Synthesizer based on STM32 Nucleo F446
// Author and (c)2019-22 Wolfgang Schemmert

// The copyright claim and GPL-3 License is restricted to this file and the
// depending files
// "main.c", "main.h, "comm.c", "panel.c", "filter.c", "synthi.c","drums.c",
// "usb-otg-4611.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org

// Contact: www.midi-and-more.de
// A detailed hardware diy construction manual and a user operating manual
// is provided at this website

// for the IIR filter algorithms used here, see detailled description:
// www.IowaHills.com/A4IIRBilinearTransform.html
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f4xx_modified.h" //modifications see top lines of this file
#include "globaldefines.h"

extern uint32_t gVolume, gDelayTime;
extern int32_t gVolumeFactor, gVibratoVolume;
extern int32_t gDelayLevelFactor, gDryFactor, gFeedbackFactor;
extern int32_t gChorus;             //actually used delay scaled as buffer index
extern float gModCent1, gModCent2;
extern float gTestF;                                //for debug only
extern int32_t gTest1, gTest2, gTest3, gTest4;      //for debug only
//USART
extern void sendB(uint8_t); //for debug only
extern void sendX32(uint32_t);

//from filter.c
extern int32_t gfOut0, gfOut12, gfOutN, gfOut4, gfGain;
extern uint32_t gfRuntimeMask;
extern void filterProcess_0(void);
extern void gainProcess(void);
extern int32_t filterTestProcess(uint32_t);
extern void filterProcess_34(int32_t);
extern int32_t filterProcess_1(int32_t in);
extern void filterProcess_N(int32_t);

//from synthi.c
extern uint32_t gsMonoPoly;  //gsMonoPoly default=0 =polyphonic
extern uint32_t gsO2Sync;//(if set,Osc2 wave start is sync.with Osc1 wave start)
extern uint32_t gsWAVEFORM_1, gsWAVEFORM_2;
extern int16_t gsWave[4096], gsWaveX[2048];
extern float gsWavPtr0, gsWavPtr0X, gsWavPtr1, gsWavPtr1X;
extern float gsWavPtr2, gsWavPtr2X, gsWavPtr3, gsWavPtr3X;
extern float gsWavPtr4, gsWavPtr4X, gsWavPtr5, gsWavPtr5X;
extern float gsWavPtr6, gsWavPtr6X, gsWavPtr7, gsWavPtr7X;
extern float gsWavPtr8, gsWavPtr8X, gsWavPtr9, gsWavPtr9X;
extern float gsRunStep0, gsRunStep0X, gsRunStep1, gsRunStep1X;
extern float gsRunStep2, gsRunStep2X, gsRunStep3, gsRunStep3X;
extern float gsRunStep4, gsRunStep4X, gsRunStep5, gsRunStep5X;
extern float gsRunStep6, gsRunStep6X, gsRunStep7, gsRunStep7X;
extern float gsRunStep8, gsRunStep8X, gsRunStep9, gsRunStep9X;
extern uint32_t gsPhaseNs, gsPhaseG;
extern uint32_t gsPhase0, gsPhase0X, gsPhase1, gsPhase1X, gsPhase2, gsPhase2X;
extern uint32_t gsPhase3, gsPhase3X, gsPhase4, gsPhase4X, gsPhase5, gsPhase5X;
extern uint32_t gsPhase6, gsPhase6X, gsPhase7, gsPhase7X, gsPhase8, gsPhase8X;
extern uint32_t gsPhase9, gsPhase9X;
extern int32_t gsValueNs, gsValueFree, gsFreeAdsr;
extern int32_t gsValue0, gsValue0X, gsValue1, gsValue1X, gsValue2, gsValue2X;
extern int32_t gsValue3, gsValue3X, gsValue4, gsValue4X, gsValue5, gsValue5X;
extern int32_t gsValue6, gsValue6X, gsValue7, gsValue7X, gsValue8, gsValue8X;
extern int32_t gsValue9, gsValue9X;
extern uint32_t gsOffNs, gsOffFree;
extern uint32_t gsOff0, gsOff0X, gsOff1, gsOff1X, gsOff2, gsOff2X, gsOff3;
extern uint32_t gsOff3X, gsOff4, gsOff4X, gsOff5, gsOff5X, gsOff6, gsOff6X;
extern uint32_t gsOff7, gsOff7X, gsOff8, gsOff8X, gsOff9, gsOff9X;
extern uint32_t gsAdsrOffset;
extern int32_t gsHiValNs,gsHiValG,gsHiValFM1,gsHiValFM2,gsHiValF0,gsHiValFree;
extern int32_t gsHiVal0, gsHiVal0X, gsHiVal1, gsHiVal1X, gsHiVal2, gsHiVal2X;
extern int32_t gsHiVal3, gsHiVal3X, gsHiVal4, gsHiVal4X, gsHiVal5, gsHiVal5X;
extern int32_t gsHiVal6, gsHiVal6X, gsHiVal7, gsHiVal7X, gsHiVal8, gsHiVal8X;
extern int32_t gsHiVal9, gsHiVal9X;
extern int32_t gsPWM1Hi, gsPWM1Lo, gsPWM2Hi, gsPWM2Lo;
extern uint32_t gsMOD_1, gsMOD_2, gsOsc1PWM, gsOsc2PWM;
extern float gsNewPWM1EdgeMod, gsNewPWM2EdgeMod;

//from drums.c
extern uint32_t gdKickOn;
extern int32_t gdKickLast, gdKickVolFactor, gdKickVelo;
extern const int16_t *gdKickPtr, * gdKickTop;

extern uint32_t gdHiTomOn;
extern int32_t gdHiTomLast, gdHiTomVolFactor, gdHiTomVelo;
extern const int16_t *gdHiTomPtr, * gdHiTomTop;

extern uint32_t gdSnareOn;
extern int32_t gdSnareLast, gdSnareVolFactor, gdSnareVelo;
extern const int16_t *gdSnarePtr, * gdSnareTop;

extern uint32_t gdRimOn;
extern int32_t gdRimLast, gdRimVolFactor, gdRimVelo;
extern const int16_t *gdRimPtr, * gdRimTop;

extern uint32_t gdHihatOn;
extern int32_t gdHihatLast, gdHihatVolFactor, gdHihatVelo;
extern const int16_t *gdHihatPtr,* gdHihatTop;

extern uint32_t gdClapOn;
extern int32_t gdClapLast, gdClapVolFactor, gdClapVelo;
extern const int16_t *gdClapPtr, * gdClapTop;

extern uint32_t gdCymbalOn;
extern int32_t gdCymbalLast, gdCymbalVolFactor, gdCymbalVelo;
extern const int16_t *gdCymbalPtr, * gdCymbalTop;

extern uint32_t gdBellOn;
extern int32_t gdBellLast, gdBellVolFactor, gdBellVelo;
extern const int16_t *gdBellPtr, * gdBellTop;

// ************************************

int32_t gaCHORUSBUF[MAXCHORUS+8];                 //MAXCHORUS=9930 (=3000*3.31)
int32_t *gaCHORUSBUFTOP = &(gaCHORUSBUF[MAXCHORUS]);
int32_t *gaCHORUSHEAD=0,*gaCHORUSTAIL=0;

int32_t gaLast_N, gaLevel;
int32_t gaO1New0, gaO1New1, gaO1New2, gaO1New3, gaO1New4, gaO1New5;
int32_t gaO1New6, gaO1New7, gaO1New8, gaO1New9;
int32_t gaO2New0, gaO2New1, gaO2New2, gaO2New3, gaO2New4, gaO2New5;
int32_t gaO2New6, gaO2New7, gaO2New8, gaO2New9;
float gaPWM0EdgeMod, gaPWM0XEdgeMod, gaPWM1EdgeMod, gaPWM1XEdgeMod;
float gaPWM2EdgeMod, gaPWM2XEdgeMod, gaPWM3EdgeMod, gaPWM3XEdgeMod;
float gaPWM4EdgeMod, gaPWM4XEdgeMod, gaPWM5EdgeMod, gaPWM5XEdgeMod;
float gaPWM6EdgeMod, gaPWM6XEdgeMod, gaPWM7EdgeMod, gaPWM7XEdgeMod;
float gaPWM8EdgeMod, gaPWM8XEdgeMod, gaPWM9EdgeMod, gaPWM9XEdgeMod;

uint32_t gaRoute0N, gaRoutePreset[N_SOUNDS];	//bits23..16=FLEX F0N
												//bits 7..0= EASY F0N
uint32_t gaRouteOsc1,gaRouteOsc2,gaRouteNoise;  //shortflags for runtime speedup
uint32_t gaNoise;                               //actual xorshift value

uint32_t gaTimer, gaFTestFactor;
int32_t gaFilterTest;
uint32_t gaZero1, gaZero2;//=1:more harmonics, =0:more underground noise

void initTimer6(void);
void initDAC(void);
void KeyAndFilterCascade(void);
void handlePWM1(void);
void handlePWM2(void);

//###########################################################################

void initTimer6(void)  // ***************************************************
//Parameter: Zeitintervall Setzen des Interrupt-Flag in Mikrosekunden
// **************************************************************************
{
    RCC-> APB1ENR |= RCC_APB1ENR_TIM6EN;
    TIM6->CR1 = TIM_CR1_URS;
                    //TIM_CR1_URS only overflow generates update interrupt
    TIM6->CR2 = 0;
    TIM6->DIER = 1;         //set UIE Flag, activates "update Interrupt"
    TIM6->PSC = 0;
//APB1 Clock = 42 MHz, Timer clock is APB1 clock * 2 !
    TIM6->ARR = 2540;       //timer frequency 33.071 kHz
    TIM6->CR1 = TIM_CR1_URS | TIM_CR1_CEN;

    NVIC_EnableIRQ(TIM6_DAC_IRQn);
}


void initDAC(void)  // ******************************************************
// **************************************************************************
{
    RCC-> APB1ENR |= RCC_APB1ENR_DACEN;

    DAC->CR = 0;
    DAC->CR =  DAC_CR_EN1;	//PA4 is DAC OUT
    gaCHORUSHEAD = gaCHORUSBUF;
    gaCHORUSTAIL = gaCHORUSBUF;
    gaFilterTest = 0;
    gdKickOn = 0;
    gdHiTomOn = 0;
    gdSnareOn = 0;
    gdHihatOn = 0;
    gdClapOn = 0;
    gdCymbalOn = 0;

}

//###########################################################################

// Interrupt Handler
//******************

void TIM6_DAC_IRQHandler(void)   // *********************************************
//SPI2 is Master TX
// **************************************************************************
{
int32_t effFactor, drums;
int16_t gaNoise16;
static int32_t sFeedback;

    if (TIM6->SR & TIM_SR_UIF)  {       	//Timer overflow update?
//clear interupt flag
       TIM6->SR = 0;

#ifdef OSCI_DAC
        GPIOA->BSRR = GPIO_BSRR_BR_15;      // bit PA15 test pin LOW
#endif

// drive drums
		drums = 0;
        if (gdKickOn) {
            drums += (int32_t)*gdKickPtr++ * gdKickVolFactor * gdKickVelo;
			if (gdKickPtr > gdKickTop) gdKickOn = 0;
		}
        if (gdHiTomOn) {
            drums += (int32_t)*gdHiTomPtr++ * gdHiTomVolFactor * gdHiTomVelo;
            if (gdHiTomPtr > gdHiTomTop) gdHiTomOn = 0;
        }
        if (gdSnareOn) {
            drums += (int32_t)*gdSnarePtr++ * gdSnareVolFactor * gdSnareVelo;
            if (gdSnarePtr > gdSnareTop) gdSnareOn = 0;
        }
        if (gdRimOn) {
            drums += (int32_t)*gdRimPtr++ * gdRimVolFactor * gdRimVelo;
            if (gdRimPtr > gdRimTop) gdRimOn = 0;
        }
        if (gdClapOn) {
            drums += (int32_t)*gdClapPtr++ * gdClapVolFactor * gdClapVelo;
			if (gdClapPtr > gdClapTop) gdClapOn = 0;
        }
        if (gdHihatOn) {
            drums += (int32_t)*gdHihatPtr++ * gdHihatVolFactor * gdHihatVelo;
			if (gdHihatPtr > gdHihatTop) gdHihatOn = 0;
        }
        if (gdCymbalOn) {
            drums += (int32_t)*gdCymbalPtr++ *gdCymbalVolFactor *gdCymbalVelo;
            if (gdCymbalPtr > gdCymbalTop) gdCymbalOn = 0;
        }
        if (gdBellOn) {
            drums += (int32_t)*gdBellPtr++ *gdBellVolFactor *gdBellVelo;
            if (gdBellPtr > gdBellTop) gdBellOn = 0;
        }

        if (gaFilterTest)  {
            gaNoise ^= gaNoise << 13; //xorshift32 algorithm/white noise
            gaNoise ^= gaNoise >> 17;
            gaNoise ^= gaNoise << 5;
            gaLast_N = (int32_t)gaNoise / 0x30000; //optimized empirically
            gaLast_N = gaLast_N * gaFTestFactor;
        }
        else if (gsPhaseNs)  {
            gaNoise ^= gaNoise << 13; //xorshift32 algorithm/white noise
            gaNoise ^= gaNoise >> 17;
            gaNoise ^= gaNoise << 5;
            gaLast_N = (int32_t)gaNoise / 0x300000; //optimized empirically
            gaLast_N = gaLast_N * gsValueNs;
            if (gsOffNs)  {
                if ((gaLast_N >= - 0x00000400)&&(gaLast_N <= 0x00000400)) {
                                            //wait for almost zero transition
                    gsPhaseNs = 0;
                    gsOffNs = 0;
                    gsValueNs = 0;
                    gaLast_N = 0;
                }
            }
        }
        else gaLast_N = 0;

// drive chromatic synthesizer
        KeyAndFilterCascade();                      //returns gaLevel

        if (gVibratoVolume != 100)  {				//=100 while Vibrato is OFF
			gaLevel = (int32_t)((gaLevel * gVibratoVolume) / 100);
        }

		if (gsFreeAdsr == 5)  gaLevel += (gaLevel * gsValueFree) / 2048;
													//"Dry" level ADSR

		if (gDelayTime)  {     						 //do only if Delay ON
			if (gDelayLevelFactor && (gsFreeAdsr == 7) && gsValueFree) {
				effFactor = (16*gFeedbackFactor + gsValueFree) / 16;
                if (effFactor > 99) effFactor = 99;
                if (effFactor < -99) effFactor = -99;
                *gaCHORUSHEAD = gaLevel +(int32_t)((sFeedback*effFactor)/100);
            }
			else  *gaCHORUSHEAD = gaLevel + (int32_t)((sFeedback*gFeedbackFactor)/100);
			gaCHORUSHEAD++;
            if (gaCHORUSHEAD == gaCHORUSBUFTOP)  gaCHORUSHEAD = gaCHORUSBUF;
            gaCHORUSTAIL = gaCHORUSHEAD - gChorus;//can be modulated by Lfo2 or3
			if (gaCHORUSTAIL < gaCHORUSBUF) gaCHORUSTAIL += MAXCHORUS;
                                                //MAXCHORUS is CHORUSBUF length
            sFeedback = *gaCHORUSTAIL;			//keep in mind for next cycle

            if ((gsFreeAdsr == 6) && gsValueFree) {	//Delay level ADSR
				effFactor = (16*gDelayLevelFactor + gsValueFree) / 16;
				if (effFactor > 127)  effFactor = 127;
				if (effFactor < -127)  effFactor = -127;
				gaLevel += (*gaCHORUSTAIL * effFactor)/100;
	//complicated division nesting necessary,else int32 unsaturation may happen
            }
			else gaLevel += (*gaCHORUSTAIL * gDelayLevelFactor) / 100;
        }
		//else gaLevel is not modified by delay

		gaLevel += drums / 400;

//emergency brake:
        if (gaLevel > 0x003FFFFF) gaLevel = 0x003FFFFF;     //precaution to keep
        if (gaLevel < -0x003FFFFF) gaLevel = - 0x003FFFFF;  //32 bit saturated
//0x3FFFFF * 0x200(max gVolumeFactor,see below) = 0x7FFFFE00
//multiplication with a factor > 0x200 would unsaturate the 32bit signal !!

		gaLevel = gaLevel * gVolumeFactor;  //default 0dB = 250, max. 6dB =500
		gaLevel = gaLevel / 0x41000;		//0dB = ca. 0.775V eff adjustment
											//compensate gVolumeFactor and
										//level multiplication factor by ADSR
        gaLevel += 2048;
        if (gaLevel < 0) gaLevel = 0;
        if (gaLevel > 4095) gaLevel = 4095;//hard clip to saturate 12 bit signal

        DAC->DHR12R1 = gaLevel;   //ready for next timer interrupt

//max unsaturated linear DAC output level = 3.3Vss with Volume=-6dB
#ifdef OSCI_DAC
        GPIOA->BSRR = GPIO_BSRR_BS_15;  // bit PA15 test pin HIGH
#endif
    }
}


void KeyAndFilterCascade(void)   // *****************************************
// **************************************************************************
{
uint32_t index;
int16_t noise16;
int32_t temp, last_1, last_2, filter1In;
float  waveLen1, waveLen2;

	if (gsWAVEFORM_1 <= 24) waveLen1 = 1024.0;			//simple waveform
	else if (gsWAVEFORM_1 <= 36) waveLen1 = 2048.0;		//more complex
	else if (gsWAVEFORM_1 == 37) waveLen1 = 3072.0;		//more complex
	else  waveLen1 = 4096.0;							//complex w. subOctaves
	if (gsWAVEFORM_2 <= 24) waveLen2 = 1024.0;
	else  {
		waveLen2 = 2048.0;
		if (gsWAVEFORM_2 == 36)  {
            gaNoise ^= gaNoise << 13; //xorshift32 algorithm/white noise
            gaNoise ^= gaNoise >> 17;
            gaNoise ^= gaNoise << 5;
            noise16 = ((int16_t)gaNoise) / 4;
		}
	}

    if ((gfRuntimeMask & 0x00000400) && gsOsc2PWM) handlePWM2();
												//returns gaO2New0...gaO2New9
    else  {
        if (gsPhase0X)  {
			if (gsWAVEFORM_2 == 36)  {			//Noise
				gaO2New0 = (int32_t)(noise16 * gsValue0X);
				if (gsOff0X)  gsPhase0X = 0;
			}
			else  {
				index = (uint32_t)gsWavPtr0X;
				temp = (int32_t)gsWaveX[index];
				gaO2New0 = temp * gsValue0X;
				if (gsOff0X)  {
					if  ((temp >= -LEVEL0) && (temp <= LEVEL0))  gsPhase0X = 0;
											//wait for almost zero transition
				}
				gsWavPtr0X += gsRunStep0X;
				if (gsWavPtr0X >= waveLen2)  {
					if ((gsMOD_2) || (gaZero2 == 0))  gsWavPtr0X -= waveLen2;
					else  gsWavPtr0X = 0;
				}
			}
        }
        else gaO2New0 = 0;

        if (gsMonoPoly == 0)  {				//Polyphonic
             if (gsPhase1X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New1 = (int32_t)(noise16 * gsValue1X);
					if (gsOff1X)  gsPhase1X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr1X;
					temp = (int32_t)gsWaveX[index];
					gaO2New1 = temp * gsValue1X;
					if (gsOff1X)  {
						if  ((temp >= -LEVEL0)&&(temp <= LEVEL0)) gsPhase1X = 0;
					}
					gsWavPtr1X += gsRunStep1X;
					if (gsWavPtr1X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr1X -= waveLen2;
						else  gsWavPtr1X = 0;
					}
				}
            }
            else gaO2New1 = 0;

            if (gsPhase2X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New2 = (int32_t)(noise16 * gsValue2X);
					if (gsOff2X)  gsPhase2X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr2X;
					temp = (int32_t)gsWaveX[index];
					gaO2New2 = temp * gsValue2X;
					if (gsOff2X)  {
						if  ((temp >= -LEVEL0)&&(temp <= LEVEL0)) gsPhase2X = 0;
					}
					gsWavPtr2X += gsRunStep2X;
					if (gsWavPtr2X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr2X -= waveLen2;
						else  gsWavPtr2X = 0;
					}
				}
            }
            else gaO2New2 = 0;

            if (gsPhase3X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New3 = (int32_t)(noise16 * gsValue3X);
					if (gsOff3X)  gsPhase3X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr3X;
					temp = (int32_t)gsWaveX[index];
					gaO2New3 = temp * gsValue3X;
					if (gsOff3X)  {
						if ((temp >= - LEVEL0)&&(temp <= LEVEL0)) gsPhase3X = 0;
					}
					gsWavPtr3X += gsRunStep3X;
					if (gsWavPtr3X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr3X -= waveLen2;
						else  gsWavPtr3X = 0;
					}
				}
            }
            else gaO2New3 = 0;

            if (gsPhase4X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New4 = (int32_t)(noise16 * gsValue4X);
					if (gsOff4X)  gsPhase4X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr4X;
					temp = (int32_t)gsWaveX[index];
					gaO2New4 = temp * gsValue4X;
					if (gsOff4X)  {
						if  ((temp >= -LEVEL0)&&(temp <= LEVEL0)) gsPhase4X = 0;
					}
					gsWavPtr4X += gsRunStep4X;
					if (gsWavPtr4X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr4X -= waveLen2;
						else  gsWavPtr4X = 0;
					}
				}
            }
            else gaO2New4 = 0;

            if (gsPhase5X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New5 = (int32_t)(noise16 * gsValue5X);
					if (gsOff5X)  gsPhase5X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr5X;
					temp = (int32_t)gsWaveX[index];
					gaO2New5 = temp * gsValue5X;
					if (gsOff5X)  {
						if ((temp >= -LEVEL0)&&(temp <= LEVEL0)) gsPhase5X = 0;
					}
					gsWavPtr5X += gsRunStep5X;
					if (gsWavPtr5X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr5X -= waveLen2;
						else  gsWavPtr5X = 0;
					}
				}
            }
            else gaO2New5 = 0;

            if (gsPhase6X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New6 = (int32_t)(noise16 * gsValue6X);
					if (gsOff6X)  gsPhase6X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr6X;
					temp = (int32_t)gsWaveX[index];
					gaO2New6 = temp * gsValue6X;
					if (gsOff6X)  {
						if  ((temp >= -LEVEL0)&&(temp <= LEVEL0)) gsPhase6X = 0;
					}
					gsWavPtr6X += gsRunStep6X;
					if (gsWavPtr6X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr6X -= waveLen2;
						else  gsWavPtr6X = 0;
					}
				}
            }
            else gaO2New6 = 0;

            if (gsPhase7X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New7 = (int32_t)(noise16 * gsValue7X);
					if (gsOff7X)  gsPhase7X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr7X;
					temp = (int32_t)gsWaveX[index];
					gaO2New7 = temp * gsValue7X;
					if (gsOff7X)  {
						if ((temp >= - LEVEL0)&&(temp <= LEVEL0)) gsPhase7X = 0;
					}
					gsWavPtr7X += gsRunStep7X;
					if (gsWavPtr7X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr7X -= waveLen2;
						else  gsWavPtr7X = 0;
					}
				}
            }
            else gaO2New7 = 0;

            if (gsPhase8X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New8 = (int32_t)(noise16 * gsValue8X);
					if (gsOff8X)  gsPhase8X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr8X;
					temp = (int32_t)gsWaveX[index];
					gaO2New8 = temp * gsValue8X;
					if (gsOff8X)  {
						if ((temp >= -LEVEL0)&&(temp <= LEVEL0)) gsPhase8X = 0;
					}
					gsWavPtr8X += gsRunStep8X;
					if (gsWavPtr8X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr8X -= waveLen2;
						else  gsWavPtr8X = 0;
					}
				}
            }
            else gaO2New8 = 0;

            if (gsPhase9X)  {
				if (gsWAVEFORM_2 == 36)  {
					gaO2New9 = (int32_t)(noise16 * gsValue9X);
					if (gsOff9X)  gsPhase9X = 0;
				}
				else  {
					index = (uint32_t)gsWavPtr9X;
					temp = (int32_t)gsWaveX[index];
					gaO2New9 = temp * gsValue9X;
					if (gsOff9X)  {
						if ((temp >= -LEVEL0)&&(temp <= LEVEL0)) gsPhase9X = 0;
					}
					gsWavPtr9X += gsRunStep9X;
					if (gsWavPtr9X >= waveLen2)  {
						if ((gsMOD_2) || (gaZero2 == 0)) gsWavPtr9X -= waveLen2;
						else  gsWavPtr9X = 0;
					}
				}
            }
            else gaO2New9 = 0;
        }	//Polyphonic
	}		//standard OSC2

	if ((gfRuntimeMask & 0x00000200) && gsOsc1PWM)  handlePWM1();
												//returns gaO1New0...gaO1New9
    else  {
        if (gsPhase0)  {
            index = (uint32_t)gsWavPtr0;
            temp = (int32_t)gsWave[index] ;
            gaO1New0 = temp * gsValue0;
            if (gsOff0)  {
                if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase0 = 0;
                                            //wait for almost zero transition
            }
            gsWavPtr0 += gsRunStep0;
            if (gaRoute0N > 6) {
				gsWavPtr0 += (float)gaO2New0 / 8000;
				// float(gaO2New0 / 8000) takes same CPU time, ... but
                // (float)gaO2New0 / 8000.0 takes SIGNIFICANTLY more CPU time !!
				if (gsWavPtr0 < 0.0)  gsWavPtr0 += waveLen1;
			}
			if (gsWavPtr0 >= waveLen1)  {
				if ((gsMOD_1) || (gaZero1 == 0)) gsWavPtr0 -= waveLen1;
				else  gsWavPtr0 = 0;
				if (gsO2Sync)  {
					gsWavPtr0X = 0.0;
					gaPWM0XEdgeMod = gsNewPWM2EdgeMod;
				}
			}
        }
        else gaO1New0 = 0;

		if (gsMonoPoly == 0)   {							//Polyhonic
            if (gsPhase1)  {
                index = (uint32_t)gsWavPtr1;
				temp = (int32_t)gsWave[index] ;
				gaO1New1 = temp * gsValue1;
                if (gsOff1)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0)) gsPhase1= 0;
                }
                gsWavPtr1 += gsRunStep1;
				if (gaRoute0N > 6) {
					gsWavPtr1 += (float)gaO2New1 / 8000;
					if (gsWavPtr1 < 0.0)  gsWavPtr1 += waveLen1;
				}
				if (gsWavPtr1 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr1 -= waveLen1;
					else  gsWavPtr1 = 0;
					if (gsO2Sync)  {
						gsWavPtr1X = 0.0;
						gaPWM1XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New1 = 0;

            if (gsPhase2)  {
                index = (uint32_t)gsWavPtr2;
				temp = (int32_t)gsWave[index] ;
				gaO1New2 = temp * gsValue2;
                if (gsOff2)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase2 = 0;
                }
                gsWavPtr2 += gsRunStep2;
				if (gaRoute0N > 6)  {
					gsWavPtr2 += (float)gaO2New2 / 8000;
					if (gsWavPtr2 < 0.0)  gsWavPtr2 += waveLen1;
				}
				if (gsWavPtr2 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr2 -= waveLen1;
					else  gsWavPtr2 = 0;
					if (gsO2Sync)  {
						gsWavPtr2X = 0.0;
						gaPWM2XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New2 = 0;

            if (gsPhase3)  {
                index = (uint32_t)gsWavPtr3;
				temp = (int32_t)gsWave[index] ;
				gaO1New3 = temp * gsValue3;
                if (gsOff3)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase3 = 0;
                }
                gsWavPtr3 += gsRunStep3;
                if (gaRoute0N > 6)  {
					gsWavPtr3 += (float)gaO2New3 / 8000;
					if (gsWavPtr3 < 0.0)  gsWavPtr3 += waveLen1;
				}
				if (gsWavPtr3 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr3 -= waveLen1;
					else  gsWavPtr3 = 0;
					if (gsO2Sync)  {
						gsWavPtr3X = 0.0;
						gaPWM3XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New3 = 0;

            if (gsPhase4)  {
                index = (uint32_t)gsWavPtr4;
				temp = (int32_t)gsWave[index] ;
				gaO1New4 = temp * gsValue4;
                if (gsOff4)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase4 = 0;
                }
                gsWavPtr4 += gsRunStep4;
                if (gaRoute0N > 6)  {
					gsWavPtr4 += (float)gaO2New4 / 8000;
					if (gsWavPtr4 < 0.0)  gsWavPtr4 += waveLen1;
				}
				if (gsWavPtr4 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr4 -= waveLen1;
					else  gsWavPtr4 = 0;
					if (gsO2Sync)  {
						gsWavPtr4X = 0.0;
						gaPWM4XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New4 = 0;

            if (gsPhase5)  {
                index = (uint32_t)gsWavPtr5;
				temp = (int32_t)gsWave[index] ;
				gaO1New5 = temp * gsValue5;
                if (gsOff5)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase5 = 0;
                }
                gsWavPtr5 += gsRunStep5;
                if (gaRoute0N > 6)  {
					gsWavPtr5 += (float)gaO2New5 / 8000;
					if (gsWavPtr5 < 0.0)  gsWavPtr5 += waveLen1;
				}
				if (gsWavPtr5 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr5 -= waveLen1;
					else  gsWavPtr5 = 0;
					if (gsO2Sync)  {
						gsWavPtr5X = 0.0;
						gaPWM5XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New5 = 0;

            if (gsPhase6)  {
                index = (uint32_t)gsWavPtr6;
				temp = (int32_t)gsWave[index] ;
				gaO1New6 = temp * gsValue6;
                if (gsOff6)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase6 = 0;
                }
                gsWavPtr6 += gsRunStep6;
                if (gaRoute0N > 6)  {
					gsWavPtr6 += (float)gaO2New6 / 8000;
					if (gsWavPtr6 < 0.0)  gsWavPtr6 += waveLen1;
				}
				if (gsWavPtr6 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr6 -= waveLen1;
					else  gsWavPtr6 = 0;
					if (gsO2Sync)  {
						gsWavPtr6X = 0.0;
						gaPWM6XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New6 = 0;

            if (gsPhase7)  {
                index = (uint32_t)gsWavPtr7;
				temp = (int32_t)gsWave[index] ;
				gaO1New7 = temp * gsValue7;
                if (gsOff7)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase7 = 0;
                }
                gsWavPtr7 += gsRunStep7;
                if (gaRoute0N > 6)  {
					gsWavPtr7 += (float)gaO2New7 / 8000;
					if (gsWavPtr7 < 0.0)  gsWavPtr7 += waveLen1;
				}
				if (gsWavPtr7 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr7 -= waveLen1;
					else  gsWavPtr7 = 0;
					if (gsO2Sync)  {
						gsWavPtr7X = 0.0;
						gaPWM7XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New7 = 0;

            if (gsPhase8)  {
                index = (uint32_t)gsWavPtr8;
				temp = (int32_t)gsWave[index] ;
				gaO1New8 = temp * gsValue8;
                if (gsOff8)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase8 = 0;
                }
                gsWavPtr8 += gsRunStep8;
                if (gaRoute0N > 6)  {
					gsWavPtr8 += (float)gaO2New8 / 8000;
					if (gsWavPtr8 < 0.0)  gsWavPtr8 += waveLen1;
				}
				if (gsWavPtr8 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr8 -= waveLen1;
					else  gsWavPtr8 = 0;
					if (gsO2Sync)  {
						gsWavPtr8X = 0.0;
						gaPWM8XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New8 = 0;

            if (gsPhase9)  {
                index = (uint32_t)gsWavPtr9;
				temp = (int32_t)gsWave[index] ;
				gaO1New9 = temp * gsValue9;
                if (gsOff9)  {
                    if  ((temp >= - LEVEL0) && (temp <= LEVEL0))  gsPhase9 = 0;
                }
                gsWavPtr9 += gsRunStep9;
                if (gaRoute0N > 6)  {
					gsWavPtr9 += (float)gaO2New9 / 8000;
					if (gsWavPtr9 < 0.0)  gsWavPtr9 += waveLen1;
				}
				if (gsWavPtr9 >= waveLen1)  {
					if ((gsMOD_1) || (gaZero1 == 0))  gsWavPtr9 -= waveLen1;
					else  gsWavPtr9 = 0;
					if (gsO2Sync)  {
						gsWavPtr9X = 0.0;
						gaPWM9XEdgeMod = gsNewPWM2EdgeMod;
					}
				}
            }
            else gaO1New9 = 0;

        }  //polyphonic
    }   //standard Osc1

	if (gaFilterTest != 0)  {
		if ((gfRuntimeMask & 0x00000001) && (gaFilterTest >= 0))  {
			gfOut0 = filterTestProcess(0);
		}
		else if ((gfRuntimeMask & 0x00000002) && (gaFilterTest < 0))  {
			gfOut0 = filterTestProcess(1);
		}
		else gfOut0 = gaLast_N;;
	}
	else  {		//normal synthesizer operation

		if (gfRuntimeMask & 0x00000001)  {			//filter#0 is active
			if (gaRouteOsc1 || gaRouteOsc2)  {
				filterProcess_0(); 					//returns gfOut0
				gfOut0 = gfOut0 / 16;				//reduce stretched overlevel
				if ((gfGain > 1) && gsPhaseG) gainProcess();	//in/out=gfOut0
			}
			else gfOut0 = 0;

		}
		else  {  									//filter#0 is NOT active
			gfOut0 = 0;
			if (gaRouteOsc1)  {
				gfOut0 += gaO1New0 + gaO1New1 + gaO1New2 + gaO1New3 + gaO1New4 +
							gaO1New5 +gaO1New6 + gaO1New7 + gaO1New8 + gaO1New9;
			}
			if (gaRouteOsc2)  {
				gfOut0 += gaO2New0 + gaO2New1 + gaO2New2 + gaO2New3 + gaO2New4 +
							gaO2New5 +gaO2New6 + gaO2New7 + gaO2New8 + gaO2New9;
			}
			gfOut0 = gfOut0 / 16;					//reduce stretched overlevel
			if ((gfGain > 1) && gsPhaseG) gainProcess();
		}

		filter1In = 0;
		if (gaRouteOsc1 == 0)  {
			filter1In += gaO1New0 + gaO1New1 + gaO1New2 + gaO1New3 + gaO1New4 +
						   gaO1New5 +gaO1New6 + gaO1New7 + gaO1New8 + gaO1New9;
											//add here if not through filter#0
		}
		if ((gaRouteOsc2 == 0) && (gaRoute0N <= 6))  {
			filter1In += gaO2New0 + gaO2New1 + gaO2New2 + gaO2New3 + gaO2New4 +
						   gaO2New5 +gaO2New6 + gaO2New7 + gaO2New8 + gaO2New9;
											//add here if not through filter#0
		}

		if ((gaRouteOsc1 == 0) || ((gaRouteOsc2 == 0) && (gaRoute0N <= 6)))  {
			filter1In = filter1In / 16;			//reduce stretched overlevel
			if (gfRuntimeMask & 0x00000002) {	//filter#1 is active
				gfOut0 += filterProcess_1(filter1In);
			}
			else  gfOut0 += filter1In;
		}

		if (gaRouteNoise )  {
			if ((gfRuntimeMask & 0x00000004) == 0)  gfOut0 += gaLast_N;
			else  {
				filterProcess_N(gaLast_N);		//filter#2(Noise) returns gfOutN
				gfOut0 += gfOutN; 				//add Noise after Filter0
			}
		}
	}
	filterProcess_34(gfOut0); //returns gaLevel
//Bass,Treble is controlled via Peak of Filter#3 and Filter#4, see adjTone3,4()
}


void handlePWM1(void)   // **************************************************
//returns gaO1New0...gaO1New9
// **************************************************************************
{
    if (gsPhase0)  {
        if (gsWavPtr0 < gaPWM0EdgeMod) gaO1New0 =(int32_t)(gsPWM1Hi * gsValue0);
        else  gaO1New0 = (int32_t)(gsPWM1Lo * gsValue0);
        if (gsOff0)  gsPhase0 = 0;
        gsWavPtr0 += gsRunStep0;

		if (gaRoute0N > 6) {
            gsWavPtr0 += (float)gaO2New0 / 8000;
                // float(gaO2New0 / 8000) takes same CPU time, ... but
                // (float)gaO2New0 / 8000.0 takes SIGNIFICANTLY more CPU time !!
            if (gsWavPtr0 < 0.0)  gsWavPtr0 += 1024.0;
		}
        if (gsWavPtr0 >= 1024.0)  {
            gsWavPtr0 -= 1024.0;
            gaPWM0EdgeMod = gsNewPWM1EdgeMod;
            //delayed update is necessary to avoid sound clicks
            //sometimes when PWM direction changes
        }
        if (gsO2Sync)  {
			gsWavPtr0X = 0.0;
			gaPWM0XEdgeMod = gsNewPWM2EdgeMod;
        }
    }
    else gaO1New0 = 0;

    if ((gfRuntimeMask & 0x00000080) == 0)   {              //if polyphonic

        if (gsPhase1)  {
            if (gsWavPtr1 <gaPWM1EdgeMod) gaO1New1=(int32_t)(gsPWM1Hi*gsValue1);
            else  gaO1New1 = (int32_t)(gsPWM1Lo * gsValue1);
            if (gsOff1)  gsPhase1 = 0;
            gsWavPtr1 += gsRunStep1;
            if (gaRoute0N > 6) {
                gsWavPtr1 += (float)(gaO2New1 / 8000);
                if (gsWavPtr1 < 0.0)  gsWavPtr1 += 1024.0;
            }
            if (gsWavPtr1 >= 1024.0)  {
				gsWavPtr1 -= 1024.0;
                gaPWM1EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr1X = 0.0;
				gaPWM1XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New1 = 0;

        if (gsPhase2)  {
            if (gsWavPtr2 <gaPWM2EdgeMod) gaO1New2=(int32_t)(gsPWM1Hi*gsValue2);
            else  gaO1New2= (int32_t)(gsPWM1Lo * gsValue2);
            if (gsOff2)  gsPhase2 = 0;
            gsWavPtr2 += gsRunStep2;
            if (gaRoute0N > 6)  {
                gsWavPtr2 += (float)(gaO2New2 / 8000);
                if (gsWavPtr2 < 0.0)  gsWavPtr2 += 1024.0;
            }
            if (gsWavPtr2 >= 1024.0)  {
				gsWavPtr2 -= 1024.0;
                gaPWM2EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr2X = 0.0;
				gaPWM2XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New2 = 0;

        if (gsPhase3)  {
            if (gsWavPtr3 <gaPWM3EdgeMod) gaO1New3=(int32_t)(gsPWM1Hi*gsValue3);
            else  gaO1New3 = (int32_t)(gsPWM1Lo * gsValue3);
            if (gsOff3)  gsPhase3 = 0;
            gsWavPtr3 += gsRunStep3;
            if (gaRoute0N > 6)  {
                gsWavPtr3 += (float)(gaO2New3 / 8000);
                if (gsWavPtr3 < 0.0)  gsWavPtr3 += 1024.0;
            }
            if (gsWavPtr3 >= 1024.0)  {
				gsWavPtr3 -= 1024.0;
                gaPWM3EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr3X = 0.0;
				gaPWM3XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New3 = 0;

        if (gsPhase4)  {
            if (gsWavPtr4<gaPWM4EdgeMod)gaO1New4=(int32_t)(gsPWM1Hi*gsValue4);
            else  gaO1New4 = (int32_t)(gsPWM1Lo * gsValue4);
            if (gsOff4)  gsPhase4 = 0;
            gsWavPtr4 += gsRunStep4;
            if (gaRoute0N > 6)  {
                gsWavPtr4 += (float)(gaO2New4 / 8000);
                if (gsWavPtr4 < 0.0)  gsWavPtr4 += 1024.0;
            }
            if (gsWavPtr4 >= 1024.0)  {
				gsWavPtr4 -= 1024.0;
                gaPWM4EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr4X = 0.0;
				gaPWM4XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New4 = 0;

        if (gsPhase5)  {
            if (gsWavPtr5<gaPWM5EdgeMod)gaO1New5=(int32_t)(gsPWM1Hi*gsValue5);
            else  gaO1New5 = (int32_t)(gsPWM1Lo * gsValue5);
            if (gsOff5)  gsPhase5 = 0;
            gsWavPtr5 += gsRunStep5;
            if (gaRoute0N > 6)  {
                gsWavPtr5 += (float)gaO2New5 / 8000;
                if (gsWavPtr5 < 0.0)  gsWavPtr5 += 1024.0;
            }
            if (gsWavPtr5 >= 1024.0)  {
				gsWavPtr5 -= 1024.0;
                gaPWM5EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr5X = 0.0;
				gaPWM5XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New5 = 0;

        if (gsPhase6)  {
            if (gsWavPtr6<gaPWM6EdgeMod)gaO1New6=(int32_t)(gsPWM1Hi*gsValue6);
            else  gaO1New6 = (int32_t)(gsPWM1Lo * gsValue6);
            if (gsOff6)  gsPhase6 = 0;
            gsWavPtr6 += gsRunStep6;
            if (gaRoute0N > 6)  {
                gsWavPtr6 += (float)gaO2New6 / 8000;
                if (gsWavPtr6 < 0.0)  gsWavPtr6 += 1024.0;
            }
            if (gsWavPtr6 >= 1024.0)  {
				gsWavPtr6 -= 1024.0;
                gaPWM6EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr6X = 0.0;
				gaPWM6XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New6 = 0;

        if (gsPhase7)  {
            if (gsWavPtr7<gaPWM7EdgeMod)gaO1New7=(int32_t)(gsPWM1Hi*gsValue7);
            else  gaO1New7 = (int32_t)(gsPWM1Lo * gsValue7);
            if (gsOff7)  gsPhase7 = 0;
            gsWavPtr7 += gsRunStep7;
            if (gaRoute0N > 6)  {
                gsWavPtr7 += (float)gaO2New7 / 8000;
                if (gsWavPtr7 < 0.0)  gsWavPtr7 += 1024.0;
            }
            if (gsWavPtr7 >= 1024.0)  {
				gsWavPtr7 -= 1024.0;
                gaPWM7EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr7X = 0.0;
				gaPWM7XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New7 = 0;

        if (gsPhase8)  {
            if (gsWavPtr8<gaPWM8EdgeMod)gaO1New8=(int32_t)(gsPWM1Hi*gsValue8);
            else  gaO1New8 = (int32_t)(gsPWM1Lo * gsValue8);
            if (gsOff8)  gsPhase8 = 0;
            gsWavPtr8 += gsRunStep8;
            if (gaRoute0N > 6)  {
                gsWavPtr8 += (float)gaO2New8 / 8000;
                if (gsWavPtr8 < 0.0)  gsWavPtr8 += 1024.0;
            }
            if (gsWavPtr8 >= 1024.0)  {
				gsWavPtr8 -= 1024.0;
                gaPWM8EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr8X = 0.0;
				gaPWM8XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New8 = 0;

        if (gsPhase9)  {
            if (gsWavPtr9<gaPWM9EdgeMod)gaO1New9=(int32_t)(gsPWM1Hi*gsValue9);
            else  gaO1New9 = (int32_t)(gsPWM1Lo * gsValue9);
            if (gsOff9)  gsPhase9 = 0;
            gsWavPtr9 += gsRunStep9;
            if (gaRoute0N > 6)  {
                gsWavPtr9 += (float)gaO2New9 / 8000;
                if (gsWavPtr9 < 0.0)  gsWavPtr9 += 1024.0;
            }
            if (gsWavPtr9 >= 1024.0)  {
				gsWavPtr9 -= 1024.0;
                gaPWM9EdgeMod = gsNewPWM1EdgeMod;
            }
			if (gsO2Sync)  {
				gsWavPtr9X = 0.0;
				gaPWM9XEdgeMod = gsNewPWM2EdgeMod;
			}
        }
        else gaO1New9 = 0;
    }  //monophonic/polyphonic
}



void handlePWM2(void)   // **************************************************
//returns gaO2New0...gaO2New9
// **************************************************************************
{
    if (gsPhase0X)  {
        if (gsWavPtr0X<gaPWM0XEdgeMod) gaO2New0 = (int32_t)(gsPWM2Hi*gsValue0X);
        else  gaO2New0 = (int32_t)(gsPWM2Lo * gsValue0X);
        if (gsOff0X)  gsPhase0X = 0;
        gsWavPtr0X += gsRunStep0X;
        if (gsWavPtr0X >= 1024.0)  {
            gsWavPtr0X -= 1024.0;
            gaPWM0XEdgeMod = gsNewPWM2EdgeMod;
        }
    }
    else gaO2New0 = 0;

    if ((gfRuntimeMask & 0x00000080) == 0)   {              //polyphonic
        if (gsPhase1X)  {
           if(gsWavPtr1X<gaPWM1XEdgeMod)gaO2New1=(int32_t)(gsPWM2Hi*gsValue1X);
            else  gaO2New1 = (int32_t)(gsPWM2Lo * gsValue1X);
            if (gsOff1X)  gsPhase1X = 0;
            gsWavPtr1X += gsRunStep1X;
            if (gsWavPtr1X >= 1024.0)  {
				gsWavPtr1X -= 1024.0;
                gaPWM1XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New1 = 0;

        if (gsPhase2X)  {
           if(gsWavPtr2X<gaPWM2XEdgeMod)gaO2New2=(int32_t)(gsPWM2Hi*gsValue2X);
            else  gaO2New2 = (int32_t)(gsPWM2Lo * gsValue2X);
            if (gsOff2X)  gsPhase2X = 0;
            gsWavPtr2X += gsRunStep2X;
            if (gsWavPtr2X >= 1024.0)  {
				gsWavPtr2X -= 1024.0;
                gaPWM2XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New2 = 0;

        if (gsPhase3X)  {
           if(gsWavPtr3X<gaPWM3XEdgeMod)gaO2New3=(int32_t)(gsPWM2Hi*gsValue3X);
            else  gaO2New3 = (int32_t)(gsPWM2Lo * gsValue3X);
            if (gsOff3X)  gsPhase3X = 0;
            gsWavPtr3X += gsRunStep3X;
            if (gsWavPtr3X >= 1024.0)  {
				gsWavPtr3X -= 1024.0;
                gaPWM3XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New3 = 0;

        if (gsPhase4X)  {
           if(gsWavPtr4X<gaPWM4XEdgeMod)gaO2New4=(int32_t)(gsPWM2Hi*gsValue4X);
            else  gaO2New4 = (int32_t)(gsPWM2Lo * gsValue4X);
            if (gsOff4X)  gsPhase4X = 0;
            gsWavPtr4X += gsRunStep4X;
            if (gsWavPtr4X >= 1024.0)  {
				gsWavPtr4X -= 1024.0;
                gaPWM4XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New4 = 0;

        if (gsPhase5X)  {
           if(gsWavPtr5X<gaPWM5XEdgeMod)gaO2New5=(int32_t)(gsPWM2Hi*gsValue5X);
            else  gaO2New5 = (int32_t)(gsPWM2Lo * gsValue5X);
            if (gsOff5X)  gsPhase5X = 0;
            gsWavPtr5X += gsRunStep5X;
            if (gsWavPtr5X >= 1024.0)  {
				gsWavPtr5X -= 1024.0;
                gaPWM5XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New5 = 0;

        if (gsPhase6X)  {
           if(gsWavPtr6X<gaPWM6XEdgeMod)gaO2New6=(int32_t)(gsPWM2Hi*gsValue6X);
            else  gaO2New6 = (int32_t)(gsPWM2Lo * gsValue6X);
            if (gsOff6X)  gsPhase6X = 0;
            gsWavPtr6X += gsRunStep6X;
            if (gsWavPtr6X >= 1024.0)  {
				gsWavPtr6X -= 1024.0;
                gaPWM6XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New6 = 0;

        if (gsPhase7X)  {
          if(gsWavPtr7X<gaPWM7XEdgeMod)gaO2New7=(int32_t)(gsPWM2Hi*gsValue7X);
            else  gaO2New7 = (int32_t)(gsPWM2Lo * gsValue7X);
            if (gsOff7X)  gsPhase7X = 0;
            gsWavPtr7X += gsRunStep7X;
            if (gsWavPtr7X >= 1024.0)  {
				gsWavPtr7X -= 1024.0;
                gaPWM7XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New7 = 0;

        if (gsPhase8X)  {
           if(gsWavPtr8X<gaPWM8XEdgeMod)gaO2New8=(int32_t)(gsPWM2Hi*gsValue8X);
            else  gaO2New8 = (int32_t)(gsPWM2Lo * gsValue8X);
            if (gsOff8X)  gsPhase8X = 0;            gsWavPtr8X += gsRunStep8X;
            if (gsWavPtr8X >= 1024.0)  {
				gsWavPtr8X -= 1024.0;
                gaPWM8XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New8 = 0;

        if (gsPhase9X)  {
           if(gsWavPtr9X<gaPWM9XEdgeMod)gaO2New9=(int32_t)(gsPWM2Hi*gsValue9X);
            else  gaO2New9  = (int32_t)(gsPWM2Lo * gsValue9X);
            if (gsOff9X)  gsPhase9X = 0;
            gsWavPtr9X += gsRunStep9X;
            if (gsWavPtr9X >= 1024.0)  {
				gsWavPtr9X -= 1024.0;
                gaPWM9XEdgeMod = gsNewPWM2EdgeMod;
            }
        }
        else gaO2New9 = 0;
    }
}

//###########################################################################

