// ditial IIR Filter specific routines of "NucleoSynth-v3.9" *****************
// digital Audio Synthesizer based on STM32 Nucleo F446
// Author and (c)2019-22 Wolfgang Schemmert

// The copyright claim and GPL-3 License is restricted to this file and the
// depending files
// "main.c", "main.h, "comm.c", "panel.c", "dacfreq.c", "synthi.c","drums.c",
// "usb-otg-4611.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org

// Contact: www.midi-and-more.de
// A detailed hardware diy construction manual and a user operating manual
// is provided at this website

// for the IIR filter algorithms used here, see detailled description:
// www.IowaHills.com/A4IIRBilinearTransform.html

// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f4xx_modified.h" //modifications see top lines of this file
#include "globaldefines.h"

extern uint32_t gSoundSet;
extern int32_t gDelayFeedback;
extern int32_t gTest1, gTest2, gTest3, gTest4;     //for debug only
extern const int32_t gDbTable[53];
//RS-232
extern void sendB(uint8_t);
extern void sendM(uint8_t);
extern void sendX8(uint8_t);
extern void sendX16(uint16_t);
extern void sendX32(uint32_t);
extern void sendS32(uint32_t);
extern void sendE(float);
extern void sendString(const uint8_t *);
extern uint32_t pullRxBuf(void);
extern int32_t getDeciNum(uint32_t);
extern float getFloat(void);

//from comm.c
extern uint32_t  parseHexNibble(void);

//from dacfreq.c
extern int32_t gaLast_N, gaLevel;
extern uint32_t gaRouteOsc1, gaRouteOsc2, gaRoute0N;
                                            //short flags for runtime speedup
extern int32_t gaO1New0, gaO1New1, gaO1New2, gaO1New3, gaO1New4, gaO1New5;
extern int32_t gaO1New6, gaO1New7, gaO1New8, gaO1New9;
extern int32_t gaO2New0, gaO2New1, gaO2New2, gaO2New3, gaO2New4, gaO2New5;
extern int32_t gaO2New6, gaO2New7, gaO2New8, gaO2New9;

//from synthi.c
extern uint32_t gsNoTriggerF0;
extern int32_t gsValueG, gsValue3, gsHiValG ;
extern uint32_t gsPhaseG;
extern uint32_t gsPhase0, gsPhase0X;
extern uint32_t gsPhase1, gsPhase1X, gsPhase2, gsPhase2X, gsPhase3,gsPhase3X;
extern uint32_t gsPhase4, gsPhase4X, gsPhase5, gsPhase5X, gsPhase6, gsPhase6X;
extern uint32_t gsPhase7, gsPhase7X, gsPhase8, gsPhase8X, gsPhase9,gsPhase9X;

// ************************************
//save a little bit runtime:
#define SAMPLEFREQ  33075   //Sample Freq in Hz
#define NYQUIST  16537.5    //manual precalculation: SAMPLEFREQ/2
#define OMEGA  10.528       //manual precalculation: SAMPLEFRQ/Pi/1000
                            //tangens() expects rad*1000!!

//table of MIDI frequencies(Hz), index=MIDI note value. Reference 69=440Hz
//frequencies < MIDI NOTE 18 and > MIDI NOTE > 120 are not supported (= 0)
const float gfMidiFreqTable[128] = {
    8.2, 8.7, 9.2, 9.8, 10.3, 10.9,                         //MIDI Note 0...
    11.6, 12.2, 13.0, 13.8, 14.6, 15.43,                    //MIDI Note 6...
    16.35, 17.32, 18.35, 19.45, 20.6, 21.83,                //MIDI Note 12...
    23.124, 24.50, 25.96, 27.50, 29.135, 30.868,            //MIDI Note 18...
    32.703, 34.648, 36.708, 38.891, 41.203, 43.654,         //MIDI Note 24...
    46.249, 48.999, 51.913, 55.0, 58.270, 61.735,           //MIDI Note 30...
    65.406, 69.296, 73.416, 77.782, 82.407, 87.307,         //MIDI Note 36...
    92.499, 97.999, 103.826, 110.0, 116.541, 123.471,       //MIDI Note 42...
    130.813, 138.591, 146.832, 155.563, 164.814, 174.614,   //MIDI Note 48...
    184.997, 195.998, 207.652, 220.0, 233.082, 246.942,     //MIDI Note 54...
    261.626, 277.183, 293.665, 311.127, 329.628, 349.228,   //MIDI Note 60...
    369.994, 391.995, 415.305, 440.0, 466.164, 493.883,     //MIDI Note 66...
    523.251, 554.365, 587.330, 622.254, 659.255, 698.456,   //MIDI Note 72...
    739.989, 783.991, 830.609, 880.0, 932.328, 987.767,     //MIDI Note 78...
    1046.502, 1108.731, 1174.659, 1244.508, 1318.510, 1396.913, //MIDI Note 84..
    1479.978, 1567.982, 1661.219, 1760.0, 1864.655, 1975.533, //MIDI Note 90...
    2093.005, 2217.461, 2349.318, 2489.016, 2637.02, 2793.826, //MIDI Note 96...
    2959.955, 3135.963, 3322.438, 3520.0, 3729.31, 3951.066, //MIDI Note 102..
    4186.01, 4434.92, 4698.64, 4978.03, 5274.04, 5587.65,   //MIDI Note 108...
    5919.91, 6271.93, 6644.87, 7040.0, 7458.62, 7902.13,    //MIDI Note 114...
    8372.02, 8869.84, 9397.27, 9956.06, 10548., 11175., 11839., 12543.}; //120..

struct IIRPack {
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
uint32_t QFACTOR;           //bandwidth or 100*zeta(1-100)
uint32_t PEAK;
} gfIIRPack[N_SOUNDS*N_FILTER];
//bits 24-31 contain Flex Default, bits 16-23 contain Easy Default,
//bits 8-15 contain Flex Modify, bits 0-7 contain EasyModify filter parameters

struct IIR {  //each 9words
float A1;
float A2;
float B0;
float B1;
float B2;
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
uint32_t QFACTOR;           //bandwidth or 100*zeta(1-100)
uint32_t PEAK;
} gfACTIV_IIR[N_FILTER];
//are recalculated from gfIIRPack[] when sound or routing mode changes

//gcc processes arrays with constant (immediate) item offset much faster than
//offset defined by variable.
//Therefore, and use of precalculated parameters, introduction of gfRuntimeIIR.
struct{
float A1;
float A2;
float B0;
float B1;
float B2;
uint32_t TYPE;
uint32_t FREQ;					//= MIDI note !!
float QFMOD;					//precalculated during initFilterParams()
float FPEAKMOD;					//precalculated during initFilterParams()
}gfRuntimeIIR[N_RUN_FILT];		//each 9words

float gfF0TempPeak, gfF0TempQF;	//copy of FPEAKMOD and QFMOD for ADSRs

uint32_t gfFilterNum, gfFilterType, gfFlexMode;
uint32_t gfRuntimeMask, gfStageMask[N_SOUNDS];
//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bit 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)

float gfState1[N_RUN_FILT], gfState2[N_RUN_FILT];
uint32_t gfGain, gfGainPreset[N_SOUNDS];
uint32_t gfF0mod, gfF0modLfo2, gfF0modPreset[N_SOUNDS];
					//bit7 of Preset holds gfF0ModLfo2, bits 0..6 hold gfF0Mod
uint32_t gfBaseFreqF2;
float gfFilterFreq0;
float gfF0ModBase;	//= 1. + (F0 freq deviation by Lfo modulation)

//temporary "Memo" Filter set
uint8_t gfEasyMemoType[N_FILTER];
uint8_t gfEasyMemoFreq[N_FILTER];
uint8_t gfEasyMemoQfactor[N_FILTER];
uint8_t gfEasyMemoPeak[N_FILTER];
uint8_t gfFlexMemoType[N_FILTER];
uint8_t gfFlexMemoFreq[N_FILTER];
uint8_t gfFlexMemoQfactor[N_FILTER];
uint8_t gfFlexMemoPeak[N_FILTER];
//temporary "Undo" Filter set
uint8_t gfEasyUndoType[N_FILTER];
uint8_t gfEasyUndoFreq[N_FILTER];
uint8_t gfEasyUndoQfactor[N_FILTER];
uint8_t gfEasyUndoPeak[N_FILTER];
uint8_t gfFlexUndoType[N_FILTER];
uint8_t gfFlexUndoFreq[N_FILTER];
uint8_t gfFlexUndoQfactor[N_FILTER];
uint8_t gfFlexUndoPeak[N_FILTER];


uint32_t gfBass, gfTreble, gfBasTrebPreset[N_SOUNDS];
int32_t gfOut0, gfOutN, gfOut4;

float tangens(uint32_t);
void enterFilterParams(void);
void initFilterParams(uint32_t,uint32_t,uint32_t,uint32_t,uint32_t,uint32_t);
void packFilter(uint32_t);
void unpackFilter(uint32_t, uint32_t);
void tempInitFilter(uint32_t, uint32_t);
void filterProcess_0(void);
void gainProcess(void);
int32_t filterTestProcess(uint32_t);
void filterProcess_34(int32_t);
int32_t filterProcess_1(int32_t in);
void filterProcess_N(int32_t);
void packBasTreb(void);
void unpackBasTreb(void);

const float gfTangensTable[256] =    //index = step 0.005 rad
                      {0.0,0.005,0.01,0.015,0.02,0.025,0.03,0.035,
                        0.04,0.045,0.05,0.055,0.0601,0.0651,0.0701,0.0751,
                        0.0802,0.0852,0.0902,0.0953,0.1003,0.1054,0.1104,0.1155,
                        0.1206,0.1257,0.1307,0.1358,0.1409,0.1460,0.1511,0.1562,
                        0.1614,0.1665,0.1717,0.1768,0.1820,0.1871,0.1923,0.1975,
                        0.2027,0.2079,0.2131,0.2184,0.2236,0.2289,0.2341,0.2394,
                        0.2447,0.2500,0.2553,0.2607,0.2660,0.2714,0.2768,0.2821,
                        0.2876,0.2930,0.2984,0.3039,0.3093,0.3148,0.3203,0.3258,
                        0.3314,0.3369,0.3425,0.3481,0.3537,0.3593,0.3650,0.3707,
                        0.3764,0.3821,0.3879,0.3936,0.3994,0.4052,0.4111,0.4169,
                        0.4228,0.4288,0.4346,0.4406,0.4466,0.4526,0.4586,0.4647,
                        0.4708,0.4769,0.4831,0.4892,0.4954,0.5017,0.5080,0.5143,
                        0.5206,0.5270,0.5334,0.5398,0.5463,0.5528,0.5594,0.5659,
                        0.5726,0.5792,0.5859,0.5926,0.5994,0.6062,0.6131,0.6200,
                        0.6269,0.6339,0.6410,0.6481,0.6552,0.6623,0.6696,0.6768,
                        0.6841,0.6915,0.6989,0.7064,0.7139,0.7215,0.7291,0.7368,
                        0.7445,0.7523,0.7602,0.7681,0.7761,0.7841,0.7923,0.8004,
                        0.8087,0.8170,0.8253,0.8338,0.8423,0.8509,0.8595,0.8682,
                        0.8771,0.8860,0.8949,0.9040,0.9131,0.9223,0.9316,0.9410,
                        0.9505,0.9600,0.9697,0.9794,0.9883,0.9992,1.0092,1.0194,
                        1.0296,1.0300,1.0505,1.0610,1.0717,1.0825,1.0934,1.1045,
                        1.1156,1.1269,1.1383,1.1499,1.1616,1.1734,1.1853,1.1974,
                        1.2097,1.2221,1.2346,1.2473,1.2602,1.2732,1.2864,1.2997,
                        1.3133,1.3270,1.3409,1.3550,1.3692,1.3837,1.3984,1.4133,
                        1.4284,1.4437,1.4592,1.4750,1.4910,1.5072,1.5237,1.5404,
                        1.5574,1.5747,1.5922,1.6100,1.6281,1.6465,1.6652,1.6843,
                        1.7036,1.7233,1.7433,1.7637,1.7844,1.8055,1.8270,1.8489,
                        1.8712,1.8939,1.9171,1.9407,1.9648,1.9893,2.0143,2.0399,
                        2.0660,2.0926,2.1198,2.1475,2.1759,2.2049,2.2345,2.2648,
                        2.2958,2.3275,2.3600,2.3932,2.4273,2.4621,2.4979,2.5346,
                        2.5722,2.6107,2.6503,2.6910,2.7328,2.7757,2.8198,2.8652,
                        2.9119,2.9600,3.0096,3.0606,3.1133,3.1676,3.2236,3.2815
                      };


float tangens(uint32_t radx1000)   // *****************************************
//quick and dirty tangens calculation.
//parameter: angle in 1/1000 rad units, 0 ... 1.27 rad (i.e. param. 0 .. 1275)
//i.e.: parameter is entered as (intended rad) * 1000
//parameter range must be checked before !!
// ****************************************************************************
{
//index in table= step 0.005 rad
    if (radx1000 < 160) return (float)(radx1000) / 1000.;
    else return gfTangensTable[radx1000 / 5];
}


uint32_t freq2Midi(float freq)   // ******************************************
// ***************************************************************************
{
float old, deltaH, deltaL;
uint32_t k;

    if (freq == 0) return 0;

    old = gfMidiFreqTable[12];
    if (freq < old) return old;

    for (k = 12; k < 120; k++)  {
        if (gfMidiFreqTable[k] >= freq) {
            deltaH = gfMidiFreqTable[k] - freq;
            deltaL = freq - old;
            if (deltaH <= deltaL) return k;
            else return (k-1);
        }
        old = gfMidiFreqTable[k];
    }
    return 120;         //denotes 8372Hz, max limit of frequency warping
}


void enterFilterParams(void)   // *******************************************
// **************************************************************************
// filter types:
// 0 = empty, flat. Not evaluated during runtime
// 1 = 1 pole Low Pass
// 2 = 1 pole High Pass
// 3 = 2 pole Low Pass
// 4 = 2 pole High Pass
// 5 = 1 pole Band Pass
// 6 = 1 pole Parametric
// 7 = 2 pole Low Shelf
// 8 = 2 pole High Shelf

{
const uint8_t NumberStr[] = {"Filter Number? "};
const uint8_t TypeStr[] = {"FilterType? "};
const uint8_t FreqStr[] = {" Frequency? "};
const uint8_t BWStr[] = {" Bandwidth? "};
const uint8_t ZetaStr[] = {" Zeta? "};
const uint8_t PeakStr[] = {" Peak? "};  //parametic filter: -1=Notch,0=neutral,>0=peak
uint32_t filterType, n;
uint32_t Peak, Freq, midiFreq, midiQFactor;
float realQFactor, realPeak;

    sendString(NumberStr);
    n = parseHexNibble();           //filter number
    if (n >= N_FILTER) return;      //emergency exit, should be trapped before
    gfFilterNum = n;
    gfRuntimeMask = gfStageMask[gSoundSet];

    sendB(CRR);
    sendString(TypeStr);
    filterType = (uint32_t)parseHexNibble();
    if (filterType > 8) return;

    if (filterType == 0) {          //neutralize filter, parameters to default
        initFilterParams(n, 0, 0, 0, 0, 0);
        return;
    }

//SETFREQ:
//3 dB or resonance freq setting for all filter types
    do {
        sendString(FreqStr);
        Freq = (uint32_t)getDeciNum(8000);
    } while (Freq == 0xFFFFFFFE);
    if (Freq == 0xFFFFFFFF) return;
    if (Freq < 23) Freq = 23;
    if (Freq > 8372) Freq = 8372;
    midiFreq = freq2Midi(Freq);

//SETQ:
    if ((filterType == 3) || (filterType == 4) ||
        (filterType == 7) || (filterType == 8)) { //X
        do  {
            sendString(ZetaStr);
            realQFactor = getFloat();   //stands for <zeta>=1/2Q in these cases
        } while (realQFactor == 1.E-8) ;
        if (realQFactor >= 1.E9) return;
        realQFactor *= 100.0;

        midiQFactor = (uint32_t)realQFactor;
        if (midiQFactor < 1) midiQFactor = 1;
        if (midiQFactor > 100) midiQFactor = 100;
    }
    else if ((filterType == 5) || (filterType == 6)) { //X
        do  {
        sendString(BWStr);
        realQFactor = getFloat();       //stands for <bandwidth> in these cases
        } while (realQFactor == 1.E-8);
        if (realQFactor >= 1.E9) return;
        if (realQFactor < 23) realQFactor = 23;
        if (realQFactor > 4978) realQFactor = 4978;
        midiQFactor = freq2Midi(realQFactor);
    }
    else midiQFactor = 0;

//SETPEAK:
	do  {
		sendString(PeakStr);
		realPeak = getFloat();
	} while (realPeak == 1.E-8);
        if (realPeak >= 1.E9) realPeak = 0.0;
    if  (filterType >= 6)  {
        if (realPeak<-1.0) realPeak=-1.0; //-1=full notch, no stronger possible
        if (realPeak>10.0) realPeak = 10.0;//Peak=0 is neutral, >0 = peak level

        if (realPeak >= 0)  {
			realPeak += 2.0;			//range 2.0 .. 12.0
			realPeak *= 10.0;			//range 20.0 .. 120.0,steps=0.1
        }
        else  {
			realPeak *= 20.0;			//range -20.0 .. 0.0
			realPeak += 20.0;			//range 0.0 .. 20.0, steps 0.05
        }
        Peak = (uint32_t)realPeak;		//range 0..120
    }
    else if (filterType)  {
        if (realPeak < 0.0)  realPeak = 0.0;
        Peak = (uint32_t)realPeak;
        if (Peak>15) Peak = 15;	//Peak=0(dB) is basic,Lift ca.1dB per Peak step
        Peak += 20;				//for compatibility of different FilterTypes
    }
    else  Peak = 20;

    initFilterParams(n, filterType, midiFreq, midiQFactor, Peak, 0);
    return;
}


void initFilterParams(uint32_t filterNum,uint32_t filterType,
		uint32_t midiFreq,uint32_t midiQFactor,uint32_t PEAK,uint32_t TempInit)
// ****************************************************************************
// midiFreq is entered as MIDI note value
// QFactor for type filter types 5&6 = bandwidth entered as MIDI note value
// else filter types >=3 QFactor is resonance sharpness
// else dummy default
// filter types:
// 0 = empty, flat. Not evaluated during runtime
// 1 = 1 pole Low Pass
// 2 = 1 pole High Pass
// 3 = 2 pole Low Pass
// 4 = 2 pole High Pass
// 5 = 1 pole Band Pass
// 6 = 1 pole Parametric
// 7 = 2 pole Low Shelf
// 8 = 2 pole High Shelf
// change 29Aug22: only applicable on active SoundSet
// ****************************************************************************
{
float Q, T, Damp, fFreq, fPeak,Arg,K; //Q and T are prewarped gain and 3dB freq
                                      //Damp is zeta*2
uint32_t n, absFNum, filterOffset, temp;
uint32_t TYPE, FREQ, QFACTOR;
float tempA1, tempA2, tempB0, tempB1, tempB2;

    if (filterNum >= N_FILTER)  return;
    if (filterType > 8) return;				//silently without error message

    filterOffset = gSoundSet * N_FILTER;
    absFNum = filterOffset + filterNum;		//valid:(0,5,10,15,..)+filterNum
    TYPE = filterType;

	if (filterNum == gfFilterNum)  gfFilterType = filterType;

    if (filterType == 0)  {					//neutralize filter
        gfStageMask[gSoundSet] &= ~(1<<filterNum);
		gfRuntimeMask = gfStageMask[gSoundSet];
		gfRuntimeIIR[filterNum].A1 = 0.0;
		gfRuntimeIIR[filterNum].A2 = 0.0;
		gfRuntimeIIR[filterNum].B0 = 1.0;
		gfRuntimeIIR[filterNum].B1 = 0.0;
		gfRuntimeIIR[filterNum].B2 = 0.0;
		gfRuntimeIIR[filterNum].TYPE = 0;
		gfRuntimeIIR[filterNum].FREQ = 69;
		gfRuntimeIIR[filterNum].QFMOD = 1.42;
		gfRuntimeIIR[filterNum].FPEAKMOD = 1.0;
		if (TempInit == 0)  {
			gfACTIV_IIR[filterNum].A1 = 0.0;
			gfACTIV_IIR[filterNum].A2 = 0.0;
			gfACTIV_IIR[filterNum].B0 = 1.0;
			gfACTIV_IIR[filterNum].B1 = 0.0;
			gfACTIV_IIR[filterNum].B2 = 0.0;
			gfACTIV_IIR[filterNum].TYPE = 0;
			gfACTIV_IIR[filterNum].FREQ = 69;
			gfACTIV_IIR[filterNum].QFACTOR = 71;
			gfACTIV_IIR[filterNum].PEAK = 20;
			if ((gfRuntimeMask & 0x00000001) &&
									(gaRouteOsc1||gaRouteOsc2)) gsNoTriggerF0=0;
			else  gsNoTriggerF0 = 1;

			packFilter(filterNum);
		}
		if (filterNum == 0)  {				//neutralize clones
			for (n=4; n < N_RUN_FILT; n++)  {
			gfRuntimeIIR[n].A1 = 0.0;
			gfRuntimeIIR[n].A2 = 0.0;
			gfRuntimeIIR[n].B0 = 1.0;
			gfRuntimeIIR[n].B1 = 0.0;
			gfRuntimeIIR[n].B2 = 0.0;
			gfRuntimeIIR[n].TYPE = 0;
			gfRuntimeIIR[n].FREQ = 69;
			gfRuntimeIIR[n].QFMOD = 1.42;
			gfRuntimeIIR[n].FPEAKMOD = 1.0;
			}
		}

		return;
    }

// from here on, filterType != 0 and < 9, i.e. really working filter

    if (midiFreq < 12) midiFreq = 12;
    if (midiFreq > 120) midiFreq = 120;
    if (filterType)  FREQ = midiFreq;
    else   FREQ = 69;				//general default corresponds with 440Hz
    fFreq = gfMidiFreqTable[midiFreq];
    temp = (uint32_t) ( fFreq / OMEGA);
    if (temp > 1275) temp = 1275;   //tangens() expects rad*1000 !
    T= tangens(temp);               //result: T <=  3.2815
//differs from the normally used form T=2*tangens(temp)
//and it is inserted at bilinear transformation 2*T instead of T
//because resulting filter parameter formulae get slightly more simple then.

    if ((filterType == 3) || (filterType == 4) ||
        (filterType == 7) || (filterType == 8)) {
        if (midiQFactor == 0) midiQFactor = 71; //precaution:QFactor = Zeta*100
        if (midiQFactor < 1) midiQFactor = 1;
        if (midiQFactor > 100) midiQFactor = 100;
        QFACTOR = midiQFactor;
        Damp = (float)midiQFactor;
        Damp = Damp / 50.0;                     //Damp = Zeta*2,range 0.02...2.0
    }
    else if ((filterType == 5) || (filterType == 6))  { //X
        QFACTOR = midiQFactor;
        if (midiQFactor == 0) Damp = gfMidiFreqTable[71]; //default 440Hz
        else  Damp = gfMidiFreqTable[midiQFactor];
                                        //Damp formally used for (float)QFactor
//bandwith warping
        Q = (fFreq / NYQUIST) +1.;
        if (Q > 1.6) Q = 1.6;           //works approximately up to SampleFreq/3
        Q = .8 * tangens((uint32_t)( Q * 785.398));
                    //note:785.398 is Pi/4*1000 - tangens() expects rad*1000 !!
        Q = fFreq / (Q*Damp) ;   //QFactor implicitly normalized to SAMPLEFREQ/2
    }
    else {
        QFACTOR = 71;                 //default value, not used
        Damp = 1.42;
    }

    if (filterType >= 6) {
        if (PEAK > 0x80000000) PEAK = 20;   //input range 0 .. 120
        if (PEAK > 120) PEAK = 120;
        fPeak = (float)PEAK;
        if (PEAK < 20) fPeak = fPeak / 20;	//range 0.0...1.0
        else fPeak = (fPeak / 10) - 1.0;	//range 1.0...11.0
											//internally fPeak=1.0 is neutral
        //ATTENTION: fPeak = 110 means amplification of 10 = 20db!
    }
    else if (filterType)  {					//input range 0 .. 15
        if (PEAK > 0x80000000) PEAK = 20;			//PEAK=0dB is neutral
		if (PEAK < 10) PEAK = 10;
        if (PEAK > 35) PEAK = 35;
        fPeak = (float)gDbTable[PEAK]/100.0;		//Peak=20->0.9
													//Peak=35->5.6
													//Peak=10->0.31
    }
    else  {		//formal consistence, not used for filter calculation
        PEAK = 20;
        fPeak = 1.0;
    }

    switch (filterType)  {
    case 1:
//1 pole low pass: H(s) = fPeak / (1 + s)
        Arg = T + 1.0 ;
        tempA1 = (T - 1.0) / Arg ;
        tempA2 = 0.0 ;
        tempB0 = (fPeak * T) / Arg ;
        tempB1 = tempB0;
        tempB2 = 0.0 ;
        break;

    case 2:
//1 pole high pass: H(s) = fPeak*s / (1 + s)
        Arg = T + 1.0 ;
        tempA1 = (T - 1.0)/ Arg ;
        tempA2 = 0.0 ;
        tempB0 =  fPeak / Arg ;
        tempB1 = -1.0 * tempB0;
        tempB2 = 0.0 ;
        break;

    case 3:
//2 pole low pass: H(s) = fPeak / (1 + s*Damp + s^2)   ;Damp = 2*<zeta>
        K = T*T;
        Arg = K  + Damp*T + 1.0 ;
        tempA1 = (2*K - 2.0) / Arg ;
        tempA2 = (K - Damp*T + 1.0) / Arg ;
        tempB0 = (fPeak*K )/ Arg ;
        tempB1 = 2 * tempB0;		//(2*fPeak*K) / Arg ;
        tempB2 = tempB0; 			//(fPeak * K) / Arg ;
        break;

    case 4:
//2 pole High pass: H(s) = fPeak*(s^2) / (1 + s*Damp + s^2)
        K = T*T;
        Arg = K  + Damp*T + 1.0 ;
        tempA1 = (2*K - 2.0) / Arg ;
        tempA2 = (K - Damp*T + 1.0) / Arg ;
        tempB0 = fPeak / Arg ;
        tempB1 = -2.0 * tempB0;		//(-2.0 * fPeak)/ Arg ;
        tempB2 = tempB0;			//fPeak / Arg ;
        break;

    case 5:
//1pole Bandpass: H(s) = (fPeak*s/Q) / (1 + s/Q + s^2)
        K = Q*T*T;
        Arg = K + T + Q;
        tempA1 = (2*K - 2*Q) / Arg ;
        tempA2 = (K - T + Q) / Arg ;
        tempB0 = fPeak*T / Arg ;
        tempB1 = 0.0 ;
        tempB2 = -1.0 * tempB0;		//-1.*fPeak*T / Arg ;
        break;

    case 6:
//1pole Parametric: H(s) =  (1 + fPeak*s/Q + s^2) / (1 + s/Q + s^2)
        K = Q*T*T;
        Arg = K + T + Q;
        tempA1 = (2*K - 2*Q) / Arg ;
        tempA2 = (K - T + Q) / Arg ;
        tempB0 = (K + Q + fPeak*T) / Arg ;
        tempB1 = (2*K - 2*Q) / Arg ;
        tempB2 = (K + Q - fPeak*T) / Arg ;
        break;

    case 7:
 //2pole  Low Shelf  H(s) = (-fPeak + s^2) / (1 + s*Damp + s^2)
        K = T*T;
        Arg = K  + Damp*T + 1.0 ;
        tempA1 = (2*K - 2.0) / Arg ;
        tempA2 = (K - Damp*T + 1.0) / Arg ;
        tempB0 = (-fPeak*K + 1.) / Arg ;
        tempB1 = (-2*fPeak*K - 2.) / Arg ;
        tempB2 = tempB0;			//(-fPeak*K + 1.) / Arg ;
        break;

    case 8:
//2pole  High Shelf  H(s) = (1 - fPeak*s^2) / (1 + s*Damp + s^2)
        K = T*T;
        Arg = K  + Damp*T + 1.0 ;
        tempA1 = (2*K - 2.0) / Arg ;
        tempA2 = (K - Damp*T + 1.0) / Arg ;
        tempB0 = (K - fPeak) / Arg ;
        tempB1 = (2*K + 2*fPeak) / Arg ;
        tempB2 = tempB0;			//(K - fPeak) / Arg ;
        break;

    default:
        break;
    }

	if (TYPE) gfStageMask[gSoundSet] |= 1<<filterNum;
	else gfStageMask[gSoundSet] &= ~(1<<filterNum);
	gfRuntimeMask = gfStageMask[gSoundSet];
	if ((gfRuntimeMask&0x00000001)&&(gaRouteOsc1||gaRouteOsc2)) gsNoTriggerF0=0;
	else  gsNoTriggerF0 = 1;

	gfRuntimeIIR[filterNum].A1 = tempA1 ;
	gfRuntimeIIR[filterNum].A2 = tempA2 ;
	gfRuntimeIIR[filterNum].B0 = tempB0 ;
	gfRuntimeIIR[filterNum].B1 = tempB1 ;
	gfRuntimeIIR[filterNum].B2 = tempB2 ;
	gfRuntimeIIR[filterNum].TYPE = TYPE ;
	gfRuntimeIIR[filterNum].FREQ = FREQ ;
	gfRuntimeIIR[filterNum].QFMOD = Damp;
	gfRuntimeIIR[filterNum].FPEAKMOD = fPeak;

	if (TempInit == 0)  {
		gfACTIV_IIR[filterNum].A1 = tempA1 ;
		gfACTIV_IIR[filterNum].A2 = tempA2 ;
		gfACTIV_IIR[filterNum].B0 = tempB0 ;
		gfACTIV_IIR[filterNum].B1 = tempB1 ;
		gfACTIV_IIR[filterNum].B2 = tempB2 ;
		gfACTIV_IIR[filterNum].TYPE = TYPE;
		gfACTIV_IIR[filterNum].FREQ = FREQ;
		gfACTIV_IIR[filterNum].QFACTOR = QFACTOR;
		gfACTIV_IIR[filterNum].PEAK = PEAK;

		packFilter(filterNum);
	}

	if (filterNum == 0)  {		//init clones if filter#0
		gfFilterFreq0 = gfMidiFreqTable[FREQ];
		gfF0TempPeak = fPeak;		//copy of FPEAKMOD for Adsr "F0Pek"
		gfF0TempQF = Damp;			//copy of QFMOD for FreeAdsr "F0QF"

		for (n=N_FILTER; n < N_RUN_FILT; n++)  {
			gfRuntimeIIR[n].A1 = tempA1 ;
			gfRuntimeIIR[n].A2 = tempA2 ;
			gfRuntimeIIR[n].B0 = tempB0 ;
			gfRuntimeIIR[n].B1 = tempB1 ;
			gfRuntimeIIR[n].B2 = tempB2 ;
			gfRuntimeIIR[n].TYPE = TYPE ;
			gfRuntimeIIR[n].FREQ = FREQ ;
			gfRuntimeIIR[n].QFMOD = Damp;
			gfRuntimeIIR[n].FPEAKMOD = fPeak;
		}
	}
	if (filterNum == 2)  gfBaseFreqF2 = (uint32_t)(gfMidiFreqTable[FREQ]);
}


void packFilter(uint32_t filterNum) // **************************************
// exclusively applicable on active SoundSet
// **************************************************************************
{
uint32_t index;

    index = (gSoundSet * N_FILTER) + filterNum;

    if (gfFlexMode == 0)  {											//EASY
		gfIIRPack[index].TYPE &= 0xFFFFFF00;
		gfIIRPack[index].TYPE |= gfACTIV_IIR[filterNum].TYPE;
		gfIIRPack[index].FREQ &= 0xFFFFFF00;
		gfIIRPack[index].FREQ |= gfACTIV_IIR[filterNum].FREQ;
		gfIIRPack[index].QFACTOR &= 0xFFFFFF00;
		gfIIRPack[index].QFACTOR |= gfACTIV_IIR[filterNum].QFACTOR;
		gfIIRPack[index].PEAK &= 0xFFFFFF00;
		gfIIRPack[index].PEAK |= gfACTIV_IIR[filterNum].PEAK;
    }
	else  {															//FLEX
		gfIIRPack[index].TYPE &= 0xFFFF00FF;
		gfIIRPack[index].TYPE |= gfACTIV_IIR[filterNum].TYPE << 8;
		gfIIRPack[index].FREQ &= 0xFFFF00FF;
		gfIIRPack[index].FREQ |= gfACTIV_IIR[filterNum].FREQ << 8;
		gfIIRPack[index].QFACTOR &= 0xFFFF00FF;
		gfIIRPack[index].QFACTOR |= gfACTIV_IIR[filterNum].QFACTOR << 8;
		gfIIRPack[index].PEAK &= 0xFFFF00FF;
		gfIIRPack[index].PEAK |= gfACTIV_IIR[filterNum].PEAK << 8;
	}
}


void unpackFilter(uint32_t n, uint32_t mode) // ******************************
// ***************************************************************************
{
uint32_t index;

    index = (gSoundSet * N_FILTER) + n;

	if (gfFlexMode == 0)  {		//EASY
			gfACTIV_IIR[n].TYPE = gfIIRPack[index].TYPE & 0x000000FF;
			gfACTIV_IIR[n].FREQ = gfIIRPack[index].FREQ & 0x000000FF;
			gfACTIV_IIR[n].QFACTOR = gfIIRPack[index].QFACTOR & 0x000000FF;
			gfACTIV_IIR[n].PEAK = gfIIRPack[index].PEAK & 0x000000FF;
	}
	else   {	//FLEX
			gfACTIV_IIR[n].TYPE = (gfIIRPack[index].TYPE >> 8) & 0x000000FF;
			gfACTIV_IIR[n].FREQ = (gfIIRPack[index].FREQ >> 8) & 0x000000FF;
			gfACTIV_IIR[n].QFACTOR = (gfIIRPack[index].QFACTOR >> 8)&0x000000FF;
			gfACTIV_IIR[n].PEAK = (gfIIRPack[index].PEAK >> 8) & 0x000000FF;
	}

	if (mode)  { //exclusively reinit gfRuntimeIIR[]. If mode==0, unpack only
		initFilterParams(n,gfACTIV_IIR[n].TYPE,
		gfACTIV_IIR[n].FREQ, gfACTIV_IIR[n].QFACTOR, gfACTIV_IIR[n].PEAK,1);
	}
}


void tempInitFilter(uint32_t filterNum, uint32_t freq) // ********************
//minimal runtime version for F0 modulation
// ***************************************************************************
{
float  Q, T, Damp, fFreq,fPeak,Arg,K; //Q and T are prewarped gain and 3dB freq.
                                          //Damp is zeta*2
float tempA1, tempA2, tempB0, tempB1, tempB2;
uint32_t type, temp;

    type = gfRuntimeIIR[filterNum].TYPE;
    if (type == 0) return;

    if (freq > 8372) return;          //refuse inadeqate changes like underflow
    if (freq < 23) return;

//all filter types frequency prewarping
    fFreq = (float)freq;
    temp = (uint32_t) (fFreq / OMEGA);
 //   if (temp > 1275) temp = 1275;   //tangens() expects rad*1000 !!
    T= tangens(temp);               //result: T <=  3.2815
//differs from the normally used form T=2*tangens(temp)
//and it is inserted at bilinear transformation 2*T instead of T
//because resulting filter parameter formulae get slightly more simple then.

	if ((filterNum == 0) || (filterNum >= 3)) Damp = gfF0TempQF;
	else Damp = gfRuntimeIIR[filterNum].QFMOD;
    if ((type == 3) || (type == 4) ||
        (type == 7) || (type == 8)) {
//Damp = Zeta*2
        if (Damp < 0.02) Damp= 0.02;    //precaution: avoid self-oscillation
		if (Damp > 2.0)  Damp = 2.0;
    }
    else if ((type == 5) || (type == 6))  {
//bandwith warping
		if (Damp < 23.)  Damp = 23.;
		if (Damp > 4978.)  Damp = 4978.;
        Q = (fFreq / NYQUIST) + 1.;
 //       if (Q > 1.6) Q = 1.6;         //works approximately up to SampleFreq/3
        Q = .8 * tangens((uint32_t)( Q * 785.398));
                                        //tangens() expects rad*1000 !!
        Q = fFreq / (Q*Damp) ;  //QFactor implicitly normalized to SampleFreq/2
    }

	if ((filterNum == 0) || (filterNum >= 5)) fPeak =gfF0TempPeak;
										//copy of FPEAKMOD for FreeAdsr"F0Pek"
	else fPeak = gfRuntimeIIR[filterNum].FPEAKMOD; //range 0.0..11.0."flat"is1.0

    switch (type)  {
    case 1:
//1 pole low pass: H(s) = fPeak / (1 + s)
        Arg = T + 1.0 ;
        tempA1 = (T - 1.0) / Arg ;
        tempA2 = 0.0 ;
        tempB0 = (fPeak*T) / Arg ;
        tempB1 = tempB0;
        tempB2 = 0.0 ;
        break;

    case 2:
//1 pole high pass: H(s) = s / (1 + s)
        Arg = T + 1.0 ;
        tempA1 = (T - 1.0)/ Arg ;
        tempA2 = 0.0 ;
        tempB0 =  fPeak / Arg ;
        tempB1 = -1.0 * tempB0;
        tempB2 = 0.0 ;
        break;

    case 3:
//2 pole low pass: H(s) = fPeak / (1 + s*Damp + s^2)   ;Damp = 2*<zeta>
        K = T*T;
        Arg = K  + Damp*T + 1.0 ;
        tempA1 = (2*K - 2.0) / Arg ;
        tempA2 = (K - Damp*T + 1.0) / Arg ;
        tempB0 = (fPeak*K )/ Arg ;
        tempB1 = 2 * tempB0;		//(2*fPeak*K) / Arg ;
        tempB2 = tempB0; 			//(fPeak * K) / Arg ;
        break;

    case 4:
//2 pole High pass: H(s) = fPeak*(s^2) / (1 + s*Damp + s^2)
        K = T*T;
        Arg = K  + Damp*T + 1.0 ;
        tempA1 = (2*K - 2.0) / Arg ;
        tempA2 = (K - Damp*T + 1.0) / Arg ;
        tempB0 = fPeak / Arg ;
        tempB1 = -2.0 * tempB0;		//(-2.0 * fPeak)/ Arg ;
        tempB2 = tempB0;			//fPeak / Arg ;
        break;

    case 5:
//1pole Bandpass: H(s) = (fPeak*s/Q) / (1 + s/Q + s^2)
        K = Q*T*T;
        Arg = K + T + Q;
        tempA1 = (2*K - 2*Q) / Arg ;
        tempA2 = (K - T + Q) / Arg ;
        tempB0 = fPeak*T / Arg ;
        tempB1 = 0.0 ;
        tempB2 = -1.0 * tempB0;		//-1.*fPeak*T / Arg ;
        break;

    case 6:
//1pole Parametric: H(s) =  (1 + fPeak*s/Q + s^2) / (1 + s/Q + s^2)
        K = Q*T*T;
        Arg = K + T + Q;
        tempA1 = (2*K - 2*Q) / Arg ;
        tempA2 = (K - T + Q) / Arg ;
        tempB0 = (K + Q + fPeak*T) / Arg ;
        tempB1 = (2*K - 2*Q) / Arg ;
        tempB2 = (K + Q - fPeak*T) / Arg ;
        break;

    case 7:
 //2pole  Low Shelf  H(s) = (-fPeak + s^2) / (1 + s*Damp + s^2)
        K = T*T;
        Arg = K  + Damp*T + 1.0 ;
        tempA1 = (2*K - 2.0) / Arg ;
        tempA2 = (K - Damp*T + 1.0) / Arg ;
        tempB0 = (-fPeak*K + 1.) / Arg ;
        tempB1 = (-2*fPeak*K - 2.) / Arg ;
        tempB2 = tempB0;			//(-fPeak*K + 1.) / Arg ;
        break;

    case 8:
//2pole  High Shelf  H(s) = (1 - fPeak*s^2) / (1 + s*Damp + s^2)
        K = T*T;
        Arg = K  + Damp*T + 1.0 ;
        tempA1 = (2*K - 2.0) / Arg ;
        tempA2 = (K - Damp*T + 1.0) / Arg ;
        tempB0 = (K - fPeak) / Arg ;
        tempB1 = (2*K + 2*fPeak) / Arg ;
        tempB2 = tempB0;			//(K - fPeak) / Arg ;
        break;

    default:
        break;
    }

    gfRuntimeIIR[filterNum].A1 = tempA1;
    gfRuntimeIIR[filterNum].A2 = tempA2;
    gfRuntimeIIR[filterNum].B0 = tempB0;
    gfRuntimeIIR[filterNum].B1 = tempB1;
    gfRuntimeIIR[filterNum].B2 = tempB2;

}


void filterProcess_0(void)  // *******************************************
//filter#0 and clones
// ***************************************************************************
{
register float state0, state1, state2, out;

    gfOut0 = 0;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New0);
	if (gaRouteOsc2)  state0 += (float)(gaO2New0);
	state1 = gfState1[0];
	state2 = gfState2[0];
	state0 -= gfRuntimeIIR[0].A1 * state1;
	state0 -= gfRuntimeIIR[0].A2 * state2;
	out = gfRuntimeIIR[0].B0 * state0;
	out += gfRuntimeIIR[0].B1 * state1;
	out += gfRuntimeIIR[0].B2 * state2;
	gfState2[0] = state1;
	gfState1[0] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New1);
	if (gaRouteOsc2)  state0 += (float)(gaO2New1);
	state1 = gfState1[5];
	state2 = gfState2[5];
	state0 -= gfRuntimeIIR[5].A1 * state1;
	state0 -= gfRuntimeIIR[5].A2 * state2;
	out = gfRuntimeIIR[5].B0 * state0;
	out += gfRuntimeIIR[5].B1 * state1;
	out += gfRuntimeIIR[5].B2 * state2;
	gfState2[5] = state1;
	gfState1[5] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New2);
	if (gaRouteOsc2)  state0 += (float)(gaO2New2);
	state1 = gfState1[6];
	state2 = gfState2[6];
	state0 -= gfRuntimeIIR[6].A1 * state1;
	state0 -= gfRuntimeIIR[6].A2 * state2;
	out = gfRuntimeIIR[6].B0 * state0;
	out += gfRuntimeIIR[6].B1 * state1;
	out += gfRuntimeIIR[6].B2 * state2;
	gfState2[6] = state1;
	gfState1[6] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New3);
	if (gaRouteOsc2)  state0 += (float)(gaO2New3);
	state1 = gfState1[7];
	state2 = gfState2[7];
	state0 -= gfRuntimeIIR[7].A1 * state1;
	state0 -= gfRuntimeIIR[7].A2 * state2;
	out = gfRuntimeIIR[7].B0 * state0;
	out += gfRuntimeIIR[7].B1 * state1;
	out += gfRuntimeIIR[7].B2 * state2;
	gfState2[7] = state1;
	gfState1[7] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New4);
	if (gaRouteOsc2)  state0 += (float)(gaO2New4);
	state1 = gfState1[8];
	state2 = gfState2[8];
	state0 -= gfRuntimeIIR[8].A1 * state1;
	state0 -= gfRuntimeIIR[8].A2 * state2;
	out = gfRuntimeIIR[8].B0 * state0;
	out += gfRuntimeIIR[8].B1 * state1;
	out += gfRuntimeIIR[8].B2 * state2;
	gfState2[8] = state1;
	gfState1[8] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New5);
	if (gaRouteOsc2)  state0 += (float)(gaO2New5);
	state1 = gfState1[9];
	state2 = gfState2[9];
	state0 -= gfRuntimeIIR[9].A1 * state1;
	state0 -= gfRuntimeIIR[9].A2 * state2;
	out = gfRuntimeIIR[9].B0 * state0;
	out += gfRuntimeIIR[9].B1 * state1;
	out += gfRuntimeIIR[9].B2 * state2;
	gfState2[9] = state1;
	gfState1[9] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New6);
	if (gaRouteOsc2)  state0 += (float)(gaO2New6);
	state1 = gfState1[10];
	state2 = gfState2[10];
	state0 -= gfRuntimeIIR[10].A1 * state1;
	state0 -= gfRuntimeIIR[10].A2 * state2;
	out = gfRuntimeIIR[10].B0 * state0;
	out += gfRuntimeIIR[10].B1 * state1;
	out += gfRuntimeIIR[10].B2 * state2;
	gfState2[10] = state1;
	gfState1[10] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New7);
	if (gaRouteOsc2)  state0 += (float)(gaO2New7);
	state1 = gfState1[11];
	state2 = gfState2[11];
	state0 -= gfRuntimeIIR[11].A1 * state1;
	state0 -= gfRuntimeIIR[11].A2 * state2;
	out = gfRuntimeIIR[11].B0 * state0;
	out += gfRuntimeIIR[11].B1 * state1;
	out += gfRuntimeIIR[11].B2 * state2;
	gfState2[11] = state1;
	gfState1[11] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New8);
	if (gaRouteOsc2)  state0 += (float)(gaO2New8);
	state1 = gfState1[12];
	state2 = gfState2[12];
	state0 -= gfRuntimeIIR[12].A1 * state1;
	state0 -= gfRuntimeIIR[12].A2 * state2;
	out = gfRuntimeIIR[12].B0 * state0;
	out += gfRuntimeIIR[12].B1 * state1;
	out += gfRuntimeIIR[12].B2 * state2;
	gfState2[12] = state1;
	gfState1[12] = state0;
	gfOut0 += (int32_t)out;

	state0 = 0;
	if (gaRouteOsc1)  state0 = (float)(gaO1New9);
	if (gsPhase9X)  state0 += (float)(gaO2New9);
	state1 = gfState1[13];
	state2 = gfState2[13];
	state0 -= gfRuntimeIIR[13].A1 * state1;
	state0 -= gfRuntimeIIR[13].A2 * state2;
	out = gfRuntimeIIR[13].B0 * state0;
	out += gfRuntimeIIR[13].B1 * state1;
	out += gfRuntimeIIR[13].B2 * state2;
	gfState2[13] = state1;
	gfState1[13] = state0;
	gfOut0 += (int32_t)out;
}


void gainProcess(void)  // ***************************************************
//parameter and output is gfQut0
// ***************************************************************************
{
uint32_t  effGain;

	if (gfOut0 > 0x00100000) gfOut0 = 0x00100000;		//avoid int32 saturation
//	if ((int32_t)gfOut0 < (int32_t)0xFFF00000) gfOut0 = 0xFFF00000;
//else compare works unsigned,. this works better:
	if (gfOut0 < -0x00100000) gfOut0 = -0x00100000;		//avoid int32 saturation

	effGain = (((gfGain-1)*gsValueG)/16) + 127;
											//Gain ADSR,never below effGain=127
	if (effGain > 2047)  effGain = 2047;	//avoid int32 saturation
    gfOut0 = (gfOut0  * (int32_t)effGain) / 127;

//soft clip tube amp behaviour
    if (gfOut0 >= 0)  {
        if (gfOut0 > 0x00120000)  {
            gfOut0 = 0x00120000 + (gfOut0 - 0x00120000)/2;
            if (gfOut0 > 0x00150000)  {
                gfOut0 = 0x00150000 +(gfOut0 - 0x00150000)/4;
				if (gfOut0 > 0x00180000) gfOut0 = 0x00180000;
            }
        }
    }
    else  {
        if (gfOut0 < -0x00100000) {
            gfOut0 = -0x00100000 + (gfOut0 + 0x00100000)/2;
            if (gfOut0 < -0x00140000) {
                gfOut0 = -0x00140000 +(gfOut0 + 0x00140000)/2;
                if (gfOut0 < -0x00160000) {
                    gfOut0 = -0x00160000 +(gfOut0 + 0x00160000)/4;
					if (gfOut0 < -0x00180000) gfOut0 = -0x00180000;
                }
            }
        }
    }
}


int32_t filterTestProcess(uint32_t channel)  // ******************************
//filter#0 or filter#1 fed with Noise
// ***************************************************************************
{
register float state0, state1, state2, out;

    state0 = (float)gaLast_N;
	if (channel == 0)  {
		state1 = gfState1[0];
		state2 = gfState2[0];
		state0 -= gfRuntimeIIR[0].A1 * state1;
		state0 -= gfRuntimeIIR[0].A2 * state2;
		out = gfRuntimeIIR[0].B0 * state0;
		out += gfRuntimeIIR[0].B1 * state1;
		out += gfRuntimeIIR[0].B2 * state2;
		gfState2[0] = state1;
		gfState1[0] = state0;
	}
	else  {
		state1 = gfState1[1];
		state2 = gfState2[1];
		state0 -= gfRuntimeIIR[1].A1 * state1;
		state0 -= gfRuntimeIIR[1].A2 * state2;
		out = gfRuntimeIIR[1].B0 * state0;
		out += gfRuntimeIIR[1].B1 * state1;
		out += gfRuntimeIIR[1].B2 * state2;
		gfState2[1] = state1;
		gfState1[1] = state0;
	}
    return (int32_t)out;
}


void filterProcess_34(int32_t in)  // ****************************************
// ***************************************************************************
{
register float state0, state1, state2, out3, out4;

    out3 = (float)in;
    out4 = out3;
    state0 = out3;

    if (gfRuntimeMask & 0x00000008)  {    //Filter#3
        state1 = gfState1[3];       //this intermediate storage saves runtime
        state2 = gfState2[3];
        state0 -= gfRuntimeIIR[3].A1 * state1;
        state0 -= gfRuntimeIIR[3].A2 * state2;
        out3 = gfRuntimeIIR[3].B0 * state0;
        out3 += gfRuntimeIIR[3].B1 * state1;
        out3 += gfRuntimeIIR[3].B2 * state2;
        gfState2[3] = state1;
        gfState1[3] = state0;
        state0 = out3;				//cascade input to filter#2
    }
    out4 = out3;       				//use if filter#4 is OFF
    if (gfRuntimeMask & 0x00000010)  { 	//Filter#4
        state1 = gfState1[4];
        state2 = gfState2[4];
        state0 -= gfRuntimeIIR[4].A1 * state1;
        state0 -= gfRuntimeIIR[4].A2 * state2;
        out4 = gfRuntimeIIR[4].B0 * state0;
        out4 += gfRuntimeIIR[4].B1 * state1;
        out4 += gfRuntimeIIR[4].B2 * state2;
        gfState2[4] = state1;
        gfState1[4] = state0;
    }
	gaLevel = (int32_t)out4 ;

}

int32_t filterProcess_1(int32_t in)  // **************************************
//filter#1 is parallel filter#0
// ***************************************************************************
{
register float state0, state1, state2, out;

	state0 = (float)in;

    state1 = gfState1[1];
    state2 = gfState2[1];
    state0 -= gfRuntimeIIR[1].A1 * state1;
    state0 -= gfRuntimeIIR[1].A2 * state2;
    out = gfRuntimeIIR[1].B0 * state0;
    out += gfRuntimeIIR[1].B1 * state1;
    out += gfRuntimeIIR[1].B2 * state2;
    gfState2[1] = state1;
    gfState1[1] = state0;

    return (int32_t)out;
}

void filterProcess_N(int32_t in)  // *****************************************
//filter#2 is for Noise
// ***************************************************************************
{
register float state0, state1, state2, out;

    state0 = (float)in;

    state1 = gfState1[2];
    state2 = gfState2[2];
    state0 -= gfRuntimeIIR[2].A1 * state1;
    state0 -= gfRuntimeIIR[2].A2 * state2;
    out = gfRuntimeIIR[2].B0 * state0;
    out += gfRuntimeIIR[2].B1 * state1;
    out += gfRuntimeIIR[2].B2 * state2;
    gfState2[2] = state1;
    gfState1[2] = state0;

    gfOutN = (int32_t)out;
}


void packBasTreb(void)  // ***************************************************
//bits 31-24:FlexTreble,bits 23-16:FlexBass,
//bits 15-8:EasyTreble,bits 7-0:EasyBass,
// ***************************************************************************
{
    if (gfFlexMode == 0) {		//EASY: Bass and Treble
        gfBasTrebPreset[gSoundSet] &= 0xFFFF0000;
        gfBasTrebPreset[gSoundSet] |= (gfTreble << 8) | gfBass;
    }
    else   {					//FLEX: bass means filter#3 PEAK, treble #4 PEAK
		gfBasTrebPreset[gSoundSet] &= 0x0000FFFF;
		gfBasTrebPreset[gSoundSet] |= (gfTreble << 24) | (gfBass << 16);
    }
}


void unpackBasTreb(void)  // *************************************************
// ***************************************************************************
{
	if (gfFlexMode == 0)  {
		gfTreble = (gfBasTrebPreset[gSoundSet] >> 8) & 0x000000FF;
		gfBass = gfBasTrebPreset[gSoundSet] & 0x000000FF;
	}
    else  {
		gfTreble = (gfBasTrebPreset[gSoundSet] >> 24) & 0x000000FF;
		gfBass = (gfBasTrebPreset[gSoundSet] >> 16) & 0x000000FF;
	}
}

// ############################################################################



