// "NucleoSynth-v3.9" digital Audio Synthesizer based on STM32 Nucleo F446 ****
// Author and (c)2019-22 Wolfgang Schemmert

// The copyright claim and GPL-3 License is restricted to this file and the
// depending files
// "main.h", "comm.c, "panel.c", "filter.c", "dacfreq.c", "synthi.c","drums.c",
// "usb-otg-4611.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org

// Contact: www.midi-and-more.de
// A detailed hardware diy construction manual and a user operating manual
// is provided at this website

// for the IIR filter algorithms used here, see detailled description:
// www.IowaHills.com/A4IIRBilinearTransform.html

// *********************************************
//
// The source code is optimized for runtime - not for saving memory.
// The common argument that global variables are potentially confusing and
// error promoting has been diminished in a way that EVERY GLOBAL variable name
// starts with lower case 'g' followed by a character typical for the file
// where it is declared. NO LOCAL variable name starts with 'g' or 'G'.
// static local variables always start with 's'.
//
// Note:to get a working RELEASE version with Embitz 1.1.1,
// you must modify file "nucleosynth.ebp":
//			<Target title="Release">
//				......
//				<Compiler>
//					<Add option="-O0" />
//					<Add option="-g0" />
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f4xx_modified.h" //modifications see top lines of this file
#include "globaldefines.h"
#include "main.h"


int main(void)    // ********************************************************
// **************************************************************************
{
uint32_t recvword;

	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN; // Enable Port A clock
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOBEN; // Enable Port B clock
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOCEN; // Enable Port C clock
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIODEN; // Enable Port D clock - all will be used

//after reset, all ports are input, HiZ - except programming pins
//GPIOx->OTYPER pins are push-pull by default
//GPIOx->OSPEEDR set all to 2 MHz by default
//GPIOx->PUPDR pins are HiZ by default

	GPIOA->MODER = 0x00000000;      //primarily all input
	GPIOB->MODER = 0x00000000;      //primarily all input
	GPIOC->MODER = 0x00000000;      //primarily all input

	GPIOA->PUPDR = 0x9002AA5A;      //primarily all inputs pulldown,
									//except PA14,PA13,12,11(HiZ),PA2,3(pull-Up)
									//PA9(HiZ) is input for Vusb detect
	GPIOB->PUPDR = 0xAAAAAA8A;		//primarily all inputs pulldown
									//except PB2(HiZ)
	GPIOC->PUPDR = 0x55555555;		//primarily all inputs pull-Up

// -----------------------------------

#ifdef MINHW
//init PA5 (green onboard LED) as output push/pull, initial Low level (LED OFF)
//blinks when data from keyboard are received
//(except MIDI Timing Clock and Active Sensing)
    GPIOA->MODER &= ~GPIO_MODER_MODER5;
	GPIOA->MODER |= GPIO_MODER_MODER5_0;
#else
//Mode LEDs on user panel
//red LED is ON in PLAY mode, OFF  in CONFIG mode
	GPIOC->MODER &= ~(GPIO_MODER_MODER2);
    GPIOC->MODER |= GPIO_MODER_MODER2_0;
    GPIOC->BSRR = GPIO_BSRR_BR_2;           // bit PC2 red LED OFF
//green LED is ON in PlUS state
	GPIOB->MODER &= ~(GPIO_MODER_MODER0);
    GPIOB->MODER |= GPIO_MODER_MODER0_0;
    GPIOB->BSRR = GPIO_BSRR_BR_0;           // bit PB0 green LED OFF
#endif
// -----------------------------------------

    GPIOA->MODER |= GPIO_MODER_MODER15_0 ;  // Set PortA15 as push-pull output
    GPIOA->BSRR = GPIO_BSRR_BS_15;          // bit PA15 (DAC test pin) ON
    GPIOA->MODER |= GPIO_MODER_MODER0_0 ;  // Set PortA0 as push-pull output
    GPIOA->BSRR = GPIO_BSRR_BS_0;          // bit PA0 (ADSR test pin) ON

	gSYSCOUNT = 0;
	gLEDCOUNT = 0;
	gDELAYCOUNT = 0;
	gTIMEOUT = 0;
	gVid = DEFAULT_VID;
	gPid = DEFAULT_PID;
	gChannel = 0;
	gDrumsChannel = 9;

    gSoundSet = 0;
    gsAdsrOffset = 0;
	gsNoteCount = 0;
	gsNoteFlags = 0;
    gpConfigPanel = 0;					//start"Config General"
    gpNoServPanel = 0;
    gpPlusState = 0;
    gOldModCent1 = 0.;
    gOldModCent2 = 0.;
    gSoundSet = 0;
    gfFilterNum = 0;
    gfFilterType = 0;
    gLfoNum = 1;
    gaNoise = 314159265;				//seed for xorshift32 algorithm
										//Constant because not cryptographic

    SysTick_Config(SystemCoreClock/SYSFREQ);    //1600 Hz
	msDelay(50);

    //start Independent Watchdog  ******************
    IWDG->KR = 0x00005555;
    IWDG->RLR = 0x00000FFF;
    IWDG->PR = 0x00000002;				//resulting timeout ca. 1-2 seconds
    msDelay(5);
    IWDG->KR = 0x0000CCCC;				//start Independent Watchdog

	guDeviceClass = CDC_CLASS;			//pre-assumtion, will be specified
	gUsartCOM = 0;	//MINHW and ORIG: USART1 always=MIDI, specified for CUT_OFF

#ifdef CUT_OFF
//check DipSwitch
//left DipSw1 is PA2:  OFF=COM interface, ON = MIDI interface
//right DipSw2 is PA3: OFF:USART works as RS-232 interface
//                     ON:USART works as MIDI interface
//reconfirm:
    GPIOA->MODER &= ~(GPIO_MODER_MODER2 | GPIO_MODER_MODER3);  //inputs
    GPIOA->PUPDR &= ~(GPIO_PUPDR_PUPDR2 | GPIO_PUPDR_PUPDR3);  //with pull-up
    GPIOA->PUPDR |= (GPIO_PUPDR_PUPDR2_0 | GPIO_PUPDR_PUPDR3_0);
    msDelay(2);
//DIPSW1 = PA2 selects USB device class;
    if ((GPIOA->IDR & mask2) == 0)  guDeviceClass = MIDI_CLASS;
//DIPSW2 = PA3 selects USART mode;
//    if (GPIOA->IDR & mask3 == 0)  gUsartCOM = 1;//switch off: USART1=MIDI
//bugfix 11Jan2023
    if ((GPIOA->IDR & mask3) == 0)  gUsartCOM = 1;//switch off: USART1=MIDI
#endif

    initUsart1();
    msDelay(2);

    popFlashSounds();

    guConfigured = 0;						//exclusivly send to USART
#ifndef CUT_OFF
    initUsart2();                           //then init additionally
#endif
#ifdef MINHW
    gpPanelMode = 0xFF;                     //suppress user panel update
    sendString(gBootStr);
    sendB(' ');
    sendB('-');
    sendB(' ');
    sendB('R');
    sendB('e');
    sendB('v');
    sendB(HARDWARE_VERSION+0x30);
    sendB('.');
    sendB(REVNUM+0x30);
    sendB(' ');
    sendB('-');
    sendB(' ');
#else
	sendString(gBootStr);
    sendB(' ');
    sendB('-');
    sendB(' ');
    sendB('R');
    sendB('e');
    sendB('v');
    sendB(HARDWARE_VERSION+0x30);
    sendB('.');
    sendB(REVNUM+0x30);
    sendB(' ');
    sendB('-');
    sendB(' ');

    initDisplay();
    msDelay(10);
    displayString(0,gBootStr);

    if ((GPIOA->IDR & GPIO_IDR_IDR_9) == 0)  {
		gVUsb = 0;
		sendString(gNoUsbVddStr);
		displayString(0x10,gRevStr);
		placeCursor(0x10+5);
		writeLcdData(HARDWARE_VERSION+0x30);
		writeLcdData('.');
		writeLcdData(REVNUM+0x30);
		msDelay(1000);
        displayString(0x10,gNoUsbVddStr);
		msDelay(1000);
    }
    else  {
    	gVUsb = 1;
		reInitUSB();
    }
#endif
    sendB(CRR);
    msDelay(100);

    activateSoundSet();
    msDelay(10);
    initDAC();
    initTimer6();
#ifndef MINHW
    initControlPanel();
#endif

    while(1)
    {
#ifdef MINHW
       recvword = pullUsart2RxBuf(); 	//reads from USB_RXBUF = USART2 IN
										//watchdog is fed in the background
        if (recvword)  {
            recvword &= 0x000000FF;
            if (CMD_Interpreter(recvword) == 0) sendB('?');
        }
        recvword = pullUsart1RxBuf(); //USART1,watchdog is fed in the background
        if (recvword)  {
            recvword &= 0x000000FF;
			USB_Midi_Scanner(recvword);
        }
#else
  #ifdef CUT_OFF
        recvword = pullUsart1RxBuf();
        if (recvword)  {
            recvword &= 0x000000FF;
            if (gUsartCOM==0) USART_Midi_Scanner(recvword);
            else if (CMD_Interpreter(recvword) == 0) {
				sendB('?');
				sendB(CRR);
			}
        }
  #else		//ORIG
        recvword = pullUsart1RxBuf();
        if (recvword)  {
            recvword &= 0x000000FF;
			USART_Midi_Scanner(recvword);
        }
		recvword = pullUsart2RxBuf();
        if (recvword)  {
            recvword &= 0x000000FF;
			if (CMD_Interpreter(recvword) == 0) {
				sendB('?');
				sendB(CRR);
			}
        }
  #endif
        if (guConfigured)  {
            recvword = pullUsbRxBuf();
            if (recvword)  {
            recvword &= 0x000000FF;
                if (guDeviceClass == CDC_CLASS)  {
                   if (CMD_Interpreter(recvword) == 0)  {
                   		sendB('?');
						sendB(CRR);
                   }
                }
                else USB_Midi_Scanner(recvword);
            }
        }
#endif
    }
}


//##########################################################################

// Misc Low Level Routines
// ************************

void servSystem(void)   // **************************************************
//not very time critical but permanently repeated functions for system control
//exclusively called in pullUsart1RxBuf and pullUsbRxBuf
//this way it is regularily served when idle waiting for external input
//but watchdog resets when it hangs somewhere else
//additionally slowed down to 100/50/2 times/second
// **************************************************************************
{
static uint32_t sSecCount, sServCount, sLowCount ;

    if (gSYSCOUNT)  {
       gSYSCOUNT= 0;					//restart SYSFREQ Hz service interval
//from here SYSFREQ Hz interval  (1600Hz)------------------------
#ifdef OSCI_ADSR
		GPIOA->BSRR = GPIO_BSRR_BR_0;           // bit PA0 (ADSR test pin) LOW
#endif
        ADSR_Process();

#ifndef MINHW
        evalEncoder();
#endif
            sLowCount++;
            if (sLowCount >= 4)  {			//restart 400Hz (2.5ms) interval
                sLowCount = 0;

                LowFreqProcess();
                sServCount++;
			if (sServCount >= 8)   { 			//restart 50Hz (20ms) interval
				sServCount = 0;
				IWDG->KR = 0x0000AAAA;//feed watchdog timeout ca. 1-2 seconds


#ifdef MINHW
//LED control

				if (gLEDCOUNT) gLEDCOUNT--;
				if (gLEDCOUNT==0)GPIOA->BSRR = GPIO_BSRR_BR_5;
                                        //bit PA5 (green Nucleo LED) OFF again
#else
				if (gpNoServPanel == 0)  controlPanel();
//LED control
//User Panel LEDs
				if (gLEDCOUNT) gLEDCOUNT--;
				if (gLEDCOUNT==0)  {
					if (gpPlusState) GPIOB->BSRR=GPIO_BSRR_BS_0;//green LED ON
					else  GPIOB->BSRR=GPIO_BSRR_BR_0;			//green LED OFF
					if (gpConfigPanel & 0x80) GPIOC->BSRR=GPIO_BSRR_BS_2;//ON
					else  GPIOC->BSRR=GPIO_BSRR_BR_2;			//red LED OFF
				}
#endif
				sSecCount++;
				if (sSecCount >= 50)  {
					sSecCount = 0;				//restart one second interval

#ifdef CUT_OFF
//check DIP switch and force USB re-init if new USB enumeration is necessary
					if ((gVUsb == 0) && (GPIOA->IDR & GPIO_IDR_IDR_9))  {
						gVUsb = 1;
						if ((GPIOA->IDR & mask2)==0) guDeviceClass = MIDI_CLASS;
						else  guDeviceClass = CDC_CLASS;
						reInitUSB();
					}
					if ((gVUsb == 1) && ((GPIOA->IDR & GPIO_IDR_IDR_9) == 0))  {
						gVUsb = 0;
						sendString(gUsbUnplugStr);
						msDelay(2000);
						updateDisplay();
					}
                    if (guConfigured)  {
                        if (GPIOA->IDR & mask2) {  //Dipswitch OFF
                            if (guDeviceClass == MIDI_CLASS)  {
								guDeviceClass = CDC_CLASS;
								reInitUSB();
                            }
                        }
                        else  {
							if (guDeviceClass == CDC_CLASS)  {
								guDeviceClass = MIDI_CLASS;
								reInitUSB();
							}
                        }
                    }
                    if (GPIOA->IDR & mask3) {
										//Dipswitch OFF, select MIDI behaviour
                        if (gUsartCOM)  {
                            gUsartCOM = 0;
                            initUsart1();    //switch to MIDI behaviour
                        }
                    }
                    else {			//Dipswitch ON,select serial COM behaviour
                        if (gUsartCOM == 0)  {
                            gUsartCOM = 1;
                            initUsart1(); //switch to COM behaviour
                        }
                    }
#endif
                }
            }
        }
#ifdef OSCI_ADSR
		GPIOA->BSRR = GPIO_BSRR_BS_0;           // bit PA0 (ADSR test pin) HIGH
#endif
    }
}


void LowFreqProcess(void)   // **********************************************
// called every 2.5ms (400Hz)
// **************************************************************************
{
float factor1, factor2, f0mod, oldF0ModBase;
int32_t index, value, delta;
int32_t F0LfoValue, vibLfoValue, vibFactor, chorusLfoValue, chorFactor;

	factor1 = (float)((gsValueFMO12 + 8*gsDETUNE)/16);	//+by Osc ADSR#3
	factor2 =(float)((gsValueFMO12 + gsValueFMO2 + 8*(gsDETUNE+gsDETUNE_2))/16);
    vibLfoValue = 0;
    chorusLfoValue = 0;
    F0LfoValue = 0;
    gsNewPWM1EdgeMod = gsPWM1Edge0;
    gsNewPWM2EdgeMod = gsPWM2Edge0;
    if (gsGlideHold)  {
        gsGlideHold--;
    }

//check LFO1
    if (gsValueFLfo)  {			//Lfo1 Freq additionally modulated by ADSR#4
        delta = (16*(int32_t)gLfo1Freq + gsValueFLfo) / 16;
        if (delta > 127)  delta = 127;
        if (delta < 0) delta = 0;
    }
    else delta = (int32_t)gLfo1Freq;
    if (delta)  {						//Lfo1 is ON and/or superposed by ADSR#4
        index = gLfo1Ptr;
        gLfo1Ptr += delta;							//step is = frequency
        if (gLfo1Ptr >= 4000) gLfo1Ptr -= 4000;
        value = (int32_t)gsWaveLfo1[index];
        if (gVibratoLfo1) vibLfoValue = value;		//Vibrato modulated by Lfo1
        if (gfF0modLfo2 == 0) F0LfoValue = value;	//F0Mid modulated by Lfo1
        if (gsOsc1ModLfo2 == 0)  {					//Osc1 modulated by Lfo1
			if ((gfRuntimeMask & 0x00000200) && gsOsc1PWM)  {
				gsNewPWM1EdgeMod=(float)(gsPWM1Edge0 +
												((int32_t)gsMOD_1*value)/2048);
				if (gsNewPWM1EdgeMod < 50.0) gsNewPWM1EdgeMod = 50.0;
											//void extremely short pos. pulses
			}
			else  factor1 += (float)((value*(int32_t)gsMOD_1)/6000);	//by LFO
        }
        if (gsOsc2ModLfo2 == 0)  {					//Osc2 modulated by Lfo1
			if ((gfRuntimeMask & 0x00000400) && gsOsc2PWM)  {
				gsNewPWM2EdgeMod=(float)(gsPWM2Edge0 +
												((int32_t)gsMOD_2*value)/2048);
				if (gsNewPWM2EdgeMod < 50.0) gsNewPWM2EdgeMod = 50.0;
											//avoid extremely short pos. pulses
			}
			else  factor2 += (float)((value * (int32_t)gsMOD_2)/6000);
        }
    }

//check LFO2
	if (gsFreeAdsr == 1)  {	//Lfo2 Freq.Mod
		delta = (16*(int32_t)gLfo2Freq + gsValueFLB) / 16;
        if (delta > 127)  delta = 127;
        if (delta < 0) delta = 0;
    }
    else delta = (int32_t)gLfo2Freq;
    if (delta)  {					//Lfo2 is ON and/or superposed by FreeADSR#1
        index = gLfo2Ptr;
        gLfo2Ptr += delta;							//step is = frequency
        if (gLfo2Ptr >= 4000) gLfo2Ptr -= 4000;
        value = (int32_t)gsWaveLfo2[index];
        if (gChorusLfo3 == 0) chorusLfoValue = value;
        if (gfF0modLfo2) F0LfoValue = value;	//F0Mid modulated by Lfo2

        if (gsOsc1ModLfo2)  {					//Osc1 modulated by Lfo2
			if ((gfRuntimeMask & 0x00000200) && gsOsc1PWM) {
				gsNewPWM1EdgeMod=(float)(gsPWM1Edge0 +
												((int32_t)gsMOD_1*value)/2048);
				if (gsNewPWM1EdgeMod < 50.0) gsNewPWM1EdgeMod = 50.0;
											//void extremely short pos. pulses
			}
			else  factor1 += (float)((value*(int32_t)gsMOD_1)/6000);	//by LFO
        }
        if (gsOsc2ModLfo2)  {					//Osc2 modulated by Lfo2
			if ((gfRuntimeMask & 0x00000400) && gsOsc2PWM)  {
				gsNewPWM2EdgeMod=(float)(gsPWM2Edge0 +
												((int32_t)gsMOD_2*value)/2048);
				if (gsNewPWM2EdgeMod < 50.0) gsNewPWM2EdgeMod = 50.0;
											//avoid extremely short pos. pulses
			}
			else  factor2 += (float)((value * (int32_t)gsMOD_2)/6000);
        }
    }

//check LFO3: no ADSR is assigned to it, it doesn't contol any oscillator,
    delta = (int32_t)gLfo3Freq;
    if (delta)  {									//Lfo3 is ON
        index = gLfo3Ptr;
        gLfo3Ptr += delta;							//step is = frequency
        if (gLfo3Ptr >= 4000) gLfo3Ptr -= 4000;
        value = (int32_t)gsWaveLfo3[index];
        if (gChorusLfo3) chorusLfoValue = value;
        if (gVibratoLfo1 == 0) vibLfoValue = value;	//Vibrato modulated by Lfo3
    }
    else {			//Lfo3  OFF, no ADSR assigned
        if (gChorusLfo3) chorusLfoValue = 0;
        if (gVibratoLfo1 == 0) vibLfoValue = 0;
    }

	gModCent1 = 1.0;	//reset obsolete values
	gModCent2 = 1.0;

	if (factor1 >= 0) gModCent1 = 1.0 + 3.9E-3 * factor1;
	else  gModCent1 = 1.0 + 2.61E-3 * factor1;
    if (gOldModCent1 != gModCent1)  {
		if (gModCent1 < 0.1) gModCent1 = 0.1;
		if (gModCent1 > 2.0) gModCent1 = 2.0;
        gOldModCent1 = gModCent1;
        gsRunStep0 = gsStep0 * gModCent1;
        gsRunStep1 = gsStep1 * gModCent1;
        gsRunStep2 = gsStep2 * gModCent1;
        gsRunStep3 = gsStep3 * gModCent1;
        gsRunStep4 = gsStep4 * gModCent1;
        gsRunStep5 = gsStep5 * gModCent1;
        gsRunStep6 = gsStep6 * gModCent1;
        gsRunStep7 = gsStep7 * gModCent1;
        gsRunStep8 = gsStep8 * gModCent1;
        gsRunStep9 = gsStep9 * gModCent1;
    }

	if (factor2 >= 0) gModCent2 = 1.0 + 3.9E-3 * factor2;
	else  gModCent2 = 1.0 + 2.61E-3 * factor2;
    if (gOldModCent2 != gModCent2)  {
		if (gModCent2 < 0.1) gModCent2 = 0.1;
		if (gModCent2 > 2.0) gModCent2 = 2.0;
        gOldModCent2 = gModCent2;
        gsRunStep0X = gsStep0X * gModCent2;
        gsRunStep1X = gsStep1X * gModCent2;
        gsRunStep2X = gsStep2X * gModCent2;
        gsRunStep3X = gsStep3X * gModCent2;
        gsRunStep4X = gsStep4X * gModCent2;
        gsRunStep5X = gsStep5X * gModCent2;
        gsRunStep6X = gsStep6X * gModCent2;
        gsRunStep7X = gsStep7X * gModCent2;
        gsRunStep8X = gsStep8X * gModCent2;
        gsRunStep9X = gsStep9X * gModCent2;
    }

//filter0 freq.modulated by Lfo1 or Lfo2
	oldF0ModBase = gfF0ModBase;
    if ((gfF0mod) && (gfRuntimeMask & 0x00000001)) {
        if ((gfF0modLfo2 == 0) && (gLfo1Freq == 0)) f0mod = 0.0;
        else if ((gfF0modLfo2) && (gLfo2Freq == 0)) f0mod = 0.0;
        else  {
            if (F0LfoValue >= 0)
                f0mod = (float)((int32_t)gfF0mod * F0LfoValue) /819200.;
													//range 0... 1.0
            else  f0mod = (float)((int32_t)gfF0mod * F0LfoValue) /1638400.;
													//range 0...-0.5
        }
        gfF0ModBase = 1.0 + f0mod;
    }
    else gfF0ModBase = 1.0;

	if ((gsNoteCount <= gNTrigger[gSoundSet]) &&
			((gfF0ModBase != oldF0ModBase) || (gsPhaseF0Pk) || (gsPhaseF0QF))) {

		if ((gsPhase0) || (gsPhase0X))  {
			gsBaseFreqF0=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftF0));
			if(gsPhaseF0M == 0)  tempInitFilter(0,gsBaseFreqF0);
		}
		if ((gsPhase1) || (gsPhase1X))  {
			gsBaseFreqC1=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC1));
			if(gsPhaseC1 == 0)  tempInitFilter(5,gsBaseFreqC1);
		}
		if ((gsPhase2) || (gsPhase2X))  {
			gsBaseFreqC2=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC2));
			if(gsPhaseC2 == 0)  tempInitFilter(6,gsBaseFreqC2);
		}
		if ((gsPhase3) || (gsPhase3X))  {
			gsBaseFreqC3=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC3));
			if(gsPhaseC3 == 0)  tempInitFilter(7,gsBaseFreqC3);
		}
		if ((gsPhase4) || (gsPhase4X))  {
			gsBaseFreqC4=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC4));
			if(gsPhaseC4 == 0)  tempInitFilter(8,gsBaseFreqC4);
		}
		if ((gsPhase5) || (gsPhase5X))  {
			gsBaseFreqC5=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC5));
			if(gsPhaseC5 == 0)  tempInitFilter(9,gsBaseFreqC5);
		}
		if ((gsPhase6) || (gsPhase6X))  {
			gsBaseFreqC6=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC6));
			if(gsPhaseC6 == 0)  tempInitFilter(10,gsBaseFreqC6);
		}
		if ((gsPhase7) || (gsPhase7X))  {
			gsBaseFreqC7=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC7));
			if(gsPhaseC7 == 0)  tempInitFilter(11,gsBaseFreqC7);
		}
		if ((gsPhase8) || (gsPhase8X))  {
			gsBaseFreqC8=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC8));
			if(gsPhaseC8 == 0)  tempInitFilter(12,gsBaseFreqC8);
		}
		if ((gsPhase9) || (gsPhase9X))  {
			gsBaseFreqC9=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC9));
			if(gsPhaseC9 == 0)  tempInitFilter(13,gsBaseFreqC9);
		}
	}

//Chorus may be modulated by Lfo2 or Lfo3 or by FreeAdsr#4
	chorFactor = (chorusLfoValue * (int32_t)gChorusWidth) / 8192;
												//ChoLfoValue range +/-8192
	if (gsFreeAdsr == 4)  chorFactor = (32 * chorFactor + gsValueChor) / 32;
					//then FreeAdsr controls Chorus in addition to Lfo2 or Lfo3

	gChorus = (int32_t)gDelayOffset + chorFactor;
							//delay time ca. in ms expressed in buffer length
	if ((gChorus < 0) || (gChorus >= 0x10000)) gChorus = 0;
	if (gChorus > MAXCHORUS) gChorus = MAXCHORUS;
//vibrato works independent of Volume
	gVibratoVolume = 100;					//100% default without Vibrato
	vibFactor = (32*(int32_t)gVibrato + gsValueVib)/32;//Adsr#5 controls Vibrato
	if (vibFactor > 100) vibFactor = 100;
	if (vibFactor < 0) vibFactor = 0;
	gVibratoVolume += (vibLfoValue * vibFactor) / 8192;
											//VibLfoValue: range = +/- 8192
	if (gVibratoVolume > 200) gVibratoVolume = 200;
	if (gVibratoVolume < 0) gVibratoVolume = 0;
}

#ifdef CUT_OFF
void reInitUSB(void)  // ****************************************************
// **************************************************************************
{
const uint8_t UsbStr[] =  {"USB:"};

	if (guConfigured)  {
		stopUSB();
		msDelay(20);
	}
	initUSB();

	gTIMEOUT = 2000;
	while (gTIMEOUT && (guConfigured == 0))  IWDG->KR = 0x0000AAAA;
										//feed watchdog,timeout ca. 1-2 seconds
    if (guConfigured)  {
		displayString(0x10,gRevStr);
		placeCursor(0x10+4);
		writeLcdData(HARDWARE_VERSION+0x30);
		writeLcdData('.');
		writeLcdData(REVNUM+0x30);
		writeLcdData('.');
		displayString(0x10+8,UsbStr);
		if (gUsartCOM)  sendString(UsbStr);
		if (guDeviceClass == CDC_CLASS)  {
			displayString(0x10+12,gComStr);
			if  (gUsartCOM)  {
				writeLcdData(' ');
				sendString(gComStr);
				sendB(' ');
			}
		}
		else  {
			displayString(0x10+12,gMidiStr);
			writeLcdData('I');
			if (gUsartCOM)  {
				sendString(gMidiStr);
				sendB('I');
			}
		}
    }
	else  {
		displayString(0x10,gNoUsbConnStr);
		if (gUsartCOM)  sendString(gNoUsbConnStr);
	}
	if (gUsartCOM)  sendB(CRR);
	msDelay(2000);
	updateDisplay();
}
#else
void reInitUSB(void)  // ****************************************************
// **************************************************************************
{
const uint8_t UsbStr[] =  {"USB:"};

	if (guConfigured)  {
		stopUSB();
		msDelay(20);
	}
	initUSB();

	gTIMEOUT = 2000;
	while (gTIMEOUT && (guConfigured == 0))  IWDG->KR = 0x0000AAAA;
										//feed watchdog,timeout ca. 1-2 seconds
    if (guConfigured)  {
		displayString(0x10,gRevStr);
		placeCursor(0x10+4);
		writeLcdData(HARDWARE_VERSION+0x30);
		writeLcdData('.');
		writeLcdData(REVNUM+0x30);
		writeLcdData('.');
		displayString(0x10+8,UsbStr);
		sendString(UsbStr);
		if (guDeviceClass == CDC_CLASS)  {
			displayString(0x10+12,gComStr);
			writeLcdData(' ');
		sendString(gComStr);
		sendB(' ');
		}
		else  {
			displayString(0x10+12,gMidiStr);
			writeLcdData('I');
			sendString(gMidiStr);
			sendB('I');
		}
    }
	else  {
		displayString(0x10,gNoUsbConnStr);
		sendString(gNoUsbConnStr);
	}
	sendB(CRR);
	msDelay(2000);
	updateDisplay();
}
#endif
void LED_Off(void)  // ******************************************************
// built as subroutine to supply service to "usb-otg-4611" hardware independent
// **************************************************************************
{
	if (gpConfigPanel & 0x80) GPIOC->BSRR=GPIO_BSRR_BR_2;	//red LED OFF
	else  GPIOC->BSRR=GPIO_BSRR_BS_2;						//red LED ON
	gLEDCOUNT = 10 ;						//LED OFF time =*20ms = 200ms
}


void msDelay(uint32_t countdown)   // ***************************************
//blocking delay by 'count' ms. Max 20 seconds!
// **************************************************************************
{
uint32_t subCount;

    if (countdown == 0) return;
    if (countdown > 2) countdown = (countdown * SYSFREQ) / 1000;
												//SysTick is 0.625 ms
    if (countdown > 20000) countdown = 20000;	//max.20s
    gDELAYCOUNT = (uint16_t)countdown;
    while(gDELAYCOUNT)  {
        IWDG->KR = 0x0000AAAA;      //feed watchdog, timeout ca. 1-2 seconds
		if (gSYSCOUNT)  {
		   gSYSCOUNT= 0;			//restart SYSFREQ Hz service interval
	//from here SYSFREQ Hz interval  (1600Hz)------------------------
#ifdef OSCI_ADSR
			GPIOA->BSRR = GPIO_BSRR_BR_0;	// bit PA0 (ADSR test pin) LOW
#endif
			ADSR_Process();
			subCount++;
			if (subCount >= 4)  {			//restart 400Hz (2.5ms) interval
				subCount = 0;
				LowFreqProcess();
			}
#ifdef OSCI_ADSR
			GPIOA->BSRR = GPIO_BSRR_BS_0;	// bit PA0 (ADSR test pin) HIGH
#endif
		}
    }
}


void wait100us(void)   // *******************************************************
// blocking delay ca 100 microseconds
// *************************************************************************
{
uint32_t countdown;

    countdown = 17;
    while (countdown)  {
        wait6us();
        countdown-- ;
    }
}


void wait6us(void)   // *******************************************************
// blocking delay ca 6us
// *************************************************************************
{
uint32_t countdown;

	countdown = (uint32_t)SystemCoreClock/1500000;
    while (countdown)  countdown-- ;
}


void wait1us(void)   /*********************************************/
//blocking delay by about 1 microsecond
/***************************************************************************/
{
uint32_t countdown;


	countdown = (uint32_t)SystemCoreClock/16000000;
    while (countdown)  countdown-- ;
}

//###########################################################################

void popFlashSounds(void)   // **********************************************
//only the first  words =  bytes of the sector are read
// **************************************************************************
{
uint32_t k, soundset;
uint32_t temp;

    gFlashPtr = (uint32_t *)SECTOR7;
	temp = ~(*gFlashPtr);
	gFlashPtr++;
//one's complement for better handling of empty flash
	gVid = temp & 0x0000FFFF;
	temp = ~(*gFlashPtr);
	gFlashPtr++;
	gPid = temp & 0x0000FFFF;
	temp = ~(*gFlashPtr);
	gFlashPtr++;
	gChannel = temp & 0x0000000F;
	gDrumsChannel = (temp >> 8) & 0x0000000F;
#ifndef CUT_OFF
	guDeviceClass = (temp >> 16) & 0x00000001;
#endif
    if (gVid == 0) gVid = DEFAULT_VID;
    if (gPid == 0) gPid = DEFAULT_PID;
    if ((gChannel == gDrumsChannel) && (gDrumsChannel == 0)) gDrumsChannel = 9;
										//seems to be virgin->default Channel10

	for (k = 0; k < N_SOUNDS; k++)  {
		pop1Sound(k);
		IWDG->KR = 0x0000AAAA;			//feed watchdog
	}

// User Waves

    for (k = 0; k < 1024; k+=2) {				//2*16bit packed
        temp = *gFlashPtr;                       //NOT one's complement !
        gFlashPtr++;
        if (temp == 0xFFFFFFFF)  {              //cannot be a real user value
            gsUserWave0[k] = (4*calcTri(k,288))/3;
            gsUserWave0[k+1] = (4*calcTri(k+1,288))/3;
        }
        else  {
            gsUserWave0[k] = (int16_t)((temp >> 16) & 0x0000FFFF);
            gsUserWave0[k+1] = (int16_t)(temp & 0x0000FFFF);
        }
    }
	IWDG->KR = 0x0000AAAA;			//feed watchdog

    for (k = 0; k < 1024; k+=2) {
        temp = *gFlashPtr;
        gFlashPtr++;
        if (temp == 0xFFFFFFFF)  {
            gsUserWave1[k] = calcTri(k,508); //gsSawtooth[k];	//Sawtooth
            gsUserWave1[k+1] = calcTri(k+1,508); //gsSawtooth[k+1];	//Sawtooth
        }
        else  {
            gsUserWave1[k] = (int16_t)((temp >> 16) & 0x0000FFFF);
            gsUserWave1[k+1] = (int16_t)(temp & 0x0000FFFF);
        }
    }
	IWDG->KR = 0x0000AAAA;			//feed watchdog

    for (k = 0; k < 1024; k+=2) {
        temp = (*gFlashPtr);
        gFlashPtr++;
        if (temp == 0xFFFFFFFF)  {
            gsUserWave2[k] = calcTrapez(k,512,36);
            gsUserWave2[k+1] = calcTrapez(k+1,512,36);
        }
        else  {
            gsUserWave2[k] = (int16_t)((temp >> 16) & 0x0000FFFF);
            gsUserWave2[k+1] = (int16_t)(temp & 0x0000FFFF);
        }
    }
	IWDG->KR = 0x0000AAAA;			//feed watchdog

    for (k = 0; k < 1024; k+=2) {
        temp = (*gFlashPtr);
        gFlashPtr++;
        if (temp == 0xFFFFFFFF)  {
            gsUserWave3[k] = gsExpSin[k];
            gsUserWave3[k+1] = gsExpSin[k + 1];
        }
        else  {
            gsUserWave3[k] = (int16_t)((temp >> 16) & 0x0000FFFF);
            gsUserWave3[k+1] = (int16_t)(temp & 0x0000FFFF);
        }
    }
	IWDG->KR = 0x0000AAAA;			//feed watchdog
//terminate loading
}


void pop1Sound(uint32_t snd)   // *******************************************
// **************************************************************************
{
uint32_t k, n, temp, index;
uint32_t *mirrorPtr;

//Osc Presets, Waveform, Transpose, Detune, Glide(Portamento)
	temp = ~(*gFlashPtr);    //1'complement=better handling of empty flash
	gFlashPtr++;
	if (temp == 0) temp = 0;                //0=Sinus and Lfo1 = default
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gsWAVEFORM_1Preset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 2;                //2=Asymm.Tri and Lfo1=default
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gsWAVEFORM_2Preset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 0x21030012;		//=default range=333 slope=36(18*2)
	gsFreeSquare1[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 0x0E000008;		//=default range=140 slope=16(8*2)
	gsFreeSquare2[snd] = temp;

	gsGLIDEPreset[snd] = ~(*gFlashPtr);
	gFlashPtr++;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 36;               //=default value +/-0 semitones
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gsTRANSPreset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 36;               //=default value +/-0 semitones
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gsTRANS_2Preset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 64;               //=default value no detune
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gsDETUNEPreset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 64;               //=default value no detune
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gsDETUNE_2Preset[snd] = temp;

//Lfo1,2,3
	gLfo1WavePreset[snd] = ~(*gFlashPtr);	//Sine is default Lfo1 wave
	gFlashPtr++;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 8;				//=default Lfo1 Freq = 1Hz
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gLfo1FreqPreset[snd] = temp;

	gLfo2WavePreset[snd] = ~(*gFlashPtr);	//Sine is default Lfo2 wave
	gFlashPtr++;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 1;				//=default Lfo2 Freq = 0.1Hz
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gLfo2FreqPreset[snd] = temp;			//Lfo2 freq default = 0

	gLfo3WavePreset[snd] = ~(*gFlashPtr);	//Sine is default Lfo3 wave
	gFlashPtr++;

	gLfo3FreqPreset[snd] = ~(*gFlashPtr);	//Lfo3 freq default = 0
	gFlashPtr++;

	gsMOD_1Preset[snd] = ~(*gFlashPtr);		//default= zero FreqMod
	gFlashPtr++;							//bit7set = PWM, else Freq.Mod

	gsMOD_2Preset[snd] = ~(*gFlashPtr);		//default= zero FreqMod
	gFlashPtr++;							//bit7set = PWM, else Freq.Mod

//ADSR Presets
	gsFreeAdsrPreset[snd]= ~(*gFlashPtr); 	//default=0:
	gFlashPtr++;								//assigned to Filt.3=Noise

	for (n = 0; n < N_ASTRUCT; n++)  {
		index = (snd * N_ASTRUCT) + n;

		temp = ~(*gFlashPtr);	//1's complement=better handling of empty flash
		gFlashPtr++;
		if ((snd == 1) && (temp == 0)) {
			if (n == 0)  temp = 0x02690080;//OSC1:Att=0.02s Dec=2.0s SustH= 0
			else if (n == 1) temp = 0x00280080;//OSC1:Att=0 Dec=0.4s SustH= 0
			else if (n == 8) temp = 0x020000E4;//Gain:Att=0.02 Dec=0 SustH=100
			else  temp  =0x02640080; //Attack=0.02s,Decay=1.0s,Hold+SUSTAIN = 0
		}
		else if (temp == 0)  {
			if  ((n <= 1) || (n == 8)) temp = 0x020000E4;
								//=OSC1,2 and Gain default Hold+SUSTAIN = 100
			else  temp  =0x02640080; //Attack=0.02s,Decay=1.0s,Hold+SUSTAIN = 0
		}
		if (temp== 0xFFFFFFFF)  temp = 0;
		gsADSR[index].ATDECSUS = temp;

		temp = ~(*gFlashPtr); 	//1's complement=better handling of empty flash
		gFlashPtr++;
		if ((snd == 1) && (temp == 0)) {		//Release always 0.02s
			if (n == 0)  temp = 0x0002403C;		//OSC1 EnvFIX=64 Velo=32
			else if  (n == 1)  temp =0x0002183C;//OSC2 EnvFIX=24 Velo=32
			else if  ((n == 2) || (n == 8))  temp =0x0002101C;
												//Noise and Gain Envelope = 16
			else  temp = 0x0002401C;			//=middle of range 0..127 is 64
		}
		else if (temp == 0)  {					//Release always 0.02s {
			if (n == 0)  temp = 0x0002401C;		//=default OSC1 Envelope FIX=64
			else if  (n == 1)  {
				if (snd == 0)  temp =0x0002001C; //Osc2@SoundSet0:Envelope = 0
				else  temp =0x0002201C; 		//else Osc2 Envelope FIX=32
			}
			else if  ((n == 2) || (n == 8))  temp =0x0002101C;
												//Noise and Gain Envelope = 16
			else  temp = 0x0002401C;		//=middle of range 0..127 is 64
		}
		if (temp == 0xFFFFFFFF) temp = 0;
		gsADSR[index].REFIXVEL = temp;
	}

// Filter Parameter
	mirrorPtr = (uint32_t *)(&(gfIIRPack[snd * N_FILTER].TYPE));

	for (n = 0; n < N_FILTER; n++)  {
								//1's complement=better handling of empty flash
        *mirrorPtr = ~(*gFlashPtr);	//TYPE
        mirrorPtr++;
        gFlashPtr++;
		*mirrorPtr = ~(*gFlashPtr);	//FREQ
        mirrorPtr++;
        gFlashPtr++;
        *mirrorPtr = ~(*gFlashPtr);	//QFACTOR
        mirrorPtr++;
        gFlashPtr++;
        *mirrorPtr = ~(*gFlashPtr);	//PEAK
        mirrorPtr++;
        gFlashPtr++;
    }

//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bit 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)
	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (snd == 1)  temp |= mask7;	//set bit7=activate "Osc2 sync",clear sound
	if (temp == 0xFFFFFFFF)  temp= 0;   //preset value = 0
	gfStageMask[snd] = temp; 	//unpacked at "activateSoundSet()"

//Keytrack
	gsKeytrack0Preset[snd] = ~(*gFlashPtr);    //default=0: no keytrack
	gFlashPtr++;

//Routing, Gain, Filter Modulation, Balance, Volume Presets
	temp = ~(*gFlashPtr);
	gFlashPtr++;
//					default temp = 0 -->OSC1+OSC2 into filter0,but not noise
	if (snd == 1)  temp = 0x00040004;	//OSC1 into filter1 OSC2 into filter0
	if (temp == 0xFFFFFFFF)  temp= 0;   //preset value = 0
	gaRoutePreset[snd]= temp;			//bits23..16=FLEX F0N
										//bits 7..0= EASY F0N

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 1;         	//default: Gain=1
	if (temp > 50)  temp= 50;
	gfGainPreset[snd]= temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 0x80;         //default: Lfo2, gfF0mod=0
	if (temp == 0xFFFFFFFF)  temp= 0;   //preset value = 0
	gfF0modPreset[snd]= temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 1;         	//default: 1, preset=0 not implemented
	gNTrigger[snd]= temp;				//effects multi key trigger limit

	temp = ~(*gFlashPtr);
	gFlashPtr++;                     	//default:
	if (temp == 0)
		if (snd== 1)  temp = 0x00400040;//lowTreble Flex bits31-24,Easy bits15-8
		else temp = 0x40404040; 		//default flat bass/treble characterist.
	if (temp == 0xFFFFFFFF)  temp= 0;	//preset value = 0
	gfBasTrebPreset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;           //default: -6dB Volume setting
	if (temp == 0xFFFFFFFF)  temp= 0;   //preset value = 0
	gVolumePreset[snd] = temp;

//Delay Parameter
	gDelayTimePreset[snd] = ~(*gFlashPtr);
	gFlashPtr++;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 0;            //default: user delay level = 0
	if (temp == 0xFFFFFFFF)  temp= 0;   //preset value = 0
	gDelayLevelPreset[snd] = (int32_t)temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 0;            //default: user feedback level = 0
	if (temp == 0xFFFFFFFF)  temp= 0;   //preset value = 0
	gFeedbackPreset[snd] = (int32_t)temp;

	gChorusPreset[snd] = ~(*gFlashPtr); 	//Bit7=gChorusLfo3 (default=0,Lfo2)
	gFlashPtr++;							//depth default=0

	gVibratoPreset[snd]  = ~(*gFlashPtr);	//Bit7=gVibratoLfo1 (default=0,Lfo2)
	gFlashPtr++;							//depth default=0

//Drums Parameter
	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;           	//default: = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0; 		//preset value = 0
	gdDrumsVolPreset[snd] = (int32_t)temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp =40;            	//default:  = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0;   	//preset value = 0
	gdKickVolPreset[snd] = (int32_t)temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;            	//default: = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0;   	//preset value = 0
	gdHiTomVolPreset[snd] = (int32_t)temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;               //default: = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gdSnareVolPreset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;               //default: = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gdRimVolPreset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;               //default: = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gdClapVolPreset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;               //default: = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gdHihatVolPreset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;               //default: = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0;     	//preset value = 0
	gdCymbalVolPreset[snd] = temp;

	temp = ~(*gFlashPtr);
	gFlashPtr++;
	if (temp == 0) temp = 40;               //default: = 0dB
	if (temp == 0xFFFFFFFF)  temp= 0;       //preset value = 0
	gdBellVolPreset[snd] = temp;
}


uint32_t pushFlashSounds(void)   // ****************************************
//only the first xxx words = yyyy bytes of the sector are written
// **************************************************************************
{
uint32_t k, temp;

//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB

    while (FLASH->SR & FLASH_SR_BSY) ;  // IWDG->KR =0x0000AAAA;
                                        //feed watchdog
                                        //wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
            FLASH->KEYR = KEY1;
            FLASH->KEYR = KEY2;
    }
    while (FLASH->SR & FLASH_SR_BSY) IWDG->KR =0x0000AAAA;
                                //feed watchdog and wait until flash is not busy

//erase SECTOR7
    FLASH->CR = ((1<<9) | (7<<3) | FLASH_CR_EOPIE |FLASH_CR_SER);
                                //enable Erase Sector7
                                //for 32bit parallelism: set bit9:PSIZE=10
                                //EOPIE is set formally to get SR_EOP set
    FLASH->CR |= FLASH_CR_STRT;
    while (FLASH->SR & FLASH_SR_BSY) ;
    if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
    FLASH->CR &= ~FLASH_CR_SER;

//write mirror into Sector7
    gFlashPtr = (uint32_t *)SECTOR7;

    FLASH->CR |= FLASH_CR_PG;

//global parameters
	temp = (~gVid << 16) | (gVid & 0x0000FFFF);
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = (~gPid << 16) | (gPid & 0x0000FFFF);
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = (guDeviceClass << 16) | (gDrumsChannel << 8) | gChannel ;
    *gFlashPtr = ~(temp);	//CUT_OFF:guDeviceClass is dummy for compatibility
    gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	for (k = 0; k < N_SOUNDS; k++)  {
		IWDG->KR = 0x0000AAAA;
		if (push1Sound(k) == 0)  {
			sendB('E');
			sendB('r');
			sendB('r');
			sendX8(k);
			return 0;
		}
	}

//user Waves
    for (k = 0; k < 1024; k+=2) {
        *gFlashPtr = (((uint32_t)gsUserWave0[k] << 16) |
                                (((uint32_t)gsUserWave0[k + 1]) & 0x0000FFFF));
        gFlashPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
    }

    for (k = 0; k < 1024; k+=2) {
        *gFlashPtr = (((uint32_t)gsUserWave1[k] << 16) |
                                (((uint32_t)gsUserWave1[k + 1]) & 0x0000FFFF));
        gFlashPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
    }

    for (k = 0; k < 1024; k+=2) {
        *gFlashPtr = (((uint32_t)gsUserWave2[k] << 16) |
                                (((uint32_t)gsUserWave2[k + 1]) & 0x0000FFFF));
        gFlashPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
    }

    for (k = 0; k < 1024; k+=2) {
        *gFlashPtr = (((uint32_t)gsUserWave3[k] << 16) |
                                (((uint32_t)gsUserWave3[k + 1]) & 0x0000FFFF));
        gFlashPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
    }

 //terminate programming
    FLASH->CR &= ~FLASH_CR_PG;
//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(500);
    return 1;
}


uint32_t push1Sound(uint32_t snd)   // **************************************
// **************************************************************************
{
uint32_t k, n, index, temp;
uint32_t * mirrorPtr;

//save one's complement of all entries for better handling of empty flash
//OSC Presets, Waveform, Transpose, Detune, Portamento
	temp = gsWAVEFORM_1Preset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gsWAVEFORM_2Preset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~(gsFreeSquare1[snd]);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~(gsFreeSquare2[snd]);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~(gsGLIDEPreset[snd]);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gsTRANSPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gsTRANS_2Preset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gsDETUNEPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gsDETUNE_2Preset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

//Lfo1,2,3
	*gFlashPtr = ~(gLfo1WavePreset[snd]);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gLfo1FreqPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~(gLfo2WavePreset[snd]);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gLfo2FreqPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~(gLfo3WavePreset[snd]);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~(gLfo3FreqPreset[snd]);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gsMOD_1Preset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gsMOD_2Preset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~gsFreeAdsrPreset[snd];
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

//ADSR Presets
	for (k = 0; k < N_ASTRUCT; k++)  {
		index = (snd * N_ASTRUCT) + k;

		temp = gsADSR[index].ATDECSUS;
		if (temp == 0) temp = 0xFFFFFFFF;		//default != 0
		*gFlashPtr = ~(temp);
						//one's complement for better handling of empty flash
		gFlashPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
		temp = gsADSR[index].REFIXVEL;
		if (temp == 0) temp = 0xFFFFFFFF;		//default != 0
		*gFlashPtr = ~(temp);
		gFlashPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
	}

//Filter parameter
    mirrorPtr = (uint32_t *)(&(gfIIRPack[snd * N_FILTER].TYPE));
    for (k =0;k < N_FILTER ;k++) {
						//one's complement for better handling of empty flash
        *gFlashPtr = ~(*mirrorPtr);			//TYPE
        gFlashPtr++;
        mirrorPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
		*gFlashPtr = ~(*mirrorPtr);			//FREQ
        gFlashPtr++;
        mirrorPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
		*gFlashPtr = ~(*mirrorPtr);			//QFACTOR
        gFlashPtr++;
        mirrorPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
		*gFlashPtr = ~(*mirrorPtr);			//PEAK
        gFlashPtr++;
        mirrorPtr++;
		while (FLASH->SR & FLASH_SR_BSY) ;
		if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
    }

//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bit 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)

	*gFlashPtr = ~gfStageMask[snd];
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

//Keytrack
	*gFlashPtr = ~gsKeytrack0Preset[snd];
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

//Routing, Gain, Filter Modulation, Balance , Volume Presets
	temp = gaRoutePreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~gfGainPreset[snd];
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gfF0modPreset[snd];            	//bit7 is LFO selection flag
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~gNTrigger[snd];			//effects multi key trigger limit
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gfBasTrebPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gVolumePreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

//Delay parameter
	*gFlashPtr = ~gDelayTimePreset[snd];
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = (uint32_t)gDelayLevelPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = (uint32_t)gFeedbackPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~(gChorusPreset[snd]);		//bit7=gChorusLfo3 (default=0,Lfo2)
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	*gFlashPtr = ~(gVibratoPreset[snd]);	//bit7=gVibratoLfo1(default=0,Lfo3)
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

//Drums parameter
	temp = (uint32_t)gdDrumsVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = (uint32_t)gdKickVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = (uint32_t)gdHiTomVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gdSnareVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gdRimVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gdClapVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gdHihatVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gdCymbalVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;

	temp = gdBellVolPreset[snd];
	if (temp == 0) temp = 0xFFFFFFFF;       //default != 0
	*gFlashPtr = ~(temp);
	gFlashPtr++;
	while (FLASH->SR & FLASH_SR_BSY) ;
	if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
	return 1;
}


void eraseFlashSounds(void)   // ********************************************
//only the first xxx words = yyyy bytes of the sector are written
// **************************************************************************
{
//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB

    while (FLASH->SR & FLASH_SR_BSY) ;  // IWDG->KR =0x0000AAAA;
                                        //feed watchdog
                                        //wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
            FLASH->KEYR = KEY1;
            FLASH->KEYR = KEY2;
    }
    while (FLASH->SR & FLASH_SR_BSY) IWDG->KR =0x0000AAAA;
                                //feed watchdog and wait until flash is not busy

//erase SECTOR7
    FLASH->CR = ((1<<9) | (7<<3) | FLASH_CR_EOPIE |FLASH_CR_SER);
                                //enable Erase Sector7
                                //for 32bit parallelism: set bit9:PSIZE=10
                                //EOPIE is set formally to get SR_EOP set

    FLASH->CR |= FLASH_CR_STRT;

    while (FLASH->SR & FLASH_SR_BSY) ;

    if (FLASH->SR & FLASH_SR_EOP)  FLASH->SR |= FLASH_SR_EOP;
    FLASH->CR &= ~FLASH_CR_SER;
//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(500);
}

//###########################################################################

//USART
//*******************

#ifdef MINHW
void initUsart1(void) // ****************************************************
// USART1-RxD is PortB,7
// RX only, no TX support
// **************************************************************************
{
    RCC->APB2ENR |=  RCC_APB2ENR_USART1EN ;   // USART1 clock enable

//set PB7 alternate function mode (10=Alternate function mode)
	GPIOB->MODER &= ~(GPIO_MODER_MODER7_0);
	GPIOB->MODER |= GPIO_MODER_MODER7_1;
//USART1 is AF7  PortB
    GPIOB->AFR[0] &= 0x0FFFFFFF;        //PB7 is TX
    GPIOB->AFR[0] |= 0x70000000;
    gUSART1_RXHEAD = gUSART1_RXBUF;
    gUSART1_RXTAIL = gUSART1_RXBUF;

	USART1->BRR = 0x0540;	//always MIDI 31250 Baud: 84.0 at 42 MHz APB2 Clock
    USART1->CR2 = 0;			//default:1 Stopbit
    USART1->CR3 = 0;			//default:no Handshake
    USART1->CR1 = USART_CR1_UE | USART_CR1_RXNEIE | USART_CR1_RE;
                                                       //8Bit,no Parity,RX Int
    NVIC_EnableIRQ(USART1_IRQn);
}

#else

void initUsart1(void) // ****************************************************
// USART1-RxD is PortA,10
// USART1-TxD is PortB,6
// **************************************************************************
{
    RCC->APB2ENR |=  RCC_APB2ENR_USART1EN ;   // USART1 clock enable

//set PA10 alternate function mode (10=Alternate function mode)
	GPIOA->MODER &= ~(GPIO_MODER_MODER10_0);
	GPIOA->MODER |= GPIO_MODER_MODER10_1;
//set PB6 alternate function mode (10=Alternate function mode)
	GPIOB->MODER &= ~(GPIO_MODER_MODER6_0);
	GPIOB->MODER |= GPIO_MODER_MODER6_1;

//USART1 is AF7 on PortA and PortB
    GPIOA->AFR[1] &= 0xFFFFF0FF;        //PA10 is RX
    GPIOA->AFR[1] |= 0x00000700;
    GPIOB->AFR[0] &= 0xF0FFFFFF;        //PB6 is TX
    GPIOB->AFR[0] |= 0x07000000;

    gUSART1_RXHEAD = gUSART1_RXBUF;
    gUSART1_RXTAIL = gUSART1_RXBUF;
  #ifdef CUT_OFF
	if (gUsartCOM) USART1->BRR = 0x016D;//115200Baud: 22.79 at 42 MHz APB2 Clock
	else USART1->BRR = 0x0540;			//31250 Baud: 84.0 at 42 MHz APB2 Clock
  #else
	USART1->BRR = 0x0540;	//ORIG: always 31250Baud: 84.0 at 42 MHz APB2 Clock
  #endif
    USART1->CR2 = 0;			//default:1 Stopbit
    USART1->CR3 = 0;			//default:no Handshake
    USART1->CR1 = USART_CR1_UE | USART_CR1_RXNEIE |USART_CR1_TE | USART_CR1_RE;
                                                       //8Bit,no Parity,RX Int
    NVIC_EnableIRQ(USART1_IRQn);
}
#endif

#ifdef CUT_OFF
void sendB(uint8_t data) // *************************************************
//send 1 raw byte on active COM channels
// **************************************************************************
{
	if (guConfigured && (guDeviceClass == CDC_CLASS)) sendUsbByte(data);
	if (gUsartCOM)  {
		while(!(USART1->SR & USART_SR_TXE)) ;
		USART1->DR = data;
	}
}

void sendM(uint8_t data) // *************************************************
//send 1 raw byte on active MIDI channel
// **************************************************************************
{
    if (guConfigured)  {
		if (guDeviceClass == CDC_CLASS) sendUsbByte(data);//esp.for sound dump
		else enCodeMidi(data);
    }
	while(!(USART1->SR & USART_SR_TXE)) ;
	USART1->DR = data;		//always MIDI OUT for sendM(), don't mind gUsartCOM
}

#else

void initUsart2(void) // ****************************************************
// USART2-TxD is PortA,2
// USART2-RxD is PortA,3
// **************************************************************************
{
//PortA GPIO clock already enabled
    RCC->APB1ENR |=  RCC_APB1ENR_USART2EN ;   // USART2 clock enable

	GPIOA->MODER &= ~(GPIO_MODER_MODER2_0 | // Set pin 2,3 in alt. function mode
	                 GPIO_MODER_MODER3_0 ) ;    // 10=Alternate function mode
	GPIOA->MODER |= (GPIO_MODER_MODER2_1 |  // Set pin 2,3 in alt. function mode
	                 GPIO_MODER_MODER3_1 ) ;    // 10=Alternate function mode

    GPIOA->AFR[0] &= 0xFFFF00FF;
    GPIOA->AFR[0] |= 0x00007700;

    gUSART2_RXHEAD = gUSART2_RXBUF;			//ST-LINK is connected
    gUSART2_RXTAIL = gUSART2_RXBUF;			//USB_RX/TX is associated w. USART2

//the only possible baudrate when connected via ST-Link, ignore gUsartCOM
    USART2->BRR = 0x016D;		        //115200Baud: 22.79 at42 MHz APB1 Clock

    USART2->CR2 = 0;			                //default:1 Stopbit
    USART2->CR3 = 0;			                //default:no handshake
    USART2->CR1 = USART_CR1_UE | USART_CR1_RXNEIE |USART_CR1_TE | USART_CR1_RE;
                                                //8Bit, no Parity, RX Int
    NVIC_EnableIRQ(USART2_IRQn);
}

void sendB(uint8_t data) // *************************************************
//send 1 raw byte on active COM channels
// **************************************************************************
{
	while(!(USART2->SR & USART_SR_TXE)) ;
	USART2->DR = data;
  #ifndef MINHW
    if (guConfigured && (guDeviceClass == CDC_CLASS)) sendUsbByte(data);
  #endif
}

void sendM(uint8_t data) // *************************************************
//send 1 raw byte on active MIDI channel
// **************************************************************************
{
	while(!(USART2->SR & USART_SR_TXE)) ;		//MIDI OUT is COM
	USART2->DR = data;
  #ifndef MINHW
    if (guConfigured)  {
		if (guDeviceClass == CDC_CLASS) sendUsbByte(data);//esp. for sound dump
		else enCodeMidi(data);
    }
  #endif
}

#endif



void sendX8(uint8_t data) // ************************************************
// **************************************************************************
{
uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
    sendB(0x20);
}


void sendNibble(uint8_t data) // ********************************************
// **************************************************************************

{
uint8_t temp;

    temp = data+0x30;
    if (temp>0x39) temp+= 7;
    sendB(temp);
}


void sendX16(uint16_t data) // **********************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>12) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendNibble(nibble);
    sendB(0x20);
}


void sendX32(uint32_t data) // **********************************************
// **************************************************************************
{
uint8_t nibble;
    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sendNibble(nibble);
    sendB(0x20);

}


void sendL1000(uint32_t data) // **********************************************
//send unsigned decimal number(<1000) string. Leading zeroes are inserted
// **************************************************************************
{
    if (data >= 1000)  {
        sendB('X');
        sendB('X');
        sendB('X');
        sendB(0x20);
        return;
    }
    if (data < 100) sendB(0x20);
    if (data < 10) sendB(0x20);
    sendS32(data);
}


void sendS32(uint32_t data) // **********************************************
//send signed decimal number string. Leading zeroes are suppressed
//trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
    sendNS32(data);
    sendB(0x20);
}


 void sendLS32(uint32_t data) // ********************************************
//send signed decimal number string. Leading zeroes are suppressed
//leading SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
    sendB(0x20);
    sendNS32(data);
}


void sendNS32(uint32_t data) // **********************************************
//send signed decimal number string. Leading zeroes are suppressed
//no leading and trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
uint32_t quotient;

    if (data >= 0x80000000) {
        sendB('-');
        data = ~data + 1 ;
    }
    if (data < 10) goto ONES;
    if (data < 100) goto TENS;
    if (data < 1000) goto HUNDREDS;
    if (data < 10000) goto THOUSANDS;
    if (data < 100000) goto TH10;
    if (data < 1000000) goto TH100;
    if (data < 10000000) goto MILLIONS;
    if (data < 100000000) goto MIL10;
    if (data < 1000000000) goto MIL100;
    quotient = data/1000000000;
    data = data - quotient*1000000000;
    sendB((uint8_t)quotient+ 0x30);
MIL100:
    quotient = data/100000000;
    data = data - quotient*100000000;
    sendB((uint8_t)quotient + 0x30);
MIL10:
    quotient = data/10000000;
    data = data - quotient*10000000;
    sendB((uint8_t)quotient + 0x30);
MILLIONS:
    quotient = data/1000000;
    data = data - quotient*1000000;
    sendB((uint8_t)quotient + 0x30);
TH100:
    quotient = data/100000;
    data = data - quotient*100000;
    sendB((uint8_t)quotient+ 0x30);
TH10:
    quotient = data/10000;
    data = data - quotient*10000;
    sendB((uint8_t)quotient + 0x30);
THOUSANDS:
    quotient = data/1000;
    data = data - quotient*1000;
    sendB((uint8_t)quotient + 0x30);
HUNDREDS:
    quotient = data/100;
    data = data - quotient*100;
    sendB((uint8_t)quotient+ 0x30);
TENS:
    quotient = data/10;
    data = data - quotient*10;
    sendB((uint8_t)quotient + 0x30);
ONES:
    sendB((uint8_t)data + 0x30) ;
}


void sendString(const uint8_t * str) // ********************************************
// ***************************************************************************
{
 uint8_t byte;

    while  ((byte = *str++) != 0)  sendB(byte);
}

#ifdef CUT_OFF

uint32_t pullByte(void)  // *************************************************
// **************************************************************************
{
uint32_t recvword;

//	recvword = 0;
    do {
        if ((guConfigured)&&(guDeviceClass == CDC_CLASS)) { //ignore USB MIDI IN
            recvword = pullUsbRxBuf();
            if (recvword) return recvword;      		//priority over USART
        }
        if (gUsartCOM)  recvword = pullUsart1RxBuf();	//ignore MIDI IN
    }  while (recvword == 0);
    return recvword;
}

#else	//unmodified Nucleo: MINHW or ORIG

uint32_t pullUsart2RxBuf(void) // ********************************************
//is linked to USB_RXBUF in this case
// **************************************************************************
{
uint32_t recv;

    servSystem();       //serve watchdog, LEDs, SDIO page buffer
                        //this is the main trigger of lower main level routines
// check first if something received.
// If, then set bit 15. If not, return 0
    if (gUSART2_RXTAIL!=gUSART2_RXHEAD)  {
        recv = (uint32_t)((*gUSART2_RXTAIL++ & 0x000000FF) | 0x00008000);
        if (gUSART2_RXTAIL == gUSART2_RXBUFTOP) gUSART2_RXTAIL = gUSART2_RXBUF;
  #ifdef MINHW
        GPIOA->BSRR = GPIO_BSRR_BS_5;       //set bit PA5, green User LED ON
        gLEDCOUNT = 10 ;              		//LED ON time =200ms
  #else
		if (gpConfigPanel & 0x80) GPIOC->BSRR=GPIO_BSRR_BR_2; //red LED OFF
		else  GPIOC->BSRR=GPIO_BSRR_BS_2;					  //red LED ON
		gLEDCOUNT = 10 ;						//LED OFF time =*20ms =200ms
  #endif
        return recv;
    }
    else return 0;
}


uint32_t pullByte(void)  // *************************************************
// **************************************************************************
{
uint32_t recvword;

  #ifdef MINHW
    do {
        recvword = pullUsart2RxBuf();				//COM is always USART2
    }  while (recvword == 0);
    return recvword;
  #else		//ORIG
    do {
        if ((guConfigured)&&(guDeviceClass == CDC_CLASS)) { //ignore USB MIDI IN
            recvword = pullUsbRxBuf();
            if (recvword) return recvword;			//priority over USART
        }
        recvword = pullUsart2RxBuf();				//COM is always USART2
    }  while (recvword == 0);
    return recvword;
  #endif
}

#endif //CUT_OFF

uint32_t pullUsart1RxBuf(void) // ********************************************
// **************************************************************************
{
uint32_t recv;

    servSystem();       //serves watchdog, ADSRs, LEDs, LCD, encoder, buttons
                        //this is the main trigger of lower main level routines
// check first if something received.
// If, then set bit 15. If not, return 0
    if (gUSART1_RXTAIL != gUSART1_RXHEAD)  {
        recv= (uint32_t)*gUSART1_RXTAIL;
        gUSART1_RXTAIL++;
        if (gUSART1_RXTAIL==gUSART1_RXBUFTOP)  gUSART1_RXTAIL = gUSART1_RXBUF;
#ifdef MINHW
		GPIOA->BSRR = GPIO_BSRR_BS_5;		//set bit PA5,green User LED ON
		gLEDCOUNT = 10;						//LED ON time =200ms
#else
		if (gpConfigPanel & 0x80) GPIOC->BSRR=GPIO_BSRR_BR_2; //red LED OFF
		else  GPIOC->BSRR=GPIO_BSRR_BS_2;					  //red LED ON
		gLEDCOUNT = 10 ;						//LED OFF time =*20ms =200ms
#endif
		return (recv | 0x00008000);
    }
    else return 0;
}


uint32_t pullTimedByte(void)  // ********************************************
// **************************************************************************
{
uint32_t recvword;

#ifdef MINHW
    gTIMEOUT = SYSFREQ * 2;			//2sec,see SysTick interrupt handler
    do {
        if (gTIMEOUT == 0) return 0xFFFFFFFF;
        recvword = pullUsart1RxBuf();		//MIDI bytes first
        if (recvword)  return recvword;		//USART1 has priority over USART2
        recvword = pullUsart2RxBuf();
    }  while (recvword == 0);
    return recvword;
#else
    gTIMEOUT = SYSFREQ * 2;				//2sec,see SysTick interrupt handler
    do {
        if (gTIMEOUT == 0)  return 0xFFFFFFFF;
        if (guConfigured)  {
            recvword = pullUsbRxBuf();
            if (recvword)  return recvword;	//has priority over USART
        }
        recvword = pullUsart1RxBuf(); //combined COM and legacy MIDI IN(CUT_OFF)
									  //MIDI IN (ORIG)
  #ifndef CUT_OFF
		if (recvword)  return recvword;	//ORIG: USART1 has priority over USART2
        recvword = pullUsart2RxBuf();
  #endif
    }  while (recvword == 0);
    return recvword;
#endif
}


int32_t getDeciNum(uint32_t limit)   // **********************************
//though declared as signed, return always unsigned, 0 ...max 99000
// **************************************************************************
{
uint32_t decinum;
uint16_t recvword;
uint8_t digit;

    if (limit >= 99000) return 0xFFFFFFFF;
    decinum = 0;
    while (decinum <= limit)  {
        recvword = pullByte();
        digit = (uint8_t)(recvword & 0x00FF);
        if (digit ==  0x1B) return 0xFFFFFFFE; //ESC key=cancel intentinonally)
        if ((digit == 0x0A) || (digit == CRR) || (digit == '.')) {
            return decinum;
        }
        if ((digit < 0x30) || (digit > 0x39 )) {
            sendB('?');
            return 0xFFFFFFFF;
        }
        sendB(digit);
        digit -= 0x30;     //ASCII>num
        decinum  = (decinum * 10) + digit;
        if (decinum > limit) {
            sendB('?');
            return 0xFFFFFFFF;
        }
    }
    return decinum;
}


float getFloat(void) // *****************************************************
//error: return 9.999999E9
// **************************************************************************
{
float input;
uint32_t fixpoint;
uint16_t recvword;
uint8_t byte, sign, exponent, status, places;
    input = 0.0;
    sign = 0;
    exponent = 0;
    fixpoint = 1;
    status = 0;     //1=behind decimal point, 2 = get exponent
    places = 0;
    while(1)  {

        recvword = pullByte();
        byte = (uint8_t)(recvword&0x00FF);
        if (byte == 0x20)  continue;  //ignore SPACEs
        if (byte > 0x20) sendB(byte);

        if ((byte == CRR) || (byte == 0x0A))  break;  //termination
        if (byte == 0x1B )  return 9.999999E9;  //ESC key =cancel intentinonally
        if (byte < 0x20) {  //hack to implement a simple kind of Backspace
            sendB('X');
            sendB('X');
            sendB('X');
            sendB(0x20);
            return 1.E-8 ;
        }
        if (byte == '.') {
            if (status != 0)  return 9.999999E9;
            status = 1;
            continue;
        }
        if ((byte == 'e') || (byte == 'E')) {
        if (status == 2)  return 9.999999E9;
            status = 2;
            continue;
        }
        if (byte == '-') {
            if (status == 0)  sign = 1;
            if ((status == 1) || (status == 3))   return 9.999999E9;
            if (status == 2)  status = 3;
            continue;
        }

        if ((byte < 0x30) || (byte > 0x39)) return 9.999999E9;
        if (status < 2 ) {
            input = input*10  + (float)(byte-0x30) ;
            places++;
            if (places > 7)   return 9.999999E9;
            if (status==1) fixpoint *= 10;
        }
        else exponent = exponent*10 + (byte-0x30);
    }
    if (input == 0.0)  return 0.0;

    if (fixpoint != 1)  input = input/fixpoint;
    if ((status >= 2) && (exponent != 0)) {
        fixpoint = 1;
        while (exponent)  {
            fixpoint = fixpoint * 10;
            exponent--;
        }
        if (status == 2)  input = input * fixpoint;
        else  input = input/fixpoint;
    }
    if (sign)  input = - input;
    return input;
}

//##########################################################################

// Interrupt Handlers
//*******************

void SysTick_Handler(void)      // ******************************************
// called every 0.625ms (1600Hz)
// **************************************************************************
{
    gSYSCOUNT++;
    if (gDELAYCOUNT) gDELAYCOUNT--;     //--> msDelay()
    if (gTIMEOUT) gTIMEOUT--;           //general use
}

#ifdef CUT_OFF
void USART1_IRQHandler(void)    // *****************************************
// *************************************************************************
{
uint32_t recv;

    if (USART1->SR & USART_SR_RXNE)  {
		recv = USART1->DR & 0x00FF;
        if ((gUsartCOM) || (recv <= 0xF0 ) || (recv == 0xF7)) {
								//ignore MIDI Timecode etc. -- but forward EOX
			*gUSART1_RXHEAD++ = (uint8_t)recv;
			if (gUSART1_RXHEAD==gUSART1_RXBUFTOP) gUSART1_RXHEAD=gUSART1_RXBUF;
        }
    }
}

#else //unmodified Nucleo: MINHW or ORIG

void USART2_IRQHandler(void)    // *****************************************
//PA2,3 are connected with ST-LINK USART, not with Nucleo Morpho connector
//COM_RX/TX is always associated with USART2
// *************************************************************************
{
    if (USART2->SR & USART_SR_RXNE)  {
        *gUSART2_RXHEAD++ = (uint8_t)(USART2->DR & 0x00FF);
        if (gUSART2_RXHEAD==gUSART2_RXBUFTOP)  gUSART2_RXHEAD=gUSART2_RXBUF;
    }
}


void USART1_IRQHandler(void)    // *****************************************
//USART1 always MIDI IN, no gUsartCOM option
// *************************************************************************
{
uint32_t recv;

    if (USART1->SR & USART_SR_RXNE)  {
		recv = USART1->DR & 0x00FF;
        if ((recv <= 0xF0 ) || (recv == 0xF7)) {
								//ignore MIDI Timecode etc. -- but forward EOX
			*gUSART1_RXHEAD++ = (uint8_t)recv;
			if (gUSART1_RXHEAD==gUSART1_RXBUFTOP) gUSART1_RXHEAD=gUSART1_RXBUF;
        }
    }
}
#endif


