// Command interface specific routines of "NucleoSynth-v3.9" ******************
// digital Audio Synthesizer based on STM32 Nucleo F446
// Author and (c)2019-22 Wolfgang Schemmert

// The copyright claim and GPL-3 License is restricted to this file and the
// depending files
// "main.c", "main.h, "panel.c", "filter.c", "dacfreq.c", "synthi.c","drums.c",
// "usb-otg-4611.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org

// Contact: www.midi-and-more.de
// A detailed hardware diy construction manual and a user operating manual
// is provided at this website

// for the IIR filter algorithms used here, see detailled description:
// www.IowaHills.com/A4IIRBilinearTransform.html

// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f4xx_modified.h"
#include "globaldefines.h"

// ************************************
extern const uint8_t gBootStr[];
extern const uint8_t gErrorStr[];
extern uint32_t gTIMEOUT;

extern uint32_t gVid, gPid, gChannel, gDrumsChannel;
extern uint32_t gChannel, gDrumsChannel;
extern uint32_t gSoundSet;
extern uint32_t  gStatus8, gStatus9, gStatusB, gStatusC; //save runtime overhead


extern uint32_t gDelayTime, gDelayTimePreset[N_SOUNDS];         //100us units
extern uint32_t gDelayOffset;     //DelayTime scaled Timer6 frequency (*=3.3071)
extern int32_t gDelayLevelFactor, gDelayLevelPreset[N_SOUNDS];
//special conversion between gDelayLevelFactor and gDelayLevelPreset
extern int32_t gFeedbackFactor, gFeedbackPreset[N_SOUNDS];
//special conversion between gFeedback and gFeedbackPreset

extern uint32_t gChorusWidth, gChorusLfo3, gChorusPreset[N_SOUNDS];
//gChorusLfo3 is a short phrase for gChorusPreset,bit7,see DELAYFLAGS,bit3
extern uint32_t gVolume, gVolumePreset[N_SOUNDS];
extern int32_t gVolumeFactor, gVibrato;
extern uint32_t gVibratoLfo1, gVibratoPreset[N_SOUNDS], gNTrigger[N_SOUNDS];
//gVibratoLfo1is a short phrase for gVibratoPreset,bit7,see DELAYFLAGS,bit0 too

extern uint32_t gLfoNum;
extern uint32_t gLfo1Freq, gLfo1FreqPreset[N_SOUNDS];
extern uint32_t gLfo1Type, gLfo1Wave, gLfo1Slope, gLfo1WavePreset[N_SOUNDS];
extern uint32_t gLfo2Freq, gLfo2FreqPreset[N_SOUNDS];
extern uint32_t gLfo2Type, gLfo2Wave, gLfo2Slope, gLfo2WavePreset[N_SOUNDS];
extern uint32_t gLfo3Freq, gLfo3FreqPreset[N_SOUNDS];
extern uint32_t gLfo3Type, gLfo3Wave, gLfo3Slope, gLfo3WavePreset[N_SOUNDS];

extern const int32_t gDbTable[53];
extern void popFlashSounds(void);
extern void pushFlashSounds(void);
extern void eraseFlashSounds(void);
extern void servSystem(void);
extern void msDelay(uint32_t);

//USART
extern void sendB(uint8_t);
extern void sendM(uint8_t);
extern void sendX8(uint8_t);
extern void sendX16(uint16_t);
extern void sendX32(uint32_t);
extern void sendLS32(uint32_t);
extern void sendS32(uint32_t);
extern void sendNS32(uint32_t);
extern void sendString(const uint8_t *);
extern uint32_t pullUsart1RxBuf(void);
extern uint32_t pullByte(void);
extern uint32_t pullTimedByte(void);
extern int32_t getDeciNum(uint32_t);
extern float getFloat(void);

//from usb-otg-4611.c
extern uint32_t guConfigured, guDeviceClass;
extern void stopUSB(void);
extern void initUSB(void);
extern uint32_t pullUsbRxBuf(void);
extern void sendMidiMsg(uint8_t, uint8_t, uint8_t, uint8_t);
extern void sendUsbByte(uint8_t);

//from filter.c
extern const float gfMidiFreqTable[128];
extern struct IIRPack {
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
uint32_t QFACTOR;           //bandwidth or 100*zeta(1-100)
uint32_t PEAK;
} gfIIRPack[N_SOUNDS*N_FILTER];
//bits 24-31 contain Flex Default, bits 16-23 contain Easy Default,
//bits 8-15 contain Flex Modify, bits 0-7 contain EasyModify filter parameters

extern struct IIR {  //each 9words
float A1;
float A2;
float B0;
float B1;
float B2;
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
uint32_t QFACTOR;           //bandwidth or 100*zeta(1-100)
uint32_t PEAK;
} gfACTIV_IIR[N_FILTER], gfTempIIR;
//are recalculated from gfIIRPack[] when sound or routing mode changes

extern float gfState1[N_RUN_FILT], gfState2[N_RUN_FILT];
extern uint32_t gfFilterNum, gfFilterType, gfFlexMode;
extern uint32_t gfRuntimeMask, gfStageMask[N_SOUNDS];
//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bit 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)

//temporary "Memo" Filter set
extern uint8_t gfEasyMemoType[N_FILTER];
extern uint8_t gfEasyMemoFreq[N_FILTER];
extern uint8_t gfEasyMemoQfactor[N_FILTER];
extern uint8_t gfEasyMemoPeak[N_FILTER];
extern uint8_t gfFlexMemoType[N_FILTER];
extern uint8_t gfFlexMemoFreq[N_FILTER];
extern uint8_t gfFlexMemoQfactor[N_FILTER];
extern uint8_t gfFlexMemoPeak[N_FILTER];
//temporary "Undo" Filter set
extern uint8_t gfEasyUndoType[N_FILTER];
extern uint8_t gfEasyUndoFreq[N_FILTER];
extern uint8_t gfEasyUndoQfactor[N_FILTER];
extern uint8_t gfEasyUndoPeak[N_FILTER];
extern uint8_t gfFlexUndoType[N_FILTER];
extern uint8_t gfFlexUndoFreq[N_FILTER];
extern uint8_t gfFlexUndoQfactor[N_FILTER];
extern uint8_t gfFlexUndoPeak[N_FILTER];

extern uint32_t gfGain, gfGainPreset[N_SOUNDS];
extern uint32_t gfF0mod, gfF0modLfo2, gfF0modPreset[N_SOUNDS];
extern float gfF0ModBase;	//= 1. + (F0 freq deviation by Lfo modulation)
uint32_t gfF2mod, gfF2modLfo, gfF2modPreset[N_SOUNDS];
extern uint32_t gfFilterOffset;     //start offset inside gfIIR
extern uint32_t gfBass, gfTreble, gfBasTrebPreset[N_SOUNDS];
extern void enterFilterParams(void);
extern void initFilterParams(uint32_t, uint32_t, uint32_t, uint32_t, \
															uint32_t, uint32_t);
extern void unpackFilter(uint32_t, uint32_t);
extern void packBasTreb(void);
extern void unpackBasTreb(void);

//from panel.c
extern uint32_t gpConfigPanel, gpPlusState, gpModPlay, gpModGeneral;
extern const uint8_t gpEmptyStr[];
extern const uint8_t gpOKStr[];
extern const uint8_t gpRouteStr[] [4];
extern const uint8_t gpWaveStr[] [6];
extern const uint8_t gpLfoTypeStr[] [4];
extern const uint8_t gpAdsrTypeStr[] [6];
extern const uint8_t gpDrumTypeStr[] [5];
extern uint32_t gpAdsrIndex, gpTempAdsrIndex;
extern void initPlay(void);
extern void redrawPlayLine2(void);
extern void initGeneral(void);
extern void redrawGeneral(void);
extern void initOsc(void);
extern void redrawOscDisplay(void);
extern void initAdsr(void);
extern void initDelay(void);
extern void redrawDelayDisplay(void);
extern void initFilter(void);
extern void redrawFilterDisplay(void);
extern void reInitFilter(uint32_t);
extern void initRoute(void);
extern void saveUndo(void);
extern void saveMemo(void);
extern void updateRoute(void);
extern void redrawRouteDisplay(void);
extern void initSystem(void);
extern void adjPeak3(uint32_t);
extern void adjPeak4(uint32_t);
extern void displayString(uint32_t, const uint8_t *);
extern void displayNL99(uint32_t, uint32_t);
extern void placeCursor(uint32_t);
extern void writeLcdData(uint32_t);

//from dacfreq.c
extern uint32_t gaFTestFactor;
extern int32_t gaFilterTest;
extern uint32_t gaZero1, gaZero2;//=1:more harmonics, =0:more underground noise
extern uint32_t gaRoute0N, gaRoutePreset[N_SOUNDS];		//bits23..16=FLEX F0N
														//bits 7..0= EASY F0N
extern uint32_t gaRouteOsc1, gaRouteOsc2, gaRouteNoise;	//for runtime speedup

//from synthi.c
extern uint32_t gsMonoPoly;  //default=0 =polyphonic, 1=monophonic
extern uint32_t gsO2Sync;//(if set,Osc2 wave start is sync.with Osc1 wave start)
extern int16_t gsTrapez[1024], gsTriangle[1024];
extern int16_t gsSinus[1024], gsAsymTri[1024], gsSawtooth[1024];
extern int16_t gsWaveLfo1[4000], gsWaveLfo2[4000], gsWaveLfo3[4000];
extern int16_t gsUserWave0[1024], gsUserWave1[1024];
extern int16_t gsUserWave2[1024], gsUserWave3[1024];

extern uint32_t gsWAVEFORM_1, gsOsc1ModLfo2, gsWAVEFORM_1Preset[N_SOUNDS];
extern uint32_t gsWAVEFORM_2, gsOsc2ModLfo2, gsWAVEFORM_2Preset[N_SOUNDS];
extern int32_t gsTRANS, gsTRANSPreset[N_SOUNDS];
extern int32_t gsTRANS_2, gsTRANS_2Preset[N_SOUNDS];
extern int32_t gsDETUNE, gsDETUNEPreset[N_SOUNDS];
extern int32_t gsDETUNE_2, gsDETUNE_2Preset[N_SOUNDS];
extern uint32_t gsGLIDE, gsGLIDEPreset[N_SOUNDS];
extern uint32_t gsMOD_1, gsOsc1PWM, gsMOD_1Preset[N_SOUNDS];
extern uint32_t gsMOD_2, gsOsc2PWM, gsMOD_2Preset[N_SOUNDS];
extern uint32_t gsRangeFS1, gsSlopeFS1, gsRangeFS2, gsSlopeFS2;
extern uint32_t gsFreeSquare1[N_SOUNDS], gsFreeSquare2[N_SOUNDS];
									//bits31-16= gsRangeFS,bits15-0=gsSlopeFS

struct ADSR {			//stored in Flash preset
uint32_t ATDECSUS;		//bits31..24:Attack.bits23..16:Decay,bits15..0:Sustain
uint32_t REFIXVEL;		//bits23..16:Release,bits15..8:Fixvol,bits7..0:Velocity
};
struct ADSR gsADSR[N_SOUNDS*N_ASTRUCT];
//sets of compacted each 7 ADSR structures
//Attack, Decay and Relase times up to 1sec in 10 ms scale,big values compressed
//Sustain bit7 set=hold, else 1x mode

extern uint32_t gsAttack[N_ASTRUCT];  //Attack time in 0.625ms units(1600Hz)
extern uint32_t gsDecay[N_ASTRUCT];   //Decay time in 0.625ms units(1600Hz)
extern uint32_t gsSustain[N_ASTRUCT];
extern uint32_t gsHoldSust[N_ASTRUCT]; //=1 hold Sustain,=0 one Shoot Sustain
extern uint32_t gsRelease[N_ASTRUCT]; //Release time in 0.625ms units(1600Hz)
extern uint32_t gsFixVol[N_ASTRUCT];
extern uint32_t gsVelo[N_ASTRUCT];

extern uint32_t gsFreeAdsr, gsFreeIdx, gsFreeAdsrPreset[N_SOUNDS];
extern uint32_t gsAdsrOffset, gsNoteFlags, gsNoTriggerF0;

extern uint32_t gsKeytrack0Preset[N_SOUNDS];
extern float gsKeytrack0;
extern uint32_t gsPWM1, gsPWM1Preset[N_SOUNDS];
extern uint32_t gsPhase0, gsPhase1, gsPhase2, gsPhase3, gsPhase4, gsPhase5;
extern uint32_t gsPhase6, gsPhase7, gsPhase8, gsPhase9, gsPhase10, gsPhase11;
extern uint32_t gsPhase12, gsPhase13, gsPhase14, gsPhase15, gsPhase16,gsPhase17;
extern uint32_t gsPhase18, gsPhase19;

extern void initWaveOsc1(uint32_t);
extern void initWaveOsc2(uint32_t);
extern void distributeNoteOn(uint32_t, uint32_t);
extern void handleNoteOff(uint32_t);
extern void panic_Off(uint32_t);
extern void releaseN(void);
extern void releaseG(void);
extern void releaseFM1(void);
extern void releaseFM2(void);
extern void releaseVoice0(void);
extern void releaseVoice0X(void);
extern void releaseVoice1(void);
extern void releaseVoice1X(void);
extern void releaseVoice2(void);
extern void releaseVoice2X(void);
extern void releaseVoice3(void);
extern void releaseVoice3X(void);
extern void releaseVoice4(void);
extern void releaseVoice4X(void);
extern void releaseVoice5(void);
extern void releaseVoice5X(void);
extern void releaseVoice6(void);
extern void releaseVoice6X(void);
extern void releaseVoice7(void);
extern void releaseVoice7X(void);
extern void releaseVoice8(void);
extern void releaseVoice8X(void);
extern void releaseVoice9(void);
extern void releaseVoice9X(void);
extern void releaseFilter0(void);
extern void releaseFilter1(void);
extern void releaseFilter2(void);

// from drums.c
extern uint32_t gdDrumsVolIndex;
            //=normalized sum of (gdDrumsVol + gOutVol + gd(drum type)Vol)
extern uint32_t gdDrumsVol, gdDrumsVolPreset[N_SOUNDS];
extern uint32_t gdKickVol, gdKickVolPreset[N_SOUNDS];
extern int32_t gdKickVolFactor;
extern uint32_t gdHiTomVol, gdHiTomVolPreset[N_SOUNDS];
extern int32_t gdHiTomVolFactor;
extern uint32_t gdSnareVol, gdSnareVolPreset[N_SOUNDS];
extern int32_t gdSnareVolFactor;
extern uint32_t gdRimVol, gdRimVolPreset[N_SOUNDS];
extern int32_t gdRimVolFactor;
extern uint32_t gdClapVol, gdClapVolPreset[N_SOUNDS];
extern int32_t gdClapVolFactor;
extern uint32_t gdHihatVol, gdHihatVolPreset[N_SOUNDS];
extern int32_t gdHihatVolFactor;
extern uint32_t gdCymbalVol, gdCymbalVolPreset[N_SOUNDS];
extern int32_t gdCymbalVolFactor;
extern uint32_t gdBellVol, gdBellVolPreset[N_SOUNDS];
extern int32_t gdBellVolFactor;

extern void triggerKick(int32_t);
extern void triggerHiTom(int32_t);
extern void triggerSnare(int32_t);
extern void triggerRim(int32_t);
extern void triggerHihat(int32_t);
extern void triggerClap(int32_t);
extern void triggerCymbal(int32_t);
extern void triggerBell(int32_t);
extern int32_t getVolFactor(int32_t);
extern void getAllVolFactors(void);

// ************************************

uint32_t gcAdsrSel;

const uint8_t gcSoundSetStr[] = {"active Sound:"};
const uint8_t gcWaveformStr[] = {" Wave"};
const uint8_t gcModLfoStr[] = {" modulated by Lfo (1 or 2)?"};
const uint8_t gcRoute0NStr[] = {"Filter0N"};
const uint8_t gcRoute12Str[] = {"Filter12"};
const uint8_t gcRoutingStr[] = {"Routing"};
const uint8_t gcFilterStr[] = {"Filter#"};
const uint8_t gcBassStr[] = {"Bass"};
const uint8_t gcTrebleStr[] = {"Treble"};
const uint8_t gcGainStr[] = {"Gain "};
const uint8_t gcDelayStr[] = {"Delay"};
const uint8_t gcDryStr[] = {"Undelayed"};
const uint8_t gcFeedbackStr[] = {"Feedback"};
const uint8_t gcLevelStr[] = {" Level"};
const uint8_t gcChorusWidthStr[] = {"Chorus Width"};
const uint8_t gcVibratoStr[] = {"Vibrato Depth"};
const uint8_t gcOutputStr[] = {"Global Output "};
const uint8_t gcVolumeStr[] = {"Volume"};
const uint8_t gcDBStr[] = {" (+/-dB)? "};
const uint8_t gcVeloStr[] = {" +Velo"};
const uint8_t gcEnvelopeStr[] = {"Envelope"};
const uint8_t gcLevelShiftStr[] = {"LevelShift"};
const uint8_t gcFreqShiftStr[] = {"Freq.Shift"};
const uint8_t gcKeytrackStr[] = {"Keytrack Filter#0 "};
const uint8_t gcPriorityStr[] = {" Priority"};
const uint8_t gcTransposeStr[] = {"Osc1&2_Transpose"};
const uint8_t gcTrans2Str[] = {"Osc2+_Transpose"};
const uint8_t gcDetuneStr[] = {"Osc1&2_Detune"};
const uint8_t gcDetune2Str[] = {"Osc2+_Detune"};
const uint8_t gcFreqModStr[] = {" Freq.Mod"};
const uint8_t gcFMRangeStr[] = {"(0-99):"};
const uint8_t gcPwmStr[] = {" PWM:"};
const uint8_t gcFreqStr[] = {"Freq."};
const uint8_t gcGlideStr[] = {" Glide"};
const uint8_t gcOsc1Str[] = {"Osc1"};
const uint8_t gcOsc2Str[] = {"Osc2"};
const uint8_t gcControlLfoStr[] = {" controlled by Lfo"};
const uint8_t gcDrumsVolStr[] = {"Drums Vol(relative dB)"};
const uint8_t gcAssignStr[] = {"assign Free ADSR: 0=Lfo2FM,1=Osc2FM+,2=F0BwRes"};
const uint8_t gcFreeAdsrStr[] = {"3=NoiseFMid,4=Chorus,5=Dry,6=Delay,7=Feedback? "};

uint32_t CMD_Interpreter(uint32_t);
void uploadAllSounds(void);
uint32_t setVidPid(void);
uint32_t setMidiChannel(uint32_t);
void setBaudrate(uint32_t);
uint32_t setRouting(void);
uint32_t setADSR(void);
void packADSR(uint32_t, uint32_t, uint32_t);
void unpackADSR(uint32_t);
uint32_t initLfo(void);
void calculateLfoWave(uint32_t, uint32_t);
uint32_t setupFreeSquare(void);
uint32_t setDrumVolume(void);
uint32_t parseHexNibble(void);
uint32_t parseHexNum(uint32_t);
void activateSoundSet(void);
void activateFilter(uint32_t, uint32_t);
void createShortFlags(void);
void copyPreset(uint32_t);
void listSetup(void);
void sendRouteConfig(void);
void sendAdsrParam(uint32_t);
void sendAdsrTime(uint32_t);
void sendLfoWave(uint32_t);
uint32_t scanSysEx(uint32_t);
void USB_Midi_Scanner(uint32_t);
void USART_Midi_Scanner(uint32_t);
void Usart_Midiscanner(uint32_t);
void Usb_Midiscanner(uint32_t);
void MIDI_Executor(uint32_t, uint32_t, uint32_t);
void updateDisplay(void);
void sendThisFilter(void);
void dumpSoundSet(uint32_t);
void dumpAllSoundSets(void);
void split16(uint32_t);
uint32_t uploadSoundSet(uint32_t);
uint32_t  uploadUserWave(uint32_t);
uint32_t pack16(void);


uint32_t  CMD_Interpreter(uint32_t opcode)   // ******************************
// ***************************************************************************
{
const uint8_t PanicStr[] = {"Panic OFF !!"};
const uint8_t FmPwmStr[] = {" Freq.Mod(=0) or PWM(=1) ?"};
const uint8_t MonoPolyStr[] = {"activate Polyphonic(=0) or Monophonic(=1) mode? "};
const uint8_t O2SyncStr[] = {"Synchronize Osc2 phase with Osc1 (=1) else =0? "};
const uint8_t MidiMsgsStr[] = {"activate MIDI Messages(=1) or deactivate(=0)? "};
const uint8_t DelayTimeStr[] = {"Delay Time(xxxx.x ms)? "};
const uint8_t MidiChStr[] = {"MIDI Channel? "};
const uint8_t DrumsChStr[] = {"MIDI Channel for Drums? "};
const uint8_t ProzentStr[] = {"(0...100%):"};
const uint8_t FilterTestStr[] = {"Filter Test: 0=Off, 1..9:Filter#0, -1..-9:Filter#4, +'Enter' ? "};
const uint8_t LoadAllStr[] = {"Reload complete Setup from Flash?"};
const uint8_t CopyPresetStr[] = {"Copy this Soundset to Soundset no.?"};
const uint8_t FactoryStr[] = {"Load Factory Preset of Sound-Set no."};
const uint8_t MaxFactoryStr[] = {"Factory Preset only SoundSet 0..19"};
const uint8_t SaveAllStr[] = {"Save complete Setup to Flash?"};
const uint8_t ClearAllStr[] = {"Erase Flash Preset completely?"};
const uint8_t ConfirmStr[] = {"->confirm with 'Y'"};
const uint8_t MinusStr[] = {"Undo impossible !! confirm with '-'"};
const uint8_t DumpStr[] = {"Dump SoundSet 0...59 (A = all) ?"};
const uint8_t UploadSoundStr[] = {"Upload SoundSet (number coded in file) ?"};
const uint8_t UploadWaveStr[] = {"Upload UserWave (number coded in file) ?"};
const uint8_t UploadedStr[] = {"Uploaded: #"};
const uint8_t RecordStr[] = {" start Record within 4s "};

uint32_t  parameter, n;
int32_t intParameter;
float floatParameter;
uint32_t recvword;

    if ((opcode > 0x60) && (opcode <0x7B)) opcode -= 0x20;
    if (opcode == CRR)   {
		sendB(CRR);
        return 1;
	}
    if (opcode == LFF)   {     //ignore
     	return 1;
   	}
    if (opcode == 0x20)   {     //ignore
     	return 1;
   	}

    switch(opcode)  {

        case '!':               //exclamation mark
            panic_Off(1);
			sendString(PanicStr);
            break;

        case 0x5B:                  //'['
            setVidPid();
            break;

       case '#':
            sendString(MidiChStr);
            setMidiChannel(0);
            sendB(CRR);
            sendString(DrumsChStr);
            setMidiChannel(1);
            updateDisplay();
            break;

      case 'V':
            sendString(gcVolumeStr);
            sendString(gcDBStr);
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            if (floatParameter < -46.0) floatParameter = -46.0;
            if (floatParameter > 6.0) floatParameter = 6.0;  //ex12
            gVolume = (uint32_t)(floatParameter + 46.0); //range 0 ..52
            gVolumePreset[gSoundSet] = gVolume;
            gVolumeFactor = gDbTable[gVolume] >> 3;     //max 511 allowed
            updateDisplay();
            break;

         case 'S':
            sendString(gcSoundSetStr);
            sendB('?');
            sendB(0x20);
            parameter = getDeciNum(N_SOUNDS-1);
            if (parameter == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (parameter > (N_SOUNDS -1)) return 0;	//valid input:0,1...
            gSoundSet = parameter;
            activateSoundSet();
            updateDisplay();
            break;

        case 'W':
//Waveform, OSC_1 Setup Dialog
            sendString(gcOsc1Str);
            sendString(gcWaveformStr);
            sendB(':');
            sendB(CRR);
            sendB('(');
            for (n = 0; n <= 42; n++)  {
                sendNS32(n);
                sendB('=');
                sendString(gpWaveStr[n]);
                if ((n & 7) == 7) sendB(CRR);
                else sendB(' ');
            }
            sendB(')');
            sendB('?');
            n = getDeciNum(42);
            if (n == 0xFFFFFFFE)  return 1; 		//user cancelled
            if (n > 42) return 0;
            gsWAVEFORM_1 = n;
            gsWAVEFORM_1Preset[gSoundSet] = n;
			switch (gsWAVEFORM_1)  {
				case 0:
				case 25:
				case 36:
				case 37:
				case 38:
				case 41:
					gaZero1 = 0;
					break;
				default:
					gaZero1 = 1;
			}
            initWaveOsc1(n);
            sendString(gcModLfoStr);
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1; 		//user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if (n == 2)  {
               gsOsc1ModLfo2 = 1;					//Osc1 modulated by Lfo2
               gsWAVEFORM_1Preset[gSoundSet] |= 0x40;
            }
            else if (n == 1)  gsOsc1ModLfo2 = 0;	//Osc1 modulated by Lfo1
            else return 0;
            updateDisplay();
            break;

       case 'X':
//Waveform, OSC_2 Setup Dialog
            sendString(gcOsc2Str);
            sendString(gcWaveformStr);
            sendB(':');
            sendB(CRR);
            sendB('(');
            for (n = 0; n <= 36; n++)  {
                sendNS32(n);
                sendB('=');
                sendString(gpWaveStr[n]);
                if ((n & 7) == 7) sendB(CRR);
                else sendB(' ');
            }
            sendB(')');
            sendB('?');
            n = getDeciNum(36);
            if (n == 0xFFFFFFFE)  return 1; 		//user cancelled
            if (n  > 36) return 0;
            gsWAVEFORM_2 = n;
            gsWAVEFORM_2Preset[gSoundSet] = n;
			switch (gsWAVEFORM_2)  {
				case 0:
				case 25:
				case 36:
					gaZero2 = 0;
					break;
				default:
					gaZero2 = 1;
			}
            initWaveOsc2(n);
            sendString(gcModLfoStr);
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1; 		//user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if (n == 2)  {
               gsOsc2ModLfo2 = 1;					//Osc2 modulated by Lfo2
               gsWAVEFORM_2Preset[gSoundSet] |= 0x40;
            }
            else if (n == 1)  gsOsc2ModLfo2 = 0;	//Osc2 modulated by Lfo1
            else return 0;
            updateDisplay();
            break;

       case '%':
            sendString(O2SyncStr);
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1; //user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if (n > 1) return 0;
            gsO2Sync = n;
            gfStageMask[gSoundSet] &= 0x0000007F;
            gfStageMask[gSoundSet] |= n << 7;	//bit7 stores gsO2Sync
            updateDisplay();
			break;

       case 'T':
//Global Transpose equal for both oscillators
            sendString(gcTransposeStr);
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if (intParameter < -36) intParameter = -36;
            if (intParameter > 36) intParameter = 36;
            gsTRANS = intParameter + 36;
            gsTRANSPreset[gSoundSet] = gsTRANS;
            updateDisplay();
            break;

        case 'U':
//OSC2 extra Transpose
            sendString(gcTrans2Str);
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if (intParameter < -36) intParameter = -36;
            if (intParameter > 36) intParameter = 36;
            gsTRANS_2 = intParameter + 36;
            gsTRANS_2Preset[gSoundSet] = gsTRANS_2;
            updateDisplay();
            break;

        case 'Y':
//OSC1+2 const. Detune
            sendString(gcDetuneStr);
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if (intParameter < -64) intParameter = -64;
            if (intParameter > 63) intParameter = 63;
            gsDETUNEPreset[gSoundSet] = intParameter + 64;
            gsDETUNE = intParameter;
            updateDisplay();
            break;

        case 'Z':
//OSC2 add const. Detune
            sendString(gcDetune2Str);
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if (intParameter < -64) intParameter = -64;
            if (intParameter > 63) intParameter = 63;
            gsDETUNE_2Preset[gSoundSet] = intParameter + 64;
            gsDETUNE_2 = intParameter;
            updateDisplay();
            break;

        case 'P':
//Glide=Portamento
            sendString(gcGlideStr);
            parameter = getDeciNum(127);
            if (parameter == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (parameter > 127)  return 0;
            gsGLIDE = parameter;
            gsGLIDEPreset[gSoundSet] = parameter;
            updateDisplay();
            break;

       case 'L':
//LFO1,2,3 number, Wave, Freq dialog
            return initLfo();

        case 'I':           //upper case 'i'
//Osc1 Modulation by Lfo
            sendString(gcOsc1Str);
            if (gfRuntimeMask & 0x00000200){			//square wave
                sendString(FmPwmStr);					//Freq.Mod(=0) orPWM(=1)
                if((n = parseHexNibble()) > 1)  return 0;
                gsOsc1PWM = n;							//PWM only if squarewave
            }
            else gsOsc1PWM = 0;							//normal freq.mod,no PWM
		    sendB(' ');
            if (gsOsc1PWM)  sendString(gcPwmStr);
            else  sendString(gcFreqModStr);
            sendString(gcFMRangeStr);
            parameter = getDeciNum(99);
            if (parameter == 0xFFFFFFFE)  return 1;		//user cancelled
            if (parameter > 99)  return 0;
            gsMOD_1 = parameter;
            if (gsOsc1PWM == 0) gsMOD_1Preset[gSoundSet] = parameter;	//FM
            else  gsMOD_1Preset[gSoundSet] = parameter | 0x80;			//PWM
            updateDisplay();
            break;

       case 'J':
//Osc2 Modulation by LFO
            sendString(gcOsc2Str);
            if (gfRuntimeMask & 0x00000400){
                sendString(FmPwmStr);					//Freq.Mod(=0) orPWM(=1)
                if((n = parseHexNibble()) > 1)  return 0;
                gsOsc2PWM = n;							//PWM only if squarewave
            }
            else gsOsc2PWM = 0;							//normal freq.mod,no PWM
			sendB(' ');
            if (gsOsc2PWM)  sendString(gcPwmStr);
            else  sendString(gcFreqModStr);
            sendString(gcFMRangeStr);
            parameter = getDeciNum(99);
            if (parameter == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (parameter > 99)  return 0;
            gsMOD_2 = parameter;
            if (gsOsc2PWM == 0) gsMOD_2Preset[gSoundSet] = parameter;	//FM
            else  gsMOD_2Preset[gSoundSet] = parameter | 0x80;			//PWM
            updateDisplay();
            break;

        case 'A':
            return (setADSR());							//ADSR Dialog

        case 'F':
//Filter Dialog
            sendString(gcSoundSetStr);
            sendS32((int32_t)(gSoundSet));
            sendB(0x20);
            enterFilterParams();           //scale frequency input to MIDI notes
            updateDisplay();
            break;

        case 'M':
//Filter0 modulation by LFO
            sendString(gcFilterStr);
			sendB('0');
            sendString(gcFreqModStr);
			sendString(gcFMRangeStr);
            parameter = getDeciNum(99);
            if (parameter == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (parameter > 99)  return 0;
            gfF0mod = parameter;
            sendString(gcControlLfoStr);
            sendB('?');
            sendB(' ');
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1; //user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if ((n == 0) || (n > 2)) return 0;
            n-- ;
			gfF0mod = parameter;
			gfF0modLfo2 = n;         //values 0,1
			if (n== 0)  {           //LFO1 selected
				gfF0modPreset[gSoundSet] = gfF0mod;
			}
			else   {                //LFO2 selected
				gfF0modPreset[gSoundSet] = gfF0mod | 0x80;
			}
            updateDisplay();
            break;

        case 'K':
//Filter0 Keytrack
            sendString(gcKeytrackStr);
            sendString(ProzentStr);
            parameter = getDeciNum(127);
            if (parameter == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (parameter > 127)  return 0;
			gsKeytrack0Preset[gSoundSet] = parameter;
			gsKeytrack0 = (float)gsKeytrack0Preset[gSoundSet] / 100.;
            updateDisplay();
            break;

        case 'R':
            if (setRouting())  {
            updateDisplay();
            break;
            }
            else return 0;

        case 'B':
//Bass
            sendString(gcBassStr);
            sendString(gcDBStr);
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if (intParameter < -64) intParameter = -64;
            if (intParameter > 63) intParameter = 63;
            intParameter += 64;
            gfBass = (uint32_t)intParameter;
            packBasTreb();
			adjPeak3(gfBass);
            updateDisplay();
            break;

        case 'H':
//Tone weighes
            sendString(gcTrebleStr);
            sendString(gcDBStr);
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if (intParameter < -64) intParameter = -64;
            if (intParameter > 63) intParameter = 63;
            intParameter += 64;
            gfTreble = (uint32_t)intParameter;
            packBasTreb();
			adjPeak4(gfTreble);
            updateDisplay();
            break;

        case 'G':
//Gain
            sendString(gcGainStr);
            sendB('?');
            sendB(0x20);
            parameter = getDeciNum(50);
            if (parameter == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (parameter > 50)  return 0;
            if (parameter == 0)  parameter = 1;
            gfGain = parameter;
            gfGainPreset[gSoundSet] = parameter;
            updateDisplay();
            break;

//Delay,Chorus
        case 'D':
            sendString(DelayTimeStr);
            floatParameter = getFloat();                   //range 0 to 300.0
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)(floatParameter * 10); //int.steps of 1/10ms
            if (intParameter > 3000) intParameter = 3000;
            if (intParameter < 0) intParameter = 0;
            gDelayTime = ((intParameter /100) << 8) | (intParameter % 100);
                // packed specially:bits 8-15=hundreds+tens,bits0-7=ones+tenths
            gDelayTimePreset[gSoundSet] = gDelayTime;
            gDelayOffset = (gDelayTime >> 8) * 331 +  \
                                        ((gDelayTime & 0x000000FF) * 331)/100;
//gDelayOffset is delay time expressed in number of cells of delay memory
//i.e. difference between write(head) and read(tail) pointer
            updateDisplay();
            break;

       case 'E':
//Delay level
            sendString(gcDelayStr);
            sendString(gcLevelStr);
            sendB('?');
            sendB(' ');
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if (intParameter > 127) intParameter = 127;
            if (intParameter < -127) intParameter = -127;
            gDelayLevelFactor = intParameter;
            sendB(CRR);
			if (gDelayLevelFactor < 0) {
				gDelayLevelPreset[gSoundSet] = 128 - gDelayLevelFactor;
			}
			else  {
				gDelayLevelPreset[gSoundSet] = gDelayLevelFactor;
			}
            updateDisplay();
            break;

        case 'N':
//Delay Feedback
            sendString(gcFeedbackStr);
            sendString(gcLevelStr);
            sendB('?');
            sendB(' ');
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if (intParameter > 100) intParameter = 100; //more=self-oscillation
            if (intParameter < -100) intParameter = -100;
            gFeedbackFactor = intParameter;
            if (gFeedbackFactor < 0) {
                gFeedbackPreset[gSoundSet] =  128 - gFeedbackFactor;
            }
            else  {
                gFeedbackPreset[gSoundSet] = gFeedbackFactor;
            }
            updateDisplay();
            break;

        case 'C':
            sendString(gcChorusWidthStr);
            sendB('?');
            sendB(' ');
            parameter = (uint32_t)getDeciNum(99);
            if (parameter == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (parameter > 99)  return 0;
            gChorusWidth = parameter;
            sendString(gcControlLfoStr);
            sendB('2');
            sendB(',');
            sendB('3');
            sendB('?');
            sendB(' ');
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1; //user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if ((n < 2) || (n > 3)) return 0;
            n -= 2  ;
            gChorusLfo3 = n;		//values 0,1
            if (n == 0)  {			//LFO2 selected (default)
                gChorusPreset[gSoundSet] = gChorusWidth;
            }
            else   {				//LFO3 selected
                gChorusPreset[gSoundSet] = gChorusWidth | 0x80;
            }
            updateDisplay();
            break;

       case 'Q':
//Vibrato
            sendString(gcVibratoStr);
            sendB('?');
            sendB(' ');
            parameter = getDeciNum(100);
            if (parameter == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (parameter > 100)  return 0;
            gVibrato = parameter; 		//"int" formally for easier handling
            sendString(gcControlLfoStr);
            sendB('1');
            sendB(',');
            sendB('3');
            sendB('?');
            sendB(' ');
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1; 	//user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if (n == 3)  {						//LFO3 selected
				gVibratoLfo1= 0;
                gVibratoPreset[gSoundSet] = gVibrato;
            }
            else if (n == 1)  {					//LFO1 selected
				gVibratoLfo1= 1;
                gVibratoPreset[gSoundSet] = gVibrato | 0x80;
            }
            else return 0;
            updateDisplay();
            break;
//relative drum levels
       case '$':
            setDrumVolume();
            return 1;

        case '0':
            triggerKick(64);
            return 1;

        case '1':
            triggerHiTom(64);
            return 1;

        case '2':
            triggerSnare(64);
            return 1;

        case '3':
            triggerRim(64);
            return 1;

        case '4':
            triggerHihat(64);
            return 1;

        case '5':
            triggerClap(64);
            return 1;

        case '6':
            triggerCymbal(64);
            return 1;

        case '7':
            triggerBell(64);
            return 1;

		case '/':
			if (setupFreeSquare() == 0)  return 0;
			break;

		case ',':						//comma
//token separator for NoteOn message stream
			n = parseHexNum(2);
			if (n == 0xFFFFFFFF) return 0;
			if ((n < 18) || (n > 0x108))  return 0;
			parameter = parseHexNum(2);
			if (parameter == 0xFFFFFFFF) return 0;
			if (parameter > 127) parameter = 127;
			distributeNoteOn(n,parameter);
			break;

        case '-':
        case '<':
//token separator for NoteOff message stream
			n = parseHexNum(2);
			if (n == 0xFFFFFFFF) return 0;
			if ((n < 18) || (n > 108))  return 0;
			handleNoteOff(n);
			break;

        case '&':
            sendString(MonoPolyStr);
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1; //user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if (n > 1) return 0;
            gsMonoPoly = n;
            gfStageMask[gSoundSet] &= 0x000000DF;
            gfStageMask[gSoundSet] |= n << 5;	//bit5 doubles gsMonoPoly
            updateDisplay();
            break;

        case '.':		//dot
        case '?':
            listSetup();
            break;

         case 0x7D:					//'}'
            sendString(DumpStr);
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1;			//user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if (n == 0xA)  {
                sendString(ConfirmStr);
                recvword = pullByte();
                if (recvword != 0x00008059) return 0;//upper case 'Y' MANDATORY!
				sendString(RecordStr);
				msDelay(4000);
                dumpAllSoundSets();
                return 1;
            }
            else {
                if (n > 9) return 0;
                parameter = n;
				n = parseHexNibble();
                if (n == 0xFFFFFFFF)  return 0;
                if (n != 0xFFFFFFFE)   {            //<return> key NOT pressed
                    if (n > 9)  return 0;
                    parameter = (parameter * 10) + n;
                    if (parameter > 59) return 0;
                }
                sendString(ConfirmStr);
                recvword = pullByte();
                if (recvword != 0x00008059) return 0;//upper case 'Y' MANDATORY!
				sendString(RecordStr);
				msDelay(4000);
                dumpSoundSet(parameter);
                return 1;
            }
			break;	//only formally

		case 0x7B:					// '{'
			sendString(UploadSoundStr);
			sendString(ConfirmStr);
			recvword = pullByte();
			if (recvword != 0x00008059) return 0;   //upper case 'Y' MANDATORY !
			sendB(CRR);
			sendString(UploadedStr);

			do  {
				recvword = pullTimedByte();
				if (recvword == 0xFFFFFFFF)  {
					sendB(0x20);
					sendB('o');
					sendB('k');
					displayString(0x10,gpOKStr);
					msDelay(2000);
					updateDisplay();
					break; //return 1;
				}
				if (recvword != 0x000080F0)  return 0;
				parameter = scanSysEx(0);		//returns 1 ... N_SOUNDS
				parameter--;					//0... N_SOUNDS-1
				sendS32(parameter);
				n = uploadSoundSet(parameter);
				if (n == 0)  {
					displayString(0x30+1,gErrorStr);
					msDelay(2000);
					updateDisplay();
					return 0;
				}
			}  while (1);
			break;

		case '@':
			sendString(UploadWaveStr);
			sendString(ConfirmStr);
			recvword = pullByte();
			if (recvword != 0x00008059) return 0;   //upper case 'Y' MANDATORY !
			sendB(CRR);
			sendString(UploadedStr);
			do  {
				recvword = pullTimedByte();
				if (recvword == 0xFFFFFFFF)  {
					sendB(0x20);
					sendB('o');
					sendB('k');
					displayString(0x10,gpOKStr);
					msDelay(2000);
					updateDisplay();
					break; //return 1;
				}
				if (recvword != 0x000080F0)  return 0;
				parameter = scanSysEx(0);		//returns 1,2,3,4
				parameter--;					//0,1,2,3 loaded User Wave
				sendS32(parameter);
				n = uploadUserWave(parameter);
				if (n == 0)  {
					displayString(0x30+1,gErrorStr);
					msDelay(2000);
					updateDisplay();
					return 0;
				}
			}  while (1);
			break;

        case '*':
            sendString(FilterTestStr);
			floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
            if (floatParameter > 1.E9) return 0;
            intParameter = (int32_t)floatParameter;
            if ((intParameter < -9) || (intParameter > 9)) return 0;
			gaFilterTest = intParameter;
			if (gaFilterTest >= 0)  gaFTestFactor = gDbTable[gaFilterTest * 3];
            else {
            	parameter = -intParameter;
                gaFTestFactor = gaFTestFactor = gDbTable[parameter * 3];
            }
			break;

        case '>':
            sendString(CopyPresetStr);
            n = parseHexNibble();
			if (n == 0xFFFFFFFE)  return 1; //user cancelled
            if (n == 0xFFFFFFFF)  return 0;
            if (n > 15) return 0;
            sendString(ConfirmStr);
            recvword = pullByte();
            if (recvword == 0x00008059) {
                copyPreset(n);              //upper case 'Y' MANDATORY !
                break;
            }
            else return 0;

        case 0x5C:                  //Backslash '\'
            sendString(LoadAllStr);
            sendString(ConfirmStr);
            recvword = pullByte();
            if (recvword == 0x00008059)  {      //upper case 'Y' MANDATORY !
                popFlashSounds(); //load complete setup from Flash and activate
                activateSoundSet();
                sendB(0x20);
                sendB('o');
                sendB('k');
				break;
            }
            else return 0;

        case '|':                   //vertical line
            sendString(SaveAllStr);
            sendString(ConfirmStr);
            recvword = pullByte();
            if (recvword == 0x00008059)  {      //upper case 'Y' MANDATORY !
                pushFlashSounds();         //store complete setup permanently
                sendB(0x20);
                sendB('o');
                sendB('k');
                msDelay(50);
                break;
            }
            else return 0;

           case '~':
                sendString(ClearAllStr);
                sendString(ConfirmStr);
                recvword = pullByte();
                if (recvword == 0x00008059)  {      //upper case 'Y' MANDATORY !
                    sendB(CRR);
                    sendString(MinusStr);
                    recvword = pullByte();
                    if (recvword != 0x0000802D)  return 0;  //'-' MANDATORY !
					eraseFlashSounds();
					sendB(CRR);
					msDelay(500);
					popFlashSounds();
					gSoundSet = 0;
					activateSoundSet();
					sendB(' ');
					sendB('o');
					sendB('k');
                }
			break;
        default: return  0;
    }
    sendB(CRR);
    return 1;
}


uint32_t setVidPid(void)  // ************************************************
// **************************************************************************
{
uint32_t nibble;
uint32_t Vid, Pid;
const uint8_t VidStr[] = {"Vid="};
const uint8_t PidStr[] = {"Pid="};
    sendString(VidStr);
	nibble = parseHexNibble();
	if (nibble == 0xFFFFFFFE)  return 1; //user cancelled
    if (nibble > 0x000000FF) return 0;
    Vid = nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + nibble;                // *16
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + nibble;
    sendB(' ');

    sendString(PidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + nibble;

    gVid = Vid;
    gPid = Vid;
    gPid = Pid;
    gPid = Pid;
    return 1;
}


uint32_t setMidiChannel(uint32_t drums)  // *********************************
// **************************************************************************
{
uint32_t digit, channel;

	digit = parseHexNibble();
	if (digit == 0xFFFFFFFE)  return 1; //user cancelled
    if (digit == 0xFFFFFFFF)  return 0;
    channel = digit;

	digit = parseHexNibble();
	if (digit == 0xFFFFFFFE)  return 1; //user cancelled
    if (digit == 0xFFFFFFFF)  return 0;
    channel = channel*10 + digit;
    channel --;         //conversion to "MIDI internal" channel number
    if (channel > 15) {
        sendB(' ');
        sendB('1');
        sendB('-');
        sendB('1');
        sendB('6');
        sendB('!');
        sendB('?');
        return 0;
    }
    if (drums) gDrumsChannel = channel;
    else gChannel = channel;
    return 1;
}

uint32_t setRouting(void)  // ***********************************************
//gaRoute0N: (fed trough filter0(Osc1,2) and filter3(Noise)
//see text files below
//if OSC1 or OSC2 or Noise not selected, it is directly fed into filter1
// **************************************************************************
{
const uint8_t RouteModeStr[] = {"Filter Arrangement: enter 'E'=Easy,'F'=FLex"};
const uint8_t F0NInputStr[] = {"select input to filter#0 or filter#1 - plus filter2(Noise). Enter:"};
const uint8_t Route1Str[] = {" 0:Osc1->F0 Osc2->F0, 1:Osc1->F0 Osc2->F0 + Noise,"};
const uint8_t Route2Str[] = {" 2:Osc1->F0 Osc2->F1, 3:Osc1->F0 Osc2->F1 + Noise,"};
const uint8_t Route3Str[] = {" 4:Osc1->F1 Osc2->F0, 5:Osc1->F1 Osc2->F0 + Noise,"};
const uint8_t Route4Str[] = {" 6:Osc1->F1 Osc2->F1, 7:Osc1->F1 Osc2->F1 + Noise,"};
const uint8_t FMStr[] = {"following selections start OSC1 Freq.Mod by OSC2. Enter:"};
const uint8_t Route5Str[] = {" 8:Osc1(FM)->F0, 9:Osc1(FM)->F0 + Noise,"};
const uint8_t Route6Str[] = {" A:Osc1(FM)->F1, B:Osc1(FM)->F1 + Noise,"};
const uint8_t AgainStr[] = {"input error, type again:"};
uint32_t  n, param;
uint16_t recvword;

	sendString(RouteModeStr);
GETFILTERTYPE:
    recvword = pullByte() & 0x000000FF;
    if ((recvword > 0x60) && (recvword <0x7B)) recvword -= 0x20;
     switch (recvword)  {
        case 'E':
            n = 0;
            break;
        case 'F':
            n = 1;
            break;
        default:
			sendString(AgainStr);	//escape if invalid entry
            sendB(CRR);
            goto GETFILTERTYPE;
     }
//serial cascade,F0out is fed into F12
GETROUTEF0N:
	sendB(CRR);
	sendString(F0NInputStr);
	sendB(CRR);
	sendString(Route1Str);
	sendB(CRR);
	sendString(Route2Str);
	sendB(CRR);
	sendString(Route3Str);
	sendB(CRR);
	sendString(Route4Str);
	sendB(CRR);
	sendString(FMStr);
	sendB(CRR);
	sendString(Route5Str);
	sendB(CRR);
	sendString(Route6Str);
	sendB(CRR);
	param = parseHexNibble();
	if (param == 0xFFFFFFFE)  return 1; 	//user cancelled
	if (param > 0x0B)  {
		sendB(CRR);
		sendString(AgainStr);				//escape if invalid entry
		sendB(CRR);
		goto GETROUTEF0N;
	}
	gaRoute0N = param;
	if (param >= 8)  gaRoute0N += 8;		//8-->0x10, etc
	if (gfFlexMode == 0)  {
		gaRoutePreset[gSoundSet] &= 0xFFFFFF00;
		gaRoutePreset[gSoundSet] |= gaRoute0N;
	}
	else  {
		gaRoutePreset[gSoundSet] &= 0xFF00FFFF;
		gaRoutePreset[gSoundSet] |= gaRoute0N << 16;
	}

	gfFlexMode = n;
	gfStageMask[gSoundSet] &= 0x000000BF;
	gfStageMask[gSoundSet] |= n << 6;	//bit6 stores active Filter Set

	updateRoute();						//performs  createShortFlags();
    sendB(' ');							//and inits filters immediately
    sendB('o');
    sendB('k');
    return 1;
}


void createShortFlags(void)   // ********************************************
//assign sound generators to value of gaRoute0N
// **************************************************************************
{
    gaRouteOsc1 = 0;
    gaRouteOsc2 = 0;
    gaRouteNoise = 0;
    switch (gaRoute0N)  {
    case 1:
        gaRouteOsc1 = 1;
        gaRouteOsc2 = 1;
        gaRouteNoise = 1;
        break;
    case 2:
	case 0x10:
        gaRouteOsc1 = 1;
        break;
	case 3:
    case 0x11:
        gaRouteOsc1 = 1;
        gaRouteNoise = 1;
        break;
    case 4:
        gaRouteOsc2 = 1;
        break;
    case 5:
        gaRouteOsc2 = 1;
        gaRouteNoise = 1;
        break;
    case 6:
	case 0x12:
//all values as entered above
        break;
    case 7:
	case 0x13:
        gaRouteNoise = 1;
        break;
    default:
    case 0:
        gaRouteOsc1 = 1;
        gaRouteOsc2 = 1;
    }
	if ((gfRuntimeMask&0x00000001)&&(gaRouteOsc1||gaRouteOsc2)) gsNoTriggerF0=0;
	else  gsNoTriggerF0 = 1;
}


uint32_t setADSR(void)  // ***************************************************
// **************************************************************************
{
const uint8_t AdsrSelect1Str[] = {"Select ADSR: 1,2,N(Noise),X(FMO1+2),L(Lfo1),S(Lfo2),B(Bass),T(Treble),"};
const uint8_t AdsrSelect2Str[] = {"             G(Gain),M(F0Mid),P(F0Peak),F(Free) "};
const uint8_t AttackStr[] = {"Attack Time (seconds.xx):"};
const uint8_t DecayStr[] = {"Decay Time (seconds.xx):"};
const uint8_t SustainStr[] = {"Sustain Level:(0..100%)"};
const uint8_t HoldStr[] = {" Sustain Type 1=Hold, 0=1Shoot:"};
const uint8_t ReleaseStr[] = {"Release Time (seconds.xx);"};
uint32_t parameter, type;
int32_t intParameter;
float floatParameter;
uint16_t recvword;

    sendString(AdsrSelect1Str);
    sendB(CRR);
    sendString(AdsrSelect2Str);
    recvword = pullByte() & 0x000000FF;
    if (recvword == 0x000000FF)  return 0;
    if ((recvword > 0x60) && (recvword <0x7B)) recvword -= 0x20;
    switch (recvword)  {
        case '1':
            type = 0;
            break;
        case '2':
            type = 1;
            break;
        case 'N':
            type = 2;
            break;
        case 'X':
            type = 3;
            break;
        case 'L':
            type = 4;
            break;
        case 'S':
            type = 5;
            break;
        case 'B':
            type = 6;
            break;
       case 'T':
            type = 7;
            break;
       case 'G':
            type = 8;
            break;
       case 'M':
            type = 9;
            break;
       case 'P':
            type = 10;
            break;
        case 'F':
            type = 11;		//N_ABASIC
            break;
        default:
            return 0;       //Escape if type error
    }
    sendB(CRR);

	if (type == N_ABASIC)  {
		sendString(gcAssignStr);
		sendB(CRR);
		sendString(gcFreeAdsrStr);
		sendB(CRR);
		if((parameter = parseHexNibble()) > 7)  return 0;
		gsFreeAdsr = parameter;
		gsFreeAdsrPreset[gSoundSet] = parameter;
		gsFreeIdx = N_ABASIC + gsFreeAdsr;
		type += parameter;
	}

    gsAdsrOffset = gSoundSet * N_ASTRUCT;
    do  {
        sendString(AttackStr);
        floatParameter = getFloat();
		if (floatParameter == 1.E8) return 1; //user cancelled
    } while (floatParameter > 1.E9);
    floatParameter *= 1000.0;
    parameter = (uint32_t)(floatParameter);
    if (floatParameter < 0.0) parameter = 0;
    if (parameter > 6000) parameter = 6000;
    if (parameter <= 1000) parameter = parameter / 10;
    else parameter = 100 + (parameter - 1000) / 200;
	packADSR(type,0,parameter);
    if (parameter <= 100) gsAttack[type] = parameter * A_FACTOR;
    else  gsAttack[type] = SECFACTOR + (parameter-100) * L_FACTOR;
    sendB(CRR);

    do  {
        sendString(DecayStr);
        floatParameter = getFloat();
		if (floatParameter == 1.E8) return 1; //user cancelled
    } while (floatParameter > 1.E9);
    floatParameter *= 1000.0;
    parameter = (uint32_t)floatParameter;
    if (floatParameter < 0.0) parameter = 0;
    if (parameter > 6000) parameter = 6000;
    if (parameter <= 1000) parameter = parameter / 10;
    else parameter = 100 + (parameter - 1000) / 200;
	packADSR(type,1,parameter);
    if (parameter <= 100) gsDecay[type] = parameter * A_FACTOR;
    else  gsDecay[type] = SECFACTOR + (parameter-100) * L_FACTOR;
    sendB(CRR);

    do  {
        sendString(SustainStr);
    } while ((parameter = getDeciNum(100)) > 100);
    gsSustain[type] = parameter;
    do  {
        sendString(HoldStr);
		parameter = parseHexNibble();
		if (parameter == 0xFFFFFFFE)  return 1; //user cancelled
    } while (parameter > 1);
    gsHoldSust[type] = parameter;
    if (parameter) packADSR(type,2,gsSustain[type] | 0x80);
    else  packADSR(type,2,gsSustain[type]);
    sendB(CRR);

    do  {
        sendString(ReleaseStr);
        floatParameter = getFloat();
		if (floatParameter == 1.E8) return 1; //user cancelled
    } while (floatParameter > 1.E9);
    floatParameter *= 1000.0;
    parameter = (uint32_t)floatParameter;
    if (floatParameter < 0.0) parameter = 0;
    if (parameter > 6000) parameter = 6000;
    if (parameter <= 1000) parameter = parameter / 10;
    else parameter = 100 + (parameter - 1000) / 200;
    packADSR(type,3,parameter);
    if (parameter <= 100) gsRelease[type] = parameter * A_FACTOR;
    else  gsRelease[type] = SECFACTOR + (parameter-100) * L_FACTOR;
    sendB(CRR);

	if ((type <= 2) || (type == 8))  {
							//Osc1+2, Noise, Gain have only positive envelope
        do  {
            sendString(gcEnvelopeStr);
            sendString(gcVolumeStr);
            sendB(':');
            parameter = getDeciNum(127);
            if (parameter == 0xFFFFFFFE) return 1; //user cancelled
        } while (parameter > 127);
        gsFixVol[type] = parameter;
        sendB(CRR);
    }
    else {				//all else are acquired symmetrically MIDI conformant,
						//type specific adjustment when applied
        do  {
            if ((type == 3) || (type == 4) || (type == 9) ||
			 ((type==11)&&((gsFreeAdsr==0)||(gsFreeAdsr==1)||(gsFreeAdsr==3)))){
				sendString(gcFreqShiftStr);				//filter mid freq shift
            }
            else  {
                sendString(gcLevelShiftStr);
//                sendString(gcVolumeStr);
            }
            sendB(':');
            floatParameter = getFloat();
            if (floatParameter == 1.E8) return 1; //user cancelled
        } while (floatParameter > 1.E9);
        intParameter = (int32_t)floatParameter;
		if (intParameter > 63) intParameter = 63;
		if (intParameter < -64) intParameter = -64;
		parameter = (uint32_t)(intParameter + 64);
        gsFixVol[type] = parameter;
        sendB(CRR);
    }
	packADSR(type,4,parameter);

    do  {
     	sendString(gcVeloStr);
        sendB(':');
        floatParameter = getFloat();
		if (floatParameter == 1.E8) return 1; //user cancelled
    } while (floatParameter > 1.E9);
    intParameter = (int32_t)floatParameter;
    if (intParameter > 99) intParameter = 99;
    if (intParameter < -28) intParameter = -28;
    parameter = (uint32_t)intParameter + 28;
    packADSR(type,5,parameter);
    gsVelo[type] = parameter;
    sendB(CRR);
    if (gpConfigPanel == 2) {
        gpAdsrIndex = type;
        updateDisplay();
    }
	sendB('O');
	sendB('k');
    return 1;
}


void packADSR(uint32_t number, uint32_t feature, uint32_t value)  // *********
// ***************************************************************************
{
	gsAdsrOffset = gSoundSet * N_ASTRUCT;		//reconfirm

	switch (feature)  {
	case 0:										//Attack
		gsADSR[gsAdsrOffset + number].ATDECSUS &= 0x00FFFFFF;
		gsADSR[gsAdsrOffset + number].ATDECSUS |= (value & 0x000000FF) << 24;
		break;
	case 1:										//Decay
		gsADSR[gsAdsrOffset + number].ATDECSUS &= 0xFF00FFFF;
		gsADSR[gsAdsrOffset + number].ATDECSUS |= (value & 0x000000FF) << 16;
		break;
	case 2:										//Sustain
		gsADSR[gsAdsrOffset + number].ATDECSUS &= 0xFFFF0000;
		gsADSR[gsAdsrOffset + number].ATDECSUS |= value & 0x0000FFFF;
		break;
	case 3:										//Release
		gsADSR[gsAdsrOffset + number].REFIXVEL &= 0x0000FFFF;
		gsADSR[gsAdsrOffset + number].REFIXVEL |= (value & 0x000000FF) << 16;
		break;
	case 4:										//FixVol
		gsADSR[gsAdsrOffset + number].REFIXVEL &= 0x00FF00FF;
		gsADSR[gsAdsrOffset + number].REFIXVEL |= (value & 0x000000FF) << 8;
		break;
	case 5:										//Velo
		gsADSR[gsAdsrOffset + number].REFIXVEL &= 0x00FFFF00;
		gsADSR[gsAdsrOffset + number].REFIXVEL |= value & 0x000000FF;
	default:
		break;
	}
}


void unpackADSR(uint32_t n)  // **********************************************
// ***************************************************************************
{
uint32_t p, xtract;

	gsAdsrOffset = gSoundSet * N_ASTRUCT;					//reconfirm
	p = gsAdsrOffset + n;

	xtract = (gsADSR[p].ATDECSUS >> 24) & 0x000000FF;		//Attack
	if (xtract <= 100) gsAttack[n] = xtract * A_FACTOR;
	else gsAttack[n] = SECFACTOR + (xtract - 100) * L_FACTOR;
	xtract = (gsADSR[p].ATDECSUS >> 16) & 0x000000FF;		//Decay
	if (xtract <= 100) gsDecay[n] = xtract * A_FACTOR;
	else gsDecay[n] = SECFACTOR + (xtract - 100) * L_FACTOR;
	xtract = gsADSR[p].ATDECSUS & 0x000000FF;			//Sustain incl Hold bit
	gsSustain[n] = xtract & 0x7F;
	gsHoldSust[n] = (xtract >> 7) & 1;
	xtract = (gsADSR[p].REFIXVEL >> 16) & 0x000000FF;		//Release
	if (xtract <= 100) gsRelease[n] = xtract *A_FACTOR;
	else gsRelease[n] = SECFACTOR + (xtract - 100) * L_FACTOR;
	gsFixVol[n] = (gsADSR[p].REFIXVEL >> 8) & 0x000000FF;
	gsVelo[n] = gsADSR[p].REFIXVEL & 0x000000FF;
}


uint32_t initLfo(void)  // **************************************************
// **************************************************************************
{
const uint8_t LfoNumStr [] = {"Lfo Number(1,2,3) ?"};
const uint8_t LfoWaveTypeStr[] = {"Wave Type: s=Sine, t=Triangle/saw, r=Square ?"};
const uint8_t LfoParamStr[] = {"Wave Shape(10...90), 50=symmetric ?"};
const uint8_t LfoFreqStr[] = {"Freq(0.1...24.8Hz, 0 = OFF) ?"};
const uint8_t LfoSlopeStr[] = {"Start Slope (1=pos, 0=neg ?"};
uint32_t freq, number, type, wave, percent, slope;
float floatParameter;

AGAIN:
	sendString(LfoNumStr);
	number = parseHexNibble();
	if (number == 0xFFFFFFFE)  return 1; //user cancelled
    if (number == 0xFFFFFFFF)  return 0;
	if ((number == 0) || (number > 3)) return 0;
	sendB(CRR);
	sendString(LfoWaveTypeStr);
	freq = pullByte();
	if (freq == 0xFFFF) return 0;					//freq misused here
	freq &= 0x000000FF;
	if ((freq > 0x60) && (freq <0x7B)) freq -= 0x20;
	if (freq == 'S') type = 0;
	else if (freq == 'T') type = 1;
	else if (freq == 'R') type = 2;
	else  {
		sendB('?');
		sendB(CRR);
		goto AGAIN;
	}
	sendB('=');
	sendB(freq);                                    //freq misused here
	if (type != 0)  {
		sendB(CRR);
		sendString(LfoParamStr);
		percent = getDeciNum(99);
		if (percent == 0xFFFFFFFE)  return 1; 		//user cancelled
		if (percent > 99)  goto AGAIN;
		if (percent < 10) percent = 10;
		if (percent > 90) percent = 90;
	}
	sendB(CRR);

FREQAGAIN:
	sendString(gcFreqStr);
	sendString(LfoFreqStr);
	floatParameter = getFloat();
	if (floatParameter == 1.E8) return 1; //user cancelled
	if (floatParameter < 0.)  goto FREQAGAIN;       //0=LFO OFF
	if (floatParameter > 24.8)  {
		sendB('!');
		floatParameter = 24.8;
	};
	freq = (uint32_t)(floatParameter * 10);         //-->intParameter;

SLOPEAGAIN:
	sendB(CRR);
	sendString(LfoSlopeStr);
	number = parseHexNibble();
	if (number == 0xFFFFFFFE)  return 1; //user cancelled
    if (number == 0xFFFFFFFF)  goto SLOPEAGAIN;
	if (number > 1) goto SLOPEAGAIN;
	slope = number;
	sendB(CRR);

	if (type == 0)  wave = 0;
	else if (type == 1) wave = percent;
	else if (type == 2)  wave = percent | 0x00000080;

	gLfoNum = number;
	if (number == 1)  {
        if (freq <= 5)  {
			gLfo1FreqPreset[gSoundSet] = freq;
			gLfo1Freq = freq;						//0...5, gLfo1Step is 0.1Hz
        }
        else  {							//gLfo1Step increased in steps of 0.2Hz
			gLfo1FreqPreset[gSoundSet] = (freq / 2) + 3;
			gLfo1Freq = (gLfo1FreqPreset[gSoundSet] - 3) * 2;
        }
		gLfo1Wave = wave | (slope << 8);
		gLfo1Type = type;
		gLfo1WavePreset[gSoundSet] = gLfo1Wave;
	}
	else if (number == 2)  {
        if (freq <= 5)  {
			gLfo2FreqPreset[gSoundSet] = freq;
			gLfo2Freq = freq;						//0...5, gLfo2Step is 0.1Hz
        }
        else  {							//gLfo2Step increased in steps of 0.2Hz
			gLfo2FreqPreset[gSoundSet] = (freq / 2) + 3;
			gLfo2Freq = (gLfo2FreqPreset[gSoundSet] - 3) * 2;
        }
		gLfo2Wave = wave | (slope << 8);;
		gLfo2Type = type;
		gLfo2WavePreset[gSoundSet] = gLfo2Wave;
	}
	else   {
        if (freq <= 5)  {
			gLfo3FreqPreset[gSoundSet] = freq;
			gLfo3Freq = freq;						//0...5, gLfo3Step is 0.1Hz
        }
        else  {							//gLfo3Step increased in steps of 0.2Hz
			gLfo3FreqPreset[gSoundSet] = (freq / 2) + 3;
			gLfo3Freq = (gLfo3FreqPreset[gSoundSet] - 3) * 2;
        }
		gLfo3Wave = wave | (slope << 8);;
		gLfo3Type = type;
		gLfo3WavePreset[gSoundSet] = wave;
	}
	calculateLfoWave(number, wave);
	updateDisplay();
	return 1;
}


void calculateLfoWave(uint32_t lfoNum, uint32_t wave)  // ********************
// ***************************************************************************
{
uint32_t wavetype, parameter;
uint32_t k, n, slope;
int32_t temp;
float fTemp;
int16_t level;

	if (wave == 0)  {
		n = -4;
		for (k=0; k < 1025; k++)  {			//stretch range 0..1023 to 0..3999
			if ((k % 42) == 0) continue;
			else n += 4;
			if (k == 1024) level = 0;
			else level = gsSinus[k];

			if (lfoNum == 1)  {
				gsWaveLfo1[n] = level;
				gsWaveLfo1[n+1] = level;
				gsWaveLfo1[n+2] = level;
				gsWaveLfo1[n+3] = level;
			}
			else if (lfoNum == 2)  {
				gsWaveLfo2[n] = level;
				gsWaveLfo2[n+1] = level;
				gsWaveLfo2[n+2] = level;
				gsWaveLfo2[n+3] = level;
			}
			else  {
				gsWaveLfo3[n] = level;
				gsWaveLfo3[n+1] = level;
				gsWaveLfo3[n+2] = level;
				gsWaveLfo3[n+3] = level;
			}
		}
	}
	else  {
		wavetype = wave & 0x80;				//0=triangle<>saw, 0x80=sqare
		parameter = wave & 0x7F;			//range 10..90, 50=symmetric
		if (parameter < 10) parameter = 10;
		if (parameter > 90) parameter = 90;
		slope = parameter * 20;				//range 0(200)...2000(1800)
		parameter *= 40;					//range 0(400) ...4000(3800)

	   for (k=0; k < 4000; k++)  {
			if (wavetype == 0)  {			//variable shape triangle/saw wave
				if (k < slope) level = (int16_t)((8192 * k) / slope);
				else if ((k >= slope) && (k < (4000 - slope)))  {
					level = (int16_t)(8192 -(16384 *(k-slope))/(4000 -2*slope));
				}
				else  {
					temp = -8192 * (4000 - k);
					fTemp = (float)temp / slope;//else compiler int div problem
					level = (int16_t)fTemp;
				}
			}
			else  {							//variable pulse width square wave
				if (k < 200) level = (8192 * k) / 200;
				else if (k >= 3800) {
					temp = -8192 * (4000 - k);
					temp = (int32_t)(temp / 200);//else compiler casting problem
					level = (int16_t)temp;
				}
				else if (k < (parameter-200)) level = 8191;
				else if (k < (parameter+200)) {
					level=(int16_t)(8192-(16384*(k-parameter+200))/(400));
				}
				else  if (k < 3800) level = -8192;
			}
			if (lfoNum == 1)  gsWaveLfo1[k] = level;
			else if (lfoNum == 2)  gsWaveLfo2[k] = level;
			else  gsWaveLfo3[k] = level;
	   }
	}
}


uint32_t setupFreeSquare(void)  // **********************************************
// **************************************************************************
{
const uint8_t fsNumStr[] = {"config Free SquareWave(1,2) ?"};
const uint8_t fsRangeStr[] = {"enter HiRange (10.0-50.0%):"};
const uint8_t fsSlopeStr[] = {"enter Slope (max HiRange/2):"};
uint32_t num, hiRange, slope;
float rangeParam, slopeParam;

	sendString(fsNumStr);
	num = parseHexNibble();
	if (num == 0xFFFFFFFE)  return 1;	//user cancelled
    if (num == 0xFFFFFFFF)  return 0;
    if ((num != 1 ) && (num != 2)) return 0;
    sendB(CRR);
    sendString(fsRangeStr);
    rangeParam = getFloat();
	if (rangeParam == 1.E8) return 1;	//user cancelled
    if (rangeParam > 1.E9) return 0;
    if (rangeParam < 10.0) rangeParam= 10.0;
    if (rangeParam > 50.0) rangeParam = 50.0;
	hiRange = (uint32_t)(rangeParam * 10);
    sendB(CRR);
    sendString(fsSlopeStr);
    slopeParam = getFloat();
	if (slopeParam == 1.E8) return 1;	//user cancelled
    if (slopeParam > 1.E9) return 0;
    if (slopeParam < 0.0) slopeParam = 0.0;
	slope = (uint32_t)(slopeParam * 10);
    if (slope > (hiRange / 2)) slope = hiRange / 2;
    if (num == 1)  {
		gsRangeFS1 = hiRange + hiRange / 41;	//e.g.:adjust 500->512, 100->103
		gsSlopeFS1 = slope + slope / 41;
		gsFreeSquare1[gSoundSet] =((hiRange/10)<<24)|((hiRange%10)<<16)|slope/2;
		if (gsWAVEFORM_1 == 9) initWaveOsc1(9);
	}
	else  {
		gsRangeFS2 = hiRange + hiRange / 41;	//e.g.:adjust 500->512, 100->103
		gsSlopeFS2 = slope + slope / 41;
		gsFreeSquare2[gSoundSet] =((hiRange/10)<<24)|((hiRange%10)<<16)|slope/2;
		if (gsWAVEFORM_1 == 10) initWaveOsc1(10);
	}
    updateDisplay();
	return 1;
}


uint32_t setDrumVolume(void)  // ********************************************
// **************************************************************************
{
uint32_t drum, volume;
float floatParam;
const uint8_t drumNoStr[] = {"-enter Drum no: (A=all)"};

	sendString(gcDrumsVolStr);
    sendString(drumNoStr);
	drum= parseHexNibble();
	if (drum == 0xFFFFFFFE)  return 1;			//user cancelled
    if (drum == 0xFFFFFFFF)  return 0;
    if ((drum == 8) || (drum == 9) || (drum > 0x0A)) return 0;
	sendString(gcVolumeStr);
    sendString(gcDBStr);

    floatParam = getFloat();
	if (floatParam == 1.E8) return 1;			//user cancelled
    if (floatParam > 1.E9) return 0;
    if (floatParam < -40.0) floatParam = -40.0;
    if (floatParam > 12.0) floatParam = 12.0;
    volume = (int32_t)(floatParam + 40.0);
    switch (drum)  {
        case 0:
            gdKickVol = volume;					//range 0 ..52
            gdKickVolPreset[gSoundSet] = gdKickVol;
            gdKickVolFactor = getVolFactor(gdKickVol);
            break;
        case 1:
            gdHiTomVol = volume;				//range 0 ..52
            gdHiTomVolPreset[gSoundSet] = gdHiTomVol;
            gdHiTomVolFactor = getVolFactor(gdHiTomVol);
            break;
        case 2:
            gdSnareVol = volume;				//range 0 ..52
            gdSnareVolPreset[gSoundSet] = gdSnareVol;
            gdSnareVolFactor = getVolFactor(gdSnareVol);
            break;
        case 3:
            gdRimVol = volume;					//range 0 ..52
            gdRimVolPreset[gSoundSet] = gdRimVol;
            gdRimVolFactor = getVolFactor(gdRimVol);
            break;
        case 4:
            gdClapVol = volume;					//range 0 ..52
            gdClapVolPreset[gSoundSet] = gdClapVol;
            gdClapVolFactor = getVolFactor(gdClapVol);
            break;
        case 5:
            gdHihatVol = volume;				//range 0 ..52
            gdHihatVolPreset[gSoundSet] = gdHihatVol;
            gdHihatVolFactor = getVolFactor(gdHihatVol);
            break;
         case 6:
            gdCymbalVol = volume;				//range 0 ..52
            gdCymbalVolPreset[gSoundSet] = gdCymbalVol;
            gdCymbalVolFactor = getVolFactor(gdCymbalVol);
            break;
        case 7:
            gdBellVol = volume;					//range 0 ..52
            gdBellVolPreset[gSoundSet] = gdBellVol;
            gdBellVolFactor = getVolFactor(gdBellVol);
            break;
        case 0x0A:                          //global setting of drums volume
        default:
            gdDrumsVol = volume;            //range 0 ..52
            getAllVolFactors();
            break;
    }
    updateDisplay();
    return 1;
}


uint32_t parseHexNibble(void)   // ******************************************
// **************************************************************************
{
uint32_t recvword, nibble;

    recvword = pullByte();
    nibble = (uint32_t)(recvword & 0x00FF);
    if ((nibble == LFF)||(nibble == CRR)||(nibble == ESC))  return 0xFFFFFFFE;
															//User cancelled
    if  ((nibble >= 0x30) && (nibble <= 0x39))  {
        sendB((uint8_t)nibble);
        return (nibble - 0x30);
    }
    if ((nibble > 0x60) && (nibble < 0x7B))  nibble -= 0x20; //force uppercase
    if ((nibble >= 'A') && (nibble <= 'F')) {
        sendB((uint8_t)nibble);
        return (nibble - 0x37);
    }
    else return 0xFFFFFFFF;
}


uint32_t parseHexNum(uint32_t places)   // ***********************************
// **************************************************************************
{
uint32_t number;
uint32_t recvword;
uint8_t nibble;

    number = 0;
    if (places > 7) return 0xFFFFFFFF ;
    if (places == 0) return 0x00000000;
    while (places)   {
        recvword = pullByte();
		if (recvword == (0x0000801B)) return 0xFFFFFFFE; //ESC=User cancelled
        nibble = (uint8_t) (recvword & 0x000000FF);
        if ((nibble > 0x60) && (nibble <0x7B)) nibble -= 0x20;
        if (nibble == LFF) continue;           //ignore
        if (nibble == CRR) return number;      //CR terminates input
        if ((nibble < 0x30) || (nibble > 0x46)) return 0xFFFFFFFF ;
        if ((nibble > 0x39) && (nibble < 0x41)) return 0xFFFFFFFF ;
        sendB(nibble);
        nibble -=  0x30;
        if (nibble > 9) nibble -= 7 ;
        number = (number<<4) + (uint32_t) nibble;
        places-- ;
    }
        return number;
}


void activateSoundSet(void)  // **********************************************
// ***************************************************************************
{
uint32_t n, temp;

    gVolume = gVolumePreset[gSoundSet];
    gVolumeFactor = gDbTable[gVolume] >> 3;     	//max.511 allowed

    gsWAVEFORM_1 = gsWAVEFORM_1Preset[gSoundSet];
    if (gsWAVEFORM_1 & 0x40) gsOsc1ModLfo2 = 1; 	//Osc1 modulated by Lfo2
    else gsOsc1ModLfo2 = 0;							//Osc1 modulated by Lfo1
    gsWAVEFORM_1 &= 0x3F;
    switch (gsWAVEFORM_1)  {
		case 0:
		case 25:
		case 36:
		case 37:
		case 38:
		case 41:
			gaZero1 = 0;
			break;
		default:
			gaZero1 = 1;
    }

    gsWAVEFORM_2 = gsWAVEFORM_2Preset[gSoundSet];
    if (gsWAVEFORM_2 & 0x40) gsOsc2ModLfo2 = 1;		//Osc2 modulated by Lfo2
    else gsOsc2ModLfo2 = 0;							//Osc2 modulated by Lfo1
    gsWAVEFORM_2 &= 0x3F;
    switch (gsWAVEFORM_2)  {
		case 0:
		case 25:
		case 36:
			gaZero2 = 0;
			break;
		default:
			gaZero2 = 1;
    }

	n = (gsFreeSquare1[gSoundSet] >> 24) & 0x000000FF;
	temp = (gsFreeSquare1[gSoundSet] >> 16) & 0x000000FF;
	n = n*10 + temp;
	gsRangeFS1 = n + (n / 41);	//stretch 500->512, e.g.
	temp = (gsFreeSquare1[gSoundSet] & 0x000000FF) *2;
	if (temp > (n/2)) temp = n/2;
	gsSlopeFS1 = temp + (temp / 41);	//stretch 250->256, e.g.
	n = (gsFreeSquare2[gSoundSet] >> 24) & 0x000000FF;
	temp = (gsFreeSquare2[gSoundSet] >> 16) & 0x000000FF;
	n = n*10 + temp;
	gsRangeFS2 = n + (n / 41);
	temp = (gsFreeSquare2[gSoundSet] & 0x000000FF) *2;
	if (temp > (n/2)) temp = n/2;
	gsSlopeFS2 = temp + (temp / 41);

    gsGLIDE = gsGLIDEPreset[gSoundSet];
    gsTRANS = gsTRANSPreset[gSoundSet];
    gsTRANS_2 = gsTRANS_2Preset[gSoundSet];
    gsDETUNE = gsDETUNEPreset[gSoundSet] - 64;
    gsDETUNE_2 = gsDETUNE_2Preset[gSoundSet] - 64;

    gLfo1Freq = gLfo1FreqPreset[gSoundSet];
    if (gLfo1Freq > 5) gLfo1Freq = (gLfo1Freq - 3) * 2;
    if (gLfo1WavePreset[gSoundSet] & 0x00000100) {
		gLfo1WavePreset[gSoundSet] &= 0x000000FF; //invert bit8 = start slope
    }
    else gLfo1WavePreset[gSoundSet] |= 0x00000100;
    gLfo1Wave = gLfo1WavePreset[gSoundSet];
    if ((gLfo1Wave & 0xFF) == 0)  gLfo1Type = 0;
    else if (gLfo1Wave & 0x80)  gLfo1Type = 2;
    else  gLfo1Type = 1;
    gLfo1Slope = (gLfo1Wave >> 8) & 0x000000001;
    calculateLfoWave(1,gLfo1Wave & 0x000000FF);

    gLfo2Freq = gLfo2FreqPreset[gSoundSet];
    if (gLfo2Freq > 5) gLfo2Freq = (gLfo2Freq - 3) * 2;
    if (gLfo2WavePreset[gSoundSet] & 0x00000100) {
		gLfo2WavePreset[gSoundSet] &= 0x000000FF; //invert bit8 = start slope
    }
    else gLfo2WavePreset[gSoundSet] |= 0x00000100;
    gLfo2Wave = gLfo2WavePreset[gSoundSet];
    if ((gLfo2Wave & 0xFF) == 0)  gLfo2Type = 0;
    else if (gLfo2Wave & 0x80)  gLfo2Type = 2;
    else  gLfo2Type = 1;
    gLfo2Slope = (gLfo2Wave >> 8) & 0x000000001;
    calculateLfoWave(2,gLfo2Wave & 0x000000FF);

    gLfo3Freq = gLfo3FreqPreset[gSoundSet];
    if (gLfo3Freq > 5) gLfo3Freq = (gLfo3Freq - 3) * 2;
    if (gLfo3WavePreset[gSoundSet] & 0x00000100) {
		gLfo3WavePreset[gSoundSet] &= 0x000000FF; //invert bit8 = start slope
    }
    else gLfo3WavePreset[gSoundSet] |= 0x00000100;
    gLfo3Wave = gLfo3WavePreset[gSoundSet];
    if ((gLfo3Wave & 0xFF) == 0)  gLfo3Type = 0;
    else if (gLfo3Wave & 0x80)  gLfo3Type = 2;
    else  gLfo3Type = 1;
    gLfo3Slope = (gLfo3Wave >> 8) & 0x000000001;
    calculateLfoWave(3,gLfo3Wave & 0x000000FF);

    gsMOD_1 = gsMOD_1Preset[gSoundSet] & 0x0000007F;
    gsMOD_2 = gsMOD_2Preset[gSoundSet] & 0x0000007F;
    gsOsc1PWM = (gsMOD_1Preset[gSoundSet] >> 7) & 0x00000001;
    gsOsc2PWM = (gsMOD_2Preset[gSoundSet] >> 7) & 0x00000001;

	panic_Off(0);
    gsFreeAdsr = gsFreeAdsrPreset[gSoundSet];
    gsAdsrOffset = gSoundSet * N_ASTRUCT;
    gcAdsrSel = 9; //Selection by MIDI CtrlChange:default=Filter0 Peak ADSR

    for (n = 0; n < N_ASTRUCT; n++) unpackADSR(n);
    gDelayTime = gDelayTimePreset[gSoundSet];
			// packed specially:bits 8-15=hundreds and tens,bits0-7=ones+tenths
    gDelayOffset = ((gDelayTime >> 8) & 0x000000FF) * 331 +  \
                                        ((gDelayTime & 0x000000FF) * 331)/100;

    gVibrato = gVibratoPreset[gSoundSet] & 0x0000007F;
    if (gVibrato > 99)  gVibrato = 99;
    gVibratoLfo1= (gVibratoPreset[gSoundSet] >> 7) & 0x00000001; //default Lfo1

    if (gDelayLevelPreset[gSoundSet] >= 0x80)  {    //bit7 marks phase inversion
        gDelayLevelFactor = - (gDelayLevelPreset[gSoundSet] & 0x7F);
    }
    else  {
        gDelayLevelFactor = gDelayLevelPreset[gSoundSet];
    }
    if (gFeedbackPreset[gSoundSet] >= 0x80)  {//bit7 marks phase inversion
        gFeedbackFactor = - (gFeedbackPreset[gSoundSet] & 0x7F);
    }
    else  {
        gFeedbackFactor = gFeedbackPreset[gSoundSet];
    }

    gChorusWidth = gChorusPreset[gSoundSet] & 0x0000007F;
    gChorusLfo3 = (gChorusPreset[gSoundSet]>>7) & 0x00000001;//default Lfo2

    gdDrumsVol = gdDrumsVolPreset[gSoundSet];
    gdKickVol = gdKickVolPreset[gSoundSet];
    gdHiTomVol = gdHiTomVolPreset[gSoundSet];
    gdSnareVol = gdSnareVolPreset[gSoundSet];
    gdRimVol = gdRimVolPreset[gSoundSet];
    gdClapVol = gdClapVolPreset[gSoundSet];
    gdHihatVol = gdHihatVolPreset[gSoundSet];
    gdCymbalVol = gdCymbalVolPreset[gSoundSet];
    gdBellVol = gdBellVolPreset[gSoundSet];
    getAllVolFactors();

//Routing, Keytrack, Gain, Balance Presets
    gsKeytrack0 = (float)gsKeytrack0Preset[gSoundSet] / 100.;
    gfGain = gfGainPreset[gSoundSet];
    gfF0mod = gfF0modPreset[gSoundSet] & 0x0000007F;
    gfF0modLfo2 = (gfF0modPreset[gSoundSet] >> 7) & 0x00000001;
	gfF0ModBase = 1.0;
    unpackBasTreb();

    gsO2Sync = (gfStageMask[gSoundSet] >> 7) & 0x00000001;
	gsMonoPoly = (gfStageMask[gSoundSet] >> 5) & 0x00000001;
    gfRuntimeMask = gfStageMask[gSoundSet];
	if (gfFlexMode == 0)  gaRoute0N = gaRoutePreset[gSoundSet] & 0x000000FF;
	else  gaRoute0N = (gaRoutePreset[gSoundSet] >> 16)&0x000000FF;
    createShortFlags();

    for (n = 0;n < N_FILTER; n++)  {
//proforma:check virgin flash against user setup and complete struct gfIIRPack[]
		gfFlexMode = 0;
		activateFilter(n,0);		//replace virgin flash by specific default
		gfFlexMode = 1;
		activateFilter(n,0);		//replace virgin flash by specific default
//finally:
		gfFlexMode = (gfStageMask[gSoundSet] >> 6) & 0x00000001;
		activateFilter(n,0);
        gfState1[n] = 0;
        gfState2[n] = 0;
    }
	updateRoute();
    gfFilterNum = 0;				//assume primarily until set specifically
    gfFilterType = gfACTIV_IIR[0].TYPE;

    initWaveOsc1(gsWAVEFORM_1);		//default:0=Sinus
    initWaveOsc2(gsWAVEFORM_2);		//update StageMask and RuntimeMask
    updateDisplay();
}


void activateFilter(uint32_t n, uint32_t initDefault)  // ********************
// ***************************************************************************
{
uint32_t p;

	unpackFilter(n, 0);		//extract gfACTIV_IIR[n], dont't init immediately

	if ((gfACTIV_IIR[n].FREQ != 0) && (initDefault == 0)) {
		initFilterParams(n,gfACTIV_IIR[n].TYPE,
		gfACTIV_IIR[n].FREQ, gfACTIV_IIR[n].QFACTOR, gfACTIV_IIR[n].PEAK,0);
								//refresh gfACTIV_IIR[number], run packFilter()
	}
	else  {							//init default parameters
		if (gfFlexMode == 0)  {		//EASY
			switch (n)  {		//refreshes gfACTIV_IIR[number],run packFilter()
			case 0:
				initFilterParams(0, 6, 84, 81, 20, 0);
				break;		//initFilterParams() takes gfFlexMode into account
			case 1:
				if (gSoundSet== 1) initFilterParams(1,3,84,71,20,0);//LowPass
				else  initFilterParams(1, 0, 69, 71, 20, 0);		//off
				break;
			case 2:
				initFilterParams(2, 0, 69, 71, 20, 0);
				break;
			case 3:
				initFilterParams(3, 7, 71, 100, 20, 0);
				break;
			case 4:
				initFilterParams(4, 8, 104, 110, 20, 0);
				break;
			default:
				initFilterParams(1, 0, 69, 71, 20, 0);				//off
			}
		}
		else {		//FLEX
			switch (n)  {		//refreshes gfACTIV_IIR[number],run packFilter()
			case 0:
				initFilterParams(0, 6, 84, 81, 20, 0);
				break;		//initFilterParams() takes gfFlexMode into account
			case 1:
				if (gSoundSet== 1) initFilterParams(1,3,84,71,20,0);//LowPass
				else  initFilterParams(1, 0, 69, 71, 20, 0);		//off
				break;
			case 2:
				initFilterParams(2, 0, 69, 71, 20, 0);
				break;
			case 3:
				initFilterParams(3, 7, 71, 100, 20, 0);
				break;
			case 4:
				initFilterParams(4, 8, 104, 110, 20, 0);
				break;
			default:
				initFilterParams(1, 0, 69, 71, 20, 0);
			}
		}
    }
	if ((gfRuntimeMask&0x00000001)&&(gaRouteOsc1||gaRouteOsc2)) gsNoTriggerF0=0;
	else  gsNoTriggerF0 = 1;
	if (initDefault)  {
		if (n == 3)  adjPeak3(gfBass);
		if (n == 4)  adjPeak4(gfTreble);
		}
}


void copyPreset(uint32_t target)   // ****************************************
//preset is copied from actual soundset to target.
//but the actual soundset is maintained, target is not activated
// ***************************************************************************
{
uint32_t k, n, p;

    gsWAVEFORM_1Preset[target] = gsWAVEFORM_1Preset[gSoundSet];
    gsWAVEFORM_2Preset[target] = gsWAVEFORM_2Preset[gSoundSet];
    gsTRANSPreset[target] = gsTRANSPreset[gSoundSet];
    gsTRANS_2Preset[target] = gsTRANS_2Preset[gSoundSet];
    gsDETUNEPreset[target] = gsDETUNEPreset[gSoundSet];
    gsDETUNE_2Preset[target] = gsDETUNE_2Preset[gSoundSet];

	gsMOD_1Preset[target] = gsMOD_1Preset[gSoundSet];
    gsMOD_2Preset[target] = gsMOD_2Preset[gSoundSet];
    gsGLIDEPreset[target] = gsGLIDEPreset[gSoundSet];

    gLfo1WavePreset[target] = gLfo1WavePreset[gSoundSet];
    gLfo1FreqPreset[target] = gLfo1FreqPreset[gSoundSet];
    gLfo2WavePreset[target] = gLfo2WavePreset[gSoundSet];
    gLfo2FreqPreset[target] = gLfo2FreqPreset[gSoundSet];
    gLfo3WavePreset[target] = gLfo3WavePreset[gSoundSet];
    gLfo3FreqPreset[target] = gLfo3FreqPreset[gSoundSet];

    gfGainPreset[target] = gfGainPreset[gSoundSet];
    gfF0modPreset[target] = gfF0modPreset[gSoundSet];
    gfBasTrebPreset[target] = gfBasTrebPreset[gSoundSet];
    gaRoutePreset[target] = gaRoutePreset[gSoundSet];

    gDelayTimePreset[target]= gDelayTimePreset[gSoundSet];
    gDelayLevelPreset[target] = gDelayLevelPreset[gSoundSet];
    gFeedbackPreset[target] = gFeedbackPreset[gSoundSet];
    gChorusPreset[target] = gChorusPreset[gSoundSet];
    gVibratoPreset[target] = gVibratoPreset[gSoundSet];

    gdDrumsVolPreset[target] = gdDrumsVolPreset[gSoundSet];
    gdKickVolPreset[target] = gdKickVolPreset[gSoundSet];
    gdHiTomVolPreset[target] = gdHiTomVolPreset[gSoundSet];
    gdSnareVolPreset[target] = gdSnareVolPreset[gSoundSet];
    gdRimVolPreset[target] = gdRimVolPreset[gSoundSet];
    gdClapVolPreset[target] = gdClapVolPreset[gSoundSet];
    gdHihatVolPreset[target] = gdHihatVolPreset[gSoundSet];
    gdCymbalVolPreset[target] = gdCymbalVolPreset[gSoundSet];
    gdBellVolPreset[target] = gdBellVolPreset[gSoundSet];

    gsAdsrOffset = gSoundSet * N_ASTRUCT;
    for (n = 0; n < N_ASTRUCT; n++)  {
        p = gsAdsrOffset + n;
        k = (target * N_ASTRUCT) + n;
        gsADSR[k].ATDECSUS = gsADSR[p].ATDECSUS;
        gsADSR[k].REFIXVEL = gsADSR[p].REFIXVEL;
    }
    gsFreeAdsrPreset[target] = gsFreeAdsrPreset[gSoundSet];

    gfStageMask[target] = gfStageMask[gSoundSet];
    gsKeytrack0Preset[target] = gsKeytrack0Preset[gSoundSet];
    for (n = 0;n < N_FILTER;n++)  {
        p = (gSoundSet * N_FILTER) + n;
        k = (target * N_FILTER) + n;
        gfIIRPack[k].TYPE = gfIIRPack[p].TYPE ;
        gfIIRPack[k].FREQ = gfIIRPack[p].FREQ ;
        gfIIRPack[k].QFACTOR = gfIIRPack[p].QFACTOR ;
        gfIIRPack[k].PEAK = gfIIRPack[p].PEAK ;
    }
    sendB('o');
    sendB('k');
    sendB(CRR);
}


void listSetup(void)   // ****************************************************
// ***************************************************************************
{
const uint8_t PolyStr[] = {"- 10voices Polyphonic -"};
const uint8_t MonoStr[] = {"- Monophonic mode -"};
const uint8_t FilterNumStr[] = {"Filter# "};
const uint8_t FreqModLfoStr[] = {"     FreqMod@Lfo"};
const uint8_t PeakModStr[] = {"     PeakMod:"};
const uint8_t BwResModStr[] = {"     BwResMod:"};
const uint8_t FreeAdsrStr[] = {"Free ADSR:"};
const uint8_t FNoiseStr[] = {"NoiseFilt."};
const uint8_t VibratoStr[] = {"Vibrato"};
const uint8_t ChorusStr[] = {"Chorus"};
const uint8_t DryStr[] = {"DryLevel"};
const uint8_t DelayLevelStr[] = {"Delay"};
const uint8_t FeedbackStr[] = {"Feedback"};
const uint8_t DelayStr[] = {"Delay: Time="};
const uint8_t EasyStr[] = {" 'Easy' -"};
const uint8_t FlexStr[] = {" 'Flex' -"};

const uint8_t BassStr[] = {"Bass:  "};
const uint8_t TrebleStr[] = {"Treble:"};
const uint8_t MixStr[] = {"  Mix(+F0/-F12):"};
const uint8_t DashStr[] = {"-----------------------"};
const uint8_t Lfo1Str[] = {"Lfo1"};
const uint8_t Lfo2Str[] = {"Lfo2"};
const uint8_t Lfo3Str[] = {"Lfo3"};
const uint8_t FMOsc2XStr[] = {"Osc2FM+"};
const uint8_t NoiseStr[] = {"Noise"};
const uint8_t VidPidStr[] = {"USB Vid/Pid=(hex) "};
const uint8_t MidiStr[] = {" MIDI:CH="};
const uint8_t ChromaStr[] = {"(ChromaticInstr) / Ch="};
const uint8_t DrumsStr[] = {"(Drums)"};

uint32_t n, type, parameter;
uint32_t frequency, bandwidth;

    gsAdsrOffset = gSoundSet * N_ASTRUCT;
    sendB(CRR);
    sendString(gBootStr);
    sendB(' ');
    sendB('v');
    sendB(HARDWARE_VERSION + 0x30);
    sendB('.');


    sendB(REVNUM + 0x30);
    sendB(' ');
    sendB('-');
    sendB(' ');
    sendString(gcSoundSetStr);
    sendS32((int32_t)(gSoundSet));
    if (gsMonoPoly) sendString(MonoStr);
    else sendString(PolyStr);
    sendString(gcGlideStr);
    sendB('=');
    sendS32(gsGLIDE);
    sendB(CRR);
    sendString(DashStr);
    sendB(CRR);

    sendString(gcOsc1Str);
    sendString(gcWaveformStr);
    sendB('=');
    sendString(gpWaveStr[gsWAVEFORM_1]);
    sendAdsrParam(0);
    sendB(CRR);
    sendB(' ');
    sendB(' ');
    sendB(' ');
    sendB(' ');
    sendString(gcFreqModStr);
    sendB(':');
    sendB(' ');
    sendAdsrParam(3);
    sendB(CRR);
    sendString(gcOsc2Str);
    sendString(gcWaveformStr);
    sendB('=');
    sendString(gpWaveStr[gsWAVEFORM_2]);
    sendAdsrParam(1);
    sendB(CRR);

    sendString(gcTransposeStr);
    sendB('=');
    sendS32(gsTRANS - 36);
	sendB(' ');
    sendString(gcTrans2Str);
    sendB('=');
    sendS32(gsTRANS_2 - 36);
    sendB(' ');
    sendString(gcDetuneStr);
    sendB('=');
    sendS32(gsDETUNEPreset[gSoundSet] - 64);
    sendB(' ');
    sendString(gcDetune2Str);
    sendB('=');
    sendS32(gsDETUNE_2Preset[gSoundSet] - 64);

    sendB(CRR);

    sendString(gcOsc1Str);
    if (gsOsc1PWM)  sendString(gcPwmStr);
    else  sendString(gcFreqModStr);
    sendB(':');
    sendNS32(gsMOD_1);
    sendString(gcControlLfoStr);
    if (gsOsc1ModLfo2 == 0)  sendB('1');
    else  sendB('2');
    sendB(' ');
    sendB(' ');
    sendString(gcOsc2Str);
    if (gsOsc2PWM)  sendString(gcPwmStr);
    else  sendString(gcFreqModStr);
    sendB(':');
    sendNS32(gsMOD_2);
    sendString(gcControlLfoStr);
    if (gsOsc2ModLfo2 == 0)  sendB('1');
    else   sendB('2');
    sendB(CRR);

    sendString(Lfo1Str);
    sendLfoWave(gLfo1Wave & 0x000000FF);
    sendString(gcFreqStr);
    sendB('=');
    sendNS32(gLfo1Freq / 10);
    sendB('.');
    sendB((gLfo1Freq % 10) + 0x30);
    sendB('H');
    sendB('z');
    sendB(CRR);

    sendString(Lfo2Str);
    sendLfoWave(gLfo2Wave & 0x000000FF);
    sendString(gcFreqStr);
    sendB('=');
    sendNS32(gLfo2Freq / 10);
    sendB('.');
    sendB((gLfo2Freq % 10) + 0x30);
    sendB('H');
    sendB('z');
    sendB(CRR);

    sendString(Lfo3Str);
    sendLfoWave(gLfo3Wave & 0x000000FF);
    sendString(gcFreqStr);
    sendB('=');
    sendNS32(gLfo3Freq / 10);
    sendB('.');
    sendB((gLfo3Freq % 10) + 0x30);
    sendB('H');
    sendB('z');
    sendB(CRR);

    sendString(gcRoutingStr);
    if (gfFlexMode == 0) sendString(EasyStr);
	else sendString(FlexStr);
	sendRouteConfig();
	sendB(CRR);
	sendString(BassStr);
	if (gfBass >= 64)  sendB(' ');						//compensate '-'
	if ((gfBass > 54) && (gfBass < 74)) sendB(' ');		//compensate tens
	sendNS32(gfBass - 64);
	sendB(' ');
	sendAdsrParam(6);
	sendB(CRR);
	sendString(TrebleStr);
	if (gfTreble >= 64)  sendB(' ');					//compensate '-'
	if ((gfTreble > 54) && (gfTreble < 74)) sendB(' ');	//compensate tens
	sendNS32(gfTreble - 64);
    sendB(' ');
    sendAdsrParam(7);
	sendB(CRR);

    sendString(DashStr);
    sendB(CRR);

    for (n = 0;n < N_FILTER;n++)  {
        type = gfACTIV_IIR[n].TYPE;
        sendString(FilterNumStr);
        sendB((uint8_t)n + 0x30);
        if (n == 2)  {
            sendB(0x28);        // (
             sendString(NoiseStr);
            sendB(0x29);        // )
        }
        sendB(':');
        sendB(' ');
        sendB('o');
        if(gfRuntimeMask & (1<< n)) {
            sendB('n');
            sendB(' ');
            sendB(' ');
               if (type == 0)  {
                sendB('=');
                sendB('f');
                sendB('l');
                sendB('a');
                sendB('t');
                }
        }
        else  {
            sendB('f');
            sendB('f');
            sendB(' ');
        }
        if (type)  {
            sendB('T');
            sendB('y');
            sendB('p');
            sendB('e');
            sendB('=');
            sendB(' ');
            sendS32(type);
            sendB(' ');
            sendB('F');
            sendB('o');
            sendB('=');
            sendB(' ');
            frequency =(uint32_t)gfMidiFreqTable[gfACTIV_IIR[n].FREQ];
            sendS32(frequency);
            sendB(' ');
            if ((type == 3)||(type == 4)||(type == 8)||(type == 9))  {
                sendB('R');
                sendB('e');
                sendB('s');
                sendB('o');
                sendB('n');
                sendB('=');
                sendB(' ');
                parameter = (uint32_t)gfACTIV_IIR[n].QFACTOR;
                sendNS32(parameter / 100);
                sendB('.');
                parameter = parameter % 100;
                if (parameter < 10) sendB('0');
                sendNS32(parameter);
            }
            else if (type >= 5)  {
                sendB('B');
                sendB('W');
                sendB('i');
                sendB('d');
                sendB('t');
                sendB('h');
                sendB('=');
                sendB(' ');
                bandwidth = (uint32_t)gfMidiFreqTable[gfACTIV_IIR[n].QFACTOR];
                sendS32(bandwidth);
            }
            if (type)  {
			    sendB(' ');
                sendB('P');
                sendB('e');
                sendB('a');
                sendB('k');
                sendB('=');
            	sendB(' ');
                parameter =  (uint32_t)gfACTIV_IIR[n].PEAK;
                if (type >= 6)  {
                    if (parameter < 20)  {
                        parameter = 100 - parameter*5;			//range 0..100
                        sendB('-');
                    }
                    else  {
                        parameter -= 20;						//range 0..104
                        parameter *= 10;						//range 0..1000
                        if (parameter > 1000) parameter = 1000;
                    }
                    sendNS32(parameter / 100);
                    sendB('.');
                    parameter = parameter % 100;
                    if (parameter < 10) sendB('0');
                    sendNS32(parameter);
                }
                else  sendNS32(parameter - 20);
            }
        }
        sendB(CRR);
        if (n == 0)  {
			sendString(FreqModLfoStr);
			sendB(gfF0modLfo2 + 0x31);
			sendB('=');
            sendNS32(gfF0mod);
			sendB(',');
            sendAdsrParam(9);
            sendB(CRR);
            sendString(PeakModStr);
			sendB(' ');
            sendAdsrParam(10);
            sendB(CRR);
			sendB(' ');
			sendB(' ');
			sendB(' ');
			sendB(' ');
			sendB(' ');
            sendString(gcKeytrackStr);
            sendB('=');
			sendB(' ');
            sendS32((uint32_t)(gsKeytrack0Preset[gSoundSet]));
            sendB('%');
			sendB(CRR);
			sendB(' ');
			sendB(' ');
			sendB(' ');
			sendB(' ');
            sendB(' ');
            sendString(gcGainStr);
            sendB('=');
            sendB(' ');
            sendS32(gfGain);
            sendAdsrParam(8);
            sendB(CRR);
        }
        if (n== 2)  {
            sendString(NoiseStr);
            sendB(' ');
            sendString(gcEnvelopeStr);
            sendB(' ');
            sendAdsrParam(2);
            sendB(CRR);
        }
    }
    sendString(DashStr);
    sendB(CRR);

    sendString(FreeAdsrStr);
    sendB( ' ');
	sendString(gpAdsrTypeStr[N_ABASIC + gsFreeAdsr]);
    sendB(' ');
    sendAdsrParam(N_ABASIC + gsFreeAdsr);
    sendB(CRR);

    sendString(DelayStr);
    sendB(' ');
    sendNS32(gDelayTime >> 8);
    sendB(((gDelayTime & 0x000000FF) / 10) + 0x30);
    sendB('.');
    sendB(((gDelayTime & 0x000000FF) % 10) + 0x30);
    sendB('m');
    sendB('s');
    sendB(' ');
    sendB(' ');
    sendString(gcChorusWidthStr);
    sendB('=');
    sendB(' ');
    sendS32(gChorusWidth);
    sendString(gcControlLfoStr);
    sendB(gChorusLfo3 + 0x32);
    sendB(CRR);

    sendString(gcDelayStr);
    sendString(gcLevelStr);
    sendB('=');
    sendB(' ');
    sendNS32(gDelayLevelFactor);
    sendB('%');
    sendB(' ');
    sendB(' ');
    sendString(gcFeedbackStr);
    sendString(gcLevelStr);
    sendB('=');
    sendB(' ');
    sendNS32(gFeedbackFactor);
    sendB('%');
    sendB(CRR);

    sendString(gcDrumsVolStr);
    sendB(':');
    sendB(CRR);
    sendB('A');
    sendB('l');
    sendB('l');
    sendB(':');
    sendNS32(gdDrumsVol - 40);
    sendB(' ');
    sendString(gpDrumTypeStr[1]);
    sendB(':');
    sendNS32(gdKickVol - 40);
    sendB(' ');
    sendString(gpDrumTypeStr[2]);
    sendB(':');
    sendNS32(gdHiTomVol - 40);
    sendB(' ');
    sendString(gpDrumTypeStr[3]);
    sendB(':');
    sendNS32(gdSnareVol - 40);
	sendB(' ');
    sendString(gpDrumTypeStr[4]);
    sendB(':');
    sendNS32(gdRimVol - 40);
    sendB(' ');
    sendString(gpDrumTypeStr[5]);
    sendB(':');
    sendNS32(gdClapVol - 40);
    sendB(' ');
    sendString(gpDrumTypeStr[6]);
    sendB(':');
    sendNS32(gdHihatVol - 40);
    sendB(' ');
    sendString(gpDrumTypeStr[7]);
    sendB(':');
    sendNS32(gdCymbalVol - 40);
    sendB(' ');
    sendString(gpDrumTypeStr[8]);
    sendB(':');
    sendNS32(gdBellVol - 40);
    sendB(CRR);

    sendString(gcOutputStr);
    sendString(gcVolumeStr);
    sendB('=');
    sendLS32(gVolume - 46);
    sendB('d');
    sendB('B');
    sendB(' ');
    sendB(' ');
    sendString(gcVibratoStr);
    sendB('=');
    sendB(' ');
    sendS32(gVibrato);
    sendB('%');
    sendString(gcControlLfoStr);
    if (gVibratoLfo1) sendB('1');
    else sendB('3');
    sendB(CRR);
    sendString(VidPidStr);
    sendX16(gVid);
    sendX16(gPid);
    sendB(' ');
    sendString(MidiStr);
    sendNS32(gChannel + 1);
    sendString(ChromaStr);
    sendNS32(gDrumsChannel + 1);
    sendString(DrumsStr);
}


void sendRouteConfig(void)  // **********************************************
// **************************************************************************
{
const uint8_t Osc1Str[] = {" Osc1->F"};
const uint8_t Osc1FMStr[] = {" Osc1(FM by O2)->F"};
const uint8_t Osc2Str[] = {" Osc2->F"};
const uint8_t NStr[] = {" N->F2"};

    switch (gaRoute0N)  {
    case 0:
        sendString(Osc1Str);
        sendB('0');
        sendString(Osc2Str);
        sendB('0');
        break;
    case 1:
        sendString(Osc1Str);
        sendB('0');
        sendString(Osc2Str);
        sendB('0');
        sendString(NStr);
        break;
    case 2:
        sendString(Osc1Str);
        sendB('0');
        sendString(Osc2Str);
        sendB('1');
        break;
    case 3:
        sendString(Osc1Str);
        sendB('0');
        sendString(Osc2Str);
        sendB('1');
        sendString(NStr);
        break;
    case 4:
		sendString(Osc1Str);
        sendB('1');
        sendString(Osc2Str);
        sendB('0');
        break;
    case 5:
        sendString(Osc1Str);
        sendB('1');
        sendString(Osc2Str);
        sendB('0');
        sendString(NStr);
        break;
    case 6:
		sendString(Osc1Str);
        sendB('1');
        sendString(Osc2Str);
        sendB('1');
        break;
    case 7:
        sendString(Osc1Str);
        sendB('1');
        sendString(Osc2Str);
        sendB('1');
        sendString(NStr);
        break;
   case 0x10:
        sendString(Osc1FMStr);
        sendB('0');
        break;
   case 0x11:
        sendString(Osc1FMStr);
        sendB('0');
        sendString(NStr);
        break;
    case 0x12:
        sendString(Osc1FMStr);
        sendB('1');
        break;
   case 0x13:
        sendString(Osc1FMStr);
        sendB('1');
        sendString(NStr);
        break;
    default:
        break;
    }
}


void sendAdsrParam(uint32_t num)  // ****************************************
// **************************************************************************
{
const uint8_t AdsrStr[] = {" ADSR_Param="};
const uint8_t HoldStr[] = {"(Hold)"};
const uint8_t OneShootStr[] = {"(1Shoot)"};
const uint8_t EnVolStr[] = {"Env.Vol"};

            sendString(AdsrStr);
            sendAdsrTime(gsAttack[num]);
            sendB('|');
            sendAdsrTime(gsDecay[num]);
            sendB('|');
            sendNS32(gsSustain[num] & 0x7F);
            if (gsHoldSust[num]) sendString(HoldStr);
            else  sendString(OneShootStr);
            sendB('|');
            sendAdsrTime(gsRelease[num]);
            sendB(' ');
            if ((num <=2) || (num == 8)) {
				sendString(EnVolStr);
            }
			else  if ((num == 3) || (num == 4) || (num == 9) ||
			 (num== N_ABASIC)||(num==(N_ABASIC+1))||(num==(N_ABASIC+3))){
				sendString(gcFreqShiftStr);				//filter mid freq shift
            }
            else  {
                sendString(gcLevelShiftStr);
            }
			sendB('=');
			if ((num <=2) || (num == 8))  sendNS32(gsFixVol[num]);
			else  sendNS32(gsFixVol[num] - 64);

            sendString(gcVeloStr);
            sendB('=');
            sendNS32(gsVelo[num] - 28);
}


void sendAdsrTime(uint32_t time)  // ***************************************
// **************************************************************************
{
uint32_t remainder;

    time = time / SECDIV;
    sendB(time/100 + 0x30);
    sendB('.');
    remainder = time%100;
    time = remainder%10;
    sendB(remainder/10 + 0x30);
    sendB(time + 0x30);
}


void sendLfoWave(uint32_t wave)  // *****************************************
// **************************************************************************
{
const uint8_t SineStr[] = {" Sine "};
const uint8_t TriStr[] = {"% Tri/Saw"};
const uint8_t SquareStr[] = {"% Square "};

    sendString(gcWaveformStr);
    sendB('=');
    if (wave == 0)  sendString(SineStr);
    else if (wave < 0x80)  {
        sendNS32(wave);
        sendString(TriStr);
    }
    else  {
        sendNS32(wave & 0x0000007F);
        sendString(SquareStr);
    }
    sendB(' ');
}

// #########################################################################

// Application level USB interface
//********************************
uint32_t scanSysEx(uint32_t mode)  // **************************************
//  ************************************************************************
{
uint32_t recvword;

        recvword = pullTimedByte();
        if (recvword !=0x0000807D) {//experimental/hobbyist MIDI Manufacturer ID
            gTIMEOUT = SYSFREQ * 2; //2sec
            recvword = 0;
			do { 								//delete wrong sysEx
				if (gTIMEOUT == 0)  return 0xFFFFFFFF;
				if (guConfigured)  recvword = pullUsbRxBuf();
												//USB has priority over USART
				if (recvword == 0) recvword = pullUsart1RxBuf();
            }  while (recvword != 0x000080F7);
            return 0;
        }
        recvword = pullTimedByte();
        if (recvword == 0x00008020) {				//SoundUpload ID 0x20
            recvword = pullTimedByte();
            if (recvword > (0x00008000 | N_SOUNDS)) { //invalid Sound Set number
                gTIMEOUT = 4000;    //4sec
                recvword = 0;
				do { 							//delete wrong sysEx
					if (gTIMEOUT == 0)  return 0xFFFFFFFF;
					if (guConfigured)  recvword = pullUsbRxBuf();
												//USB has priority over USART
					if (recvword == 0) recvword = pullUsart1RxBuf();
				}  while (recvword != 0x000080F7);
                return 0;
            }
            if (mode)  uploadSoundSet(recvword & 0x0000007F);
            return (recvword & 0x0000007F) + 1;
        }
        else if (recvword == 0x00008040) {			//Wave Upload ID 0x40
            recvword = pullTimedByte();
            if (recvword > (0x00008000 | N_USERWAVES)) {  //invalid Wave number
                gTIMEOUT = 4000;    //4sec
                recvword = 0;
				do { 							//delete wrong sysEx
					if (gTIMEOUT == 0)  return 0xFFFFFFFF;
					if (guConfigured)  recvword = pullUsbRxBuf();
												//USB has priority over USART
					if (recvword == 0) recvword = pullUsart1RxBuf();
				}  while (recvword != 0x000080F7);
                return 0;
            }
            if (mode) uploadUserWave(recvword & 0x00000007);
            return (recvword & 0x00000007) +1;
        }
        else  {
           gTIMEOUT = 4000;    //ca.3sec
                recvword = 0;
				do {						 	//delete wrong sysEx
					if (gTIMEOUT == 0)  return 0xFFFFFFFF;
					if (guConfigured)  recvword = pullUsbRxBuf();
												//USB has priority over USART
					if (recvword == 0) recvword = pullUsart1RxBuf();
				}  while (recvword != 0x000080F7);
            return 0;
        }
}


void  USB_Midi_Scanner(uint32_t byte)  // ***********************************
// accepts only MIDI channel as stored in "gChannel", ignores any else
// though same code, extra file to merge input from both MIDI IN correctly
// except SysEx, isn't merged correctly
// **************************************************************************
{
static uint32_t sData1, sMsgtype, sProgress;
uint32_t recvword;

   if (byte == 0xF0)  {
		scanSysEx(1);
		return;
    }

    if (byte > 0xF0)  {						//MIDI Real Time msgs not supported
        sProgress = 0;						//reset parser
        sMsgtype = 0;
        sData1 = 0;
        return;
    }

    if (byte >= 0x80)  {                    //new status byte
        if ((byte & 0x0F) == gChannel)  {   //keyboard or control msgs
            sMsgtype = byte & 0xF0;          //don't check MIDI Channel anymore
            sProgress = 1;
            return;
        }
        else if (((byte&0xF0)==0x90) && ((byte&0x0F)==gDrumsChannel)) {
                        //Note ON at DrumsChannel: trigger drums, default Ch.10
            sMsgtype = 0x99;
            sProgress = 1;
            return;
        }
       else {
            sProgress = 0;                   //reset parser
            sMsgtype = 0;
            sData1 = 0;
            return;
       }
    }
    else  {                                 //new data byte
        if (sProgress == 0)  {               //byte out of context, reset parser
            sMsgtype = 0;
            sData1 = 0;
            return;
        }
        if (sProgress == 1)  {
            sData1 = byte;

            if ((sMsgtype == 0xC0)  || (sMsgtype == 0xD0)) {
                MIDI_Executor(sMsgtype, sData1, 0) ;
                return;					//keep sProgress==1 for Running State
            }
            sProgress = 2;
            return;
        }
        if (sProgress == 2)  {
            MIDI_Executor(sMsgtype, sData1, (uint32_t)byte);
            sProgress = 1;	//reset sProgress and keep status for Running State
        }
    }
}


void  USART_Midi_Scanner(uint32_t byte)  // *********************************
// accepts only MIDI channel as stored in "gChannel", ignores any else
// **************************************************************************
{
static uint32_t sData1, sMsgtype, sProgress;
uint32_t recvword;

   if (byte == 0xF0)  {
//#ifdef CUT_OFF
		scanSysEx(1);
		return;
   }
/*
#else
	gTIMEOUT = SYSFREQ * 2;    			//2sec
	do  {
		if (gTIMEOUT == 0) break;
		recvword = pullUsart1RxBuf();		//delete any SysEx
	}  while (recvword != 0x000080F7);
#endif
*/
    if (byte > 0xF0)  {						//MIDI Real Time msgs not supported
        sProgress = 0;						//reset parser
        sMsgtype = 0;
        sData1 = 0;
        return;
    }

    if (byte >= 0x80)  {                    //new status byte
        if ((byte & 0x0F) == gChannel)  {   //keyboard or control msgs
            sMsgtype = byte & 0xF0;          //don't check MIDI Channel anymore
            sProgress = 1;
            return;
        }
        else if (((byte&0xF0)==0x90) && ((byte&0x0F)==gDrumsChannel)) {
                        //Note ON at DrumsChannel: trigger drums, default Ch.10
            sMsgtype = 0x99;
            sProgress = 1;
            return;
        }
       else {
            sProgress = 0;                   //reset parser
            sMsgtype = 0;
            sData1 = 0;
            return;
       }
    }
    else  {                                 //new data byte
        if (sProgress == 0)  {               //byte out of context, reset parser
            sMsgtype = 0;
            sData1 = 0;
            return;
        }
        if (sProgress == 1)  {
            sData1 = byte;

            if ((sMsgtype == 0xC0)  || (sMsgtype == 0xD0)) {
                MIDI_Executor(sMsgtype, sData1, 0) ;
                return;                     //keep sProgress==1 for Running State
            }
            sProgress = 2;
            return;
        }
        if (sProgress == 2)  {
            MIDI_Executor(sMsgtype, sData1, (uint32_t)byte);
            sProgress = 1;   //reset sProgress and keep status for Running State
        }
    }
}


void MIDI_Executor(uint32_t msgtype, uint32_t data1, uint32_t data2)  // ****
// **************************************************************************
// SysEx Message: upload SoundSet or UserWave

// receive NOTE ON and NOTE OFF messages
// but ignore note value <18 and >120

//received CONTROL CHANGE messages (Rev.9:only accepted parameter setup mode):
//--------------------------------------------------------
// 0xBn  0x00   xxxxxxx       select and activate SoundSet; 7bit
// 0xBn  0x01   xxxxxxx       Bass  ;7Bit
// 0xBn  0x02   xxxxxxx       Treble  ;7Bit
// 0xBn  0x03   0 or 1		  0=EASY, 1=FLEX filter set
// 0xBn  0x04   xxxxxxx       FilterRouting Filters 0 and Noise
// 0xBn  0x05   xxxxxxx       gNTrigger=number of keys trigger special ADSR(1-5)
// 0xBn  0x06   xxxxxxx       Glide(Portamento);7Bit
// 0xBn  0x07   xxxxxxx       Global Out Volume ;7Bit
// 0xBn  0x08   xxxxxxx       Vibrato modulated by Lfo3 ;7Bit
// 0xBn  0x09   xxxxxxx       Vibrato modulated by Lfo1 ;7Bit
// 0xBn  0x0A   xxxxxxx       Gain  ;7Bit

// 0xBn  0x0B   xxxxxxx       OSC1 Waveform  ;6Bit
// 0xBn  0x0C   xxxxxxx       OSC1+2 Transpose  ;7Bit
// 0xBn  0x0D   xxxxxxx       OSC1+2 Detune ;7Bit
// 0xBn  0x0E   xxxxxxx       OSC2 Waveform  ;6Bit
// 0xBn  0x0F   xxxxxxx       OSC2 add Transpose ,7Bit
// 0xBn  0x10   xxxxxxx       OSC2 add Detune ;7Bit

// 0xBn  0x11   xxxxxxx       FreeSquare1 hiRange (0..100)
//							  internally divided by 2, i.e. range 0..50.0%
// 0xBn  0x12   xxxxxxx       FreeSquare1 Slope (0..25)
//							  internally limited to hiRange/2
// 0xBn  0x13   xxxxxxx       FreeSquare2 hiRange (0..100)
//							  internally divided by 2, i.e. range 0..50.0%
// 0xBn  0x14   xxxxxxx       FreeSquare2 Slope(0..125,internally mult. by 2)
//							  internally limited to hiRange/2
// 0xBn  0x15   0 or 1        1=Osc2 phase synchronized with Osc1,else both free

// 0xBn  0x16   xxxxxxx       OSC1 Freq.Modulation ;7Bit
// 0xBn  0x17   xxxxxxx       OSC1 PWM(works only on Square Wave,else FM select)
// 0xBn  0x18   xxxxxxx       OSC2 Freq.Modulation ;7Bit
// 0xBn  0x19   xxxxxxx       OSC2 PWM(works only on Square Wave,else FM select)

// 0xBn  0x1A   xxxxxxx       LFO1 Waveform ;7 Bit    (bit6 = Lfo for MOD)
// 0xBn  0x1B   xxxxxxx       LFO1 Freq ;7Bit
// 0xBn  0x1C   xxxxxxx       LFO2 Waveform ;7 Bit
// 0xBn  0x1D   xxxxxxx       LFO2 Freq ;7 Bit
// 0xBn  0x1E   xxxxxxx       LFO3 Waveform ;7 Bit    (bit6 = Lfo for MOD)
// 0xBn  0x1F   xxxxxxx       LFO3 Freq ;7Bit

// 0xBn  0x20   xxxxxxx       OSC1 Attack  ;7Bit
// 0xBn  0x21   xxxxxxx       OSC1 Decay  ;7Bit
// 0xBn  0x22   xxxxxxx       OSC1 Sustain Hold type  ;7Bit
// 0xBn  0x23   xxxxxxx       OSC1 Sustain 1 Shoot type  ;7Bit
// 0xBn  0x24   xxxxxxx       OSC1 Release ;7Bit
// 0xBn  0x25   xxxxxxx       OSC1 Volume;7Bit
// 0xBn  0x26   xxxxxxx       OSC1 Velo ;7Bit

// 0xBn  0x28   xxxxxxx       OSC2 Attack;7Bit
// 0xBn  0x29   xxxxxxx       OSC2 Decay  ;7Bit
// 0xBn  0x2A   xxxxxxx       OSC2 Sustain Hold type  ;7Bit
// 0xBn  0x2B   xxxxxxx       OSC2 Sustain 1 Shoot type  ;7Bit
// 0xBn  0x2C   xxxxxxx       OSC2 Release ;7Bit
// 0xBn  0x2D   xxxxxxx       OSC2 Volume ;7Bit
// 0xBn  0x2E   xxxxxxx       OSC2 Velo ;7Bit

// 0xBn  0x30   xxxxxxx       Noise Envelope Attack;7Bit
// 0xBn  0x31   xxxxxxx       Noise Envelope Decay  ;7Bit
// 0xBn  0x32   xxxxxxx       Noise Envelope Sustain Hold type ;7Bit
// 0xBn  0x33   xxxxxxx       Noise Envelope Sustain 1 Shoot type;7Bit
// 0xBn  0x34   xxxxxxx       Noise Envelope Release ;7Bit
// 0xBn  0x35   xxxxxxx       Noise Envelope Volume ;7Bit
// 0xBn  0x36   xxxxxxx       Noise Envelope Velo ;7Bit

// 0xBn  0x38   xxxxxxx       OSC1&2 Frequency Modulation Attack;7Bit
// 0xBn  0x39   xxxxxxx       OSC1&2 Frequency Modulation Decay;7Bit
// 0xBn  0x3A   xxxxxxx       OSC1&2 Frequency Modulation Sustain Hold type;7Bit
// 0xBn  0x3B   xxxxxxx       OSC1&2 Frequency Modulation Sustain 1 Shoot type;7Bit
// 0xBn  0x3C   xxxxxxx       OSC1&2 Frequency Modulation Release ;7Bit
// 0xBn  0x3D   xxxxxxx       OSC1&2 Frequency Modulation Fix Shift ;7Bit
// 0xBn  0x3E   xxxxxxx       OSC1&2 Frequency Modulation Velo;7Bit

// 0xBn  0x40   xxxxxxx       Lfo1 Balance Attack;7Bit
// 0xBn  0x41   xxxxxxx       Lfo1 Balance Decay;7Bit
// 0xBn  0x42   xxxxxxx       Lfo1 Balance  Sustain Hold type;7Bit
// 0xBn  0x43   xxxxxxx       Lfo1 Balance  Sustain 1 Shoot type;7Bit
// 0xBn  0x44   xxxxxxx       Lfo1 Balance Release ;7Bit
// 0xBn  0x45   xxxxxxx       Lfo1 Balance  Fix Shift ;7Bit
// 0xBn  0x46   xxxxxxx       Lfo1 Balance  Velo;7Bit

// 0xBn  0x48   xxxxxxx       Filter0 Mid Frequency Modulation Attack ;7Bit
// 0xBn  0x49   xxxxxxx       Filter0 Mid Frequency Modulation Decay;7Bit
// 0xBn  0x4A   xxxxxxx       Filter0 Mid Frequency Modulation Sustain Hold type;7Bit
// 0xBn  0x4B   xxxxxxx       Filter0 Mid Freq Modulation Sustain 1 Shoot type;7Bit
// 0xBn  0x4C   xxxxxxx       Filter0 Mid Frequency Modulation Release;7Bit
// 0xBn  0x4D   xxxxxxx       Filter0 Mid Frequency Modulation Fix Shift;7Bit
// 0xBn  0x4E   xxxxxxx       Filter0 Mid Frequency Modulation Velo;7Bit
// 0xBn  0x4F  xxxxxxx        Filter0 Keytrack(%)

// 0xBn  0x50   xxxxxxx       Selectable ADSR Attack;7Bit
// 0xBn  0x51   xxxxxxx       Selectable ADSR Decay  ;7Bit
// 0xBn  0x52   xxxxxxx       Selectable ADSR Sustain Hold type  ;7Bit
// 0xBn  0x53   xxxxxxx       Selectable ADSR Sustain 1 Shoot type  ;7Bit
// 0xBn  0x54   xxxxxxx       Selectable ADSR Peak Release ;7Bit
// 0xBn  0x55   xxxxxxx       Selectable ADSR Peak Fix Shift ;7Bit
// 0xBn  0x56   xxxxxxx       Selectable ADSR Peak Velo ;7Bit
// 0xBn  0x57   xxxxxxx       Select ADSR target type (default:9 = Filter0Peak)

// 0xBn  0x58   xxxxxxx       Free ADSR Attack;7Bit
// 0xBn  0x59   xxxxxxx       Free ADSR Decay  ;7Bit
// 0xBn  0x5A   xxxxxxx       Free ADSR Sustain Hold type  ;7Bit
// 0xBn  0x5B   xxxxxxx       Free ADSR Sustain 1 Shoot type  ;7Bit
// 0xBn  0x5C   xxxxxxx       Free ADSR Release ;7Bit
// 0xBn  0x5D   xxxxxxx       Free ADSR Fix Shift ;7Bit
// 0xBn  0x5E   xxxxxxx       Free ADSR Velo ;7Bit
// 0xBn  0x5F   xxxxxxx       select target of Free ADSR ;7Bit

// 0xBn  0x60   xxxxxxx       DelayTime 10ms steps, 0...30
// 0xBn  0x61   xxxxxxx       DelayTime 0.1ms steps, 0...99
// 0xBn  0x62   xxxxxxx       Delay Level 7Bit (in %, delay phase = input phase)
// 0xBn  0x63   xxxxxxx       Delay Level 7Bit (in %, delay phase inverted)
// 0xBn  0x64   xxxxxxx       Delay Feedback 7Bit(in %,feedback phase=input phase)
// 0xBn  0x65   xxxxxxx       Delay Feedback 7Bit (in %,feedback phase inverted)
// 0xBn  0x66   xxxxxxx       Chorus Width, modulated by LFO2 ;0...99
// 0xBn  0x67   xxxxxxx       Chorus Width, modulated by  LFO3 ;0...99

//adjust actually selected filter
// 0xBn  0x68   xxxxxxx       select actually modified filter number(0,1,2,3,4)
// 0xBn  0x69   xxxxxxx       TYPE  ;7Bit MIDI note value
// 0xBn  0x6A   xxxxxxx       FREQ  ;7Bit MIDI note value
// 0xBn  0x6B   xxxxxxx       QFACTOR ;7Bit MIDI note value or Zeta(Resonance)
// 0xBn  0x6C   xxxxxxx       PEAK ;7Bit

// 0xBn  0x6D   xxxxxxx       Filter0 Modulation with/selects Lfo1 ;7Bit
// 0xBn  0x6E   xxxxxxx       Filter0 Modulation with/selects Lfo2 ;7Bit

// 0xBn  0x6F   xxxxxxx       Drums global volume; 7Bit, values 0..52
// 0xBn  0x70   xxxxxxx       Kick drum volume; 7Bit, values 0..52
// 0xBn  0x71   xxxxxxx       HiTom volume; 7Bit, values 0..52
// 0xBn  0x72   xxxxxxx       Snare volume; 7Bit, values 0..52
// 0xBn  0x73   xxxxxxx       Rim volume; 7Bit, values 0..52
// 0xBn  0x74   xxxxxxx       Clap volume; 7Bit, values 0..52
// 0xBn  0x75   xxxxxxx       Hihat volume; 7Bit, values 0..52
// 0xBn  0x76   xxxxxxx       Cymbal volume; 7Bit, values 0..52
// 0xBn  0x77   xxxxxxx       Bell volume; 7Bit, values 0..52

// 0xBn  0x7B   0000000       all Notes Off

// 0xBn  0x7E   0000000      activate monophonic mode, data2 irrelevant
// 0xBn  0x7F   0000000      activate polyphonic mode, data2 irrelevant

//SysEx:F0 7D 20 Sound#  	upload SoundSet#
//                          inclusive all global, Osc, ADSR and Filter settings
//Sysex:F0 7D 40 UserWave#  upload User Waveform#


{
uint32_t fNum, temp, sound;
uint32_t midiFreq, midiQFactor, Peak;
int8_t  flutter;

	sound = gSoundSet;

	fNum = (sound * N_FILTER) + gfFilterNum;
    gsAdsrOffset = sound * 10;

    if (msgtype == 0x80)  {
        handleNoteOff(data1);
        return;
    }

    if (msgtype == 0x90) {                  //MIDI Note ON
        if (data2 == 0) {
            handleNoteOff(data1);
            return;
        }
        else {
            distributeNoteOn(data1,data2);
            return;
        }
    }

//if Drums, msgtype == 0x99 (was prefiltered)
//chromatic sound has priority when MIDI channels are equal !!
    if  (msgtype == 0x99) {  //trigger drums according to General MIDI Drum Kit
        if (data2 == 0) return;
        if (data2 < 10) data2 = 10;
        switch (data1)  {
            case 0x23:
            case 0x24:
            case 0x29:          //Low Floor Tom
            case 0x2B:          //High Floor Tom
            case 0x40:          //Low Conga
                triggerKick((int32_t)data2);
                return;
            case 0x2D:
            case 0x2F:
            case 0x30:
            case 0x32:
            case 0x3E:          //Mute High Conga
            case 0x3F:          //Open High Conga
                triggerHiTom((int32_t)data2);
                return;
            case 0x26:
            case 0x28:
                triggerSnare((int32_t)data2);
                return;
            case 0x25:
            case 0x4D:          //Low Wood Block
                triggerRim((int32_t)data2);
                return;
            case 0x27:
            case 0x4C:          //High Wood Block
                triggerClap((int32_t)data2);
                return;
            case 0x2A:
            case 0x2C:
            case 0x2E:
                triggerHihat((int32_t)data2);
                return;
            case 0x31:
            case 0x33:
            case 0x34:
            case 0x37:
            case 0x39:
                triggerCymbal((int32_t)data2);
                return;
            case 0x35:           //Ride Bell
            case 0x3B:          //Cowbell
            case 0x49:          //Guiro
            case 0x4A:
            case 0x50:          //Triangle
            case 0x51:
                triggerBell((int32_t)data2);
                return;
            default:
               return;
        }
    }

    if (msgtype != 0xB0)  return;	//only Control Change used for parameter set

    switch(data1)  {
		case 0:
			if (data2 >= N_SOUNDS)  return;
			gSoundSet = (uint32_t)data2;
            activateSoundSet();
			break;
        case 1:
            gfBass = (uint32_t)data2;
            packBasTreb();
            break;
        case 2:
            gfTreble = (uint32_t)data2;
            packBasTreb();
            break;
        case 3:
        	if (data2 > 1)  data2 = 1;				//0=EASY,1=FLEX filter set
			gfFlexMode = (uint32_t)data2;
			gfStageMask[sound] &= 0x000000BF;
			gfStageMask[sound] |= gfFlexMode << 6;	//bit6 stores Filter Set
			updateRoute();							//init filters immediately
            break;
        case 4:
//0:Osc1->F0 Osc2->F0, 1:Osc1->F0 Osc2->F0 + Noise
//2:Osc1->F0 Osc2->F1, 3:Osc1->F0 Osc2->F1 + Noise
//4:Osc1->F1 Osc2->F0, 5:Osc1->F1 Osc2->F0 + Noise
//6:Osc1->F1 Osc2->F1, 7:Osc1->F1 Osc2->F1 + Noise
//0x10:Osc1(FM)->F0, 0x11:Osc1(FM)->F0 + Noise
//0x12:Osc1(FM)->F1, 0x13:Osc1(FM)->F1 + Noise
			if ((data2 > 7) &&((data2 < 0x10)||(data2 > 0x13))) return;
            gaRoute0N = (uint32_t)data2;
            if (gfFlexMode == 0)  {					//EASY
            	gaRoutePreset[gSoundSet] &= 0xFFFFFF00;
				gaRoutePreset[gSoundSet] |= gaRoute0N;
            }
            else   {								//FLEX
            	gaRoutePreset[gSoundSet] &= 0xFF00FFFF;
				gaRoutePreset[gSoundSet] |= gaRoute0N << 16;
            }
			updateRoute();							//init filters immediately
            break;
		case 5:
			if (data2 == 0) data2 = 1;
            if (data2 > 5) data2 = 5;
            gNTrigger[sound] = data2;
            break;
        case 6:                                      //Portamento
            gsGLIDE = (uint32_t)data2;
            gsGLIDEPreset[sound] = (uint32_t)data2;
        case 7:
//scaled 0..127 for compatibility with external MIDI controllers:
            if (data2 < 17) data2 = 17;             //simulate bad potentiometer
            data2 -= 17;                        	//range 0... 110
            if (data2 > 104)  data2 = 104;
            flutter = (int8_t)((uint8_t)(gVolume << 1) - data2);
            if (flutter < 0 ) flutter = -flutter;
//avoid problems with instable/fluttering volume output from keyboard:
            if (flutter > 1)  {
				data2 = data2 >> 1;             	//range 0..52, 46 = 0dB
				gVolume = (uint32_t)data2;
				gVolumePreset[sound] = gVolume;
				gVolumeFactor = gDbTable[gVolume] >> 3;	// max.511 allowed
				getAllVolFactors();					//necessary ???
            }
            break;
        case 8:
//set Vibrato depth & switch to controlled by LFO3
        case 9:
//set Vibrato depth & switch to controlled by LFO1
            if (data2 > 99) data2 = 99;
            gVibrato = data2;
            gVibratoLfo1= data1 & 0x00000001;      //i.e. set in case 9
            gVibratoPreset[sound] = data2 | (gVibratoLfo1<< 7);
            break;
        case 0x0A:
            if (data2 == 0)  data2 = 1;
            if (data2 > 50)  data2 = 50;
            gfGain = (uint32_t)data2;
            gfGainPreset[sound] = gfGain;
            break;

//Oscillators
       case 0x0B:                                          //Osc1 Wave form
            if (data2 > 33) return;
            gsWAVEFORM_1 = (uint32_t)data2;
            gsWAVEFORM_1Preset[sound] = (uint32_t)data2;
            initWaveOsc1(gsWAVEFORM_1);
            break;
        case 0x0C:
            if (data2 > 72) data2 = 72;                 //Osc1+2 equal Transpose
            gsTRANS = (int32_t)data2;
            gsTRANSPreset[sound] = (int32_t)data2;
            break;
       case 0x0D:                                       //Osc1+2 global DETUNE
            gsDETUNEPreset[sound] = (int32_t)data2;
            gsDETUNE = (int32_t)(data2 - 64);
            break;
        case 0x0E:                                      //Osc2 Wave form
            if (data2 > 29) return;
            gsWAVEFORM_2 = (uint32_t)data2;
            gsWAVEFORM_2Preset[sound] = (uint32_t)data2;
            initWaveOsc2(gsWAVEFORM_2);
            break;
        case 0x0F:                                      //Osc2 add. Transpose
            if (data2 > 72) data2 = 72;
            gsTRANS_2 = (int32_t)data2;
            gsTRANS_2Preset[sound] = (int32_t)data2;
            break;
        case 0x10:                                      //Osc2 add. DETUNE
            gsDETUNE_2Preset[sound] = (int32_t)data2;
            gsDETUNE_2 = (int32_t)(data2 - 64);
            break;

		case 0x11:  //FreeSquare1 hiRange (0..100,internally divided by 2)
					//i.e. internal range 0..50.0%
			if (data2 > 100)  data2 = 100;
			temp = data2;
			data2 = (data2 & 0x01) * 5;			//0 or 5
			gsFreeSquare1[sound] &= 0x0000FFFF;
			gsFreeSquare1[sound] |= ((temp / 2) << 24) | (data2 << 16);
			temp = (temp * 5) + data2 ;			//range 0..500
			gsRangeFS1 = temp + (temp / 41);	//range 0..512
			break;
		case 0x12:  //FreeSquare1 Slope (0..25) internally limited to hiRange/2
			temp = ((gsFreeSquare1[sound] >> 24) & 0x000000FF) / 2;
			if (data2 > temp)  data2 = temp;
			gsFreeSquare1[sound] &= 0xFFFFFF00;
			gsFreeSquare1[sound] |= temp;
			break;
		case 0x13:  //FreeSquare2 hiRange (0..100,internally divided by 2)
					//i.e. internal range 0..50.0%
			if (data2 > 100)  data2 = 100;
			temp = data2;
			data2 = (data2 & 0x01) * 5;			//0 or 5
			gsFreeSquare1[sound] &= 0x0000FFFF;
			gsFreeSquare1[sound] |= ((temp / 2) << 24) | (data2 << 16);
			temp = (temp * 5) + data2 ;			//range 0..500
			gsRangeFS1 = temp + (temp / 41);	//range 0..512
			break;
		case 0x14:  //FreeSquare2 Slope (0..25) internally limited to hiRange/2
			temp = ((gsFreeSquare2[sound] >> 24) & 0x000000FF) / 2;
			if (data2 > temp)  data2 = temp;
			gsFreeSquare2[sound] &= 0xFFFFFF00;
			gsFreeSquare2[sound] |= temp;
			break;
		case 0x15:
            if (data2 > 1) data2 = 1;
			gsO2Sync = (uint32_t)data2;
            gfStageMask[sound] &= 0x0000007F;
            gfStageMask[sound] |= gsO2Sync << 7;	//bit7 stores gsO2Sync
            break;

        case 0x16:                          //Osc1 Frequency modulation by Lfo1
            gsMOD_1 = (uint32_t)data2;
            if (gsMOD_1 > 99)  gsMOD_1 = 99;
            gsOsc1PWM = 0;
            gsMOD_1Preset[sound]= gsMOD_1;
            break;
        case 0x17:                  //Osc1 PWM by Lfo1
            gsMOD_1 = (uint32_t)data2;
            if (gsMOD_1 > 99)  gsMOD_1 = 99;
            if ((gsWAVEFORM_1 >= 6) && (gsWAVEFORM_1 <= 9)) {
                gsOsc1PWM = 1;
                gsMOD_1Preset[sound] = gsMOD_1 | 0x80;
            }
            else {
                gsOsc1PWM = 0;    //if not square wave,select Freq.Mod instead
                gsMOD_1Preset[sound] = gsMOD_1;
            }
            break;

        case 0x18:                  //Osc2 Frequency modulation by Lfo1
            gsMOD_2 = (uint32_t)data2;
            if (gsMOD_2 > 99)  gsMOD_2 = 99;
            gsOsc2PWM = 0;
            gsMOD_2Preset[sound] = gsMOD_2;
            break;
        case 0x19:                  //Osc2 PWM by Lfo1
            gsMOD_2 = (uint32_t)data2;
            if (gsMOD_2 > 99)  gsMOD_2 = 99;
            if ((gsWAVEFORM_2 >= 6) && (gsWAVEFORM_2 <= 9)) {
                gsOsc2PWM = 1;
                gsMOD_2Preset[sound] = gsMOD_2 | 0x80;
            }
            else {
                gsOsc2PWM = 0;    //if not square wave,select Freq.Mod instead
                gsMOD_2Preset[sound] = gsMOD_2;
            }
            break;

//LFOs
		case 0x1A:  //LFO1 Waveform
//structure of MIDI msg: data2=0:Sine
//						 else:	bit6=1:Triangle/Saw, bit6=0: Square
//                       		bits 1..5 = percent/2 (range 5...45)
            gLfoNum = 1;
            if (data2 == 0)  gLfo1Wave = 0;
            else {
				if (data2 < 5) data2 = 5;
				if (data2 >= 69) temp = (uint32_t)(2*data2 -10);
                                    //= more explicitly=(128 + (data2-69)*2);
				else temp = (uint32_t)(137 - 2*data2);
				if (temp > 208) temp = 208;
				if (temp < 47) temp = 47;
            }
            gLfo1WavePreset[gSoundSet] &= 0x00000100;
			gLfo1WavePreset[gSoundSet] |= temp;			//preserve slope
			gLfo1Wave = gLfo1WavePreset[gSoundSet];
            calculateLfoWave(1, gLfo1Wave & 0x000000FF);
            break;
        case 0x1B:  //LFO1 Freq.
            gLfoNum = 1;
            gLfo1Freq = (uint32_t)data2;
            gLfo1FreqPreset[sound] = gLfo1Freq;
            break;
        case 0x1C:  //LFO2 Waveform
            gLfoNum = 2;
			if (data2 == 0)  gLfo2Wave = 0;
            else {
				if (data2 < 5) data2 = 5;
				if (data2 >= 69) temp = (uint32_t)(2*data2 -10);
				else temp = (uint32_t)(137 - 2*data2);
				if (temp > 208) temp = 208;
				if (temp < 47) temp = 47;
            }
            gLfo2WavePreset[gSoundSet] &= 0x00000100;
			gLfo2WavePreset[gSoundSet] |= temp;			//preserve slope
			gLfo2Wave = gLfo2WavePreset[gSoundSet];
            calculateLfoWave(2, gLfo2Wave & 0x000000FF);
            break;
        case 0x1D:  //LFO2 Freq.
            gLfoNum = 2;
            gLfo2Freq = (uint32_t)data2;
            gLfo2FreqPreset[sound] = gLfo2Freq;
            break;
        case 0x1E:  //LFO3 Waveform
            gLfoNum = 3;
            if (data2 == 0)  gLfo3Wave = 0;
            else {
				if(data2 < 5) data2 = 5;
				if (data2 >= 69) temp = (uint32_t)(2*data2 -10);
				else temp = (uint32_t)(137 - 2*data2);
				if (temp > 208) temp = 208;
				if (temp < 47) temp = 47;
            }
            gLfo3WavePreset[gSoundSet] &= 0x00000100;
			gLfo3WavePreset[gSoundSet] |= temp;			//preserve slope
			gLfo3Wave = gLfo3WavePreset[gSoundSet];
            calculateLfoWave(3, gLfo3Wave & 0x000000FF);
            break;
        case 0x1F:  //LFO3 Freq.
            gLfoNum = 3;
            gLfo3Freq = (uint32_t)data2;
            gLfo3FreqPreset[sound] = gLfo3Freq;
            break;

//Osc1 envelope ADSR
        case 0x20:                                  //Osc1 Envelope ATTACK
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset,0,(uint32_t)data2);
            if (data2 <= 100) gsAttack[0] = (uint32_t)data2 * A_FACTOR;
            else gsAttack[0] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x21:                                  //Osc1 DECAY
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset,1,(uint32_t)data2);
            if (data2 <= 100) gsDecay[0] = (uint32_t)data2 * A_FACTOR;
            else gsDecay[0] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x22:                                  //1Shoot
        case 0x23:                                  //Osc1 SUSTAIN Hold
			if (data2 > 100)  data2 = 100;
			packADSR(gsAdsrOffset,2,(uint32_t)(data2 | ((data1 & 1) << 7)));
            gsHoldSust[0] = (uint32_t)data1 & 1;
            gsSustain[0] = (uint32_t)data2;
            break;
        case 0x24:                                  //Osc1 RELEASE
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset,3,(uint32_t)data2);
            if (data2 <= 100) gsRelease[0] = (uint32_t)data2 * A_FACTOR;
            else gsRelease[0] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
       case 0x25:                                   //Osc1 Volume
			packADSR(gsAdsrOffset,4,(uint32_t)data2);
            gsFixVol[0] = (uint32_t)data2;
            break;
       case 0x26:                                   //Osc1 Velo
			packADSR(gsAdsrOffset,5,(uint32_t)data2);
            gsVelo[0] = (uint32_t)data2;
            break;

//Osc2 envelope ADSR
        case 0x28:                                  //Osc2 Envelope ATTACK
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+1,0,(uint32_t)data2);
            if (data2 <= 100) gsAttack[1] = (uint32_t)data2 * A_FACTOR;
            else gsAttack[1] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x29:                                  //Osc2 DECAY
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+1,1,(uint32_t)data2);
            if (data2 <= 100) gsDecay[1] = (uint32_t)data2 * A_FACTOR;
            else gsDecay[1] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x2A:                                  //1Shoot
        case 0x2B:                                  //Osc2 SUSTAIN Hold
			if (data2 > 100)  data2 = 100;
			packADSR(gsAdsrOffset+1,2,(uint32_t)(data2 | ((data1 & 1) << 7)));
            gsHoldSust[1] = (uint32_t)data1 & 1;
            gsSustain[1] = (uint32_t)data2;
            break;
        case 0x2C:                                  //Osc2 RELEASE
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+1,3,(uint32_t)data2);
            if (data2 <= 100) gsRelease[1] = (uint32_t)data2 * A_FACTOR;
            else gsRelease[1] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
       case 0x2D:                                   //Osc2 Volume
			packADSR(gsAdsrOffset+1,4,(uint32_t)data2);
            gsFixVol[1] = (uint32_t)data2;
            break;
       case 0x2E:                                   //Osc2 Velo
			packADSR(gsAdsrOffset+1,5,(uint32_t)data2);
            gsVelo[1] = (uint32_t)data2;
            break;

//Noise envelope ADSR
        case 0x30:                                  //Noise Envelope ATTACK
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+2,0,(uint32_t)data2);
            if (data2 <= 100) gsAttack[2] = (uint32_t)data2 * A_FACTOR;
            else gsAttack[2] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x31:                                  //Noise DECAY
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+2,1,(uint32_t)data2);
            if (data2 <= 100) gsDecay[2] = (uint32_t)data2 * A_FACTOR;
            else gsDecay[2] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x32:                                  //Noise SUSTAIN Hold
        case 0x33:                                  //1Shoot
			if (data2 > 100)  data2 = 100;
            gsHoldSust[2] = (~(uint32_t)data1) & 1;
            gsSustain[2] = (uint32_t)data2;
			packADSR(gsAdsrOffset+2,2,(uint32_t)(data2 | ((data1 & 1) << 7)));
            break;
        case 0x34:                                  //Noise RELEASE
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+2,3,(uint32_t)data2);
            if (data2 <= 100) gsRelease[2] = (uint32_t)data2 * A_FACTOR;
            else gsRelease[2] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
       case 0x35:                                   //Noise ADSR Volume
			packADSR(gsAdsrOffset+2,4,(uint32_t)data2);
            gsFixVol[2] = (uint32_t)data2;
            break;
       case 0x36:                                   //Noise Velo
			packADSR(gsAdsrOffset+2,5,(uint32_t)data2);
            gsVelo[2] = (uint32_t)data2;
            break;

//Frequency Modulation Osc1+Osc2 commonly
        case 0x38:                                  //FMO12 ATTACK
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+3,0,(uint32_t)data2);
            if (data2 <= 100) gsAttack[3] = (uint32_t)data2 * A_FACTOR;
            else gsAttack[3] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x39:                                  //FMO12 DECAY
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+3,1,(uint32_t)data2);
            if (data2 <= 100) gsDecay[3] = (uint32_t)data2 * A_FACTOR;
            else gsDecay[3] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x3A:                                  //1Shoot
        case 0x3B:                                  //FMO12 SUSTAIN Hold
			if (data2 > 100)  data2 = 100;
			packADSR(gsAdsrOffset+3,2,(uint32_t)(data2 | ((data1 & 1) << 7)));
            gsHoldSust[3] = (uint32_t)data1 & 1;
            gsSustain[3] = (uint32_t)data2;
            break;
        case 0x3C:                                  //FMO12 RELEASE
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+3,3,(uint32_t)data2);
            if (data2 <= 100) gsRelease[3] = (uint32_t)data2 * A_FACTOR;
            else gsRelease[3] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
       case 0x3D:                                   //FMO12 FixShift
			packADSR(gsAdsrOffset+3,4,(uint32_t)data2);
            gsFixVol[3] = (uint32_t)data2;
            break;
       case 0x3E:                                   //FMO12 Velo
			packADSR(gsAdsrOffset+3,5,(uint32_t)data2);
            gsVelo[3] = (uint32_t)data2;
            break;

//Frequency Modulation Lfo1
        case 0x40:                                  //Lfo1  ATTACK
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+4,0,(uint32_t)data2);
            if (data2 <= 100) gsAttack[4] = (uint32_t)data2 * A_FACTOR;
            else gsAttack[4] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x41:                                  //Lfo1 DECAY
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+4,1,(uint32_t)data2);
            if (data2 <= 100) gsDecay[4] = (uint32_t)data2 * A_FACTOR;
            else gsDecay[4] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x42:                                  //1Shoot
        case 0x43:                                  //TLfo1  SUSTAIN Hold
			if (data2 > 100)  data2 = 100;
			packADSR(gsAdsrOffset+4,2,(uint32_t)(data2 | ((data1 & 1) << 7)));
			gsSustain[4] = (uint32_t)data2;
			gsHoldSust[4] = (~(uint32_t)data1) & 1;
           break;
        case 0x44:                                  //Lfo1  RELEASE
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+4,3,(uint32_t)data2);
            if (data2 <= 100) gsRelease[4] = (uint32_t)data2 * A_FACTOR;
            else gsRelease[4] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x45:                                  //Lfo1  FixShift
			packADSR(gsAdsrOffset+4,4,(uint32_t)data2);
            gsFixVol[4] = (uint32_t)data2;
            break;
       case 0x46:                                   //Lfo1  Velo
			packADSR(gsAdsrOffset+4,5,(uint32_t)data2);;
            gsVelo[4] = (uint32_t)data2;
            break;

//Filter0 Mid Freq variation
        case 0x48:                                  //F0Freq ATTACK
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+9,0,(uint32_t)data2);
            if (data2 <= 100) gsAttack[9] = (uint32_t)data2 * A_FACTOR;
            else gsAttack[9] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x49:                                  //F0Freq DECAY
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+9,1,(uint32_t)data2);
            if (data2 <= 100) gsDecay[9] = (uint32_t)data2 * A_FACTOR;
            else gsDecay[9] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x4A:                                  //1Shoot
        case 0x4B:                                  //F0Freq SUSTAIN Hold
			if (data2 > 100)  data2 = 100;
			packADSR(gsAdsrOffset+9,2,(uint32_t)(data2 | ((data1 & 1) << 7)));
            gsHoldSust[9] = (~(uint32_t)data1) & 1;
            gsSustain[9] = (uint32_t)data2;
            break;
        case 0x4C:                                  //F0Freq RELEASE
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+9,3,(uint32_t)data2);
            if (data2 <= 100) gsRelease[9] = (uint32_t)data2 * A_FACTOR;
            else gsRelease[9] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x4D:                                  //F0Freq FixShift
			packADSR(gsAdsrOffset+9,4,(uint32_t)data2);
            gsFixVol[9] = (uint32_t)data2;
            break;
       case 0x4E:                                   //F0Freq Velo
			packADSR(gsAdsrOffset+9,5,(uint32_t)data2);;
            gsVelo[9] = (uint32_t)data2;
            break;

//filter#0 keytrack depth
        case 0x4F:
            gsKeytrack0Preset[sound] = (uint32_t)data2;
            gsKeytrack0 = (float)gsKeytrack0Preset[sound] / 100.;
            break;

//selectable ADSR type
        case 0x50:                                  // ATTACK
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+gcAdsrSel,0,(uint32_t)data2);
            if (data2 <= 100) gsAttack[gcAdsrSel] = (uint32_t)data2 * A_FACTOR;
            else gsAttack[gcAdsrSel] =SECFACTOR+(uint32_t)(data2-100)*L_FACTOR;
            break;
        case 0x51:                                  //DECAY
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+gcAdsrSel,1,(uint32_t)data2);
            if (data2 <= 100) gsDecay[gcAdsrSel] = (uint32_t)data2 * A_FACTOR;
            else gsDecay[gcAdsrSel] =SECFACTOR +(uint32_t)(data2-100)*L_FACTOR;
            break;
        case 0x52:                                  //1Shoot
        case 0x53:                                  //SUSTAIN Hold
			if (data2 > 100)  data2 = 100;
			packADSR(gsAdsrOffset+gcAdsrSel,2,(uint32_t)(data2|((data1&1)<<7)));
            gsHoldSust[gcAdsrSel] = (uint32_t)data1 & 1;
            gsSustain[gcAdsrSel] = (uint32_t)data2;
            break;
        case 0x54:                                  //RELEASE
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+gcAdsrSel,3,(uint32_t)data2);
            if (data2 <= 100) gsRelease[gcAdsrSel] = (uint32_t)data2 * A_FACTOR;
            else gsRelease[gcAdsrSel]=SECFACTOR+(uint32_t)(data2-100)*L_FACTOR;
            break;
       case 0x55:                                   //FixShift
			packADSR(gsAdsrOffset+gcAdsrSel,4,(uint32_t)data2);
            gsFixVol[gcAdsrSel] = (uint32_t)data2;
            break;
       case 0x56:                                   //Velo
			packADSR(gsAdsrOffset+gcAdsrSel,5,(uint32_t)data2);
            gsVelo[gcAdsrSel] = (uint32_t)data2;
            break;
//assignment of selectable ADSR
       case 0x57:                                   //select ADSR type by number
			if (data2 > 10) data2 = 9;				//then select default
			gcAdsrSel = data2;
            break;

//free assignable ADSR
       case 0x58:                                  //Free ADSR ATTACK
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+10+gsFreeAdsr,0,(uint32_t)data2);
            if (data2 <= 100) gsAttack[10] = (uint32_t)data2 * A_FACTOR;
            else gsAttack[10] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x59:                                  //Free ADSR DECAY
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+10+gsFreeAdsr,1,(uint32_t)data2);
            if (data2 <= 100) gsDecay[10] = (uint32_t)data2 * A_FACTOR;
            else gsDecay[10] = SECFACTOR + (uint32_t)(data2 - 100) * L_FACTOR;
            break;
        case 0x5A:                                  //1Shoot
        case 0x5B:                                  //Free ADSR SUSTAIN Hold
			if (data2 > 100)  data2 = 100;
            gsHoldSust[10] = (~(uint32_t)data1) & 1;
            gsSustain[10] = (uint32_t)data2;
            data2 |= (data1 & 1) << 7;
			packADSR(gsAdsrOffset+10+gsFreeAdsr,2,(uint32_t)data2);
            break;
        case 0x5C:                                  //Free ADSR RELEASE
			if (data2 > 125)  data2 = 125;
			packADSR(gsAdsrOffset+10+gsFreeAdsr,3,(uint32_t)data2);
            if (data2 <= 100) gsRelease[10] = (uint32_t)data2 * A_FACTOR;
            else gsRelease[10] = SECFACTOR + (uint32_t)(data2 - 100) *L_FACTOR;
            break;
       case 0x5D:                               //Free ADSR FixShift/Envelope
			packADSR(gsAdsrOffset+10+gsFreeAdsr,4,(uint32_t)data2);
            gsFixVol[10] = (uint32_t)data2;
            break;
       case 0x5E:                                   //Free ADSR Velo
			packADSR(gsAdsrOffset+10+gsFreeAdsr,5,(uint32_t)data2);
            gsVelo[10] = (uint32_t)data2;
            break;
//assignment of free ADSR type
       case 0x5F:
	   		if (data2 > 6) return;
           	gsFreeAdsr = (uint32_t)data2; //values 0,1,2... 6
            gsFreeAdsrPreset[sound] = gsFreeAdsr;
			break;

//Delay
         case 0x60:
            if (data2 > 30) data2 = 30;                 //max. 300.0 ms
            gDelayTime &= 0x000000FF;                   //clear bits 8...15
            gDelayTime |= (int32_t)data2 << 8;          //replace with new data
            if (gDelayTime > 0x00001E00)  gDelayTime = 0x00001E00;
            gDelayTimePreset[sound] = gDelayTime;
            gDelayOffset = (gDelayTime >> 8) * 331 + \
                                        ((gDelayTime & 0x000000FF) * 331)/100;
            break;
         case 0x61:
            if (data2 > 99) data2 = 99;     //tens and ones separately,no carry
            gDelayTime &= 0x0000FF00;                   //clear bits 0...7
            gDelayTime |= (int32_t)data2;               //replace with new data
            if (gDelayTime > 0x00001E00)  gDelayTime = 0x00001E00;
            gDelayTimePreset[sound] = gDelayTime;
            gDelayOffset = (gDelayTime >> 8) * 331 + \
                                        ((gDelayTime & 0x000000FF) * 331)/100;
            break;
        case 0x62:
            gDelayLevelFactor = (int32_t)data2;             //range 0...127
            gDelayLevelPreset[sound] = data2;           //range 0...127
            break;
         case 0x63:                                         //phase inversion
            gDelayLevelFactor = (int32_t)(-data2);          //range -127..0
            gDelayLevelPreset[sound] = data2 | 0x80;    //range 128...255
            break;
         case 0x64:
            if (data2 > 99) data2 = 99;   //more would cause self-oscillation
            gFeedbackFactor = (int32_t)data2;
            gFeedbackPreset[sound] = data2;        //range 0...99
            break;
         case 0x65:                                         //phase inversion
            if (data2 > 99) data2 = 99;   //more would cause self-oscillation
            gFeedbackFactor = (int32_t)(-data2);
            gFeedbackPreset[sound] = data2 | 0x80; //range 128...228
            break;
        case 0x66:
//set Chorus Width & switch to controlled by LFO2
        case 0x67:
//set Chorus Width & switch to controlled by LFO3
            if (data2 > 99) data2 = 99;
            gChorusWidth = data2;
            gChorusLfo3 = data1 & 0x00000001;      //i.e. set in case 0x67
            gChorusPreset[sound] = data2 | (gChorusLfo3 << 7);
            break;

//change filter number in actual sound set for subsequent handling
		case 0x68:
            if (data2 > 4) return;
            gfFilterNum = (uint32_t)data2;
			break;
//actually selected Filter number/instance (see Ctrl CHANGE 0x68)
		case 0x69:
            if (data1 > 8) return;
            gfACTIV_IIR[fNum].TYPE = (uint32_t)data2;
            reInitFilter(gfFilterNum);
			break;
        case 0x6A:                          //receive FREQ from controller
            if (gfFilterType == 0) return;
            midiFreq = (uint32_t)data2;
            if (midiFreq < 18)  midiFreq = 18;      //MIDI Note#18 = 23Hz
            if (midiFreq > 120)  midiFreq = 120;    //MIDI Note#120 = 8372Hz
            gfACTIV_IIR[fNum].FREQ = midiFreq;
            reInitFilter(gfFilterNum);
            break;
        case 0x6B:                          //receive QFACTOR from controller
            if (gfFilterType < 3) return;
            midiQFactor = (uint32_t)data2;
            if ((gfFilterType <= 4) || (gfFilterType >= 7)) { //X
                if(midiQFactor < 1) midiQFactor = 1;    //Zeta * 100
                if(midiQFactor >100) midiQFactor = 100;
            }
            else if ((gfFilterType == 5) || (gfFilterType == 6)) { //X
                if (midiQFactor == 0) midiQFactor = 18;//Bandwidth minimum 23Hz
                if (midiQFactor == 0) midiQFactor = 120;//max8372Hz
            }
            else midiQFactor = 71;          //general default
            gfACTIV_IIR[fNum].QFACTOR = midiQFactor;
            reInitFilter(gfFilterNum);
            break;
        case 0x6C:                          //receive PEAK from controller
            if (gfFilterType == 0)  Peak = 20;
            else data2 += 20;				//shift up by 20 for internal adj.
			if (gfFilterType <= 5)  {
                if (data2 > 35) data2 = 35;
                if (data2 < 20) data2 = 20;
                Peak = (uint32_t)data2 ;
            }
            else  {                         //gfFilterType == 6,7,8
                if (data2 > 120) data2 = 120;
                Peak = (uint32_t)data2;
            }
                gfACTIV_IIR[fNum].PEAK = Peak;
                reInitFilter(gfFilterNum);
            break;

//Filter0 modulation by LFO
       case 0x6D:                               //F0mod controlled by LFO1
            if (data2 > 99) data2 = 99;
            gfF0mod = (uint32_t)data2;
            if (data2 == 0) gfF0modLfo2 = 1;     //LFO2 is default
            else  gfF0modLfo2 = 0;
            gfF0modPreset[sound] = gfF0mod | (gfF0modLfo2 << 7);
            break;
        case 0x6E:                              //F0mod controlled by LFO2
            if (data2 > 99) data2 = 99;
            gfF0mod = (uint32_t)data2;
            gfF0modLfo2 = 1;
            gfF0modPreset[sound] = gfF0mod | 0x80;
            break;

//Drums
        case 0x6F:
		    if (data2 > 52)  data2 = 52;
            gdDrumsVol = (uint32_t)data2;
            gdDrumsVolPreset[sound] = gdDrumsVol;
            getAllVolFactors();
            break;
        case 0x70:
            if (data2 > 52) data2 = 52;
            gdKickVol = (uint32_t)data2;
            gdKickVolPreset[sound] = gdKickVol;
            gdKickVolFactor = getVolFactor(gdKickVol);
            break;
         case 0x71:
            if (data2 > 52) data2 = 52;
            gdHiTomVol = (uint32_t)data2;
            gdHiTomVolPreset[sound] = gdHiTomVol;
            gdHiTomVolFactor = getVolFactor(gdHiTomVol);
            break;
        case 0x72:
            if (data2 > 52) data2 = 52;
            gdSnareVol = (uint32_t)data2;
            gdSnareVolPreset[sound] = gdSnareVol;
            gdSnareVolFactor = getVolFactor(gdSnareVol);
            break;
        case 0x73:
            if (data2 > 52) data2 = 52;
            gdRimVol = (uint32_t)data2;
            gdRimVolPreset[sound] = gdRimVol;
            gdRimVolFactor = getVolFactor(gdRimVol);
            break;
        case 0x74:
            if (data2 > 52) data2 = 52;
            gdClapVol = (uint32_t)data2;
            gdClapVolPreset[sound] = gdClapVol;
            gdClapVolFactor = getVolFactor(gdClapVol);
            break;
        case 0x75:
            if (data2 > 52) data2 = 52;
            gdHihatVol = (uint32_t)data2;
            gdHihatVolPreset[sound] = gdHihatVol;
            gdHihatVolFactor = getVolFactor(gdHihatVol);
            break;
        case 0x76:
            if (data2 > 52) data2 = 52;
            gdCymbalVol = (uint32_t)data2;
            gdCymbalVolPreset[sound] = gdCymbalVol;
            gdCymbalVolFactor = getVolFactor(gdCymbalVol);
            break;
        case 0x77:
            if (data2 > 52) data2 = 52;
            gdBellVol = (uint32_t)data2;
            gdBellVolPreset[sound] = gdBellVol;
            gdBellVolFactor = getVolFactor(gdBellVol);
            break;
//all Notes Off
        case 0x7B:									//data2 doesn't mind
			panic_Off(0);
            break;
//Mono-Poly-phony
        case 0x7E:									//select Monophony
        case 0x7F:									//select Polyphony
        	data1 = ~data1;							//invert bit0 value
        	gsMonoPoly = (uint32_t)(data1 & 0x01);	//ignore data2
            gfStageMask[sound] &= 0x000000DF;
            gfStageMask[sound] |= gsMonoPoly << 5;	//bit5 doubles gsMonoPoly
            break;

        default:
            return;                         // every MIDI msg else is not valid
    }
    updateDisplay();
}


void updateDisplay(void)  // ************************************************
// **************************************************************************
{
    if (gpConfigPanel & 0x80)  initPlay();
    else  {
        if (gpConfigPanel == 0)  initGeneral();
        else if (gpConfigPanel == 1)  initOsc();
        else if (gpConfigPanel == 2)  initAdsr();
        else if (gpConfigPanel == 3)  initDelay();
        else if (gpConfigPanel == 4)  initFilter();
        else if (gpConfigPanel == 5)  initRoute();
        else if (gpConfigPanel == 6)  initSystem();
    }
}


void  dumpSoundSet(uint32_t soundset)  // ***********************************
//transfer size is 240 bytes incl.Sysex Frame and ID,EOX, i.e.236 bytes payload
// **************************************************************************
{
uint32_t k,n;
uint8_t delayflags, delayLevel, feedback;

		placeCursor(0x10);
		displayNL99(soundset,0);
		placeCursor(0x10+16);		//invisible
        sendM(0xF0);
        sendM(0x7D);                //experimental/hobbyist MIDI Manufacturer ID
        sendM(0x20);                //dump Sound Set file type ID
        sendM((uint8_t)soundset);				//hex bytes:
        split16(gVid);							//default = 00,04,08,03
        split16(gPid);							//default = 05,07,04,00
        sendM((uint8_t)gChannel);              //default = 00
        sendM((uint8_t)gDrumsChannel);         //default = 09
//14 bytes
        sendM((uint8_t)gVolumePreset[soundset]);        //default =0x28

        sendM((uint8_t)((gaRoutePreset[soundset] >>16) & 0x0000001F));
													//gaRouteS0N default =0x0
        sendM((uint8_t)(gaRoutePreset[soundset]) & 0x0000001F);
													//gaRoute3B default =0x01
        sendM((uint8_t)gfGainPreset[soundset]);         //default =0x00
        sendM((uint8_t)(gfBasTrebPreset[soundset] & 0x0000007F));
                                                        //default =0x40(dec64)
        sendM((uint8_t)((gfBasTrebPreset[soundset] >> 8) & 0x0000007F));
                                                        //default =0x40(dec64)
        sendM((uint8_t)((gfBasTrebPreset[soundset] >> 16) & 0x0000007F));
                                                        //default =0x40(dec64)
        sendM((uint8_t)((gfBasTrebPreset[soundset] >> 24) & 0x0000007F));
                                                        //default =0x40(dec64)
        sendM((uint8_t)gsWAVEFORM_1Preset[soundset]);   //default =0x00
        sendM((uint8_t)gsWAVEFORM_2Preset[soundset]);   //default =0x02

        sendM((uint8_t)(gsFreeSquare1[soundset] >> 24) & 0x7F);	//default=34
        sendM((uint8_t)(gsFreeSquare1[soundset] >> 16) & 0x7F);	//default=1
        sendM((uint8_t)gsFreeSquare1[soundset] & 0x7F);			//default=36
        sendM((uint8_t)(gsFreeSquare2[soundset] >> 24) & 0x7F);	//default=14
        sendM((uint8_t)(gsFreeSquare2[soundset] >> 16) & 0x7F);	//default=3
        sendM((uint8_t)gsFreeSquare2[soundset] & 0x7F);			//default=16

        sendM((uint8_t)gsGLIDEPreset[soundset]);        //default =0x00

        sendM((uint8_t)gsTRANSPreset[soundset]);        //default =0x24
        sendM((uint8_t)gsTRANS_2Preset[soundset]);      //default =0x30
        sendM((uint8_t)gsDETUNEPreset[soundset]);       //default =0x40
        sendM((uint8_t)gsDETUNE_2Preset[soundset]);     //default =0x40

        sendM((uint8_t)(gsMOD_1Preset[soundset] & 0x7F)); //default =0x00
        sendM((uint8_t)(gsMOD_2Preset[soundset] & 0x7F)); //default =0x00

        sendM((uint8_t)gfF0modPreset[soundset] & 0x7F); //default =0x00
        sendM((uint8_t)((gfF0modPreset[soundset] >> 7) & 0x01) |	//MODFLAGS
              (uint8_t)((gsMOD_1Preset[soundset] >> 6) & 0x02) |
              (uint8_t)((gsMOD_2Preset[soundset] >> 5) & 0x04));
        sendM((uint8_t)(gLfo1WavePreset[soundset] & 0x7F)); //default =0x28
        sendM((uint8_t)gLfo1FreqPreset[soundset]);          //default =0x00
        sendM((uint8_t)(gLfo2WavePreset[soundset] & 0x7F)); //default =0x28
        sendM((uint8_t)gLfo2FreqPreset[soundset]);      //default =0x0A (dec.10)
        sendM((uint8_t)(gLfo3WavePreset[soundset] & 0x7F)); //default =0x28
        sendM((uint8_t)gLfo3FreqPreset[soundset]);          //default =0x00
        sendM((uint8_t)((gLfo1WavePreset[soundset] >> 7) & 0x03) |	//LFOFLAGS
              (uint8_t)((gLfo2WavePreset[soundset] >> 5) & 0x0C) | //bit 87->32
              (uint8_t)((gLfo3WavePreset[soundset] >> 3) & 0x30)); //default=0
//46 bytes
        for (k = 0; k < N_ASTRUCT; k++) {		//N_ASTRUCT=19, total 133 bytes
            n = (soundset * N_ASTRUCT) + k;
			sendM((uint8_t)((gsADSR[n].ATDECSUS>>24)&0x0000007F)); 	//Attack
			sendM((uint8_t)((gsADSR[n].ATDECSUS>>16)&0x0000007F)); 	//Decay
			sendM((uint8_t)(gsADSR[n].ATDECSUS & 0x0000007F)); 		//Sustain
			sendM((uint8_t)((gsADSR[n].ATDECSUS>>7)&0x00000001));//Sust.OneShoot
			sendM((uint8_t)((gsADSR[n].REFIXVEL>>16)&0x0000007F));	//Release
			sendM((uint8_t)((gsADSR[n].REFIXVEL>>8)&0x0000007F));	//Fixvol
			sendM((uint8_t)(gsADSR[n].REFIXVEL&0x0000007F));		//Velocity
        }
//179 bytes
        for (k = 0; k < N_FILTER; k++) {		//N_Filter=5, total 40 bytes
            n = (soundset * N_FILTER) + k;
            sendM((uint8_t)(gfIIRPack[n].TYPE & 0x0000007F)); 		//Easy
            sendM((uint8_t)(gfIIRPack[n].FREQ & 0x0000007F));
            sendM((uint8_t)(gfIIRPack[n].QFACTOR & 0x0000007F));
            sendM((uint8_t)(gfIIRPack[n].PEAK & 0x0000007F));
        }
        for (k = 0; k < N_FILTER; k++) {
            n = (soundset * N_FILTER) + k;
            sendM((uint8_t)((gfIIRPack[n].TYPE>>8) & 0x0000007F)); 	//Flex
            sendM((uint8_t)((gfIIRPack[n].FREQ>>8) & 0x0000007F));
            sendM((uint8_t)((gfIIRPack[n].QFACTOR>>8) & 0x0000007F));
            sendM((uint8_t)((gfIIRPack[n].PEAK>>8)&0x0000007F));

        }
//219 bytes

        sendM((uint8_t)((gfStageMask[soundset] >> 5) & 0x0000007F)); //def =0x0
//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bits 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)

        sendM((uint8_t)gsFreeAdsrPreset[soundset]);     //default = 00
		sendM((uint8_t)gNTrigger[soundset]);			//default = 01
        sendM((uint8_t)gsKeytrack0Preset[soundset]);    //default = 00
        sendM((uint8_t)((gDelayTimePreset[soundset] >> 8) & 0x0000007F));//def=0
        sendM((uint8_t)(gDelayTimePreset[soundset] & 0x0000007F)); //default =00
        delayLevel = (uint8_t)gDelayLevelPreset[soundset];
        sendM(delayLevel & 0x7F);                       //default =00
        delayflags = 0;
        if (delayLevel >= 0x80)  delayflags = 2;        //if phase inversion
        feedback =  (uint8_t)gFeedbackPreset[soundset];
        sendM(feedback & 0x7F);                         //default =00
        if (feedback >= 0x80)  delayflags |= 4;         //if phase inversion
		sendM((uint8_t)gChorusPreset[soundset] & 0x7F);	//default = 0
		sendM((uint8_t)gVibratoPreset[soundset] & 0x7F);//default = 0
        if (gChorusPreset[soundset] >= 0x80) delayflags |= 8; //default =0(Lfo3)
        if (gVibratoPreset[soundset] >= 0x80) delayflags |= 1;//default =0(Lfo1)
        sendM(delayflags);                          	//default =0
//230bytes
        sendM((uint8_t)gdDrumsVol);                     //default =0x28
        sendM((uint8_t)gdKickVol);                      //default =0x28
        sendM((uint8_t)gdHiTomVol);                     //default =0x28
        sendM((uint8_t)gdSnareVol);                     //default =0x28
        sendM((uint8_t)gdRimVol);                       //default =0x28
        sendM((uint8_t)gdClapVol);                      //default =0x28
        sendM((uint8_t)gdHihatVol);                     //default =0x28
        sendM((uint8_t)gdCymbalVol);                    //default =0x28
        sendM((uint8_t)gdBellVol);                      //default =0x28
        sendM(0xF7);
//240 bytes
}


void  dumpAllSoundSets(void)  // ********************************************
// **************************************************************************
{
uint32_t k;

    for (k=0; k < N_SOUNDS; k++) {
        dumpSoundSet(k);
        msDelay(300);                                //feeds watchdog
    }
}


void split16(uint32_t word)  // *********************************************
//splits lower 16 bit into 4 nibbles with bits7-4 = 0.  ms nibbles sent first
// **************************************************************************
{
        sendM((uint8_t)((word >> 12) & 0x0000000F));
        sendM((uint8_t)((word >> 8) & 0x0000000F));
        sendM((uint8_t)((word >> 4) & 0x0000000F));
        sendM((uint8_t)(word & 0x0000000F));
}


uint32_t uploadSoundSet(uint32_t soundset)  // ******************************
//only save completely transferred soundsets
//transfer size is 240 bytes incl.Sysex Frame and ID,EOX, i.e.236 bytes payload
// **************************************************************************
{
uint32_t temp;
uint32_t k, n, recvword;
uint32_t vid, pid, channel, drumsChannel;

uint32_t wave1, wave2;
uint32_t range1H, range1L, slope1, range2H, range2L, slope2;
uint32_t glide;
int32_t trans, trans2;
int32_t detune, detune2;
uint32_t mod1, mod2, modflags;

uint32_t Lfo1Wave, Lfo1Freq, Lfo2Wave, Lfo2Freq, Lfo3Wave, Lfo3Freq, lfoflags;

uint32_t attack[N_ASTRUCT+2];
uint32_t dekay[N_ASTRUCT+2];
uint32_t sustain[N_ASTRUCT+2];
uint32_t holdSustain[N_ASTRUCT+2];
uint32_t release[N_ASTRUCT+2];
uint32_t fixvol[N_ASTRUCT+2];
uint32_t velo[N_ASTRUCT+2];

uint32_t keytrack0, fuzz, route_E, route_F;
uint32_t bass_E, bass_F, treble_E, treble_F;
uint32_t delayLevel, feedback, delayflags, delayTime, chorusWidth, vibrato;
uint32_t volume, specialflags, freeAdsr, f0mod, ntrigger;
uint32_t kickVol,hiTomVol,snareVol,rimVol,clapVol,hihatVol,cymbalVol, bellVol;
uint32_t drumsVol;

struct FILTER {
uint32_t TYPE;                      //uint32_t for flash 32bit parallelism
uint32_t FREQ;
uint32_t QFACTOR;                   //bandwidth or zeta
uint32_t PEAK;
} Filter_E[N_FILTER], Filter_F[N_FILTER];

const uint8_t UploadStr[] = {"upload SndSet#  "};

    displayString(0,UploadStr);
    placeCursor(14);
    displayNL99(soundset,0);
    displayString(0x10,gpEmptyStr);

    temp = pack16();
    if (temp == 0) return 0;
    if (temp == 0xFFFFFFFF)  return 0; //data timeout or unexpected EOX
    vid = temp;
    temp = pack16();
    if (temp == 0) return 0;
    if (temp == 0xFFFFFFFF)  return 0; //data timeout or unexpected EOX
    pid = temp;

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    channel = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    drumsChannel = (uint32_t)(recvword & 0x0000007F);
//10 bytes
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    volume = (uint32_t)(recvword & 0x0000007F);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    route_E = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    route_F = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    fuzz= (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    bass_E = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    treble_E = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    bass_F = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    treble_F = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    wave1 = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    wave2 = (uint32_t)(recvword & 0x0000007F);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    range1H = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    range1L = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    slope1 = (uint32_t)(recvword & 0x0000007F);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    range2H = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    range2L = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    slope2 = (uint32_t)(recvword & 0x0000007F);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    glide = (uint32_t)(recvword & 0x0000007F);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    trans = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    trans2 = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    detune = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    detune2 = (int32_t)(recvword & 0x0000007F);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    mod1 = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    mod2 = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    f0mod = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    modflags = (uint32_t)(recvword & 0x00000077);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    Lfo1Wave = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    Lfo1Freq = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    Lfo2Wave = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    Lfo2Freq = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    Lfo3Wave = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    Lfo3Freq = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    lfoflags = (uint32_t)(recvword & 0x0000007F);
//42
    for (n = 0; n < N_ASTRUCT; n++)  {  //19struct a 7bytes =133 bytes
        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        attack[n] = (uint32_t)(recvword & 0x0000007F);
        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        dekay[n] = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        sustain[n] = (uint32_t)(recvword & 0x0000007F);
        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        holdSustain[n] = (uint32_t)(recvword & 0x0001);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        release[n] = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        fixvol[n] = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        velo[n] = (uint32_t)(recvword & 0x0000007F);
    }
//175
    for (k = 0; k < N_FILTER; k++)  {
        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        Filter_E[k].TYPE = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        Filter_E[k].FREQ = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        Filter_E[k].QFACTOR = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        Filter_E[k].PEAK = (uint32_t)(recvword & 0x0000007F);
    }
    for (k = 0; k < N_FILTER; k++)  {
        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        Filter_F[k].TYPE = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        Filter_F[k].FREQ = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        Filter_F[k].QFACTOR = (uint32_t)(recvword & 0x0000007F);

        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0;
        Filter_F[k].PEAK = (uint32_t)(recvword & 0x0000007F);
    }
//215
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    specialflags = (uint32_t)(recvword & 0x0000007F);	//gFStageMask specials

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    freeAdsr = (uint32_t)(recvword & 0x000F);     //function assigned to ADSR7

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    ntrigger = (uint32_t)(recvword & 0x0000007F);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    keytrack0 = (uint32_t)(recvword & 0x0000007F);

    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    delayTime = (uint32_t)(recvword & 0x0000007F) << 8;
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    delayTime |= (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    delayLevel = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    feedback = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    chorusWidth = (int32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    vibrato= (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    delayflags = (uint32_t)(recvword & 0x0000007F);
//226
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    drumsVol = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    kickVol = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    hiTomVol = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    snareVol = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    rimVol= (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    clapVol = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    hihatVol= (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    cymbalVol = (uint32_t)(recvword & 0x0000007F);
    recvword = pullTimedByte();
    if (recvword == 0xFFFFFFFF) return 0;
    bellVol = (uint32_t)(recvword & 0x0000007F);
//235
    recvword = pullTimedByte();
    if (recvword != 0x000080F7)  {
        return 0;     //no terminating EOX received
    }
//236bytes + 4 bytes header = 240 bytes
//soundset obviously received successfully, finally store to  SRAM
    gVid = vid;
    gPid = pid;
    gChannel = channel;
    gDrumsChannel = drumsChannel;

    gVolumePreset[soundset] = volume;
    gaRoutePreset[soundset] =  (route_F << 16) | route_E;
    gfGainPreset[soundset] = fuzz;
    gfBasTrebPreset[soundset] = (treble_F << 24) | (bass_F << 16) |
													(treble_E << 8) | bass_E;
    gsWAVEFORM_1Preset[soundset] = wave1;
    gsWAVEFORM_2Preset[soundset] = wave2;
    gsFreeSquare1[soundset] = (range1H<<24) | (range1H<<16) | slope1;
    gsFreeSquare2[soundset] = (range2H<<24) | (range2H<<16) | slope2;

    gsGLIDEPreset[soundset] = glide;
    gsTRANSPreset[soundset] = trans;
    gsTRANS_2Preset[soundset] = trans2;
    gsDETUNEPreset[soundset] = detune;
    gsDETUNE_2Preset[soundset] = detune2;
    gfF0modPreset[soundset] = f0mod | ((modflags & 1) << 7);		//bit1<<bit7
    gsMOD_1Preset[soundset] = mod1 | ((modflags & 2) << 6);
    gsMOD_2Preset[soundset] = mod2 | ((modflags & 4) << 5);

    gLfo1WavePreset[soundset] = Lfo1Wave | ((lfoflags & 0x03) << 7);//bit7<<bit0
    gLfo1FreqPreset[soundset] = Lfo1Freq;							//bit8<<bit1
    gLfo2WavePreset[soundset] = Lfo2Wave | ((lfoflags & 0xC0) << 5);//bit7<<bit2
    gLfo2FreqPreset[soundset] = Lfo2Freq;							//bit8<<bit3
    gLfo3WavePreset[soundset] = Lfo3Wave | ((lfoflags & 0x30) << 3);//bit7<<bit4
    gLfo3FreqPreset[soundset] = Lfo3Freq;							//bit8<<bit5

//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bit 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)

    gfStageMask[soundset] = specialflags << 5; //bits5-10 are special bits
										//bits0..4 updated at activateSoundSet()
    gsFreeAdsrPreset[soundset] = freeAdsr;
	gNTrigger[soundset] = ntrigger;

    for (k=0;k < N_FILTER;k++)  {
//        n = (gSoundSet * N_FILTER) + k;		//bugfix 17Dec22
        n = (soundset * N_FILTER) + k;
        gfIIRPack[n].TYPE &= 0xFFFF0000;		//preserve Default values
        gfIIRPack[n].TYPE |= ((Filter_F[k].TYPE << 8) | Filter_E[k].TYPE);
        gfIIRPack[n].FREQ &= 0xFFFF0000;
        gfIIRPack[n].FREQ |= ((Filter_F[k].FREQ << 8) | Filter_E[k].FREQ);
        gfIIRPack[n].QFACTOR &= 0xFFFF0000;
        gfIIRPack[n].QFACTOR |= ((Filter_F[k].QFACTOR<<8)|Filter_E[k].QFACTOR);
        gfIIRPack[n].PEAK &= 0xFFFF0000;
        gfIIRPack[n].PEAK  |= ((Filter_F[k].PEAK << 8) | Filter_E[k].PEAK);
    }	//will be unpacked during activateSoundSet();

    gsKeytrack0Preset[soundset] = keytrack0;

    gDelayTimePreset[soundset] = delayTime;
    gVibratoPreset[soundset] = vibrato | ((delayflags & 1) << 7);
    gDelayLevelPreset[soundset] = (int32_t)(delayLevel | ((delayflags & 2)<<6));
    gFeedbackPreset[soundset] = (int32_t)(feedback | ((delayflags & 4) << 5 ));
    gChorusPreset[soundset] = (int32_t)(chorusWidth  | ((delayflags & 8) << 4));

    gdKickVolPreset[soundset] = kickVol;
    gdHiTomVolPreset[soundset] = hiTomVol;
    gdSnareVolPreset[soundset] = snareVol;
    gdRimVolPreset[soundset] = rimVol;
    gdClapVolPreset[soundset] = clapVol;
    gdHihatVolPreset[soundset] = hihatVol;
    gdCymbalVolPreset[soundset] = cymbalVol;
    gdBellVolPreset[soundset] = bellVol;
    gdDrumsVolPreset[soundset] = drumsVol;

    if (soundset == gSoundSet)  activateSoundSet();
    return 1;
}


uint32_t  uploadUserWave(uint32_t waveNum)  // **********************************
//only save completely transferred waves
//transfer size is 5001 bytes incl. Sysex Frame(4bytes) + 4096 bytes data + EOX
// **************************************************************************
{
uint32_t n, recvword, temp;
const uint8_t UploadStr[] = {"upload UserWave"};
int16_t tempWave[1024];

    displayString(0,UploadStr);
    writeLcdData(waveNum + 0x30);
    displayString(0x10,gpEmptyStr);

    for (n = 0; n< 1024;n++)  {
        temp = pack16();
		if (temp == 0xFFFFFFFF)  return 0; //data timeout or unexpected EOX
        tempWave[n] = (int16_t)temp;
    }
    recvword = pullTimedByte();
    if (recvword != 0x80F7)  {
        displayString(0x31,gErrorStr);
        msDelay(2000);
        updateDisplay();
        return 0;     //no terminating EOX
    }

    for (n = 0; n< 1024;n++)  {
    if (waveNum == 0) gsUserWave0[n] = tempWave[n];	//misuse temporarily
    else if (waveNum == 1) gsUserWave1[n] = tempWave[n];
    else if (waveNum == 2) gsUserWave2[n] = tempWave[n];
    else if (waveNum == 3) gsUserWave3[n] = tempWave[n];
    }
	return 1;
}


uint32_t pack16(void)  // ***************************************************
//receive one 16bit halfword packed as 4 SysEx compatible nibbles
// **************************************************************************
{
uint8_t byte[4];
uint16_t k, recvword;

    for (k=0;k < 4;k++)  {
        recvword = pullTimedByte();
        if (recvword == 0xFFFFFFFF) return 0xFFFFFFFF;  //timeout
        if (recvword == 0x000080F7) return 0xFFFFFFFF;  //EOX
        byte[k] = (uint8_t) (recvword & 0x0000000F);
    }
    return ( (byte[0]<<12) | (byte[1]<<8) | (byte[2]<<4) | byte[3] );
}

// ##########################################################################