// User operation panel  specific routines of "NucleoSynth-v3.9" **************
// digital Audio Synthesizer based on STM32 Nucleo F446
// Author and (c)2019-22 Wolfgang Schemmert

// The copyright claim and GPL-3 License is restricted to this file and the
// depending files
// "main.c", "main.h, "comm.c", "filter.c", "dacfreq.c", "synthi.c","drums.c",
// "usb-otg-4611.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org

// Contact: www.midi-and-more.de
// A detailed hardware diy construction manual and a user operating manual
// is provided at this website

// for the IIR filter algorithms used here, see detailled description:
// www.IowaHills.com/A4IIRBilinearTransform.html

// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f4xx_modified.h"
#include "globaldefines.h"

// ************************************
extern int32_t gTest1, gTest2, gTest3, gTest4;      //for debug only
extern uint32_t gVid, gPid, gUsartCOM, gChannel, gDrumsChannel, gTIMEOUT;
extern uint32_t gSendMessage;
extern uint32_t gSoundSet;
extern uint32_t gStatus8, gStatus9, gStatusB, gStatusC; //save runtime overhead

extern uint32_t gVolume, gVolumePreset[N_SOUNDS];
extern int32_t gVolumeFactor;

extern uint32_t gDelayTime, gDelayTimePreset[N_SOUNDS]; //DelayTime 1/10ms units
extern uint32_t gDelayOffset;     //DelayTime scaled Timer6 frequency (*=3.3071)
extern int32_t gDelayLevelFactor, gDelayLevelPreset[N_SOUNDS];
//special conversion between gDelayLevel and gDelayLevelPreset
extern int32_t gFeedbackFactor, gFeedbackPreset[N_SOUNDS];
//special conversion between gFeedbackFactor and gFeedbackPreset

extern uint32_t gChorusWidth, gChorusLfo3, gChorusPreset[N_SOUNDS];
//gChorusLfo3 is a short phrase for gChorusPreset,bit7, see DELAYFLAGS,bit3 too
//extern int32_t gChorus;
extern uint32_t gVibrato, gVibratoLfo1, gVibratoPreset[N_SOUNDS];
//gVibratoLfo1 is a short phrase for gVibratoPreset,bit7,see DELAYFLAGS,bit0 too
extern uint32_t  gNTrigger[N_SOUNDS];

extern uint32_t gLfoNum;
extern uint32_t gLfo1Freq, gLfo1FreqPreset[N_SOUNDS];
extern uint32_t gLfo1Type, gLfo1Wave, gLfo1Slope, gLfo1WavePreset[N_SOUNDS];
extern uint32_t gLfo2Freq, gLfo2FreqPreset[N_SOUNDS];
extern uint32_t gLfo2Type, gLfo2Wave, gLfo2Slope, gLfo2WavePreset[N_SOUNDS];
extern uint32_t gLfo3Freq, gLfo3FreqPreset[N_SOUNDS];
extern uint32_t gLfo3Type, gLfo3Wave, gLfo3Slope, gLfo3WavePreset[N_SOUNDS];
extern const int32_t gDbTable[53];
extern const uint8_t gComStr[];
extern const uint8_t gMidiStr[];

extern void servSystem(void);
extern void reInitUSB(void);
extern void msDelay(uint32_t);
extern void wait100us(void);
extern void wait6us(void);
extern void popFlashSounds(void);
extern uint32_t pushFlashSounds(void);

void initUsart1(void);
extern void sendB(uint8_t);
extern void sendX8(uint8_t);
extern void sendX16(uint16_t);
extern void sendX32(uint32_t);
extern void sendS32(uint32_t);
extern void sendString(const uint8_t *);

//from comm.c
extern void loadFactorySet(uint32_t);
extern void packADSR(uint32_t, uint32_t, uint32_t);
extern void calculateLfoWave(uint32_t,uint32_t);
extern void msgLfoWave(uint32_t);
extern void activateSoundSet(void);
extern void unpackADSR(uint32_t);
extern void activateFilter(uint32_t, uint32_t);
extern void createShortFlags(void);
extern void copyPreset(uint32_t);
extern void updateDisplay(void);
extern void dumpSoundSet(uint32_t);
extern void dumpAllSoundSets(void);

//from synthi.c
struct ADSR {			//stored in Flash preset
uint32_t ATDECSUS;		//bits31..24:Attack.bits23..16:Decay,bits15..0:Sustain
uint32_t REFIXVEL;		//bits23..16:Release,bits15..8:Fixvol,bits7..0:Velocity
};
extern struct ADSR gsADSR[N_SOUNDS*N_ASTRUCT];
//sets of compacted each 7 ADSR structures
//Attack, Decay and Relase times up to 1sec in 10 ms scale,big values compressed
//Sustain bit7 set=hold, else 1x mode

extern uint32_t gsAttack[N_ASTRUCT];	//Attack time in 0.625ms units(1600Hz)
extern uint32_t gsDecay[N_ASTRUCT];		//Decay time in 0.625ms units(1600Hz)
extern uint32_t gsSustain[N_ASTRUCT];
extern uint32_t gsHoldSust[N_ASTRUCT];	//=1 hold Sustain,=0 one Shoot Sustain
extern uint32_t gsRelease[N_ASTRUCT];	//Release time in 0.625ms units(1600Hz)
extern uint32_t gsFixVol[N_ASTRUCT];
extern uint32_t gsVelo[N_ASTRUCT];
extern uint32_t gsAdsrOffset, gsNoTriggerF0;
extern uint32_t gsFreeAdsr, gsFreeIdx, gsFreeAdsrPreset[N_SOUNDS];
extern uint32_t gsRangeFS1, gsSlopeFS1, gsRangeFS2, gsSlopeFS2;
extern uint32_t gsFreeSquare1[N_SOUNDS], gsFreeSquare2[N_SOUNDS];
										//bits31-16= gsRangeFS,bits15-0=gsSlopeF
extern uint32_t gsMonoPoly;				//default=0 =polyphonic, 1=monophonic
extern uint32_t gsO2Sync;//(if set,Osc2 wave start is sync with Osc1 wave start)
extern uint32_t gsWAVEFORM_1, gsOsc1ModLfo2,  gsWAVEFORM_1Preset[N_SOUNDS];
extern uint32_t gsWAVEFORM_2, gsOsc2ModLfo2, gsWAVEFORM_2Preset[N_SOUNDS];
extern int32_t gsTRANS, gsTRANSPreset[N_SOUNDS];
extern int32_t gsTRANS_2, gsTRANS_2Preset[N_SOUNDS];
extern int32_t gsDETUNE, gsDETUNEPreset[N_SOUNDS];
extern int32_t gsDETUNE_2, gsDETUNE_2Preset[N_SOUNDS];
extern uint32_t gsGLIDE, gsGLIDEPreset[N_SOUNDS];
extern uint32_t gsMOD_1, gsOsc1PWM, gsMOD_1Preset[N_SOUNDS];
extern uint32_t gsMOD_2, gsOsc2PWM, gsMOD_2Preset[N_SOUNDS];
extern uint32_t gsKeytrack0Preset[N_SOUNDS];
extern float gsKeytrack0;
extern void initWaveOsc1(uint32_t);
extern void initWaveOsc2(uint32_t);

//from filter.c
extern struct IIRPack {
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
uint32_t QFACTOR;           //bandwidth or 100*zeta(1-100)
uint32_t PEAK;
} gfIIRPack[N_SOUNDS*N_FILTER];
//bits 24-31 contrain Flex Default, bits 16-23 contain Easy Default,
//bits 0-15 contain Flex Modify, bits 0-7 contain EasyModify filter parameters

extern struct IIR {  //each 9words
float A1;
float A2;
float B0;
float B1;
float B2;
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
uint32_t QFACTOR;           //bandwidth or 100*zeta(1-100)
uint32_t PEAK;
} gfACTIV_IIR[N_FILTER], gfTempIIR;
//are recalculated from gfIIRPack[] when sound or routing mode changes

extern const float gfMidiFreqTable[128];
extern uint32_t gfFilterNum, gfFilterType, gfFlexMode;
extern uint32_t gfRuntimeMask, gfStageMask[N_SOUNDS];
//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bit 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)

//temporary "Memo" Filter set
extern uint8_t gfEasyMemoType[N_FILTER];
extern uint8_t gfEasyMemoFreq[N_FILTER];
extern uint8_t gfEasyMemoQfactor[N_FILTER];
extern uint8_t gfEasyMemoPeak[N_FILTER];
extern uint8_t gfFlexMemoType[N_FILTER];
extern uint8_t gfFlexMemoFreq[N_FILTER];
extern uint8_t gfFlexMemoQfactor[N_FILTER];
extern uint8_t gfFlexMemoPeak[N_FILTER];
//temporary "Undo" Filter set
extern uint8_t gfEasyUndoType[N_FILTER];
extern uint8_t gfEasyUndoFreq[N_FILTER];
extern uint8_t gfEasyUndoQfactor[N_FILTER];
extern uint8_t gfEasyUndoPeak[N_FILTER];
extern uint8_t gfFlexUndoType[N_FILTER];
extern uint8_t gfFlexUndoFreq[N_FILTER];
extern uint8_t gfFlexUndoQfactor[N_FILTER];
extern uint8_t gfFlexUndoPeak[N_FILTER];

extern uint32_t gfGain, gfGainPreset[N_SOUNDS];
extern uint32_t gfF0mod, gfF0modLfo2, gfF0modPreset[N_SOUNDS];
extern uint32_t gfBass, gfTreble, gfBasTrebPreset[N_SOUNDS];
extern void initFilterParams(uint32_t, uint32_t, uint32_t, uint32_t, \
															uint32_t, uint32_t);
extern void unpackFilter(uint32_t, uint32_t);
extern void packBasTreb(void);
extern void unpackBasTreb(void);

//from dacfreq.c
extern uint32_t gaFTestFactor;
extern int32_t gaFilterTest;
extern uint32_t gaZero1, gaZero2;
extern uint32_t gaRoute0N, gaRoutePreset[N_SOUNDS];
										//bits23..16=FLEX F0N
										//bits 7..0= EASY F0N
extern uint32_t gaRouteOsc1,gaRouteOsc2,gaRouteNoise;  //shortflags


// from drums.c
extern uint32_t gdDrumsVolIndex;
            //= normalized sum of gdDrumsVol+gOutVol+gd(drum type)Vol
extern uint32_t gdDrumsVol, gdDrumsVolPreset[N_SOUNDS];
extern uint32_t gdKickVol, gdKickVolPreset[N_SOUNDS];
extern int32_t gdKickVolFactor;
extern uint32_t gdHiTomVol, gdHiTomVolPreset[N_SOUNDS];
extern int32_t gdHiTomVolFactor;
extern uint32_t gdSnareVol, gdSnareVolPreset[N_SOUNDS];
extern int32_t gdSnareVolFactor;
extern uint32_t gdRimVol, gdRimVolPreset[N_SOUNDS];
extern int32_t gdRimVolFactor;
extern uint32_t gdClapVol, gdClapVolPreset[N_SOUNDS];
extern int32_t gdClapVolFactor;
extern uint32_t gdHihatVol, gdHihatVolPreset[N_SOUNDS];
extern int32_t gdHihatVolFactor;
extern uint32_t gdCymbalVol, gdCymbalVolPreset[N_SOUNDS];
extern int32_t gdCymbalVolFactor;
extern uint32_t gdBellVol, gdBellVolPreset[N_SOUNDS];
extern int32_t gdBellVolFactor;

extern int32_t getVolFactor(int32_t);
extern void getAllVolFactors(void);

//from usb-otg-g4611.c
extern uint32_t guDeviceClass, guConfigured;

// ************************************

uint32_t gpAdsrIndex;
uint32_t gpOscSelect, gpSelectAction, gpSetNumber;
uint32_t gpKeyTest, gpKeyState, gpZero; //gpZero = delay at zero level
uint32_t gpLockKey_F, gpLockKey_B, gpLockKey_U, gpLockKey_D, gpLockPlus;
uint32_t gpPanelMode;
uint32_t gpConfigPanel, gpPlusState;

uint32_t gpNewPanel;
uint32_t gpEncCheck, gpEncTest, gpEncState, gpEncNew;
int32_t  gpEncValue, gpEncOldValue;

uint32_t gpModPlay, gpModXPlay;
uint32_t gpModGeneral, gpModOsc, gpModAdsr, gpModDelay, gpModFilter, gpModRoute;
uint32_t gpModSystem;
uint32_t gpModXGeneral, gpModXOsc, gpModXAdsr, gpModXRoute, gpModXSystem;
uint32_t gpCursorPos, gpPosPlay, gpPosXPlay;
uint32_t gpPosGeneral, gpPosOsc, gpPosAdsr, gpPosDelay, gpPosFilter, gpPosRoute;
uint32_t gpPosXGeneral, gpPosXOsc, gpPosXAdsr, gpPosXRoute;
uint32_t gpPlus[8] = {0,0,0,0,0,0,0,0};
uint32_t gpPlusPlay = 0;

uint32_t gpTempVid, gpTempPid, gpTempCh, gpTempDrumsCh;

uint32_t gpDrumType, gpLfoType;
int32_t gpLfoValue;

uint32_t gpFilterSet, gpWaitKeyB, gpNoServPanel;

const uint8_t gpEmptyStr[] =    {"                "};
const uint8_t gpSelectStr[] =   {"select w. KeyFWD"};
const uint8_t gpConfirmStr[] =  {"confirm KeyBK+FW"};
const uint8_t gpOKStr[] =       {"       OK       "};
const uint8_t gpVSoundPGStr[] = {"V    S   Py Gl  "};
const uint8_t gpVibToneStr[] =  {"Vib:   B    T   "};
const uint8_t gpBwidthStr[] = {"Bwidth"};
const uint8_t gpResonaStr[] = {"Resona"};

const uint8_t gpWaveStr[] [6] = {
	{"Sine "},{"Trian"},{"aTriS"},{"aTriH"},{"Saw  "},{"Trpez"},
	{"Sq50%"},{"Sq25%"},{"Sq20%"},{"FrSq1"},{"FrSq2"},{"Spike"},
	{"Z+Si2"},{"Z+2S2"},{"Si2--"},{"S+Si3"},{"S+Si4"},{"SiTrp"},//17
	{"Strat"},{"ExpS3"},{"ExpS5"},{"User0"},{"User1"},{"User2"},{"User3"},//24

	{"Si5th"},{"Si~2S"},{"Si~Tp"},{"S~Str"},{"S~E5S"},{"Us0~1"},//30
	{"Us0~2"},{"Us0~3"},{"Us1~2"},{"Us1~3"},{"Us2~3"},{"Noise"},//36
	{"Si4th"},{"SM3rd"},{"UsAll"},{"ZS2~S"},{"4Si--"},{"Tp~3S"}};//42

const uint8_t gpEasyStr[] = {"Easy"};
const uint8_t gpFlexStr[] = {"Flex"};
const uint8_t gpDefaultStr[] = {"Default"};
const uint8_t gpActFiltStr[] = {"actFilt"};
const uint8_t gpMemoStr[] =    {"Memo   "};
const uint8_t gpSquarStr[]  = {"Squar"};
const uint8_t gpTrianStr[]  = {"Trian"};

const uint8_t gpAdsrTypeStr[] [6] = {
		{"Osc1 "},{"Osc2 "},{"NoisE"},{"FmO12"},{"Lfo1F"},{"Vibra"},
		{"Bass "},{"Trebl"},{"Gain "},{"F0mid"},{"F0Pek"},
        {"FmO2+"},{"Lfo2F"},{"F0BwR"},{"NoisF"},
        {"Chors"},{"Dry  "},{"Delay"},{"FeedB"} };

const uint8_t gpDrumTypeStr[] [5] = {
    {"All "},{"Kick"}, {"HTom"}, {"Snar"}, {"Rim "},
    {"Clap"},{"Hhat"},{"Cymb"},{"Bell"} };

void initControlPanel(void);
void controlPanel(void);
uint32_t keyCheck(void);
void initPlay(void);
void redrawPlayDisplay(void);
void redrawRoute(void);
void controlPlay(void);
void redrawLine1(void);
void redrawPlayLine2(void);
void controlSounds(void);
void controlVolume(void);
void controlBass(uint32_t);
void controlTreble(uint32_t);
void controlMonoPoly(void);
void controlVibrato(void);
void displayVibrato(void);
void initGeneral(void);
void redrawGeneral(void);
void redrawGenLine2(void);
void controlGeneral(void);
void initOsc(void);
void redrawOscDisplay(void);
void redrawOscMod(void);
void controlOsc(void);
void initAdsr(void);
void redrawAdsrDisplay(void);
void controlAdsr(void);
void initDelay(void);
void controlDelay(void);
void initFilter(void);
void redrawFilterDisplay(void);
void controlFilter(void);
void reInitFilter(uint32_t);
void initRoute(void);
void updateRoute(void);
void redrawRouteLine1(void);
void redrawRouteLine2(void);
void controlRoute(void);
void newFilterSet(void);
void saveUndo(void);
void saveMemo(void);
void restoreFilterSet(void);
void initSystem(void);
void redrawSystemLine2(void);
void redrawSystemLine1(void);
void controlSystem(void);

//Display
void adjPeak3(uint32_t);
void adjPeak4(uint32_t);
void displayPeak(uint32_t);
void displayQFactor(uint32_t);
void displayMidiFreq(uint32_t,uint32_t);
void displayDelayVol(int32_t);
void displayDelayTime(void);

void displayString(uint32_t, const uint8_t *);
void displayHexWord(uint32_t);
void displayHexByte(uint32_t);
void displayHexNibble(uint32_t);
void display255(uint32_t, uint32_t);
void display99(uint32_t, uint32_t);
void displayNL99(uint32_t, uint32_t);
void displaySigned(uint32_t, uint32_t);
void placeCursor(uint32_t);
void initDisplay(void);
void displaySpi(uint32_t);
void writeLcdInstr(uint32_t);
void writeLcdData(uint32_t);
//void defineTilde(void);
void evalEncoder(void);

// #########################################################################
#define KEY_F	GPIO_IDR_IDR_7		//PC7	KEY_F
#define KEY_B	GPIO_IDR_IDR_3		//PC3	KEY_B
#define KEY_U	GPIO_IDR_IDR_0		//PC0	KEY_U
#define KEY_D	GPIO_IDR_IDR_1		//PC1	KEY_D

#define KEYPLUS  GPIO_IDR_IDR_13	//PC13 Encoder pushbutton

#define KEY_F_PRESSED     0x000B
#define KEY_B_PRESSED     0x0083
#define KEY_U_PRESSED     0x008A
#define KEY_D_PRESSED     0x0089
#define KEY_BF_PRESSED    0x0003
#define KEY_IDLE          0x008B

void initControlPanel(void)  // *********************************************
// **************************************************************************
{
//init keys (for NucleoF446 with STM32F446RE)
//KEY_F=PC7, KEY_B=PC3, KEY_U=PC0, KEY_D=PC1
//init all of them as input with PullUp:
    GPIOC->MODER &= ~(GPIO_MODER_MODER0 | GPIO_MODER_MODER1 |
                      GPIO_MODER_MODER3 | GPIO_MODER_MODER7);

    GPIOC->PUPDR &=  ~(GPIO_PUPDR_PUPDR0 | GPIO_PUPDR_PUPDR1 |
                     GPIO_PUPDR_PUPDR3 | GPIO_PUPDR_PUPDR7);
    GPIOC->PUPDR |=  (GPIO_PUPDR_PUPDR0_0 | GPIO_PUPDR_PUPDR1_0
                      | GPIO_PUPDR_PUPDR3_0 | GPIO_PUPDR_PUPDR7_0);
//KEYPLUS=PC13
//pushbutton of the encoder
    GPIOC->MODER &= ~(GPIO_MODER_MODER13);
    GPIOC->PUPDR &=  ~(GPIO_PUPDR_PUPDR13);
    GPIOC->PUPDR |=  GPIO_PUPDR_PUPDR13_0;
    wait6us();
    gpKeyTest = GPIOC->IDR & (KEY_F |KEY_B |KEY_U |KEY_D);
    gpKeyState = gpKeyTest;

//init Encoder (PC14+15) as input with pull-up
//the others (left and middle) are init. formally to be in defined state
    GPIOC->MODER &= ~(GPIO_MODER_MODER14 | GPIO_MODER_MODER15); //cfg. as input
    GPIOC->PUPDR &= ~(GPIO_PUPDR_PUPDR14 | GPIO_PUPDR_PUPDR15);   //neutralize
    GPIOC->PUPDR |=  (GPIO_PUPDR_PUPDR14_0 | GPIO_PUPDR_PUPDR15_0 );   //pull_Up
    wait6us();

    gpEncTest = (GPIOC->IDR >> 14) & 0x00000003;   //now bits0,1= Encoder->3

    gpEncState = gpEncTest;
    gpEncCheck = 0;
    gpEncNew = 0;

    gpLockKey_F = 0;
    gpLockKey_B = 0;
    gpLockKey_U = 0;
    gpLockKey_D = 0;
    gpLockPlus = 0;
    gpNewPanel = 0;

//	resetPanel();
	gpModPlay = 0;						//default modificators
	gpModXPlay = 0;
	gpModGeneral = 0;
	gpModXGeneral = 0;
	gpModOsc = 0;
	gpModXOsc = 0;
	gpModAdsr = 0;
	gpModXAdsr = 0;
	gpModDelay = 0;
	gpModFilter = 0;
	gpModRoute = 0;
	gpModXRoute = 0;
	gpModSystem = 0;

	gpPosPlay = 0;						//default cursor positions
	gpPosXPlay = 0;
	gpPosGeneral = 0;
	gpPosXGeneral = 2;
	gpPosOsc = 2;
	gpPosXOsc = 5;
	gpPosAdsr = 4;
	gpPosXAdsr = 2;
	gpPosDelay = 3;
	gpPosFilter = 2;
	gpPosRoute = 1;
	gpPosXRoute = 0;

	gpFilterSet = 1;	//displayed filter set.Defacto always "active" is active
	gpWaitKeyB = 0;

	gpOscSelect = 0;	//start with OSC1
	gpAdsrIndex = 0;

    gSoundSet = 0;
    gpPlusState = 0;
    gpConfigPanel = 0;//during development start "General",finally start "Play"
    writeLcdInstr(0x0E);	//"Display ON" and cursor ON (bit1 set)
    writeLcdInstr(0x0D);
	initGeneral();
}


void controlPanel(void)  // **************************************************
// gpConfigPanel:
// Config Modes/Pages
// 0x80 (bit7 set)= Play
// 0 = General
// 1 = Oscillators
// 2 = ADSRs
// 3 = Delay
// 4 = Filter
// 5 = Routing
// 6 = System (Flash,Presets,communication parameters)
// USER LEDs are handled in servSystem() !!
// ***************************************************************************
{
uint32_t keyIn, newPanel, cursorPos;

//handling of PLUS and PLAY state (encoder button and LEDs)
//Encoder Button toggles additional settings and LED basic color
	if ((GPIOC->IDR & KEYPLUS) != KEYPLUS) { //=0 when key pressed
		if (gpLockPlus == 0) {
			gpLockPlus = 1; 					//debounce
			gTIMEOUT = SYSFREQ;					//1sec
			do  {
				servSystem();
				if (gTIMEOUT == 0) {
//Encoder button long time pressed -->init PLAY or re-init previous theme
					if ((gpConfigPanel & 0x00000080) == 0)  {
						gpConfigPanel |= 0x00000080;
						gpModPlay = 0;			//always restart new
						gpPosPlay = 0;
						gpPlusPlay = 0;			//always start basically
						gpPlusState = 0;
						initPlay();
					}
					else  {
						gpConfigPanel &= 0x0000007F;
						gpPlusState = gpPlus[gpConfigPanel];
						if (gpConfigPanel == 0) initGeneral();
						if (gpConfigPanel == 1) {
							if (gpOscSelect > 1)  {
								gpPlusState  = 0; //not if Lfo
								gpPlus[gpConfigPanel] = 0;
							}
							initOsc();
						}
						if (gpConfigPanel == 2) initAdsr();
						if (gpConfigPanel == 3) initDelay();
						if (gpConfigPanel == 4) initFilter();
						if (gpConfigPanel == 5) initRoute();
						if (gpConfigPanel == 6) initSystem();
					}
					while ((GPIOC->IDR & KEYPLUS)!=KEYPLUS)  servSystem();
					return;								//theme changed
				}
			}  while ((GPIOC->IDR & KEYPLUS)!=KEYPLUS);//=0 when key pressed

//early Encoder button release-->change PLUS state
//lock this state
			if ((gpConfigPanel== 3)||(gpConfigPanel== 4)) {
//PLUS mode not necessary/not provided
				gpPlusState = 0;
				gpLockPlus = 0;
				return;
			}
			if  (gpPlusState == 0) {
				gpPlusState = 1;
			}
			else  {
				gpPlusState = 0;					//unlock gpPlusState
			}
			if (gpConfigPanel & 0x80) gpPlusPlay = gpPlusState;
			else  gpPlus[gpConfigPanel] = gpPlusState;

//re-init state and display of actual theme
			if (gpConfigPanel & 0x80)  initPlay();
			else  {
				if (gpConfigPanel == 0) initGeneral();
				if (gpConfigPanel == 1) {
					if (gpOscSelect > 1) {
						gpPlusState = 0; //ignore if Lfo
						gpPlus[gpConfigPanel] = 0;
					}
					initOsc();
				}
				if (gpConfigPanel == 2) initAdsr();
				if (gpConfigPanel == 5) initRoute();
				if (gpConfigPanel == 6) initSystem();
			}
		}
		return;
	}
	else gpLockPlus = 0;

// ---------------------
//the subsequent functions include control and evaluation of rotary encoders:
    if (gpConfigPanel & 0x80)  {
        controlPlay();
    }
    else {
		gpPlusState = gpPlus[gpConfigPanel];			//reconfirm
        if (gpConfigPanel == 0)  controlGeneral();
        if (gpConfigPanel == 1)  controlOsc();
        if (gpConfigPanel == 2)  controlAdsr();
        if (gpConfigPanel == 3)  controlDelay();
        if (gpConfigPanel == 4)  controlFilter();
        if (gpConfigPanel == 5)  controlRoute();
        if (gpConfigPanel == 6)  controlSystem();
    }
//-----------------------------------------------

	if (keyCheck() == 0)  return;

//new button state, check for mode change
	newPanel = 0;
    switch (gpKeyState)  {
    case KEY_IDLE:						//no key pressed
        gpLockKey_F = 0;				//release buttons from beeing pressed
        gpLockKey_B = 0;
        gpLockKey_U = 0;
        gpLockKey_D = 0;
        return;
    case KEY_U_PRESSED:
        if (gpLockKey_U) return;
        gpLockKey_U = 1;
        if (gpConfigPanel & 0x80)  {	//=PLAY state
            if (gSoundSet < (N_SOUNDS - 1)) gSoundSet++;
            else gSoundSet = 0;
            activateSoundSet();
            cursorPos = gpCursorPos;
            redrawPlayDisplay();
            placeCursor(cursorPos);
        }
        else  {
            gpConfigPanel --;
            if (gpConfigPanel > 0x000000FF) gpConfigPanel = 6; //underflow
            newPanel = 1;
        }
        break;
    case KEY_D_PRESSED:
        if (gpLockKey_D) return;
        gpLockKey_D = 1;
        if (gpConfigPanel & 0x80)  {	//=PLAY state
            if (gSoundSet > 0) gSoundSet--;
            else gSoundSet = N_SOUNDS - 1;
            activateSoundSet();
            cursorPos = gpCursorPos;
            redrawPlayDisplay();
            placeCursor(cursorPos);
        }
        else {
            gpConfigPanel ++;
            if (gpConfigPanel > 6) gpConfigPanel = 0;
            newPanel = 1;
        }
        break;
    }

    if (newPanel)   {							//theme has changed
        newPanel = 0;
        gpPlusState = 0;						//always release PLUS state
		if (gpConfigPanel & 0x00000080) {
			initPlay();
			return;
		}

		gpPlusState = gpPlus[gpConfigPanel];
        switch (gpConfigPanel)  {				//case |=0x80 see above
        case 0:
            initGeneral();
            break;
        case 1:
        	if (gpOscSelect > 1)  {
				gpPlusState = 0; //ignore if Lfo config
				gpPlus[gpConfigPanel] = 0;
        	}
            initOsc();
            break;
        case 2:
            initAdsr();
            break;
        case 3:
            initDelay();
            break;
        case 4:
            initFilter();
            break;
        case 5:
            initRoute();
            break;
        case 6:
            initSystem();
            break;
        }
    }
}


uint32_t keyCheck(void)   // ************************************************
// **************************************************************************
{
uint32_t keyIn;
	//debounce buttons
    keyIn = GPIOC->IDR & (KEY_F |KEY_B |KEY_U |KEY_D);

    if (keyIn != gpKeyTest)  {  //key status changed, test again for debounce
        gpKeyTest = keyIn;
        return 0;
    }
    else if (keyIn == gpKeyState)  return 0;  //short spike
//keyIn != gpKeyState; new stable key status debounced successfully:
    gpKeyState = keyIn;
    return keyIn;
}


void initPlay(void)   // ****************************************************
// **************************************************************************
{
const uint8_t DrumSGRStr[] =   {"D    S   G   EAS"};
const uint8_t DelayStr[] =     {"DL     T    F   "};
const uint8_t FbGlidEAPStr[] = {"Fb    Gl   EAS P"};



    if (gpPlusPlay == 0)  {
		displayString(0, gpVSoundPGStr);
        displayString(0x10,gpVibToneStr);
		redrawPlayDisplay();
		placeCursor(gpPosPlay);
    }
    else  {
		displayString(0, DrumSGRStr);
		displayString(0x10, DelayStr);
		redrawPlayDisplay();
		placeCursor(gpPosXPlay);
    }
}


void redrawPlayDisplay(void)   // *******************************************
// ***************************************************************************
{
	if (gpPlusPlay == 0){
		redrawLine1();
		displayVibrato();
		placeCursor(0x10+7);
		writeLcdData('B');
		displaySigned(gfBass,64);
		placeCursor(0x10+12);
		writeLcdData('T');
		displaySigned(gfTreble,64);
	}
	else  {
		placeCursor(1);
        displaySigned(gdDrumsVol, 40);
		placeCursor(6);
		display99(gSoundSet,5);
		placeCursor(10);
        display99(gfGain,5);
		placeCursor(13);
		if (gfFlexMode == 0) {
			writeLcdData('E');
			writeLcdData('A');
			writeLcdData('S');
		}
		else {
			writeLcdData('F');
			writeLcdData('L');
			writeLcdData('X');
		}
		redrawPlayLine2();
	}
}


void redrawPlayLine2(void)   // **********************************************
// ***************************************************************************
{
	if (gpPlusPlay)  {
		placeCursor(0x10+2);
		displayDelayVol(gDelayLevelFactor);
		placeCursor(0x10+8);
	   display255(((gDelayTime>>8)&0x000000FF)*10+(gDelayTime&0x000000FF)/10,5);
		placeCursor(0x10+13);
		displayDelayVol(gFeedbackFactor);
	}
}


void controlPlay(void)   // **************************************************
// ***************************************************************************
{
uint32_t modPlay, cursorPos, delayTime;

    if (gpPlusState == 0)  {
		modPlay = gpModPlay;
	    if (modPlay == 0)  controlVolume();
		if (modPlay == 1)  controlSounds();
		if (modPlay == 2)  controlMonoPoly();

        if (modPlay == 3) {
            if (gpEncNew == 0) {
                gpEncValue = (int32_t)gsGLIDE;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 99) gpEncValue = 99;
                if (gpEncValue != gpEncOldValue) {
                    gsGLIDE = (uint32_t)gpEncValue;
                    gsGLIDEPreset[gSoundSet] = gsGLIDE;
                    placeCursor(14);
                    display99(gsGLIDE,5);
					placeCursor(13);
                }
            }
        }

        if (modPlay == 4)  controlVibrato();
		if (modPlay == 5)  controlBass(0x10+8);
		if (modPlay == 6)  controlTreble(0x10+13);

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modPlay == 0)  {
				modPlay = 1;
				cursorPos = 5;
			}
			else if (modPlay == 1)  {
				modPlay = 2;
				cursorPos = 9;
			}
			else  if (modPlay == 2){
				modPlay = 3;
				cursorPos = 13;
			}
		   else if (modPlay == 3)  {
				modPlay = 4;
				cursorPos = 0x10;
			}
			else if (modPlay == 4)  {
				modPlay = 5;
				cursorPos = 0x10+7;
			}
			else if (modPlay == 5)  {
				modPlay = 6;
				cursorPos = 0x10+12;
			}
			else if (modPlay == 6)  {
				modPlay = 0;
				cursorPos = 0;
			}
			placeCursor(cursorPos);
			gpModPlay = modPlay;
			gpPosPlay = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modPlay == 0) {
				modPlay = 6;
				cursorPos = 0x10+12;
			}
			else if (modPlay == 1) {
				modPlay = 0;
				cursorPos = 0;
			}
			else if (modPlay == 2) {
				modPlay = 1;
				cursorPos = 5;
			}
			else if (modPlay == 3) {
				modPlay = 2;
				cursorPos = 9;
			}
			else if (modPlay == 4) {
				modPlay = 3;
				cursorPos = 13;
			}
			else if (modPlay == 5) {
				modPlay = 4;
				cursorPos = 0x10;
			}
			else if (modPlay == 6) {
				modPlay = 5;
				cursorPos = 0x10+7;
			}
			placeCursor(cursorPos);
			gpModPlay = modPlay;
			gpPosPlay = cursorPos;
		}
    }

    else  {		//PLUS state
		modPlay = gpModXPlay;

        if (modPlay == 0)  {
            if (gpEncNew == 0) {
                gpEncValue = (int32_t)gdDrumsVol;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 52) gpEncValue = 52;
                if (gpEncValue != gpEncOldValue) {
                    if ((gpEncValue == 40) && (gpEncOldValue != 40)) gpZero = 1;
                    else if ((gpEncValue<= 40)&&(gpEncOldValue >40)) gpZero = 1;
                    else if ((gpEncValue >= 40)&&(gpEncOldValue<40)) gpZero = 1;
                    if (gpZero) {
                        gpEncValue = 40;
                        gpEncOldValue = 40;
                    }
                    gdDrumsVol = (uint32_t)(gpEncValue);
                    gdDrumsVolPreset[gSoundSet] = gdDrumsVol;
                    getAllVolFactors();
                    placeCursor(1);
                    displaySigned(gdDrumsVol, 40);
                    placeCursor(0);
                }
            }
        }

	    if (modPlay == 1)  controlSounds();

		if (modPlay == 2)  {
           if (gpEncNew == 0) {
                gpEncValue = (int32_t)gfGain;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 1) gpEncValue = 1;
                if (gpEncValue > 50) gpEncValue = 50;
                if (gpEncValue != gpEncOldValue) {
                    gfGain = (uint32_t)gpEncValue;
                    gfGainPreset[gSoundSet] = gfGain;
                    placeCursor(10);
                    display99(gfGain,5);
                    placeCursor(9);
                }
            }
        }

		if (modPlay == 3)  {
           if (gpEncNew == 0) {
                gpEncValue = (int32_t)gfFlexMode;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 1) gpEncValue = 1;
                if (gpEncValue != gpEncOldValue) {
                    gfFlexMode = (uint32_t)gpEncValue;
					gfStageMask[gSoundSet] &= 0x000000BF;
					gfStageMask[gSoundSet] |= gfFlexMode << 6;
					updateRoute();				//init filters immediately
					placeCursor(13);
					if (gfFlexMode == 0)  {
						writeLcdData('E');
						writeLcdData('A');
						writeLcdData('S');
					}
					else  {
						writeLcdData('F');
						writeLcdData('L');
						writeLcdData('X');
					}
					placeCursor(13);
                }
            }
		}

        if (modPlay == 4)  {
            if (gpEncNew == 0) {
                gpEncValue = gDelayLevelFactor;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < -127) gpEncValue = -127;
                if (gpEncValue > 127) gpEncValue = 127;
                if (gpEncValue != gpEncOldValue) {
                    if ((gpEncValue == 0) && (gpEncOldValue != 0)) gpZero = 1;
                    else if ((gpEncValue <= 0) && (gpEncOldValue > 0)) gpZero=1;
                    else if ((gpEncValue >= 0) && (gpEncOldValue < 0)) gpZero=1;
                    if (gpZero) {
                        gpEncValue = 0;
                        gpEncOldValue = 0;
                    }
                    gDelayLevelFactor = gpEncValue;
                    if (gDelayLevelFactor < 0) {
                        gDelayLevelPreset[gSoundSet] = 128 - gDelayLevelFactor;
                    }
                    else  {
                        gDelayLevelPreset[gSoundSet] = gDelayLevelFactor;
                    }
                    placeCursor(0x10+2);
                    displayDelayVol(gDelayLevelFactor);
                    placeCursor(0x10+1);
                }
			}
        }

	    if (modPlay == 5)  {
           if (gpEncNew == 0) {
				delayTime = ((gDelayTime>>8)&0x000000FF)*10 +
													(gDelayTime&0x000000FF)/10;
                gpEncValue = (int32_t)delayTime;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 300) gpEncValue = 300;
                if (gpEncValue != gpEncOldValue) {
                    delayTime = (uint32_t)gpEncValue;
					gDelayTime = ((delayTime/10) << 8) | ((delayTime%10)*10);
					gDelayTimePreset[gSoundSet] = gDelayTime;
                    placeCursor(0x10+8);
					display255(delayTime,5);
                    placeCursor(0x10+7);
                }
            }
	    }

		if (modPlay == 6)  {
			if (gpEncNew == 0) {
				gpEncValue = gFeedbackFactor;
				gpEncOldValue = gpEncValue;
			}
			else  {
				gpEncNew = 0;
				if (gpEncValue < -99) gpEncValue = -99; //avoid self-oscillation
				if (gpEncValue > 99) gpEncValue = 99;
				if (gpEncValue != gpEncOldValue) {
					if ((gpEncValue == 0) && (gpEncOldValue !=0 )) gpZero = 1;
					else if ((gpEncValue <= 0)&&(gpEncOldValue > 0)) gpZero = 1;
					else if ((gpEncValue >= 0)&&(gpEncOldValue < 0)) gpZero = 1;
					if (gpZero) {
						gpEncValue = 0;
						gpEncOldValue = 0;
					}
					gFeedbackFactor = gpEncValue;
					if (gFeedbackFactor < 0) {
						gFeedbackPreset[gSoundSet] =  128 - gFeedbackFactor;
					}
					else  {
						gFeedbackPreset[gSoundSet] = gFeedbackFactor;
					}
					placeCursor(0x10+13);
					displayDelayVol(gFeedbackFactor);
					placeCursor(0x10+12);
				}
			}
		}

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modPlay == 0)  {
				modPlay = 1;
				cursorPos = 5;
			}
			else if (modPlay == 1)  {
				modPlay = 2;
				cursorPos = 9;
			}
			else  if (modPlay == 2){
				modPlay = 3;
				cursorPos = 13;
			}
		   else if (modPlay == 3)  {
				modPlay = 4;
				cursorPos = 0x10+1;
			}
			else if (modPlay == 4)  {
				modPlay = 5;
				cursorPos = 0x10+7;
			}
			else if (modPlay == 5)  {
				modPlay = 6;
				cursorPos = 0x10+12;
			}
			else if (modPlay == 6)  {
				modPlay = 0;
				cursorPos = 0;
			}
			placeCursor(cursorPos);
			gpModXPlay = modPlay;
			gpPosXPlay = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modPlay == 0) {
				modPlay = 6;
				cursorPos = 0x10+12;
			}
			else if (modPlay == 1) {
				modPlay = 0;
				cursorPos = 0;
			}
			else if (modPlay == 2) {
				modPlay = 1;
				cursorPos = 5;
			}
			else if (modPlay == 3) {
				modPlay = 2;
				cursorPos = 9;
			}
			else if (modPlay == 4) {
				modPlay = 3;
				cursorPos = 13;
			}
			else if (modPlay == 5) {
				modPlay = 4;
				cursorPos = 0x10+1;
			}
			else if (modPlay == 6) {
				modPlay = 5;
				cursorPos = 0x10+7;
			}
			placeCursor(cursorPos);
			gpModXPlay = modPlay;
			gpPosXPlay = cursorPos;
		}
    }
    if (gpZero) {
        msDelay(500);
        gpZero = 0;
    }
}


void redrawLine1(void)   // **********************************************
// ***************************************************************************
{
    placeCursor(1);
    displaySigned(gVolume, 46);
    placeCursor(6);
    display99(gSoundSet,5);
    placeCursor(14);
	display99(gsGLIDE,5);

}


void controlSounds(void)   // ************************************************
// ***************************************************************************
{
	if (gpEncNew == 0) {
		gpEncValue = (int32_t)gSoundSet;
           gpEncOldValue = gpEncValue;
	}
	else  {
		gpEncNew = 0;

		if (gpEncValue < 0) gpEncValue = N_SOUNDS-1;
		if (gpEncValue > N_SOUNDS-1) gpEncValue = 0;
		if (gpEncValue != gpEncOldValue) {
			if ((gpEncValue == 0)&&(gpEncOldValue != 0)) gpZero = 1;
			else if ((gpEncValue <=0)&&(gpEncOldValue >0)) gpZero=1;
			else if ((gpEncValue >=0 )&&(gpEncOldValue<0)) gpZero=1;
			if (gpZero) {
			 gpEncValue = 0;
				gpEncOldValue = 0;
			}
			gSoundSet = (uint32_t)gpEncValue;
			placeCursor(6);
			display99(gSoundSet,5);
			activateSoundSet();
			placeCursor(5);
		}
	}
}


void controlVolume(void)   // ************************************************
// ***************************************************************************
{
	if (gpEncNew == 0) {
		gpEncValue = (int32_t)(gVolume);
		gpEncOldValue = gpEncValue;
	}
	else  {
		gpEncNew = 0;
		if (gpEncValue < 0) gpEncValue = 0;
		if (gpEncValue > 52) gpEncValue = 52;
		if (gpEncValue != gpEncOldValue) {
			if ((gpEncValue== 46)&&(gpEncOldValue!=46)) gpZero = 1;
			else if ((gpEncValue <= 46)&&(gpEncOldValue > 46)) gpZero = 1;
			else if ((gpEncValue >= 46)&&(gpEncOldValue < 46)) gpZero = 1;
			if (gpZero) {
				gpEncValue = 46;
				gpEncOldValue = 46;
			}
			gVolume = (uint32_t)(gpEncValue);
			gVolumePreset[gSoundSet] = gVolume;
			gVolumeFactor = gDbTable[gVolume] >> 3;	//max.500 allowed;
			placeCursor(1);
			displaySigned(gVolume, 46);
			getAllVolFactors();
			placeCursor(0);
		}
	}
}


void controlBass(uint32_t position)   // *************************************
// ***************************************************************************
{
	if (gpEncNew == 0) {
		gpEncValue = (int32_t)gfBass;
		gpEncOldValue = gpEncValue;
	}
	else  {
		gpEncNew = 0;
		if (gpEncValue < 0) gpEncValue = 0;
		if (gpEncValue > 127) gpEncValue = 127;
		if (gpEncValue != gpEncOldValue) {
			if ((gpEncValue == 64)&&(gpEncOldValue != 64)) gpZero = 1;
			else if ((gpEncValue <= 64)&&(gpEncOldValue > 64)) gpZero=1;
			else if ((gpEncValue >= 64 )&&(gpEncOldValue < 64)) gpZero=1;
			if (gpZero) {
			 gpEncValue = 64;
				gpEncOldValue = 64;
			}
			gfBass = (uint32_t)gpEncValue;
			packBasTreb(); //transfer gfBass correctly into gfBasTrebPreset[]
			placeCursor(position);
			displaySigned(gfBass,64);
			adjPeak3(gfBass);
			placeCursor(position);
		}
	}
}


void controlTreble(uint32_t position)   // ***********************************
// ***************************************************************************
{
	if (gpEncNew == 0) {
		gpEncValue = (int32_t)gfTreble;
		gpEncOldValue = gpEncValue;
	}
	else  {
		gpEncNew = 0;
		if (gpEncValue < 0) gpEncValue = 0;
		if (gpEncValue > 127) gpEncValue = 127;
		if (gpEncValue != gpEncOldValue) {
			if ((gpEncValue == 64)&&(gpEncOldValue != 64)) gpZero = 1;
			else if ((gpEncValue <= 64)&&(gpEncOldValue > 64)) gpZero=1;
			else if ((gpEncValue >= 64 )&&(gpEncOldValue < 64)) gpZero=1;
			if (gpZero) {
			 gpEncValue = 64;
				gpEncOldValue = 64;
			}
			gfTreble = (uint32_t)gpEncValue;
			packBasTreb(); //transfer gfBass correctly into gfBasTrebPreset[]
			placeCursor(position);
			displaySigned(gfTreble,64);
			adjPeak4(gfTreble);
			placeCursor(position);
		}
	}
}


void controlMonoPoly(void)   // **********************************************
// ***************************************************************************
{
	if (gpEncNew == 0) {
		gpEncValue = (int32_t) gsMonoPoly;
		gpEncOldValue = gpEncValue;
	}
	else  {
		gpEncNew = 0;
		if (gpEncValue < 0) gpEncValue = 0;
		if (gpEncValue > 1) gpEncValue = 1;
		if (gpEncValue != gpEncOldValue) {
			gsMonoPoly = (uint32_t)gpEncValue;				//1=Mono, 0= Poly
            gfStageMask[gSoundSet] &= 0x000000DF;
            gfStageMask[gSoundSet] |= gsMonoPoly << 5; //bit5 doubles gsMonoPoly
			placeCursor(9);
			if (gsMonoPoly)  {
				writeLcdData('M');
				writeLcdData('o');
			}
			else  {
				writeLcdData('P');
				writeLcdData('y');
			}
			placeCursor(9);
		}
	}
}


void controlVibrato(void)   // ***********************************************
// ***************************************************************************
{
	if (gpEncNew == 0) {
			gpEncValue = (int32_t)gVibrato + 128;
			if (gVibratoLfo1) gpEncValue = 256 - gpEncValue;
		gpEncOldValue = gpEncValue;
	}
	else  {
		gpEncNew = 0;
		if (gpEncValue < 28) gpEncValue = 28;
		if (gpEncValue > 228) gpEncValue = 228;
		if (gpEncValue != gpEncOldValue) {
			if ((gpEncValue== 128)&&(gpEncOldValue!=128)) gpZero = 1;
			else if ((gpEncValue<=128)&&(gpEncOldValue >128)) gpZero =1;
			else if ((gpEncValue>=128)&&(gpEncOldValue <128)) gpZero =1;
			if (gpZero) {
				gpEncValue = 128;
				gpEncOldValue = 128;
			}
			if (gpEncValue < 128) {
				gVibratoLfo1 = 1;
				gVibrato = (uint32_t)(128 - gpEncValue) & 0x7F;
				gVibratoPreset[gSoundSet] = gVibrato | 0x80;
			}
			else  {
				gVibratoLfo1 = 0;
				gVibrato= (uint32_t)gpEncValue - 128;
				gVibratoPreset[gSoundSet] = gVibrato;
			}
			displayVibrato();
		}
	}
}


void displayVibrato(void)   // ***********************************************
// ***************************************************************************
{
		placeCursor(0x10);
		writeLcdData('V');
		if (gVibrato == 0)  {
			writeLcdData('i');
			writeLcdData('b');
		}
		else  {
			writeLcdData('L');
		if (gVibratoLfo1) writeLcdData('1');
			else writeLcdData('3');
		}
		writeLcdData(':');
		displayNL99(gVibrato, 0);
		placeCursor(0x10+3);
}


void initGeneral(void)   // **************************************************
// ***************************************************************************
{
const uint8_t DrumLevRStr[]= {"Ds:     L    EAS"};
const uint8_t F0MGFtStr[] =  {"F0M2:   G   F0T "};

    if (gpPlusState == 0)  {
        displayString(0, gpVSoundPGStr);
        displayString(0x10,gpVibToneStr);
		redrawGeneral();
		placeCursor(gpPosGeneral);
    }
    else  {
        displayString(0, DrumLevRStr);
        displayString(0x10,F0MGFtStr);
		redrawGeneral();
		placeCursor(gpPosXGeneral);
    }
}


void redrawGeneral(void)   // ************************************************
// ***************************************************************************
{
uint32_t k, n;

    if (gpPlusState == 0)  redrawLine1();
    else  {
		displayString(3, gpDrumTypeStr[gpDrumType]);
        placeCursor(9);
        switch (gpDrumType)  {
            case 1:
                displaySigned(gdKickVol, 40);
                break;
            case 2:
                displaySigned(gdHiTomVol, 40);
                break;
            case 3:
                displaySigned(gdSnareVol, 40);
                break;
            case 4:
                displaySigned(gdRimVol, 40);
                break;
            case 5:
                displaySigned(gdClapVol, 40);
                break;
            case 6:
                displaySigned(gdHihatVol, 40);
                break;
            case 7:
                displaySigned(gdCymbalVol, 40);
                break;
            case 8:
                displaySigned(gdBellVol, 40);
                break;
            case 0:
            default:                                 //should be impossible
                displaySigned(gdDrumsVol, 40);
                break;
        }
		placeCursor(13);
		if (gfFlexMode == 0) {
			writeLcdData('E');
			writeLcdData('A');
			writeLcdData('S');
		}
		else  {
			writeLcdData('F');
			writeLcdData('L');
			writeLcdData('X');
		}
    }
    redrawGenLine2();
}


void redrawGenLine2(void)   // **********************************************
// **************************************************************************
{
int32_t test;

	if (gpPlusState == 0)  {
		displayVibrato();
		placeCursor(0x10+7);
		writeLcdData('B');
		displaySigned(gfBass,64);
		placeCursor(0x10+12);
		writeLcdData('T');
		displaySigned(gfTreble,64);
	}
	else  {
		placeCursor(0x10+3);
		displayHexNibble(gfF0modLfo2 + 1);
		writeLcdData(':');
		displayNL99(gfF0mod, 0);
		placeCursor(0x10+9);
		display99(gfGain,5);
		test = gaFilterTest;
		placeCursor(0x10+13);
		if (test >= 0)  writeLcdData('0');
		else  {
			test = -test;
		writeLcdData('1');
		}
		placeCursor(0x10+15);
		displayHexNibble(test);
	}
}


void controlGeneral(void)   // ***********************************************
// ***************************************************************************
{
uint32_t k,n;
uint32_t modGeneral, cursorPos;

    if (gpPlusState == 0) {
		modGeneral = gpModGeneral;
        if (modGeneral == 0)  controlVolume();
        if (modGeneral == 1)  controlSounds();
        if (modGeneral == 2)  controlMonoPoly();

		if (modGeneral == 3)  {
           if (gpEncNew == 0) {
                gpEncValue = (int32_t)gsGLIDE;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 99) gpEncValue = 99;
                if (gpEncValue != gpEncOldValue) {
                    gsGLIDE = (uint32_t)gpEncValue;
                    gsGLIDEPreset[gSoundSet] = gsGLIDE;
                    placeCursor(14);
					display99(gsGLIDE,5);
                    placeCursor(13);
                }
            }
        }

		if (modGeneral == 4)  controlVibrato();
        if (modGeneral == 5)  controlBass(0x10+8);
        if (modGeneral == 6)  controlTreble(0x10+13);

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modGeneral == 0)  {
				modGeneral = 1;
				cursorPos = 5;
			}
			else if (modGeneral == 1)  {
				modGeneral = 2;
				cursorPos = 9;
			}
			else if (modGeneral == 2)  {
				modGeneral = 3;
				cursorPos = 13;
			}
			else if (modGeneral == 3)  {
				modGeneral = 4;
				cursorPos = 0x10+3;
			}
			else if (modGeneral == 4)  {
				modGeneral = 5;
				cursorPos = 0x10+7;
			}
			else if (modGeneral == 5)  {
				modGeneral = 6;
				cursorPos = 0x10+12;
			}
			else if (modGeneral == 6)  {
				modGeneral = 0;
				cursorPos = 0;
			}
			placeCursor(cursorPos);
			gpModGeneral = modGeneral;
			gpPosGeneral = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modGeneral == 0)  {
				modGeneral = 6;
				cursorPos = 0x10+12;
			}
			else if (modGeneral == 1)  {
				modGeneral = 0;
				cursorPos = 0;
			}
			else if (modGeneral == 2)  {
				modGeneral = 1;
				cursorPos = 5;
			}
			else  if (modGeneral == 3)  {
				modGeneral = 2;
				cursorPos = 9;
			}
			else if (modGeneral == 4)  {
				modGeneral = 3;
				cursorPos = 13;
			}
			else  if (modGeneral == 5)  {
				modGeneral = 4;
				cursorPos = 0x10+3;
			}
			else  if (modGeneral == 6)  {
				modGeneral = 5;
				cursorPos = 0x10+7;
			}
			placeCursor(cursorPos);
			gpModGeneral = modGeneral;
			gpPosGeneral = cursorPos;
		}

		if (gpZero) {
			msDelay(500);
			gpZero = 0;
		}

    }

    else  {  //PLUS state
		modGeneral = gpModXGeneral;
		if (modGeneral == 0)  {
            if (gpEncNew == 0) {
            gpEncValue = (int32_t)gpDrumType;
            gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 8) gpEncValue = 8;
                if (gpEncValue != gpEncOldValue) {
                    gpDrumType = (uint32_t)gpEncValue;
                    redrawGeneral();
                    placeCursor(2);
                }
            }
		}
        if (modGeneral == 1)  {
            if (gpEncNew == 0) {
				switch (gpDrumType)  {
					case 1:
						gpEncValue = (int32_t)gdKickVol;
						break;
					case 2:
						gpEncValue = (int32_t)gdHiTomVol;
						break;
					case 3:
						gpEncValue = (int32_t)gdSnareVol;
						break;
					case 4:
						gpEncValue = (int32_t)gdRimVol;
						break;
					case 5:
						gpEncValue = (int32_t)gdClapVol;
						break;
					case 6:
						gpEncValue = (int32_t)gdHihatVol;
						break;
					case 7:
						gpEncValue = (int32_t)gdCymbalVol;
						break;
					case 8:
						gpEncValue = (int32_t)gdBellVol;
						break;
					case 0:
					default:   							//should be impossible
						gpEncValue = (int32_t)gdDrumsVol;
						break;
				}
            gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 52) gpEncValue = 52;
                if (gpEncValue != gpEncOldValue) {
                    if ((gpEncValue == 40) && (gpEncOldValue != 40)) gpZero = 1;
                    else if ((gpEncValue<= 40)&&(gpEncOldValue > 40)) gpZero =1;
                    else if ((gpEncValue >= 40)&&(gpEncOldValue < 40)) gpZero=1;
                    if (gpZero) {
                        gpEncValue = 40;
                        gpEncOldValue = 40;
                    }
                    switch (gpDrumType)  {
                        case 1:
                        gdKickVol = (uint32_t)(gpEncValue);
                        gdKickVolPreset[gSoundSet] = gdKickVol;
                            gdKickVolFactor = getVolFactor(gdKickVol);
                            break;
                        case 2:
                            gdHiTomVol = (uint32_t)(gpEncValue);
                            gdHiTomVolPreset[gSoundSet] = gdHiTomVol;
                            gdHiTomVolFactor = getVolFactor(gdHiTomVol);
                            break;
                        case 3:
                            gdSnareVol = (uint32_t)(gpEncValue);
                            gdSnareVolPreset[gSoundSet] = gdSnareVol;
                            gdSnareVolFactor = getVolFactor(gdSnareVol);
                            break;
                        case 4:
                            gdRimVol = (uint32_t)(gpEncValue);
                            gdRimVolPreset[gSoundSet] = gdRimVol;
                            gdRimVolFactor = getVolFactor(gdRimVol);
                            break;
                        case 5:
                            gdClapVol = (uint32_t)(gpEncValue);
                            gdClapVolPreset[gSoundSet] = gdClapVol;
                            gdClapVolFactor = getVolFactor(gdClapVol);
                            break;
                        case 6:
                            gdHihatVol = (uint32_t)(gpEncValue);
                            gdHihatVolPreset[gSoundSet] = gdHihatVol;
                            gdHihatVolFactor = getVolFactor(gdHihatVol);
                            break;
                        case 7:
                            gdCymbalVol = (uint32_t)(gpEncValue);
                            gdCymbalVolPreset[gSoundSet] = gdCymbalVol;
                            gdCymbalVolFactor = getVolFactor(gdCymbalVol);
                            break;
                        case 8:
                            gdBellVol = (uint32_t)(gpEncValue);
                            gdBellVolPreset[gSoundSet] = gdBellVol;
                            gdBellVolFactor = getVolFactor(gdBellVol);
                            break;
                        default:                        //should be impossible
                        case 0:
                            gdDrumsVol = (uint32_t)(gpEncValue);
                            gdDrumsVolPreset[gSoundSet] = gdDrumsVol;
                            getAllVolFactors();
                            break;
                    }
                }
                redrawGeneral();
                placeCursor(8);
            }
        }

        if (modGeneral == 2)  {
           if (gpEncNew == 0) {
                gpEncValue = (int32_t)gfFlexMode;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 1) gpEncValue = 1;
                if (gpEncValue != gpEncOldValue) {
                    gfFlexMode = (uint32_t)gpEncValue;
					gfStageMask[gSoundSet] &= 0x000000BF;
					gfStageMask[gSoundSet] |= gfFlexMode << 6;
					updateRoute();				//init filters immediately
					placeCursor(13);
					if (gfFlexMode == 0) {
						writeLcdData('E');
						writeLcdData('A');
						writeLcdData('S');
					}
					else  {
						writeLcdData('F');
						writeLcdData('L');
						writeLcdData('X');
					}
					placeCursor(13);
                }
            }
        }

		if (modGeneral == 3)  {
			if (gpEncNew == 0) {
				n = (int32_t)gfF0modPreset[gSoundSet];
				k = gfF0modLfo2;
				if (k) gpEncValue = n;
				else  gpEncValue = (int32_t)(128 - (n & 0x7F));
				gpEncOldValue = gpEncValue;
			}
			else  {
				gpEncNew = 0;
				if (gpEncValue < 29) gpEncValue = 29; 			//128-99
				if (gpEncValue > 227) gpEncValue = 227;			//128+99
				if (gpEncValue != gpEncOldValue) {
					if ((gpEncValue==128)&&(gpEncOldValue!=128)) gpZero = 1;
					else if((gpEncValue<=128)&&(gpEncOldValue>128))gpZero=1;
					else if((gpEncValue>=128)&&(gpEncOldValue<128))gpZero=1;
					if (gpZero) {
						gpEncValue = 128;
						gpEncOldValue = 128;
					}
					k = ((uint32_t)gpEncValue >> 7) & 1;  //gpEncValue>=128
						gfF0modLfo2 = k;
						if (gfF0modLfo2) {
							gfF0mod = (uint32_t)gpEncValue & 0x7F;
							gfF0modPreset[gSoundSet]= gfF0mod | 0x80;
						}
						else  {
							gfF0mod = 128 - (uint32_t)gpEncValue;
							gfF0modPreset[gSoundSet]= gfF0mod;
						}
						redrawGenLine2();
						placeCursor(0x10);
				}
			}
		}

		if (modGeneral == 4)  {
			if (gpEncNew == 0) {
                gpEncValue = (int32_t)gfGain;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 1) gpEncValue = 1;
                if (gpEncValue > 50) gpEncValue = 50;
                if (gpEncValue != gpEncOldValue) {
                    gfGain = (uint32_t)gpEncValue;
                    gfGainPreset[gSoundSet] = gfGain;
                    placeCursor(0x10+9);
                    display99(gfGain,5);
                    placeCursor(0x10+8);
                }
            }
		}

        if (modGeneral == 5)  {
           if (gpEncNew == 0) {
                gpEncValue = gaFilterTest + 9;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
				if (gpEncValue < 0) gpEncValue = 0;
				if (gpEncValue > 18) gpEncValue = 18;
				if (gpEncValue != gpEncOldValue) {
					if ((gpEncValue == 9) && (gpEncOldValue != 9)) gpZero = 1;
					else if ((gpEncValue <=9 ) && (gpEncOldValue > 9)) gpZero=1;
					else if ((gpEncValue >= 9) && (gpEncOldValue < 9)) gpZero=1;
					if (gpZero) {
						gpEncValue = 9;
						gpEncOldValue = 9;
					}
					gpEncValue -= 9;
                    gaFilterTest = gpEncValue;
					if (gpEncValue < 0) gpEncValue = -gpEncValue;
					gaFTestFactor = gDbTable[gpEncValue*3];
                    redrawGenLine2();
                    placeCursor(0x10+14);
                }
            }
        }

		if (gpZero) {
			msDelay(500);
			gpZero = 0;
		}

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modGeneral == 0)  {
				modGeneral = 1;
				cursorPos = 8;
			}
			else if (modGeneral == 1)  {
				modGeneral = 2;
				cursorPos = 13;
			}
			else if (modGeneral == 2)  {
				modGeneral = 3;
				cursorPos = 0x10+4;
			}
			else if (modGeneral == 3)  {
				modGeneral = 4;
				cursorPos = 0x10+8;
			}
			else if (modGeneral == 4)  {
				modGeneral = 5;
				cursorPos = 0x10+14;
				}
			else if (modGeneral == 5)  {
				modGeneral = 0;
				cursorPos = 2;
			}
			placeCursor(cursorPos);
			gpModXGeneral = modGeneral;
			gpPosXGeneral = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modGeneral == 0)  {
				modGeneral = 5;
				cursorPos = 0x10+14;
			}
			else if (modGeneral == 1)  {
				modGeneral = 0;
				cursorPos = 2;
			}
			else if (modGeneral == 2)  {
				modGeneral = 1;
				cursorPos = 8;
			}
			else  if (modGeneral == 3)  {
				modGeneral = 2;
				cursorPos = 13;
			}
			else if (modGeneral == 4)  {
				modGeneral = 3;
				cursorPos = 0x10+4;
			}
			else  if (modGeneral == 5)  {
				modGeneral = 4;
				cursorPos = 0x10+8;
			}
			placeCursor(cursorPos);
			gpModXGeneral = modGeneral;
			gpPosXGeneral = cursorPos;
		}
    }
}


void initOsc(void)   // ******************************************************
// ***************************************************************************
{

const uint8_t Osc1Str[] =   {"Osc1 Tr    Dt   "};
const uint8_t Osc2Str[] =   {"Osc2 Tx    Dx   "};
const uint8_t LfoNFStr[] =  {"Lfo1 Freq:    Hz"};
const uint8_t LfoWSStr[] =  {"      Slp  Sym  "};
const uint8_t FreeSqStr[] = {"FS1Hi  .  SL  . "};

    if (gpPlusState == 0) {
		if (gpOscSelect == 0)  {
			displayString(0,Osc1Str);
			displayString(0x10,gpEmptyStr);
		}
		else if (gpOscSelect == 1)  {
			displayString(0,Osc2Str);
			displayString(0x10,gpEmptyStr);
		}
		else  {
			displayString(0,LfoNFStr);
			displayString(0x10,LfoWSStr);
		}
		redrawOscDisplay();
		placeCursor(gpPosOsc);
    }
    else  {
		displayString(0,FreeSqStr);
		displayString(0x10,FreeSqStr);
		placeCursor(0x10+2);
		writeLcdData('2');
		redrawOscDisplay();
		placeCursor(5);
    }
}


void redrawOscDisplay(void)   // *********************************************
// ***************************************************************************
{
uint32_t temp;
//no matter if PLUS state or not. Strings depend heavily on selection
	if (gpPlusState == 0)  {
		if (gpOscSelect == 0)  {
			placeCursor(7);
			displaySigned(gsTRANS, 36);
			placeCursor(13);
			displaySigned(gsDETUNEPreset[gSoundSet], 64);
			redrawOscMod();
			displayString(0x10+11,gpWaveStr[gsWAVEFORM_1]);
		}
		else if (gpOscSelect == 1)  {
			placeCursor(7);
			displaySigned(gsTRANS_2, 36);
			placeCursor(13);
			displaySigned(gsDETUNE_2Preset[gSoundSet], 64);
			redrawOscMod();
			displayString(0x10+11,gpWaveStr[gsWAVEFORM_2]);
		}
		else  {
			placeCursor(3);
			writeLcdData(gpOscSelect + 0x2F);
			placeCursor(10);
			if (gpOscSelect == 2)  {
				display255(gLfo1Freq, 1);
				temp = gLfo1Wave & 0x000000FF;
				if (temp == 0)  displayString(0x10,gpWaveStr[0]);
				else if (temp < 0x80)  displayString(0x10,gpWaveStr[1]);
				else  displayString(0x10,gpSquarStr);
				placeCursor(0x10+9);
				if (gLfo1Slope) writeLcdData('+');
				else writeLcdData('-');
				placeCursor(0x10+14);
				if (temp == 0) {
					writeLcdData('-');
					writeLcdData('-');
				}
				else displayNL99(gLfo1Wave & 0x7F,0);
			}
			else if (gpOscSelect == 3)  {
				display255(gLfo2Freq, 1);
				temp = gLfo2Wave & 0x000000FF;
				if (temp == 0) displayString(0x10,gpWaveStr[0]);
				else if (temp < 0x80)  displayString(0x10,gpWaveStr[1]);
				else  displayString(0x10,gpSquarStr);
				placeCursor(0x10+9);
				if (gLfo2Slope) writeLcdData('+');
				else writeLcdData('-');
				placeCursor(0x10+14);
				if (temp == 0) {
					writeLcdData('-');
					writeLcdData('-');
				}
				else displayNL99(gLfo2Wave & 0x7F,0);
			}
			else  {			//gpOscSelect == 4
				display255(gLfo3Freq, 1);
				temp = gLfo3Wave & 0x000000FF;
				if (temp == 0)  displayString(0x10,gpWaveStr[0]);
				else if (temp < 0x80) displayString(0x10,gpWaveStr[1]);
				else  displayString(0x10,gpSquarStr);
				placeCursor(0x10+9);
				if (gLfo3Slope) writeLcdData('+');
				else writeLcdData('-');
				placeCursor(0x10+14);
				if (temp == 0) {
					writeLcdData('-');
					writeLcdData('-');
				}
				else displayNL99(gLfo3Wave & 0x7F,0);
			}
		}
    }
    else  {
		placeCursor(5);
		display99((gsFreeSquare1[gSoundSet] >> 24) & 0x000000FF, 0);
		placeCursor(8);
		displayHexNibble((gsFreeSquare1[gSoundSet] >> 16) & 0x000000FF);
		temp = (gsFreeSquare1[gSoundSet] & 0x000000FF) * 2;
		placeCursor(12);
		display99(temp / 10, 0);
		placeCursor(15);
		displayHexNibble(temp % 10);
		placeCursor(0x10+5);
		display99((gsFreeSquare2[gSoundSet] >> 24) & 0x000000FF, 0);
		placeCursor(0x10+8);
		displayHexNibble((gsFreeSquare2[gSoundSet] >> 16) & 0x000000FF);
		temp = (gsFreeSquare2[gSoundSet] & 0x000000FF) * 2;
		placeCursor(0x10+12);
		display99(temp / 10, 0);
		placeCursor(0x10+15);
		displayHexNibble(temp % 10);
    }
}


void redrawOscMod(void)   // *************************************************
// ***************************************************************************
{
const uint8_t LStr[] =   {"  L  || "};
const uint8_t ModStr[] = {"Mod"};
const uint8_t PwmStr[] = {"Pwm"};

	displayString(0x10+3,LStr);
    if (gpOscSelect == 0)   {
		if (gsOsc1PWM)  displayString(0x10,PwmStr);
		else  displayString(0x10,ModStr);
        placeCursor(0x10+3);
		display99(gsMOD_1,5);
        placeCursor(0x10+6);
        if (gsOsc1ModLfo2 == 0) writeLcdData('1');
        else writeLcdData('2');
		if (gaRoute0N > 7)  {
			placeCursor(0x10+8);
			writeLcdData('F');
			writeLcdData('M');
		}
		displayString(0x10+11,gpWaveStr[gsWAVEFORM_1]);
    }
    if (gpOscSelect == 1)   {
        if (gsOsc2PWM)  displayString(0x10,PwmStr);
        else  displayString(0x10,ModStr);
        placeCursor(0x10+3);
		display99(gsMOD_2,5);
        placeCursor(0x10+6);
        if (gsOsc2ModLfo2 == 0) writeLcdData('1');
        else writeLcdData('2');
		if (gsO2Sync)  {
			placeCursor(0x10+8);
			writeLcdData('S');
			writeLcdData('Y');
		}
		displayString(0x10+11,gpWaveStr[gsWAVEFORM_2]);
    }
}


void controlOsc(void)   // **************************************************
// **************************************************************************
{
uint32_t modOsc, cursorPos;
uint32_t lfoType, lfoSymm, tempH, tempL, tempX;
// no matter if Plus state or not

	if (gpPlusState == 0)  {
		modOsc = gpModOsc;

		if (modOsc == 0)  {
			if (gpEncNew == 0) {
				gpEncValue = (int32_t)gpOscSelect;
				gpEncOldValue = gpEncValue;
			}
			else  {
				gpEncNew = 0;
				if (gpEncValue < 0) gpEncValue = 0;
				if (gpEncValue > 4) gpEncValue = 4;
				if (gpEncValue != gpEncOldValue) {
					gpOscSelect = (uint32_t)gpEncValue;
					initOsc();
					gpPosOsc = 2;
					return;
				}
			}
		}

		else if (modOsc == 1)  {
			if (gpOscSelect < 2)  {
				if (gpEncNew == 0) {
					if (gpOscSelect == 0) gpEncValue = gsTRANS;
					else gpEncValue = gsTRANS_2;
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 0) gpEncValue = 0;
					if (gpEncValue > 72) gpEncValue = 72;
					if (gpEncValue != gpEncOldValue) {
						if ((gpEncValue == 36) && (gpEncOldValue != 36)) gpZero = 1;
						else if ((gpEncValue <= 36)&&(gpEncOldValue > 36)) gpZero = 1;
						else if ((gpEncValue >= 36)&&(gpEncOldValue < 36)) gpZero = 1;
						if (gpZero) {
							gpEncValue = 36;
							gpEncOldValue = 36;
						}
						if (gpOscSelect == 0)  {
							gsTRANS = gpEncValue;
							gsTRANSPreset[gSoundSet] = gsTRANS;
						}
						else  {
							gsTRANS_2 = gpEncValue;
							gsTRANS_2Preset[gSoundSet] = gsTRANS_2;
						}
						placeCursor(7);
						displaySigned((uint32_t)gpEncValue,36);
						placeCursor(6);
					}
				}
			}
			else  {			//LFO Freq
				if (gpEncNew == 0) {
					if (gpOscSelect==2) gpEncValue=(int32_t)gLfo1FreqPreset[gSoundSet];
					else if(gpOscSelect==3) gpEncValue=(int32_t)gLfo2FreqPreset[gSoundSet];
					else gpEncValue = (int32_t)gLfo3FreqPreset[gSoundSet];
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 0) gpEncValue = 0;
					if (gpEncValue > 127) gpEncValue = 127;
					if (gpEncValue != gpEncOldValue) {
						placeCursor(10);
						if (gpOscSelect == 2) {
							gLfo1Freq = (uint32_t)gpEncValue;
							gLfo1FreqPreset[gSoundSet] = gLfo1Freq;
							if (gLfo1Freq > 5)  gLfo1Freq = (gLfo1Freq - 3) * 2;
							display255(gLfo1Freq, 1);
						}
						else if (gpOscSelect == 3) {
							gLfo2Freq= (uint32_t)gpEncValue;
							gLfo2FreqPreset[gSoundSet] = gLfo2Freq;
							if (gLfo2Freq > 5) gLfo2Freq = (gLfo2Freq - 3) * 2;
							display255(gLfo2Freq, 1);
						}
						else  {
							gLfo3Freq = (uint32_t)gpEncValue;
							gLfo3FreqPreset[gSoundSet] = gLfo3Freq;
							if (gLfo3Freq > 5)  gLfo3Freq = (gLfo3Freq - 3) * 2;
							display255(gLfo3Freq, 1);
						}
						placeCursor(9);
					}
				}
			}
		}

		if ((modOsc==2)&&(gpOscSelect<=1)) { //accessible for OSC1 and OSC2 only
			if (gpEncNew == 0) {
				if (gpOscSelect == 0) gpEncValue = gsDETUNEPreset[gSoundSet];
				else gpEncValue = gsDETUNE_2Preset[gSoundSet];
				gpEncOldValue = gpEncValue;
			}
			else  {
				gpEncNew = 0;
				if (gpEncValue < 0) gpEncValue = 0;
				if (gpEncValue > 127) gpEncValue = 127;
				if (gpEncValue != gpEncOldValue) {
					if ((gpEncValue == 64) && (gpEncOldValue != 64)) gpZero = 1;
					else if ((gpEncValue <= 64)&&(gpEncOldValue > 64)) gpZero=1;
					else if ((gpEncValue>= 64)&&(gpEncOldValue < 64)) gpZero=1;
					if (gpZero) {
						gpEncValue = 64;
						gpEncOldValue = 64;
					}
					if (gpOscSelect == 0)  {
						gsDETUNEPreset[gSoundSet] = gpEncValue;
						gsDETUNE = gsDETUNEPreset[gSoundSet] - 64;
					}
					else  {
						gsDETUNE_2Preset[gSoundSet] = gpEncValue;
						gsDETUNE_2 = gsDETUNE_2Preset[gSoundSet] - 64;
					}
					placeCursor(13);
					displaySigned(gpEncValue, 64);
					placeCursor(12);
				}
			}
		}

		if (modOsc == 3)  {						//Osc:Modulation, Lfo:Waveform
			if (gpOscSelect == 0)  {
				if (gfRuntimeMask & 0x00000200)  { 	//wave type is square wave
					if (gpEncNew == 0) {
						gpEncValue = (int32_t)gsMOD_1 + 128;
						if (gsOsc1PWM) gpEncValue = 256 - gpEncValue;
						gpEncOldValue = gpEncValue;
					}
					else  {
						gpEncNew = 0;
						if (gpEncValue < 29) gpEncValue = 29;
						if (gpEncValue > 227) gpEncValue = 227;
						if (gpEncValue != gpEncOldValue) {
							if ((gpEncValue==128)&&(gpEncOldValue!=128)) gpZero = 1;
							else if((gpEncValue<=128)&&(gpEncOldValue>128))gpZero=1;
							else if((gpEncValue>=128)&&(gpEncOldValue<128))gpZero=1;
							if (gpZero) {
								gpEncValue = 128;
								gpEncOldValue = 128;
							}
							if ((uint32_t)gpEncValue >= 128) {
								gsMOD_1 = (uint32_t)gpEncValue - 128;
								gsOsc1PWM = 0;            //Freq.Mod
							}
							else  {
								gsMOD_1 =  128 - (uint32_t)gpEncValue;
								gsOsc1PWM = 1;            //PWM
							}
							gsMOD_1Preset[gSoundSet] = gsMOD_1|(gsOsc1PWM << 7);
							redrawOscMod();
							placeCursor(0x10+2);
						}
					}
				}
				else  {
					 if (gpEncNew == 0) {
						gpEncValue = (int32_t)gsMOD_1;
						gpEncOldValue = gpEncValue;
					}
					else  {
						gpEncNew = 0;
						if (gpEncValue < 0) gpEncValue = 0;
						if (gpEncValue > 99) gpEncValue = 99;
						if (gpEncValue != gpEncOldValue) {
							gsMOD_1 = (uint32_t)gpEncValue;
							if (gsOsc1PWM)  gsMOD_1 = 0; //reset for clearness
							gsMOD_1Preset[gSoundSet] = gsMOD_1;
							gsOsc1PWM = 0; //always 0 if wave type is not sqare
							redrawOscMod();
							placeCursor(0x10+2);
						}
					}
				}
			}
			else if (gpOscSelect == 1) {
				if (gfRuntimeMask & 0x00000400)  {	//wave type is square wave
					if (gpEncNew == 0) {
						gpEncValue = (int32_t)gsMOD_2 + 128;
						if (gsOsc2PWM) gpEncValue = 256 - gpEncValue;
						gpEncOldValue = gpEncValue;
					}
					else  {
						gpEncNew = 0;
						if (gpEncValue < 29) gpEncValue = 29;
						if (gpEncValue > 227) gpEncValue = 227;
						if (gpEncValue != gpEncOldValue) {
							if ((gpEncValue==128)&&(gpEncOldValue!=128)) gpZero = 1;
							else if((gpEncValue<=128)&&(gpEncOldValue>128))gpZero=1;
							else if((gpEncValue>=128)&&(gpEncOldValue<128))gpZero=1;
							if (gpZero) {
								gpEncValue = 128;
								gpEncOldValue = 128;
							}
							if ((uint32_t)gpEncValue >= 128) {
								gsMOD_2 = (uint32_t)gpEncValue - 128;
								gsOsc2PWM = 0;            //Freq.Mod
							}
							else  {
								gsMOD_2 =  128 - (uint32_t)gpEncValue;
								gsOsc2PWM = 1;            //PWM
							}
							gsMOD_2Preset[gSoundSet] = gsMOD_2|(gsOsc2PWM << 7);
							redrawOscMod();
							placeCursor(0x10+2);
						}
					}
				}
				else  {
					 if (gpEncNew == 0) {
						gpEncValue = (int32_t)gsMOD_2;
						gpEncOldValue = gpEncValue;
					}
					else  {
						gpEncNew = 0;
						if (gpEncValue < 0) gpEncValue = 0;
						if (gpEncValue > 99) gpEncValue = 99;
						if (gpEncValue != gpEncOldValue) {
							gsMOD_2 = (uint32_t)gpEncValue;
							if (gsOsc2PWM)  gsMOD_2 = 0; //reset for clearness
							gsMOD_2Preset[gSoundSet] = gsMOD_2;
							gsOsc2PWM = 0;  //always 0 if wave type is not sqare
							redrawOscMod();
							placeCursor(0x10+2);
						}
					}
				}
			}
			else  {										//Lfo Waveform
				if (gpEncNew == 0) {
					if (gpOscSelect == 2) gpEncValue = (int32_t)gLfo1Type;
					else if (gpOscSelect == 3) gpEncValue =(int32_t)gLfo2Type;
					else gpEncValue = (int32_t)gLfo3Type;
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 0) gpEncValue = 0;
					if (gpEncValue > 2) gpEncValue = 2;
					if (gpEncValue != gpEncOldValue) {
						lfoType = (uint32_t)gpEncValue;
						if (gpOscSelect == 2)  {			//Lfo1
							gLfo1Type = lfoType;
							if (lfoType == 0)  gLfo1Wave &= 0x00000100;
							else  {
								if ((gLfo1Wave & 0x000000FF) == 0)  {
									gLfo1Wave &= 0x00000100;
									gLfo1Wave |= 50; //from sine to 50% Symmetry
								}
								if (lfoType == 1)  gLfo1Wave &= 0x0000017F;
								else  gLfo1Wave |= 0x80;
							}
							lfoSymm = gLfo1Wave & 0x0000007F;
							gLfo1WavePreset[gSoundSet] = gLfo1Wave;
							calculateLfoWave(1, gLfo1Wave & 0x000000FF);
						}
						else if (gpOscSelect == 3)  {		//Lfo2
							gLfo2Type = lfoType;
							if (lfoType == 0)  gLfo2Wave &= 0x00000100;
							else  {
								if ((gLfo2Wave & 0x000000FF) == 0)  {
									gLfo2Wave &= 0x00000100;
									gLfo2Wave |= 50; //from sine to 50% Symmetry
								}
								if (lfoType == 1)  gLfo2Wave &= 0x0000017F;
								else  gLfo2Wave |= 0x80;
							}
							lfoSymm = gLfo2Wave & 0x0000007F;
							gLfo2WavePreset[gSoundSet] = gLfo2Wave;
							calculateLfoWave(2, gLfo2Wave & 0x000000FF);
						}
						else  {						//gpOscSelect = 4, Lfo3
							gLfo3Type = lfoType;
							if (lfoType == 0)  gLfo3Wave &= 0x00000100;
							else  {
								if ((gLfo3Wave & 0x000000FF) == 0)  {
									gLfo3Wave &= 0x00000100;
									gLfo3Wave |= 50; //from sine to 50% Symmetry
								}
								if (lfoType == 1)  gLfo3Wave &= 0x0000017F;
								else  gLfo3Wave |= 0x80;
							}
							lfoSymm = gLfo3Wave & 0x0000007F;
							gLfo3WavePreset[gSoundSet] = gLfo3Wave;
							calculateLfoWave(3, gLfo3Wave & 0x000000FF);
						}
						if (lfoType == 0) displayString(0x10,gpWaveStr[0]);
						else if (lfoType == 1) displayString(0x10,gpWaveStr[1]);
						else  displayString(0x10,gpSquarStr);
						placeCursor(0x10+14);
						if (lfoSymm == 0)  {
							writeLcdData('-');
							writeLcdData('-');
						}
						else  displayNL99(lfoSymm,0);
						placeCursor(0x10);
					}
				}
				gpZero = 0;
			}
		}

		if (modOsc == 4)  {
			if (gpOscSelect <= 1)  {
				if (gpEncNew == 0) {
					if (gpOscSelect == 0) gpEncValue = (int32_t)gsOsc1ModLfo2;
					else gpEncValue = (int32_t)gsOsc2ModLfo2;
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 0) gpEncValue = 0;
					if (gpEncValue > 1) gpEncValue = 1;
					if (gpEncValue != gpEncOldValue) {
						if (gpOscSelect == 0)  {
							gsOsc1ModLfo2 = (uint32_t)gpEncValue;
							gsWAVEFORM_1Preset[gSoundSet] &= 0x0000003F;
							if (gsOsc1ModLfo2)  {
								gsWAVEFORM_1Preset[gSoundSet] |= 0x40;
							}
						}
						else  {
							gsOsc2ModLfo2 = (uint32_t)gpEncValue;
							gsWAVEFORM_2Preset[gSoundSet] &= 0x0000003F;
							if (gsOsc2ModLfo2)  {
								gsWAVEFORM_2Preset[gSoundSet] |= 0x40;
							}
						}
						redrawOscDisplay();
						placeCursor(0x10+5);
					}
				}
			}
			else  {			//gpOscSelect=2,3,4  Lfo start Slope plus/minus
				 if (gpEncNew == 0)  {
					if (gpOscSelect == 2)  {
						gpEncValue =(int32_t)gLfo1Slope;
						gpEncOldValue = gpEncValue;
					}
					else if (gpOscSelect == 3) {
						gpEncValue = (int32_t)gLfo2Slope;
						gpEncOldValue = gpEncValue;
					}
					else  {
						gpEncValue = (int32_t)gLfo3Slope;
						gpEncOldValue = gpEncValue;
					}
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 0) gpEncValue = 0;
					if (gpEncValue > 1) gpEncValue = 1;
					if (gpEncValue != gpEncOldValue) {
						if (gpOscSelect == 2)  {
							gLfo1Slope = (uint32_t)gpEncValue;
							gLfo1Wave &= 0x0000000FF;
							gLfo1Wave |= (uint32_t)gpEncValue << 8;
							gLfo1WavePreset[gSoundSet] = gLfo1Wave;
						}
						else if (gpOscSelect == 3) {
							gLfo2Slope = (uint32_t)gpEncValue;
							gLfo2Wave &= 0x0000000FF;
							gLfo2Wave |= (uint32_t)gpEncValue << 8;
							gLfo2WavePreset[gSoundSet] = gLfo2Wave;
						}
						else  {
							gLfo3Slope = (uint32_t)gpEncValue;
							gLfo3Wave &= 0x0000000FF;
							gLfo3Wave |= (uint32_t)gpEncValue << 8;
							gLfo3WavePreset[gSoundSet] = gLfo3Wave;
						}
						placeCursor(0x10+9);
						if (gpEncValue == 0) writeLcdData('-');
						else writeLcdData('+');
						placeCursor(0x10+8);
					}
				}
			}
		}

		if (modOsc == 5) {
			if (gpOscSelect <= 1)  {
	//OSC1 FM modulated by OSC2 ?  OSC2 synchronized by OSC1?
				if (gpEncNew == 0) {
					if (gpOscSelect == 0)  gpEncValue = (int32_t)(gaRoute0N>>4);
					else  gpEncValue = (int32_t)(gsO2Sync);
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 0) gpEncValue = 0;
					if (gpEncValue > 1) gpEncValue = 1;
					if (gpEncValue != gpEncOldValue) {
						placeCursor(0x10+8);
						if (gpEncValue == 0)  {
							if (gpOscSelect == 0)  gaRoute0N &= 0x0000000F;
							else  gsO2Sync = 0;
							writeLcdData('|');
							writeLcdData('|');
						}
						else  {
							if (gpOscSelect == 0)  {
								if (gaRoute0N > 3) gaRoute0N -= 4;
								gaRoute0N |= 0x00000010;
								writeLcdData('F');
								writeLcdData('M');
							}
							else  {
								gsO2Sync = 1;
								writeLcdData('S');
								writeLcdData('Y');
							}
						}
						if (gpOscSelect == 0)  {
							if (gfFlexMode == 0)  {
								gaRoutePreset[gSoundSet] &= 0xFFFF0000;
								gaRoutePreset[gSoundSet] |= gaRoute0N;
							}
							else {
								gaRoutePreset[gSoundSet] &= 0x0000FFFF;
								gaRoutePreset[gSoundSet] |= gaRoute0N << 16;
							}
						}
						else  {
							gfStageMask[gSoundSet] &= 0x0000007F;
							gfStageMask[gSoundSet] |=gsO2Sync<<7;//bit7=gsO2Sync
						}
						placeCursor(0x10+8);
					}
				}
			}
			else  {			//gpOscSelect=2,3,4  Wave Symmetry
				 if (gpEncNew == 0)  {
					if ((gpOscSelect == 2) && gLfo1Type)  {
							gpEncValue =(int32_t)(gLfo1Wave & 0x0000007F);
							gpEncOldValue = gpEncValue;
					}
					else if ((gpOscSelect == 3) && gLfo2Type)  {
						gpEncValue = (int32_t)(gLfo2Wave & 0x0000007F);
						gpEncOldValue = gpEncValue;
					}
					else if (gLfo3Type)  {
						gpEncValue = (int32_t)(gLfo3Wave & 0x0000007F);
						gpEncOldValue = gpEncValue;
					}
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 10) gpEncValue = 10;
					if (gpEncValue > 90) gpEncValue = 90;
					if (gpEncValue != gpEncOldValue) {
						if ((gpOscSelect == 2) && gLfo1Type)  {
							gLfo1Wave &= 0x00000180;
							gLfo1Wave |= (uint32_t)gpEncValue;
							gLfo1WavePreset[gSoundSet] = gLfo1Wave;
							calculateLfoWave(1, gLfo1Wave & 0x000000FF);
							redrawOscDisplay();
							placeCursor(0x10+13);
						}
						else if ((gpOscSelect == 3) && gLfo2Type) {
							gLfo2Wave &= 0x00000180;
							gLfo2Wave |= (uint32_t)gpEncValue;
							gLfo2WavePreset[gSoundSet] = gLfo2Wave;
							calculateLfoWave(2, gLfo2Wave & 0x000000FF);
							redrawOscDisplay();
							placeCursor(0x10+13);
						}
						else if ((gpOscSelect == 4) && gLfo3Type)  {
							gLfo3Wave &= 0x00000180;
							gLfo3Wave |= (uint32_t)gpEncValue;
							gLfo3WavePreset[gSoundSet] = gLfo3Wave;
							calculateLfoWave(3, gLfo3Wave & 0x000000FF);
							redrawOscDisplay();
							placeCursor(0x10+13);
						}
					}
				}
			}
		}

		if (modOsc == 6)  {		//Waveform of OSC1, OSC2
			if (gpOscSelect == 0)  {
				 if (gpEncNew == 0) {
					gpEncValue = (int32_t)gsWAVEFORM_1;
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 0) gpEncValue = 0;
					if (gpEncValue > 42) gpEncValue = 42;
					if (gpEncValue != gpEncOldValue) {
						gsWAVEFORM_1 = (uint32_t)gpEncValue;
						gsWAVEFORM_1Preset[gSoundSet] = gsWAVEFORM_1;
						switch (gsWAVEFORM_1)  {
							case 0:
							case 25:
							case 36:
							case 37:
							case 38:
							case 41:
								gaZero1 = 0;
								break;
							default:
								gaZero1 = 1;
						}
						initWaveOsc1(gsWAVEFORM_1);
						redrawOscMod();
						placeCursor(0x10+11);
					}
				}
			}
			else if (gpOscSelect == 1) {
				if (gpEncNew == 0) {
					gpEncValue = (int32_t)gsWAVEFORM_2;
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 0) gpEncValue = 0;
					if (gpEncValue > 37) gpEncValue = 37;
					if (gpEncValue != gpEncOldValue) {
						gsWAVEFORM_2 = (uint32_t)gpEncValue;
						gsWAVEFORM_2Preset[gSoundSet] = gsWAVEFORM_2;
						switch (gsWAVEFORM_2)  {
							case 0:
							case 25:
							case 36:
								gaZero2 = 0;
								break;
							default:
								gaZero2 = 1;
						}
						initWaveOsc2(gsWAVEFORM_2);
						redrawOscMod();
						placeCursor(0x10+11);
					}
				}
			}
		}
		if (gpZero) {
			msDelay(500);
			gpZero = 0;
		}

		if (gpOscSelect == 2)  lfoType = gLfo1Type;
		else if (gpOscSelect == 3)  lfoType = gLfo2Type;
		else if (gpOscSelect == 4)  lfoType = gLfo3Type;

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modOsc == 0)  {
				modOsc = 1;
				if (gpOscSelect <= 1)  cursorPos = 6;
				else  cursorPos = 9;
			}
			else if (modOsc == 1)  {
				if (gpOscSelect < 2)  {
					modOsc = 2;
					cursorPos = 12;
				}
				else  {
					modOsc = 3;
					cursorPos =  0x10;
				}
			}
			else if ((modOsc == 2) && (gpOscSelect <= 1)) {
				modOsc = 3;
				cursorPos = 0x10+2;
			}
			else if (modOsc == 3)  {
				modOsc = 4;
				if (gpOscSelect <= 1)  cursorPos = 0x10+5;
				else  cursorPos = 0x10+8;
			}
			else if (modOsc == 4)  {
				if (gpOscSelect <= 1)  {
					modOsc = 5;
					cursorPos = 0x10+8;
				}
				else  {
					if (lfoType == 0)  {
						modOsc = 0;
						cursorPos = 2;
					}
					else  {
						modOsc = 5;
						cursorPos = 0x10+13;
					}
				}
			}
			else if (modOsc == 5)  {
				if (gpOscSelect <= 1)  {
					modOsc = 6;
					cursorPos = 0x10+11;
				}
				else  {
					modOsc = 0;
					cursorPos = 2;
				}
			}
			else  {			//if (modOsc == 6)
				modOsc = 0;
				cursorPos = 2;
			}
			placeCursor(cursorPos);
			gpModOsc = modOsc;
			gpPosOsc = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modOsc == 0) {
				if (gpOscSelect <= 1) {
					modOsc = 6;
					cursorPos = 0x10+11;
				}
				else  {
					if (lfoType == 0)  {
						modOsc = 4;
						cursorPos = 0x10 + 8;
					}
					else  {
						modOsc = 5;
						cursorPos = 0x10+13;
					}
				}
			}
			else if (modOsc == 1) {
				modOsc = 0;
				cursorPos = 2;
			}
			else if (modOsc == 2) {
				modOsc = 1;
				if (gpOscSelect <= 1)  cursorPos = 6;
				else  cursorPos = 9;
			}
			else if (modOsc == 3) {
				if (gpOscSelect <= 1) {
					modOsc = 2;
					cursorPos = 12;
				}
				else  {
					modOsc = 1;
					cursorPos = 9;
				}
			}
			else if (modOsc == 4) {
				modOsc = 3;
				if (gpOscSelect <= 1) cursorPos = 0x10+2;
				else cursorPos = 0x10;
			}
			else if (modOsc == 5)  {
				modOsc = 4;
				if (gpOscSelect <= 1)  cursorPos = 0x10+5;
				else cursorPos = 0x10+8;
			}
			else if (modOsc == 6)  {
				modOsc = 5;
				cursorPos = 0x10+8;
			}
			placeCursor(cursorPos);
			gpModOsc = modOsc;
			gpPosOsc = cursorPos;
		}
	}

	else  {		//Plus state
		modOsc = gpModXOsc;

		if (modOsc == 0)  {
            if (gpEncNew == 0) {
				gpEncValue=(int32_t)((gsFreeSquare1[gSoundSet]>>24)&0x000000FF);
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 50) gpEncValue = 50;
                if (gpEncValue != gpEncOldValue) {
					tempH = (uint32_t)gpEncValue;
					tempL = (gsFreeSquare1[gSoundSet] >> 16) & 0x000000FF;
					tempX = tempH*10 + tempL;
					if (tempX > 500)  {
						tempH = 50;
						tempL = 0;
					}
					placeCursor(5);
                    display99(tempH,0);
					placeCursor(8);
                    displayHexNibble(tempL);
					placeCursor(4);
					gsFreeSquare1[gSoundSet] &= 0x0000FFFF;
					gsFreeSquare1[gSoundSet] |= (tempH << 24) | (tempL << 16);
					gsRangeFS1 = tempX + (tempX / 41);	//stretch 500->512, e.g.
					tempH = (gsFreeSquare1[gSoundSet]& 0x000000FF) * 2;
					if (tempH >= (tempX /2 )) {
						tempH = tempX / 2;
						tempH &= 0xFFFFFFFE;	//clear bit 0, only even values
						gsSlopeFS1 = tempH + tempH / 41;
						gsFreeSquare1[gSoundSet] &= 0xFFFFFF00;
						gsFreeSquare1[gSoundSet] |= tempH / 2;
						redrawOscDisplay();
						placeCursor(4);
					}
					if (gsWAVEFORM_1 == 9) initWaveOsc1(9);
                }
            }
		}

		if (modOsc == 1)  {
            if (gpEncNew == 0) {
				gpEncValue=(int32_t)((gsFreeSquare1[gSoundSet]>>16)&0x000000FF);
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = -1;
                if (gpEncValue > 10) gpEncValue = 10;
                if (gpEncValue != gpEncOldValue) {
					tempH = (gsFreeSquare1[gSoundSet] >> 24) & 0x000000FF;
					if (gpEncValue < 0)  {
						tempL = 9;
						if (tempH > 0)  tempH--;
						else  tempH = 0;
					}
					else if (gpEncValue > 9)  {
						tempL = 0;
						tempH++;
					}
					else  tempL = (uint32_t)gpEncValue;
					tempX = tempH*10 + tempL;
					if (tempX > 500)  {
						tempH = 50;
						tempL = 0;
					}
					placeCursor(5);
                    display99(tempH,0);
					placeCursor(8);
                    displayHexNibble(tempL);
					placeCursor(7);
					gsFreeSquare1[gSoundSet] &= 0x0000FFFF;
					gsFreeSquare1[gSoundSet] |= (tempH << 24) | (tempL << 16);
					gsRangeFS1 = tempX + (tempX / 41);	//stretch 500->512, e.g.
					tempH = (gsFreeSquare1[gSoundSet]& 0x000000FF) * 2;
					if (tempH >= (tempX / 2)) {
						tempH = tempX/ 2;
						tempH &= 0xFFFFFFFE;	//clear bit 0, only even values
						gsSlopeFS1 = tempH + tempH / 41;
						gsFreeSquare1[gSoundSet] &= 0xFFFFFF00;
						gsFreeSquare1[gSoundSet] |= tempH / 2;
						redrawOscDisplay();
						placeCursor(7);
					}
					if (gsWAVEFORM_1 == 9) initWaveOsc1(9);
                }
            }
		}

		if (modOsc == 2)  {
            if (gpEncNew == 0) {
				gpEncValue =(int32_t)(gsFreeSquare1[gSoundSet] & 0x000000FF);
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 125) gpEncValue = 125;
                if (gpEncValue != gpEncOldValue) {
					tempX = (uint32_t)gpEncValue * 2;
					tempH = (gsFreeSquare1[gSoundSet] >> 24) & 0x000000FF;
					if (tempX >= (tempH * 5)) {
						tempX = tempH * 5;
						tempX &= 0xFFFFFFFE;	//clear bit 0, only even values
					}
					placeCursor(12);
                    display99(tempX / 10, 0);
					placeCursor(15);
					displayHexNibble(tempX % 10);
					placeCursor(11);
					gsFreeSquare1[gSoundSet] &= 0xFFFFFF00;
					gsFreeSquare1[gSoundSet] |= tempX / 2;
					gsSlopeFS1 = tempX + (tempX / 41); //stretch 250->256,e.g.
					if (gsWAVEFORM_1 == 9) initWaveOsc1(9);
                }
            }
		}

		if (modOsc == 3)  {
            if (gpEncNew == 0) {
				gpEncValue=(int32_t)((gsFreeSquare2[gSoundSet]>>24)&0x000000FF);
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 50) gpEncValue = 50;
                if (gpEncValue != gpEncOldValue) {
					tempH = (uint32_t)gpEncValue;
					tempL = (gsFreeSquare2[gSoundSet] >> 16) & 0x000000FF;
					tempX = tempH*10 + tempL;
					if (tempX > 500)  {
						tempH = 50;
						tempL = 0;
					}
					placeCursor(0x10+5);
                    display99(tempH,0);
					placeCursor(0x10+8);
                    displayHexNibble(tempL);
					placeCursor(0x10+4);
					gsFreeSquare2[gSoundSet] &= 0x0000FFFF;
					gsFreeSquare2[gSoundSet] |= (tempH << 24) | (tempL << 16);
					gsRangeFS2 = tempX + (tempX / 41);	//stretch 500->512, e.g.
					tempH = (gsFreeSquare2[gSoundSet]& 0x000000FF) * 2;
					if (tempH >= (tempX / 2)) {
						tempH = tempX / 2;
						tempH &= 0xFFFFFFFE;	//clear bit 0, only even values
						gsSlopeFS2 = tempH + tempH / 41;
						gsFreeSquare2[gSoundSet] &= 0xFFFFFF00;
						gsFreeSquare2[gSoundSet] |= tempH / 2;
						redrawOscDisplay();
						placeCursor(0x10+4);
					}
					if (gsWAVEFORM_1 == 10) initWaveOsc1(10);
                }
            }
		}

		if (modOsc == 4)  {
            if (gpEncNew == 0) {
				gpEncValue=(int32_t)((gsFreeSquare2[gSoundSet]>>16)&0x000000FF);
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = -1;
                if (gpEncValue > 10) gpEncValue = 10;
                if (gpEncValue != gpEncOldValue) {
					tempH = (gsFreeSquare2[gSoundSet] >> 24) & 0x000000FF;
					if (gpEncValue < 0)  {
						tempL = 9;
						if (tempH > 0)  tempH--;
						else  tempH = 0;
					}
					else if (gpEncValue > 9)  {
						tempL = 0;
						tempH++;
					}
					else  tempL = (uint32_t)gpEncValue;
					tempX = tempH*10 + tempL;
					if (tempX > 500)  {
						tempH = 50;
						tempL = 0;
					}
					placeCursor(0x10+5);
                    display99(tempH,0);
					placeCursor(0x10+8);
                    displayHexNibble(tempL);
					placeCursor(0x10+7);
					gsFreeSquare2[gSoundSet] &= 0x0000FFFF;
					gsFreeSquare2[gSoundSet] |= (tempH << 24) | (tempL << 16);
					gsRangeFS2 = tempX + (tempX / 41);	//stretch 500->512, e.g.
					tempH = (gsFreeSquare2[gSoundSet]& 0x000000FF) * 2;
					if (tempH >= (tempX / 2)) {
						tempH = tempX / 2;
						tempH &= 0xFFFFFFFE;	//clear bit 0, only even values
						gsSlopeFS2 = tempH + tempH / 41;
						gsFreeSquare2[gSoundSet] &= 0xFFFFFF00;
						gsFreeSquare2[gSoundSet] |= tempH / 2;
						redrawOscDisplay();
						placeCursor(0x10+7);
					}
					if (gsWAVEFORM_1 == 10) initWaveOsc1(10);
                }
            }
		}

		if (modOsc == 5)  {
            if (gpEncNew == 0) {
				gpEncValue =(int32_t)(gsFreeSquare2[gSoundSet] & 0x000000FF);
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 125) gpEncValue = 125;
                if (gpEncValue != gpEncOldValue) {
					tempX = (uint32_t)gpEncValue * 2;
					tempH = (gsFreeSquare2[gSoundSet] >> 24) & 0x000000FF;
					if (tempX >= (tempH * 5)) {
						tempX = tempH * 5;
						tempX &= 0xFFFFFFFE;	//clear bit 0, only even values
					}
					placeCursor(0x10+12);
                    display99(tempX / 10, 0);
					placeCursor(0x10+15);
					displayHexNibble(tempX % 10);
					placeCursor(0x10+11);
					gsFreeSquare2[gSoundSet] &= 0xFFFFFF00;
					gsFreeSquare2[gSoundSet] |= tempX / 2;
					gsSlopeFS2 = tempX + (tempX / 41); //stretch 250->256,e.g.
					if (gsWAVEFORM_1 == 10) initWaveOsc1(10);
                }
            }
		}

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modOsc == 0)  {
				modOsc = 1;
				cursorPos = 7;
			}
			else if (modOsc == 1)  {
				modOsc = 2;
				cursorPos = 11;
			}
			else if (modOsc == 2)  {
				modOsc = 3;
				cursorPos = 0x10+4;
			}
			else if (modOsc == 3)  {
				modOsc = 4;
				cursorPos = 0x10+7;
			}
			else if (modOsc == 4)  {
				modOsc = 5;
				cursorPos = 0x10+11;
			}
			else  {  // (modOsc == 5)
				modOsc = 0;
				cursorPos = 4;
			}
			placeCursor(cursorPos);
			gpModXOsc = modOsc;
			gpPosXOsc = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modOsc == 0) {
				modOsc = 5;
				cursorPos = 0x10+11;
			}
			else if (modOsc == 1) {
				modOsc = 0;
				cursorPos = 4;
			}
			else if (modOsc == 2) {
				modOsc = 1;
				cursorPos = 7;
			}
			else if (modOsc == 3) {
				modOsc = 2;
				cursorPos = 11;
			}
			else if (modOsc == 4) {
				modOsc = 3;
				cursorPos = 0x10+4;
			}
			else  {		//(modOsc == 5)
				modOsc = 4;
				cursorPos = 0x10+7;
			}
			placeCursor(cursorPos);
			gpModXOsc = modOsc;
			gpPosXOsc = cursorPos;
		}
	}
}


void initAdsr(void)   // *****************************************************
// ***************************************************************************
{
const uint8_t AdsrStr[] =    {"ADSR:           "};
const uint8_t FreeAdStr[] =  {"Free:      !    "};
const uint8_t VelEnvStr[] =  {"   Vel    Env   "};
const uint8_t KVelEnvStr[] = {"k  Vel    Env   "};
const uint8_t VelFMStr[] =   {"   Vel    FMI   "};

const uint8_t AttDecStr[] = {"Att      Dec    "};
const uint8_t SusRelStr[] = {"SusHo    Rls    "};


    if (gpPlusState == 0)  {
	if (gpAdsrIndex < N_ABASIC)  displayString(0,AdsrStr);
        else  displayString(0,FreeAdStr);
        if ((gpAdsrIndex == 1)&&(gaRoute0N >= 7))  displayString(0x10,VelFMStr);
		else if ((gpAdsrIndex<=1)||(gpAdsrIndex==9)) displayString(0x10,VelEnvStr);
		else  displayString(0x10,KVelEnvStr);
		redrawAdsrDisplay();
		placeCursor(gpPosAdsr);
    }
    else  {
        displayString(0,AttDecStr);
        displayString(0x10,SusRelStr);
		redrawAdsrDisplay();
		placeCursor(gpPosXAdsr);
    }
}


void redrawAdsrDisplay(void)   // ********************************************
//indices as used in synthi.c:
//0=OSC1Env, 1=Osc2Env, 2=NoiseEnv, 3=Osc1&2Freq.Mod, 4=Lfo1Freq.Mod,
//5=Lfo2Freq.Mod, 6=Bass, 7=Treble, 8=GainEnv, 9=Filter0MidFreq/Cutoff, 10=F0Peak
//FreeAdsr's: 	11=Vibrato, 12=Chorus, 13=Osc2+FreqMod, 14=F0Bandw/Resonance,
//				15=NoiseFilt2MidFreq, 16=Dry, 17=Delay, 18=Feedback
// ***************************************************************************
{
uint32_t AdsrIdx;

	AdsrIdx = gpAdsrIndex;

    if (gpPlusState == 0)  {
        displayString(5,gpAdsrTypeStr[AdsrIdx]);
        placeCursor(11);
        if (AdsrIdx >= N_ABASIC)  {
			placeCursor(10);
			writeLcdData('!');
			displayString(11,gpAdsrTypeStr[N_ABASIC+gsFreeAdsr]);
        }
		if ((AdsrIdx >= 2) && (AdsrIdx != 9)) {
			placeCursor(0x10+1);
			displayHexNibble(gNTrigger[gSoundSet]);
		}
        placeCursor(0x10 + 6);
        displaySigned(gsVelo[AdsrIdx], 28);
		placeCursor(0x10+13);
//OSC1, OSC2, Noise, Gain  only have positive envlope
		if ((AdsrIdx <= 2) || (AdsrIdx == 8))  display255(gsFixVol[AdsrIdx],5);
        else  displaySigned(gsFixVol[AdsrIdx],64);
    }
    else  {
        placeCursor(3);
        display255(gsAttack[AdsrIdx]/SECDIV, 2);
        placeCursor(12);
        display255(gsDecay[AdsrIdx]/SECDIV, 2);
        placeCursor(0x10 + 3);
        if (gsSustain[AdsrIdx] == 0) {
            writeLcdData('t');
            writeLcdData('n');
        }
        else if (gsHoldSust[AdsrIdx])   {
            writeLcdData('H');
            writeLcdData('o');
        }
        else  {
            writeLcdData('1');
            writeLcdData('x');
        }
        display255(gsSustain[AdsrIdx],5);
        placeCursor(0x10 + 12);
        display255(gsRelease[AdsrIdx]/SECDIV, 2);
    }
}


void controlAdsr(void)   // **************************************************
// ***************************************************************************
{
uint32_t AdsrAdr, AdsrIdx, value;
uint32_t modAdsr, cursorPos;

	gsAdsrOffset = gSoundSet * N_ASTRUCT;		//reconfirm
    AdsrIdx = gpAdsrIndex;
	AdsrAdr = gsAdsrOffset + AdsrIdx;

	if (gpPlusState == 0)  {
		modAdsr = gpModAdsr;
		if (modAdsr == 0) {
            if (gpEncNew == 0) {
				gpEncValue = (int32_t)AdsrIdx;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > (N_ASTRUCT - 1)) gpEncValue = N_ASTRUCT - 1;
                if (gpEncValue != gpEncOldValue) {
					AdsrIdx = (uint32_t)gpEncValue;
					gpAdsrIndex = AdsrIdx;
					AdsrAdr = gsAdsrOffset + AdsrIdx;
					gpPosXAdsr = 2;		//always reset if ADSR type is changed
					gpModXAdsr = 0;
                    initAdsr();
                }
            }
		}

		if ((modAdsr == 1) && (AdsrIdx >= N_ABASIC)) {
			if (gpEncNew == 0) {
				gpEncValue = (int32_t)gsFreeAdsr;
				gpEncOldValue = gpEncValue;
			}
            else  {
                gpEncNew = 0;
				if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 7) gpEncValue = 7;
                if (gpEncValue != gpEncOldValue) {
					gsFreeAdsr = (uint32_t)gpEncValue;
					gsFreeIdx = N_ABASIC + gsFreeAdsr;
					AdsrIdx = gsFreeIdx;
					gpAdsrIndex = AdsrIdx;
					AdsrAdr = gsAdsrOffset + AdsrIdx;
                    initAdsr();
					placeCursor(10);
                }
            }
		}

		if ((modAdsr == 2) && (AdsrIdx >= 2) && (AdsrIdx != 9)) {
            if (gpEncNew == 0) {
                gpEncValue = (int32_t)gNTrigger[gSoundSet];
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
               if (gpEncValue < 1) gpEncValue = 1;
                if (gpEncValue > 5) gpEncValue = 5;
                if (gpEncValue != gpEncOldValue) {
					gNTrigger[gSoundSet] = (uint32_t)gpEncValue;
					placeCursor(0x10+1);
					writeLcdData(gNTrigger[gSoundSet]+ 0x30);
					placeCursor(0x10);
                }
            }
		}

		if (modAdsr == 3)  {
           if (gpEncNew == 0) {
                gpEncValue=(int32_t)(gsADSR[AdsrAdr].REFIXVEL & 0x000000FF);
																		//=Velo
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 127) gpEncValue = 127;
                if (gpEncValue != gpEncOldValue) {
                    if ((gpEncValue == 28) && (gpEncOldValue != 28)) gpZero = 1;
                    else if ((gpEncValue <= 28)&&(gpEncOldValue > 28)) gpZero=1;
                    else if ((gpEncValue >= 28)&&(gpEncOldValue < 28)) gpZero=1;
                    if (gpZero) {
                        gpEncValue =  28;
                        gpEncOldValue =  28;
                    }
                    value = (uint32_t)gpEncValue;
					gsVelo[AdsrIdx] = value;
					packADSR(AdsrIdx,5,value);
                    placeCursor(0x10+6);
                    displaySigned(value, 28);
					placeCursor(0x10+5);
                }
            }
        }

		if (modAdsr == 4)  {
            if (gpEncNew == 0) {
                gpEncValue=(int32_t)((gsADSR[AdsrAdr].REFIXVEL>>8)&0x000000FF);
																	//=FixVol
                gpEncOldValue = gpEncValue;
            }
            else  {
				gpEncNew = 0;
				if (gpEncValue < 0) gpEncValue = 0;
				if (gpEncValue > 127) gpEncValue = 127;
                if (gpEncValue != gpEncOldValue) {
					value = (uint32_t)gpEncValue;
					placeCursor(0x10+13);
					if ((AdsrIdx <= 2) || (AdsrIdx == 8))  {
						gsFixVol[AdsrIdx] = value;
						packADSR(AdsrIdx,4,value);
						display255(value,5);
					}
					else  {
                        if ((gpEncValue == 64)&&(gpEncOldValue != 64)) gpZero=1;
                        else if ((gpEncValue<=64)&&(gpEncOldValue>64)) gpZero=1;
                        else if ((gpEncValue>=64)&&(gpEncOldValue<64)) gpZero=1;
                        if (gpZero)  value = 64;
						gsFixVol[AdsrIdx] = value;
						packADSR(AdsrIdx,4,value);
						displaySigned(value, 64);
					}
					placeCursor(0x10+12);
                }
            }
		}

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modAdsr == 0)  {
				if (AdsrIdx >= N_ABASIC)  {
					modAdsr = 1;
					cursorPos = 10;
				}
				else if ((AdsrIdx >= 2) && (AdsrIdx != 9))  {
					modAdsr = 2;
					cursorPos = 0x10;
				}
				else  {
					modAdsr = 3;
					cursorPos = 0x10+5;
				}
			}
			else if (modAdsr == 1)  {
				modAdsr = 2;
				cursorPos = 0x10;
			}
			else if (modAdsr == 2)  {
				modAdsr = 3;
				cursorPos = 0x10+5;
			}
			else if (modAdsr == 3)  {
				modAdsr = 4;
				cursorPos = 0x10+12;
			}
			else if (modAdsr == 4)  {
				modAdsr = 0;
				cursorPos = 4;
			}
			placeCursor(cursorPos);
			gpModAdsr = modAdsr;
			gpPosAdsr = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modAdsr == 0)  {
				modAdsr = 4;
				cursorPos = 0x10+12;
			}
			else if (modAdsr == 1)  {
				modAdsr = 0;
				cursorPos = 4;
			}
			else if (modAdsr == 2)  {
				if (AdsrIdx < N_ABASIC)  {
					modAdsr = 0;
					cursorPos = 4;
				}
				else  {
					modAdsr = 1;
					cursorPos = 10;
				}
			}
			else if (modAdsr == 3)  {
				if ((AdsrIdx >= 2) && (AdsrIdx != 9))  {
					modAdsr = 2;
					cursorPos = 0x10;
				}
				else  {
					modAdsr = 0;
					cursorPos = 4;
				}
			}
			else if (modAdsr == 4)  {
				modAdsr = 3;
				cursorPos = 0x10+5;
			}
			placeCursor(cursorPos);
			gpModAdsr = modAdsr;
			gpPosAdsr = cursorPos;
		}
    }

    else  {  //PLUS State -------------------------
		modAdsr = gpModXAdsr;
        if (modAdsr == 0)  {
            if (gpEncNew == 0) {
                gpEncValue=(int32_t)((gsADSR[AdsrAdr].ATDECSUS>>24)&0x000000FF);
																	//=Attack
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 125) gpEncValue = 125;
                if (gpEncValue != gpEncOldValue) {
                    value = (uint32_t)gpEncValue;
					placeCursor(3);
					if (value <= 100) gsAttack[AdsrIdx] = value * A_FACTOR;
					else gsAttack[AdsrIdx] = SYSFREQ +(value-100)*L_FACTOR;
					packADSR(AdsrIdx,0,value);
					display255(gsAttack[AdsrIdx]/SECDIV, 2);
                    placeCursor(2);
                }
            }
        }
        if (modAdsr == 1)  {
            if (gpEncNew == 0) {
            gpEncValue=(int32_t)((gsADSR[AdsrAdr].ATDECSUS>>16)&0x000000FF);
																	//=Decay
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 125) gpEncValue = 125;
                if (gpEncValue != gpEncOldValue) {
                    value = (uint32_t)gpEncValue;
                    placeCursor(12);
					if (value <= 100) gsDecay[AdsrIdx] = value * A_FACTOR;
					else gsDecay[AdsrIdx] = SECFACTOR +(value-100)*L_FACTOR;
					packADSR(AdsrIdx,1,value);
					display255(gsDecay[AdsrIdx]/SECDIV, 2);
                    placeCursor(11);
                }
            }
        }
        if (modAdsr == 2)  {
            if (gpEncNew == 0) {
				value= gsADSR[AdsrAdr].ATDECSUS & 0x000000FF;
													//=Sustain incl Hold bit
                if (gsHoldSust[AdsrIdx])  {
                    gpEncValue = (int32_t)(value | 0x80);
                }
                else gpEncValue = (int32_t)(128 - value);
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 28) gpEncValue = 28;
                if (gpEncValue > 228) gpEncValue = 228;
                if (gpEncValue != gpEncOldValue) {
                    if ((gpEncValue == 128)&&(gpEncOldValue!=128)) gpZero = 1;
                    else if ((gpEncValue<=128)&&(gpEncOldValue>128)) gpZero = 1;
                    else if ((gpEncValue>=128)&&(gpEncOldValue<128)) gpZero = 1;
                    if (gpZero) {
                        gpEncValue = 128;
                        gpEncOldValue = 128;
                    }
					gsHoldSust[AdsrIdx] = ((uint32_t)gpEncValue >> 7) & 1;
					if (gsHoldSust[AdsrIdx])  {
						gsSustain[AdsrIdx] = (uint32_t)gpEncValue & 0x7F;
						packADSR(AdsrIdx,2,(uint32_t)gpEncValue);
					}
					else  {
						gsSustain[AdsrIdx]  =  128 - (uint32_t)gpEncValue;
						packADSR(AdsrIdx,2,gsSustain[AdsrIdx]);
					}
					redrawAdsrDisplay();
                    placeCursor(0x10+4);
                }
            }
        }
        if (modAdsr == 3)  {
            if (gpEncNew == 0) {
                gpEncValue=(int32_t)((gsADSR[AdsrAdr].REFIXVEL>>16)&0x000000FF);
																	//=Release
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 125) gpEncValue = 125;
                if (gpEncValue != gpEncOldValue) {
                    value = (uint32_t)gpEncValue;
                    placeCursor(0x10+12);
					if (value <= 100) gsRelease[AdsrIdx] = value * A_FACTOR;
					else gsRelease[AdsrIdx] = SECFACTOR+(value-100)*L_FACTOR;
					packADSR(AdsrIdx,3,value);
					display255(gsRelease[AdsrIdx]/SECDIV, 2);
                    placeCursor(0x10+11);
                }
            }
        }
		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modAdsr == 0)  {
				modAdsr = 1;
				cursorPos = 11;
			}
			else if (modAdsr == 1)  {
				modAdsr = 2;
				cursorPos = 0x10+4;
			}
			else if (modAdsr == 2)  {
				modAdsr = 3;
				cursorPos = 0x10+11;
			}
			else if (modAdsr == 3)  {
				modAdsr = 0;
				cursorPos = 2;
			}
			placeCursor(cursorPos);
			gpModXAdsr = modAdsr;
			gpPosXAdsr = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modAdsr == 0)  {
				modAdsr = 3;
				cursorPos = 0x10+11;
			}
			else if (modAdsr == 1)  {
				modAdsr = 0;
				cursorPos = 2;
			}
			else if (modAdsr == 2)  {
				modAdsr = 1;
				cursorPos = 11;
			}
			else if (modAdsr == 3)  {
				modAdsr = 2;
				cursorPos = 0x10+4;
			}
			placeCursor(cursorPos);
			gpModXAdsr = modAdsr;
			gpPosXAdsr = cursorPos;
		}
    }
	if (gpZero) {
        msDelay(500);
        gpZero = 0;
    }
}


void initDelay(void)   // ****************************************************
// ***************************************************************************
{
const uint8_t DelayStr[] =    {"DLv:     T:     "};
const uint8_t ChorusFbStr[] = {"Chorus:   Fb:   "};

    displayString(0,DelayStr);
    placeCursor(4);
	displayDelayVol(gDelayLevelFactor);
	placeCursor(11);
    displayDelayTime();
	displayString(0x10,ChorusFbStr);
	if (gChorusWidth != 0)  {
		placeCursor(0x10+4);
		writeLcdData('L');
		writeLcdData(gChorusLfo3 + 0x32);
	}
	placeCursor(0x10+7);
	displayNL99(gChorusWidth, 0);
	placeCursor(0x10+13);
	displayDelayVol(gFeedbackFactor);
	placeCursor(gpPosDelay);
}


void controlDelay(void)   // *************************************************
// ***************************************************************************
{
uint32_t delayTimeH, delayTimeL;
uint32_t modDelay, cursorPos;

	modDelay = gpModDelay;

//PLUS state not used for delay
	if (modDelay == 0)
	{
		if (gpEncNew == 0) {
			gpEncValue = gDelayLevelFactor;
			gpEncOldValue = gpEncValue;
		}
		else  {
			gpEncNew = 0;
			if (gpEncValue < -127) gpEncValue = -127;
			if (gpEncValue > 127) gpEncValue = 127;
			if (gpEncValue != gpEncOldValue) {
				if ((gpEncValue == 0) && (gpEncOldValue != 0)) gpZero = 1;
				else if ((gpEncValue <=0 ) && (gpEncOldValue > 0)) gpZero = 1;
				else if ((gpEncValue >= 0) && (gpEncOldValue < 0)) gpZero = 1;
				if (gpZero) {
					gpEncValue = 0;
					gpEncOldValue = 0;
				}
				gDelayLevelFactor = gpEncValue;
				if (gDelayLevelFactor < 0) {
					gDelayLevelPreset[gSoundSet] =  128 - gDelayLevelFactor;
				}
				else  {
					gDelayLevelPreset[gSoundSet] = gDelayLevelFactor;
				}
				placeCursor(4);
				displayDelayVol(gDelayLevelFactor);
				placeCursor(3);
			}
		}
	}

    if (modDelay == 1)  {
		delayTimeL = gDelayTime & 0x000000FF;
        if (gpEncNew == 0) {
            gpEncValue = (int32_t)((gDelayTime >> 8) & 0x000000FF);
            gpEncOldValue = gpEncValue;
        }
        else  {
            gpEncNew = 0;
            if (gpEncValue < -1) gpEncValue = -1;
            if (gpEncValue > 30) gpEncValue = 30;
            if (gpEncValue != gpEncOldValue) {
                if (gpEncValue == -1)  {
                    delayTimeH = 0;                 //full reset to zero
                    delayTimeL = 0;
                }
				else if (gpEncValue == 30)  {
					delayTimeH = 30;                 //max value
					delayTimeL = 0;
				}
                else delayTimeH = (uint32_t)gpEncValue;
                gDelayTime = (delayTimeH << 8) | delayTimeL;
                gDelayTimePreset[gSoundSet] = gDelayTime;
                gDelayOffset = (gDelayTime >> 8) * 331 + \
                                        ((gDelayTime & 0x000000FF) * 331)/100;
//gDelayOffset is delay time expressed in number of cells of delay memory
//i.e. the difference between write(head) and read(tail) pointer
                placeCursor(11);
                displayDelayTime();
				placeCursor(10);
            }
        }
    }

    if (modDelay == 2)  {
		delayTimeH = (gDelayTime >> 8) & 0x000000FF;
       if (gpEncNew == 0) {
            gpEncValue = (int32_t)(gDelayTime & 0x000000FF);
            gpEncOldValue = gpEncValue;
        }
        else  {
            gpEncNew = 0;
            if (gpEncValue < 0) {
                if (delayTimeH == 0) gpEncValue = 0;
                else gpEncValue = -1;
            }
            if (gpEncValue > 100) gpEncValue = 100;
            if (gpEncValue != gpEncOldValue) {

                if ((delayTimeH == 30) && (gpEncValue >= 0))  delayTimeL = 0;
                else if (gpEncValue == -1)  {
                    delayTimeL = 99;
                    delayTimeH-- ;
                }
                else if (gpEncValue == 100) {
					delayTimeL = 0;
					delayTimeH++ ;
                }
                else delayTimeL = (uint32_t)gpEncValue;

                gDelayTime = (delayTimeH << 8) | delayTimeL;
                gDelayTimePreset[gSoundSet] = gDelayTime;
                gDelayOffset = (gDelayTime >> 8) * 331 + \
                                        ((gDelayTime & 0x000000FF) * 331)/100;
                placeCursor(11);
                displayDelayTime();
                placeCursor(14);
            }
        }
    }

	if (modDelay == 3)  {
		if (gpEncNew == 0) {
			gpEncValue = (int32_t)gChorusWidth + 128;
			if (gChorusLfo3 == 0) gpEncValue = 256 - gpEncValue;
			gpEncOldValue = gpEncValue;
		}
		else  {
			gpEncNew = 0;
			if (gpEncValue < 29) gpEncValue = 29;
			if (gpEncValue > 227) gpEncValue = 227;
			if (gpEncValue != gpEncOldValue) {
				if ((gpEncValue== 128)&&(gpEncOldValue!=128)) gpZero = 1;
				else if ((gpEncValue<=128)&&(gpEncOldValue >128)) gpZero =1;
				else if ((gpEncValue>=128)&&(gpEncOldValue <128)) gpZero =1;
				if (gpZero) {
					gpEncValue = 128;
					gpEncOldValue = 128;
				}
				if (gpEncValue < 128) {
					gChorusLfo3 = 0;
					gChorusWidth = (uint32_t)(128 - gpEncValue) & 0x7F;
					gChorusPreset[gSoundSet] = gChorusWidth;
				}
				else  {
					gChorusLfo3 = 1;
					gChorusWidth = (uint32_t)gpEncValue - 128;
					gChorusPreset[gSoundSet] = gChorusWidth | 0x80;
				}
				placeCursor(0x10+4);
				if (gChorusWidth == 0)  {
					writeLcdData('u');
					writeLcdData('s');
				}
				else  {
					writeLcdData('L');
					writeLcdData(gChorusLfo3 + 0x32);
				}
				placeCursor(0x10+7);
				displayNL99(gChorusWidth,0);
				placeCursor(0x10+6);
			}
		}
	}

	if (modDelay == 4)  {
		if (gpEncNew == 0) {
			gpEncValue = gFeedbackFactor;
			gpEncOldValue = gpEncValue;
		}
		else  {
			gpEncNew = 0;
			if (gpEncValue < -99) gpEncValue = -99; //avoid self-oscillation
			if (gpEncValue > 99) gpEncValue = 99;
			if (gpEncValue != gpEncOldValue) {
				if ((gpEncValue == 0) && (gpEncOldValue !=0 )) gpZero = 1;
				else if ((gpEncValue <= 0) && (gpEncOldValue > 0)) gpZero = 1;
				else if ((gpEncValue >= 0) && (gpEncOldValue < 0)) gpZero = 1;
				if (gpZero) {
					gpEncValue = 0;
					gpEncOldValue = 0;
				}
				gFeedbackFactor = gpEncValue;
				if (gFeedbackFactor < 0) {
					gFeedbackPreset[gSoundSet] =  128 - gFeedbackFactor;
				}
				else  {
					gFeedbackPreset[gSoundSet] = gFeedbackFactor;
				}
				placeCursor(0x10+13);
				displayDelayVol(gFeedbackFactor);
				placeCursor(0x10+12);
			}
		}
    }
    if (gpZero) {
        msDelay(500);
        gpZero = 0;
    }

	if (gpKeyState == KEY_F_PRESSED)  {
		if (gpLockKey_F) return;
		gpLockKey_F = 1;
		if (modDelay == 0)  {
			modDelay = 1;
			cursorPos = 10;
		}
		else if (modDelay == 1)  {
			modDelay = 2;
			cursorPos = 14;
		}
		else if (modDelay == 2)  {
			modDelay = 3;
			cursorPos = 0x10+6;
		}
		else if (modDelay == 3)  {
			modDelay = 4;
			cursorPos = 0x10+12;
		}
		else if (modDelay == 4)  {
			modDelay = 0;
			cursorPos = 3;
		}
		placeCursor(cursorPos);
        gpModDelay = modDelay;
        gpPosDelay = cursorPos;
	}
	if (gpKeyState == KEY_B_PRESSED)  {
		if (gpLockKey_B) return;
		gpLockKey_B = 1;
		if (modDelay == 0)  {
			modDelay = 4;
			cursorPos = 0x10+12;
		}
		else if (modDelay == 1)  {
			modDelay = 0;
			cursorPos = 3;
		}
		else if (modDelay == 2)  {
			modDelay = 1;
			cursorPos = 10;
		}
		else if (modDelay == 3)  {
			modDelay = 2;
			cursorPos = 14;
		}
		else if (modDelay == 4)  {
			modDelay = 3;
			cursorPos = 0x10+6;
		}
		placeCursor(cursorPos);
        gpModDelay = modDelay;
        gpPosDelay = cursorPos;
	}
}


void initFilter(void)   // ***************************************************
// ***************************************************************************
{
//	updateRoute();
	if (gfACTIV_IIR[gfFilterNum].TYPE == 0)  {
		gpModFilter = 0;
		gpPosFilter = 3;
	}
	redrawFilterDisplay();
	placeCursor(gpPosFilter);
}


void redrawFilterDisplay(void)   // ******************************************
// ***************************************************************************
{
uint32_t filterType;
const uint8_t fixedStr[] = {"params fixed"};

    filterType = gfACTIV_IIR[gfFilterNum].TYPE;
    gfFilterType = filterType;						//reconfirm for global use

	displayString(0, gpEmptyStr);
	displayString(0x10, gpEmptyStr);

    placeCursor(0);
    if (gfFlexMode== 0)  {
		writeLcdData('E');
		writeLcdData('A');
		if ((gfFilterNum == 3) || (gfFilterNum == 4))  {
			gpModFilter = 0;
			gpPosFilter = 3;
		}
    }
    else  {
		writeLcdData('F');
		writeLcdData('X');
    }
	writeLcdData('#');
	writeLcdData(gfFilterNum + 0x30);
	placeCursor(0x10);
	writeLcdData('T');
	writeLcdData(filterType + 0x30);

	if ((gfFlexMode == 0) && ((gfFilterNum == 3)||(gfFilterNum == 4)))  {
		displayString(0x10+3,fixedStr);
	}
	else  {
		if (filterType)  {
			placeCursor(5);
			writeLcdData('F');
			displayMidiFreq(filterType,gfACTIV_IIR[gfFilterNum].FREQ);
			if (gfFilterNum == 0)  {
				placeCursor(11);
				writeLcdData('T');
				writeLcdData('r');
				display255(gsKeytrack0Preset[gSoundSet],5);
			}
			placeCursor(0x10+3);
			if ((filterType == 5) || (filterType == 6)) {
				writeLcdData('B');
				writeLcdData('w');
				writeLcdData('i');
			}
			else  {
				writeLcdData('R');
				writeLcdData('e');
				writeLcdData('s');
			}
			displayQFactor(gfFilterNum);
			placeCursor(0x10+11);
			writeLcdData('P');
			displayPeak(gfFilterNum);
		}
	}
}


void controlFilter(void)   // ************************************************
// ***************************************************************************
{
uint32_t index, filterType, keytrack;
uint32_t modFilter, cursorPos;

	index = (gSoundSet * N_FILTER) + gfFilterNum;
	modFilter = gpModFilter;

	filterType = gfACTIV_IIR[gfFilterNum].TYPE;
	gfFilterType = filterType;						//reconfirm for global use

//PLUS state doesnt exist here
    if (modFilter == 0)  {
       if (gpEncNew == 0) {
            gpEncValue = (int32_t)gfFilterNum;
            gpEncOldValue = gpEncValue;
        }
        else  {
            gpEncNew = 0;
            if (gpEncValue < 0) gpEncValue = 0;
            if (gpEncValue > N_FILTER - 1) gpEncValue = N_FILTER - 1;
            if (gpEncValue != gpEncOldValue) {
                gfFilterNum = (uint32_t)gpEncValue;
				index = (gSoundSet * N_FILTER) + gfFilterNum;
                redrawFilterDisplay();  //displays params of new selected filter
                placeCursor(2);         //and reconfirms filterType
            }
        }
    }

	if ((gfFlexMode == 0) && ((gfFilterNum == 3)||(gfFilterNum == 4)))  return;

    if (modFilter == 1)  {
        if  (filterType)  {
            if (gpEncNew == 0) {
				gpEncValue = (int32_t)gfACTIV_IIR[gfFilterNum].FREQ;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 12) gpEncValue = 12;     //min ca. 16Hz
                if (gpEncValue > 120) gpEncValue = 120;   //max 8372Hz
                if (gpEncValue != gpEncOldValue) {
                    placeCursor(6);
					gfACTIV_IIR[gfFilterNum].FREQ = (uint32_t)gpEncValue;
					reInitFilter(gfFilterNum);		//performs packFilter()
					displayMidiFreq(filterType,gfACTIV_IIR[gfFilterNum].FREQ);
                    placeCursor(5);
                }
            }
        }
    }
    if (modFilter == 2)  {
        if ((gfFilterNum == 0) && filterType) {
            keytrack = gsKeytrack0Preset[gSoundSet];
            if (gpEncNew == 0) {
                gpEncValue = (int32_t)keytrack;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 127) gpEncValue = 127;
                if (gpEncValue != gpEncOldValue) {
                    keytrack = (uint32_t)gpEncValue;
					gsKeytrack0Preset[gSoundSet] = keytrack;
					gsKeytrack0 = (float)keytrack / 100.;
                    placeCursor(13);
                    display255(keytrack,5);
                    placeCursor(12);
                }
            }
        }
    }
    if (modFilter == 3)  {
        if (gpEncNew == 0) {
            gpEncValue = filterType;
            gpEncOldValue = gpEncValue;
        }
        else  {
            gpEncNew = 0;
            if (gpEncValue < 0) gpEncValue = 0;
            if (gpEncValue > 8) gpEncValue = 8;
            if (gpEncValue != gpEncOldValue) {
                filterType = (uint32_t)gpEncValue;
                gfACTIV_IIR[gfFilterNum].TYPE = filterType;
                gfFilterType = filterType;				//for global use
                reInitFilter(gfFilterNum);				//performs packFilter()
                redrawFilterDisplay();
				placeCursor(0x10);
            }
        }
    }
    if (modFilter == 4)  {
        if (filterType >= 3)  {
            if (gpEncNew == 0) {
				gpEncValue = (int32_t)gfACTIV_IIR[gfFilterNum].QFACTOR;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if ((filterType == 5) || (filterType == 6))  {
                //QFACTOR describes bandwidth
                    if (gpEncValue < 18) gpEncValue = 18;
                    if (gpEncValue > 111) gpEncValue = 111; //Bw.limit=4978Hz
                }
                 else  {
                //QFACTOR describes Zeta
                    if (gpEncValue < 1) gpEncValue = 1;
                    if (gpEncValue > 100) gpEncValue = 100;
                 }
                if (gpEncValue != gpEncOldValue) {
					gfACTIV_IIR[gfFilterNum].QFACTOR = (uint32_t)gpEncValue;
                    reInitFilter(gfFilterNum);		//performs packFilter()
                    placeCursor(0x10+6) ;
                    displayQFactor(gfFilterNum);
                    placeCursor(0x10+5);
                }
            }
        }
    }
    if (modFilter == 5)  {
       if (filterType)  {
            if (gpEncNew == 0) {
				gpEncValue = (int32_t)gfACTIV_IIR[gfFilterNum].PEAK;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (filterType >= 6)  {		//types 6,7,8 = parametric
                    if (gpEncValue < 0) gpEncValue = 0;
                    if (gpEncValue > 120) gpEncValue = 120;
                }
                else  {						//type0 excluded before
                    if (gpEncValue < 10) gpEncValue = 10;
                    if (gpEncValue > 35) gpEncValue = 35;
                }
                if (gpEncValue != gpEncOldValue) {
					if ((gpEncValue==20)&&(gpEncOldValue!=20)) gpZero = 1;
					else if ((gpEncValue<=20)&&(gpEncOldValue>20)) gpZero=1;
					else if ((gpEncValue>=20)&&(gpEncOldValue<20)) gpZero=1;
					if (gpZero) {
						gpEncValue = 20;
						gpEncOldValue = 20;
					}
                    gfACTIV_IIR[gfFilterNum].PEAK = (uint32_t)gpEncValue;
                    reInitFilter(gfFilterNum);		//performs packFilter()
					placeCursor(0x10+12);
					displayPeak(gfFilterNum);
					placeCursor(0x10+11);
                }
            }
        }
    }

    if (gpKeyState == KEY_F_PRESSED)  {
        if (gpLockKey_F) return;
        gpLockKey_F = 1;
        if (modFilter == 0)  {
			if (filterType == 0)  {
				modFilter = 3;
				cursorPos = 0x10;
			}
			else  {
				modFilter = 1;
				cursorPos = 5;
			}
        }
        else if (modFilter == 1)  {
				if ((gfFilterNum == 0) && filterType) {
				modFilter = 2;
				cursorPos = 12;
			}
			else  {
				modFilter = 3;
				cursorPos = 0x10;
			}
        }
        else if (modFilter == 2)  {
            modFilter = 3;
            cursorPos = 0x10;
        }
        else if (modFilter == 3)  {
			if (filterType == 0)  {
				modFilter = 0;
				cursorPos = 3;
			}
			else if (filterType > 2) {
				modFilter = 4;
				cursorPos = 0x10+5;
			}
			else  {
				modFilter = 5;
				cursorPos = 0x10+11;
			}
        }
        else if (modFilter == 4)  {
			if (filterType)  {
				modFilter = 5;
				cursorPos = 0x10+11;
			}
			else  {
				modFilter = 0;
				cursorPos = 3;
			}
        }
        else if (modFilter == 5)  {
            modFilter = 0;
            cursorPos = 2;
        }
        placeCursor(cursorPos);
		gpModFilter = modFilter;
		gpPosFilter = cursorPos;
    }
    if (gpKeyState == KEY_B_PRESSED)  {
        if (gpLockKey_B) return;
        gpLockKey_B = 1;
        if (modFilter == 0)  {
			if (filterType) {
				modFilter = 5;
				cursorPos = 0x10+11;
			}
			else {
				modFilter = 3;
				cursorPos = 0x10;
			}
        }
        else if (modFilter == 1)  {
            modFilter = 0;
            cursorPos = 2;
        }
        else if (modFilter == 2)  {
            modFilter = 1;
            cursorPos = 5;
        }
        else if (modFilter == 3)  {
			if (filterType)  {
				if (gfFilterNum == 0) {
					modFilter = 2;
					cursorPos = 12;
				}
				else  {
				modFilter = 1;
				cursorPos = 5;
				}
			}
			else  {
				modFilter = 0;
				cursorPos = 2;
			}
        }
        else if (modFilter == 4)  {
            modFilter = 3;
            cursorPos = 0x10;
        }
        else if (modFilter == 5)  {
			if (filterType > 2) {
				modFilter = 4;
				cursorPos = 0x10+5;
			}
			else  {
				modFilter = 3;
				cursorPos = 0x10;
			}
        }
        placeCursor(cursorPos);
		gpModFilter = modFilter;
		gpPosFilter = cursorPos;
    }

    if (gpZero) {
        msDelay(500);
        gpZero = 0;
    }
}


void reInitFilter(uint32_t number)   // **************************************
// ***************************************************************************
{
	initFilterParams(number,
			gfACTIV_IIR[number].TYPE, gfACTIV_IIR[number].FREQ,
					gfACTIV_IIR[number].QFACTOR, gfACTIV_IIR[number].PEAK, 0);
								//refresh gfACTIV_IIR[number], run packFilter()
	if (gfFilterNum == 3) adjPeak3(gfBass);		//update bass temporarily
	if (gfFilterNum == 4) adjPeak4(gfTreble);	//update treble temporarily
}


void initRoute(void)   // ***************************************************
// **************************************************************************
{
	gpWaitKeyB = 0;
    redrawRouteLine1();
	redrawRouteLine2();
    if (gpPlusState == 0)  placeCursor(gpPosRoute);
    else  placeCursor(gpPosXRoute);
}


void updateRoute(void)   // **************************************************
// ***************************************************************************
{
uint32_t n;

	gpFilterSet = 1;
	if (gfFlexMode == 0)  gaRoute0N = gaRoutePreset[gSoundSet] & 0x000000FF;
	else  gaRoute0N = (gaRoutePreset[gSoundSet] >> 16)&0x000000FF;

	for (n = 0;n < N_FILTER;n++)  unpackFilter(n, 1);//init filters immediately
	gfFilterNum == 0;	//keep default
	saveUndo();
	saveMemo();
	unpackBasTreb();
	adjPeak3(gfBass);
	adjPeak4(gfTreble);
	createShortFlags();
}


void redrawRouteLine1(void)   // *********************************************
// ***************************************************************************
{
uint32_t filter0Type;
const uint8_t RouteEasyStr[] = {"R:Easy          "};
const uint8_t RouteFlexStr[] = {"R:Flex          "};

	filter0Type = gfACTIV_IIR[0].TYPE;
    if (gpPlusState == 0)  {
		if (gfFlexMode == 0)  displayString(0,RouteEasyStr);
		else  displayString(0,RouteFlexStr);
		placeCursor(7);
		writeLcdData('B');
		displaySigned(gfBass,64);
		placeCursor(12);
		writeLcdData('T');
		displaySigned(gfTreble,64);
    }
    else  {
		displayString(0,gpEmptyStr);
		placeCursor(0);
		writeLcdData('F');
		if (filter0Type == 0)  {
			writeLcdData(':');
			writeLcdData('n');
			writeLcdData('.');
			writeLcdData('a');
			writeLcdData('.');
			return;
		}
		else displayMidiFreq(filter0Type,gfACTIV_IIR[0].FREQ);

//		if (filter0Type <= 2)  return;

		placeCursor(5);
		if ((filter0Type == 5) || (filter0Type == 6)) {
			writeLcdData('B');
			writeLcdData('w');
			displayQFactor(0);
		}
		else if (filter0Type >= 3) {
			writeLcdData('R');
			writeLcdData('s');
			displayQFactor(0);
		}
		placeCursor(11);
		writeLcdData('P');
		displayPeak(0);
	}
}


void redrawRouteLine2(void)   // ********************************************
// **************************************************************************
{
uint32_t filter0Type;
const uint8_t Osc12Str[] =     {"Os1>F0 Os2>F0   "};
const uint8_t Osc1FMStr[] =    {"Osc1(FM)->F0    "};
const uint8_t EasyStr[] =      {"Tp  Easy actFilt"};

	unpackBasTreb();				//reconfirm
	if (gfFlexMode == 0)  gaRoute0N = gaRoutePreset[gSoundSet] & 0x000000FF;
	else  gaRoute0N = (gaRoutePreset[gSoundSet] >> 16)&0x000000FF;
	filter0Type = gfACTIV_IIR[0].TYPE;
    if (gpPlusState == 0)  {
		if (gaRoute0N <= 7)  {
			displayString(0x10,Osc12Str);
			if (gaRouteOsc1 == 0)  {
				placeCursor(0x10+5);
				writeLcdData('1');
			}
			if (gaRouteOsc2 == 0)  {
				placeCursor(0x10+12);
				writeLcdData('1');
			}
			placeCursor(0x10+14);
		}
		else  {
			displayString(0x10,Osc1FMStr);
			if (gaRouteOsc1 == 0)  {
				placeCursor(0x10+11);
				writeLcdData('1');
			}
			placeCursor(0x10+13);
		}
		if (gaRouteNoise)  {
			writeLcdData('+');
			writeLcdData('N');
		}
    }
    else  {
		displayString(0x10,EasyStr);
		if (gfFlexMode)  displayString(0x10+4,gpFlexStr);
		placeCursor(0x10+2);
		writeLcdData(filter0Type + 0x30);
		if (gpFilterSet == 0)  displayString(0x10+9,gpDefaultStr);
		else if (gpFilterSet == 3) displayString(0x10+9,gpMemoStr);
		else  displayString(0x10+9,gpActFiltStr);
    }
}


void controlRoute(void)   // *************************************************
// ***************************************************************************
{
uint32_t filter0Type, temp;
uint32_t modRoute, cursorPos;
const uint8_t toMemoStr[] =   {"-->Memo"};
const uint8_t loadMemStr[] =  {"LoadMem"};
const uint8_t undoStr[] =     {"Undo   "};
const uint8_t noActionStr[] = {"   no action    "};
	filter0Type = gfACTIV_IIR[0].TYPE;

    if (gpPlusState == 0)  {		//Select filter configuration for routing
		modRoute = gpModRoute;
		if (modRoute == 0)  {
            if (gpEncNew == 0) {
                gpEncValue = (int32_t)gfFlexMode;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 1) gpEncValue = 1;
                if (gpEncValue != gpEncOldValue) {
                    gfFlexMode = (uint32_t)gpEncValue;
					gfStageMask[gSoundSet] &= 0x000000BF;
					gfStageMask[gSoundSet] |= gfFlexMode << 6;
					updateRoute();
                    initRoute();
                    placeCursor(1);
                }
            }
		}
		if (modRoute == 1) controlBass(8);	//is independent of gfFlexMode
		if (modRoute == 2) controlTreble(13);	//is independent of gfFlexMode

		if (modRoute == 3) {
            if (gpEncNew == 0) {
				gpEncValue = (int32_t)(gaRoute0N & 0x0000000F);
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gaRoute0N < 0x10) {
					if (gpEncValue > 7) gpEncValue = 7;
                }
				else if (gpEncValue > 3) gpEncValue = 3;
                if (gpEncValue != gpEncOldValue) {
					if (gaRoute0N < 0x10) gaRoute0N = (uint32_t)gpEncValue;
					else  gaRoute0N = (uint32_t)gpEncValue | 0x00000010;
					if (gfFlexMode == 0)  {
						gaRoutePreset[gSoundSet] &= 0xFFFFFF00;
						gaRoutePreset[gSoundSet] |= gaRoute0N;
					}
					else  {
						gaRoutePreset[gSoundSet] &= 0xFF00FFFF;
						gaRoutePreset[gSoundSet] |= gaRoute0N << 16;
					}
					createShortFlags();
					redrawRouteLine2();
					placeCursor(0x10+3);
                }
            }
		}

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modRoute == 0)  {
					modRoute = 1;
					cursorPos = 7;
			}
			else if (modRoute == 1)  {
					modRoute = 2;
					cursorPos = 12;
			}
			else if (modRoute == 2)  {
					modRoute = 3;
					cursorPos = 0x10+3;
			}
			else if (modRoute == 3)  {
				modRoute = 0;
				cursorPos = 1;
			}
			placeCursor(cursorPos);
			gpModRoute = modRoute;
			gpPosRoute = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpLockKey_B) return;
			gpLockKey_B = 1;
			if (modRoute == 0)  {
					modRoute = 3;
					cursorPos = 0x10+3;
			}
			else if (modRoute == 1)  {
					modRoute = 0;
					cursorPos = 1;
			}
			else if (modRoute == 2)  {
				modRoute = 1;
				cursorPos = 7;
			}
			else if (modRoute == 3)  {
				modRoute = 2;
				cursorPos = 12;
			}
			placeCursor(cursorPos);
			gpModRoute = modRoute;
			gpPosRoute = cursorPos;
		}
    }

    else  {				//detailled setup of routing parameters
		modRoute = gpModXRoute;
//exclusively filter#0 is handled below
		if (modRoute == 0)  {
			if (filter0Type)  {
				if (gpEncNew == 0) {
					gpEncValue = (int32_t)gfACTIV_IIR[0].FREQ;
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (gpEncValue < 18) gpEncValue = 18;     //min ca.23Hz
					if (gpEncValue > 121) gpEncValue = 121;   //max 8372Hz
					if (gpEncValue != gpEncOldValue) {
						gfACTIV_IIR[0].FREQ=(uint32_t)gpEncValue;
						reInitFilter(0);		//performs packFilter()
						placeCursor(1);
						displayMidiFreq(filter0Type,(uint32_t)gpEncValue);
						placeCursor(0);
					}
				}
			}
		}
		else if (modRoute == 1)  {
			if (filter0Type >= 3) {
				if (gpEncNew == 0) {
					gpEncValue = (int32_t)gfACTIV_IIR[0].QFACTOR;
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if ((filter0Type == 5) || (filter0Type == 6))  {
						//QFACTOR describes bandwidth
						if (gpEncValue < 18) gpEncValue = 18;
						if (gpEncValue >111) gpEncValue =111;
														//Bw limited to 4978Hz
					}
					else  {
						//QFACTOR describes Zeta(Resonance)
						if (gpEncValue < 1) gpEncValue = 1;
						if (gpEncValue > 100) gpEncValue = 100;
					}
					if (gpEncValue != gpEncOldValue) {
						gfACTIV_IIR[0].QFACTOR=(uint32_t)gpEncValue;
						reInitFilter(0);		//performs packFilter()
						redrawRouteLine1();
						placeCursor(6) ;
					}
				}
			}
		}

		if (modRoute == 2)  {
			if (filter0Type >= 3)  {
				if (gpEncNew == 0) {
					gpEncValue=(int32_t)gfACTIV_IIR[0].PEAK;
					gpEncOldValue = gpEncValue;
				}
				else  {
					gpEncNew = 0;
					if (filter0Type >= 6) {//types 6,7,8 = parametric before
						if (gpEncValue < 0) gpEncValue = 0;
						if (gpEncValue > 120) gpEncValue = 120;
					}
					else   {					//type 0 excluded
						if (gpEncValue < 20) gpEncValue = 20;
						if (gpEncValue > 35) gpEncValue = 35;
					}
					if (gpEncValue != gpEncOldValue) {
						if ((gpEncValue ==20)&&(gpEncOldValue != 20)) gpZero=1;
						else if ((gpEncValue<=20)&&(gpEncOldValue >20))gpZero=1;
						else if ((gpEncValue>=20)&&(gpEncOldValue <20))gpZero=1;
						if (gpZero) {
							gpEncValue = 20;
							gpEncOldValue = 20;
						}
						gfACTIV_IIR[0].PEAK = (uint32_t)gpEncValue;
						reInitFilter(0);		//performs packFilter()
						placeCursor(12);
						displayPeak(0);
						placeCursor(11);
					}
				}
			}
		}
		if (modRoute == 3)  {
            if (gpEncNew == 0) {
				gpEncValue = (int32_t)gpFilterSet;
                gpEncOldValue = gpEncValue;
            }
            else  {
                gpEncNew = 0;
                if (gpEncValue < 0) gpEncValue = 0;
                if (gpEncValue > 4) gpEncValue = 4;
                if (gpEncValue != gpEncOldValue) {
					gpFilterSet = (uint32_t)gpEncValue;
					if (gpFilterSet == 0)  displayString(0x10+9,gpDefaultStr);
					else if (gpFilterSet==1) displayString(0x10+9,gpActFiltStr);
					else if (gpFilterSet == 2)  displayString(0x10+9,toMemoStr);
					else if (gpFilterSet == 3) displayString(0x10+9,loadMemStr);
					else  displayString(0x10+9,undoStr);
					displayString(0,gpConfirmStr);
					gpWaitKeyB = 1;
					placeCursor(0);
				}
			}
		}

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (gpWaitKeyB)  {
				restoreFilterSet();
				modRoute = 3;
				cursorPos = 0x10+9;
			}
			else if (modRoute == 0)  {
				if (filter0Type >= 3)  {
					modRoute = 1;
					cursorPos = 6;
				}
				else if (filter0Type)  {
					modRoute = 2;
					cursorPos = 11;
				}
				else  {
					modRoute = 3;
					cursorPos = 0x10+9;
				}
			}
			else if (modRoute == 1)  {
				modRoute = 2;
				cursorPos = 11;
			}
			else if (modRoute == 2)  {
				modRoute = 3;
				cursorPos = 0x10+9;
			}
			else if (modRoute == 3)  {
				modRoute = 0;
				cursorPos = 0;
			}
			placeCursor(cursorPos);
			gpModXRoute = modRoute;
			gpPosXRoute = cursorPos;
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			if (gpWaitKeyB == 0)  {
				if (gpLockKey_B) return;
				gpLockKey_B = 1;
			}
			if (gpWaitKeyB)  {
				do  {
					gpNoServPanel = 1;
					msDelay(50);
					temp = keyCheck();
					if (temp == KEY_BF_PRESSED)  {
						newFilterSet(); 					//resets gpWaitKeyB
						displayString(0, gpOKStr);
						break;
					}
				} 		while (temp  != KEY_IDLE);
				gpNoServPanel = 0;
				if (temp == KEY_IDLE)  {
					restoreFilterSet();						//resets gpWaitKeyB
					displayString(0,noActionStr);
				}
				msDelay(1000);
				redrawRouteLine1();
				modRoute = 3;
				cursorPos = 0x10+9;
			}
			else if (modRoute == 0)  {
				modRoute = 3;
				cursorPos = 0x10+9;
			}
			else if (modRoute == 1)  {
				modRoute = 0;
				cursorPos = 0;
			}
			else if (modRoute == 2)  {
				if (filter0Type >= 3)  {
					modRoute = 1;
					cursorPos = 6;
				}
				else  {
					modRoute = 0;
					cursorPos = 0;
				}
			}
			else if (modRoute == 3)  {
				if (filter0Type)  {
					modRoute = 2;
					cursorPos = 11;
				}
				else  {
					modRoute = 0;
					cursorPos = 0;
				}
			}
			placeCursor(cursorPos);
			gpModXRoute = modRoute;
			gpPosXRoute = cursorPos;
		}
	}

	if (gpZero) {
		msDelay(500);
		gpZero = 0;
	}
}

void newFilterSet(void)   // *************************************************
// ***************************************************************************
{
uint32_t n;

	if(gpFilterSet == 0) {			//change to Default
		saveUndo();
		for(n=0;n<N_FILTER;n++)  activateFilter(n,1);	//,1 = activate default
	}
//(gpFilterSet == 1) //formal option: change to "active Filter",i.e. do nothing
	else if(gpFilterSet == 2) {			//store "active" into "memo"
		saveMemo();
		gpFilterSet = 1;			//don't change active filter set
	}
	else if(gpFilterSet == 3)  {
		saveUndo();
		for(n=0;n<N_FILTER;n++)  {
			if (gfFlexMode == 0)  {
				gfACTIV_IIR[n].TYPE = (uint32_t)gfEasyMemoType[n];
				gfACTIV_IIR[n].FREQ = (uint32_t)gfEasyMemoFreq[n];
				gfACTIV_IIR[n].QFACTOR = (uint32_t)gfEasyMemoQfactor[n];
				gfACTIV_IIR[n].PEAK = (uint32_t)gfEasyMemoPeak[n];
			}
			else  {
				gfACTIV_IIR[n].TYPE = (uint32_t)gfFlexMemoType[n];
				gfACTIV_IIR[n].FREQ = (uint32_t)gfFlexMemoFreq[n];
				gfACTIV_IIR[n].QFACTOR = (uint32_t)gfFlexMemoQfactor[n];
				gfACTIV_IIR[n].PEAK = (uint32_t)gfFlexMemoPeak[n];
			}
			initFilterParams(n,gfACTIV_IIR[n].TYPE,
									gfACTIV_IIR[n].FREQ, gfACTIV_IIR[n].QFACTOR,
														gfACTIV_IIR[n].PEAK,1);
								//don't refresh gfACTIV_IIR[],don't packFilter()
		}
	}
	else  {		//gpFilterSet = 4 (Undo)
		for(n=0;n<N_FILTER;n++)  {
			if (gfFlexMode== 0)  {
				gfACTIV_IIR[n].TYPE = (uint32_t)gfEasyUndoType[n];
				gfACTIV_IIR[n].FREQ = (uint32_t)gfEasyUndoFreq[n];
				gfACTIV_IIR[n].QFACTOR = (uint32_t)gfEasyUndoQfactor[n];
				gfACTIV_IIR[n].PEAK = (uint32_t)gfEasyUndoPeak[n];
			}
			else  {
				gfACTIV_IIR[n].TYPE = (uint32_t)gfFlexUndoType[n];
				gfACTIV_IIR[n].FREQ = (uint32_t)gfFlexUndoFreq[n];
				gfACTIV_IIR[n].QFACTOR = (uint32_t)gfFlexUndoQfactor[n];
				gfACTIV_IIR[n].PEAK = (uint32_t)gfFlexUndoPeak[n];
			}
			initFilterParams(n,gfACTIV_IIR[n].TYPE,
									gfACTIV_IIR[n].FREQ, gfACTIV_IIR[n].QFACTOR,
														gfACTIV_IIR[n].PEAK,1);
								//don't refresh gfACTIV_IIR[],don't packFilter()
		}
	}
	restoreFilterSet();
}


void saveUndo(void)   // ****************************************************
// ***************************************************************************
{
uint32_t n;

		for(n=0;n<N_FILTER;n++)  {
			if (gfFlexMode == 0)  {
				gfEasyUndoType[n] = (uint8_t)gfACTIV_IIR[n].TYPE;
				gfEasyUndoFreq[n] = (uint8_t)gfACTIV_IIR[n].FREQ;
				gfEasyUndoQfactor[n] = (uint8_t)gfACTIV_IIR[n].QFACTOR;
				gfEasyUndoPeak[n] = (uint8_t)gfACTIV_IIR[n].PEAK;
			}
			else  {
				gfFlexUndoType[n] = (uint8_t)gfACTIV_IIR[n].TYPE;
				gfFlexUndoFreq[n] = (uint8_t)gfACTIV_IIR[n].FREQ;
				gfFlexUndoQfactor[n] = (uint8_t)gfACTIV_IIR[n].QFACTOR;
				gfFlexUndoPeak[n] = (uint8_t)gfACTIV_IIR[n].PEAK;
			}
		}
}


void saveMemo(void)   // ************************  ****************************
// ***************************************************************************
{
uint32_t n;

		for(n=0;n<N_FILTER;n++)  {
			if (gfFlexMode == 0)  {
				gfEasyMemoType[n] = (uint8_t)gfACTIV_IIR[n].TYPE;
				gfEasyMemoFreq[n] = (uint8_t)gfACTIV_IIR[n].FREQ;
				gfEasyMemoQfactor[n] = (uint8_t)gfACTIV_IIR[n].QFACTOR;
				gfEasyMemoPeak[n] = (uint8_t)gfACTIV_IIR[n].PEAK;
			}
			else  {
				gfFlexMemoType[n] = (uint8_t)gfACTIV_IIR[n].TYPE;
				gfFlexMemoFreq[n] = (uint8_t)gfACTIV_IIR[n].FREQ;
				gfFlexMemoQfactor[n] = (uint8_t)gfACTIV_IIR[n].QFACTOR;
				gfFlexMemoPeak[n] = (uint8_t)gfACTIV_IIR[n].PEAK;
			}
		}
}


void restoreFilterSet(void)   // *********************************************
// ***************************************************************************
{
	gpWaitKeyB = 0;
	gpFilterSet = 1;
	redrawRouteLine1();
	redrawRouteLine2();
	placeCursor(0x10+9);
}


void initSystem(void)   // ***************************************************
// ***************************************************************************
{
const uint8_t MemoryStr[] =  {"Configure System"};
const uint8_t VidPidStr[] =  {"Vid:    Pid:    "};

	gpModSystem = 0;
	gpModXSystem = 0;
    gpSetNumber = 0;

    if (gpPlusState == 0)  {
	    displayString(0,MemoryStr);
		displayString(0x10,gpSelectStr);
		gpTempCh = gChannel;
		gpTempDrumsCh = gDrumsChannel;
    }
    else  {
		displayString(0,VidPidStr);
		displayString(0x10,gpSelectStr);
		gpTempVid = gVid;
		gpTempPid = gPid;
		redrawSystemLine1();
		placeCursor(4);
    }
    gpEncValue = 0;
}


void redrawSystemLine1(void)   // ********************************************
// ***************************************************************************
{
uint32_t  modSystem;
const uint8_t LoadStr[] =     {"Load Flash>Activ"};
const uint8_t SaveStr[] =     {"Burn Activ>Flash"};
const uint8_t CopySoundStr[] ={"CopySnd#?Enc:0  "};;
const uint8_t DumpSoundStr[] ={"DumpSnd#?Enc:All"};
const uint8_t NotesChStr[] =  {"NotesMiCh?Enc:  "};
const uint8_t DrumsChStr[] =  {"DrumsMiCh?Enc:  "};
const uint8_t USBClassStr[] = {"USBClass?Enc:   "};
const uint8_t UsartStr[] =    {"USART?Enc:      "};
const uint8_t ErrorStr[] =    {"  Write Error ! "};

	if (gpPlusState == 0)  {
		modSystem = gpModSystem;
		if (modSystem == 0)  {
			displayString(0,LoadStr);
			if (gpKeyState == KEY_F_PRESSED) displayString(0x10,gpConfirmStr);
			else displayString(0x10,gpSelectStr);
		}
		else if (modSystem == 1)  {
			displayString(0,SaveStr);
			if (gpKeyState == KEY_F_PRESSED) displayString(0x10,gpConfirmStr);
			else displayString(0x10,gpSelectStr);
		}
	else if (modSystem == 2)  {
			displayString(0,CopySoundStr);
			if (gpKeyState == KEY_F_PRESSED) displayString(0x10,gpConfirmStr);
			else displayString(0x10,gpSelectStr);
			placeCursor(12);
		}
		else if (modSystem == 3)  {
			displayString(0,DumpSoundStr);
			displayString(0x10,gpSelectStr);
			placeCursor(12);
		}
		else if (modSystem == 4)  {
			displayString(0,NotesChStr);
			placeCursor(14);
			displayNL99(gChannel+1,0);
			gpSetNumber = gChannel + 1;
			if (gpKeyState == KEY_F_PRESSED) displayString(0x10,gpConfirmStr);
			else displayString(0x10,gpSelectStr);
			placeCursor(13);
		}
		else if (modSystem == 5)  {
			displayString(0,DrumsChStr);
			placeCursor(14);
			displayNL99(gDrumsChannel+1,0);
			gpSetNumber = gDrumsChannel + 1;
			if (gpKeyState == KEY_F_PRESSED) displayString(0x10,gpConfirmStr);
			else displayString(0x10,gpSelectStr);
			placeCursor(13);
		}
		else if (modSystem == 6)  {
			if (GPIOA->IDR & GPIO_IDR_IDR_15)  {		//USB_Vdd present?
				displayString(0,USBClassStr);
				if (guDeviceClass == MIDI_CLASS) displayString(13,gMidiStr);
				else  displayString(13,gComStr);
				gpSetNumber = guDeviceClass;
				displayString(0x10,gpConfirmStr);
				placeCursor(13);
			}
		}
#ifdef CUT_OFF
		else if (modSystem == 7)  {
			displayString(0,UsartStr);
//			if (gUsartCOM) gpSetNumber = 0;
//			else gpSetNumber = 1;//exchange for better displ.sequence of options
			if (gUsartCOM == 0) displayString(13,gMidiStr);
			else  displayString(13,gComStr);
			gpSetNumber = gUsartCOM;
			displayString(0x10,gpConfirmStr);
			placeCursor(13);
		}
#endif
	}
	else  {
		modSystem = gpModXSystem;
		placeCursor(4);
		displayHexWord(gpTempVid);
		placeCursor(12);
		displayHexWord(gpTempPid);
		if (modSystem < 4)  placeCursor(4 + modSystem);
		else if (modSystem < 8)  placeCursor(8 + modSystem);
	}
}


void controlSystem(void)   // ************************************************
// ***************************************************************************
{
uint32_t  modSystem, cursorPos;
uint32_t temp;
const uint8_t ErrorStr[] =    {"  Write Error ! "};

	if (gpPlusState == 0)  {
		modSystem = gpModSystem;

		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
			if (modSystem == 0)  {
				modSystem = 1;
				redrawSystemLine1();
			}
			else if (modSystem == 1)  {
				modSystem = 2;
				redrawSystemLine1();
			}
			else if (modSystem == 2)  {
				modSystem = 3;
				redrawSystemLine1();
			}
			else if (modSystem == 3)  {
				modSystem = 4;
				redrawSystemLine1();
			}
			else if (modSystem == 4)  {
				modSystem = 5;
				redrawSystemLine1();
			}
			else if (modSystem == 5)  {
				modSystem = 6;
				redrawSystemLine1();
				if(guConfigured == 0)  modSystem = 7;
			}
			else if (modSystem == 6)  {
				modSystem = 7;
				redrawSystemLine1();
			}
#ifdef CUT_OFF
			else if (modSystem == 7)  {
				modSystem = 8;
				redrawSystemLine1();
			}
			else if (modSystem == 8)  {
#else
			else if (modSystem == 7)  {
#endif
				gpSetNumber = 0;
				initSystem();
				return;
			}
			gpModSystem = modSystem;
		}

		if ((modSystem == 3) || (modSystem == 4)) {
			if (gpEncNew == 0) {
				gpEncValue = (int32_t)gpSetNumber;
				gpEncOldValue = gpEncValue;
			}
			else  {
				gpEncNew = 0;
				if (gpEncValue <= 0) gpEncValue = 0;
				if (modSystem == 3)  {					//Copy Sound
					if (gpEncValue >= (N_SOUNDS - 1)) gpEncValue = N_SOUNDS - 1;
				}
				if (modSystem == 4)  {					//Dump Sound
					if (gpEncValue >= (N_SOUNDS)) gpEncValue = N_SOUNDS;
				}
				if (gpEncValue != gpEncOldValue) {
					gpSetNumber = (uint32_t)gpEncValue;
					placeCursor(13);
					if (modSystem == 4)  {
						if  (gpSetNumber == 0)  {
							writeLcdData('A');
							writeLcdData('l');
							writeLcdData('l');
						}
						else {
							displayNL99(gpSetNumber-1,0);
							writeLcdData(' ');
						}
					}
					else  {
						displayNL99(gpSetNumber,0);
						writeLcdData(' ');
					}
					placeCursor(12);
				}
			}
		}
		if ((modSystem == 5) || (modSystem == 6))  {		//MIDI Channel
			if (gpEncNew == 0) {
				gpEncValue = (int32_t)gpSetNumber;
				gpEncOldValue = gpEncValue;
			}
			else  {
				gpEncNew = 0;
				if (gpEncValue <= 1) gpEncValue = 1;
				if (gpEncValue >= 16) gpEncValue = 16;
				if (gpEncValue != gpEncOldValue) {
					gpSetNumber = (uint32_t)gpEncValue;
					placeCursor(14);
					displayNL99(gpSetNumber,0);
					placeCursor(13);
				}
			}
		}
		if ((modSystem== 7)&&(GPIOA->IDR&GPIO_IDR_IDR_9)) {	//USB Class
			if (gpEncNew == 0) {
				gpEncValue = (int32_t)gpSetNumber;
				gpEncOldValue = gpEncValue;
			}
			else  {
				gpEncNew = 0;
				if (gpEncValue <= 0) gpEncValue = 0;
				if (gpEncValue >= 1) gpEncValue = 1;
				if (gpEncValue != gpEncOldValue) {
					gpSetNumber = (uint32_t)gpEncValue;
					if (gpSetNumber == MIDI_CLASS)  displayString(13,gMidiStr);
					else  displayString(13,gComStr);
					placeCursor(13);
				}
			}
		}
#ifdef CUT_OFF
		if (modSystem == 8) {								//USART1:COM/MIDI
			if (gpEncNew == 0) {
				gpEncValue = (int32_t)gpSetNumber;
				gpEncOldValue = gpEncValue;
			}
			else  {
				gpEncNew = 0;
				if (gpEncValue <= 0) gpEncValue = 0;
				if (gpEncValue >= 1) gpEncValue = 1;
				if (gpEncValue != gpEncOldValue) {
					gpSetNumber = (uint32_t)gpEncValue;
					gUsartCOM = gpSetNumber;
					if (gpSetNumber == 0)  displayString(13,gMidiStr);
					else  displayString(13,gComStr);
//bugfix 11Jan23: displayed strings wrongly assigned to gpSetNumber
					placeCursor(13);
				}
			}
		}
#endif
		if (modSystem)  {
			if (gpKeyState == KEY_B_PRESSED)  {
				do  {
					gpNoServPanel = 1;
					msDelay(50);
					temp = keyCheck();
					if (temp == KEY_BF_PRESSED)  {
						if (modSystem == 1)  {
							popFlashSounds();
							activateSoundSet();
						}
						else if (modSystem == 2)  {
							displayString(0x10,gpEmptyStr);
							if (pushFlashSounds() == 0) displayString(0x10,ErrorStr);
							else displayString(0x10, gpOKStr);
						}
						else if (modSystem == 3) copyPreset(gpSetNumber);
						else if (modSystem == 4)  {
							displayString(0x10,gpEmptyStr);
							if (gpSetNumber == 0) dumpAllSoundSets();
							else dumpSoundSet(gpSetNumber - 1);
							displayString(0x10, gpOKStr);
						}
						else if (modSystem == 5)  gChannel = gpSetNumber -1;
						else if (modSystem == 6)  gDrumsChannel = gpSetNumber-1;
						else if (modSystem == 7)  {
							guDeviceClass = gpSetNumber;
#ifdef CUT_OFF
							if (gpSetNumber) {
								guDeviceClass = CDC_CLASS;	//fake reInitUsb()
								GPIOA->MODER &= ~(GPIO_MODER_MODER2);
								GPIOA->MODER |= GPIO_MODER_MODER2_0;  //output
								GPIOA->BSRR = GPIO_BSRR_BR_2; //0;         //LOW
							}
							else  {
								guDeviceClass = MIDI_CLASS;	//fake reInitUsb()
								GPIOA->MODER &= ~(GPIO_MODER_MODER2);  //input
								GPIOA->PUPDR &= ~(GPIO_PUPDR_PUPDR2);  //pull-up
								GPIOA->PUPDR |= GPIO_PUPDR_PUPDR2_0;
							}
							msDelay(2);
#else	//ORIG
							if (gpSetNumber)  guDeviceClass = MIDI_CLASS;
							else  guDeviceClass = CDC_CLASS;
							reInitUSB();
#endif
							displayString(0x10, gpOKStr);
						}
#ifdef CUT_OFF
						else if (modSystem == 8)  {
							if (gpSetNumber) {
								GPIOA->MODER &= ~(GPIO_MODER_MODER3);	//COM is
								GPIOA->MODER |= GPIO_MODER_MODER3_0;	//output
								GPIOA->BSRR = GPIO_BSRR_BR_3;        	//LOW
								gUsartCOM = 1;						//reconfirm
								initUsart1();
							}
							else  {
								GPIOA->MODER &= ~(GPIO_MODER_MODER3);  //MIDI is
								GPIOA->PUPDR &= ~(GPIO_PUPDR_PUPDR3);  //input
								GPIOA->PUPDR |= GPIO_PUPDR_PUPDR3_0;  //pull-up
								gUsartCOM = 0;						//reconfirm
								initUsart1();
							}
							displayString(0x10, gpOKStr);
						}
#endif
						if (modSystem != 2)  displayString(0x10, gpOKStr);
						msDelay(1000);
						initSystem();
						break;
					}
				} while (temp  != KEY_IDLE);
				gpNoServPanel = 0;
				if (temp == KEY_IDLE)  {
					gpModSystem--;
					redrawSystemLine1();
					gpModSystem++;
				}
			}
		}
	}

	else  {		//PlusState:change VID, PID
		modSystem = gpModXSystem;
		if (modSystem >= 8) modSystem = 0;
		if (gpEncNew == 0) {
			if (modSystem < 4)  {
				gpEncValue = (int32_t)((gpTempVid >> ((3-modSystem) * 4)) & 0x0000000F);
			}
			else  {
				gpEncValue = (int32_t)((gpTempPid >> ((7-modSystem) * 4)) & 0x0000000F);
			}
		}
		else  {
            gpEncNew = 0;
            if (gpEncValue < 0) gpEncValue = 0;
            if (gpEncValue > 15) gpEncValue = 15;
            if (gpEncValue != gpEncOldValue) {
				if (modSystem < 4)  {
					gpTempVid &=  ~(0x0000000F << ((3-modSystem) * 4));
					gpTempVid |= (uint32_t)gpEncValue << ((3-modSystem) * 4);
				}
				else  {
					gpTempPid &=  ~(0x0000000F << ((7-modSystem) * 4));
					gpTempPid |= (uint32_t)gpEncValue << ((7-modSystem) * 4);
				}
            }
			displayString(0x10,gpConfirmStr);
            redrawSystemLine1();
		}
		if (gpKeyState == KEY_F_PRESSED)  {
			if (gpLockKey_F) return;
			gpLockKey_F = 1;
				modSystem++;
				if (modSystem >= 8) modSystem = 0;
				gpModXSystem = modSystem;
				redrawSystemLine1();
		}
		if (gpKeyState == KEY_B_PRESSED)  {
			do  {
				gpNoServPanel = 1;
				msDelay(50);
				temp = keyCheck();
				if (temp == KEY_BF_PRESSED)  {
					gVid = gpTempVid;
					gPid = gpTempPid;
					displayString(0x10,gpOKStr);
					msDelay(1000);
					initSystem();
					break;
				}
			} while (temp  != KEY_IDLE);
			gpNoServPanel = 0;
			if (temp == KEY_IDLE)  initSystem();
		}
	}
}

// #########################################################################

void adjPeak3(uint32_t bass)   // *******************************************
//handles PEAK of filter#3
//gfBass ranges always 0..127, 64 is neutral
//filter#3 Peak has to be increased or diminuished due difference from neutral
// **************************************************************************
{
int32_t delta, peak;

	if (gfACTIV_IIR[3].TYPE)  {
		peak = (int32_t)gfACTIV_IIR[3].PEAK;
		delta = (int32_t)bass - 64;
		if (gfACTIV_IIR[3].TYPE >= 6)  {
			if (delta < 0) delta = delta/3;	//shrink range -64..0 to-20..0
			else delta = (delta*157)/100; 	//expand range 0..64 to 0-100
			peak += delta;
			if ( peak < 0)  peak =  0;
			if (peak > 120) peak = 120;
		}
		else  {		//TYPE==0 excluded before
			if (delta < 0)  delta = 0;		//no negative values
			else delta = delta / 4;			//shrink range 0..64 to 0-15
			peak += delta;
			if ( peak < 20)  peak =  20;
			if (peak > 35) peak = 35;
		}
		initFilterParams(3, gfACTIV_IIR[3].TYPE, gfACTIV_IIR[3].FREQ,
											gfACTIV_IIR[3].QFACTOR, peak, 1);
													//init runtime temporarily
	}
}


void adjPeak4(uint32_t treble)   // *****************************************
//handles PEAK of filter#4
//gfTreble ranges always 0..127, 64 is neutral
//filter#4 Peak has to be increased or diminuished due difference from neutral
// **************************************************************************
{
int32_t delta, peak;

	if (gfACTIV_IIR[4].TYPE)  {
		peak = (int32_t)gfACTIV_IIR[4].PEAK;
		delta =  ((int32_t)treble) - 64;
		if (gfACTIV_IIR[4].TYPE >= 6)  {
			if (delta < 0) delta = delta/3;	//shrink range -64..0 to-20..0
			else delta = (delta*157)/100; 	//expand range 0..64 to 0-100
			peak += delta;
			if ( peak < 0)  peak =  0;
			if (peak > 120) peak = 120;
		}
		else  {		//TYPE==0 excluded before
			if (delta < 0)  delta = 0;		//no negative values
			else delta = delta / 4;			//shrink range 0..64 to 0-15
			peak += delta;
			if ( peak < 20)  peak =  20;
			if (peak > 35) peak = 35;
		}
		initFilterParams(4, gfACTIV_IIR[4].TYPE, gfACTIV_IIR[4].FREQ,
											gfACTIV_IIR[4].QFACTOR, peak, 1);
													//init runtime temporarily
	}
}


void displayPeak(uint32_t number)   // ****************************************
// ***************************************************************************
{
uint32_t peak;

    if (gfACTIV_IIR[number].TYPE == 0)  {
        writeLcdData('-');
        writeLcdData('-');
        writeLcdData('-');
        return;
    }
	peak = gfACTIV_IIR[number].PEAK;		//range 0..120

    if (gfACTIV_IIR[number].TYPE >= 6)  {  //type = 6,7,8
        if (peak >= 20)  {
            peak -= 20;						//range 0..100
            if (peak == 100)  {
				writeLcdData('1');
				writeLcdData('0');
				writeLcdData('.');
				writeLcdData('0');
            }
            else  display255(peak*10, 2);	//display 0.00...9.99, step=0.1
        }
        else if (peak) {
			writeLcdData('-');
			writeLcdData('.');
			peak = 20 - peak;				//inverted range 0..20
			display99(peak*5,3);			//display 0.00 ...-1.00, step=0.05
        }
        else  {		//peak==0
			writeLcdData('-');
			writeLcdData('1');
			writeLcdData('.');
			writeLcdData('0');
        }
    }
    else  {				//here range 10...35,type 0
		writeLcdData(':');
		displaySigned(peak,20);
    }
}


void displayQFactor(uint32_t number)   // ************************************
// ***************************************************************************
{
uint32_t filterType, qFactor;

	filterType = gfACTIV_IIR[number].TYPE;

    if (filterType < 3) {
        writeLcdData('-');
        writeLcdData('-');
        writeLcdData('-');
        writeLcdData('-');
        return;
    }
	qFactor = gfACTIV_IIR[number].QFACTOR;
    if ((filterType == 5) || (filterType == 6))
										displayMidiFreq(filterType,qFactor);
    else display255(qFactor,2);
}


void displayMidiFreq(uint32_t type, uint32_t noteval)  // ********************
// ***************************************************************************
{
uint32_t freq, div=0;

	if (type)  {
		freq = (uint32_t)gfMidiFreqTable[noteval] /10;      //range 2 ... 837
		if (freq >= 100)  {
			div = freq / 100;
			writeLcdData(div + 0x30);
			freq = freq - (div * 100);                 //range 23 ... 999
		}
		else writeLcdData('0');
		writeLcdData('.');
		if (freq < 10) {
			writeLcdData('0');
			writeLcdData(freq + 0x30);
		}
		else   {
			div = freq / 10; 							//range 0 ... 9
			writeLcdData(div + 0x30);
			freq = freq - (div * 10);                  //range 0 ... 9
			writeLcdData(freq + 0x30);
		}
	}
	else  {
		writeLcdData('-');
		writeLcdData('-');
		writeLcdData('-');
		writeLcdData('-');
		}
}


void displayDelayVol(int32_t level)   // *************************************
// ***************************************************************************
{
	if (level == 0)  writeLcdData(0); //+/-
    else if (level > 0)  writeLcdData('+');
    else {
        writeLcdData('-');
        level = -level;
    }
    display255(level, 4);
}


void displayDelayTime(void) // ***********************************************
//display range 0.0 ... 999.9
// ***************************************************************************
{
uint32_t byte;

//gDelayTime is coded specially :   bits 8-15 contain hundreds and tens,
//                                  bits 0-7 contain ones and tenths
//formally displayed range 0 to 3000, but dot inserted before last place
    byte = (gDelayTime >> 8) & 0x000000FF;
    display99(byte, 0);
    byte = gDelayTime & 0x000000FF;
    display99(byte,1);

}

// ###########################################################################

void displayString(uint32_t position, const uint8_t *string)   // ************
// ***************************************************************************
{
uint8_t byte;
    placeCursor(position);
    while  ((byte = *string++) != 0)  writeLcdData(byte);
}


void displayHexWord(uint32_t word)   // **************************************
// ***************************************************************************
{
uint32_t byte;

    byte = (word >> 8) & 0x000000FF;
    displayHexByte(byte);
    byte = word & 0x000000FF;
    displayHexByte(byte);
}


void displayHexByte(uint32_t byte)   // **************************************
// ***************************************************************************
{
    byte &= 0x000000FF;                     //precaution
    displayHexNibble(byte>>4);
    displayHexNibble(byte & 0x0000000F);
}


void displayHexNibble(uint32_t nibble)   // **********************************

// ***************************************************************************
{
    if (nibble < 10) writeLcdData(nibble + 0x30);
    else writeLcdData(nibble + 0x37);
}


void display255(uint32_t byte, uint32_t dot)  // *****************************
//for some display situations, exceptionally "byte" may range up to 599
//Option 'dot:
//=0 display small values without leading zero, <10 placed at cursor pos+2
//=1:display with decimal dot after first place
//=2:display with decimal dot after second place
//=3:display up to 3 places without leading Space,<10 placed at cursor pos+1
//=4:special mode for display PEAK
//=5:display up to 3 places without leading Space,<10 placed at cursor pos
// ***************************************************************************
{
uint32_t hundreds;

     if ((byte < 10) && (dot == 0))  {
        writeLcdData(' ');
        writeLcdData(' ');
        writeLcdData(byte + 0x30);
        return;
    }
     if ((byte < 100) && (dot == 3))  {
		display99(byte,0);
        writeLcdData(' ');
        return;
    }
     if ((byte < 100) && (dot == 4))  {
        displayNL99(byte,0);
        writeLcdData(' ');
        return;
    }
     if ((byte < 100) && (dot == 5))  {
		if (byte >= 10)  {
			writeLcdData(':');
			displayNL99(byte,0);
		}
		else  {
			display99(byte,5);
			writeLcdData(' ');
		}
        return;
    }
	if (byte >= 100)  {
		hundreds = byte / 100;
        writeLcdData(hundreds + 0x30);
        byte -= hundreds * 100;
    }
    else {
        if (dot == 2) writeLcdData('0');
        else writeLcdData(' ');
    }
    if (dot == 2)  {
        writeLcdData('.');
    }
    if ((dot == 0) || (dot == 5)) display99(byte,2);
    else display99(byte,dot);
}


void display99(uint32_t byte, uint32_t dot)  // ******************************
// display small bytes with leading Space or Zero
// ***************************************************************************
{
uint32_t div;

    if (byte > 99) {
		writeLcdData('^');
		writeLcdData('^');
		return;
    }
    div = byte / 10;
    if (div == 0)  {	//byte<10
        if(dot == 0) writeLcdData(' ');
        else if(dot == 5) writeLcdData(':');
        else writeLcdData('0');
    }
    else writeLcdData(div + 0x30);
    if (dot == 1) writeLcdData('.');
    writeLcdData(byte - (div * 10) + 0x30);
}


void displayNL99(uint32_t byte, uint32_t dot)  // ****************************
// display small bytes without leading Space or Zero
// ***************************************************************************
{
uint32_t div;

    if (byte > 99) {
		writeLcdData('^');
		writeLcdData('^');
		return;
    }
    div = byte / 10;
   if (div == 0) {
        if (dot) writeLcdData('0');
    }
    else  writeLcdData(div + 0x30);
    if (dot) writeLcdData('.');
    writeLcdData(byte - (div * 10) + 0x30);
    if (div == 0) writeLcdData(' ');
}


void displaySigned(uint32_t byte, uint32_t zero)  // *************************
// display number 0 ... 127 signed as -zero ... +99 (-zero >= -99 !)
// ***************************************************************************
{
    if (byte == zero)  {
	    writeLcdData(0);
	    byte = 0;
    }
    else if (byte < zero)  {
        writeLcdData('-');
        byte = zero - byte;
    }
    else  {
		writeLcdData('+');
		byte -= zero;
    }
    if (byte < 100) displayNL99(byte, 0);
    else display255(byte,3);
}


void wait1ms(void)   // *******************************************************
// blocking delay ca 1 millisecond
// *************************************************************************
{
uint32_t countdown;

    countdown = 80;
    while (countdown)  {
        wait6us();
        countdown-- ;
    }
}


void placeCursor(uint32_t position)   // ************************************
//2-Line Display 2x16
//Parameter : HighNibble Line(0,1), LowNibble Column(0...F)
// *************************************************************************
{
uint32_t line, column;

	gpCursorPos = position;
	if (position >= 0x20) position -= 0x20;

    line = position & 0x00000030;
    column = position & 0x0000000F;
    switch (line)  {
        case 0x10:
            column |= 0x000000C0;
            break;
        case 0:
        default:
            column |= 0x00000080;
            break;
    }
    writeLcdInstr(column);  //"set DDRAM address"
}


void initDisplay(void)   // **************************************************
//level NOT inverted for DOG LCD 3.3V operation
//PC9=SCLK
//PC8=MOSI(DAT)
//PC6=/CS
//PC5=RS
// ***************************************************************************
{
	GPIOC->MODER &= ~(GPIO_MODER_MODER5 | GPIO_MODER_MODER6 |
						GPIO_MODER_MODER8 | GPIO_MODER_MODER9);	//neutralize
	GPIOC->MODER |= (GPIO_MODER_MODER5_0 | GPIO_MODER_MODER6_0 |
					GPIO_MODER_MODER8_0 | GPIO_MODER_MODER9_0);	//output
	GPIOC->OTYPER &= ~(GPIO_OTYPER_OT_5 | GPIO_OTYPER_OT_6 |
					GPIO_OTYPER_OT_8 | GPIO_OTYPER_OT_9);	//push/pull out
	GPIOC->BSRR = GPIO_BSRR_BS_9;		//idle SCLK HIGH
	GPIOC->BSRR = GPIO_BSRR_BR_8;		//data LOW
	GPIOC->BSRR = GPIO_BSRR_BS_6;		// /CS HIGH
	GPIOC->BSRR = GPIO_BSRR_BR_5;		//RS LOW
	msDelay(100);

    writeLcdInstr(0x39);    //Function set:8bit,2lines,instr.table 1
    msDelay(2);
    writeLcdInstr(0x14);    //Bias1/5, 2 line LCD
    msDelay(2);
    writeLcdInstr(0x55);    //Booster On, Contrast C5, set C4
    msDelay(2);
    writeLcdInstr(0x6E); //(0x6D);    //set voltage follower and gain
    msDelay(200);
    writeLcdInstr(0x71); //(0x78);    //set contrast C1,C2,C3
    msDelay(2);
    writeLcdInstr(0x38);    //switch back to instruction table 0
    msDelay(2);
    writeLcdInstr(0x0F);    //Cursor ON ,no blink
    msDelay(2);
    writeLcdInstr(0x01);    //delete display, cursor home
    msDelay(2);
    writeLcdInstr(0x06);    //cursor auto-increment
    msDelay(10);

//define +/- at CGRAM position 0
    writeLcdInstr(0x40);
    msDelay(10);
    writeLcdData(0);
    msDelay(10);
    writeLcdData(4);
    msDelay(10);
    writeLcdData(0x1F);
    msDelay(10);
    writeLcdData(4);
    msDelay(10);
    writeLcdData(0);
    msDelay(10);
    writeLcdData(0x1F);
    msDelay(10);
    writeLcdData(0);
    msDelay(10);
    writeLcdData(0);
    msDelay(10);
//define Tilde('~') at CGRAM position 1
    writeLcdData(0);
    msDelay(10);
    writeLcdData(0);
    msDelay(10);
    writeLcdData(8);
    msDelay(10);
    writeLcdData(0x15);
    msDelay(10);
    writeLcdData(2);
    msDelay(10);
    writeLcdData(0);
    msDelay(10);
    writeLcdData(0);
    msDelay(10);
    writeLcdData(0);
    msDelay(10);

}


void displaySpi(uint32_t byte)  // ****************************************
//PC9=SCLK
//PC8=MOSI(DAT)
//PC6=/CS
//PC5=RS
// ************************************************************************
{
#define DELAY 10

uint32_t k, loops;

	GPIOC->BSRR = GPIO_BSRR_BR_6;						// /CS LOW

	loops = DELAY;
	while (loops) {
		IWDG->KR = 0x0000AAAA;
		//feed watchdog,empty loop ignored by compiler optimization
		loops--;
	}

	k = 8;
	do  {
		GPIOC->BSRR = GPIO_BSRR_BR_9;							//SCLK Low
		k--;
		if (byte & (1 << k))  GPIOC->BSRR = GPIO_BSRR_BS_8;		//MOSI high
		else GPIOC->BSRR = GPIO_BSRR_BR_8;						//MOSI Low

		loops = DELAY;
		while (loops) {
			IWDG->KR = 0x0000AAAA;
			//feed watchdog,empty loop ignored by compiler optimization
			loops--;
		}

		GPIOC->BSRR = GPIO_BSRR_BS_9;				//SCLK High

		loops = DELAY;
		while (loops) {
			IWDG->KR = 0x0000AAAA;
			//feed watchdog,empty loop ignored by compiler optimization
			loops--;
		}
	}  while (k);

	GPIOC->BSRR = GPIO_BSRR_BS_9;					//confirm idle SCLK High

	loops = DELAY;
	while (loops) {
		IWDG->KR = 0x0000AAAA;
		//feed watchdog,empty loop ignored by compiler optimization
		loops--;
	}

	GPIOC->BSRR = GPIO_BSRR_BS_6;					// /CS HIGH again
	wait6us();
	wait6us();
	wait6us();
	wait6us();
	wait6us();
	wait6us();
}


void writeLcdInstr(uint32_t instruction)   // *******************************
//PC6 is RS ->output LOW
// **************************************************************************
{

    GPIOC->BSRR = GPIO_BSRR_BR_5;       //clear LCD_RS
    wait6us();
	displaySpi(instruction);
}


void writeLcdData(uint32_t data)   // ***************************************
//PC6 is RS ->output HIGH
// *************************************************************************
{
    GPIOC->BSRR = GPIO_BSRR_BS_5;      //set LCD_RS
    wait6us();
    if (data == 0x7E)  data = 1; //tilde
	displaySpi(data);
	gpCursorPos++;
}

// ###########################################################################

void evalEncoder(void)   // *************************************************
//left rotary encoder
//debounce and evaluate encoders. Update global variables
// ***************************************************************************
{
volatile uint32_t encoderIn;
static int32_t  sEncLazy;

    encoderIn =  (GPIOC->IDR >> 14) & 0x00000003;  //PC14,15=Encoder,now bits0,1
    if (sEncLazy < 256) sEncLazy++ ;
    if (encoderIn != gpEncState)  {
        gpEncState = encoderIn;
//algorithm for encoder type ALPS STEC11B3:
//this model performs one complete quadrature period per raster step
//input state after one full quadrature period is both lines high
//input state after one half quadrature period is both lines low
//rotation clockwise makes intermediate state sequence: 3 1 0 2 3
//rotation counter clockwise makes intermediate state sequence: 3 2 0 1 3
//the 3 intermediate states are shifted from right into a byte
//the result is evaluated at the next full quadrature period
        gpEncCheck = (gpEncCheck<<2) | encoderIn;
        if (encoderIn == 3) {
//successful clockwise rotation results in gpEncoderCheck=0x87
//successful counter clockwise rotation results in gpEncoderCheck=0x4B
            gpEncCheck &= 0x000000FF;
            if (gpEncCheck == 0x87)  {
                if (sEncLazy && (sEncLazy < 0x20)) gpEncValue += 8;
                else gpEncValue++;
                gpEncNew = 1;
                sEncLazy = 0;
            }
            if (gpEncCheck==0x4B)  {
                if (sEncLazy && (sEncLazy < 0x20)) gpEncValue -= 8;
                else gpEncValue--;
                gpEncNew = 1;
                sEncLazy = 0;
            }
        }
    }
}

// ##########################################################################

