// Tone generation specific routines of "NucleoSynth-v3.9" *******************
// digital Audio Synthesizer based on STM32 Nucleo F446
// Author and (c)2019-22 Wolfgang Schemmert

// The copyright claim and GPL-3 License is restricted to this file and the
// depending files
// "main.c", "main.h, "comm.c", "panel.c", "filter.c", "dacfreq.c","drums.c",
// "usb-otg-4611.c" and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and EMBITZ.org

// Contact: www.midi-and-more.de
// A detailed hardware diy construction manual and a user operating manual
// is provided at this website

// for the IIR filter algorithms used here, see detailled description:
// www.IowaHills.com/A4IIRBilinearTransform.html

// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f4xx_modified.h" //modifications see top lines of this file
#include "globaldefines.h"

//from "main"
extern uint32_t gTIMEOUT;
extern int32_t gTest1, gTest2; // debug only
extern float gTestF;
extern uint32_t gSoundSet;
extern uint32_t gNTrigger[N_SOUNDS];
extern uint32_t gLfo1Ptr, gLfo1Slope, gLfo2Ptr, gLfo2Slope, gLfo3Ptr,gLfo3Slope;
extern float gModCent1, gModCent2;
extern void servSystem(void);
//here used for debug only:
extern void sendB(uint8_t);
extern void sendM(uint8_t);
extern void sendX8(uint8_t);
extern void sendX16(uint16_t);
extern void sendX32(uint32_t);
extern void sendS32(uint32_t);
extern void sendE(float);

//from "panel.c"
extern void adjPeak3(uint32_t);
extern void adjPeak4(uint32_t);

//from "dacfreq.c"
extern uint32_t gaRouteOsc1, gaRouteOsc2;
extern uint32_t gaNoise;
extern uint32_t gaFilterTest;
extern int32_t gaCHORUSBUF[MAXCHORUS];

//from "filter.c"
extern const float gfMidiFreqTable[128];

extern struct IIR {  //each 9words
float A1;
float A2;
float B0;
float B1;
float B2;
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
uint32_t QFACTOR;           //bandwidth or 100*zeta(1-100)
uint32_t PEAK;
} gfACTIV_IIR[N_FILTER];
//are recalculated from gfIIRPack[] when sound or routing mode changes

extern struct{  //each 9words
float A1;
float A2;
float B0;
float B1;
float B2;
uint32_t TYPE;
uint32_t FREQ;              //= MIDI note !!
float QFMOD;                //precalculated during initFilterParams()
float FPEAKMOD;             //precalculated during initFilterParams()
}gfRuntimeIIR[N_FILTER]; //each 9word

extern float gfF0TempPeak, gfF0TempQF;	//copy of QFMOD for FreeAdsr "F0BwR"

extern uint32_t gfRuntimeMask, gfStageMask[N_SOUNDS];
//gFStageMask[] and gfRuntimeMask:
//bits 0,1,2,3,4 are set if filter 0,1,2,3,4 is ON (has executable parameters)
//storage of bits 0,1,2,3,4 is updateded at activateSoundSet()
//bit5 doubles gsMonoPoly to be saved in / loaded from Flash
//bit6 stores active Filter Set: 0=EASY, 1= Filter1,2 FLEX setup
//bit7 stores gsO2Sync (if set, Osc2 wave start is sync. with Osc1 wave start)
//bit 8 not assigned
//bits 9,10 are set for square wave types(calculated instead of const.array)

extern uint32_t gfFilterOffset;
extern uint32_t gfGain;
extern uint32_t gfBass, gfTreble;
extern uint32_t gfBaseFreqF2;
extern float gfFilterFreq0;
extern float gfF0ModBase;
extern void tempInitFilter(uint32_t, uint32_t);
extern void unpackBasTreb(void);

// *******************************
#define SHORT_RELEASE 3     //if active voice gets retriggered

const int16_t gsSinus[1026] = {
//14 bit resolution DAC data for SINUS with 1024 steps
0x0,0x32,0x64,0x96,0xc9,0xfb,0x12d,0x15f,
0x191,0x1c4,0x1f6,0x228,0x25a,0x28c,0x2be,0x2f0,
0x322,0x354,0x386,0x3b8,0x3ea,0x41c,0x44e,0x480,
0x4b2,0x4e3,0x515,0x546,0x578,0x5aa,0x5db,0x60c,
0x63e,0x66f,0x6a0,0x6d1,0x702,0x733,0x764,0x795,
0x7c6,0x7f7,0x827,0x858,0x888,0x8b9,0x8e9,0x919,
0x94a,0x97a,0x9aa,0x9d9,0xa09,0xa39,0xa68,0xa98,
0xac7,0xaf7,0xb26,0xb55,0xb84,0xbb3,0xbe1,0xc10,
0xc3e,0xc6d,0xc9b,0xcc9,0xcf7,0xd25,0xd53,0xd81,
0xdae,0xddb,0xe09,0xe36,0xe63,0xe90,0xebc,0xee9,
0xf15,0xf41,0xf6e,0xf9a,0xfc5,0xff1,0x101c,0x1048,
0x1073,0x109e,0x10c9,0x10f4,0x111e,0x1149,0x1173,0x119d,
0x11c7,0x11f0,0x121a,0x1243,0x126d,0x1296,0x12be,0x12e7,
0x130f,0x1338,0x1360,0x1388,0x13af,0x13d7,0x13fe,0x1425,
0x144c,0x1473,0x149a,0x14c0,0x14e6,0x150c,0x1532,0x1558,
0x157d,0x15a2,0x15c7,0x15ec,0x1610,0x1635,0x1659,0x167c,
0x16a0,0x16c4,0x16e7,0x170a,0x172d,0x174f,0x1771,0x1794,
0x17b5,0x17d7,0x17f8,0x181a,0x183b,0x185b,0x187c,0x189c,
0x18bc,0x18dc,0x18fb,0x191b,0x193a,0x1958,0x1977,0x1995,
0x19b3,0x19d1,0x19ef,0x1a0c,0x1a29,0x1a46,0x1a63,0x1a7f,
0x1a9b,0x1ab7,0x1ad2,0x1aee,0x1b09,0x1b23,0x1b3e,0x1b58,
0x1b72,0x1b8c,0x1ba5,0x1bbe,0x1bd7,0x1bf0,0x1c08,0x1c20,
0x1c38,0x1c50,0x1c67,0x1c7e,0x1c95,0x1cab,0x1cc1,0x1cd7,
0x1ced,0x1d02,0x1d17,0x1d2c,0x1d41,0x1d55,0x1d69,0x1d7d,
0x1d90,0x1da3,0x1db6,0x1dc8,0x1ddb,0x1ded,0x1dfe,0x1e10,
0x1e21,0x1e31,0x1e42,0x1e52,0x1e62,0x1e72,0x1e81,0x1e90,
0x1e9f,0x1ead,0x1ebb,0x1ec9,0x1ed7,0x1ee4,0x1ef1,0x1efe,
0x1f0a,0x1f16,0x1f22,0x1f2d,0x1f38,0x1f43,0x1f4e,0x1f58,
0x1f62,0x1f6c,0x1f75,0x1f7e,0x1f87,0x1f8f,0x1f97,0x1f9f,
0x1fa7,0x1fae,0x1fb5,0x1fbc,0x1fc2,0x1fc8,0x1fce,0x1fd3,
0x1fd8,0x1fdd,0x1fe1,0x1fe5,0x1fe9,0x1fed,0x1ff0,0x1ff3,
0x1ff6,0x1ff8,0x1ffa,0x1ffc,0x1ffd,0x1ffe,0x1fff,0x1fff,
0x2000,0x1fff,0x1fff,0x1ffe,0x1ffd,0x1ffc,0x1ffa,0x1ff8,
0x1ff6,0x1ff3,0x1ff0,0x1fed,0x1fe9,0x1fe5,0x1fe1,0x1fdd,
0x1fd8,0x1fd3,0x1fce,0x1fc8,0x1fc2,0x1fbc,0x1fb5,0x1fae,
0x1fa7,0x1f9f,0x1f97,0x1f8f,0x1f87,0x1f7e,0x1f75,0x1f6c,
0x1f62,0x1f58,0x1f4e,0x1f43,0x1f38,0x1f2d,0x1f22,0x1f16,
0x1f0a,0x1efe,0x1ef1,0x1ee4,0x1ed7,0x1ec9,0x1ebb,0x1ead,
0x1e9f,0x1e90,0x1e81,0x1e72,0x1e62,0x1e52,0x1e42,0x1e31,
0x1e21,0x1e10,0x1dfe,0x1ded,0x1ddb,0x1dc8,0x1db6,0x1da3,
0x1d90,0x1d7d,0x1d69,0x1d55,0x1d41,0x1d2c,0x1d17,0x1d02,
0x1ced,0x1cd7,0x1cc1,0x1cab,0x1c95,0x1c7e,0x1c67,0x1c50,
0x1c38,0x1c20,0x1c08,0x1bf0,0x1bd7,0x1bbe,0x1ba5,0x1b8c,
0x1b72,0x1b58,0x1b3e,0x1b23,0x1b09,0x1aee,0x1ad2,0x1ab7,
0x1a9b,0x1a7f,0x1a63,0x1a46,0x1a29,0x1a0c,0x19ef,0x19d1,
0x19b3,0x1995,0x1977,0x1958,0x193a,0x191b,0x18fb,0x18dc,
0x18bc,0x189c,0x187c,0x185b,0x183b,0x181a,0x17f8,0x17d7,
0x17b5,0x1794,0x1771,0x174f,0x172d,0x170a,0x16e7,0x16c4,
0x16a0,0x167c,0x1659,0x1635,0x1610,0x15ec,0x15c7,0x15a2,
0x157d,0x1558,0x1532,0x150c,0x14e6,0x14c0,0x149a,0x1473,
0x144c,0x1425,0x13fe,0x13d7,0x13af,0x1388,0x1360,0x1338,
0x130f,0x12e7,0x12be,0x1296,0x126d,0x1243,0x121a,0x11f0,
0x11c7,0x119d,0x1173,0x1149,0x111e,0x10f4,0x10c9,0x109e,
0x1073,0x1048,0x101c,0xff1,0xfc5,0xf9a,0xf6e,0xf41,
0xf15,0xee9,0xebc,0xe90,0xe63,0xe36,0xe09,0xddb,
0xdae,0xd81,0xd53,0xd25,0xcf7,0xcc9,0xc9b,0xc6d,
0xc3e,0xc10,0xbe1,0xbb3,0xb84,0xb55,0xb26,0xaf7,
0xac7,0xa98,0xa68,0xa39,0xa09,0x9d9,0x9aa,0x97a,
0x94a,0x919,0x8e9,0x8b9,0x888,0x858,0x827,0x7f7,
0x7c6,0x795,0x764,0x733,0x702,0x6d1,0x6a0,0x66f,
0x63e,0x60c,0x5db,0x5aa,0x578,0x546,0x515,0x4e3,
0x4b2,0x480,0x44e,0x41c,0x3ea,0x3b8,0x386,0x354,
0x322,0x2f0,0x2be,0x28c,0x25a,0x228,0x1f6,0x1c4,
0x191,0x15f,0x12d,0xfb,0xc9,0x96,0x64,0x32,
0x0,0xffce,0xff9c,0xff6a,0xff37,0xff05,0xfed3,0xfea1,
0xfe6f,0xfe3c,0xfe0a,0xfdd8,0xfda6,0xfd74,0xfd42,0xfd10,
0xfcde,0xfcac,0xfc7a,0xfc48,0xfc16,0xfbe4,0xfbb2,0xfb80,
0xfb4e,0xfb1d,0xfaeb,0xfaba,0xfa88,0xfa56,0xfa25,0xf9f4,
0xf9c2,0xf991,0xf960,0xf92f,0xf8fe,0xf8cd,0xf89c,0xf86b,
0xf83a,0xf809,0xf7d9,0xf7a8,0xf778,0xf747,0xf717,0xf6e7,
0xf6b6,0xf686,0xf656,0xf627,0xf5f7,0xf5c7,0xf598,0xf568,
0xf539,0xf509,0xf4da,0xf4ab,0xf47c,0xf44d,0xf41f,0xf3f0,
0xf3c2,0xf393,0xf365,0xf337,0xf309,0xf2db,0xf2ad,0xf27f,
0xf252,0xf225,0xf1f7,0xf1ca,0xf19d,0xf170,0xf144,0xf117,
0xf0eb,0xf0bf,0xf092,0xf066,0xf03b,0xf00f,0xefe4,0xefb8,
0xef8d,0xef62,0xef37,0xef0c,0xeee2,0xeeb7,0xee8d,0xee63,
0xee39,0xee10,0xede6,0xedbd,0xed93,0xed6a,0xed42,0xed19,
0xecf1,0xecc8,0xeca0,0xec78,0xec51,0xec29,0xec02,0xebdb,
0xebb4,0xeb8d,0xeb66,0xeb40,0xeb1a,0xeaf4,0xeace,0xeaa8,
0xea83,0xea5e,0xea39,0xea14,0xe9f0,0xe9cb,0xe9a7,0xe984,
0xe960,0xe93c,0xe919,0xe8f6,0xe8d3,0xe8b1,0xe88f,0xe86c,
0xe84b,0xe829,0xe808,0xe7e6,0xe7c5,0xe7a5,0xe784,0xe764,
0xe744,0xe724,0xe705,0xe6e5,0xe6c6,0xe6a8,0xe689,0xe66b,
0xe64d,0xe62f,0xe611,0xe5f4,0xe5d7,0xe5ba,0xe59d,0xe581,
0xe565,0xe549,0xe52e,0xe512,0xe4f7,0xe4dd,0xe4c2,0xe4a8,
0xe48e,0xe474,0xe45b,0xe442,0xe429,0xe410,0xe3f8,0xe3e0,
0xe3c8,0xe3b0,0xe399,0xe382,0xe36b,0xe355,0xe33f,0xe329,
0xe313,0xe2fe,0xe2e9,0xe2d4,0xe2bf,0xe2ab,0xe297,0xe283,
0xe270,0xe25d,0xe24a,0xe238,0xe225,0xe213,0xe202,0xe1f0,
0xe1df,0xe1cf,0xe1be,0xe1ae,0xe19e,0xe18e,0xe17f,0xe170,
0xe161,0xe153,0xe145,0xe137,0xe129,0xe11c,0xe10f,0xe102,
0xe0f6,0xe0ea,0xe0de,0xe0d3,0xe0c8,0xe0bd,0xe0b2,0xe0a8,
0xe09e,0xe094,0xe08b,0xe082,0xe079,0xe071,0xe069,0xe061,
0xe059,0xe052,0xe04b,0xe044,0xe03e,0xe038,0xe032,0xe02d,
0xe028,0xe023,0xe01f,0xe01b,0xe017,0xe013,0xe010,0xe00d,
0xe00a,0xe008,0xe006,0xe004,0xe003,0xe002,0xe001,0xe001,
0xe000,0xe001,0xe001,0xe002,0xe003,0xe004,0xe006,0xe008,
0xe00a,0xe00d,0xe010,0xe013,0xe017,0xe01b,0xe01f,0xe023,
0xe028,0xe02d,0xe032,0xe038,0xe03e,0xe044,0xe04b,0xe052,
0xe059,0xe061,0xe069,0xe071,0xe079,0xe082,0xe08b,0xe094,
0xe09e,0xe0a8,0xe0b2,0xe0bd,0xe0c8,0xe0d3,0xe0de,0xe0ea,
0xe0f6,0xe102,0xe10f,0xe11c,0xe129,0xe137,0xe145,0xe153,
0xe161,0xe170,0xe17f,0xe18e,0xe19e,0xe1ae,0xe1be,0xe1cf,
0xe1df,0xe1f0,0xe202,0xe213,0xe225,0xe238,0xe24a,0xe25d,
0xe270,0xe283,0xe297,0xe2ab,0xe2bf,0xe2d4,0xe2e9,0xe2fe,
0xe313,0xe329,0xe33f,0xe355,0xe36b,0xe382,0xe399,0xe3b0,
0xe3c8,0xe3e0,0xe3f8,0xe410,0xe429,0xe442,0xe45b,0xe474,
0xe48e,0xe4a8,0xe4c2,0xe4dd,0xe4f7,0xe512,0xe52e,0xe549,
0xe565,0xe581,0xe59d,0xe5ba,0xe5d7,0xe5f4,0xe611,0xe62f,
0xe64d,0xe66b,0xe689,0xe6a8,0xe6c6,0xe6e5,0xe705,0xe724,
0xe744,0xe764,0xe784,0xe7a5,0xe7c5,0xe7e6,0xe808,0xe829,
0xe84b,0xe86c,0xe88f,0xe8b1,0xe8d3,0xe8f6,0xe919,0xe93c,
0xe960,0xe984,0xe9a7,0xe9cb,0xe9f0,0xea14,0xea39,0xea5e,
0xea83,0xeaa8,0xeace,0xeaf4,0xeb1a,0xeb40,0xeb66,0xeb8d,
0xebb4,0xebdb,0xec02,0xec29,0xec51,0xec78,0xeca0,0xecc8,
0xecf1,0xed19,0xed42,0xed6a,0xed93,0xedbd,0xede6,0xee10,
0xee39,0xee63,0xee8d,0xeeb7,0xeee2,0xef0c,0xef37,0xef62,
0xef8d,0xefb8,0xefe4,0xf00f,0xf03b,0xf066,0xf092,0xf0bf,
0xf0eb,0xf117,0xf144,0xf170,0xf19d,0xf1ca,0xf1f7,0xf225,
0xf252,0xf27f,0xf2ad,0xf2db,0xf309,0xf337,0xf365,0xf393,
0xf3c2,0xf3f0,0xf41f,0xf44d,0xf47c,0xf4ab,0xf4da,0xf509,
0xf539,0xf568,0xf598,0xf5c7,0xf5f7,0xf627,0xf656,0xf686,
0xf6b6,0xf6e7,0xf717,0xf747,0xf778,0xf7a8,0xf7d9,0xf809,
0xf83a,0xf86b,0xf89c,0xf8cd,0xf8fe,0xf92f,0xf960,0xf991,
0xf9c2,0xf9f4,0xfa25,0xfa56,0xfa88,0xfaba,0xfaeb,0xfb1d,
0xfb4e,0xfb80,0xfbb2,0xfbe4,0xfc16,0xfc48,0xfc7a,0xfcac,
0xfcde,0xfd10,0xfd42,0xfd74,0xfda6,0xfdd8,0xfe0a,0xfe3c,
0xfe6f,0xfea1,0xfed3,0xff05,0xff37,0xff6a,0xff9c,0xffce,0,0x32};


const int16_t gsExpSin[1024] = {
//14 bit resolution DAC data for E-Guitar simulation with 1024 steps
//Burst with exponentially decaying oscillations of 5th harmonic
0x0,0xbc,0x178,0x234,0x2f0,0x3aa,0x464,0x51D,
0x5d4,0x68a,0x73f,0x7f2,0x8a3,0x951,0x9fe,0xAA8,
0xb50,0xbf5,0xc97,0xd36,0xdd1,0xe6a,0xeff,0xF90,
0x101e,0x10a7,0x112d,0x11af,0x122c,0x12a5,0x1319,0x1389,
0x13f4,0x145a,0x14bc,0x1518,0x156f,0x15c2,0x160f,0x1656,
0x1698,0x16d5,0x170c,0x173e,0x176a,0x1791,0x17b1,0x17CD,
0x17e2,0x17f2,0x17fb,0x17ff,0x17fe,0x17f6,0x17e9,0x17D6,
0x17bd,0x179e,0x177a,0x1750,0x1721,0x16ec,0x16b1,0x1671,
0x162c,0x15e1,0x1591,0x153c,0x14e1,0x1482,0x141e,0x13B4,
0x1346,0x12d4,0x125d,0x11e1,0x1161,0x10dd,0x1055,0xFC9,
0xf39,0xea6,0xe0f,0xd74,0xcd7,0xc36,0xb92,0xAEC,
0xa42,0x997,0x8e9,0x839,0x787,0x6d3,0x61d,0x566,
0x4ae,0x3f5,0x33a,0x27f,0x1c3,0x107,0x7d,0xFF44,
0xfe0a,0xfcd1,0xfb98,0xfa60,0xf92a,0xf7f5,0xf6c2,0xF592,
0xf464,0xf339,0xf211,0xf0ec,0xefcb,0xeeae,0xed95,0xEC80,
0xeb70,0xea65,0xe95f,0xe85f,0xe765,0xe670,0xe581,0xE499,
0xe3b8,0xe2dd,0xe209,0xe13d,0xe078,0xdfba,0xdf05,0xDE57,
0xddb1,0xdd14,0xdc7f,0xdbf3,0xdb6f,0xdaf4,0xda82,0xDA19,
0xd9b9,0xd963,0xd916,0xd8d2,0xd897,0xd866,0xd83f,0xD821,
0xd80d,0xd802,0xd801,0xd80a,0xd81c,0xd838,0xd85e,0xD88D,
0xd8c5,0xd907,0xd953,0xd9a7,0xda05,0xda6c,0xdadd,0xDB56,
0xdbd8,0xdc62,0xdcf6,0xdd91,0xde35,0xdee1,0xdf95,0xE051,
0xe115,0xe1e0,0xe2b2,0xe38b,0xe46c,0xe552,0xe640,0xE733,
0xe82d,0xe92c,0xea30,0xeb3a,0xec49,0xed5d,0xee75,0xEF91,
0xf0b2,0xf1d6,0xf2fd,0xf428,0xf555,0xf685,0xf7b8,0xF8EC,
0xfa22,0xfb59,0xfc92,0xfdcb,0xff05,0x3e,0x177,0x2AC,
0x3de,0x50c,0x636,0x75c,0x87e,0x99a,0xab2,0xBC5,
0xcd3,0xddb,0xedd,0xfd9,0x10cf,0x11bf,0x12a8,0x138B,
0x1466,0x153b,0x1609,0x16cf,0x178e,0x1846,0x18f6,0x199E,
0x1a3f,0x1ad7,0x1b68,0x1bf1,0x1c71,0x1cea,0x1d5a,0x1DC2,
0x1e22,0x1e7a,0x1ec9,0x1f11,0x1f50,0x1f86,0x1fb5,0x1FDB,
0x1ff9,0x200f,0x201d,0x2023,0x2021,0x2017,0x2006,0x1FEC,
0x1fcc,0x1fa3,0x1f73,0x1f3c,0x1efe,0x1eb9,0x1e6d,0x1E1A,
0x1dc0,0x1d60,0x1cfa,0x1c8d,0x1c1b,0x1ba2,0x1b24,0x1AA1,
0x1a18,0x198a,0x18f6,0x185e,0x17c2,0x1721,0x167c,0x15D3,
0x1525,0x1475,0x13c1,0x1309,0x124f,0x1192,0x10d2,0x1010,
0xf4b,0xe85,0xdbd,0xcf3,0xc28,0xb5c,0xa8f,0x9C1,
0x8f2,0x823,0x754,0x685,0x5b7,0x4e8,0x41b,0x34E,
0x282,0x1b8,0xef,0x27,0xff62,0xfe9f,0xfddd,0xFD1D,
0xfc60,0xfba6,0xfaee,0xfa39,0xf987,0xf8d8,0xf82d,0xF785,
0xf6e1,0xf640,0xf5a4,0xf50b,0xf476,0xf3e5,0xf359,0xF2D1,
0xf24d,0xf1ce,0xf154,0xf0de,0xf06d,0xf001,0xef9a,0xEF38,
0xeeda,0xee82,0xee2f,0xede1,0xed98,0xed54,0xed16,0xECDC,
0xeca8,0xec79,0xec4f,0xec2b,0xec0b,0xebf1,0xebdc,0xEBCC,
0xebc1,0xebbc,0xebbb,0xebbf,0xebc8,0xebd6,0xebe9,0xEC01,
0xec1d,0xec3e,0xec63,0xec8d,0xecbb,0xecee,0xed25,0xED60,
0xed9f,0xede2,0xee28,0xee73,0xeec1,0xef13,0xef68,0xEFC0,
0xf01c,0xf07a,0xf0dc,0xf140,0xf1a8,0xf211,0xf27e,0xF2EC,
0xf35d,0xf3d0,0xf445,0xf4bb,0xf533,0xf5ad,0xf629,0xF6A5,
0xf723,0xf7a2,0xf822,0xf8a2,0xf924,0xf9a5,0xfa28,0xFAAA,
0xfb2c,0xfbaf,0xfc31,0xfcb4,0xfd36,0xfdb7,0xfe38,0xFEB8,
0xff37,0xffb5,0x31,0xae,0x128,0x1a2,0x21a,0x290,
0x304,0x377,0x3e8,0x457,0x4c4,0x52f,0x598,0x5FE,
0x662,0x6c3,0x722,0x77e,0x7d8,0x82f,0x883,0x8D4,
0x922,0x96e,0x9b6,0x9fc,0xa3e,0xa7d,0xab9,0xAF2,
0xb28,0xb5b,0xb8a,0xbb6,0xbdf,0xc04,0xc27,0xC46,
0xc61,0xc7a,0xc8f,0xca0,0xcaf,0xcba,0xcc3,0xCC7,
0xcc9,0xcc8,0xcc3,0xcbc,0xcb1,0xca3,0xc93,0xC7F,
0xc69,0xc4f,0xc33,0xc14,0xbf3,0xbcf,0xba8,0xB7F,
0xb53,0xb25,0xaf5,0xac2,0xa8d,0xa56,0xa1d,0x9E2,
0x9a6,0x967,0x927,0x8e4,0x8a1,0x85c,0x815,0x7CD,
0x784,0x73a,0x6ee,0x6a2,0x654,0x606,0x5b7,0x567,
0x517,0x4c6,0x474,0x423,0x3d1,0x37e,0x32c,0x2DA,
0x287,0x235,0x1e3,0x191,0x140,0xef,0x9f,0x4F,
0x0,0xffb2,0xff64,0xff17,0xfecb,0xfe80,0xfe36,0xFDED,
0xfda6,0xfd5f,0xfd1a,0xfcd6,0xfc93,0xfc52,0xfc13,0xFBD5,
0xfb98,0xfb5d,0xfb24,0xfaed,0xfab7,0xfa83,0xfa51,0xFA20,
0xf9f2,0xf9c5,0xf99b,0xf972,0xf94b,0xf927,0xf904,0xF8E3,
0xf8c5,0xf8a8,0xf88d,0xf875,0xf85e,0xf84a,0xf838,0xF828,
0xf81a,0xf80d,0xf803,0xf7fb,0xf7f6,0xf7f2,0xf7f0,0xF7F0,
0xf7f2,0xf7f6,0xf7fc,0xf804,0xf80d,0xf819,0xf827,0xF836,
0xf847,0xf85a,0xf86e,0xf884,0xf89c,0xf8b5,0xf8d0,0xF8ED,
0xf90b,0xf92a,0xf94b,0xf96d,0xf990,0xf9b5,0xf9db,0xFA02,
0xfa2a,0xfa53,0xfa7e,0xfaa9,0xfad5,0xfb02,0xfb30,0xFB5F,
0xfb8e,0xfbbe,0xfbef,0xfc20,0xfc52,0xfc84,0xfcb6,0xFCE9,
0xfd1c,0xfd50,0xfd83,0xfdb7,0xfdeb,0xfe1f,0xfe53,0xFE87,
0xfebb,0xfeee,0xff22,0xff55,0xff88,0xffba,0xffed,0x1D,
0x4f,0x7f,0xaf,0xdf,0x10e,0x13c,0x16a,0x197,
0x1c3,0x1ee,0x218,0x241,0x26a,0x292,0x2b8,0x2DE,
0x302,0x326,0x348,0x369,0x389,0x3a8,0x3c6,0x3E3,
0x3fe,0x418,0x431,0x449,0x45f,0x474,0x488,0x49B,
0x4ac,0x4bc,0x4cb,0x4d8,0x4e4,0x4ef,0x4f8,0x500,
0x507,0x50d,0x511,0x514,0x516,0x516,0x515,0x513,
0x510,0x50b,0x506,0x4ff,0x4f7,0x4ee,0x4e3,0x4D8,
0x4cc,0x4be,0x4af,0x4a0,0x48f,0x47e,0x46b,0x458,
0x443,0x42e,0x418,0x401,0x3ea,0x3d1,0x3b8,0x39F,
0x384,0x369,0x34d,0x331,0x315,0x2f7,0x2da,0x2BC,
0x29d,0x27e,0x25f,0x23f,0x220,0x200,0x1df,0x1BF,
0x19e,0x17e,0x15d,0x13c,0x11b,0xfb,0xda,0xB9,
0x99,0x79,0x58,0x38,0x19,0xfffa,0xffdb,0xFFBC,
0xff9e,0xff7f,0xff62,0xff44,0xff27,0xff0b,0xfeef,0xFED4,
0xfeb9,0xfe9e,0xfe85,0xfe6b,0xfe53,0xfe3b,0xfe24,0xFE0D,
0xfdf7,0xfde2,0xfdcd,0xfdb9,0xfda6,0xfd94,0xfd82,0xFD72,
0xfd62,0xfd52,0xfd44,0xfd36,0xfd29,0xfd1d,0xfd12,0xFD08,
0xfcfe,0xfcf5,0xfced,0xfce6,0xfce0,0xfcdb,0xfcd6,0xFCD2,
0xfccf,0xfccd,0xfccc,0xfccb,0xfccb,0xfccc,0xfcce,0xFCD0,
0xfcd4,0xfcd8,0xfcdc,0xfce2,0xfce8,0xfcef,0xfcf7,0xFCFF,
0xfd08,0xfd11,0xfd1c,0xfd27,0xfd32,0xfd3e,0xfd4b,0xFD58,
0xfd65,0xfd74,0xfd82,0xfd91,0xfda1,0xfdb1,0xfdc2,0xFDD3,
0xfde4,0xfdf6,0xfe07,0xfe1a,0xfe2c,0xfe3f,0xfe52,0xFE66,
0xfe79,0xfe8d,0xfea1,0xfeb5,0xfeca,0xfede,0xfef3,0xFF07,
0xff1c,0xff30,0xff45,0xff5a,0xff6e,0xff83,0xff98,0xFFAC,
0xffc0,0xffd5,0xffe9,0xfffd,0xf,0x23,0x36,0x49,
0x5c,0x6f,0x81,0x93,0xa5,0xb6,0xc8,0xD8,
0xe9,0xf9,0x108,0x118,0x127,0x135,0x143,0x151,
0x15e,0x16a,0x177,0x182,0x18e,0x198,0x1a3,0x1AD,
0x1b6,0x1bf,0x1c7,0x1cf,0x1d6,0x1dd,0x1e3,0x1E9,
0x1ee,0x1f3,0x1f7,0x1fb,0x1fe,0x200,0x202,0x204,
0x205,0x206,0x206,0x205,0x204,0x203,0x201,0x1FF,
0x1fc,0x1f9,0x1f5,0x1f1,0x1ec,0x1e7,0x1e2,0x1DC,
0x1d5,0x1cf,0x1c8,0x1c0,0x1b8,0x1b0,0x1a8,0x19F,
0x196,0x18c,0x183,0x179,0x16e,0x164,0x159,0x14E,
0x143,0x137,0x12b,0x120,0x114,0x107,0xfb,0xEF,
0xe2,0xd5,0xc9,0xbc,0xaf,0xa2,0x95,0x88,
0x7b,0x6e,0x61,0x54,0x47,0x3a,0x2d,0x20,
0x14,0x7,0xfffc,0xffef,0xffe3,0xffd7,0xffcb,0xFFBF,
0xffb3,0xffa8,0xff9d,0xff91,0xff87,0xff7c,0xff71,0xFF67,
0xff5d,0xff54,0xff4a,0xff41,0xff38,0xff2f,0xff27,0xFF1F,
0xff17,0xff0f,0xff08,0xff01,0xfefb,0xfef4,0xfeee,0xFEE9,
0xfee3,0xfede,0xfed9,0xfed5,0xfed1,0xfecd,0xfeca,0xFEC7,
0xfec4,0xfec2,0xfebf,0xfebe,0xfebc,0xfebb,0xfeba,0xFEBA,
0xfeba,0xfeba,0xfeba,0xfebb,0xfebc,0xfebd,0xfebf,0xFEC1,
0xfec3,0xfec6,0xfec9,0xfecc,0xfecf,0xfed3,0xfed6,0xFEDB,
0xfedf,0xfee4,0xfee8,0xfeed,0xfef3,0xfef8,0xfefe,0xFF04,
0xff0a,0xff10,0xff17,0xff1d,0xff24,0xff2b,0xff32,0xFF39,
0xff40,0xff48,0xff4f,0xff57,0xff5f,0xff66,0xff6e,0xFF76,
0xff7e,0xff86,0xff8f,0xff97,0xff9f,0xffa7,0xffaf,0xFFB8,
0xffc0,0xffc8,0xffd0,0xffd8,0xffe0,0xffe9,0xfff1,0xFFF9};

int16_t gsWave[4096];
int16_t gsWaveX[2048];
int16_t gsWaveLfo1[4000];
int16_t gsWaveLfo2[4000];
int16_t gsWaveLfo3[4000];
int16_t gsUserWave0[1024];
int16_t gsUserWave1[1024];
int16_t gsUserWave2[1024];
int16_t gsUserWave3[1024];

const float gsFloatStep[128] =  {
//table of number of Forward Steps per Sample in 1024 steps Wave as Float number
//sample Freq = 33.075 kHz
//frequencies < MIDI NOTE 18 and > MIDI NOTE > 108 are not supported (= 0)
0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000,
0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000,
0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000,
0.71593606, 0.75850797, 0.80361116, 0.85139638, 0.90202314, 0.95566028,
1.01248693, 1.07269251, 1.13647819, 1.20405674, 1.27565372, 1.35150814,
1.43187308, 1.51701665, 1.60722327, 1.70279384, 1.80404723, 1.91132152,
2.02497506, 2.14538622, 2.27295756, 2.40811491, 2.55130911, 2.70301771,
2.86374760, 3.03403544, 3.21444821, 3.40558982, 3.60809660, 3.82264543,
4.04995203, 4.29077482, 4.54591751, 4.81623268, 5.10262060, 5.40603828,
5.72749853, 6.06807327, 6.42890024, 6.81118250, 7.21619749, 7.64529514,
8.09990883, 8.58155441, 9.09184074, 9.63247108, 10.20524693, 10.81208229,
11.45500278, 12.13615322, 12.85780811, 13.62237358, 14.43240261, 15.29059887,
16.19982529, 17.16311836, 18.18369102, 19.26494980, 20.41050529, 21.62417793,
22.91002083, 24.27231979, 25.71562958, 27.24476433, 28.86482048, 30.58121300,
32.39966583, 34.32625961, 36.36740494, 38.52992249, 40.82103348, 43.24837875,
45.82006454, 48.54466629, 51.43128967, 54.48955154, 57.72967529, 61.16246033,
64.79936981, 68.65254974, 72.73484802, 77.05988312, 81.64210510, 86.49681091,
91.64017487, 97.08939362, 102.86263275, 108.97917175, 115.45941925,122.32499695,
129.59881592, 0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000,
0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000,
0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000, 0.00000000,
0.00000000, 0.00000000};

const float gsTrackFactor[128]  = {
-0.981, -0.980, -0.979, -0.978, -0.977, -0.975,
-0.974, -0.972, -0.971, -0.969, -0.967, -0.965,
-0.963, -0.961, -0.958, -0.956, -0.953, -0.950,
-0.947, -0.944, -0.941, -0.938, -0.934, -0.930,
-0.926, -0.921, -0.917, -0.912, -0.906, -0.901,
-0.895, -0.889, -0.882, -0.875, -0.868, -0.860,
-0.851, -0.843, -0.833, -0.823, -0.813, -0.802,
-0.790, -0.777, -0.764, -0.750, -0.735, -0.719,
-0.703, -0.685, -0.666, -0.646, -0.625, -0.603,
-0.580, -0.555, -0.528, -0.500, -0.470, -0.439,
-0.405, -0.370, -0.333, -0.293, -0.251, -0.206,
-0.159, -0.109, -0.056, 0.000, 0.059, 0.122,
0.189, 0.260, 0.335, 0.414, 0.498, 0.587,
0.682, 0.782, 0.888, 1.000, 1.119, 1.245,
1.378, 1.520, 1.670, 1.828, 1.997, 2.175,
2.364, 2.564, 2.776, 3.000, 3.238, 3.490,
3.757, 4.040, 4.339, 4.657, 4.993, 5.350,
5.727, 6.127, 6.551, 7.000, 7.476, 7.980,
8.514, 9.079, 9.679, 10.314, 10.986, 11.699,
12.454, 13.254, 14.102, 15.000, 15.951, 16.959,
18.027, 19.159, 20.357, 21.627, 22.973, 24.398,
25.909, 27.509  };

uint32_t gsWAVEFORM_1, gsOsc1ModLfo2, gsWAVEFORM_1Preset[N_SOUNDS];
uint32_t gsWAVEFORM_2, gsOsc2ModLfo2, gsWAVEFORM_2Preset[N_SOUNDS];
int32_t gsTRANS, gsTRANSPreset[N_SOUNDS];
int32_t gsTRANS_2, gsTRANS_2Preset[N_SOUNDS];
int32_t gsDETUNE, gsDETUNEPreset[N_SOUNDS];
int32_t gsDETUNE_2, gsDETUNE_2Preset[N_SOUNDS];
uint32_t gsGLIDE, gsGLIDEPreset[N_SOUNDS];
uint32_t gsMOD_1, gsOsc1PWM, gsMOD_1Preset[N_SOUNDS];
uint32_t gsMOD_2, gsOsc2PWM, gsMOD_2Preset[N_SOUNDS];
int32_t gsPWM1Edge0, gsPWM2Edge0, gsPWM1Hi, gsPWM1Lo, gsPWM2Hi, gsPWM2Lo;
float gsNewPWM1EdgeMod, gsNewPWM2EdgeMod;
uint32_t gsRangeFS1, gsSlopeFS1, gsRangeFS2, gsSlopeFS2;
uint32_t gsFreeSquare1[N_SOUNDS], gsFreeSquare2[N_SOUNDS];
									//bits31-16= gsRangeFS,bits15-0=gsSlopeFS

struct ADSR {			//stored in Flash preset
uint32_t ATDECSUS;		//bits31..24:Attack.bits23..16:Decay,bits7..0:Sustain
uint32_t REFIXVEL;		//bits23..16:Release,bits15..8:Fixvol,bits7..0:Velocity
};
struct ADSR gsADSR[N_SOUNDS*N_ASTRUCT];
//sets of compacted each 7 ADSR structures
//Attack, Decay and Relase times up to 1sec in 10 ms scale,big values compressed
//Sustain bit7 set=hold, else 1x mode

//following arrays save runtime and memory:
//only 1 set for the actual soundSet
//each is reloaded from preset or must be calculated new when ADSR# is changed
//All instances of FREE ADSR share the same parameter preset
//index 0 holds envelope params for OSC1
//index 1 holds envelope params for OSC2
//index 2 holds envelope params for Noise
//index 3 holds values for Frequency Modulation of OSC1+2 commonly
//index 4 holds values for Frequency Modulation of LMO1
//index 5 holds values for Frequency Modulation of Vibrato
//index 6 holds values for Bass up/down
//index 7 holds values for Treble up/down
//index 8 holds envelope params for Gain
//index 9 holds values for Filter0 MidFreqfrequency shift
//index 10 holds values for Filter0 Peak up/down
//index 11 holds values for the actually selected FreeAdsr
//indices 8-N_ASTRUCT-1 hold the values for the different items of FreeAdsr

uint32_t gsAttack[N_ASTRUCT];	//Attack time in 0.625ms units(1600Hz)
uint32_t gsDecay[N_ASTRUCT];	//Decay time in 0.625ms units(1600Hz)
uint32_t gsSustain[N_ASTRUCT];
uint32_t gsHoldSust[N_ASTRUCT];	//=1 for Hold Sustain,=0 for One Shoot
uint32_t gsRelease[N_ASTRUCT];	//Release time in 0.625ms units(1600Hz)
uint32_t gsFixVol[N_ASTRUCT];
uint32_t gsVelo[N_ASTRUCT];

uint32_t gsAdsrOffset, gsNoteFlags, gsNoteCount, gsNoTriggerF0;
uint32_t gsFreeIdx, gsFreeAdsr, gsFreeAdsrPreset[N_SOUNDS];
uint32_t gsNoteBusy[10];
uint32_t gsMonoStack[20];
uint32_t gsMonoStackPtr, gsMonoPoly;  //gsMonoPoly default=0 =polyphonic
uint32_t gsO2Sync;//(if set, Osc2 wave start is sync. with Osc1 wave start)

float gsWavPtr0, gsWavPtr0X, gsWavPtr1, gsWavPtr1X;
float gsWavPtr2, gsWavPtr2X, gsWavPtr3, gsWavPtr3X;
float gsWavPtr4, gsWavPtr4X, gsWavPtr5, gsWavPtr5X;
float gsWavPtr6, gsWavPtr6X, gsWavPtr7, gsWavPtr7X;
float gsWavPtr8, gsWavPtr8X, gsWavPtr9, gsWavPtr9X;
float gsStep0, gsStep0X, gsStep1, gsStep1X;
float gsStep2, gsStep2X, gsStep3, gsStep3X;
float gsStep4, gsStep4X, gsStep5, gsStep5X;
float gsStep6, gsStep6X, gsStep7, gsStep7X;
float gsStep8, gsStep8X, gsStep9, gsStep9X;
float gsRunStep0, gsRunStep0X, gsRunStep1, gsRunStep1X; //saves runtime
float gsRunStep2, gsRunStep2X, gsRunStep3, gsRunStep3X; //during DAC interrupt
float gsRunStep4, gsRunStep4X, gsRunStep5, gsRunStep5X;
float gsRunStep6, gsRunStep6X, gsRunStep7, gsRunStep7X;
float gsRunStep8, gsRunStep8X, gsRunStep9, gsRunStep9X;
float gsStart, gsStartX, gsFinal, gsFinalX, gsDeltaStep, gsDeltaStepX;
uint32_t gsStepSign, gsGlideWait, gsGlideBusy;
uint32_t gsStepSignX, gsGlideWaitX, gsGlideBusyX, gsGlideHold;

uint32_t gsPolarityFMO12, gsPolarityFMO2, gsPolarityFLfo, gsPolarityVib;
uint32_t gsPolarityBass, gsPolarityTreble, gsPolarityF0Pk, gsPolarityF0QF;
uint32_t gsPolarityFree;
uint32_t gsPolarityF0M, gsPolarityC1, gsPolarityC2, gsPolarityC3, gsPolarityC4;
uint32_t gsPolarityC5, gsPolarityC6, gsPolarityC7, gsPolarityC8, gsPolarityC9;

uint32_t gsPhaseNs, gsPhaseG, gsPhaseFMO12, gsPhaseFLfo, gsPhaseVib;
uint32_t gsPhaseBass, gsPhaseTreble, gsPhaseF0Pk, gsPhaseF0QF, gsPhaseFree;
uint32_t gsPhase0, gsPhase0X, gsPhase1, gsPhase1X, gsPhase2, gsPhase2X;
uint32_t gsPhase3, gsPhase3X, gsPhase4, gsPhase4X, gsPhase5, gsPhase5X;
uint32_t gsPhase6, gsPhase6X, gsPhase7, gsPhase7X, gsPhase8, gsPhase8X;
uint32_t gsPhase9, gsPhase9X;
uint32_t gsPhaseF0M, gsPhaseC1, gsPhaseC2, gsPhaseC3, gsPhaseC4, gsPhaseC5;
uint32_t gsPhaseC6, gsPhaseC7, gsPhaseC8, gsPhaseC9;

int32_t gsHiValNs, gsHiValG, gsHiValFMO12, gsHiValFLfo, gsHiValVib;
int32_t gsHiValBass, gsHiValTreble, gsHiValF0Pk, gsHiValF0QF, gsHiValFree;
int32_t gsHiVal0, gsHiVal0X, gsHiVal1, gsHiVal1X, gsHiVal2, gsHiVal2X;
int32_t gsHiVal3, gsHiVal3X, gsHiVal4, gsHiVal4X, gsHiVal5, gsHiVal5X;
int32_t gsHiVal6, gsHiVal6X, gsHiVal7, gsHiVal7X, gsHiVal8, gsHiVal8X;
int32_t gsHiVal9, gsHiVal9X;
int32_t gsHiValF0M, gsHiValC1, gsHiValC2, gsHiValC3, gsHiValC4, gsHiValC5;
int32_t gsHiValC6, gsHiValC7, gsHiValC8, gsHiValC9;

int32_t gsLimitNs, gsLimitG, gsLimitFMO12, gsLimitFLfo, gsLimitVib;
int32_t gsLimitBass, gsLimitTreble, gsLimitF0Pk, gsLimitF0QF, gsLimitFree;
int32_t gsLimit0, gsLimit0X, gsLimit1, gsLimit1X, gsLimit2, gsLimit2X;
int32_t gsLimit3, gsLimit3X, gsLimit4, gsLimit4X, gsLimit5, gsLimit5X;
int32_t gsLimit6, gsLimit6X, gsLimit7, gsLimit7X, gsLimit8, gsLimit8X;
int32_t gsLimit9, gsLimit9X;
int32_t gsLimitF0M, gsLimitC1, gsLimitC2, gsLimitC3, gsLimitC4, gsLimitC5;
int32_t gsLimitC6, gsLimitC7, gsLimitC8, gsLimitC9;

int32_t gsDeltaNs, gsDeltaG, gsDeltaFMO12, gsDeltaFLfo, gsDeltaVib;
int32_t gsDeltaBass, gsDeltaTreble, gsDeltaF0Pk, gsDeltaF0QF, gsDeltaFree;
int32_t gsDelta0, gsDelta0X, gsDelta1, gsDelta1X, gsDelta2, gsDelta2X;
int32_t gsDelta3, gsDelta3X, gsDelta4, gsDelta4X, gsDelta5, gsDelta5X;
int32_t gsDelta6, gsDelta6X, gsDelta7, gsDelta7X, gsDelta8, gsDelta8X;
int32_t gsDelta9, gsDelta9X;
int32_t gsDeltaF0M, gsDeltaC1, gsDeltaC2, gsDeltaC3, gsDeltaC4, gsDeltaC5;
int32_t gsDeltaC6, gsDeltaC7, gsDeltaC8, gsDeltaC9;

int32_t gsSignNs, gsSignG, gsSignFMO12, gsSignFLfo, gsSignVib;
int32_t gsSignBass, gsSignTreble, gsSignF0Pk, gsSignF0QF, gsSignFree;
int32_t gsSign0, gsSign0X, gsSign1, gsSign1X, gsSign2, gsSign2X;
int32_t gsSign3, gsSign3X, gsSign4, gsSign4X, gsSign5, gsSign5X;
int32_t gsSign6, gsSign6X, gsSign7, gsSign7X, gsSign8, gsSign8X;
int32_t gsSign9, gsSign9X;
int32_t gsSignF0M, gsSignC1, gsSignC2, gsSignC3, gsSignC4, gsSignC5;
int32_t gsSignC6, gsSignC7, gsSignC8, gsSignC9;

int32_t gsSustainHNs, gsSustainHG, gsSustainHFMO12, gsSustainHFLfo;
int32_t gsSustainHVib, gsSustainHBass, gsSustainHTreble, gsSustainHF0Pk;
int32_t gsSustainHF0QF, gsSustainHFree;
int32_t gsSustainH0, gsSustainH0X, gsSustainH1, gsSustainH1X, gsSustainH2;
int32_t gsSustainH2X, gsSustainH3, gsSustainH3X, gsSustainH4, gsSustainH4X;
int32_t gsSustainH5, gsSustainH5X, gsSustainH6, gsSustainH6X, gsSustainH7;
int32_t gsSustainH7X, gsSustainH8, gsSustainH8X, gsSustainH9, gsSustainH9X;
int32_t gsSustainHF0M, gsSustainHC1, gsSustainHC2, gsSustainHC3, gsSustainHC4;
int32_t gsSustainHC5, gsSustainHC6, gsSustainHC7, gsSustainHC8, gsSustainHC9;

int32_t gsValueNs, gsValueG, gsValueFMO12;
int32_t gsValueFLfo, gsValueVib, gsValueBass, gsValueTreble, gsValueFree;
int32_t gsValueFMO2, gsValueFLB, gsValueChor;
int32_t gsValue0, gsValue0X, gsValue1, gsValue1X, gsValue2, gsValue2X;
int32_t gsValue3, gsValue3X, gsValue4, gsValue4X, gsValue5, gsValue5X;
int32_t gsValue6, gsValue6X, gsValue7, gsValue7X, gsValue8, gsValue8X;
int32_t gsValue9, gsValue9X;
float gsValueF0M, gsValueF0Pk, gsValueF0QF, gsValueF2M;
float gsValueC1, gsValueC2, gsValueC3,gsValueC4, gsValueC5;
float gsValueC6, gsValueC7, gsValueC8,gsValueC9;

uint32_t gsOffNs;
uint32_t gsOff0, gsOff0X, gsOff1, gsOff1X, gsOff2, gsOff2X, gsOff3, gsOff3X;
uint32_t gsOff4, gsOff4X, gsOff5, gsOff5X, gsOff6, gsOff6X, gsOff7, gsOff7X;
uint32_t gsOff8, gsOff8X, gsOff9, gsOff9X;


uint32_t gsKeytrack0Preset[N_SOUNDS];
float gsKeytrack0;

float gsTrackShiftF0, gsTrackShiftC1, gsTrackShiftC2, gsTrackShiftC3;
float gsTrackShiftC4, gsTrackShiftC5, gsTrackShiftC6;
float gsTrackShiftC7, gsTrackShiftC8, gsTrackShiftC9;
uint32_t gsBaseFreqF0, gsBaseFreqC1, gsBaseFreqC2, gsBaseFreqC3, gsBaseFreqC4;
uint32_t gsBaseFreqC5, gsBaseFreqC6, gsBaseFreqC7, gsBaseFreqC8, gsBaseFreqC9;

uint32_t gsNoteVal0,gsVelocity0,gsNoteVal1,gsVelocity1,gsNoteVal2,gsVelocity2;
uint32_t gsNoteVal3,gsVelocity3,gsNoteVal4,gsVelocity4,gsNoteVal5,gsVelocity5;
uint32_t gsNoteVal6,gsVelocity6,gsNoteVal7,gsVelocity7,gsNoteVal8,gsVelocity8;
uint32_t gsNoteVal9, gsVelocity9, gsVelocityNs, gsVelocityG;

int16_t calcTrapez(uint32_t, uint32_t, uint32_t);
int16_t calcTri(uint32_t, uint32_t);
int16_t calcNoise(void);
int16_t strat(uint32_t);

void initWaveOsc1(uint32_t);
void initWaveOsc2(uint32_t);
void distributeNoteOn(uint32_t, uint32_t);
void handleNoteOff(uint32_t);
void panic_Off(uint32_t);
uint32_t scanNotes(void);
void triggerEtc(uint32_t,uint32_t);
void triggerNoise(uint32_t);
void triggerGain(uint32_t);
void shutdownNoise(uint32_t);
void releaseNoise(void);
void shutdownGain(uint32_t);
void releaseGain(void);
void releaseFMO12(void);
void releaseFLfo(void);
void releaseVib(void);
void releaseBass(void);
void releaseTreble(void);
void releaseF0Pk(void);
void intoPhaseNs2(void);
void intoPhaseNs3(void);
void intoPhaseG2(void);
void intoPhaseG3(void);
void intoPhaseFMO122(void);
void intoPhaseFMO123(void);
void intoPhaseFLfo2(void);
void intoPhaseFLfo3(void);
void intoPhaseVib2(void);
void intoPhaseVib3(void);
void intoPhaseBass2(void);
void intoPhaseBass3(void);
void intoPhaseTreble2(void);
void intoPhaseTreble3(void);
void intoPhaseF0Pk2(void);
void intoPhaseF0Pk3(void);
void tuneVoice0(uint32_t, uint32_t);
void fastRelease(void);
void shutdownVoice0(uint32_t, uint32_t);
void releaseVoice0(void);
void intoPhase02(void);
void intoPhase03(void);
void tuneVoice0X(uint32_t, uint32_t);
void shutdownVoice0X(uint32_t, uint32_t);
void releaseVoice0X(void);
void intoPhase0X2(void);
void intoPhase0X3(void);
void tuneVoice1(uint32_t, uint32_t);
void shutdownVoice1(uint32_t, uint32_t);
void releaseVoice1(void);
void intoPhase12(void);
void intoPhase13(void);
void tuneVoice1X(uint32_t, uint32_t);
void shutdownVoice1X(uint32_t, uint32_t);
void releaseVoice1X(void);
void intoPhase1X2(void);
void intoPhase1X3(void);
void tuneVoice2(uint32_t, uint32_t);
void shutdownVoice2(uint32_t, uint32_t);
void releaseVoice2(void);
void intoPhase22(void);
void intoPhase23(void);
void tuneVoice2X(uint32_t, uint32_t);
void shutdownVoice2X(uint32_t, uint32_t);
void releaseVoice2X(void);
void intoPhase2X2(void);
void intoPhase2X3(void);
void tuneVoice3(uint32_t, uint32_t);
void shutdownVoice3(uint32_t, uint32_t);
void releaseVoice3(void);
void intoPhase32(void);
void intoPhase33(void);
void tuneVoice3X(uint32_t, uint32_t);
void shutdownVoice3X(uint32_t, uint32_t);
void releaseVoice3X(void);
void intoPhase3X2(void);
void intoPhase3X3(void);
void tuneVoice4(uint32_t, uint32_t);
void shutdownVoice4(uint32_t, uint32_t);
void releaseVoice4(void);
void intoPhase42(void);
void intoPhase43(void);
void tuneVoice4X(uint32_t, uint32_t);
void shutdownVoice4X(uint32_t, uint32_t);
void releaseVoice4X(void);
void intoPhase4X2(void);
void intoPhase4X3(void);
void tuneVoice5(uint32_t, uint32_t);
void shutdownVoice5(uint32_t, uint32_t);
void releaseVoice5(void);
void intoPhase52(void);
void intoPhase53(void);
void tuneVoice5X(uint32_t, uint32_t);
void shutdownVoice5X(uint32_t, uint32_t);
void releaseVoice5X(void);
void intoPhase5X2(void);
void intoPhase5X3(void);
void tuneVoice6(uint32_t, uint32_t);
void shutdownVoice6(uint32_t, uint32_t);
void releaseVoice6(void);
void intoPhase62(void);
void intoPhase63(void);
void tuneVoice6X(uint32_t, uint32_t);
void shutdownVoice6X(uint32_t, uint32_t);
void releaseVoice6X(void);
void intoPhase6X2(void);
void intoPhase6X3(void);
void tuneVoice7(uint32_t, uint32_t);
void shutdownVoice7(uint32_t, uint32_t);
void releaseVoice7(void);
void intoPhase72(void);
void intoPhase73(void);
void tuneVoice7X(uint32_t, uint32_t);
void shutdownVoice7X(uint32_t, uint32_t);
void releaseVoice7X(void);
void intoPhase7X2(void);
void intoPhase7X3(void);
void tuneVoice8(uint32_t, uint32_t);
void shutdownVoice8(uint32_t, uint32_t);
void releaseVoice8(void);
void intoPhase82(void);
void intoPhase83(void);
void tuneVoice8X(uint32_t, uint32_t);
void shutdownVoice8X(uint32_t, uint32_t);
void releaseVoice8X(void);
void intoPhase8X2(void);
void intoPhase8X3(void);
void tuneVoice9(uint32_t, uint32_t);
void shutdownVoice9(uint32_t, uint32_t);
void releaseVoice9(void);
void intoPhase92(void);
void intoPhase93(void);
void tuneVoice9X(uint32_t, uint32_t);
void shutdownVoice9X(uint32_t, uint32_t);
void releaseVoice9X(void);
void intoPhase9X2(void);
void intoPhase9X3(void);

void triggerFilter0(uint32_t, uint32_t, uint32_t);
void triggerFree(uint32_t);
void triggerFilterC1(uint32_t, uint32_t, uint32_t);
void triggerFilterC2(uint32_t, uint32_t, uint32_t);
void triggerFilterC3(uint32_t, uint32_t, uint32_t);
void triggerFilterC4(uint32_t, uint32_t, uint32_t);
void triggerFilterC5(uint32_t, uint32_t, uint32_t);
void triggerFilterC6(uint32_t, uint32_t, uint32_t);
void triggerFilterC7(uint32_t, uint32_t, uint32_t);
void triggerFilterC8(uint32_t, uint32_t, uint32_t);
void triggerFilterC9(uint32_t, uint32_t, uint32_t);
void releaseFilter0(void);
void releaseFree(void);
void releaseFilterC1(void);
void releaseFilterC2(void);
void releaseFilterC3(void);
void releaseFilterC4(void);
void releaseFilterC5(void);
void releaseFilterC6(void);
void releaseFilterC7(void);
void releaseFilterC8(void);
void releaseFilterC9(void);
void intoPhaseF0M_2(void);
void intoPhaseF0M_3(void);
void intoPhaseFree_2(void);
void intoPhaseFree_3(void);
void intoPhaseC1_2(void);
void intoPhaseC1_3(void);
void intoPhaseC2_2(void);
void intoPhaseC2_3(void);
void intoPhaseC3_2(void);
void intoPhaseC3_3(void);
void intoPhaseC4_2(void);
void intoPhaseC4_3(void);
void intoPhaseC5_2(void);
void intoPhaseC5_3(void);
void intoPhaseC6_2(void);
void intoPhaseC6_3(void);
void intoPhaseC7_2(void);
void intoPhaseC7_3(void);
void intoPhaseC8_2(void);
void intoPhaseC8_3(void);
void intoPhaseC9_2(void);
void intoPhaseC9_3(void);
void ADSR_Process(void);


int16_t calcTrapez(uint32_t k, uint32_t hiRange, uint32_t slope)  // ********
// level 14bit (-8192 ... +8192)
// k= index of 1024 step waveform
//standard Trapez: hiRange 512, slope 36	//MAX255 = triangle !!
//hiRange acceptable ca.100 up to max 512
// **************************************************************************
{
int32_t xWave, hiLevel, loLevel, gradient;

	if (k >= 1024)  return 0;

	if (hiRange > 512) hiRange = 512;
	if (hiRange < 143) hiRange = 143;
	if (slope > (hiRange/2)) slope = hiRange/2;
	hiLevel = 16 * (1024-hiRange); // 16 = 8192/512;
	loLevel = 16384 - hiLevel;
	gradient = hiLevel/slope;

	if (k < slope)  {							//from 0 up tp 100% level
		xWave = k * gradient;
	}
	else if (k < (hiRange - slope))  {			//stay at 100%
		xWave = hiLevel;
	}
	else if (k < (hiRange + slope))  {
												//from 100% down to -100%
		xWave = hiLevel - gradient * (k - hiRange + slope);
	}
	else if (k < (1023 - slope))  {				//stay at -100%
		xWave = -loLevel;
	}
	else  xWave = (k-1024) * gradient;			//from -100% up to 0

//emergency filter
	if  (xWave < -loLevel) xWave = -loLevel;
	if (xWave > hiLevel) xWave = hiLevel;
	return ((int16_t) xWave);
}


int16_t calcTri(uint32_t k, uint32_t slope)  // ****************************
// level 14bit (-8192 ... +8191)
// k= index of 1024 step waveform
// slope < 256 : fast rise to 8192, slow fall to -8192, fast rise to 0
// slope == 256 : symmetric triangle
// slope > 256 : slow rise to 8192, fast fall to -8192, slow rise to 0
// slope=2 or slope=511: sawtooth
// **************************************************************************
{
int16_t xWave, gradient_p, gradient_m;

	if (k >= 1024)  return 0;

	gradient_p = 8192/slope;
	gradient_m = 16384/(1024 - 2*slope);

	if (k <= slope)  xWave = k * gradient_p;	// from 0 up tp 100% leve
	else if (k <= (1024 - slope)) xWave = 8192 - gradient_m * (k - slope);
												//from 100% down to -100%
	else  xWave = (k-1024) * gradient_p;		//up to 0%

//emergency filter
	if  (xWave < -8192) xWave = -8192;
	if (xWave > 8191) xWave = 8191;
	return ( xWave);
}


int16_t calcNoise(void)  // *************************************************
// level 14bit (-8192 ... +8191)
// **************************************************************************
{
int32_t k,noise;

	for (k=0;k<3;k++)  {
		gaNoise ^= gaNoise << 13;		//xorshift32 algorithm/white noise
		gaNoise ^= gaNoise >> 17;
		gaNoise ^= gaNoise << 5;
	}
	noise = ((int16_t)gaNoise) / 16;
	return noise;					//optimized empirically
}


int16_t strat(uint32_t k)   // **********************************************
//2 decaying waves od 3rd harmonic, last third=zero
// **************************************************************************
{
int32_t xWave;

	if (k < 171)  xWave = 10 * gsSinus[3 * k]/ 8;
	else if (k < 341)  xWave = 12 * gsSinus[3 * k] / 8;
	else if (k < 512)  xWave = 4 * gsSinus[3 * (k - 341)] /8;
	else if (k < 683) xWave= 2* gsSinus[3 * (k - 341)] / 8;
	else xWave = 0;
	return ((int16_t) xWave);
}


void initWaveOsc1(uint32_t waveform)   // ***********************************
// **************************************************************************
{
uint32_t k;

	gfRuntimeMask &= 0xFFFFFDFF;			//bit 9: marks Osc1 square wave
    if (waveform == 6)  {
        gfRuntimeMask |= 0x00000200;
        gsPWM1Hi = 8192;
        gsPWM1Lo = -8192;
        gsPWM1Edge0 = 512;
    }
    else if (waveform == 7)  {
        gfRuntimeMask |= 0x00000200;
        gsPWM1Hi = 12288;
        gsPWM1Lo = -4096;
        gsPWM1Edge0 = 256;
    }
    else if (waveform == 8)  {
        gfRuntimeMask |= 0x00000200;
        gsPWM1Hi = 13107;
        gsPWM1Lo = -3277;
        gsPWM1Edge0 = 205;
    }
    else if (waveform == 9)  {
        gfRuntimeMask |= 0x00000200;
        gsPWM1Hi = 16 * (1024-gsRangeFS1); 		// 16 = 8192/512;
        gsPWM1Lo = gsPWM1Hi - 16384;
        gsPWM1Edge0 = gsRangeFS1;
    }
    else if (waveform == 10)  {
        gfRuntimeMask |= 0x00000200;
        gsPWM1Hi = 16 * (1024-gsRangeFS2); 		// 16 = 8192/512;
        gsPWM1Lo = gsPWM1Hi - 16384;
        gsPWM1Edge0 = gsRangeFS2;
    }
    else  {
        if (gsOsc1PWM)  {
            gsOsc1PWM = 0;
            gsMOD_1 = 0;
            gsMOD_1Preset[gSoundSet] = 0;		//reset previous PWM to default
        }
    }
	for (k = 0; k < 1024; k++)  {
		switch (waveform)   {
			//case 0 is sinus, see below
			case 1:
				gsWave[k] = (4*calcTri(k,288))/3;	//differs better from SIN
				break;
			case 2:
				gsWave[k] = (5*calcTri(k,432))/4;	// soft asym.Triangle
				break;
			case 3:
				gsWave[k] = calcTri(k,472);			// hard asym.Triangle
				break;
			case 4:
				gsWave[k] = calcTri(k,508);			// softened sawtooth
				break;
			case 5:
			case 42:
				gsWave[k] = calcTrapez(k,512,36);
				break;
//case 6,7,8,9,10 is square wave.Use array if not pulse width mod.Slope smoothed
			case 6:
				gsWave[k] = calcTrapez(k,512,16);
				break;
			case 7:
				gsWave[k] = calcTrapez(k,256,16);
				break;
			case 8:
				gsWave[k] = calcTrapez(k,205,16);
				break;
			case 9:
				gsWave[k] = calcTrapez(k,gsRangeFS1,gsSlopeFS1);
				break;
			case 10:
				gsWave[k] = calcTrapez(k,gsRangeFS2,gsSlopeFS2);
				break;
			case 11:								//Spike
				if ((k > 170)&&(k < 512)) gsWave[k] =calcTri(3*(k-170),508);
												//sawtooth, fits into 0..1023
				else gsWave[k] = 0;
				break;
			case 12:
				if (k  < 512) gsWave[k] = 4 * gsSinus[2 * k] / 3;
									//gsSinus() double freq. fits into 0--1024
				else gsWave[k] = 0;
				break;
			case 13:
				if (k < 256) gsWave[k] = 4*gsSinus[2 * k]/3;
				else if ((k>512)&&(k<768)) gsWave[k]=4*gsSinus[2*(k-256)]/3;
									//gsSinus() double freq. fits into 0--1024
				else gsWave[k] = 0;
				break;
			case 14:
				if (k< 256)  gsWave[k] = gsSinus[2*k];
				else if (k < 512)  gsWave[k] = (5*gsSinus[2*k])/4;
				else if (k < 768)  gsWave[k] = gsSinus[2*(k-512)] / 2;
				else  gsWave[k] = gsSinus[2*(k-512)] / 4;
				break;
			case 15:
				gsWave[k] = 2*gsSinus[k]/3;
				if (k < 341) gsWave[k] -=  2*gsSinus[3*k]/3;
				else if (k < 682) gsWave[k] -= gsSinus[3*(k-341)]/3;
				break;
			case 16:
				gsWave[k] =  gsSinus[k];
				if ((k >256)&&(k <512)) gsWave[k] -= gsSinus[4*(k-256)]*2/3;
				else if ((k>512) && (k<768)) gsWave[k]-= gsSinus[4*(k-512)]*2/3;
				break;
			case 17:
				if (k < 512) gsWave[k] = gsSinus[k];
				else if (k < 768)  gsWave[k] = calcTrapez(k,512,36);
				else  gsWave[k] =  calcTrapez(k,512,128);
				break;
			case 18:
				gsWave[k] = strat(k);
				break;
			case 19:
				gsWave[k] = gsExpSin[(3*k)/5];			// fits into 0--1024
				break;
			case 20:
				gsWave[k] = gsExpSin[k];
				break;
			case 21:
			case 30:
			case 31:
			case 32:
			case 39:
				gsWave[k] = gsUserWave0[k];
				break;
			case 22:
			case 33:
			case 34:
				gsWave[k] = gsUserWave1[k];
				break;
			case 23:
			case 35:
				gsWave[k] = gsUserWave2[k];
				break;
			case 24:
				gsWave[k] = gsUserWave3[k];
				break;
			case 25:
				if (k < 682) gsWave[k] = gsSinus[k*3/2]/ 2;
				else gsWave[k] = gsSinus[(k - 682)*3/2]/ 2;
				gsWave[k] +=  gsSinus[k] / 2;
				break;
			case 36:
				gsWave[k] = calcNoise();
				break;
			case 37:
				if ( k < 768) gsWave[k] = gsSinus[k*4/3] / 3;
				else  gsWave[k] = gsSinus[(k-768)*4/3] / 3;
				gsWave[k] += gsSinus[k] * 2 / 3;
				break;
			case 38:
				if ( k < 819) gsWave[k] = gsSinus[k*5/4] / 4;
				else  gsWave[k] = gsSinus[(k-819)*5/4] / 4;
				gsWave[k] += gsSinus[k] * 3 / 4;
				break;
			case 41:
				gsWave[k] =  3*gsSinus[k] / 2;	//shrinking sinus
				break;
			case 0:
			case 26:
			case 27:
			case 28:
			case 29:
                default:
				gsWave[k] =  gsSinus[k];    //is default
				break;
		}
	}
	for (k = 1024; k < 2048; k++)  {
		switch (waveform)   {
			case 25:
				if (k < 1366) gsWave[k] = gsSinus[(k - 682)*3/2] / 2;
				else gsWave[k] = gsSinus[(k - 1366)*3/2] / 2;
				gsWave[k] +=  gsSinus[k-1024] / 2 ;
				break;
			case 26:
				if (k < 1536) gsWave[k] =  gsSinus[2 * (k-1024)];
				else gsWave[k] = gsSinus[2 * (k-1536)];
				break;
			case 27:
				gsWave[k] = calcTrapez(k-1024,512,36);
				break;
			case 28:
				gsWave[k] = strat(k-1024);
				break;
			case 29:
				gsWave[k] = gsExpSin[k-1024];
				break;
			case 30:
			case 39:
				gsWave[k] = gsUserWave1[k-1024];
				break;
			case 37:
				if ( k < 1536) gsWave[k] = gsSinus[(k-768)*4/3] / 3;
				else  gsWave[k] = gsSinus[(k-1536)*4/3] / 3;
				gsWave[k] += gsSinus[k-1024] * 2 / 3;
				break;
			case 31:
			case 33:
				gsWave[k] = gsUserWave2[k-1024];
				break;
			case 32:
			case 34:
			case 35:
				gsWave[k] = gsUserWave3[k-1024];
				break;
                case 36:
				gsWave[k] = calcNoise();
				break;
			case 38:
				if ( k < 1638) gsWave[k] = gsSinus[(k-819)*5/4] / 4;
				else gsWave[k] = gsSinus[(k-1638)*5/4] / 4;
				gsWave[k] += gsSinus[k-1024] * 3 / 4;
				break;
			case 41:
				gsWave[k] = gsSinus[k-1024];	//shrinking sinus
				break;
			case 40:
			case 42:
			default:
				gsWave[k] = gsSinus[k-1024];    //is default
				break;
		}
	}
	for (k = 2048; k < 3072; k++)  {
		switch (waveform)   {
			case 37:
				if ( k < 2304) gsWave[k] = gsSinus[(k-1536)*4/3] / 3;
				else  gsWave[k] = gsSinus[(k-2304)*4/3] / 3;
				gsWave[k] += gsSinus[k-2048] * 2 / 3;
				break;
               case 38:
				if (k < 2457) gsWave[k] = gsSinus[(k-1638)*5/4] / 4;
				else gsWave[k] = gsSinus[(k-2457)*5/4] / 4;
				gsWave[k] += gsSinus[k-2048] * 3 / 4;
				break;
			case 39:
				gsWave[k] = gsUserWave2[k-2048];
				break;
			case 40:
				if (k < 2560) gsWave[k] =  gsSinus[2 * (k - 2048)];
				else gsWave[k] = 0;
				break;
			case 41:
				gsWave[k] = 2*gsSinus[k-2048] / 3;	//shrinking sinus
				break;
			case 42:
			default:
				gsWave[k] = gsSinus[k-2048];    //is default
				break;
		}
	}
	for (k = 3072; k < 4096; k++)  {
		switch (waveform)   {
			case 38:
				if ( k < 3276) gsWave[k] = gsSinus[(5 * (k-2457))/4] / 4;
				else  gsWave[k] = gsSinus[(k-3276)*5/4] / 4;
				gsWave[k] += gsSinus[k-3072] * 3 / 4;
				break;
			case 39:
				gsWave[k] = gsUserWave3[k-3072];
				break;
			case 40:
				if (k < 3584) gsWave[k] =  gsSinus[2 * (k-3072)];
				else gsWave[k] = gsSinus[2 * (k-3584)];
				break;
			case 41:
				gsWave[k] = gsSinus[k-3072] / 3;	//shrinking sinus
				break;
			case 42:
			default:
				gsWave[k] = gsSinus[k-3072];    //is default
				break;
		}
	}
    gfStageMask[gSoundSet] = gfRuntimeMask;
}


void initWaveOsc2(uint32_t waveform)   // ***************************************
// **************************************************************************
{
uint32_t k;

    gfRuntimeMask &= 0xFFFFFBFF;			//bit 10: marks Osc2 square wave

    if (waveform == 6)  {
        gfRuntimeMask |= 0x00000400;
        gsPWM2Hi = 8191;
        gsPWM2Lo = -8192;
        gsPWM2Edge0 = 512;
    }
    else if (waveform == 7)  {
        gfRuntimeMask |= 0x00000400;
        gsPWM2Hi = 10923;
        gsPWM2Lo = -5461;
        gsPWM2Edge0 = 341;
    }
    else if (waveform == 8)  {
        gfRuntimeMask |= 0x00000400;
        gsPWM2Hi = 12288;
        gsPWM2Lo = -4096;
        gsPWM2Edge0 = 256;
    }
    else if (waveform == 9)  {
        gfRuntimeMask |= 0x00000400;
        gsPWM2Hi = 13107;
        gsPWM2Lo = -3277;
        gsPWM2Edge0 = 205;
    }
    else if (waveform == 10)  {
        gfRuntimeMask |= 0x00000400;
        gsPWM2Hi = 14043;
        gsPWM2Lo = -2341;
        gsPWM2Edge0 = 146;
    }
    else  {
        if (gsOsc2PWM)  {
            gsOsc2PWM = 0;
            gsMOD_2 = 0;
            gsMOD_2Preset[gSoundSet] = 0;		//reset previous PWM to default
        }
    }
	for (k = 0; k < 1024; k++)  {
		switch (waveform)   {
			//case 0 is sinus, see below
			case 1:
				gsWaveX[k] = (4*calcTri(k,288))/3; //differs better from SIN
				break;
			case 2:
				gsWaveX[k] = (5*calcTri(k,432))/4;	// soft asym.Triangle
				break;
			case 3:
				gsWaveX[k] = calcTri(k,472);		// hard asym.Triangle
				break;
			case 4:
				gsWaveX[k] = calcTri(k,508); 		//softened Sawtooth
				break;
			case 5:
				gsWaveX[k] = calcTrapez(k,512,36);
				break;
//case 6,7,8,9,10 is square wave.Use array if not pulse width mod.Slope smoothed
			case 6:
				gsWaveX[k] = calcTrapez(k,512,16);
				break;
			case 7:
				gsWaveX[k] = calcTrapez(k,341,16);
				break;
			case 8:
				gsWaveX[k] = calcTrapez(k,256,16);
				break;
			case 9:
				gsWaveX[k] = calcTrapez(k,205,16);
				break;
			case 10:
				gsWaveX[k] = calcTrapez(k,143,16);
				break;
				case 11:								//Spike
				if ((k> 170)&&(k< 512)) gsWaveX[k] = calcTri(3*(k-170),508);
												//sawtooth, fits into 0..1023
				else gsWaveX[k] = 0;
				break;
			case 12:
				if (k  < 512) gsWaveX[k] =  4*gsSinus[2 * k]/3;
				else gsWaveX[k] = 0;
				break;
			case 13:
				if (k < 256) gsWaveX[k] = 4*gsSinus[2 * k]/3;
				else if ((k>512)&&(k<768))gsWaveX[k]=4*gsSinus[2*(k-256)]/3;
				else gsWaveX[k] = 0;
				break;
			case 14:
				if (k< 256)  gsWaveX[k] = gsSinus[2*k];
				else if (k < 512)  gsWaveX[k] = (5*gsSinus[2*k])/4;
				else if (k < 768)  gsWaveX[k] = gsSinus[2*(k-512)] / 2;
				else  gsWaveX[k] = gsSinus[2*(k-512)] / 4;
				break;
			case 15:
				gsWaveX[k] = 2*gsSinus[k]/3;
				if (k < 341) gsWaveX[k] -=  2*gsSinus[3*k]/3;
				else if (k < 682) gsWaveX[k] -= gsSinus[3*(k-341)]/3;
				break;
			case 16:
				gsWaveX[k] =  gsSinus[k];
				if ((k >256)&&(k <512)) gsWaveX[k] -= gsSinus[4*(k-256)]*2/3;
				else if ((k>512) && (k<768)) gsWaveX[k]-=gsSinus[4*(k-512)]*2/3;
				break;
			case 17:
				if (k < 512) gsWaveX[k] = gsSinus[k];
				else if (k < 768)  gsWaveX[k] = calcTrapez(k,512,36);
				else  gsWaveX[k] =  calcTrapez(k,512,128);
				break;
			case 18:
				gsWaveX[k] = strat(k);
				break;
			case 19:
				gsWaveX[k] = gsExpSin[(3 * k)/5];
				break;
			case 20:
				gsWaveX[k] = gsExpSin[k];
				break;
			case 21:
			case 30:
			case 31:
			case 32:
				gsWaveX[k] = gsUserWave0[k];
				break;
			case 22:
			case 33:
			case 34:
				gsWaveX[k] = gsUserWave1[k];
				break;
			case 23:
			case 35:
				gsWaveX[k] = gsUserWave2[k];
				break;
			case 24:
				gsWaveX[k] = gsUserWave3[k];
				break;
			case 25:
				if (k < 682) gsWaveX[k] = gsSinus[k*3/2]/ 2;
				else gsWaveX[k] = gsSinus[(k - 682)*3/2]/ 2;
				gsWaveX[k] +=  gsSinus[k] / 2;
				break;
			case 36:
				gsWaveX[k] = 0; //calcNoise() proforma, not used
				break;
			case 0:
			case 26:
			case 27:
			case 28:
			case 29:
			default:
				gsWaveX[k] =  gsSinus[k];    //is default
				break;
		}
	}
	for (k = 1024; k < 2048; k++)  {
		switch (waveform)   {
			case 25:
				if (k < 1366) gsWaveX[k] = gsSinus[(k - 682)*3/2] / 2;
				else gsWaveX[k] = gsSinus[(k - 1366)*3/2] / 2;
				gsWaveX[k] +=  gsSinus[k-1024] / 2 ;
				break;
			case 26:
				if (k < 1536) gsWaveX[k] =  gsSinus[2 * (k-1024)];
				else gsWaveX[k] = gsSinus[2 * (k-1536)];
				break;
			case 27:
				gsWaveX[k] = calcTrapez(k-1024,512,36);
				break;
			case 28:
				gsWaveX[k] = strat(k-1024);
				break;
			case 29:
				gsWaveX[k] = gsExpSin[k-1024];
				break;
			case 30:
				gsWaveX[k] = gsUserWave1[k-1024];
				break;
			case 31:
			case 33:
				gsWaveX[k] = gsUserWave2[k-1024];
				break;
			case 32:
			case 34:
			case 35:
				gsWaveX[k] = gsUserWave3[k-1024];
				break;
			case 36:
				gsWaveX[k] = 0; //calcNoise() proforma, not used
				break;
			default:
                    gsWaveX[k] = gsSinus[k-1024];    //is default
				break;
		}
	}
    gfStageMask[gSoundSet] = gfRuntimeMask;
}


void distributeNoteOn(uint32_t noteval, uint32_t velocity)  // **************
// **************************************************************************
{
uint32_t maxFTrigger, nTrigger;

    if ((noteval < 18) || (noteval > 108))  return; //ignore extreme notes

    if (gsNoteFlags == 0) {			//synchronize Lfo's with first key stroke
        if (gLfo1Slope) gLfo1Ptr = 0; 	//start Lfo with POS. slope
        else  gLfo1Ptr = 2000; 			//start Lfo with NEG. slope
        if (gLfo2Slope) gLfo2Ptr = 0;
        else  gLfo2Ptr = 2000;
        if (gLfo3Slope) gLfo3Ptr = 0;
        else  gLfo3Ptr = 2000;

		gfF0ModBase = 1.0;				//normalize Filter0 and clones mid Freq
    }
    gsAdsrOffset = gSoundSet * N_ASTRUCT;			//refresh as precaution

	if ((gsNoteBusy[0] == 0) && (gsGlideHold == 0)) {//sound already terminated
		if (gsPhase0 == 0) {
			gsGlideWait = 0;						//phase 4 of Glide process
			gsGlideBusy = 0;						//reset, start new glide
		}
		if (gsPhase0X == 0) {
			gsGlideWaitX = 0;						//phase 4 of Glide process
			gsGlideBusyX = 0;						//reset, start new glide
		}
	}

    if (gsMonoPoly)  {								//bit1 set: monophonic
        if (gsMonoStackPtr < 19) {
            gsMonoStack[gsMonoStackPtr] = (velocity << 8) | noteval;
            gsMonoStackPtr++;
        }
        if ((gsPhase0) || (gsPhase0X)) fastRelease();
        gsNoteBusy[0] = noteval;
        tuneVoice0(noteval,velocity);
        tuneVoice0X(noteval,velocity);
        triggerEtc(noteval,velocity);
        triggerFilter0(noteval,velocity,1);
        gsNoteFlags |= 0x00000001;
        return;
    }
    else  {                                         //Polyphonic mode

		nTrigger = gNTrigger[gSoundSet];
//        maxFTrigger = 10;
        if (gsADSR[gsAdsrOffset + 1].REFIXVEL & 0x0000FF00)  {
												//FixVol != 0->Osc2 is active
        }
        if (gsNoteBusy[0] == 0)  {
            gsNoteBusy[0] = noteval;
            if (gsPhase0) shutdownVoice0(noteval,velocity);
            else tuneVoice0(noteval,velocity);
            if (gsPhase0X) shutdownVoice0X(noteval,velocity);
            else tuneVoice0X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
			gsNoteCount++;
			triggerFilter0(noteval,velocity,1);
 //           if (gsNoteCount <= maxFTrigger) triggerFilter0(noteval,velocity,1);
//            else triggerFilter0(noteval,velocity,0);
            gsNoteFlags |= 0x00000001;
            return;
        }
        if (gsNoteBusy[1] == 0)  {
            gsNoteBusy[1] = noteval;
            if (gsPhase1) shutdownVoice1(noteval,velocity);
            else tuneVoice1(noteval,velocity);
            if (gsPhase1X) shutdownVoice1X(noteval,velocity);
            else tuneVoice1X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
			triggerFilterC1(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC1(noteval,velocity,1);
//            else triggerFilterC1(noteval,velocity,0);
            gsNoteFlags |= 0x00000002;
            return;
        }
        if (gsNoteBusy[2] == 0)  {
            gsNoteBusy[2] = noteval;
            if (gsPhase2) shutdownVoice2(noteval,velocity);
            else tuneVoice2(noteval,velocity);
            if (gsPhase2X) shutdownVoice2X(noteval,velocity);
            else tuneVoice2X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
            triggerFilterC2(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC2(noteval,velocity,1);
//            else triggerFilterC2(noteval,velocity,0);
            gsNoteFlags |= 0x00000004;
            return;
        }
        if (gsNoteBusy[3] == 0)  {
            gsNoteBusy[3] = noteval;
            if (gsPhase3) shutdownVoice3(noteval,velocity);
            else tuneVoice3(noteval,velocity);
            if (gsPhase3X) shutdownVoice3X(noteval,velocity);
            else tuneVoice3X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
            triggerFilterC3(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC3(noteval,velocity,1);
//            else triggerFilterC3(noteval,velocity,0);
            gsNoteFlags |= 0x00000008;
            return;
        }
        if (gsNoteBusy[4] == 0)  {
            gsNoteBusy[4] = noteval;
            if (gsPhase4) shutdownVoice4(noteval,velocity);
            else tuneVoice4(noteval,velocity);
            if (gsPhase4X) shutdownVoice4X(noteval,velocity);
            else tuneVoice4X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
            triggerFilterC4(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC4(noteval,velocity,1);
//            triggerFilterC4(noteval,velocity,0);
            gsNoteFlags |= 0x00000010;
            return;
        }
        if (gsNoteBusy[5] == 0)  {
            gsNoteBusy[5] = noteval;
            if (gsPhase5) shutdownVoice5(noteval,velocity);
            else tuneVoice5(noteval,velocity);
            if (gsPhase5X) shutdownVoice5X(noteval,velocity);
            else tuneVoice5X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
            triggerFilterC5(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC5(noteval,velocity,1);
//            else triggerFilterC5(noteval,velocity,0);
            gsNoteFlags |= 0x00000020;
            return;
        }
        if (gsNoteBusy[6] == 0)  {
            gsNoteBusy[6] = noteval;
            if (gsPhase6) shutdownVoice6(noteval,velocity);
            else tuneVoice6(noteval,velocity);
            if (gsPhase6X) shutdownVoice6X(noteval,velocity);
            else tuneVoice6X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
            triggerFilterC6(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC6(noteval,velocity,1);
//            else triggerFilterC6(noteval,velocity,0);
            gsNoteFlags |= 0x00000040;
            return;
        }
        if (gsNoteBusy[7] == 0)  {
            gsNoteBusy[7] = noteval;
            if (gsPhase7) shutdownVoice7(noteval,velocity);
            else tuneVoice7(noteval,velocity);
            if (gsPhase7X) shutdownVoice7X(noteval,velocity);
            else tuneVoice7X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
            triggerFilterC7(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC7(noteval,velocity,1);
//            else triggerFilterC7(noteval,velocity,0);
            gsNoteFlags |= 0x00000080;
            return;
        }
        if (gsNoteBusy[8] == 0)  {
            gsNoteBusy[8] = noteval;
            if (gsPhase8) shutdownVoice8(noteval,velocity);
            else tuneVoice8(noteval,velocity);
            if (gsPhase8X) shutdownVoice8X(noteval,velocity);
            else tuneVoice8X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
            triggerFilterC8(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC8(noteval,velocity,1);
//            else triggerFilterC8(noteval,velocity,0);
            gsNoteFlags |= 0x00000100;
            return;
        }
        if (gsNoteBusy[9] == 0)  {
            gsNoteBusy[9] = noteval;
            if (gsPhase9) shutdownVoice9(noteval,velocity);
            else tuneVoice9(noteval,velocity);
            if (gsPhase9X) shutdownVoice9X(noteval,velocity);
            else tuneVoice9X(noteval,velocity);
			if (gsNoteCount < nTrigger)  triggerEtc(noteval,velocity);
            gsNoteCount++;
            triggerFilterC9(noteval,velocity,1);
//            if (gsNoteCount <= maxFTrigger) triggerFilterC9(noteval,velocity,1);
//            else triggerFilterC9(noteval,velocity,0);
            gsNoteFlags |= 0x00000200;
        }
    }
}


void handleNoteOff(uint32_t noteval)  // ************************************
// **************************************************************************
{
uint32_t n, v;

    if (gsMonoPoly)  {						// bit1 set: monophonic mode
        if (gsNoteBusy[0] == noteval)  {    //note of released key is playing
            releaseVoice0();
            releaseVoice0X();
            releaseNoise();
            releaseGain();
            releaseFMO12();
            releaseFLfo();
            releaseVib();
            releaseBass();
            releaseTreble();
			releaseF0Pk();
            releaseFilter0();
            releaseFree();
            gsNoteBusy[0] = 0;
            gsNoteFlags &= 0xFFFFFFFE;
            if (gsMonoStackPtr == 1) gsMonoStackPtr = 0;    //terminate
            else  {             //take next entry from the stack and play it
                fastRelease();	//turn Osc1&Osc2 fast down, avoid click noise
                gsMonoStackPtr -= 2;
                n = gsMonoStack[gsMonoStackPtr];
                v = n >> 8;         //v is velocity
                n &= 0x0000000FF;   //n is note
                distributeNoteOn(n,v);
            }
        }
        else {                      //remove an entry from the middle of stack
                                    //but do nothing with audible sound
            if (gsMonoStackPtr == 1)  {
                gsMonoStackPtr = 0; //should be impossible, as precaution
                releaseVoice0();
                releaseVoice0X();
                releaseNoise();
                releaseGain();
                releaseFMO12();
                releaseFLfo();
                releaseVib();
                releaseBass();
                releaseTreble();
				releaseF0Pk();
                releaseFilter0();
                releaseFree();
                gsNoteBusy[0] = 0;
                gsNoteFlags &= 0xFFFFFFFE;
                return;
            }
            for (v = 0; v < gsMonoStackPtr; v++)  {
                if ((gsMonoStack[v] & 0x000000FF) == noteval)  {
                    for (n = v; n < gsMonoStackPtr; n++)  {
                       gsMonoStack[n] = gsMonoStack[n+1]; //remove it from stack
                    }
                    gsMonoStackPtr--;
                    return;
                }
            }
            //emergency action if the released note is not in the stack:
            gsMonoStackPtr = 0;		//should be impossible, as precaution
            releaseVoice0();
            releaseVoice0X();
            releaseNoise();
            releaseGain();
            releaseFMO12();
			releaseFLfo();
            releaseVib();
			releaseBass();
			releaseTreble();
            releaseF0Pk();
            releaseFilter0();
            gsNoteBusy[0] = 0;
            gsNoteFlags &= 0xFFFFFFFE;
        }
        return;
    }
    else  {							//bit7 clear: polyphonic
        if (gsNoteBusy[0] == noteval)  {
            releaseVoice0();
            releaseVoice0X();
            releaseFilter0();
            gsNoteBusy[0] = 0;
            gsNoteFlags &= 0xFFFFFFFE;
            if (gsNoteCount) gsNoteCount--;
        }
        if (gsNoteBusy[1] == noteval)  {
            releaseVoice1();
            releaseVoice1X();
            releaseFilterC1();
            gsNoteBusy[1] = 0;
            gsNoteFlags &= 0xFFFFFFFD;
            if (gsNoteCount) gsNoteCount--;
        }
        if (gsNoteBusy[2] == noteval)  {
            releaseVoice2();
            releaseVoice2X();
            releaseFilterC2();
            gsNoteBusy[2] = 0;
            gsNoteFlags &= 0xFFFFFFFB;
            if (gsNoteCount) gsNoteCount--;
        }
        if (gsNoteBusy[3] == noteval)  {
            releaseVoice3();
            releaseVoice3X();
            releaseFilterC3();
            gsNoteBusy[3] = 0;
            gsNoteFlags &= 0xFFFFFFF7;
            if (gsNoteCount) gsNoteCount--;
        }
        if (gsNoteBusy[4] == noteval)  {
            releaseVoice4();
            releaseVoice4X();
            releaseFilterC4();
            gsNoteBusy[4] = 0;
            gsNoteFlags &= 0xFFFFFFEF;
            if (gsNoteCount) gsNoteCount--;
        }
        if (gsNoteBusy[5] == noteval)  {
            releaseVoice5();
            releaseVoice5X();
            releaseFilterC5();
            gsNoteBusy[5] = 0;
            gsNoteFlags &= 0xFFFFFFDF;
            if (gsNoteCount) gsNoteCount--;
        }
        if (gsNoteBusy[6] == noteval)  {
            releaseVoice6();
            releaseVoice6X();
            releaseFilterC6();
            gsNoteBusy[6] = 0;
            gsNoteFlags &= 0xFFFFFFBF;
            if (gsNoteCount) gsNoteCount--;
        }
       if (gsNoteBusy[7] == noteval)  {
            releaseVoice7();
            releaseVoice7X();
            releaseFilterC7();
            gsNoteBusy[7] = 0;
            gsNoteFlags &= 0xFFFFFF7F;
            if (gsNoteCount) gsNoteCount--;
        }
        if (gsNoteBusy[8] == noteval)  {
            releaseVoice8();
            releaseVoice8X();
            releaseFilterC8();
            gsNoteBusy[8] = 0;
            gsNoteFlags &= 0xFFFFFEFF;
            if (gsNoteCount) gsNoteCount--;
        }
       if (gsNoteBusy[9] == noteval)  {
            releaseVoice9();
            releaseVoice9X();
            releaseFilterC9();
            gsNoteBusy[9] = 0;
            gsNoteFlags &= 0xFFFFFDFF;
            if (gsNoteCount) gsNoteCount--;
        }
    }

    if (gsNoteFlags == 0) {
        gsNoteCount = 0;
        releaseNoise();
        releaseGain();
        releaseFMO12();
		releaseFLfo();
		releaseVib();
		releaseBass();
		releaseTreble();
		releaseF0Pk();
        if (gsPhaseFMO12) releaseFilter0();
        if (gsPhaseFree) releaseFree();
        if (gsPhaseC1) releaseFilterC1();
        if (gsPhaseC2) releaseFilterC2();
        if (gsPhaseC3) releaseFilterC3();
        if (gsPhaseC4) releaseFilterC4();
        if (gsPhaseC5) releaseFilterC5();
        if (gsPhaseC6) releaseFilterC6();
        if (gsPhaseC7) releaseFilterC7();
        if (gsPhaseC8) releaseFilterC8();
        if (gsPhaseC9) releaseFilterC9();
        releaseVoice0();                          //a measure of precaution
        releaseVoice0X();
        releaseVoice1();
        releaseVoice1X();
        releaseVoice2();
        releaseVoice2X();
        releaseVoice3();
        releaseVoice3X();
        releaseVoice4();
        releaseVoice4X();
        releaseVoice5();
        releaseVoice5X();
        releaseVoice6();
        releaseVoice6X();
        releaseVoice7();
        releaseVoice7X();
        releaseVoice8();
        releaseVoice8X();
        releaseVoice9();
        releaseVoice9X();
    }
}


void panic_Off(uint32_t mode)  // *******************************************
// **************************************************************************
{
uint32_t k;

    for (k = 0; k < 10; k++)  {
        gsNoteBusy[k] = 0;
    }
    gsNoteCount = 0;
    gsNoteFlags = 0;
    gsGlideWaitX = 0;
    gsGlideBusyX = 0;

	if (mode)  {
		if (gsPhase0)  {
			gsDelta0 = gsHiVal0 / SHORT_RELEASE;
			gsPhase0 = 4;
		}
		if (gsPhase0X)  {
			gsDelta0X = gsHiVal0X / SHORT_RELEASE;
			gsPhase0X = 4;
		}
		if (gsPhase1)  {
			gsDelta1 = gsHiVal1 / SHORT_RELEASE;
			gsPhase1 = 4;
		}
		if (gsPhase1X)  {
			gsDelta1X = gsHiVal1X / SHORT_RELEASE;
			gsPhase1X = 4;
		}
		if (gsPhase2)  {
			gsDelta2 = gsHiVal2 / SHORT_RELEASE;
			gsPhase2 = 4;
		}
		if (gsPhase2X)  {
			gsDelta2X = gsHiVal2X / SHORT_RELEASE;
			gsPhase2X = 4;
		}
		if (gsPhase3)  {
			gsDelta3 = gsHiVal3 / SHORT_RELEASE;
			gsPhase3 = 4;
		}
		if (gsPhase3X)  {
			gsDelta3X = gsHiVal3X / SHORT_RELEASE;
			gsPhase3X = 4;
		}
		if (gsPhase4)  {
			gsDelta4 = gsHiVal4 / SHORT_RELEASE;
			gsPhase4 = 4;
		}
		if (gsPhase4X)  {
			gsDelta4X = gsHiVal4X / SHORT_RELEASE;
			gsPhase4X = 4;
		 }
		if (gsPhase5)  {
			gsDelta5 = gsHiVal5 / SHORT_RELEASE;
			gsPhase5 = 4;
		}
		if (gsPhase5X)  {
			gsDelta5X = gsHiVal5X / SHORT_RELEASE;
			gsPhase5X = 4;
		}
		if (gsPhase6)  {
			gsDelta6 = gsHiVal6 / SHORT_RELEASE;
			gsPhase6 = 4;
		}
		if (gsPhase6X)  {
			gsDelta6X = gsHiVal6X / SHORT_RELEASE;
			gsPhase6X = 4;
		}
		if (gsPhase7)  {
			gsDelta7 = gsHiVal7 / SHORT_RELEASE;
			gsPhase7 = 4;
		}
		if (gsPhase7X)  {
			gsDelta7X = gsHiVal7X / SHORT_RELEASE;
		gsPhase7X = 4;
		}
		if (gsPhase8)  {
			gsDelta8 = gsHiVal8 / SHORT_RELEASE;
			gsPhase8 = 4;
		}
		if (gsPhase8X)  {
			gsDelta8X = gsHiVal8X / SHORT_RELEASE;
			gsPhase8X = 4;
		}
		if (gsPhase9)  {
			gsDelta9 = gsHiVal9 / SHORT_RELEASE;
			gsPhase9 = 4;
		}
		if (gsPhase9X)  {
			gsDelta9X = gsHiVal9X / SHORT_RELEASE;
			gsPhase9X = 4;
		}
	}
    gsMonoStackPtr = 0;
    gsPhaseNs = 0;
    gsHiValNs = 0;
    gsPhaseG = 0;
    gsHiValG = 0;
    gsPhaseFMO12 = 0;
    gsHiValFMO12 = 0;
    gsPhaseFLfo = 0;
    gsHiValFLfo = 0;
    gsPhaseVib = 0;
    gsHiValVib = 0;
    gsPhaseBass = 0;
    gsPhaseTreble= 0;
    gsHiValBass = 0;
    gsHiValTreble = 0;
    gsPhaseF0M = 0;
    gsHiValF0M = 0;
    gsPhaseF0Pk = 0;
    gsHiValF0Pk = 0;
    gsPhaseFree = 0;
    gsHiValFree = 0;
    gsPhaseC1 = 0;
    gsHiValC1 = 0;
    gsPhaseC2 = 0;
    gsHiValC2 = 0;
    gsPhaseC3 = 0;
    gsHiValC3 = 0;
    gsPhaseC4 = 0;
    gsHiValC4 = 0;
    gsPhaseC5 = 0;
    gsHiValC5 = 0;
    gsPhaseC6 = 0;
    gsHiValC6 = 0;
    gsPhaseC7 = 0;
    gsHiValC7 = 0;
    gsPhaseC8 = 0;
    gsHiValC8 = 0;
    gsPhaseC9 = 0;
    gsHiValC9 = 0;
    for (k = 0; k < MAXCHORUS; k++)  gaCHORUSBUF[k] = 0;    //clear delay chain
}


void triggerEtc(uint32_t noteval, uint32_t velocity) // *********************
//trigger ADSRs for Noise, Gain and frequency modulation
// **************************************************************************
{
//ADSR0 Controls OSC1/Voice0, see below
//ADSR1 Controls OSC2/Voice0, see below
//ADSR9 controls MidFreq/Cutoff of Filter#0  and its clones, see below

//ADSR2 controls envelope of Noise
    if (gsPhaseNs) shutdownNoise(velocity);
    else triggerNoise(velocity);

//ADSRADSR11[N_ABASIC = 11] contols free selectable features
    triggerFree(velocity);

// ADSR3 controls Frequency Modulation of OSC1 and OSC2 commonly
    gsPhaseFMO12 = 0; //these settings as precaution,if no useable parameters
    gsValueFMO12 = 0;

    gsLimitFMO12 = ((int32_t)(gsFixVol[3] - 64) * 2048) +		//2048=64*32
                                (32 * velocity * ((int32_t)(gsVelo[3] - 28)));
		//stretched for better resolution of high Attack,Decay,Release values
    gsPolarityFMO12 = 0;
    if (gsLimitFMO12 < 0)  {
        gsLimitFMO12 = -gsLimitFMO12;			//ADSR only handles pos levels
        gsPolarityFMO12 = 1;					//for neg.sign restauration
    }
    if (gsLimitFMO12 > 0x7F000) gsLimitFMO12=0x7F000;//theoretically max 0x71800
    if (gsLimitFMO12 < 0) gsLimitFMO12 = 0;

    if (gsLimitFMO12 > 0x100) {
        gsSustainHFMO12 = (gsSustain[3] * gsLimitFMO12) / 100;

		if ((gsAttack[3] == 0) && (gsDecay[3] == 0)) {
			gsHiValFMO12 = gsSustainHFMO12;
            gsValueFMO12 = gsHiValFMO12 >> 6;			//restore neg.sign
			intoPhaseFMO123();
		}
        else if (gsAttack[3])  {
            gsDeltaFMO12 = gsLimitFMO12 / gsAttack[3];
            if (gsDeltaFMO12 == 0) gsDeltaFMO12 = 1;
            gsHiValFMO12 = gsDeltaFMO12;
            if (gsPolarityFMO12 == 0) gsValueFMO12 = gsHiValFMO12 >> 6;
            else  gsValueFMO12 = -(gsHiValFMO12 >> 6);		//restore neg.sign
            gsPhaseFMO12 = 1;
        }
        else if (gsDecay[3])  {
            if (gsLimitFMO12 >= gsSustainHFMO12) {
                gsSignFMO12 = 0;
                gsDeltaFMO12 = (gsLimitFMO12 - gsSustainHFMO12) / gsDecay[3];
            }
            else {
                gsSignFMO12 = 1;
                gsDeltaFMO12 = (gsSustainHFMO12 - gsLimitFMO12) / gsDecay[3];
            }
            if (gsDeltaFMO12 == 0) gsDeltaFMO12 = 1 ;
            gsHiValFMO12 = gsLimitFMO12;
            if (gsPolarityFMO12 == 0) gsValueFMO12 = gsHiValFMO12 >> 6;
            else  gsValueFMO12 = -(gsHiValFMO12 >> 6);		//restore neg.sign
            gsPhaseFMO12 = 2;
        }
        else if (gsSustainHFMO12)  {
            gsHiValFMO12 = gsSustainHFMO12;
            if (gsPolarityFMO12 == 0) gsValueFMO12 = gsHiValFMO12 >> 6;
            else  gsValueFMO12 = -(gsHiValFMO12 >> 6);		//restore neg.sign
            gsDeltaFMO12 = 0;		//no Countdown, gets terminated by NOTE OFF
            gsPhaseFMO12 = 3;
            if (gsHoldSust[3] == 0)  releaseFMO12();
        }
    }

// -------------------------------------------

// ADSR4 controls frequency modulation of Lfo1
    gsPhaseFLfo = 0; //these settings as precaution,if no useable parameters
    gsValueFLfo = 0;

    gsLimitFLfo = ((int32_t)(gsFixVol[4] - 64) * 2048) +  \
                                (32 * velocity * ((int32_t)(gsVelo[4] - 28)));
		//stretched for better resolution of high Attack,Decay,Release values
    gsPolarityFLfo = 0;
    if (gsLimitFLfo < 0)  {
        gsLimitFLfo = -gsLimitFLfo;
        gsPolarityFLfo = 1;
    }
    if (gsLimitFLfo> 0x7F000) gsLimitFLfo = 0x7F000;//theoretically max 0x71800
    if (gsLimitFLfo < 0) gsLimitFLfo = 0;

    if (gsLimitFLfo > 0x100)  {
        gsSustainHFLfo = (gsSustain[4] * gsLimitFLfo) / 100;

		if ((gsAttack[4] == 0) && (gsDecay[4] == 0)) {
			gsHiValFLfo = gsSustainHFLfo;
            gsValueFLfo = gsHiValFLfo >> 6;
			intoPhaseFLfo3();
		}
        else if (gsAttack[4])  {
            gsDeltaFLfo = gsLimitFLfo / gsAttack[4];
            if (gsDeltaFLfo == 0) gsDeltaFLfo = 1;
            gsHiValFLfo = gsDeltaFLfo;
            if (gsPolarityFLfo == 0) gsValueFLfo = gsHiValFLfo >> 6;
            else  gsValueFLfo = -(gsHiValFLfo >> 6);		//restore neg.sign
            gsPhaseFLfo = 1;
        }
        else if (gsDecay[4])  {
            if (gsLimitFLfo >= gsSustainHFLfo) {
                gsSignFLfo = 0;
                gsDeltaFLfo = (gsLimitFLfo - gsSustainHFLfo) / gsDecay[4];
            }
            else {
                gsSignFLfo = 1;
                gsDeltaFLfo = (gsSustainHFLfo - gsLimitFLfo) / gsDecay[4];
            }
            if (gsDeltaFLfo == 0) gsDeltaFLfo = 1 ;
            gsHiValFLfo = gsLimitFLfo;
            if (gsPolarityFLfo == 0) gsValueFLfo = gsHiValFLfo >> 6;
            else  gsValueFLfo = -(gsHiValFLfo >> 6);		//restore neg.sign
            gsPhaseFLfo = 2;
        }
        else if (gsSustainHFLfo)  {
            gsHiValFLfo = gsSustainHFLfo;
            if (gsPolarityFLfo == 0) gsValueFLfo = gsHiValFLfo >> 6;
            else  gsValueFLfo = -(gsHiValFLfo >> 6);		//restore neg.sign
            gsDeltaFLfo = 0;		//no Countdown, gets terminated by NOTE OFF
            gsPhaseFLfo = 3;
            if (gsHoldSust[4] == 0)  releaseFLfo();
        }
    }

// ADSR5 controls frequency modulation of Vibrato
    gsPhaseVib = 0; //these settings as precaution,if no useable parameters
    gsValueVib = 0;

    gsLimitVib = ((int32_t)(gsFixVol[5] - 64) * 2048) +  \
                                (32 * velocity * ((int32_t)(gsVelo[5] - 28)));
    gsPolarityVib = 0;
    if (gsLimitVib < 0)  {
        gsLimitVib = -gsLimitVib;
        gsPolarityVib = 1;
    }
    if (gsLimitVib > 0x7F000) gsLimitVib = 0x7F000;//theoretically max 0x71800

    if (gsLimitVib > 0x100)  {
        gsSustainHVib = (gsSustain[5] * gsLimitVib) / 100;

		if ((gsAttack[5] == 0) && (gsDecay[5] == 0)) {
			gsHiValVib = gsSustainHVib;
            if (gsPolarityVib == 0) gsValueVib = gsHiValVib >> 6;
            else  gsValueVib = -(gsHiValVib >> 6); //max possible ca.+/-100
			intoPhaseVib3();
		}
        else if (gsAttack[5])  {
            gsDeltaVib = gsLimitVib / gsAttack[5];
            if (gsDeltaVib == 0) gsDeltaVib = 1;
            gsHiValVib = gsDeltaVib;
            if (gsPolarityVib == 0) gsValueVib = gsHiValVib >> 6;
            else  gsValueVib = -(gsHiValVib >> 6);//max possible ca.+/-100
            gsPhaseVib = 1;
        }
        else if (gsDecay[5])  {
            if (gsLimitVib >= gsSustainHVib) {
                gsSignVib = 0;
                gsDeltaVib = (gsLimitVib - gsSustainHVib) / gsDecay[5];
            }
            else {
                gsSignVib = 1;
                gsDeltaVib = (gsSustainHVib - gsLimitVib) / gsDecay[5];
            }
            if (gsDeltaVib == 0) gsDeltaVib = 1 ;
            gsHiValVib = gsLimitVib;
            if (gsPolarityVib==0) gsValueVib = gsHiValVib >>6;//restore neg.sign
            else  gsValueVib = -(gsHiValVib >> 6);		//max possible ca.+/-100
            gsPhaseVib = 2;
        }
        else if (gsSustainHVib)  {
            gsHiValVib = gsSustainHVib;
            if (gsPolarityVib == 0) gsValueVib = gsHiValVib >> 6;
            else  gsValueVib = -(gsHiValVib >> 6);		//restore neg.sign
            gsDeltaVib = 0;		//no Countdown, gets terminated by NOTE OFF
            gsPhaseVib = 3;
            if (gsHoldSust[5] == 0)  releaseVib();
        }
    }

// ADSR6 controls Bass
    gsPhaseBass = 0; //these settings as precaution,if no useable parameters
    gsValueBass = 0;

	gsLimitBass = ((int32_t)(gsFixVol[6] - 64) * 2048) +  \
									(32*velocity * ((int32_t)(gsVelo[6] - 28)));
		//stretched for better resolution of high Attack,Decay,Release values
    gsPolarityBass = 0;
    if (gsLimitBass < 0)  {
        gsLimitBass = -gsLimitBass;
        gsPolarityBass = 1;
    }
    if (gsLimitBass > 0x7F000) gsLimitBass = 0x7F000;//theoretically max 0x71800
    if (gsLimitBass < 0) gsLimitBass = 0;

    if (gsLimitBass > 0x100)  {
		unpackBasTreb();						//measure of precaution
        gsSustainHBass = (gsSustain[6] * gsLimitBass) / 100 ;

		if ((gsAttack[6] == 0) && (gsDecay[6] == 0)) {
			gsHiValBass = gsSustainHBass;
            if (gsPolarityBass == 0) gsValueBass = gsHiValBass >> 6;
            else  gsValueBass = -(gsHiValBass >> 6);		//restore neg.sign
			intoPhaseBass3();
		}
        else if (gsAttack[6])  {
            gsDeltaBass = gsLimitBass / gsAttack[6];
            if (gsDeltaBass == 0) gsDeltaBass = 1;
            gsHiValBass = gsDeltaBass;
            if (gsPolarityBass == 0) gsValueBass = gsHiValBass >> 6;
            else  gsValueBass = -(gsHiValBass >> 6);		//restore neg.sign
            gsPhaseBass = 1;
        }
        else if (gsDecay[6])  {
            if (gsLimitBass >= gsSustainHBass) {
                gsSignBass = 0;
                gsDeltaBass = (gsLimitBass - gsSustainHBass) / gsDecay[6];
            }
            else {
                gsSignBass = 1;
                gsDeltaBass = (gsSustainHBass - gsLimitBass) / gsDecay[6];
            }
            if (gsDeltaBass == 0) gsDeltaBass = 1 ;
            gsHiValBass = gsLimitBass;
            if (gsPolarityBass == 0) gsValueBass = gsHiValBass >> 6;
            else  gsValueBass = -(gsHiValBass >> 6);		//restore neg.sign
            gsPhaseBass = 2;
        }
        else if (gsSustainHBass)  {
            gsHiValBass = gsSustainHBass;
            if (gsPolarityBass == 0) gsValueBass = gsHiValBass >> 6;
            else  gsValueBass = -(gsHiValBass >> 6);		//restore neg.sign
            gsDeltaBass = 0;		//no Countdown, gets terminated by NOTE OFF
            gsPhaseBass = 3;
            if (gsHoldSust[6] == 0)  releaseBass();
        }
    }

// ADSR7 controls Treble
    gsPhaseTreble = 0; //these settings as precaution,if no useable parameters
    gsValueTreble = 0;

    gsLimitTreble= ((int32_t)(gsFixVol[7] - 64) * 2048) +  \
									(32*velocity * ((int32_t)(gsVelo[7] - 28)));
		//stretched for better resolution of high Attack,Decay,Release values
    gsPolarityTreble = 0;
    if (gsLimitTreble < 0)  {
        gsLimitTreble = -gsLimitTreble;
        gsPolarityTreble = 1;
    }
    if (gsLimitTreble>0x7F000) gsLimitTreble=0x7F000;//theoretically max 0x71800
    if (gsLimitTreble < 0) gsLimitTreble = 0;

    if (gsLimitTreble > 0x100)  {
		unpackBasTreb();						//measure of precaution
        gsSustainHTreble = (gsSustain[7] * gsLimitTreble) / 100;

		if ((gsAttack[7] == 0) && (gsDecay[7] == 0)) {
			gsHiValTreble = gsSustainHTreble;
            if (gsPolarityTreble == 0) gsValueTreble = gsHiValTreble >> 6;
            else  gsValueTreble = -(gsHiValTreble >> 6);		//restore neg.sign
			intoPhaseTreble3();
		}
        else if (gsAttack[7])  {
            gsDeltaTreble = gsLimitTreble / gsAttack[7];
            if (gsDeltaTreble == 0) gsDeltaTreble = 1;
            gsHiValTreble = gsDeltaTreble;
            if (gsPolarityTreble == 0) gsValueTreble = gsHiValTreble >> 6;
            else  gsValueTreble = -(gsHiValTreble >> 6);		//restore neg.sign
            gsPhaseTreble = 1;
        }
        else if (gsDecay[7])  {
            if (gsLimitTreble >= gsSustainHTreble) {
                gsSignTreble = 0;
                gsDeltaTreble = (gsLimitTreble - gsSustainHTreble) / gsDecay[7];
            }
            else {
                gsSignTreble = 1;
                gsDeltaTreble = (gsSustainHTreble - gsLimitTreble) / gsDecay[7];
            }
            if (gsDeltaTreble == 0) gsDeltaTreble = 1 ;
            gsHiValTreble = gsLimitTreble;
            if (gsPolarityTreble == 0) gsValueTreble = gsHiValTreble >> 6;
            else  gsValueTreble = -(gsHiValTreble >> 6);	//restore neg.sign
            gsPhaseTreble = 2;
        }
        else if (gsSustainHTreble)  {
            gsHiValTreble = gsSustainHTreble;
            if (gsPolarityTreble == 0) gsValueTreble = gsHiValTreble >> 6;
            else  gsValueTreble = -(gsHiValTreble >> 6);	//restore neg.sign
            gsDeltaTreble = 0;		//no Countdown, gets terminated by NOTE OFF
            gsPhaseTreble = 3;
            if (gsHoldSust[7] == 0)  releaseTreble();
        }
    }

// ADSR8 controls envelope of Gain
    if (gsPhaseG) shutdownGain(velocity);
    else triggerGain(velocity);

//ADSR9 controls filter#0 mid freq., see below

// ADSR10 controls Filter0 Peak
    gsPhaseF0Pk = 0; //these settings as precaution,if no useable parameters
    gsValueF0Pk = 0;

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through
		gsLimitF0Pk = ((int32_t)(gsFixVol[10] - 64) * 2048) +  \
									(32*velocity * ((int32_t)(gsVelo[10] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityF0Pk = 0;
		if (gsLimitF0Pk < 0)  {
			gsLimitF0Pk = -gsLimitF0Pk;
			gsPolarityF0Pk = 1;
		}
//		if (gsLimitTreble> 0x7F000) gsLimitTreble = 0x7F000;//theory:max 0x71800
//bugfix 22Dec22
		if (gsLimitF0Pk > 0x7F000) gsLimitF0Pk = 0x7F000;//theory:max 0x71800
		if (gsLimitF0Pk > 0x100)  {
			gsSustainHF0Pk = (gsSustain[10] * gsLimitF0Pk) / 100; // >> 7;

			if (gsAttack[10])  {
				gsDeltaF0Pk = gsLimitF0Pk / gsAttack[10];
				if (gsDeltaF0Pk == 0) gsDeltaF0Pk = 1;
				gsHiValF0Pk = gsDeltaF0Pk;
				if (gsPolarityF0Pk == 0) gsValueF0Pk = gsHiValF0Pk >> 6;
				else  gsValueF0Pk = -(gsHiValF0Pk >> 6);	//restore neg.sign
				gsPhaseF0Pk = 1;
			}
			else if (gsDecay[10])  {
				if (gsLimitF0Pk >= gsSustainHF0Pk) {
					gsSignF0Pk = 0;
					gsDeltaF0Pk = (gsLimitF0Pk - gsSustainHF0Pk) / gsDecay[10];
				}
				else {
					gsSignF0Pk = 1;
					gsDeltaF0Pk = (gsSustainHF0Pk - gsLimitF0Pk) / gsDecay[10];
				}
				if (gsDeltaF0Pk == 0) gsDeltaF0Pk = 1 ;
				gsHiValF0Pk = gsLimitF0Pk;
				gsPhaseF0Pk = 2;
			}
			else if (gsSustainHF0Pk)  {
				gsHiValF0Pk = gsSustainHF0Pk;
				gsDeltaF0Pk = 0;	//no Countdown, gets terminated by NOTE OFF
				gsPhaseF0Pk = 3;
				if (gsHoldSust[10] == 0)  releaseF0Pk();
			}
			if (gsPolarityF0Pk == 0) gsValueF0Pk = (float)(gsHiValF0Pk) / 744.;
			else  gsValueF0Pk = (float)(-gsHiValF0Pk) / 744.; //744=8192/11
			gfF0TempPeak = (32 * gfRuntimeIIR[0].FPEAKMOD + gsValueF0Pk) / 32;
			if (gfF0TempPeak < 0.0)  gfF0TempPeak = 0.0;
			if (gfF0TempPeak > 11.0)  gfF0TempPeak = 11.0;
		}
		else gfF0TempPeak = gfRuntimeIIR[0].FPEAKMOD;
    }
}

void triggerNoise(uint32_t velocity)  // ***********************************
//ADSR2 controls envelope of Noise
// *************************************************************************
{
    gsPhaseNs = 0; //these settings as precaution,if no useable parameters
    gsValueNs = 0;
    gsOffNs = 0;

    gsLimitNs = (gsFixVol[2]* 2048) + (16*velocity *((int32_t)(gsVelo[2] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
    if (gsLimitNs > 0x7F000) gsLimitNs = 0x7F000; //theoretically max 0x71800
    if (gsLimitNs < 0) gsLimitNs = 0;

    if (gsLimitNs > 0x100)  {
        gsSustainHNs = (gsSustain[2] * gsLimitNs) / 100;

		if ((gsAttack[2] == 0) && (gsDecay[2] == 0)) {
			gsHiValNs = gsSustainHNs;
            gsValueNs = gsHiValNs >> 7;
			intoPhaseNs3();
		}
        else if (gsAttack[2])  {
            gsDeltaNs = gsLimitNs / gsAttack[2];
            if (gsDeltaNs == 0) gsDeltaNs = 1;
            gsHiValNs = gsDeltaNs;
            gsValueNs = gsHiValNs >> 7;
            gsPhaseNs = 1;
        }
        else if (gsDecay[2])  {
            if (gsLimitNs >= gsSustainHNs) {
                gsSignNs = 0;
                gsDeltaNs = (gsLimitNs - gsSustainHNs) / gsDecay[2];
            }
            else {
                gsSignNs = 1;
                gsDeltaNs = (gsSustainHNs - gsLimitNs) / gsDecay[2];
            }
            if (gsDeltaNs == 0) gsDeltaNs = 1 ;
            gsHiValNs = gsLimitNs;
            gsValueNs = gsLimitNs >> 7;
            gsPhaseNs = 2;
        }
        else if (gsSustainHNs)  {
            gsHiValNs = gsSustainHNs;
            gsValueNs = gsSustainHNs >> 7;
            gsDeltaNs = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhaseNs = 3;
            if (gsHoldSust[2] == 0)  releaseNoise();
        }
    }
}


void triggerGain(uint32_t velocity)  // ***********************************
// ADSR8 controls envelope of Gain
// *************************************************************************
{
    gsPhaseG = 0; //these settings as precaution,if no useable parameters
    gsValueG = 0;

    gsLimitG = (gsFixVol[8] * 2048) + (16*velocity *((int32_t)(gsVelo[8] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
    if (gsLimitG > 0x7F000) gsLimitG = 0x7F000; //theoretically max 0x71800
    if (gsLimitG < 0) gsLimitG = 0;

    if (gsLimitG > 0x100)  {
        gsSustainHG = (gsSustain[8] * gsLimitG) / 100;

		if ((gsAttack[8] == 0) && (gsDecay[8] == 0)) {
			gsHiValG = gsSustainHG;
            gsValueG = gsHiValG >> 7;
			intoPhaseG3();
		}
        else if (gsAttack[8])  {
            gsDeltaG = gsLimitG / gsAttack[8];
            if (gsDeltaG == 0) gsDeltaG = 1;
            gsHiValG = gsDeltaG;
            gsValueG = gsHiValG >> 7;
            gsPhaseG = 1;
        }
        else if (gsDecay[8])  {
            if (gsLimitG >= gsSustainHG) {
                gsSignG = 0;
                gsDeltaG = (gsLimitG - gsSustainHG) / gsDecay[8];
            }
            else {
                gsSignG = 4;
                gsDeltaG = (gsSustainHG - gsLimitG) / gsDecay[8];
            }
            if (gsDeltaG == 0) gsDeltaG = 1 ;
            gsHiValG = gsLimitG;
            gsValueG = gsLimitG >>7;
            gsPhaseG = 2;
            if (gsHoldSust[8] == 0)  releaseGain();
        }
        else if (gsSustainHG)  {
            gsHiValG = gsSustainHG;
            gsValueG = gsSustainHG >> 7;
            gsDeltaG = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhaseG = 3;
        }
    }
}

// ---------------------------------------------------------------------------

void shutdownNoise(uint32_t velocity)   // **********************************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDeltaNs = gsHiValNs / SHORT_RELEASE;
    if (gsDeltaNs == 0)  {
        gsHiValNs = 0;
        triggerNoise(velocity);
        return;
    }
    gsVelocityNs = velocity;
    gsPhaseNs = 5;
}


void releaseNoise(void)   // ************************************************
//start Noise Release phase
// **************************************************************************
{
    if (gsPhaseNs) {
        if (gsRelease[2]) {
            gsDeltaNs = gsHiValNs / gsRelease[2];  //FINAL always = 0
            if (gsDeltaNs == 0)  gsDeltaNs = 1;
            gsPhaseNs = 4;
        }
        else  gsOffNs = 1;//don't switch off immediately/zero transition trigger
    }
}


void releaseFMO12(void)   // **************************************************
//start Frequency Mod Osc1 Release phase
// **************************************************************************
{
    if (gsPhaseFMO12) {
        if (gsRelease[3]) {
            gsDeltaFMO12 = gsHiValFMO12 / gsRelease[3];  //FINAL always = 0
            if (gsDeltaFMO12 == 0)  gsDeltaFMO12 = 1;
            gsPhaseFMO12 = 4;
        }
        else {
            gsPhaseFMO12 = 0;
            gsValueFMO12 = 0;
        }
    }
}


void releaseFLfo(void)   // **************************************************
//start Frequency Mod Lfo1 Release Phase
// **************************************************************************
{
    if (gsPhaseFLfo) {
        if (gsRelease[4]) {
            gsDeltaFLfo = gsHiValFLfo / gsRelease[4];  //FINAL always = 0
            if (gsDeltaFLfo == 0)  gsDeltaFLfo = 1;
            gsPhaseFLfo = 4;
        }
        else {
            gsPhaseFLfo = 0;
            gsValueFLfo = 0;
        }
    }
}


void releaseVib(void)   // **************************************************
//start Lfo2 ADSR Release Phase
// **************************************************************************
{
    if (gsPhaseVib) {
        if (gsRelease[5]) {
            gsDeltaVib = gsHiValVib / gsRelease[5];  //FINAL always = 0
            if (gsDeltaVib == 0)  gsDeltaVib = 1;
            gsPhaseVib = 4;
        }
        else {
            gsPhaseVib = 0;
            gsValueVib = 0;
        }
    }
}


void releaseBass(void)   // *************************************************
//start Bass Release Phase
// **************************************************************************
{
    if (gsPhaseBass) {
        if (gsRelease[6]) {
            gsDeltaBass = gsHiValBass / gsRelease[6];  //FINAL always = 0
            if (gsDeltaBass == 0)  gsDeltaBass = 1;
            gsPhaseBass = 4;
        }
        else {
            gsPhaseBass = 0;
            gsValueBass = 0;
        }
    }
}


void releaseTreble(void)   // ***********************************************
//start Treble Release Phase
// **************************************************************************
{
    if (gsPhaseTreble) {
        if (gsRelease[7]) {
            gsDeltaTreble = gsHiValTreble / gsRelease[7];  //FINAL always = 0
            if (gsDeltaTreble == 0)  gsDeltaTreble = 1;
            gsPhaseTreble = 4;
        }
        else {
            gsPhaseTreble = 0;
            gsValueTreble = 0;
        }
    }
}


void shutdownGain(uint32_t velocity)   // ***********************************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDeltaG = gsHiValG / SHORT_RELEASE;
    if (gsDeltaG == 0)  {
        gsHiValG = 0;
        triggerGain(velocity);
        return;
    }
    gsVelocityG = velocity;
    gsPhaseG = 5;
}


void releaseGain(void)   // *************************************************
//start Gain Release phase
// **************************************************************************
{
    if (gsPhaseG) {
        if (gsRelease[8]) {
            gsDeltaG = gsHiValG / gsRelease[8];  //FINAL always = 0
            if (gsDeltaG == 0)  gsDeltaG = 1;
            gsPhaseG = 4;
        }
        else  {
            gsPhaseG = 0;
            gsValueG = 0;
        }
    }
}


void releaseF0Pk(void)   // *************************************************
//start Filter0 Peak ADSR Release Phase
// **************************************************************************
{
    if (gsPhaseF0Pk) {
        if (gsRelease[10]) {
            gsDeltaF0Pk = gsHiValF0Pk / gsRelease[10];  //FINAL always = 0
            if (gsDeltaF0Pk == 0)  gsDeltaF0Pk = 1;
            gsPhaseF0Pk = 4;
        }
        else {
            gfF0TempPeak = gfRuntimeIIR[0].FPEAKMOD;
            gsPhaseF0Pk = 0;
        }
    }
}


void intoPhaseNs2(void)  // ************************************************
//  ************************************************************************
{
    if (gsDecay[2])  {
        if (gsLimitNs == gsSustainHNs) {
            intoPhaseNs3();
            return;
        }
        if (gsLimitNs <= gsSustainHNs) {
            gsSignNs = 1;
            gsDeltaNs = (gsSustainHNs - gsLimitNs) / gsDecay[2];
        }
        else {
            gsSignNs = 0;
            gsDeltaNs = (gsLimitNs - gsSustainHNs) / gsDecay[2];
        }
        if (gsDeltaNs == 0) gsDeltaNs = 1;
        gsPhaseNs = 2;
    }
    else  intoPhaseNs3();
}


void intoPhaseNs3(void)  // ************************************************
//  ************************************************************************
{
    if (gsHoldSust[2] == 0)  {
        releaseNoise();
        return;
    }
    if ((gsSustainHNs) && (gsHiValNs > 0))  {
        gsHiValNs = gsSustainHNs;
        gsValueNs = gsSustainHNs >> 7;
        gsDeltaNs = 0;
        gsPhaseNs = 3;
    }
    else gsPhaseNs = 0;
}


void intoPhaseFMO122(void)  // ***********************************************
//  ************************************************************************
{
    if (gsDecay[3])  {
        if (gsLimitFMO12 == gsSustainHFMO12) {
            intoPhaseFMO123();
            return;
        }
        if (gsLimitFMO12 <= gsSustainHFMO12) {
            gsSignFMO12 = 1;
            gsDeltaFMO12 = (gsSustainHFMO12 - gsLimitFMO12) / gsDecay[3];
        }
        else {
            gsSignFMO12 = 0;
            gsDeltaFMO12 = (gsLimitFMO12 - gsSustainHFMO12) / gsDecay[3];
        }
        if (gsDeltaFMO12 == 0) gsDeltaFMO12 = 1;
        gsPhaseFMO12 = 2;

    }
    else  intoPhaseFMO123();
}


void intoPhaseFMO123(void)  // ***********************************************
//  ************************************************************************
{
    if (gsHoldSust[3] == 0)  {
        releaseFMO12();
        return;
    }
    if ((gsSustainHFMO12) && (gsHiValFMO12 > 0))  {
        gsHiValFMO12 = gsSustainHFMO12;
        if (gsPolarityFMO12 == 0) gsValueFMO12 = gsHiValFMO12 >> 6;
        else  gsValueFMO12 = -(gsHiValFMO12 >> 6);		//restore neg.sign
        gsDeltaFMO12 = 0;
        gsPhaseFMO12 = 3;
    }
    else {
        gsValueFMO12 = 0;
        gsPhaseFMO12 = 0;
    }
}


void intoPhaseFLfo2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[4])  {
        if (gsLimitFLfo == gsSustainHFLfo) {
            intoPhaseFLfo3();
            return;
        }
        if (gsLimitFLfo <= gsSustainHFLfo) {
            gsSignFLfo = 1;
            gsDeltaFLfo = (gsSustainHFLfo - gsLimitFLfo) / gsDecay[4];
        }
        else {
            gsSignFLfo = 0;
            gsDeltaFLfo = (gsLimitFLfo - gsSustainHFLfo) / gsDecay[4];
        }
        if (gsDeltaFLfo == 0) gsDeltaFLfo = 1;
        gsPhaseFLfo = 2;
    }
    else  intoPhaseFLfo3();
}


void intoPhaseFLfo3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[4] == 0)  {
        releaseFLfo();
        return;
    }
    if ((gsSustainHFLfo) && (gsHiValFLfo > 0))  {
        gsHiValFLfo = gsSustainHFLfo;
		if (gsPolarityFLfo == 0) gsValueFLfo = gsHiValFLfo >> 6;
		else  gsValueFLfo = -(gsHiValFLfo >> 6);		//restore neg.sign
        gsDeltaFLfo = 0;
        gsPhaseFLfo = 3;
    }
    else {
        gsValueFLfo = 0;
        gsPhaseFLfo = 0;
    }
}


void intoPhaseVib2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[5])  {
        if (gsLimitVib == gsSustainHVib) {
            intoPhaseVib3();
            return;
        }
        if (gsLimitVib <= gsSustainHVib) {
            gsSignVib = 1;
            gsDeltaVib = (gsSustainHVib - gsLimitVib) / gsDecay[5];
        }
        else {
            gsSignVib = 0;
            gsDeltaVib = (gsLimitVib - gsSustainHVib) / gsDecay[5];
        }
        if (gsDeltaVib == 0) gsDeltaVib = 1;
        gsPhaseVib = 2;
    }
    else  intoPhaseVib3();
}


void intoPhaseVib3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[5] == 0)  {
        releaseVib();
        return;
    }
    if ((gsSustainHVib) && (gsHiValVib > 0))  {
        gsHiValVib = gsSustainHVib;
		if (gsPolarityVib == 0) gsValueVib = gsHiValVib >> 6;
		else  gsValueVib = -(gsHiValVib >> 6);		//restore neg.sign
        gsDeltaVib = 0;
        gsPhaseVib = 3;
    }
    else {
        gsValueVib = 0;
        gsPhaseVib = 0;
    }
}


void intoPhaseBass2(void)  // **********************************************
//  ************************************************************************
{
    if (gsDecay[6])  {
        if (gsLimitBass == gsSustainHBass) {
            intoPhaseBass3();
            return;
        }
        if (gsLimitBass <= gsSustainHBass) {
            gsSignBass = 1;
            gsDeltaBass = (gsSustainHBass - gsLimitBass) / gsDecay[6];
        }
        else {
            gsSignBass = 0;
            gsDeltaBass = (gsLimitBass - gsSustainHBass) / gsDecay[6];
        }
        if (gsDeltaBass == 0) gsDeltaBass = 1;
        gsPhaseBass = 2;
    }
    else  intoPhaseBass3();
}


void intoPhaseBass3(void)  // **********************************************
//  ************************************************************************
{
    if (gsHoldSust[6] == 0)  {
        releaseBass();
        return;
    }
    if ((gsSustainHBass) && (gsHiValBass > 0))  {
        gsHiValBass = gsSustainHBass;
		if (gsPolarityBass == 0) gsValueBass = gsHiValBass >> 6;
		else  gsValueBass = -(gsHiValBass >> 6);		//restore neg.sign
        gsDeltaBass = 0;
        gsPhaseBass = 3;
    }
    else {
        gsValueBass = 0;
        gsPhaseBass = 0;
    }
}

void intoPhaseTreble2(void)  // *********************************************
//  *************************************************************************
{
    if (gsDecay[7])  {
        if (gsLimitTreble == gsSustainHTreble) {
            intoPhaseTreble3();
            return;
        }
        if (gsLimitTreble <= gsSustainHTreble) {
            gsSignTreble = 1;
            gsDeltaTreble = (gsSustainHTreble - gsLimitTreble) / gsDecay[7];
        }
        else {
            gsSignTreble = 0;
            gsDeltaTreble = (gsLimitTreble - gsSustainHTreble) / gsDecay[7];
        }
        if (gsDeltaTreble == 0) gsDeltaTreble = 1;
        gsPhaseTreble = 2;
    }
    else  intoPhaseTreble3();
}


void intoPhaseTreble3()  //*****************************************************
{
    if (gsHoldSust[7] == 0)  {
        releaseTreble();
        return;
    }
    if ((gsSustainHTreble) && (gsHiValTreble > 0))  {
        gsHiValTreble = gsSustainHTreble;
		if (gsPolarityTreble == 0) gsValueTreble = gsHiValTreble >> 6;
		else  gsValueTreble = -(gsHiValTreble >> 6);		//restore neg.sign
        gsDeltaTreble = 0;
        gsPhaseTreble = 3;
    }
    else {
        gsValueTreble = 0;
        gsPhaseTreble = 0;
    }
}


void intoPhaseG2(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[8])  {

        if (gsLimitG == gsSustainHG) {
            intoPhaseG3();
            return;
        }
        if (gsLimitG <= gsSustainHG) {
            gsSignG = 1;
            gsDeltaG = (gsSustainHG - gsLimitG) / gsDecay[8];
        }
        else {
            gsSignG = 0;
            gsDeltaG = (gsLimitG - gsSustainHG) / gsDecay[8];
        }
        if (gsDeltaG == 0) gsDeltaG = 1;
        gsPhaseG = 2;
    }
    else  intoPhaseG3();
}


void intoPhaseG3(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[8] == 0)  {
        releaseGain();
        return;
    }
    if ((gsSustainHG) && (gsHiValG > 0))  {
        gsHiValG = gsSustainHG;
        gsValueG = gsSustainHG >> 7;
        gsDeltaG = 0;
        gsPhaseG = 3;
    }
    else gsPhaseG = 0;
}


void intoPhaseF0Pk2(void)  // **********************************************
//start Decay phase
//  ************************************************************************
{
    if (gsDecay[10])  {
        if (gsLimitF0Pk == gsSustainHF0Pk) {
            intoPhaseF0Pk3();
            return;
        }
        if (gsLimitF0Pk <= gsSustainHF0Pk) {
            gsSignF0Pk = 1;
            gsDeltaF0Pk = (gsSustainHF0Pk - gsLimitF0Pk) / gsDecay[10];
        }
        else {
            gsSignF0Pk = 0;
            gsDeltaF0Pk = (gsLimitF0Pk - gsSustainHF0Pk) / gsDecay[10];
        }
        if (gsDeltaF0Pk == 0) gsDeltaF0Pk = 1;
        gsPhaseF0Pk = 2;
    }
    else  intoPhaseF0Pk3();
}


void intoPhaseF0Pk3(void)  // **********************************************
//start Sustain phase
//  ************************************************************************
{
    if (gsHoldSust[10] == 0)  {
        releaseF0Pk();
        return;
    }
    if ((gsSustainHF0Pk) && (gsHiValF0Pk > 0x100))  {
        gsHiValF0Pk = gsSustainHF0Pk;
        gsDeltaF0Pk = 0;
        gsPhaseF0Pk = 3;
    }
    else {
		gfF0TempPeak = gfRuntimeIIR[0].FPEAKMOD;
        gsPhaseF0Pk = 0;
    }
}

// -------------------------------------------------------------------------

void tuneVoice0(uint32_t noteval, uint32_t velocity)   // *******************
//noteval is the corresponding MIDI note value
//a correction for Transpose is added/subtracted
//makes a jump to read table of real frequencies
//Waveforms are copied from Flash to Sram before (save a little bit of runtime)
// **************************************************************************
{
// ADSR0 controls envelope of OSC1
// ADSR 1, 2, .. 9  control additional voices of OSC1
    gsPhase0 = 0; //these settings as precaution,if no useable parameters
    gsValue0 = 0;
    gsOff0 = 0;
    gsWavPtr0 = 0.0;

//a remaining level of gsHival0 will be used for smooth transition
    gsLimit0 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
    if (gsLimit0 > 0x7F000) gsLimit0 = 0x7F000; //theoretically max 0x71800
    if (gsLimit0 < 0) gsLimit0 = 0;

    if (gsLimit0 > 0x100)  {
        gsSustainH0 = (gsSustain[0] * gsLimit0) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal0 = gsSustainH0;
            gsValue0 = gsHiVal0 >> 7;
			intoPhase03();
		}
        else if (gsAttack[0])  {
            gsDelta0 = (gsLimit0 - gsHiVal0) / gsAttack[0];
            if (gsDelta0 == 0) gsDelta0 = 1;
            gsHiVal0 = gsDelta0;
            gsValue0 = gsHiVal0 >> 7;
            gsPhase0 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit0 >= gsSustainH0) {
                gsSign0 = 0;
                gsDelta0 = (gsLimit0 - gsSustainH0) / gsDecay[0];
            }
            else {
                gsSign0 = 1;
                gsDelta0 = (gsSustainH0 - gsLimit0) / gsDecay[0];
            }
            if (gsDelta0 == 0) gsDelta0 = 1 ;
			gsHiVal0 = gsLimit0;
			gsValue0 = gsLimit0  >> 7;
            gsPhase0 = 2;
        }
        else if (gsSustainH0)  {
            gsHiVal0 = gsSustainH0;
            gsValue0 = gsSustainH0 >> 7;
            gsDelta0 = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase0 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice0();
        }

        if (gsPhase0) {
            if (gsGLIDE == 0)  {				//then Glide is off
                gsStep0 = gsFloatStep[noteval + gsTRANS - 36];
                gsGlideBusy = 0;
            }
            else  {
                if (gsGlideBusy == 0)  {			//startGlide Process phase1
                    if (gsGlideWait == 0)  {		//keep actual note in mind
                        gsStep0 = gsFloatStep[noteval + gsTRANS - 36];
                        gsStart = gsStep0;
                        gsGlideWait = 1;
                    }
                    else  { //Glide Process phase 2:new note in mind,start glide
						gsStart = gsStep0;          //from previus key touch
						gsFinal = gsFloatStep[noteval + gsTRANS - 36];
						gsDeltaStep = (gsFinal - gsStart) /(A_FACTOR * gsGLIDE);
						gsStepSign = 0;
						if (gsDeltaStep < 0.0) gsStepSign = 1;
						gsGlideBusy = 1;	//will be reset in ADSR_Process()
						gsGlideHold = 0;
                    }
                }
                else  {//Glide Process phase 3:note off, terminate glide process
                    gsStart = gsStep0;			//from previus key touch
                    gsFinal = gsFloatStep[noteval + gsTRANS - 36];
                    gsDeltaStep = (gsFinal - gsStart) / (A_FACTOR * gsGLIDE);
                    gsStepSign = 0;
                    if (gsDeltaStep < 0.0) gsStepSign = 1;
                    gsGlideHold = 0;
                }
            }
			gsRunStep0 = gsStep0 * gModCent1;
        }
    }
}


void tuneVoice0X(uint32_t noteval, uint32_t velocity)   // *******************
// ADSR0X controls envelope of OSC2
// ADSR 1X, 2X, .. 9X control additional voices of OSC2
// ***************************************************************************
{
    gsPhase0X = 0; //these settings as precaution,if no useable parameters
    gsValue0X = 0;
    gsOff0X = 0;
    gsWavPtr0X = 0.0;

    gsLimit0X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit0X > 0x7F000) gsLimit0X = 0x7F000; //theoretically max 0x71800
    if (gsLimit0X < 0) gsLimit0X= 0;

     if (gsLimit0X > 0x100)  {
        gsSustainH0X = (gsSustain[1] * gsLimit0X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal0X = gsSustainH0X;
            gsValue0X = gsHiVal0X >> 7;
			intoPhase0X3();
		}
        else if (gsAttack[1])  {
            gsDelta0X = (gsLimit0X - gsHiVal0X) / gsAttack[1];
            if (gsDelta0X == 0) gsDelta0X = 1;
            gsHiVal0X = gsDelta0X;
            gsValue0X = gsHiVal0X >> 7;
            gsPhase0X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit0X >= gsSustainH0X) {
                gsSign0X = 0;
                gsDelta0X = (gsLimit0X- gsSustainH0X) / gsDecay[1];
            }
            else {
                gsSign0X = 1;
                gsDelta0X = (gsSustainH0X - gsLimit0X) / gsDecay[1];
            }
            if (gsDelta0X == 0) gsDelta0X = 1 ;
            gsHiVal0X = gsLimit0X;
            gsValue0X = gsLimit0X >> 7;
            gsPhase0X = 2;
        }
        else if (gsSustainH0X)  {
            gsHiVal0X = gsSustainH0X;
            gsValue0X = gsSustainH0X >> 7;
            gsDelta0X = 0;                       //gets terminated by NOTE OFF
            gsPhase0X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice0X();
        }

        if (gsPhase0X)  {
           if (gsGLIDE == 0)  {
                gsStep0X = gsFloatStep[noteval + gsTRANS + gsTRANS_2 - 72];
                gsGlideBusyX = 0;
            }
            else  {
                if (gsGlideBusyX == 0)  {
                    if (gsGlideWaitX == 0)  {           //Glide Process phase 1
                        gsStep0X =gsFloatStep[noteval + gsTRANS +gsTRANS_2-72];
                        gsStartX = gsStep0X;
                        gsGlideWaitX = 1;
                    }
                    else  {                             //Glide Process phase 2
                        gsStartX = gsStep0X;            //from previus key touch
                        gsFinalX =gsFloatStep[noteval + gsTRANS +gsTRANS_2-72];
                        gsDeltaStepX = (gsFinalX -gsStartX) /(A_FACTOR*gsGLIDE);
                        gsStepSignX = 0;
                        if (gsDeltaStepX < 0.0) gsStepSignX = 1;
                        gsGlideBusyX = 1;
                    }
                }
                else {                                  //Glide Process phase 3
                    gsStartX = gsStep0X;   //actually calculated in servSystem()
                    gsFinalX = gsFloatStep[noteval + gsTRANS + gsTRANS_2-72];
                    gsDeltaStepX = (gsFinalX - gsStartX) / (A_FACTOR * gsGLIDE);
                    gsStepSignX = 0;
                    if (gsDeltaStepX < 0.0) gsStepSignX = 1;
                }
            }
            gsRunStep0X = gsStep0X * gModCent2;
        }
    }
}


void fastRelease(void)   // *************************************************
// special release for monophonic multikey stack handling
// to avoid sound click noise if keytrack is high
// **************************************************************************
{
    if (gsPhase0) {
        gsDelta0 = gsHiVal0 / 5;    //Release 5ms, FINAL always = 0
        if (gsDelta0 == 0)  gsDelta0 = 1;
                        // hold glide state for 0.5 sec
                        //different values due to different countdown rates
        gsGlideHold = 200;
        gsPhase0 = 4;
    }
    else if (gsGlideHold == 0)  {
        gsGlideWait = 0;                        //phase4 of Glide process
        gsGlideBusy = 0;  //sound already terminated: reset, start new glide
        gsGlideWaitX = 0;                       //phase4 of Glide process
        gsGlideBusyX = 0; //sound already terminated: reset, start new glide
    }
    if (gsPhase0X) {
        gsDelta0X = gsHiVal0X / 5;    //Release 5ms, FINAL always = 0
        if (gsDelta0X == 0)  gsDelta0X = 1;
        gsPhase0X = 4;
    }

    while (gsPhase0 || gsPhase0X) servSystem();	//counted down in ADSR_Process()
}


void shutdownVoice0(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid sound click noise
// **************************************************************************
{
    gsDelta0 = gsHiVal0 / SHORT_RELEASE;
    if (gsDelta0 == 0)  {
        gsHiVal0 = 0;
        tuneVoice0(noteval, velocity);
        return;
    }
    gsNoteVal0 = noteval;
    gsVelocity0 = velocity;
    gsPhase0 = 5;
}


void releaseVoice0(void)   // ************************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase0) {
        if (gsRelease[0]) {
            gsDelta0 = gsHiVal0 / gsRelease[0];  //FINAL always = 0
            if (gsDelta0 == 0)  gsDelta0 = 1;
            if (gsRelease[0] < 800) gsGlideHold = 200;
                        // if release < 0.5 sec, hold glide state for 0.5 sec
                        //different values due to different countdown rates
            gsPhase0 = 4;
        }
        else  {
            gsOff0 = 1; //don't switch off immediately/zero transition trigger
            gsGlideHold = 200;       //if RELEASE=0,hold glide state for 0.5sec
        }
    }
    else if (gsGlideHold == 0)  {
        gsGlideWait = 0;                         //phase4 of Glide process
        gsGlideBusy = 0;  //sound already terminated: reset, start new glide
        gsGlideWaitX = 0;                        //phase4 of Glide process
        gsGlideBusyX = 0; //sound already terminated: reset, start new glide
    }
}


void intoPhase02(void)  // ***********************************************
//  switch to decay phase or if decay=0: switch to sustain phase
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit0 == gsSustainH0) {
            intoPhase03();
            return;
        }
        if (gsLimit0 <= gsSustainH0) {
            gsSign0 = 1;
            gsDelta0 = (gsSustainH0 - gsLimit0) / gsDecay[0];

        }
        else {
            gsSign0 = 0;
            gsDelta0 = (gsLimit0 - gsSustainH0) / gsDecay[0];
        }
        if (gsDelta0 == 0) gsDelta0 = 1;
        gsPhase0 = 2;
    }
    else  intoPhase03();
}


void intoPhase03(void)  // ***********************************************
// switch to sustain phase. If sustain=0, terminate sound
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice0();
        return;
    }
    if ((gsSustainH0) && (gsHiVal0 > 0)) {
        gsHiVal0 = gsSustainH0;
        gsValue0 = gsSustainH0 >> 7;
        gsDelta0 = 0;                           //gets terminated by NOTE OFF
        gsPhase0 = 3;
    }
    else  {
        gsPhase0 = 0;
        gsHiVal0 = 0;
        gsGlideHold = 200;	//if sound already =0, hold glide state for 0.5sec
    }
}


void shutdownVoice0X(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid sound click noise
// **************************************************************************
{
    gsDelta0X = gsHiVal0X / SHORT_RELEASE;
    if (gsDelta0X == 0)  {
        gsHiVal0X = 0;
        tuneVoice0X(noteval, velocity);
        return;
    }
    gsNoteVal0 = noteval;
    gsVelocity0 = velocity;
    gsPhase0X = 5;
}


void releaseVoice0X(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase0X) {
        if (gsRelease[1]) {
            gsDelta0X = gsHiVal0X / gsRelease[1];     //FINAL always = 0
            if (gsDelta0X == 0)  gsDelta0X = 1;
            gsPhase0X = 4;
        }
        else  gsOff0X = 1;//don't switch off immediately/zero transition trigger
    }
}


void intoPhase0X2(void)  // ***********************************************
//  switch to decay phase or if decay=0: switch to sustain phase
//  *************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit0X== gsSustainH0X) {
            intoPhase0X3();
            return;
        }
        if (gsLimit0X<= gsSustainH0X) {
            gsSign0X = 1;
            gsDelta0X = (gsSustainH0X - gsLimit0X) / gsDecay[1];
        }
        else {
            gsSign0X = 0;
            gsDelta0X = (gsLimit0X- gsSustainH0X) / gsDecay[1];
        }
        if (gsDelta0X == 0) gsDelta0X = 1;
        gsPhase0X = 2;
    }
    else  intoPhase0X3();
}


void intoPhase0X3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice0X();
        return;
    }
    if ((gsSustainH0X) && (gsHiVal0X > 0))  {
        gsHiVal0X = gsSustainH0X;
        gsValue0X = gsSustainH0X >> 7;
        gsDelta0X = 0;                           //gets terminated by NOTE OFF
        gsPhase0X = 3;
    }
    else  {
        gsHiVal0X = 0;
        gsPhase0X = 0;
    }
}


void tuneVoice1(uint32_t noteval, uint32_t velocity)   // *********************
// **************************************************************************
{
    gsPhase1 = 0; //these settings as precaution,if no useable parameters
    gsValue1 = 0;
    gsOff1 = 0;
    gsWavPtr1 = 0.0;

    gsLimit1 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
	if (gsLimit1 > 0x7F000) gsLimit1 = 0x7F000; //theoretically max 0x71800
    if (gsLimit1 < 0) gsLimit1 = 0;

     if (gsLimit1 > 0x100)  {
		gsSustainH1 = (gsSustain[0] * gsLimit1) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal1 = gsSustainH1;
            gsValue1 = gsHiVal1 >> 7;
			intoPhase13();
		}
        else if (gsAttack[0])  {
            gsDelta1 = (gsLimit1 - gsHiVal1) / gsAttack[0];
            if (gsDelta1 == 0) gsDelta1 = 1;
            gsHiVal1 = gsDelta1;
            gsValue1 = gsHiVal1 >> 7;
            gsPhase1 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit1 >= gsSustainH1) {
                gsSign1 = 0;
                gsDelta1 = (gsLimit1 - gsSustainH1) / gsDecay[0];
            }
            else {
                gsSign1 = 1;
                gsDelta1 = (gsSustainH1 - gsLimit1) / gsDecay[0];
            }
            if (gsDelta1 == 0) gsDelta1 = 1 ;
            gsHiVal1 = gsLimit1;
            gsValue1 = gsLimit1 >> 7;
            gsPhase1 = 2;
        }
        else if (gsSustainH1)  {
            gsHiVal1 = gsSustainH1;
            gsValue1 = gsSustainH1 >> 7;
            gsDelta1 = 0;                       //gets terminated by NOTE OFF
            gsPhase1 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice1();
        }
        if (gsPhase1)  {
            gsStep1 = gsFloatStep[noteval + gsTRANS - 36];
            gsRunStep1 = gsStep1 * gModCent1;
        }
    }
}

void tuneVoice1X(uint32_t noteval, uint32_t velocity)   // *********************
// **************************************************************************
{
    gsPhase1X = 0; //these settings as precaution,if no useable parameters
    gsValue1X = 0;
    gsOff1X = 0;
    gsWavPtr1X = 0.0;

    gsLimit1X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit1X > 0x7F000) gsLimit1X = 0x7F000; //theoretically max 0x71800
    if (gsLimit1X < 0) gsLimit1X = 0;

    if (gsLimit1X > 0x100)  {
        gsSustainH1X = (gsSustain[1] * gsLimit1X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal1X = gsSustainH1X;
            gsValue1X = gsHiVal1X >> 7;
			intoPhase1X3();
		}
        else if (gsAttack[1])  {
            gsDelta1X = (gsLimit1X - gsHiVal1X) / gsAttack[1];
            if (gsDelta1X == 0) gsDelta1X = 1;
            gsHiVal1X = gsDelta1X;
            gsValue1X = gsHiVal1X >> 7;
            gsPhase1X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit1X >= gsSustainH1X) {
                gsSign1X = 0;
                gsDelta1X = (gsLimit1X - gsSustainH1X) / gsDecay[1];
            }
            else {
                gsSign1X = 1;
                gsDelta1X = (gsSustainH1X - gsLimit1X) / gsDecay[1];
            }
            if (gsDelta1X == 0) gsDelta1X = 1 ;
            gsHiVal1X = gsLimit1X;
            gsValue1X = gsLimit1X >> 7;
            gsPhase1X = 2;
        }
        else if (gsSustainH1X)  {
            gsHiVal1X = gsSustainH1X;
            gsValue1X = gsSustainH1X >> 7;
            gsDelta1X = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase1X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice1X();
        }
        if (gsPhase1X)  {
            gsStep1X = gsFloatStep[noteval + gsTRANS + gsTRANS_2 -72];
            gsRunStep1X = gsStep1X * gModCent2;
        }
    }
}


void shutdownVoice1(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta1 = gsHiVal1 / SHORT_RELEASE;
    if (gsDelta1 == 0)  {
        gsHiVal1 = 0;
        tuneVoice1(noteval,velocity);
        return;
    }
    gsNoteVal1 = noteval;
    gsVelocity1 = velocity;
    gsPhase1 = 5;
}


void releaseVoice1(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase1) {
        if (gsRelease[0]) {
            gsDelta1 = gsHiVal1 / gsRelease[0];  //FINAL always = 0
            if (gsDelta1 == 0)  gsDelta1 = 1;
            gsPhase1 = 4;
        }
        else  gsOff1 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase12(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit1 == gsSustainH1) {
            intoPhase13();
            return;
        }
        if (gsLimit1 <= gsSustainH1) {
            gsSign1 = 1;
            gsDelta1 = (gsSustainH1 - gsLimit1) / gsDecay[0];

        }
        else {
            gsSign1 = 0;
            gsDelta1 = (gsLimit1 - gsSustainH1) / gsDecay[0];
        }
        if (gsDelta1 == 0) gsDelta1 = 1;
        gsPhase1 = 2;
    }
    else  intoPhase13();
}


void intoPhase13(void)  // *************************************************
//  ************************************************************************
{

    if (gsHoldSust[0] == 0)  {
        releaseVoice1();
        return;
    }
    if ((gsSustainH1) && (gsHiVal1 > 0)) {
        gsHiVal1 = gsSustainH1;
        gsValue1 = gsSustainH1 >> 7;
        gsDelta1 = 0;                           //gets terminated by NOTE OFF
        gsPhase1 = 3;
    }
    else  {
        gsHiVal1 = 0;
        gsPhase1 = 0;
    }
}


void shutdownVoice1X(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta1X = gsHiVal1X / SHORT_RELEASE;
    if (gsDelta1X == 0)  {
        gsHiVal1X = 0;
        tuneVoice1X(noteval,velocity);
        return;
    }
    gsNoteVal1 = noteval;
    gsVelocity1 = velocity;
    gsPhase1X = 5;
}


void releaseVoice1X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase1X) {
        if (gsRelease[1]) {
            gsDelta1X = gsHiVal1X / gsRelease[1];  //FINAL always = 0
            if (gsDelta1X == 0)  gsDelta1X = 1;
            gsPhase1X = 4;
        }
        else  gsOff1X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase1X2(void)  // *************************************************
//  *************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit1X == gsSustainH1X) {
            intoPhase1X3();
            return;
        }
        if (gsLimit1X <= gsSustainH1X) {
            gsSign1X = 1;
            gsDelta1X = (gsSustainH1X - gsLimit1X) / gsDecay[1];
        }
        else {
            gsSign1X = 0;
            gsDelta1X = (gsLimit1X - gsSustainH1X) / gsDecay[1];
        }
        if (gsDelta1X == 0) gsDelta1X = 1;
        gsPhase1X = 2;
    }
    else  intoPhase1X3();
}


void intoPhase1X3(void)  // *************************************************
//  *************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice1X();
        return;
    }
    if ((gsSustainH1X) && (gsHiVal1X > 0))  {
        gsHiVal1X = gsSustainH1X;
        gsValue1X = gsSustainH1X >> 7;
        gsDelta1X = 0;                       //gets terminated by NOTE OFF
        gsPhase1X = 3;
    }
    else  {
        gsHiVal1X = 0;
        gsPhase1X = 0;
    }
}


void tuneVoice2(uint32_t noteval, uint32_t velocity)   // ********************
// **************************************************************************
{
    gsPhase2 =0; //these settings as precaution,if no useable parameters
    gsValue2 = 0;
    gsOff2 = 0;
    gsWavPtr2 = 0.0;

    gsLimit2 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
    if (gsLimit2 > 0x7F000) gsLimit2 = 0x7F000; //theoretically max 0x71800
    if (gsLimit2 < 0) gsLimit2 = 0;

    if (gsLimit2 > 0x100)  {
        gsSustainH2 = (gsSustain[0] * gsLimit2) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal2 = gsSustainH2;
            gsValue2 = gsHiVal2 >> 7;
			intoPhase23();
		}
        else if (gsAttack[0])  {
            gsDelta2 = (gsLimit2 - gsHiVal2) / gsAttack[0];
            if (gsDelta2 == 0) gsDelta2 = 1;
            gsHiVal2 = gsDelta2;
            gsValue2 = gsHiVal2 >> 7;
            gsPhase2 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit2 >= gsSustainH2) {
                gsSign2 = 0;
                gsDelta2 = (gsLimit2 - gsSustainH2) / gsDecay[0];
            }
            else {
                gsSign2 = 1;
                gsDelta2 = (gsSustainH2 - gsLimit2) / gsDecay[0];
            }
            if (gsDelta2 == 0) gsDelta2 = 1 ;
            gsHiVal2 = gsLimit2;
            gsValue2 = gsLimit2 >> 7;
            gsPhase2 = 2;
        }
        else if (gsSustainH2)  {
            gsHiVal2 = gsSustainH2;
            gsValue2 = gsSustainH2 >> 7;
            gsDelta2 = 0;                       //gets terminated by NOTE OFF
            gsPhase2 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice2();
        }
        if (gsPhase2) {
            gsStep2 =gsFloatStep[noteval + gsTRANS-36];
            gsRunStep2 = gsStep2 * gModCent1;
        }
    }
}


void tuneVoice2X(uint32_t noteval, uint32_t velocity)   // ******************
// **************************************************************************
{
    gsPhase2X =0; //these settings as precaution,if no useable parameters
    gsValue2X = 0;
    gsOff2X = 0;
    gsWavPtr2X = 0.0;

    gsLimit2X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit2X > 0x7F000) gsLimit2X = 0x7F000; //theoretically max 0x71800
    if (gsLimit2X < 0) gsLimit2X = 0;

    if (gsLimit2X > 0x100)  {
        gsSustainH2X = (gsSustain[1] * gsLimit2X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal2X = gsSustainH2X;
            gsValue2X = gsHiVal2X >> 7;
			intoPhase2X3();
		}
        else if (gsAttack[1])  {
            gsDelta2X = (gsLimit2X - gsHiVal2X) / gsAttack[1];
            if (gsDelta2X == 0) gsDelta2X = 1;
            gsHiVal2X = gsDelta2X;
            gsValue2X = gsHiVal2X >> 7;
            gsPhase2X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit2X >= gsSustainH2X) {
                gsSign2X = 0;
                gsDelta2X = (gsLimit2X - gsSustainH2X) / gsDecay[1];
            }
            else {
                gsSign2X = 1;
                gsDelta2X = (gsSustainH2X - gsLimit2X) / gsDecay[1];
            }
            if (gsDelta2X == 0) gsDelta2X = 1 ;
            gsHiVal2X = gsLimit2X;
            gsValue2X = gsLimit2X >> 7;
            gsPhase2X = 2;
        }
        else if (gsSustainH2X)  {
            gsHiVal2X = gsSustainH2X;
            gsValue2X = gsSustainH2X >> 7;
            gsDelta2X = 0;                       //gets terminated by NOTE OFF
            gsPhase2X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice2X();
        }
        if (gsPhase2X)  {
            gsStep2X=gsFloatStep[noteval + gsTRANS + gsTRANS_2 -72];
            gsRunStep2X = gsStep2X * gModCent2;
        }
    }
}


void shutdownVoice2(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta2 = gsHiVal2 / SHORT_RELEASE;
    if (gsDelta2 == 0)  {
        gsHiVal2 = 0;
        tuneVoice2(noteval, velocity);
        return;
    }
    gsNoteVal2 = noteval;
    gsVelocity2 = velocity;
    gsPhase2 = 5;
}


void releaseVoice2(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase2) {
        if (gsRelease[0]) {
            gsDelta2 = gsHiVal2 / gsRelease[0];  //FINAL always = 0
            if (gsDelta2 == 0)  gsDelta2 = 1;
            gsPhase2 = 4;
        }
        else  gsOff2 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase22(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit2 == gsSustainH2) {
            intoPhase23();
            return;
        }
        if (gsLimit2 <= gsSustainH2) {
            gsSign2 = 1;
            gsDelta2 = (gsSustainH2 - gsLimit2) / gsDecay[0];

        }
        else {
            gsSign2 = 0;
            gsDelta2 = (gsLimit2 - gsSustainH2) / gsDecay[0];
        }
        if (gsDelta2 == 0) gsDelta2 = 1;
        gsPhase2 = 2;
    }
    else  intoPhase23();
}


void intoPhase23(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice2();
        return;
    }
    if ((gsSustainH2) && (gsHiVal2 > 0)) {
        gsHiVal2 = gsSustainH2;
        gsValue2 = gsSustainH2 >> 7;
        gsDelta2 = 0;                           //gets terminated by NOTE OFF
        gsPhase2 = 3;
    }
    else  {
        gsHiVal2 = 0;
        gsPhase2 = 0;
    }
}


void shutdownVoice2X(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta2X = gsHiVal2X / SHORT_RELEASE;
    if (gsDelta2X == 0)  {
        gsHiVal2X = 0;
        tuneVoice2X(noteval, velocity);
        return;
    }
    gsNoteVal2 = noteval;
    gsVelocity2 = velocity;
    gsPhase2X = 5;
}


void releaseVoice2X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase2X) {
        if (gsRelease[1]) {
            gsDelta2X = gsHiVal2X / gsRelease[1];  //FINAL always = 0
            if (gsDelta2X == 0)  gsDelta2X = 1;
            gsPhase2X = 4;
        }
        else  gsOff2X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase2X2(void)  // *************************************************
//  *************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit2X == gsSustainH2X) {
            intoPhase2X3();
            return;
        }
        if (gsLimit2X <= gsSustainH2X) {
            gsSign2X = 1;
            gsDelta2X = (gsSustainH2X - gsLimit2X) / gsDecay[1];
        }
        else {
            gsSign2X = 0;
            gsDelta2X = (gsLimit2X - gsSustainH2X) / gsDecay[1];
        }
        if (gsDelta2X == 0) gsDelta2X = 1;
        gsPhase2X = 2;
    }
    else  intoPhase2X3();
}


void intoPhase2X3(void)  // *************************************************
//  *************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice2X();
        return;
    }
    if ((gsSustainH2X) && (gsHiVal2X > 0))  {
        gsHiVal2X = gsSustainH2X;
        gsValue2X = gsSustainH2X >> 7;
        gsDelta2X = 0;           //no Countdown, gets terminated by NOTE OFF
        gsPhase2X = 3;
    }
    else  {
        gsHiVal2X = 0;
        gsPhase2X = 0;
    }
}


void tuneVoice3(uint32_t noteval, uint32_t velocity)   // *******************
// *******************13*****************************************************
{
    gsPhase3 =0; //these settings as precaution,if no useable parameters
    gsValue3 = 0;
    gsOff3 = 0;
    gsWavPtr3 = 0.0;

    gsLimit3 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
    if (gsLimit3 > 0x7F000) gsLimit3 = 0x7F000; //theoretically max 0x71800
    if (gsLimit3 < 0) gsLimit3 = 0;

    if (gsLimit3 > 0x100)  {
        gsSustainH3 = (gsSustain[0] * gsLimit3) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal3 = gsSustainH3;
            gsValue3 = gsHiVal3 >> 7;
			intoPhase33();
		}
        else if (gsAttack[0])  {
            gsDelta3 = (gsLimit3 - gsHiVal3) /gsAttack[0];
            if (gsDelta3 == 0) gsDelta3 = 1;
            gsHiVal3 = gsDelta3;
            gsValue3 = gsHiVal3 >> 7;
            gsPhase3 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit3 >= gsSustainH3) {
                gsSign3 = 0;
                gsDelta3 = (gsLimit3 - gsSustainH3) / gsDecay[0];
            }
            else {
                gsSign3 = 1;
                gsDelta3 = (gsSustainH3 - gsLimit3) / gsDecay[0];
            }
            if (gsDelta3 == 0) gsDelta3 = 1 ;
            gsHiVal3 = gsLimit3;
            gsValue3 = gsLimit3 >> 7;
            gsPhase3 = 2;
        }
        else if (gsSustainH3)  {
            gsHiVal3 = gsSustainH3;
            gsValue3 = gsSustainH3 >> 7;
            gsDelta3 = 0;                       //gets terminated by NOTE OFF
            gsPhase3 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice3();
        }
        if (gsPhase3)   {
            gsStep3 = gsFloatStep[noteval + gsTRANS-36];
            gsRunStep3 = gsStep3 * gModCent1;
        }
    }
}


void tuneVoice3X(uint32_t noteval, uint32_t velocity)   // *******************
// *******************13*****************************************************
{
    gsPhase3X =0; //these settings as precaution,if no useable parameters
    gsValue3X = 0;
    gsOff3X = 0;
    gsWavPtr3X = 0.0;

    gsLimit3X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1] -28)));
    if (gsLimit3X > 0x7F000) gsLimit3X = 0x7F000; //theoretically max 0x71800
    if (gsLimit3X < 0) gsLimit3X = 0;

    if (gsLimit3X > 0x100)  {
        gsSustainH3X = (gsSustain[1] * gsLimit3X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal3X = gsSustainH3X;
            gsValue3X = gsHiVal3X >> 7;
			intoPhase3X3();
		}
        else  if (gsAttack[1])  {
            gsDelta3X = (gsLimit3X - gsHiVal3X) / gsAttack[1];
            if (gsDelta3X == 0) gsDelta3X = 1;
            gsHiVal3X = gsDelta3X;
            gsValue3X = gsHiVal3X >> 7;
            gsPhase3X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit3X >= gsSustainH3X) {
                gsSign3X = 0;
                gsDelta3X = (gsLimit3X - gsSustainH3X) / gsDecay[1];
            }
            else {
                gsSign3X = 1;
                gsDelta3X = (gsSustainH3X - gsLimit3X) / gsDecay[1];
            }
            if (gsDelta3X == 0) gsDelta3X = 1 ;
            gsHiVal3X = gsLimit3X;
            gsValue3X = gsLimit3X >> 7;
            gsPhase3X = 2;
        }
        else if (gsSustainH3X)  {
            gsHiVal3X = gsSustainH3X;
            gsValue3X = gsSustainH3X >> 7;
            gsDelta3X = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase3X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice3X();
        }
        if (gsPhase3X)  {
            gsStep3X=gsFloatStep[noteval + gsTRANS + gsTRANS_2 -72];
            gsRunStep3X = gsStep3X * gModCent2;
        }
    }
}


void shutdownVoice3(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta3 = gsHiVal3 / SHORT_RELEASE;
    if (gsDelta3 == 0)  {
        gsHiVal3 = 0;
        tuneVoice3(noteval, velocity);
        return;
    }
    gsNoteVal3 = noteval;
    gsVelocity3 = velocity;
    gsPhase3 = 5;
}


void releaseVoice3(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase3) {
        if (gsRelease[0]) {
            gsDelta3 = gsHiVal3 / gsRelease[0]; //FINAL always = 0
            if (gsDelta3 == 0)  gsDelta3 = 1;
            gsPhase3 = 4;
        }
        else  gsOff3 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase32(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit3 == gsSustainH3) {
            intoPhase33();
            return;
        }
        if (gsLimit3 <= gsSustainH3) {
            gsSign3 = 1;
            gsDelta3 = (gsSustainH3 - gsLimit3) / gsDecay[0];

        }
        else {
            gsSign3 = 0;
            gsDelta3 = (gsLimit3 - gsSustainH3) / gsDecay[0];
        }
        if (gsDelta3 == 0) gsDelta3 = 1;
        gsPhase3 = 2;
    }
    else  intoPhase33();
}


void intoPhase33(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice3();
        return;
    }
    if ((gsSustainH3) && (gsHiVal3 > 0)) {
        gsHiVal3 = gsSustainH3;
        gsValue3 = gsSustainH3 >> 7;
        gsDelta3 = 0;                           //gets terminated by NOTE OFF
        gsPhase3 = 3;
    }
    else  {
        gsHiVal3 = 0;
        gsPhase3 = 0;
    }
}


void shutdownVoice3X(uint32_t noteval, uint32_t velocity)   // **************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta3X = gsHiVal3X / SHORT_RELEASE;
    if (gsDelta3X == 0)  {
        gsHiVal3X = 0;
        tuneVoice3X(noteval, velocity);
        return;
    }
    gsNoteVal3 = noteval;
    gsVelocity3 = velocity;
    gsPhase3X = 5;
}


void releaseVoice3X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase3X) {
        if (gsRelease[1]) {
            gsDelta3X = gsHiVal3X / gsRelease[1]; //FINAL always = 0
            if (gsDelta3X == 0)  gsDelta3X = 1;
            gsPhase3X = 4;
        }
        else  gsOff3X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase3X2(void)  // *************************************************
//  *************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit3X == gsSustainH3X) {
            intoPhase3X3();
            return;
        }
        if (gsLimit3X <= gsSustainH3X) {
            gsSign3X = 1;
            gsDelta3X = (gsSustainH3X - gsLimit3X) / gsDecay[1];
        }
        else {
            gsSign3X = 0;
            gsDelta3X = (gsLimit3X - gsSustainH3X) / gsDecay[1];
        }
        if (gsDelta3X == 0) gsDelta3X = 1;
        gsPhase3X = 2;
    }
    else  intoPhase3X3();
}


void intoPhase3X3(void)  // ************************************************
//  ************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice3X();
        return;
    }
    if ((gsSustainH3X) && (gsHiVal3X > 0))  {
        gsHiVal3X = gsSustainH3X;
        gsValue3X = gsSustainH3X >> 7;
        gsDelta3X = 0;           //no Countdown, gets terminated by NOTE OFF
        gsPhase3X = 3;
    }
     else  {
        gsHiVal3X = 0;
        gsPhase3X = 0;
    }
}


void tuneVoice4(uint32_t noteval, uint32_t velocity)   // *******************
// **************************************************************************
{
    gsPhase4 =0; //these settings as precaution,if no useable parameters
    gsValue4 = 0;
    gsOff4 = 0;
    gsWavPtr4 = 0.0;

    gsLimit4 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
    if (gsLimit4 > 0x7F000) gsLimit4 = 0x7F000; //theoretically max 0x71800
    if (gsLimit4 < 0) gsLimit4 = 0;

    if (gsLimit4 > 0x100)  {
        gsSustainH4 = (gsSustain[0] * gsLimit4) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal4 = gsSustainH4;
            gsValue4 = gsHiVal4 >> 7;
			intoPhase43();
		}
        else if (gsAttack[0])  {
            gsDelta4 = (gsLimit4 - gsHiVal4) / gsAttack[0];
            if (gsDelta4 == 0) gsDelta4 = 1;
            gsHiVal4 = gsDelta4;
            gsValue4 = gsHiVal4 >> 7;
            gsPhase4 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit4 >= gsSustainH4) {
                gsSign4 = 0;
                gsDelta4 = (gsLimit4 - gsSustainH4) / gsDecay[0];
            }
            else {
                gsSign4 = 1;
                gsDelta4 = (gsSustainH4 - gsLimit4) / gsDecay[0];
            }
            if (gsDelta4 == 0) gsDelta4 = 1 ;
            gsHiVal4 = gsLimit4;
            gsValue4 = gsLimit4 >> 7;
            gsPhase4 = 2;
        }
        else if (gsSustainH4)  {
            gsHiVal4 = gsSustainH4;
            gsValue4 = gsSustainH4 >> 7;
            gsDelta4 = 0;                       //gets terminated by NOTE OFF
            gsPhase4 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice4();
        }
        if (gsPhase4)  {
            gsStep4 = gsFloatStep[noteval + gsTRANS - 36];
            gsRunStep4 = gsStep4 * gModCent1;
        }
    }
}


void tuneVoice4X(uint32_t noteval, uint32_t velocity)   // ******************
// **************************************************************************
{
    gsPhase4X =0; //these settings as precaution,if no useable parameters
    gsValue4X = 0;
    gsOff4X = 0;
    gsWavPtr4X = 0.0;

    gsLimit4X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit4X > 0x7F000) gsLimit4X = 0x7F000; //theoretically max 0x71800
    if (gsLimit4X < 0) gsLimit4X = 0;

    if (gsLimit4X > 0x100)  {
        gsSustainH4X = (gsSustain[1] * gsLimit4X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal4X = gsSustainH4X;
            gsValue4X = gsHiVal4X >> 7;
			intoPhase4X3();
		}
        else if (gsAttack[1])  {
            gsDelta4X = (gsLimit4X - gsHiVal4X) / gsAttack[1];
            if (gsDelta4X == 0) gsDelta4X = 1;
            gsHiVal4X = gsDelta4X;
            gsValue4X = gsHiVal4X >> 7;
            gsPhase4X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit4X >= gsSustainH4X) {
                gsSign4X = 0;
                gsDelta4X = (gsLimit4X - gsSustainH4X) / gsDecay[1];
            }
            else {
                gsSign4X = 1;
                gsDelta4X = (gsSustainH4X - gsLimit4X) / gsDecay[1];
            }
            if (gsDelta4X == 0) gsDelta4X = 1 ;
            gsHiVal4X = gsLimit4X;
            gsValue4X = gsLimit4X >> 7;
            gsPhase4X = 2;
        }
        else if (gsSustainH4X)  {
            gsHiVal4X = gsSustainH4X;
            gsValue4X = gsSustainH4X >> 7;
            gsDelta4X = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase4X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice4X();
        }
        if (gsPhase4X)  {
            gsStep4X=gsFloatStep[noteval + gsTRANS + gsTRANS_2 -72];
            gsRunStep4X = gsStep4X * gModCent2;
        }
    }
}


void shutdownVoice4(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta4 = gsHiVal4 / SHORT_RELEASE;
    if (gsDelta4 == 0)  {
        gsHiVal4 = 0;
        tuneVoice4(noteval, velocity);
        return;
    }
    gsNoteVal4 = noteval;
    gsVelocity4 = velocity;
    gsPhase4 = 5;
}


void releaseVoice4(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase4) {
        if (gsRelease[0]) {
            gsDelta4 = gsHiVal4 / gsRelease[0]; //FINAL always = 0
            if (gsDelta4 == 0)  gsDelta4 = 1;
            gsPhase4 = 4;
        }
        else  gsOff4 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase42(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit4 == gsSustainH4) {
            intoPhase43();
            return;
        }
        if (gsLimit4 <= gsSustainH4) {
            gsSign4 = 1;
            gsDelta4= (gsSustainH4 - gsLimit4) / gsDecay[0];

        }
        else {
            gsSign4 = 0;
            gsDelta4 = (gsLimit4 - gsSustainH4) / gsDecay[0];
        }
        if (gsDelta4 == 0) gsDelta4 = 1;
        gsPhase4 = 2;
    }
    else  intoPhase43();
}


void intoPhase43(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice4();
        return;
    }
    if ((gsSustainH4) && (gsHiVal4 > 0)) {
        gsHiVal4 = gsSustainH4;
        gsValue4 = gsSustainH4 >> 7;
        gsDelta4 = 0;                           //gets terminated by NOTE OFF
        gsPhase4 = 3;
    }
    else  {
        gsHiVal4 = 0;
        gsPhase4 = 0;
    }
}


void shutdownVoice4X(uint32_t noteval, uint32_t velocity)   // **************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta4X = gsHiVal4X / SHORT_RELEASE;
    if (gsDelta4X == 0)  {
        gsHiVal4X = 0;
        tuneVoice4X(noteval, velocity);
        return;
    }
    gsNoteVal4 = noteval;
    gsVelocity4 = velocity;
    gsPhase4X = 5;
}


void releaseVoice4X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase4X) {
        if (gsRelease[1]) {
            gsDelta4X = gsHiVal4X / gsRelease[1]; //FINAL always = 0
            if (gsDelta4X == 0)  gsDelta4X = 1;
            gsPhase4X = 4;
        }
        else  gsOff4X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase4X2(void)  // ************************************************
//  ************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit4X == gsSustainH4X) {
            intoPhase4X3();
            return;
        }
        if (gsLimit4X <= gsSustainH4X) {
            gsSign4X = 1;
            gsDelta4X = (gsSustainH4X - gsLimit4X) / gsDecay[1];
        }
        else {
            gsSign4X = 0;
            gsDelta4X = (gsLimit4X - gsSustainH4X) / gsDecay[1];
        }
        if (gsDelta4X == 0) gsDelta4X = 1;
        gsPhase4X = 2;
    }
    else  intoPhase4X3();
}


void intoPhase4X3(void)  // ************************************************
//  ************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice4X();
        return;
    }
    if ((gsSustainH4X) && (gsHiVal4X > 0))  {
        gsHiVal4X = gsSustainH4X;
        gsValue4X = gsSustainH4X >> 7;
        gsDelta4X = 0;                           //gets terminated by NOTE OFF
        gsPhase4X = 3;
    }
    else  {
        gsHiVal4X = 0;
        gsPhase4X = 0;
    }
}


void tuneVoice5(uint32_t noteval, uint32_t velocity)   // *******************
// **************************************************************************
{
    gsPhase5 =0; //these settings as precaution,if no useable parameters
    gsValue5 = 0;
    gsOff5 = 0;
    gsWavPtr5 = 0.0;

    gsLimit5 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
    if (gsLimit5 > 0x7F000) gsLimit5 = 0x7F000; //theoretically max 0x71800
    if (gsLimit5 < 0) gsLimit5 = 0;

    if (gsLimit5 > 0x100)  {
        gsSustainH5 = (gsSustain[0] * gsLimit5) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal5 = gsSustainH5;
            gsValue5 = gsHiVal5 >> 7;
			intoPhase53();
		}
        else  if (gsAttack[0])  {
            gsDelta5 = (gsLimit5 - gsHiVal5) / gsAttack[0];
            if (gsDelta5 == 0) gsDelta5 = 1;
            gsHiVal5 = gsDelta5;
            gsValue5 = gsHiVal5 >> 7;
            gsPhase5 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit5 >= gsSustainH5) {
                gsSign5 = 0;
                gsDelta5 = (gsLimit5 - gsSustainH5) / gsDecay[0];
            }
            else {
                gsSign5 = 1;
                gsDelta5 = (gsSustainH5 - gsLimit5) / gsDecay[0];
            }
            if (gsDelta5 == 0) gsDelta5 = 1 ;
            gsHiVal5 = gsLimit5;
            gsValue5 = gsLimit5 >> 7;
            gsPhase5 = 2;
        }
        else if (gsSustainH5)  {
            gsHiVal5 = gsSustainH5;
            gsValue5 = gsSustainH5 >> 7;
            gsDelta5 = 0;                       //gets terminated by NOTE OFF
            gsPhase5 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice5();
        }
        if (gsPhase5)   {
            gsStep5 = gsFloatStep[noteval + gsTRANS - 36];
            gsRunStep5 = gsStep5 * gModCent1;
        }
    }
}


void tuneVoice5X(uint32_t noteval, uint32_t velocity)   // ******************
// **************************************************************************
{
    gsPhase5X =0; //these settings as precaution,if no useable parameters
    gsValue5X = 0;
    gsOff5X = 0;
    gsWavPtr5X = 0.0;

    gsLimit5X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit5X > 0x7F000) gsLimit5X = 0x7F000; //theoretically max 0x71800
    if (gsLimit5X < 0) gsLimit5X = 0;

    if (gsLimit5X > 0x100)  {
        gsSustainH5X = (gsSustain[1] * gsLimit5X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal5X = gsSustainH5X;
            gsValue5X = gsHiVal5X >> 7;
			intoPhase5X3();
		}
        else if (gsAttack[1])  {
            gsDelta5X = (gsLimit5X - gsHiVal5X) / gsAttack[1];
            if (gsDelta5X == 0) gsDelta5X = 1;
            gsHiVal5X = gsDelta5X;
            gsValue5X = gsHiVal5X >> 7;
            gsPhase5X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit5X >= gsSustainH5X) {
                gsSign5X = 0;
                gsDelta5X = (gsLimit5X - gsSustainH5X) / gsDecay[1];
            }
            else {
                gsSign5X = 1;
                gsDelta5X = (gsSustainH5X - gsLimit5X) / gsDecay[1];
            }
            if (gsDelta5X == 0) gsDelta5X = 1 ;
            gsHiVal5X = gsLimit5X;
            gsValue5X = gsLimit5X >> 7;
            gsPhase5X = 2;
        }
        else if (gsSustainH5X)  {
            gsHiVal5X = gsSustainH5X;
            gsValue5X = gsSustainH5X >> 7;
            gsDelta5X = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase5X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice5X();
        }
        if (gsPhase5X)  {
            gsStep5X=gsFloatStep[noteval + gsTRANS + gsTRANS_2 -72];
            gsRunStep5X = gsStep5X * gModCent2;
        }
    }
}


void shutdownVoice5(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta5 = gsHiVal5 / SHORT_RELEASE;
    if (gsDelta5 == 0)  {
        gsHiVal5 = 0;
        tuneVoice5(noteval, velocity);
        return;
    }
    gsNoteVal5 = noteval;
    gsVelocity5 = velocity;
    gsPhase5 = 5;
}


void releaseVoice5(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase5) {
        if (gsRelease[0]) {
            gsDelta5 = gsHiVal5 / gsRelease[0]; //FINAL always = 0
            if (gsDelta5 == 0)  gsDelta5 = 1;
            gsPhase5 = 4;
        }
        else  gsOff5 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase52(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit5 == gsSustainH5) {
            intoPhase53();
            return;
        }
        if (gsLimit5 <= gsSustainH5) {
            gsSign5 = 1;
            gsDelta5= (gsSustainH5 - gsLimit5) / gsDecay[0];

        }
        else {
            gsSign5 = 0;
            gsDelta5 = (gsLimit5 - gsSustainH5) / gsDecay[0];
        }
        if (gsDelta5 == 0) gsDelta5 = 1;
        gsPhase5 = 2;
    }
    else  intoPhase53();
}


void intoPhase53(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice5();
        return;
    }
    if ((gsSustainH5) && (gsHiVal5 > 0)) {
        gsHiVal5 = gsSustainH5;
        gsValue5 = gsSustainH5 >> 7;
        gsDelta5 = 0;                           //gets terminated by NOTE OFF
        gsPhase5 = 3;
    }
    else  {
        gsHiVal5 = 0;
        gsPhase5 = 0;
    }
}


void shutdownVoice5X(uint32_t noteval, uint32_t velocity)   // **************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta5X = gsHiVal5X / SHORT_RELEASE;
    if (gsDelta5X == 0)  {
        gsHiVal5X = 0;
        tuneVoice5X(noteval, velocity);
        return;
    }
    gsNoteVal5 = noteval;
    gsVelocity5 = velocity;
    gsPhase5X = 5;
}


void releaseVoice5X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase5X) {
        if (gsRelease[1]) {
            gsDelta5X = gsHiVal5X / gsRelease[1]; //FINAL always = 0
            if (gsDelta5X == 0)  gsDelta5X = 1;
            gsPhase5X = 4;
        }
        else  gsOff5X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase5X2(void)  // ************************************************
//  ************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit5X == gsSustainH5X) {
            intoPhase5X3();
            return;
        }
        if (gsLimit5X <= gsSustainH5X) {
            gsSign5X = 1;
            gsDelta5X = (gsSustainH5X - gsLimit5X) / gsDecay[1];
        }
        else {
            gsSign5X = 0;
            gsDelta5X = (gsLimit5X - gsSustainH5X) / gsDecay[1];
        }
        if (gsDelta5X == 0) gsDelta5X = 1;
        gsPhase5X = 2;
    }
    else  intoPhase5X3();
}


void intoPhase5X3(void)  // *************************************************
//  *************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice5X();
        return;
    }
    if ((gsSustainH5X) && (gsHiVal5X > 0))  {
        gsHiVal5X = gsSustainH5X;
        gsValue5X = gsSustainH5X >> 7;
        gsDelta5X = 0;                           //gets terminated by NOTE OFF
        gsPhase5X = 3;
    }
     else  {
        gsHiVal5X = 0;
        gsPhase5X = 0;
    }
}


void tuneVoice6(uint32_t noteval, uint32_t velocity)   // *******************
// **************************************************************************
{
    gsPhase6 =0; //these settings as precaution,if no useable parameters
    gsValue6 = 0;
    gsOff6 = 0;
    gsWavPtr6 = 0.0;

    gsLimit6 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
    if (gsLimit6 > 0x7F000) gsLimit6 = 0x7F000; //theoretically max 0x71800
    if (gsLimit6 < 0) gsLimit6 = 0;

    if (gsLimit6 > 0x100)  {
        gsSustainH6 = (gsSustain[0] * gsLimit6) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal6 = gsSustainH6;
            gsValue6 = gsHiVal6 >> 7;
			intoPhase63();
		}
        else if (gsAttack[0])  {
            gsDelta6 = (gsLimit6 - gsHiVal6) / gsAttack[0];
            if (gsDelta6 == 0) gsDelta6 = 1;
            gsHiVal6 = gsDelta6;
            gsValue6 = gsHiVal6 >> 7;
            gsPhase6 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit6 >= gsSustainH6) {
                gsSign6 = 0;
                gsDelta6 = (gsLimit6 - gsSustainH6) / gsDecay[0];
            }
            else {
                gsSign6 = 1;
                gsDelta6 = (gsSustainH6 - gsLimit6) / gsDecay[0];
            }
            if (gsDelta6 == 0) gsDelta6 = 1 ;
            gsHiVal6 = gsLimit6;
            gsValue6 = gsLimit6 >> 7;
            gsPhase6 = 2;
        }
        else if (gsSustainH6)  {
            gsHiVal6 = gsSustainH6;
            gsValue6 = gsSustainH6 >> 7;
            gsDelta6 = 0;                       //gets terminated by NOTE OFF
            gsPhase6 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice6();
        }
        if (gsPhase6)   {
            gsStep6 = gsFloatStep[noteval+gsTRANS-36];
            gsRunStep6 = gsStep6 * gModCent1;
        }
    }
}


void tuneVoice6X(uint32_t noteval, uint32_t velocity)   // *****************
// **************************************************************************
{
    gsPhase6X =0; //these settings as precaution,if no useable parameters
    gsValue6X = 0;
    gsOff6X = 0;
    gsWavPtr6X = 0.0;

    gsLimit6X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit6X > 0x7F000) gsLimit6X = 0x7F000; //theoretically max 0x71800
    if (gsLimit6X < 0) gsLimit6X = 0;

    if (gsLimit6X > 0x100)  {
        gsSustainH6X = (gsSustain[1] * gsLimit6X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal6X = gsSustainH6X;
            gsValue6X = gsHiVal6X >> 7;
			intoPhase6X3();
		}
		else if (gsAttack[1])  {
            gsDelta6X = (gsLimit6X - gsHiVal6X) / gsAttack[1];
            if (gsDelta6X == 0) gsDelta6X = 1;
            gsHiVal6X = gsDelta6X;
            gsValue6X = gsHiVal6X >> 7;
            gsPhase6X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit6X >= gsSustainH6X) {
                gsSign6X = 0;
                gsDelta6X = (gsLimit6X - gsSustainH6X) / gsDecay[1];
            }
            else {
                gsSign6X = 1;
                gsDelta6X = (gsSustainH6X - gsLimit6X) / gsDecay[1];
            }
            if (gsDelta6X == 0) gsDelta6X = 1 ;
            gsHiVal6X = gsLimit6X;
            gsValue6X = gsLimit6X >> 7;
            gsPhase6X = 2;
        }
        else if (gsSustainH6X)  {
            gsHiVal6X = gsSustainH6X;
            gsValue6X = gsSustainH6X >> 7;
            gsDelta6X = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase6X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice6X();
        }
        if (gsPhase6X)  {
            gsStep6X = gsFloatStep[noteval +gsTRANS +gsTRANS_2 -72];
            gsRunStep6X = gsStep6X * gModCent2;
        }
    }
}


void shutdownVoice6(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta6 = gsHiVal6 / SHORT_RELEASE;
    if (gsDelta6 == 0)  {
        gsHiVal6 = 0;
        tuneVoice6(noteval, velocity);
        return;
    }
    gsNoteVal6 = noteval;
    gsVelocity6 = velocity;
    gsPhase6 = 5;
}


void releaseVoice6(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase6) {
        if (gsRelease[0]) {
            gsDelta6 = gsHiVal6 / gsRelease[0]; //FINAL always = 0
            if (gsDelta6 == 0)  gsDelta6 = 1;
            gsPhase6 = 4;
        }
        else  gsOff6 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase62(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit6 == gsSustainH6) {
            intoPhase63();
            return;
        }
        if (gsLimit6 <= gsSustainH6) {
            gsSign6 = 1;
            gsDelta6= (gsSustainH6 - gsLimit6) / gsDecay[0];

        }
        else {
            gsSign6 = 0;
            gsDelta6 = (gsLimit6 - gsSustainH6) / gsDecay[0];
        }
        if (gsDelta6 == 0) gsDelta6 = 1;
        gsPhase6 = 2;
    }
    else  intoPhase63();
}


void intoPhase63(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice6();
        return;
    }
    if ((gsSustainH6) && (gsHiVal6 > 0)) {
        gsHiVal6 = gsSustainH6;
        gsValue6 = gsSustainH6 >> 7;
        gsDelta6 = 0;                           //gets terminated by NOTE OFF
        gsPhase6 = 3;
    }
    else  {
        gsHiVal6 = 0;
        gsPhase6 = 0;
    }
}


void shutdownVoice6X(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta6X = gsHiVal6X / SHORT_RELEASE;
    if (gsDelta6X == 0)  {
        gsHiVal6X = 0;
        tuneVoice6X(noteval, velocity);
        return;
    }
    gsNoteVal6 = noteval;
    gsVelocity6 = velocity;
    gsPhase6X = 5;
}


void releaseVoice6X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase6X) {
        if (gsRelease[1]) {
            gsDelta6X = gsHiVal6X / gsRelease[1]; //FINAL always = 0
            if (gsDelta6X == 0)  gsDelta6X = 1;
            gsPhase6X = 4;
        }
        else  gsOff6X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase6X2(void)  // ************************************************
//  ************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit6X == gsSustainH6X) {
            intoPhase6X3();
            return;
        }
        if (gsLimit6X <= gsSustainH6X) {
            gsSign6X = 1;
            gsDelta6X = (gsSustainH6X - gsLimit6X) / gsDecay[1];
        }
        else {
            gsSign6X = 0;
            gsDelta6X = (gsLimit6X - gsSustainH6X) / gsDecay[1];
        }
        if (gsDelta6X == 0) gsDelta6X = 1;
        gsPhase6X = 2;
    }
    else  intoPhase6X3();
}


void intoPhase6X3(void)  // ************************************************
//  ************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice6X();
        return;
    }
    if ((gsSustainH6X) && (gsHiVal6X > 0))  {
        gsHiVal6X = gsSustainH6X;
        gsValue6X = gsSustainH6X >> 7;
        gsDelta6X = 0;                           //gets terminated by NOTE OFF
        gsPhase6X = 3;
    }
    else  {
        gsHiVal6X = 0;
        gsPhase6X = 0;
    }
}


void tuneVoice7(uint32_t noteval, uint32_t velocity)   // *******************
// **************************************************************************
{
    gsPhase7 =0; //these settings as precaution,if no useable parameters
    gsValue7 = 0;
    gsOff7 = 0;
    gsWavPtr7 = 0.0;

    gsLimit7 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
    if (gsLimit7 > 0x7F000) gsLimit7 = 0x7F000; //theoretically max 0x71800
    if (gsLimit7 < 0) gsLimit7 = 0;

    if (gsLimit7 > 0x100)  {
        gsSustainH7 = (gsSustain[0] * gsLimit7) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal7 = gsSustainH7;
            gsValue7 = gsHiVal7 >> 7;
			intoPhase73();
		}
        else if (gsAttack[0])  {
            gsDelta7 = (gsLimit7 - gsHiVal7) / gsAttack[0];
            if (gsDelta7 == 0) gsDelta7 = 1;
            gsHiVal7 = gsDelta7;
            gsValue7 = gsHiVal7 >> 7;
            gsPhase7 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit7 >= gsSustainH7) {
                gsSign7 = 0;
                gsDelta7 = (gsLimit7 - gsSustainH7) / gsDecay[0];
            }
            else {
                gsSign7 = 1;
                gsDelta7 = (gsSustainH7 - gsLimit7) / gsDecay[0];
            }
            if (gsDelta7 == 0) gsDelta7 = 1 ;
            gsHiVal7 = gsLimit7;
            gsValue7 = gsLimit7 >> 7;
            gsPhase7 = 2;
        }
        else if (gsSustainH7)  {
            gsHiVal7 = gsSustainH7;
            gsValue7 = gsSustainH7 >> 7;
            gsDelta7 = 0;                       //gets terminated by NOTE OFF
            gsPhase7 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice7();
        }
        if (gsPhase7)  {
            gsStep7 = gsFloatStep[noteval+gsTRANS-36];
            gsRunStep7 = gsStep7 * gModCent1;
        }
    }
}


void tuneVoice7X(uint32_t noteval, uint32_t velocity)   // *******************
// **************************************************************************
{
    gsPhase7X =0; //these settings as precaution,if no useable parameters
    gsValue7X = 0;
    gsOff7X = 0;
    gsWavPtr7X = 0.0;

    gsLimit7X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit7X > 0x7F000) gsLimit7X = 0x7F000; //theoretically max 0x71800
    if (gsLimit7X < 0) gsLimit7X = 0;

    if (gsLimit7X > 0x100)  {
        gsSustainH7X = (gsSustain[1] * gsLimit7X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal7X = gsSustainH7X;
            gsValue7X = gsHiVal7X >> 7;
			intoPhase7X3();
		}
        else if (gsAttack[1])  {
            gsDelta7X = (gsLimit7X - gsHiVal7X) / gsAttack[1];
            if (gsDelta7X == 0) gsDelta7X = 1;
            gsHiVal7X = gsDelta7X;
            gsValue7X = gsHiVal7X >> 7;
            gsPhase7X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit7X >= gsSustainH7X) {
                gsSign7X = 0;
                gsDelta7X = (gsLimit7X - gsSustainH7X) / gsDecay[1];
            }
            else {
                gsSign7X = 1;
                gsDelta7X = (gsSustainH7X - gsLimit7X) / gsDecay[1];
            }
            if (gsDelta7X == 0) gsDelta7X = 1 ;
            gsHiVal7X = gsLimit7X;
            gsValue7X = gsLimit7X >> 7;
            gsPhase7X = 2;
        }
        else if (gsSustainH7X)  {
            gsHiVal7X = gsSustainH7X;
            gsValue7X = gsSustainH7X >> 7;
            gsDelta7X = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase7X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice7X();
        }
        if (gsPhase7X)  {
            gsStep7X =  gsFloatStep[noteval +gsTRANS +gsTRANS_2 -72];
            gsRunStep7X = gsStep7X * gModCent2;
        }
    }
}


void shutdownVoice7(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta7 = gsHiVal7 / SHORT_RELEASE;
    if (gsDelta7 == 0)  {
        gsHiVal7 = 0;
        tuneVoice7(noteval, velocity);
        return;
    }
    gsNoteVal7 = noteval;
    gsVelocity7 = velocity;
    gsPhase7 = 5;
}


void releaseVoice7(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase7) {
        if (gsRelease[0]) {
            gsDelta7 = gsHiVal7 / gsRelease[0]; //FINAL always = 0
            if (gsDelta7 == 0)  gsDelta7 = 1;
            gsPhase7 = 4;
        }
        else  gsOff7 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase72(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit7 == gsSustainH7) {
            intoPhase73();
            return;
        }
        if (gsLimit7 <= gsSustainH7) {
            gsSign7 = 1;
            gsDelta7= (gsSustainH7 - gsLimit7) / gsDecay[0];

        }
        else {
            gsSign7 = 0;
            gsDelta7 = (gsLimit7 - gsSustainH7) / gsDecay[0];
        }
        if (gsDelta7 == 0) gsDelta7 = 1;
        gsPhase7 = 2;
    }
    else  intoPhase73();
}


void intoPhase73(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice7();
        return;
    }
    if ((gsSustainH7) && (gsHiVal7 > 0)) {
        gsHiVal7 = gsSustainH7;
        gsValue7 = gsSustainH7 >> 7;
        gsDelta7 = 0;                           //gets terminated by NOTE OFF
        gsPhase7 = 3;
    }
    else  {
        gsHiVal7 = 0;
        gsPhase7 = 0;
    }
}


void shutdownVoice7X(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta7X = gsHiVal7X / SHORT_RELEASE;
    if (gsDelta7X == 0)  {
        gsHiVal7X = 0;
        tuneVoice7X(noteval, velocity);
        return;
    }
    gsNoteVal7 = noteval;
    gsVelocity7 = velocity;
    gsPhase7X = 5;
}


void releaseVoice7X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase7X) {
        if (gsRelease[1]) {
            gsDelta7X = gsHiVal7X / gsRelease[1]; //FINAL always = 0
            if (gsDelta7X == 0)  gsDelta7X = 1;
            gsPhase7X = 4;
        }
        else  gsOff7X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase7X2(void)  // ************************************************
//  ************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit7X == gsSustainH7X) {
            intoPhase7X3();
            return;
        }
        if (gsLimit7X <= gsSustainH7X) {
            gsSign7X = 1;
            gsDelta7X = (gsSustainH7X - gsLimit7X) / gsDecay[1];
        }
        else {
            gsSign7X = 0;
            gsDelta7X = (gsLimit7X - gsSustainH7X) / gsDecay[1];
        }
        if (gsDelta7X == 0) gsDelta7X = 1;
        gsPhase7X = 2;
    }
    else  intoPhase7X3();
}


void intoPhase7X3(void)  // ************************************************
//  ************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice7X();
        return;
    }
    if ((gsSustainH7X) && (gsHiVal7X > 0))  {
        gsHiVal7X = gsSustainH7X;
        gsValue7X = gsSustainH7X >> 7;
        gsDelta7X = 0;                           //gets terminated by NOTE OFF
        gsPhase7X = 3;
    }
     else  {
        gsHiVal7X = 0;
        gsPhase7X = 0;
    }
}


void tuneVoice8(uint32_t noteval, uint32_t velocity)   // *******************
// **************************************************************************
{
    gsPhase8 =0; //these settings as precaution,if no useable parameters
    gsValue8 = 0;
    gsOff8 = 0;
    gsWavPtr8 = 0.0;

    gsLimit8 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
    if (gsLimit8 > 0x7F000) gsLimit8 = 0x7F000; //theoretically max 0x71800
    if (gsLimit8 < 0) gsLimit8 = 0;

    if (gsLimit8 > 0x100)  {
        gsSustainH8 = (gsSustain[0] * gsLimit8) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal8 = gsSustainH8;
            gsValue8 = gsHiVal8 >> 7;
			intoPhase83();
		}
        else if (gsAttack[0])  {
            gsDelta8 = (gsLimit8 - gsHiVal8) / gsAttack[0];
            if (gsDelta8 == 0) gsDelta8 = 1;
            gsHiVal8 = gsDelta8;
            gsValue8 = gsHiVal8 >> 7;
            gsPhase8 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit8 >= gsSustainH8) {
                gsSign8 = 0;
                gsDelta8 = (gsLimit8 - gsSustainH8) / gsDecay[0];
            }
            else {
                gsSign8 = 1;
                gsDelta8 = (gsSustainH8 - gsLimit8) / gsDecay[0];
            }
            if (gsDelta8 == 0) gsDelta8 = 1 ;
            gsHiVal8 = gsLimit8;
            gsValue8 = gsLimit8 >> 7;
            gsPhase8 = 2;
        }
        else if (gsSustainH8)  {
            gsHiVal8 = gsSustainH8;
            gsValue8 = gsSustainH8 >> 7;
            gsDelta8 = 0;                       //gets terminated by NOTE OFF
            gsPhase8 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice8();
        }
        if (gsPhase8)  {
            gsStep8 = gsFloatStep[noteval+gsTRANS-36];
            gsRunStep8 = gsStep8 * gModCent1;
        }
    }
}


void tuneVoice8X(uint32_t noteval, uint32_t velocity)   // ******************
// **************************************************************************
{
    gsPhase8X =0; //these settings as precaution,if no useable parameters
    gsValue8X = 0;
    gsOff8X = 0;
    gsWavPtr8X = 0.0;

    gsLimit8X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit8X > 0x7F000) gsLimit8X = 0x7F000; //theoretically max 0x71800
    if (gsLimit8X < 0) gsLimit8X = 0;

    if (gsLimit8X > 0x100)  {
        gsSustainH8X = (gsSustain[1] * gsLimit8X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal8X = gsSustainH8X;
            gsValue8X = gsHiVal8X >> 7;
			intoPhase8X3();
		}
        else if (gsAttack[1])  {
            gsDelta8X = (gsLimit8X - gsHiVal8X) / gsAttack[1];
            if (gsDelta8X == 0) gsDelta8X = 1;
            gsHiVal8X = gsDelta8X;
            gsValue8X = gsHiVal8X >> 7;
            gsPhase8X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit8X >= gsSustainH8X) {
                gsSign8X = 0;
                gsDelta8X = (gsLimit8X - gsSustainH8X) / gsDecay[1];
            }
            else {
                gsSign8X = 1;
                gsDelta8X = (gsSustainH8X - gsLimit8X) / gsDecay[1];
            }
            if (gsDelta8X == 0) gsDelta8X = 1 ;
            gsHiVal8X = gsLimit8X;
            gsValue8X = gsLimit8X >> 7;
            gsPhase8X = 2;
        }
        else if (gsSustainH8X)  {
            gsHiVal8X = gsSustainH8X;
            gsValue8X = gsSustainH8X >> 7;
            gsDelta8X = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase8X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice8X();
        }
        if (gsPhase8X)   {
            gsStep8X = gsFloatStep[noteval +gsTRANS +gsTRANS_2 -72];
            gsRunStep8X = gsStep8X * gModCent2;
        }
    }
}


void shutdownVoice8(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta8 = gsHiVal8 / SHORT_RELEASE;
    if (gsDelta8 == 0)  {
        gsHiVal8 = 0;
        tuneVoice8(noteval, velocity);
        return;
    }
    gsNoteVal8 = noteval;
    gsVelocity8 = velocity;
    gsPhase8 = 5;
}


void releaseVoice8(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase8) {
        if (gsRelease[0]) {
            gsDelta8 = gsHiVal8 / gsRelease[0]; //FINAL always = 0
            if (gsDelta8 == 0)  gsDelta8 = 1;
            gsPhase8 = 4;
        }
        else  gsOff8 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase82(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit8 == gsSustainH8) {
            intoPhase83();
            return;
        }
        if (gsLimit8 <= gsSustainH8) {
            gsSign8 = 1;
            gsDelta8= (gsSustainH8 - gsLimit8) / gsDecay[0];
        }
        else {
            gsSign8 = 0;
            gsDelta8 = (gsLimit8 - gsSustainH8) / gsDecay[0];
        }
        if (gsDelta8 == 0) gsDelta8 = 1;
        gsPhase8 = 2;
    }
    else  intoPhase83();
}


void intoPhase83(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice8();
        return;
    }
    if ((gsSustainH8) && (gsHiVal8 > 0)) {
        gsHiVal8 = gsSustainH8;
        gsValue8 = gsSustainH8 >> 7;
        gsDelta8 = 0;                           //gets terminated by NOTE OFF
        gsPhase8 = 3;
    }
    else  {
        gsHiVal8 = 0;
        gsPhase8 = 0;
    }
}


void shutdownVoice8X(uint32_t noteval, uint32_t velocity)   // **************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta8X = gsHiVal8X / SHORT_RELEASE;
    if (gsDelta8X == 0)  {
        gsHiVal8X = 0;
        tuneVoice8(noteval, velocity);
        return;
    }
    gsNoteVal8 = noteval;
    gsVelocity8 = velocity;
    gsPhase8X = 5;
}


void releaseVoice8X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase8X) {
        if (gsRelease[1]) {
            gsDelta8X = gsHiVal8X / gsRelease[1]; //FINAL always = 0
            if (gsDelta8X == 0)  gsDelta8X = 1;
            gsPhase8X = 4;
        }
        else  gsOff8X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase8X2(void)  // ************************************************
//  ************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit8X == gsSustainH8X) {
            intoPhase8X3();
            return;
        }
        if (gsLimit8X <= gsSustainH8X) {
            gsSign8X = 1;
            gsDelta8X = (gsSustainH8X - gsLimit8X) / gsDecay[1];
        }
        else {
            gsSign8X = 0;
            gsDelta8X = (gsLimit8X - gsSustainH8X) / gsDecay[1];
        }
        if (gsDelta8X == 0) gsDelta8X = 1;
        gsPhase8X = 2;
    }
    else  intoPhase8X3();
}


void intoPhase8X3(void)  // ************************************************
//  ************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice8X();
        return;
    }
    if ((gsSustainH8X) && (gsHiVal8X > 0))  {
        gsHiVal8X = gsSustainH8X;
        gsValue8X = gsSustainH8X >> 7;
        gsDelta8X = 0;                           //gets terminated by NOTE OFF
        gsPhase8X = 3;
    }
     else  {
        gsHiVal8X = 0;
        gsPhase8X = 0;
    }
}


void tuneVoice9(uint32_t noteval, uint32_t velocity)   // *******************
// **************************************************************************
{
    gsPhase9 =0; //these settings as precaution,if no useable parameters
    gsValue9 = 0;
    gsOff9 = 0;
    gsWavPtr9 = 0.0;

    gsLimit9 = (gsFixVol[0] * 2048) + (16*velocity *((int32_t)(gsVelo[0] -28)));
    if (gsLimit9 > 0x7F000) gsLimit9 = 0x7F000; //theoretically max 0x71800
    if (gsLimit9 < 0) gsLimit9 = 0;

    if (gsLimit9 > 0x100)  {
        gsSustainH9 = (gsSustain[0] * gsLimit9) / 100;

		if ((gsAttack[0] == 0) && (gsDecay[0] == 0)) {
			gsHiVal9 = gsSustainH9;
            gsValue9 = gsHiVal9 >> 7;
			intoPhase93();
		}
        else if (gsAttack[0])  {
            gsDelta9 = (gsLimit9 - gsHiVal9) / gsAttack[0];
            if (gsDelta9 == 0) gsDelta9 = 1;
            gsHiVal9 = gsDelta9;
            gsValue9 = gsHiVal9 >> 7;
            gsPhase9 = 1;
        }
        else if (gsDecay[0])  {
            if (gsLimit9 >= gsSustainH9) {
                gsSign9 = 0;
                gsDelta9 = (gsLimit9 - gsSustainH9) / gsDecay[0];
            }
            else {
                gsSign9 = 1;
                gsDelta9 = (gsSustainH9 - gsLimit9) / gsDecay[0];
            }
            if (gsDelta9 == 0) gsDelta9 = 1 ;
            gsHiVal9 = gsLimit9;
            gsValue9 = gsLimit9 >> 7;
            gsPhase9 = 2;
        }
        else if (gsSustainH9)  {
            gsHiVal9 = gsSustainH9;
            gsValue9 = gsSustainH9 >> 7;
            gsDelta9 = 0;                       //gets terminated by NOTE OFF
            gsPhase9 = 3;
            if (gsHoldSust[0] == 0)  releaseVoice9();
        }
        if (gsPhase9)  {
            gsStep9 = gsFloatStep[noteval+gsTRANS-36];
            gsRunStep9 = gsStep9 * gModCent1;
        }
    }
}


void tuneVoice9X(uint32_t noteval, uint32_t velocity)   // ******************
// **************************************************************************
{
    gsPhase9X =0; //these settings as precaution,if no useable parameters
    gsValue9X = 0;
    gsOff9X = 0;
    gsWavPtr9X = 0.0;

    gsLimit9X = (gsFixVol[1] * 2048) + (16*velocity *((int32_t)(gsVelo[1]-28)));
    if (gsLimit9X > 0x7F000) gsLimit9X = 0x7F000; //theoretically max 0x71800
    if (gsLimit9X < 0) gsLimit9X = 0;

    if (gsLimit9X > 0x100)  {
        gsSustainH9X = (gsSustain[1] * gsLimit9X) / 100;

		if ((gsAttack[1] == 0) && (gsDecay[1] == 0)) {
			gsHiVal9X = gsSustainH9X;
            gsValue9X = gsHiVal9X >> 7;
			intoPhase9X3();
		}
        else if (gsAttack[1])  {
            gsDelta9X = (gsLimit9X - gsHiVal9X) / gsAttack[1];
            if (gsDelta9X == 0) gsDelta9X = 1;
            gsHiVal9X = gsDelta9X;
            gsValue9X = gsHiVal9X >> 7;
            gsPhase9X = 1;
        }
        else if (gsDecay[1])  {
            if (gsLimit9X >= gsSustainH9X) {
                gsSign9X = 0;
                gsDelta9X = (gsLimit9X - gsSustainH9X) / gsDecay[1];
            }
            else {
                gsSign9X = 1;
                gsDelta9X = (gsSustainH9X - gsLimit9X) / gsDecay[1];
            }
            if (gsDelta9X == 0) gsDelta9X = 1 ;
            gsHiVal9X = gsLimit9X;
            gsValue9X = gsLimit9X >> 7;
            gsPhase9X = 2;
        }
        else if (gsSustainH9X)  {
            gsHiVal9X = gsSustainH9X;
            gsValue9X = gsSustainH9X >> 7;
            gsDelta9X = 0;           //no Countdown, gets terminated by NOTE OFF
            gsPhase9X = 3;
            if (gsHoldSust[1] == 0)  releaseVoice9X();
        }
        if (gsPhase9X)  {
            gsStep9X = gsFloatStep[noteval +gsTRANS +gsTRANS_2 -72];
            gsRunStep9X = gsStep9X * gModCent2;
        }
    }
}


void shutdownVoice9(uint32_t noteval, uint32_t velocity)   // ***************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta9 = gsHiVal9 / SHORT_RELEASE;
    if (gsDelta9 == 0)  {
        gsHiVal9 = 0;
        tuneVoice9(noteval, velocity);
        return;
    }
    gsNoteVal9 = noteval;
    gsVelocity9 = velocity;
    gsPhase9 = 5;
}


void releaseVoice9(void)   // ***********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase9) {
        if (gsRelease[0]) {
            gsDelta9 = gsHiVal9 / gsRelease[0]; //FINAL always = 0
            if (gsDelta9 == 0)  gsDelta9 = 1;
            gsPhase9 = 4;
        }
        else  gsOff9 = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase92(void)  // *************************************************
//  ************************************************************************
{
    if (gsDecay[0])  {
        if (gsLimit9 == gsSustainH9) {
            intoPhase93();
            return;
        }
        if (gsLimit9 <= gsSustainH9) {
            gsSign9 = 1;
            gsDelta9= (gsSustainH9 - gsLimit9) / gsDecay[0];
        }
        else {
            gsSign9 = 0;
            gsDelta9 = (gsLimit9 - gsSustainH9) / gsDecay[0];
        }
        if (gsDelta9 == 0) gsDelta9 = 1;
        gsPhase9 = 2;
    }
    else  intoPhase93();
}


void intoPhase93(void)  // *************************************************
//  ************************************************************************
{
    if (gsHoldSust[0] == 0)  {
        releaseVoice9();
        return;
    }
    if ((gsSustainH9) && (gsHiVal9 > 0)) {
        gsHiVal9 = gsSustainH9;
        gsValue9 = gsSustainH9 >> 7;
        gsDelta9 = 0;                           //gets terminated by NOTE OFF
        gsPhase9 = 3;
    }
    else  {
        gsHiVal9 = 0;
        gsPhase9 = 0;
    }
}


void shutdownVoice9X(uint32_t noteval, uint32_t velocity)   // **************
//introduce short Release Phase to avoid click noise
// **************************************************************************
{
    gsDelta9X = gsHiVal9X / SHORT_RELEASE;
    if (gsDelta9X == 0)  {
        gsHiVal9X = 0;
        tuneVoice9X(noteval, velocity);
        return;
    }
    gsNoteVal9 = noteval;
    gsVelocity9 = velocity;
    gsPhase9X = 5;
}


void releaseVoice9X(void)   // **********************************************
// start Osc Release phase
// **************************************************************************
{
    if (gsPhase9X) {
        if (gsRelease[1]) {
            gsDelta9X = gsHiVal9X / gsRelease[1]; //FINAL always = 0
            if (gsDelta9X == 0)  gsDelta9X = 1;
            gsPhase9X = 4;
        }
        else  gsOff9X = 1; //dont switch off immediately/zero transition trigger
    }
}


void intoPhase9X2(void)  // ************************************************
//  ************************************************************************
{
    if (gsDecay[1])  {
        if (gsLimit9X == gsSustainH9X) {
            intoPhase9X3();
            return;
        }
        if (gsLimit9X <= gsSustainH9X) {
            gsSign9X = 1;
            gsDelta9X = (gsSustainH9X - gsLimit9X) / gsDecay[1];
        }
        else {
            gsSign9X = 0;
            gsDelta9X = (gsLimit9X - gsSustainH9X) / gsDecay[1];
        }
        if (gsDelta9X == 0) gsDelta9X = 1;
        gsPhase9X = 2;
    }
    else  intoPhase9X3();
}


void intoPhase9X3(void)  // ************************************************
//  ************************************************************************
{
    if (gsHoldSust[1] == 0)  {
        releaseVoice9X();
        return;
    }
    if ((gsSustainH9X) && (gsHiVal9X > 0))  {
        gsHiVal9X = gsSustainH9X;
        gsValue9X = gsSustainH9X >> 7;
        gsDelta9X = 0;                           //gets terminated by NOTE OFF
        gsPhase9X = 3;
    }
    else  {
        gsHiVal9X = 0;
        gsPhase9X = 0;
    }
}

// ############################################################################

void triggerFilter0(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// ADSR9 controls MidFreq/Cutoff of Filter#0  and its clones
// **************************************************************************
{

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftF0 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftF0 = 0.0;
		gsBaseFreqF0=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftF0));
								//triggered by note, independently from ADSR
								//gsBaseFreqF0 remains at end of note
								//until next note due to release of note


        gsValueF0M = 0.0;
        gsPhaseF0M = 0; //these settings as precaution,if no useable parameters


		if (gsMonoPoly)  {
			gTIMEOUT = 2;
			while (gTIMEOUT) servSystem();  //short delay for re-tune filter
		}

        if (mode == 0)  {
			tempInitFilter(0,gsBaseFreqF0);
			return;
        }

		gsLimitF0M = ((int32_t)(gsFixVol[9] - 64) * 2048) +  \
									(32*velocity * ((int32_t)(gsVelo[9] - 28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityF0M = 0;
		if (gsLimitF0M < 0)  {
			gsLimitF0M = -gsLimitF0M;
			gsPolarityF0M = 1;
		}
		if (gsLimitF0M > 0x7F000) gsLimitF0M=0x7F000;//theoretically max 0x71800
		if (gsLimitF0M < 0) gsLimitF0M = 0;

		if (gsLimitF0M > 0x100) {

			gsSustainHF0M = (gsSustain[9] * gsLimitF0M) / 100;

			if (gsAttack[9])  {
				gsDeltaF0M = (gsLimitF0M) / gsAttack[9];
				if (gsDeltaF0M == 0) gsDeltaF0M = 1;
				gsHiValF0M = gsDeltaF0M;
				if (gsPolarityF0M== 0) gsValueF0M = (float)(gsHiValF0M) / 3200.;
				else  gsValueF0M = (float)(-gsHiValF0M) / 6400.;
				gsPhaseF0M = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitF0M >= gsSustainHF0M) {
					gsSignF0M = 0;
					gsDeltaF0M = (gsLimitF0M - gsSustainHF0M) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignF0M = 1;
					gsDeltaF0M = (gsSustainHF0M - gsLimitF0M) / gsDecay[9];
				}
				if (gsDeltaF0M == 0) gsDeltaF0M = 1 ;
				gsHiValF0M = gsLimitF0M;
				if (gsPolarityF0M== 0) gsValueF0M = (float)(gsHiValF0M) / 3200.;
				else  gsValueF0M = (float)(-gsHiValF0M) / 6400.;
				gsPhaseF0M = 2;
			}
			else if (gsSustainHF0M)  {
				gsHiValF0M = gsSustainHF0M;
				if (gsPolarityF0M == 0) gsValueF0M = (float)(gsHiValF0M) / 3200.;
				else  gsValueF0M = (float)(-gsHiValF0M) / 6400.;
				gsDeltaF0M = 0;       //no Countdown,gets terminated by NOTE OFF
				gsPhaseF0M = 3;
				if (gsHoldSust[9] == 0)  releaseFilter0();
			}
			tempInitFilter(0,(uint32_t)(gsBaseFreqF0 * (32 + gsValueF0M) / 32));
		}
		else tempInitFilter(0,gsBaseFreqF0);     //keytrack independent of ADSR
	}
}

// -----------------------------------

void triggerFree(uint32_t velocity)  // *************************************
// ADSR[gsFreeIdx] controls freely selectable features
// (gsFreeIdx = N_ABASIC + gsFreeAdsr)
// redesign in firmware REVNUM 9:
// Free ADSR's are addressed in a linear array after standard ADSR's
// **************************************************************************
{
	gsFreeIdx = N_ABASIC + gsFreeAdsr;	//reconfirm
    gsValueFree = 0;
	gsValueFMO2 = 0;	//specialized subtype, will be used if gsFreeAdsr = 0
	gsValueFLB = 0;		//specialized subtype, will be used if gsFreeAdsr = 1
	gsValueF0QF = 0.0;	//specialized subtype, will be used if gsFreeAdsr = 2
    gsPhaseF0QF = 0;
    gsValueF2M = 0.0;	//specialized subtype, will be used if gsFreeAdsr = 3
	gsValueChor = 0;	//specialized subtype, will be used if gsFreeAdsr = 4
    gsPhaseFree = 0; 	//these settings as precaution,if no useable parameters


    if ((gsFreeAdsr == 2) && gsNoTriggerF0)  return;
    if ((gsFreeAdsr == 3) && ((gfRuntimeMask & 0x00000004) == 0))  return;

    gsPolarityFree = 0;
	gsLimitFree = ((int32_t)(gsFixVol[gsFreeIdx] - 64) * 2048) +
							(32 * velocity * ((int32_t)(gsVelo[gsFreeIdx]-28)));
		//stretched for better resolution of high Attack,Decay,Release values
	if (gsLimitFree < 0)  {
		gsLimitFree = -gsLimitFree;
		gsPolarityFree = 1;
	}
    if (gsLimitFree > 0x7F000) gsLimitFree = 0x7F000;//theoretically max 0x71800

    if (gsLimitFree > 0x100)  {
		gsSustainHFree = (gsSustain[gsFreeIdx] * gsLimitFree) / 100;

        sendB('#');
        sendS32(gsFreeAdsr);
        sendS32(gsAttack[gsFreeIdx]);
        sendS32(gsLimitFree);

        if (gsAttack[gsFreeIdx])  {
            gsDeltaFree = (gsLimitFree) / gsAttack[gsFreeIdx];
            if (gsDeltaFree == 0) gsDeltaFree = 1;
            gsHiValFree = gsDeltaFree;
			if (gsPolarityFree == 0) gsValueFree = gsHiValFree >> 6;
			else  gsValueFree = -(gsHiValFree >> 6);		//restore neg.sign

        sendS32(gsDeltaFree);
        sendS32(gsHiValFree);
        sendS32(gsValueFree);
        sendB('/');

            gsPhaseFree = 1;
        }
        else if (gsDecay[gsFreeIdx])  {
            if (gsLimitFree >= gsSustainHFree) {
                gsSignFree = 0;
                gsDeltaFree = (gsLimitFree-gsSustainHFree) / gsDecay[gsFreeIdx];
            }
            else {          //should be impossible, introduced as precaution
                gsSignFree = 1;
                gsDeltaFree = (gsSustainHFree-gsLimitFree) / gsDecay[gsFreeIdx];
            }
            if (gsDeltaFree == 0) gsDeltaFree = 1 ;
            gsHiValFree = gsLimitFree;
			if (gsPolarityFree == 0) gsValueFree = gsHiValFree >> 6;
			else  gsValueFree = -(gsHiValFree >> 6);		//restore neg.sign
            gsPhaseFree = 2;
        }
        else if (gsSustainHFree)  {
            gsHiValFree = gsSustainHFree;
			if (gsPolarityFree == 0) gsValueFree = gsHiValFree >> 6;
			else  gsValueFree = -(gsHiValFree >> 6);		//restore neg.sign
            gsDeltaFree = 0;       //no Countdown, gets terminated by NOTE OFF
            gsPhaseFree = 3;
            if (gsHoldSust[gsFreeIdx] == 0)  releaseFree();
        }
        if (gsFreeAdsr == 0)  gsValueFMO2 = gsValueFree;	//Osc2 additional FM
        if (gsFreeAdsr == 1)  gsValueFLB = gsValueFree;		//Lfo2 Freq.Mod
        if (gsFreeAdsr == 2)  {
			gsPhaseF0QF = gsPhaseFree;
			if ((gfACTIV_IIR[0].TYPE == 5) || (gfACTIV_IIR[0].TYPE == 6))  {
				if (gsPolarityFree == 0) gsValueF0QF = (float)(gsHiValFree) /2;
				else  gsValueF0QF = (float)(-gsHiValFree) / 2;
			}
			else  {
				if (gsPolarityFree==0) gsValueF0QF = (float)(gsHiValFree)/8192.;
				else  gsValueF0QF= (float)(-gsHiValFree) / 8192.;
			}
			gfF0TempQF = (32*gfRuntimeIIR[0].QFMOD + gsValueF0QF) / 32;
        }
        if (gsFreeAdsr == 3)  {
			if (gsPolarityFree == 0) gsValueF2M = (float)(gsHiValFree) / 3200.;
			else  gsValueF2M = (float)(-gsHiValFree) / 6400.;
			tempInitFilter(2,(uint32_t)(gfBaseFreqF2* (32.0+ gsValueF2M) / 32));
                                        //gfBaseFreqF3 is calculated in filter.c
        }
        if (gsFreeAdsr == 4) gsValueChor = gsValueFree;		//Chorus
    }
    else  {
		if (gsFreeAdsr == 2)  gfF0TempQF = gfRuntimeIIR[0].QFMOD;
		if (gsFreeAdsr == 3)  tempInitFilter(2,gfBaseFreqF2);
    }
}


void triggerFilterC1(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// **************************************************************************
{
// ADSRC1 controls Cutoff of Filter#5 (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC1 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC1 = 0.0;
		gsBaseFreqC1 = (uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC1));
									//triggered by note, independently of ADSR

        gsValueC1 = 0.0;
        gsPhaseC1 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(5,gsBaseFreqC1);
			return;
        }

		gsLimitC1 = ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC1 = 0;
		if (gsLimitC1 < 0)  {
			gsLimitC1 = -gsLimitC1;
			gsPolarityC1 = 1;
		}
		if (gsLimitC1 > 0x7F000) gsLimitC1 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC1 < 0) gsLimitC1 = 0;

		if (gsLimitC1 > 0x100)  {
			gsSustainHC1 = (gsSustain[9] * gsLimitC1) / 100;

			if (gsAttack[9])  {
				gsDeltaC1 = (gsLimitC1) / gsAttack[9];
				if (gsDeltaC1 == 0) gsDeltaC1 = 1;
				gsHiValC1 = gsDeltaC1;
				if (gsPolarityC1 == 0) gsValueC1 = (float)(gsHiValC1) / 3200.;
				else  gsValueC1 = (float)(-gsHiValC1) / 6400.;
				gsPhaseC1 = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitC1 >= gsSustainHC1) {
					gsSignC1 = 0;
					gsDeltaC1 = (gsLimitC1 - gsSustainHC1) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC1 = 1;
					gsDeltaC1 = (gsSustainHC1 - gsLimitC1) / gsDecay[9];
				}
				if (gsDeltaC1 == 0) gsDeltaC1 = 1 ;
				gsHiValC1 = gsLimitC1;
				if (gsPolarityC1 == 0) gsValueC1 = (float)(gsHiValC1) / 3200.;
				else  gsValueC1 = (float)(-gsHiValC1) / 6400.;
				gsPhaseC1 = 2;
			}
			else if (gsSustainHC1)  {
				gsHiValC1 = gsSustainHC1;
				if (gsPolarityC1 == 0) gsValueC1 = (float)(gsHiValC1) / 3200.;
				else  gsValueC1 = (float)(-gsHiValC1) / 6400.;
				gsDeltaC1 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC1 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC1();
			}
			tempInitFilter(5,(uint32_t)(gsBaseFreqC1 * (32 + gsValueC1) / 32));
		}
		else tempInitFilter(5,gsBaseFreqC1);     //keytrack independent of ADSR
    }
}

void triggerFilterC2(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// **************************************************************************
{
// ADSRC2 controls Cutoff of Filter#6  (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC2 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC2 = 0.0;
		gsBaseFreqC2 = (uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC2));
									//triggered by note, independently of ADSR

        gsValueC2 = 0.0;
        gsPhaseC2 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(6,gsBaseFreqC2);
			return;
        }

		gsLimitC2 = ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC2 = 0;
		if (gsLimitC2 < 0)  {
			gsLimitC2 = -gsLimitC2;
			gsPolarityC2 = 1;
		}
		if (gsLimitC2 > 0x7F000) gsLimitC2 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC2 < 0) gsLimitC2 = 0;

		if (gsLimitC2 > 0x100)  {

			gsSustainHC2 = (gsSustain[9] * gsLimitC2) / 100;

			if (gsAttack[9])  {
				gsDeltaC2 = (gsLimitC2) / gsAttack[9];
				if (gsDeltaC2 == 0) gsDeltaC2 = 1;
				gsHiValC2 = gsDeltaC2;
				if (gsPolarityC2 == 0) gsValueC2 = (float)(gsHiValC2) / 3200.;
				else  gsValueC2 = (float)(-gsHiValC2) / 6400.;
			gsPhaseC2 = 1;
				}
			else if (gsDecay[9])  {
				if (gsLimitC2 >= gsSustainHC2) {
					gsSignC2 = 0;
					gsDeltaC2 = (gsLimitC2 - gsSustainHC2) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC2 = 1;
					gsDeltaC2 = (gsSustainHC2 - gsLimitC2) / gsDecay[9];
				}
				if (gsDeltaC2 == 0) gsDeltaC2 = 1 ;
				gsHiValC2 = gsLimitC2;
				if (gsPolarityC2 == 0) gsValueC2 = (float)(gsHiValC2) / 3200.;
				else  gsValueC2 = (float)(-gsHiValC2) / 6400.;
				gsPhaseC2 = 2;
			}
			else if (gsSustainHC2)  {
				gsHiValC2 = gsSustainHC2;
				if (gsPolarityC2 == 0) gsValueC2 = (float)(gsHiValC2) / 3200.;
				else  gsValueC2 = (float)(-gsHiValC2) / 6400.;
				gsDeltaC2 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC2 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC2();
			}
			tempInitFilter(6,(uint32_t)(gsBaseFreqC2 * (32 + gsValueC2) / 32));
		}
		else tempInitFilter(6,gsBaseFreqC2);     //keytrack independent of ADSR
    }
}


void triggerFilterC3(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// **************************************************************************
{
// ADSRC3 controls cutoff of Filter#7 (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC3 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC3 = 0.0;
		gsBaseFreqC3=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC3));
									//triggered by note, independently of ADSR

        gsValueC3 = 0.0;
        gsPhaseC3 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(7,gsBaseFreqC3);
			return;
        }

		gsLimitC3 = ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC3 = 0;
		if (gsLimitC3 < 0)  {
			gsLimitC3 = -gsLimitC3;
			gsPolarityC3 = 1;
		}
		if (gsLimitC3 > 0x7F000) gsLimitC3 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC3 < 0) gsLimitC3 = 0;

		if (gsLimitC3 > 0x100)  {

			gsSustainHC3 = (gsSustain[9] * gsLimitC3) / 100;

			if (gsAttack[9])  {
				gsDeltaC3 = (gsLimitC3) / gsAttack[9];
				if (gsDeltaC3 == 0) gsDeltaC3 = 1;
				gsHiValC3 = gsDeltaC3;
				if (gsPolarityC3 == 0) gsValueC3 = (float)(gsHiValC3) / 3200.;
				else  gsValueC3 = (float)(-gsHiValC3) / 6400.;
				gsPhaseC3 = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitC3 >= gsSustainHC3) {
					gsSignC3 = 0;
					gsDeltaC3 = (gsLimitC3 - gsSustainHC3) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC3 = 1;
					gsDeltaC3 = (gsSustainHC3 - gsLimitC3) / gsDecay[9];
				}
				if (gsDeltaC3 == 0) gsDeltaC3 = 1 ;
				gsHiValC3 = gsLimitC3;
				if (gsPolarityC3 == 0) gsValueC3 = (float)(gsHiValC3) / 3200.;
				else  gsValueC3 = (float)(-gsHiValC3) / 6400.;
				gsPhaseC3 = 2;
			}
			else if (gsSustainHC3)  {
				gsHiValC3 = gsSustainHC3;
				if (gsPolarityC3 == 0) gsValueC3 = (float)(gsHiValC3) / 3200.;
				else  gsValueC3 = (float)(-gsHiValC3) / 6400.;
				gsDeltaC3 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC3 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC3();
			}
			tempInitFilter(7,(uint32_t)(gsBaseFreqC3 * (32 + gsValueC3) / 32));
		}
		else tempInitFilter(7,gsBaseFreqC3);     //keytrack independent of ADSR
    }
}


void triggerFilterC4(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// **************************************************************************
{
// ADSRC4 controls cutoff of Filter#8  (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC4 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC4 = 0.0;
		gsBaseFreqC4=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC4));
									//triggered by note, independently of ADSR

        gsValueC4 = 0.0;
        gsPhaseC4 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(8,gsBaseFreqC4);
			return;
        }

		gsLimitC4 = ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC4 = 0;
		if (gsLimitC4 < 0)  {
			gsLimitC4 = -gsLimitC4;
			gsPolarityC4 = 1;
		}
		if (gsLimitC4 > 0x7F000) gsLimitC4 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC4 < 0) gsLimitC4 = 0;

		if (gsLimitC4 > 0x100)  {

			gsSustainHC4 = (gsSustain[9] * gsLimitC4) / 100;

			if (gsAttack[9])  {
				gsDeltaC4 = (gsLimitC4) / gsAttack[9];
				if (gsDeltaC4 == 0) gsDeltaC4 = 1;
				gsHiValC4 = gsDeltaC4;
				if (gsPolarityC4 == 0) gsValueC4 = (float)(gsHiValC4) / 3200.;
				else  gsValueC4 = (float)(-gsHiValC4) / 6400.;
				gsPhaseC4 = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitC4 >= gsSustainHC4) {
					gsSignC4 = 0;
					gsDeltaC4 = (gsLimitC4 - gsSustainHC4) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC4 = 1;
					gsDeltaC4 = (gsSustainHC4 - gsLimitC4) / gsDecay[9];
				}
				if (gsDeltaC4 == 0) gsDeltaC4 = 1 ;
				gsHiValC4 = gsLimitC4;
				if (gsPolarityC4 == 0) gsValueC4 = (float)(gsHiValC4) / 3200.;
				else  gsValueC4 = (float)(-gsHiValC4) / 6400.;
				gsPhaseC4 = 2;
			}
			else if (gsSustainHC4)  {
				gsHiValC4 = gsSustainHC4;
				if (gsPolarityC4 == 0) gsValueC4 = (float)(gsHiValC4) / 3200.;
				else  gsValueC4 = (float)(-gsHiValC4) / 6400.;
				gsDeltaC4 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC4 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC4();
			}
			tempInitFilter(8,(uint32_t)(gsBaseFreqC4 * (32 + gsValueC4) / 32));
		}
		else tempInitFilter(8,gsBaseFreqC4);     //keytrack independent of ADSR
    }
}


void triggerFilterC5(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// **************************************************************************
{
// ADSRC5 controls cutoff of Filter#9  (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC5 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC5 = 0.0;
		gsBaseFreqC5=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC5));
									//triggered by note, independently of ADSR

        gsValueC5 = 0.0;
        gsPhaseC5 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(9,gsBaseFreqC5);
			return;
        }

		gsLimitC5 = ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC5 = 0;
		if (gsLimitC5 < 0)  {
			gsLimitC5 = -gsLimitC5;
			gsPolarityC5 = 1;
		}
		if (gsLimitC5 > 0x7F000) gsLimitC5 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC5 < 0) gsLimitC5 = 0;

		if (gsLimitC5 > 0x100)  {

			gsSustainHC5 = (gsSustain[9] * gsLimitC5) / 100;

			if (gsAttack[9])  {
				gsDeltaC5 = (gsLimitC5) / gsAttack[9];
				if (gsDeltaC5 == 0) gsDeltaC5 = 1;
				gsHiValC5 = gsDeltaC5;
				if (gsPolarityC5 == 0) gsValueC5 = (float)(gsHiValC5) / 3200.;
				else  gsValueC5 = (float)(-gsHiValC5) / 6400.;
				gsPhaseC5 = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitC5 >= gsSustainHC5) {
					gsSignC5 = 0;
					gsDeltaC5 = (gsLimitC5 - gsSustainHC5) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC5 = 1;
					gsDeltaC5 = (gsSustainHC5 - gsLimitC5) / gsDecay[9];
				}
				if (gsDeltaC5 == 0) gsDeltaC5 = 1 ;
				gsHiValC5 = gsLimitC5;
				if (gsPolarityC5 == 0) gsValueC5 = (float)(gsHiValC5) / 3200.;
				else  gsValueC5 = (float)(-gsHiValC5) / 6400.;
				gsPhaseC5 = 2;
			}
			else if (gsSustainHC5)  {
				gsHiValC5 = gsSustainHC5;
				if (gsPolarityC5 == 0) gsValueC5 = (float)(gsHiValC5) / 3200.;
				else  gsValueC5 = (float)(-gsHiValC5) / 6400.;
				gsDeltaC5 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC5 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC5();
			}
			tempInitFilter(9,(uint32_t)(gsBaseFreqC5 * (32 + gsValueC5) / 32));
		}
		else tempInitFilter(9,gsBaseFreqC5);     //keytrack independent of ADSR
    }
}

void triggerFilterC6(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// ***************************************************************************
{
// ADSRC6 controls Cutoff of Filter#10 (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC6 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC6 = 0.0;
		gsBaseFreqC6=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC6));
									//triggered by note, independently of ADSR

        gsValueC6 = 0.0;
        gsPhaseC6 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(10,gsBaseFreqC6);
			return;
        }

		gsLimitC6 = ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC6 = 0;
		if (gsLimitC6 < 0)  {
			gsLimitC6 = -gsLimitC6;
			gsPolarityC6 = 1;
		}
		if (gsLimitC6 > 0x7F000) gsLimitC6 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC6 < 0) gsLimitC6 = 0;

		if (gsLimitC6 > 0x100)  {

			gsSustainHC6 = (gsSustain[9] * gsLimitC6) / 100;

			if (gsAttack[9])  {
				gsDeltaC6 = (gsLimitC6) / gsAttack[9];
				if (gsDeltaC6 == 0) gsDeltaC6 = 1;
				gsHiValC6 = gsDeltaC6;
				if (gsPolarityC6 == 0) gsValueC6 = (float)(gsHiValC6) / 3200.;
				else  gsValueC6 = (float)(-gsHiValC6) / 6400.;
				gsPhaseC6 = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitC6 >= gsSustainHC6) {
					gsSignC6 = 0;
					gsDeltaC6 = (gsLimitC6 - gsSustainHC6) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC6 = 1;
					gsDeltaC6 = (gsSustainHC6 - gsLimitC6) / gsDecay[9];
				}
				if (gsDeltaC6 == 0) gsDeltaC6 = 1 ;
				gsHiValC6 = gsLimitC6;
				if (gsPolarityC6 == 0) gsValueC6 = (float)(gsHiValC6) / 3200.;
				else  gsValueC6 = (float)(-gsHiValC6) / 6400.;
				gsPhaseC6 = 2;
			}
			else if (gsSustainHC6)  {
				gsHiValC6 = gsSustainHC6;
				if (gsPolarityC6 == 0) gsValueC6 = (float)(gsHiValC6) / 3200.;
				else  gsValueC6 = (float)(-gsHiValC6) / 6400.;
				gsDeltaC6 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC6 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC6();

			}
			tempInitFilter(10,(uint32_t)(gsBaseFreqC6 * (32 + gsValueC6) / 32));
		}
		else tempInitFilter(10,gsBaseFreqC6);     //keytrack independent of ADSR
    }
}


void triggerFilterC7(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// ***************************************************************************
{
// ADSRC7 controls Cutoff of Filter#11  (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC7 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC7 = 0.0;
		gsBaseFreqC7=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC7));
									//triggered by note, independently of ADSR

        gsValueC7 = 0.0;
        gsPhaseC7 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(11,gsBaseFreqC7);
			return;
        }

		gsLimitC7 =  ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC7 = 0;
		if (gsLimitC7 < 0)  {
			gsLimitC7 = -gsLimitC7;
			gsPolarityC7 = 1;
		}
		if (gsLimitC7 > 0x7F000) gsLimitC7 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC7 < 0) gsLimitC7 = 0;

		if (gsLimitC7 > 0x100)  {

			gsSustainHC7 = (gsSustain[9] * gsLimitC7) / 100;

			if (gsAttack[9])  {
				gsDeltaC7 = (gsLimitC7) / gsAttack[9];
				if (gsDeltaC7 == 0) gsDeltaC7 = 1;
				gsHiValC7 = gsDeltaC7;
				if (gsPolarityC7 == 0) gsValueC7 = (float)(gsHiValC7) / 3200.;
				else  gsValueC7 = (float)(-gsHiValC7) / 6400.;
				gsPhaseC7 = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitC7 >= gsSustainHC7) {
					gsSignC7 = 0;
					gsDeltaC7 = (gsLimitC7 - gsSustainHC7) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC7 = 1;
					gsDeltaC7 = (gsSustainHC7 - gsLimitC7) / gsDecay[9];
				}
				if (gsDeltaC7 == 0) gsDeltaC7 = 1 ;
				gsHiValC7 = gsLimitC7;
				if (gsPolarityC7 == 0) gsValueC7 = (float)(gsHiValC7) / 3200.;
				else  gsValueC7 = (float)(-gsHiValC7) / 6400.;
				gsPhaseC7 = 2;
			}
			else if (gsSustainHC7)  {
				gsHiValC7 = gsSustainHC7;
				if (gsPolarityC7 == 0) gsValueC7 = (float)(gsHiValC7) / 3200.;
				else  gsValueC7 = (float)(-gsHiValC7) / 6400.;
				gsDeltaC7 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC7 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC7();
			}
			tempInitFilter(11,(uint32_t)(gsBaseFreqC7 * (32 + gsValueC7) / 32));
		}
		else tempInitFilter(11,gsBaseFreqC7);   //keytrack independent of ADSR
    }
}


void triggerFilterC8(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// **************************************************************************
{
// ADSRC8 controls Cutoff of Filter#12 (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC8 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC8 = 0.0;
		gsBaseFreqC8=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC8));
									//triggered by note, independently of ADSR

        gsValueC8 = 0.0;
        gsPhaseC8 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(12,gsBaseFreqC8);
			return;
        }

		gsLimitC8 = ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC8 = 0;
		if (gsLimitC8 < 0)  {
			gsLimitC8 = -gsLimitC8;
			gsPolarityC8 = 1;
		}
		if (gsLimitC8 > 0x7F000) gsLimitC8 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC8 < 0) gsLimitC8 = 0;

		if (gsLimitC8 > 0x100)  {

			gsSustainHC8 = (gsSustain[9] * gsLimitC8) / 100;

			if (gsAttack[9])  {
				gsDeltaC8 = (gsLimitC8) / gsAttack[9];
				if (gsDeltaC8 == 0) gsDeltaC8 = 1;
				gsHiValC8 = gsDeltaC8;
				if (gsPolarityC8 == 0) gsValueC8 = (float)(gsHiValC8) / 3200.;
				else  gsValueC8 = (float)(-gsHiValC8) / 6400.;
				gsPhaseC8 = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitC8 >= gsSustainHC8) {
					gsSignC8 = 0;
					gsDeltaC8 = (gsLimitC8 - gsSustainHC8) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC8 = 1;
					gsDeltaC8 = (gsSustainHC8 - gsLimitC8) / gsDecay[9];
				}
				if (gsDeltaC8 == 0) gsDeltaC8 = 1 ;
				gsHiValC8 = gsLimitC8;
				if (gsPolarityC8 == 0) gsValueC8 = (float)(gsHiValC8) / 3200.;
				else  gsValueC8 = (float)(-gsHiValC8) / 6400.;
				gsPhaseC8 = 2;
			}
			else if (gsSustainHC8)  {
				gsHiValC8 = gsSustainHC8;
				if (gsPolarityC8 == 0) gsValueC8 = (float)(gsHiValC8) / 3200.;
				else  gsValueC8 = (float)(-gsHiValC8) / 6400.;
				gsDeltaC8 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC8 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC8();
			}
			tempInitFilter(12,(uint32_t)(gsBaseFreqC8 * (32 + gsValueC8) / 32));
		}
		else tempInitFilter(12,gsBaseFreqC8);   //keytrack independent of ADSR
    }
}


void triggerFilterC9(uint32_t noteval, uint32_t velocity, uint32_t mode) // **
// ***************************************************************************
{
// ADSRC9 controls Cutoff of Filter#13 (clone of ADSR8/filter#0)

    if (gsNoTriggerF0 == 0)  { //filter#0 is active and signals are fed through

		if (gsKeytrack0) gsTrackShiftC9 = gsTrackFactor[noteval] * gsKeytrack0;
		else gsTrackShiftC9 = 0.0;
		gsBaseFreqC9=(uint32_t)(gfFilterFreq0*(gfF0ModBase+gsTrackShiftC9));
									//triggered by note, independently of ADSR

        gsValueC8 = 0.0;
        gsPhaseC8 = 0; //these settings as precaution,if no useable parameters

        if (mode == 0)  {
			tempInitFilter(13,gsBaseFreqC9);
			return;
        }

		gsLimitC9 = ((int32_t)(gsFixVol[9] - 64) * 2048) +
									(32* velocity * ((int32_t)(gsVelo[9] -28)));
		//stretched for better resolution of high Attack,Decay,Release values
		gsPolarityC9 = 0;
		if (gsLimitC9 < 0)  {
			gsLimitC9 = -gsLimitC9;
			gsPolarityC9 = 1;
		}
		if (gsLimitC9 > 0x7F000) gsLimitC9 = 0x7F000;//theoretically max 0x71800
		if (gsLimitC9 < 0) gsLimitC9 = 0;

		if (gsLimitC9 > 0x100)  {

			gsSustainHC9 = (gsSustain[9] * gsLimitC9) / 100;

			if (gsAttack[9])  {
				gsDeltaC9 = (gsLimitC9) / gsAttack[9];
				if (gsDeltaC9 == 0) gsDeltaC9 = 1;
				gsHiValC9 = gsDeltaC9;
				if (gsPolarityC9 == 0) gsValueC9 = (float)(gsHiValC9) / 3200.;
				else  gsValueC9 = (float)(-gsHiValC9) / 6400.;
				gsPhaseC9 = 1;
			}
			else if (gsDecay[9])  {
				if (gsLimitC9 >= gsSustainHC9) {
					gsSignC9 = 0;
					gsDeltaC9 = (gsLimitC9 - gsSustainHC9) / gsDecay[9];
				}
				else {          //should be impossible, introduced as precaution
					gsSignC9 = 1;
					gsDeltaC9 = (gsSustainHC9 - gsLimitC9) / gsDecay[9];
				}
				if (gsDeltaC9 == 0) gsDeltaC9 = 1 ;
				gsHiValC9 = gsLimitC9;
				if (gsPolarityC9 == 0) gsValueC9 = (float)(gsHiValC9) / 3200.;
				else  gsValueC9 = (float)(-gsHiValC9) / 6400.;
				gsPhaseC9 = 2;
			}
			else if (gsSustainHC9)  {
				gsHiValC9 = gsSustainHC9;
				if (gsPolarityC9 == 0) gsValueC9 = (float)(gsHiValC9) / 3200.;
				else  gsValueC9 = (float)(-gsHiValC9) / 6400.;
				gsDeltaC9 = 0;       //no Countdown, gets terminated by NOTE OFF
				gsPhaseC9 = 3;
				if (gsHoldSust[9] == 0)  releaseFilterC9();
			}
			tempInitFilter(13,(uint32_t)(gsBaseFreqC9 * (32 + gsValueC9) / 32));
		}
		else tempInitFilter(13,gsBaseFreqC9);    //keytrack independent of ADSR
    }
}


void releaseFilter0(void)   // **********************************************
//start Release Phase
// **************************************************************************
{
    if (gsPhaseF0M)  {
        if (gsRelease[9])  {
            gsDeltaF0M = gsHiValF0M / gsRelease[9];  //FINAL always = 0
            if (gsDeltaF0M == 0)  gsDeltaF0M = 1;
            gsPhaseF0M = 4;
        }
        else  {
            tempInitFilter(0,gsBaseFreqF0);
            gsPhaseF0M = 0;
        }
    }
}


void releaseFree(void)   // **********************************************
//start Release Phase
// **************************************************************************
{
    if (gsPhaseFree)  {
        if (gsRelease[gsFreeIdx])  {
            gsDeltaFree = gsHiValFree / gsRelease[gsFreeIdx];	//FINAL always=0
            if (gsDeltaFree == 0)  gsDeltaFree = 1;
            gsPhaseFree = 4;
        }
        else  {
			if (gsFreeAdsr == 2)  gfF0TempQF = gfRuntimeIIR[0].QFMOD;
            if (gsFreeAdsr == 3)  tempInitFilter(2,gfBaseFreqF2);
            gsPhaseFree = 0;
        }
        if (gsFreeAdsr == 2)  gsPhaseF0QF = gsPhaseFree;
    }
}


void releaseFilterC1(void)   // **********************************************
//start Release Phase
// **************************************************************************
{
    if (gsPhaseC1) {
       if (gsRelease[9])  {
            gsDeltaC1 = gsHiValC1 / gsRelease[9];
            if (gsDeltaC1 == 0)  gsDeltaC1 = 1;
            gsPhaseC1 = 4;
        }
        else  {
            tempInitFilter(5,gsBaseFreqC1);
            gsPhaseC1 = 0;
        }
    }
}



void releaseFilterC2(void)   // **********************************************
// **************************************************************************
{
    if (gsPhaseC2) {
        if  (gsRelease[9])  {
            gsDeltaC2 = gsHiValC2 / gsRelease[9];
            if (gsDeltaC2 == 0)  gsDeltaC2 = 1;
            gsPhaseC2 = 4;
        }
        else  {
            tempInitFilter(6,gsBaseFreqC2);
            gsPhaseC2 = 0;
        }
    }
}


void releaseFilterC3(void)   // **********************************************
// **************************************************************************
{
    if (gsPhaseC3) {
        if (gsRelease[9])  {
        gsDeltaC3 = gsHiValC3 / gsRelease[9];
        if (gsDeltaC3 == 0)  gsDeltaC3 = 1;
            gsPhaseC3 = 4;
        }
        else  {
            tempInitFilter(7,gsBaseFreqC3);
            gsPhaseC3 = 0;
        }
    }
}


void releaseFilterC4(void)   // **********************************************
// **************************************************************************
{
    if (gsPhaseC4) {
        if (gsRelease[9])  {
            gsDeltaC4 = gsHiValC4 / gsRelease[9];
            if (gsDeltaC4 == 0)  gsDeltaC4 = 1;
            gsPhaseC4 = 4;
        }
        else  {
            tempInitFilter(8,gsBaseFreqC4);
            gsPhaseC4 = 0;
        }
    }
}


void releaseFilterC5(void)   // **********************************************
// **************************************************************************
{
    if (gsPhaseC5) {
        if (gsRelease[9])  {
            gsDeltaC5 = gsHiValC5 / gsRelease[9];
            if (gsDeltaC5 == 0)  gsDeltaC5 = 1;
            gsPhaseC5 = 4;
        }
        else  {
            tempInitFilter(9,gsBaseFreqC5);
            gsPhaseC5 = 0;
        }
    }
}


void releaseFilterC6(void)   // **********************************************
// **************************************************************************
{
    if (gsPhaseC6) {
        if (gsRelease[9])  {
            gsDeltaC6 = gsHiValC6 / gsRelease[9];
            if (gsDeltaC6 == 0)  gsDeltaC6 = 1;
            gsPhaseC6 = 4;
        }
        else  {
            tempInitFilter(10,gsBaseFreqC6);
            gsPhaseC6 = 0;
        }
    }
}


void releaseFilterC7(void)   // *********************************************
// **************************************************************************
{
    if (gsPhaseC7) {
        if  (gsRelease[9])  {
            gsDeltaC7 = gsHiValC7 / gsRelease[9];
            if (gsDeltaC7 == 0)  gsDeltaC7 = 1;
            gsPhaseC7 = 4;
        }
        else  {
            tempInitFilter(11,gsBaseFreqC7);
            gsPhaseC7 = 0;
        }
    }
}


void releaseFilterC8(void)   // *********************************************
// **************************************************************************
{
    if (gsPhaseC8) {
        if  (gsRelease[9])  {
            gsDeltaC8 = gsHiValC8 / gsRelease[9];
            if (gsDeltaC8 == 0)  gsDeltaC8 = 1;
            gsPhaseC8 = 4;
        }
        else  {
            tempInitFilter(12,gsBaseFreqC8);
            gsPhaseC8 = 0;
        }
    }
}


void releaseFilterC9(void)   // *********************************************
// **************************************************************************
{
    if (gsPhaseC9) {
            if (gsRelease[9])  {
            gsDeltaC9 = gsHiValC9 / gsRelease[9];
            if (gsDeltaC9 == 0)  gsDeltaC9 = 1;
            gsPhaseC9 = 4;
        }
        else  {
            tempInitFilter(13,gsBaseFreqC9);
            gsPhaseC9 = 0;
        }
    }
}

void intoPhaseF0M_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitF0M == gsSustainHF0M) {
            intoPhaseF0M_3();
            return;
        }
        if (gsLimitF0M <= gsSustainHF0M) {
            gsSignF0M = 1;
            gsDeltaF0M = (gsSustainHF0M - gsLimitF0M) / gsDecay[9];

        }
        else {
            gsSignF0M = 0;
            gsDeltaF0M = (gsLimitF0M - gsSustainHF0M) / gsDecay[9];
        }
        if (gsDeltaF0M == 0) gsDeltaF0M = 1;
        gsPhaseF0M = 2;
    }
    else  intoPhaseF0M_3();
}


void intoPhaseF0M_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilter0();
        return;
    }
    if ((gsSustainHF0M) && (gsHiValF0M > 0x100)) {
        gsHiValF0M = gsSustainHF0M;
        if (gsPolarityF0M == 0) gsValueF0M = (float)(gsHiValF0M) / 3200.;
        else  gsValueF0M = (float)(-gsHiValF0M) / 6400.;
		tempInitFilter(0,(uint32_t)(gsBaseFreqF0 * (32 + gsValueF0M) / 32));
        gsDeltaF0M = 0;                           //is terminated by NOTE OFF
        gsPhaseF0M = 3;
    }
    else {
        tempInitFilter(0,gsBaseFreqF0);
        gsPhaseF0M = 0;
    }
}


void intoPhaseFree_2(void)  // **********************************************
// start Decay phase
//  *************************************************************************
{
    if (gsDecay[gsFreeIdx])  {
        if (gsLimitFree == gsSustainHFree) {
            intoPhaseFree_3();
            return;
        }
        if (gsLimitFree <= gsSustainHFree) {
            gsSignFree = 1;
            gsDeltaFree = (gsSustainHFree - gsLimitFree) / gsDecay[gsFreeIdx];

        }
        else {
            gsSignFree = 0;
            gsDeltaFree = (gsLimitFree - gsSustainHFree) / gsDecay[gsFreeIdx];
        }
        if (gsDeltaFree == 0) gsDeltaFree = 1;
        gsPhaseFree = 2;
		if (gsFreeAdsr == 3)  gsPhaseF0QF = 2;
    }
    else  intoPhaseFree_3();
}


void intoPhaseFree_3(void)  // **********************************************
// start Sustain phase
//  *************************************************************************
{
    if (gsHoldSust[gsFreeIdx] == 0)  {
        releaseFree();
        return;
    }
    if ((gsSustainHFree) && (gsHiValFree > 0x100))  {
        gsHiValFree = gsSustainHFree;
        gsDeltaFree = 0;							//is terminated by NOTE OFF
        gsPhaseFree = 3;
        if (gsFreeAdsr == 0)  {							//Osc2 additional FM
			if (gsPolarityFree == 0)  gsValueFMO2 = gsHiValFree >> 6;
			else  gsValueFMO2 = -(gsHiValFree >> 6);	//restore neg.sign
        }
        if (gsFreeAdsr == 1)  {							//Lfo2 FM
			if (gsPolarityFree == 0)  gsValueFLB = gsHiValFree >> 6;
			else  gsValueFLB = -(gsHiValFree >> 6);	//restore neg.sig
        }
		if (gsFreeAdsr == 2)  {
			gsPhaseF0QF = gsPhaseFree;
			if ((gfACTIV_IIR[0].TYPE == 5) || (gfACTIV_IIR[0].TYPE == 6))  {
				if (gsPolarityFree == 0) gsValueF0QF = (float)(gsHiValFree) /2.;
				else  gsValueF0QF = (float)(-gsHiValFree) / 2.;
			}
			else  {
				if (gsPolarityFree==0) gsValueF0QF = (float)(gsHiValFree)/8192.;
				else  gsValueF0QF= (float)(-gsHiValFree) / 8192.;
			}
			gfF0TempQF = (32*gfRuntimeIIR[0].QFMOD + gsValueF0QF) / 32;
		}
		if (gsFreeAdsr == 3)  {
			if (gsPolarityFree == 0) gsValueF2M = (float)(gsHiValFree) / 3200.;
			else  gsValueF2M = (float)(-gsHiValFree) / 6400.;
			tempInitFilter(2,(uint32_t)(gfBaseFreqF2* (32.0+ gsValueF2M) / 32));
		}
		if (gsFreeAdsr == 4)  {							//ChorusM
			if (gsPolarityFree == 0)  gsValueChor = gsHiValFree >> 6;
			else  gsValueChor= -(gsHiValFree >> 6);	//restore neg.sign
		}
    }
    else {
        if (gsFreeAdsr == 0)  {
			gsValueFMO2 = 0;
			gsPhaseFree = 0;
        }
		if (gsFreeAdsr == 1)  {
			gsValueFLB = 0;
			gsPhaseFree = 0;
		}
        if (gsFreeAdsr == 2)  {
			gfF0TempQF = gfRuntimeIIR[0].QFMOD;
			gsPhaseF0QF = 0;
			gsPhaseFree = 0;
        }
        if (gsFreeAdsr == 3)  {
			tempInitFilter(2,gfBaseFreqF2);
			gsPhaseFree = 0;
        }
        if (gsFreeAdsr == 4)  {
			gsValueChor = 0;
			gsPhaseFree = 0;
        }
        else if (gsFreeAdsr >= 5)  {
            gsDeltaFree = gsHiValFree / SHORT_RELEASE;
            if (gsDeltaFree) gsPhaseFree = 4;
            else  {
				gsValueFree = 0;
				gsPhaseFree = 0;
            }
        }
    }

}


void intoPhaseC1_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC1 == gsSustainHC1) {
            intoPhaseC1_3();
            return;
        }
        if (gsLimitC1 <= gsSustainHC1) {
            gsSignC1 = 1;
            gsDeltaC1 = (gsSustainHC1 - gsLimitC1) / gsDecay[9];

        }
        else {
            gsSignC1 = 0;
            gsDeltaC1 = (gsLimitC1 - gsSustainHC1) / gsDecay[9];
        }
        if (gsDeltaC1 == 0) gsDeltaC1 = 1;
        gsPhaseC1 = 2;
    }
    else  intoPhaseC1_3();
}


void intoPhaseC1_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC1();
        return;
    }
    if ((gsSustainHC1) && (gsHiValC1 > 0x100)) {
        gsHiValC1 = gsSustainHC1;
        if (gsPolarityC1 == 0) gsValueC1 = (float)(gsHiValC1) / 6400;
        else  gsValueC1 = (float)(-gsHiValC1) / 6400;
        tempInitFilter(5,(uint32_t)(gsBaseFreqC1 * (32 + gsValueC1) / 32));
        gsDeltaC1 = 0;                           //is terminated by NOTE OFF
        gsPhaseC1 = 3;
    }
    else {
        tempInitFilter(5,gsBaseFreqC1);
        gsPhaseC1 = 0;
    }
}


void intoPhaseC2_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC2 == gsSustainHC2) {
            intoPhaseC2_3();
            return;
        }
        if (gsLimitC2 <= gsSustainHC2) {
            gsSignC2 = 1;
            gsDeltaC2 = (gsSustainHC2 - gsLimitC2) / gsDecay[9];
        }
        else {
            gsSignC2 = 0;
            gsDeltaC2 = (gsLimitC2 - gsSustainHC2) / gsDecay[9];
        }
        if (gsDeltaC2 == 0) gsDeltaC2 = 1;
        gsPhaseC2 = 2;
    }
    else  intoPhaseC2_3();
}


void intoPhaseC2_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC2();
        return;
    }
    if ((gsSustainHC2) && (gsHiValC2 > 0x100))  {
        gsHiValC2 = gsSustainHC2;
        if (gsPolarityC2 == 0) gsValueC2 = (float)(gsHiValC2) / 6400;
        else  gsValueC2 = (float)(-gsHiValC2) / 6400;
        tempInitFilter(6,(uint32_t)(gsBaseFreqC2 * (32 + gsValueC2) / 32));
        gsDeltaC2 = 0;                           //is terminated by NOTE OFF
        gsPhaseC2 = 3;
    }
    else {
        tempInitFilter(6,gsBaseFreqC2);
        gsPhaseC2 = 0;
    }
}


void intoPhaseC3_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC3 == gsSustainHC3) {
            intoPhaseC3_3();
            return;
        }
        if (gsLimitC3 <= gsSustainHC3) {
            gsSignC3 = 1;
            gsDeltaC3 = (gsSustainHC3 - gsLimitC3) / gsDecay[9];
        }
        else {
            gsSignC3 = 0;
            gsDeltaC3 = (gsLimitC3 - gsSustainHC3) / gsDecay[9];
        }
        if (gsDeltaC3 == 0) gsDeltaC3 = 1;
        gsPhaseC3 = 2;
    }
    else  intoPhaseC3_3();
}


void intoPhaseC3_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC3();
        return;
    }
    if ((gsSustainHC3) && (gsHiValC3 > 0x100))  {
        gsHiValC3 = gsSustainHC3;
        if (gsPolarityC3 == 0) gsValueC3 = (float)(gsHiValC3) / 6400;
        else  gsValueC3 = (float)(-gsHiValC3) / 6400;
        tempInitFilter(7,(uint32_t)(gsBaseFreqC3 * (32 + gsValueC3) / 32));
        gsDeltaC3 = 0;                           //is terminated by NOTE OFF
        gsPhaseC3 = 3;
    }
    else {
        tempInitFilter(7,gsBaseFreqC3);
        gsPhaseC3 = 0;
    }
}


void intoPhaseC4_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC4 == gsSustainHC4) {
            intoPhaseC4_3();
            return;
        }
        if (gsLimitC4 <= gsSustainHC4) {
            gsSignC4 = 1;
            gsDeltaC4 = (gsSustainHC4 - gsLimitC4) / gsDecay[9];
        }
        else {
            gsSignC4 = 0;
            gsDeltaC4 = (gsLimitC4 - gsSustainHC4) / gsDecay[9];
        }
        if (gsDeltaC4 == 0) gsDeltaC4 = 1;
        gsPhaseC4 = 2;
    }
    else  intoPhaseC4_3();
}


void intoPhaseC4_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC4();
        return;
    }
    if ((gsSustainHC4) && (gsHiValC4 > 0x100))  {
        gsHiValC4 = gsSustainHC4;
        if (gsPolarityC4 == 0) gsValueC4 = (float)(gsHiValC4) / 6400;
        else  gsValueC4 = (float)(-gsHiValC4) / 6400;
        tempInitFilter(8,(uint32_t)(gsBaseFreqC4 * (32 + gsValueC4) / 32));
        gsDeltaC4 = 0;                           //is terminated by NOTE OFF
        gsPhaseC4 = 3;
    }
    else {
        tempInitFilter(8,gsBaseFreqC4);
        gsPhaseC4 = 0;
    }
}

void intoPhaseC5_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC5 == gsSustainHC5) {
            intoPhaseC5_3();
            return;
        }
        if (gsLimitC5 <= gsSustainHC5) {
            gsSignC5 = 1;
            gsDeltaC5 = (gsSustainHC5 - gsLimitC5) / gsDecay[9];
        }
        else {
            gsSignC5 = 0;
            gsDeltaC5 = (gsLimitC5 - gsSustainHC5) / gsDecay[9];
        }
        if (gsDeltaC5 == 0) gsDeltaC5 = 1;
        gsPhaseC5 = 2;
    }
    else  intoPhaseC5_3();
}


void intoPhaseC5_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC5();
        return;
    }
    if ((gsSustainHC5) && (gsHiValC5 > 0x100))  {
        gsHiValC5 = gsSustainHC5;
        if (gsPolarityC5 == 0) gsValueC5 = (float)(gsHiValC5) / 6400;
        else  gsValueC5 = (float)(-gsHiValC5) / 6400;
        tempInitFilter(9,(uint32_t)(gsBaseFreqC5 * (32 + gsValueC5) / 32));
        gsDeltaC5 = 0;                           //is terminated by NOTE OFF
        gsPhaseC5 = 3;
    }
    else {
        tempInitFilter(9,gsBaseFreqC5);
        gsPhaseC5 = 0;
    }
}


void intoPhaseC6_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC6 == gsSustainHC6) {
            intoPhaseC6_3();
            return;
        }
        if (gsLimitC6 <= gsSustainHC6) {
            gsSignC6 = 1;
            gsDeltaC6 = (gsSustainHC6 - gsLimitC6) / gsDecay[9];
        }
        else {
            gsSignC6 = 0;
            gsDeltaC6 = (gsLimitC6 - gsSustainHC6) / gsDecay[9];
        }
        if (gsDeltaC6 == 0) gsDeltaC6 = 1;
        gsPhaseC6 = 2;
    }
    else  intoPhaseC6_3();
}


void intoPhaseC6_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC6();
        return;
    }
    if ((gsSustainHC6) && (gsHiValC6 > 0x100))  {
        gsHiValC6 = gsSustainHC6;
        if (gsPolarityC6 == 0) gsValueC6 = (float)(gsHiValC6) / 6400;
        else  gsValueC6 = (float)(-gsHiValC6) / 6400;
        tempInitFilter(10,(uint32_t)(gsBaseFreqC6 * (32 + gsValueC6) / 32));
        gsDeltaC6 = 0;                           //is terminated by NOTE OFF
        gsPhaseC6 = 3;
    }
    else {
        tempInitFilter(10,gsBaseFreqC6);
        gsPhaseC6 = 0;
    }
}


void intoPhaseC7_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC7 == gsSustainHC7) {
            intoPhaseC7_3();
            return;
        }
        if (gsLimitC7 <= gsSustainHC7) {
            gsSignC7 = 1;
            gsDeltaC7 = (gsSustainHC7 - gsLimitC7) / gsDecay[9];
        }
        else {
            gsSignC7 = 0;
            gsDeltaC7 = (gsLimitC7 - gsSustainHC7) / gsDecay[9];
        }
        if (gsDeltaC7 == 0) gsDeltaC7 = 1;
        gsPhaseC7 = 2;
    }
    else  intoPhaseC7_3();
}


void intoPhaseC7_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC7();
        return;
    }
    if ((gsSustainHC7) && (gsHiValC7 > 0x100))  {
        gsHiValC7 = gsSustainHC7;
        if (gsPolarityC7 == 0) gsValueC7 = (float)(gsHiValC7) / 6400;
        else  gsValueC7 = (float)(-gsHiValC7) / 6400;
        tempInitFilter(11,(uint32_t)(gsBaseFreqC7 * (32 + gsValueC7) / 32));
        gsDeltaC7 = 0;                           //is terminated by NOTE OFF
        gsPhaseC7 = 3;
    }
    else {
        tempInitFilter(11,gsBaseFreqC7);
        gsPhaseC7 = 0;
    }
}


void intoPhaseC8_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC8 == gsSustainHC8) {
            intoPhaseC8_3();
            return;
        }
        if (gsLimitC8 <= gsSustainHC8) {
            gsSignC8 = 1;
            gsDeltaC8 = (gsSustainHC8 - gsLimitC8) / gsDecay[9];
        }
        else {
            gsSignC8 = 0;
            gsDeltaC8 = (gsLimitC8 - gsSustainHC8) / gsDecay[9];
        }
        if (gsDeltaC8 == 0) gsDeltaC8 = 1;
        gsPhaseC8 = 2;
    }
    else  intoPhaseC8_3();
}


void intoPhaseC8_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC8();
        return;
    }
    if ((gsSustainHC8) && (gsHiValC8 > 0x100))  {
        gsHiValC8 = gsSustainHC8;
        if (gsPolarityC8 == 0) gsValueC8 = (float)(gsHiValC8) / 6400;
        else  gsValueC8 = (float)(-gsHiValC8) / 6400;
        tempInitFilter(12,(uint32_t)(gsBaseFreqC8 * (32 + gsValueC8) / 32));
        gsDeltaC8 = 0;                           //is terminated by NOTE OFF
        gsPhaseC8 = 3;
    }
    else {
        tempInitFilter(12,gsBaseFreqC8);
        gsPhaseC8 = 0;
    }
}


void intoPhaseC9_2(void)  // ***********************************************
//  *************************************************************************
{
    if (gsDecay[9])  {
        if (gsLimitC9 == gsSustainHC9) {
            intoPhaseC9_3();
            return;
        }
        if (gsLimitC9 <= gsSustainHC9) {
            gsSignC9 = 1;
            gsDeltaC9 = (gsSustainHC9 - gsLimitC9) / gsDecay[9];
        }
        else {
            gsSignC9 = 0;
            gsDeltaC9 = (gsLimitC9 - gsSustainHC9) / gsDecay[9];
        }
        if (gsDeltaC9 == 0) gsDeltaC9 = 1;
        gsPhaseC9 = 2;
    }
    else  intoPhaseC9_3();
}


void intoPhaseC9_3(void)  // ***********************************************
//  *************************************************************************
{
    if (gsHoldSust[9] == 0)  {
        releaseFilterC9();
        return;
    }
    if ((gsSustainHC9) && (gsHiValC9 > 0x100))  {
        gsHiValC9 = gsSustainHC9;
        if (gsPolarityC9 == 0) gsValueC9 = (float)(gsHiValC9) / 6400;
        else  gsValueC9 = (float)(-gsHiValC9) / 6400;
        tempInitFilter(13,(uint32_t)(gsBaseFreqC9 * (32 + gsValueC9) / 32));
        gsDeltaC9 = 0;                           //is terminated by NOTE OFF
        gsPhaseC9 = 3;
    }
    else {
        tempInitFilter(13,gsBaseFreqC9);
        gsPhaseC9 = 0;
    }
}


// ##########################################################################

void ADSR_Process(void)   // ************************************************
//Attack/Decay control
// **************************************************************************
{
float value;
int32_t intValue;
uint32_t tempTone;

//ADSRs for Oscillator levels
    if (gsPhase0)  {
        switch (gsPhase0)  {
        case 1:							//Attack
            gsHiVal0 += gsDelta0;
            if (gsHiVal0 >= gsLimit0) intoPhase02();
            break;
        case 2:							//Decay
            if (gsSign0) {
                gsHiVal0 += gsDelta0;
                if (gsHiVal0 >= gsSustainH0)  intoPhase03();
            }
            else  {
                gsHiVal0 -= gsDelta0;
                if (gsHiVal0 <= gsSustainH0)  intoPhase03();
            }
            break;
//case3: not handled here, Sustain is terminated by NOTE OFF
        case 4:							//Release
            gsHiVal0 -= gsDelta0;
            if (gsHiVal0 <= 0)  {
                gsHiVal0 = 0;
                gsPhase0 = 0;
            }
            break;
        case 5:							//shutdown
            gsHiVal0 -= gsDelta0;
            if (gsHiVal0 <= 0)  {
                gsHiVal0 = 0;
                gsPhase0 = 0;
                tuneVoice0(gsNoteVal0, gsVelocity0);
            }
            break;
        default:
            break;
        }
        gsValue0 = gsHiVal0 >> 7;
    }

    if (gsPhase0X)  {
        switch (gsPhase0X)  {
        case 1:
            gsHiVal0X += gsDelta0X;
            if (gsHiVal0X >= gsLimit0X)  intoPhase0X2();
            break;
        case 2:
            if (gsSign0X) {
                gsHiVal0X += gsDelta0X;
                if (gsHiVal0X >= gsSustainH0X)  intoPhase0X3();
            }
            else  {
                gsHiVal0X -= gsDelta0X;
                if (gsHiVal0X <= gsSustainH0X)  intoPhase0X3();
            }
            break;
//case3: not handled here, Sustain is terminated by NOTE OFF
        case 4:
            gsHiVal0X -= gsDelta0X;
            if (gsHiVal0X <= 0) {
                gsHiVal0X = 0;
                gsPhase0X = 0;
            }
            break;
        case 5:
            gsHiVal0X -= gsDelta0X;
            if (gsHiVal0X <= 0) {
                gsHiVal0X = 0;
                gsPhase0X = 0;
                tuneVoice0X(gsNoteVal0, gsVelocity0);
            }
            break;
        default:
            break;
        }
        gsValue0X = gsHiVal0X >> 7;
    }

    if (gsPhase1) {
        switch (gsPhase1)  {
        case 1:
            gsHiVal1 += gsDelta1;
            if (gsHiVal1 >= gsLimit1) intoPhase12();
            break;
        case 2:
            if (gsSign1) {
                gsHiVal1 += gsDelta1;
                if (gsHiVal1 >= gsSustainH1)  intoPhase13();
            }
            else  {
                gsHiVal1 -= gsDelta1;
                if (gsHiVal1 <= gsSustainH1)  intoPhase13();
            }
            break;
        case 4:
            gsHiVal1 -= gsDelta1;
            if (gsHiVal1 <= 0)  {
                gsHiVal1 = 0;
                gsPhase1 = 0;
            }
            break;
        case 5:
            gsHiVal1 -= gsDelta1;
            if (gsHiVal1 <= 0)  {
                gsHiVal1 = 0;
                gsPhase1 = 0;
                tuneVoice1(gsNoteVal1, gsVelocity1);
            }
            break;
        default:
            break;
        }
        gsValue1 = gsHiVal1 >> 7;
    }

    if (gsPhase1X)  {
        switch (gsPhase1X)  {
        case 1:
            gsHiVal1X += gsDelta1X;
            if (gsHiVal1X >= gsLimit1X)  intoPhase1X2();
            break;
        case 2:
            if (gsSign1X) {
                gsHiVal1X += gsDelta1X;
                if (gsHiVal1X >= gsSustainH1X)  intoPhase1X3();
            }
            else  {
                gsHiVal1X -= gsDelta1X;
                if (gsHiVal1X <= gsSustainH1X)  intoPhase1X3();
            }
            break;
        case 4:
           gsHiVal1X -= gsDelta1X;
            if (gsHiVal1X <= 0) {
                gsHiVal1X = 0;
                gsPhase1X = 0;
            }
            break;
        case 5:
            gsHiVal1X -= gsDelta1X;
            if (gsHiVal1X <= 0) {
                gsHiVal1X = 0;
                gsPhase1X = 0;
                tuneVoice1X(gsNoteVal1, gsVelocity1);
            }
            break;
        default:
            break;
        }
        gsValue1X = gsHiVal1X >> 7;
    }

    if (gsPhase2)  {
        switch (gsPhase2)  {
        case 1:
            gsHiVal2 += gsDelta2;
            if (gsHiVal2 >= gsLimit2) intoPhase22();
            break;
        case 2:
            if (gsSign2) {
                gsHiVal2 += gsDelta2;
                if (gsHiVal2 >= gsSustainH2)  intoPhase23();
            }
            else  {
                gsHiVal2 -= gsDelta2;
                if (gsHiVal2 <= gsSustainH2)  intoPhase23();
            }
            break;
        case 4:
            gsHiVal2 -= gsDelta2;
            if (gsHiVal2 <= 0)  {
                gsHiVal2 = 0;
                gsPhase2 = 0;
            }
            break;
        case 5:
            gsHiVal2 -= gsDelta2;
            if (gsHiVal2 <= 0)  {
                gsHiVal2 = 0;
                gsPhase2 = 0;
                tuneVoice2(gsNoteVal2, gsVelocity2);
            }
            break;
        default:
            break;
        }
        gsValue2 = gsHiVal2 >> 7;
    }

    if (gsPhase2X)  {
        switch (gsPhase2X)  {
        case 1:
            gsHiVal2X += gsDelta2X;
            if (gsHiVal2X >= gsLimit2X)  intoPhase2X2();
            break;
        case 2:
            if (gsSign2X) {
                gsHiVal2X += gsDelta2X;
                if (gsHiVal2X >= gsSustainH2X)  intoPhase2X3();
            }
            else  {
                gsHiVal2X -= gsDelta2X;
                if (gsHiVal2X <= gsSustainH2X)  intoPhase2X3();
            }
            break;
        case 4:
            gsHiVal2X -= gsDelta2X;
            if (gsHiVal2X <= 0) {
                gsHiVal2X = 0;
                gsPhase2X = 0;
            }
            break;
        case 5:
            gsHiVal2X -= gsDelta2X;
            if (gsHiVal2X <= 0) {
                gsHiVal2X = 0;
                gsPhase2X = 0;
                tuneVoice2X(gsNoteVal2, gsVelocity2);
            }
            break;
        default:
            break;
        }
        gsValue2X = gsHiVal2X >> 7;
    }

    if (gsPhase3)  {
        switch (gsPhase3)  {
        case 1:
            gsHiVal3 += gsDelta3;
            if (gsHiVal3 >= gsLimit3) intoPhase32();
            break;
        case 2:
            if (gsSign3) {
                gsHiVal3 += gsDelta3;
                if (gsHiVal3 >= gsSustainH3)  intoPhase33();
            }
            else  {
                gsHiVal3 -= gsDelta3;
                if (gsHiVal3 <= gsSustainH3)  intoPhase33();
            }
            break;
        case 4:
            gsHiVal3 -= gsDelta3;
            if (gsHiVal3 <= 0)  {
                gsHiVal3 = 0;
                gsPhase3 = 0;
            }
            break;
        case 5:
            gsHiVal3 -= gsDelta3;
            if (gsHiVal3 <= 0)  {
                gsHiVal3 = 0;
                gsPhase3 = 0;
                tuneVoice3(gsNoteVal3, gsVelocity3);
            }
            break;
        default:
            break;
        }
        gsValue3 = gsHiVal3 >> 7;
    }

    if (gsPhase3X)  {
        switch (gsPhase3X)  {
        case 1:
            gsHiVal3X += gsDelta3X;
            if (gsHiVal3X >= gsLimit3X)  intoPhase3X2();
            break;
        case 2:
            if (gsSign3X) {
                gsHiVal3X += gsDelta3X;
                if (gsHiVal3X >= gsSustainH3X)  intoPhase3X3();
            }
            else  {
                gsHiVal3X -= gsDelta3X;
                if (gsHiVal3X <= gsSustainH3X)  intoPhase3X3();
            }
            break;
        case 4:
            gsHiVal3X -= gsDelta3X;
            if (gsHiVal3X <= 0) {
                gsHiVal3X = 0;
                gsPhase3X = 0;
            }
            break;
        case 5:
            gsHiVal3X -= gsDelta3X;
            if (gsHiVal3X <= 0) {
                gsHiVal3X = 0;
                gsPhase3X = 0;
                tuneVoice3X(gsNoteVal3, gsVelocity3);
            }
            break;
        default:
            break;
        }
        gsValue3X = gsHiVal3X >> 7;
    }

    if (gsPhase4)  {
        switch (gsPhase4)  {
        case 1:
            gsHiVal4 += gsDelta4;
            if (gsHiVal4 >= gsLimit4) intoPhase42();
            break;
        case 2:
            if (gsSign4) {
                gsHiVal4 += gsDelta4;
                if (gsHiVal4 >= gsSustainH4)  intoPhase43();
            }
            else  {
                gsHiVal4 -= gsDelta4;
                if (gsHiVal4 <= gsSustainH4)  intoPhase43();
            }
            break;
        case 4:
            gsHiVal4 -= gsDelta4;
            if (gsHiVal4 <= 0)  {
                gsHiVal4 = 0;
                gsPhase4 = 0;
            }
            break;
        case 5:
            gsHiVal4 -= gsDelta4;
            if (gsHiVal4 <= 0)  {
                gsHiVal4 = 0;
                gsPhase4 = 0;
                tuneVoice4(gsNoteVal4, gsVelocity4);
            }
            break;
        default:
            break;
        }
        gsValue4 = gsHiVal4 >> 7;
    }

    if (gsPhase4X)  {
        switch (gsPhase4X)  {
        case 1:
            gsHiVal4X += gsDelta4X;
            if (gsHiVal4X >= gsLimit4X)  intoPhase4X2();
            break;
        case 2:
            if (gsSign4X) {
                gsHiVal4X += gsDelta4X;
                if (gsHiVal4X >= gsSustainH4X)  intoPhase4X3();
            }
            else  {
                gsHiVal4X -= gsDelta4X;
                if (gsHiVal4X <= gsSustainH4X)  intoPhase4X3();
            }
            break;
        case 4:
            gsHiVal4X -= gsDelta4X;
            if (gsHiVal4X <= 0) {
                gsHiVal4X = 0;
                gsPhase4X = 0;
            }
            break;
        case 5:
            gsHiVal4X -= gsDelta4X;
            if (gsHiVal4X <= 0) {
                gsHiVal4X = 0;
                gsPhase4X = 0;
                tuneVoice4X(gsNoteVal4, gsVelocity4);
            }
            break;
        default:
            break;
        }
        gsValue4X = gsHiVal4X >> 7;
    }
    if (gsPhase5)  {
        switch (gsPhase5)  {
        case 1:
            gsHiVal5 += gsDelta5;
            if (gsHiVal5 >= gsLimit5) intoPhase52();
            break;
        case 2:
            if (gsSign5) {
                gsHiVal5 += gsDelta5;
                if (gsHiVal5 >= gsSustainH5)  intoPhase53();
            }
            else  {
                gsHiVal5 -= gsDelta5;
                if (gsHiVal5 <= gsSustainH5)  intoPhase53();
            }
            break;
        case 4:
            gsHiVal5 -= gsDelta5;
            if (gsHiVal5 <= 0)  {
                gsHiVal5 = 0;
                gsPhase5 = 0;
            }
            break;
        case 5:
            gsHiVal5 -= gsDelta5;
            if (gsHiVal5 <= 0)  {
                gsHiVal5 = 0;
                gsPhase5 = 0;
                tuneVoice5(gsNoteVal5, gsVelocity5);
            }
            break;
        default:
            break;
        }
        gsValue5 = gsHiVal5 >> 7;
    }

    if (gsPhase5X)  {
        switch (gsPhase5X)  {
        case 1:
            gsHiVal5X += gsDelta5X;
            if (gsHiVal5X >= gsLimit5X)  intoPhase5X2();
            break;
        case 2:
            if (gsSign5X) {
                gsHiVal5X += gsDelta5X;
                if (gsHiVal5X >= gsSustainH5X)  intoPhase5X3();
            }
            else  {
                gsHiVal5X -= gsDelta5X;
                if (gsHiVal5X <= gsSustainH5X)  intoPhase5X3();
            }
            break;
        case 4:
            gsHiVal5X -= gsDelta5X;
            if (gsHiVal5X <= 0) {
                gsHiVal5X = 0;
                gsPhase5X = 0;
            }
            break;
        case 5:
            gsHiVal5X -= gsDelta5X;
            if (gsHiVal5X <= 0) {
                gsHiVal5X = 0;
                gsPhase5X = 0;
                tuneVoice5X(gsNoteVal5, gsVelocity5);
            }
            break;
        default:
            break;
        }
        gsValue5X = gsHiVal5X >> 7;
    }

    if (gsPhase6)  {
        switch (gsPhase6)  {
        case 1:
            gsHiVal6 += gsDelta6;
            if (gsHiVal6 >= gsLimit6) intoPhase62();
            break;
        case 2:
            if (gsSign6) {
                gsHiVal6 += gsDelta6;
                if (gsHiVal6 >= gsSustainH6)  intoPhase63();
            }
               else  {
                gsHiVal6 -= gsDelta6;
                if (gsHiVal6 <= gsSustainH6)  intoPhase63();
            }
            break;
        case 4:
            gsHiVal6 -= gsDelta6;
            if (gsHiVal6 <= 0)  {
                gsHiVal6 = 0;
                gsPhase6 = 0;
            }
            break;
       case 5:
            gsHiVal6 -= gsDelta6;
            if (gsHiVal6 <= 0)  {
                gsHiVal6 = 0;
                gsPhase6 = 0;
                tuneVoice6(gsNoteVal6, gsVelocity6);
            }
            break;
        default:
            break;
        }
        gsValue6 = gsHiVal6 >> 7;
    }

    if (gsPhase6X)  {
        switch (gsPhase6X)  {
        case 1:
            gsHiVal6X += gsDelta6X;
            if (gsHiVal6X >= gsLimit6X)  intoPhase6X2();
            break;
        case 2:
            if (gsSign6X) {
                gsHiVal6X += gsDelta6X;
                if (gsHiVal6X >= gsSustainH6X)  intoPhase6X3();
            }
            else  {
                gsHiVal6X -= gsDelta6X;
                if (gsHiVal6X <= gsSustainH6X)  intoPhase6X3();
            }
            break;
        case 4:
            gsHiVal6X -= gsDelta6X;
            if (gsHiVal6X <= 0) {
                gsHiVal6X = 0;
                gsPhase6X = 0;
            }
            break;
        case 5:
            gsHiVal6X -= gsDelta6X;
            if (gsHiVal6X <= 0) {
                gsHiVal6X = 0;
                gsPhase6X = 0;
                tuneVoice6X(gsNoteVal6, gsVelocity6);
            }
            break;
        default:
            break;
        }
        gsValue6X = gsHiVal6X >> 7;
    }

    if (gsPhase7)  {
        switch (gsPhase7)  {
        case 1:
            gsHiVal7 += gsDelta7;
            if (gsHiVal7 >= gsLimit7) intoPhase72();
            break;
        case 2:
            if (gsSign7) {
                gsHiVal7 += gsDelta7;
                if (gsHiVal7 >= gsSustainH7)  intoPhase73();
            }
               else  {
                gsHiVal7 -= gsDelta7;
                if (gsHiVal7 <= gsSustainH7)  intoPhase73();
            }
            break;
        case 4:
            gsHiVal7 -= gsDelta7;
            if (gsHiVal7 <= 0)  {
                gsHiVal7 = 0;
                gsPhase7 = 0;
            }
            break;
       case 5:
            gsHiVal7 -= gsDelta7;
            if (gsHiVal7 <= 0)  {
                gsHiVal7 = 0;
                gsPhase7 = 0;
                tuneVoice7(gsNoteVal7, gsVelocity7);
            }
            break;
        default:
            break;
        }
        gsValue7 = gsHiVal7 >> 7;
    }

    if (gsPhase7X)  {
        switch (gsPhase7X)  {
        case 1:
            gsHiVal7X += gsDelta7X;
            if (gsHiVal7X >= gsLimit7X)  intoPhase7X2();
            break;
        case 2:
            if (gsSign7X) {
                gsHiVal7X += gsDelta7X;
                if (gsHiVal7X >= gsSustainH7X)  intoPhase7X3();
            }
            else  {
                gsHiVal7X -= gsDelta7X;
                if (gsHiVal7X <= gsSustainH7X)  intoPhase7X3();
            }
            break;
        case 4:
            gsHiVal7X -= gsDelta7X;
            if (gsHiVal7X <= 0) {
                gsHiVal7X = 0;
                gsPhase7X = 0;
            }
            break;
        case 5:
            gsHiVal7X -= gsDelta7X;
            if (gsHiVal7X <= 0) {
                gsHiVal7X = 0;
                gsPhase7X = 0;
                tuneVoice7X(gsNoteVal7, gsVelocity7);
            }
            break;
        default:
            break;
        }
        gsValue7X = gsHiVal7X >> 7;
    }

    if (gsPhase8)  {
        switch (gsPhase8)  {
        case 1:
            gsHiVal8 += gsDelta8;
            if (gsHiVal8 >= gsLimit8) intoPhase82();
            break;
        case 2:
            if (gsSign8) {
                gsHiVal8 += gsDelta8;
                if (gsHiVal8 >= gsSustainH8)  intoPhase83();
            }
               else  {
                gsHiVal8 -= gsDelta8;
                if (gsHiVal8 <= gsSustainH8)  intoPhase83();
            }
            break;
        case 4:
            gsHiVal8 -= gsDelta8;
            if (gsHiVal8 <= 0)  {
                gsHiVal8 = 0;
                gsPhase8 = 0;
            }
            break;
        case 5:
            gsHiVal8 -= gsDelta8;
            if (gsHiVal8 <= 0)  {
                gsHiVal8 = 0;
                gsPhase8 = 0;
                tuneVoice8(gsNoteVal8, gsVelocity8);
            }
            break;
        default:
            break;
        }
        gsValue8 = gsHiVal8 >> 7;
    }

    if (gsPhase8X)  {
        switch (gsPhase8X)  {
        case 1:
            gsHiVal8X += gsDelta8X;
            if (gsHiVal8X >= gsLimit8X)  intoPhase8X2();
            break;
        case 2:
            if (gsSign8X) {
                gsHiVal8X += gsDelta8X;
                if (gsHiVal8X >= gsSustainH8X)  intoPhase8X3();
            }
            else  {
                gsHiVal8X -= gsDelta8X;
                if (gsHiVal8X <= gsSustainH8X)  intoPhase8X3();
            }
            break;
        case 4:
            gsHiVal8X -= gsDelta8X;
            if (gsHiVal8X <= 0) {
                gsHiVal8X = 0;
                gsPhase8X = 0;
            }
            break;
        case 5:
            gsHiVal8X -= gsDelta8X;
            if (gsHiVal8X <= 0) {
                gsHiVal8X = 0;
                gsPhase8X = 0;
                tuneVoice8X(gsNoteVal8, gsVelocity8);
            }
            break;
        default:
            break;
        }
        gsValue8X = gsHiVal8X >> 7;
    }

    if (gsPhase9)  {
        switch (gsPhase9)  {
        case 1:
            gsHiVal9 += gsDelta9;
            if (gsHiVal9 >= gsLimit9) intoPhase92();
            break;
        case 2:
            if (gsSign9) {
                gsHiVal9 += gsDelta9;
                if (gsHiVal9 >= gsSustainH9)  intoPhase93();
            }
               else  {
                gsHiVal9 -= gsDelta9;
                if (gsHiVal9 <= gsSustainH9)  intoPhase93();
            }
            break;
        case 4:
            gsHiVal9 -= gsDelta9;
            if (gsHiVal9 <= 0)  {
                gsHiVal9 = 0;
                gsPhase9 = 0;
            }
            break;
       case 5:
            gsHiVal9 -= gsDelta9;
            if (gsHiVal9<= 0)  {
                gsHiVal9= 0;
                gsPhase9= 0;
                tuneVoice9(gsNoteVal9, gsVelocity9);
            }
            break;
        default:
            break;
        }
        gsValue9 = gsHiVal9 >> 7;
    }

    if (gsPhase9X)  {
        switch (gsPhase9X)  {
        case 1:
            gsHiVal9X += gsDelta9X;
            if (gsHiVal9X >= gsLimit9X)  intoPhase9X2();
            break;
        case 2:
            if (gsSign9X) {
                gsHiVal9X += gsDelta9X;
                if (gsHiVal9X >= gsSustainH9X)  intoPhase9X3();
            }
            else  {
                gsHiVal9X -= gsDelta9X;
                if (gsHiVal9X <= gsSustainH9X)  intoPhase9X3();
            }
            break;
        case 4:
            gsHiVal9X -= gsDelta9X;
            if (gsHiVal9X <= 0) {
                gsHiVal9X = 0;
                gsPhase9X = 0;
            }
            break;
        case 5:
            gsHiVal9X -= gsDelta9X;
            if (gsHiVal9X <= 0) {
                gsHiVal9X = 0;
                gsPhase9X = 0;
                tuneVoice9X(gsNoteVal9, gsVelocity9);
            }
            break;
        default:
            break;
        }
        gsValue9X = gsHiVal9X >> 7;
    }

//GLIDE:
    if (gsGlideBusy) {
        gsStep0 += gsDeltaStep;
        if (gsStepSign == 0){               //glide towards higher pitch
            if (gsStep0 >= gsFinal) {       //reset to process phase 2
                gsStep0 = gsFinal;
                gsGlideBusy = 0;

            }
        }
        else  {                             //glide towards lower pitch
            if (gsStep0 <= gsFinal) {       //reset to process phase 2
                gsStep0 = gsFinal;
                gsGlideBusy = 0;
            }
        }
        gsRunStep0 = gsStep0 * gModCent1;
    }
    if (gsGlideBusyX) {
        gsStep0X += gsDeltaStepX;
        if (gsStepSignX == 0){    			//glide towards higher pitch
            if (gsStep0X >= gsFinalX) { 	//reset to process phase 2
                gsStep0X = gsFinalX;
                gsGlideBusyX = 0;
            }
        }
        else  {                             //glide towards lower pitch
            if (gsStep0X <= gsFinalX) {		//reset to process phase 2
                gsStep0X = gsFinalX;
                gsGlideBusyX = 0;
            }
        }
        gsRunStep0X = gsStep0X * gModCent2;
    }

//ADSR for special features
    if (gsPhaseNs)  {                             //controls Noise envelope
        switch (gsPhaseNs)  {
        case 1:
        gsHiValNs += gsDeltaNs;
            if (gsHiValNs >= gsLimitNs) intoPhaseNs2();
            break;
        case 2:
            if (gsSignNs) {
                gsHiValNs += gsDeltaNs;
                if (gsHiValNs >= gsSustainHNs)  intoPhaseNs3();
            }
            else  {
                gsHiValNs -= gsDeltaNs;
                if (gsHiValNs <= gsSustainHNs)  intoPhaseNs3();
                }
                break;
//case3: handled as "default", Sustain is terminated by NOTE OFF
            case 4:
                gsHiValNs -= gsDeltaNs;
                if (gsHiValNs <= 0)  {
                    gsHiValNs = 0;
                    gsPhaseNs = 0;
                }
                break;
            case 5:
                gsHiValNs -= gsDeltaNs;
                if (gsHiValNs <= 0)  {
                    gsHiValNs = 0;
                    gsPhaseNs = 0;
                    triggerNoise(gsVelocityNs);
                }
                break;
            default:
                break;
        }
        gsValueNs = gsHiValNs >> 7;
    }

   if (gsPhaseFMO12)  {			//controls frequency modulation Osc1+2 commonly
        switch (gsPhaseFMO12)  {
        case 1:
            gsHiValFMO12 += gsDeltaFMO12;
            if (gsHiValFMO12 >= gsLimitFMO12) intoPhaseFMO122();
            break;
        case 2:
            if (gsSignFMO12) {
                gsHiValFMO12 += gsDeltaFMO12;
                if (gsHiValFMO12 >= gsSustainHFMO12)  intoPhaseFMO123();
            }
            else  {
                gsHiValFMO12 -= gsDeltaFMO12;
                if (gsHiValFMO12 <= gsSustainHFMO12)  intoPhaseFMO123();
            }
            break;
        case 4:
            gsHiValFMO12 -= gsDeltaFMO12;
            if (gsHiValFMO12 <= 0)  {
                gsHiValFMO12 = 0;
                gsPhaseFMO12 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityFMO12 == 0) gsValueFMO12 = gsHiValFMO12 >> 6;
		else  gsValueFMO12 = -(gsHiValFMO12 >> 6);		//restore neg.sign
    }

    if (gsPhaseFLfo)  {				//controls Lfo1 frequency modulation
        switch (gsPhaseFLfo)  {
        case 1:
            gsHiValFLfo += gsDeltaFLfo;
            if (gsHiValFLfo >= gsLimitFLfo) intoPhaseFLfo2();
            break;
        case 2:
            if (gsSignFLfo) {
                gsHiValFLfo += gsDeltaFLfo;
                if (gsHiValFLfo >= gsSustainHFLfo)  intoPhaseFLfo3();
            }
            else  {
                gsHiValFLfo -= gsDeltaFLfo;
                if (gsHiValFLfo <= gsSustainHFLfo)  intoPhaseFLfo3();
            }
            break;
//case3: handled as "default", Sustain is terminated by NOTE OFF
        case 4:
            gsHiValFLfo -= gsDeltaFLfo;
            if (gsHiValFLfo <= 0)  {
                gsHiValFLfo = 0;
                gsPhaseFLfo = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityFLfo == 0) gsValueFLfo = gsHiValFLfo >> 6;
		else  gsValueFLfo = -(gsHiValFLfo >> 6);		//restore neg.sign
    }

    if (gsPhaseVib)  {					//controls Vibrato depth modulation
        switch (gsPhaseVib)  {
        case 1:
            gsHiValVib += gsDeltaVib;
            if (gsHiValVib >= gsLimitVib) intoPhaseVib2();
            break;
        case 2:
            if (gsSignVib) {
                gsHiValVib += gsDeltaVib;
                if (gsHiValVib >= gsSustainHVib)  intoPhaseVib3();
            }
            else  {
                gsHiValVib -= gsDeltaVib;
                if (gsHiValVib <= gsSustainHVib)  intoPhaseVib3();
            }
            break;
        case 4:
            gsHiValVib -= gsDeltaVib;
            if (gsHiValVib <= 0)  {
                gsHiValVib = 0;
                gsPhaseVib = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityVib == 0) gsValueVib = gsHiValVib >> 6;
		else  gsValueVib = -(gsHiValVib >> 6);	//max possible ca.+/-100
    }

    if (gsPhaseBass)  {						//controls Bass Tone modulation
		intValue = gsValueBass;
        switch (gsPhaseBass)  {
        case 1:
            gsHiValBass += gsDeltaBass;
            if (gsHiValBass >= gsLimitBass) intoPhaseBass2();
            break;
        case 2:
            if (gsSignBass) {
                gsHiValBass += gsDeltaBass;
                if (gsHiValBass >= gsSustainHBass)  intoPhaseBass3();
            }
            else  {
                gsHiValBass -= gsDeltaBass;
                if (gsHiValBass <= gsSustainHBass)  intoPhaseBass3();
            }
            break;
//case3: handled as "default", Sustain is terminated by NOTE OFF
        case 4:
            gsHiValBass -= gsDeltaBass;
            if (gsHiValBass <= 0)  {
                gsHiValBass = 0;
                gsPhaseBass = 0;

            }
            break;
        default:
            break;
        }
		if (gsPolarityBass == 0) gsValueBass = gsHiValBass >> 6;
		else  gsValueBass = -(gsHiValBass >> 6);		//restore neg.sign
		if (gsValueBass != intValue)  {
			tempTone = (32*gfBass + gsValueBass) / 32;
			if (tempTone > 0x1000) tempTone = 0;
			if (tempTone > 127) tempTone = 127;
			if (tempTone < 0 )tempTone = 0;
			adjPeak3(tempTone);
        }
    }

    if (gsPhaseTreble)  {				//controls Treble modulation
		intValue = gsValueTreble;
        switch (gsPhaseTreble)  {
        case 1:
            gsHiValTreble += gsDeltaTreble;
            if (gsHiValTreble >= gsLimitTreble) intoPhaseTreble2();
            break;
        case 2:
            if (gsSignTreble) {
                gsHiValTreble += gsDeltaTreble;
                if (gsHiValTreble >= gsSustainHTreble)  intoPhaseTreble3();
            }
            else  {
                gsHiValTreble -= gsDeltaTreble;
                if (gsHiValTreble <= gsSustainHTreble)  intoPhaseTreble3();
            }
            break;
//case3: handled as "default", Sustain is terminated by NOTE OFF
        case 4:
            gsHiValTreble -= gsDeltaTreble;
            if (gsHiValTreble <= 0)  {
                gsHiValTreble = 0;
                gsPhaseTreble = 0;

            }
            break;
        default:
            break;
        }
		if (gsPolarityTreble == 0) gsValueTreble = gsHiValTreble >> 6;
		else  gsValueTreble = -(gsHiValTreble >> 6); 		//restore neg.sign
		if (gsValueTreble != intValue)  {
			tempTone = (32*gfTreble + gsValueTreble) / 32;
			if (tempTone > 0x1000) tempTone = 0;
			if (tempTone > 127) tempTone = 127;
			if (tempTone < 0 )tempTone = 0;
			adjPeak4(tempTone);
        }
    }

    if ((gfGain > 1) && gsPhaseG)  {                  //controls Gain envelope
        switch (gsPhaseG)  {
        case 1:
            gsHiValG += gsDeltaG;
            if (gsHiValG >= gsLimitG) intoPhaseG2();
            break;
        case 2:
            if (gsSignG) {
                gsHiValG += gsDeltaG;
                if (gsHiValG >= gsSustainHG)  intoPhaseG3();
            }
            else  {
                gsHiValG -= gsDeltaG;
                if (gsHiValG <= gsSustainHG)  intoPhaseG3();
            }
            break;
        case 4:
            gsHiValG -= gsDeltaG;
            if (gsHiValG <= 0)  {
                gsHiValG = 0;
                gsPhaseG = 0;
            }
            break;
        case 5:
            gsHiValG -= gsDeltaG;
            if (gsHiValG <= 0)  {
                gsHiValG = 0;
                gsPhaseG = 0;
                triggerGain(gsVelocityG);
            }
        default:
            break;
        }
        gsValueG = gsHiValG >> 7;
        if (gsValueG < 0)  gsValueG = 0; 	//precaution
    }

    if (gsPhaseF0M)  {						//controls Filter0 frequency shift
        value = gsValueF0M;
        switch (gsPhaseF0M)  {
        case 1:
            gsHiValF0M += gsDeltaF0M;
            if (gsHiValF0M >= gsLimitF0M) intoPhaseF0M_2();
            break;
        case 2:
            if (gsSignF0M) {
                gsHiValF0M += gsDeltaF0M;
                if (gsHiValF0M >= gsSustainHF0M)  intoPhaseF0M_3();
            }
            else  {
                gsHiValF0M -= gsDeltaF0M;
                if (gsHiValF0M <= gsSustainHF0M)  intoPhaseF0M_3();
            }
            break;
//case3: handled as "default", Sustain is terminated by NOTE OFF
        case 4:
            gsHiValF0M -= gsDeltaF0M;
            if (gsHiValF0M <= 0) {
                gsHiValF0M = 0;
                tempInitFilter(0,gsBaseFreqF0);
                gsPhaseF0M = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityF0M == 0) gsValueF0M = (float)(gsHiValF0M) / 3200.;
		else  gsValueF0M = (float)(-gsHiValF0M) / 6400.;
		if (gsValueF0M != value) {
			tempInitFilter(0,(uint32_t)(gsBaseFreqF0 * (32 + gsValueF0M) / 32));
		}
    }

    if (gsPhaseF0Pk)  {						//controls F0 Peak modulation
		value = gsValueF0Pk;
        switch (gsPhaseF0Pk)  {
        case 1:
            gsHiValF0Pk += gsDeltaF0Pk;
            if (gsHiValF0Pk >= gsLimitF0Pk) intoPhaseF0Pk2();
            break;
        case 2:
            if (gsSignF0Pk) {
                gsHiValF0Pk += gsDeltaF0Pk;
                if (gsHiValF0Pk >= gsSustainHF0Pk)  intoPhaseF0Pk3();
            }
            else  {
                gsHiValF0Pk -= gsDeltaF0Pk;
                if (gsHiValF0Pk <= gsSustainHF0Pk)  intoPhaseF0Pk3();
            }
            break;
        case 4:
            gsHiValF0Pk -= gsDeltaF0Pk;
            if (gsHiValF0Pk <= 0)  {
                gsHiValF0Pk = 0;
                gsPhaseF0Pk = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityF0Pk == 0) gsValueF0Pk =(float)(gsHiValF0Pk) / 744.;
		else  gsValueF0Pk = (float)(-gsHiValF0Pk) / 744.;
		if ((gsPhaseF0Pk == 3) || (gsValueF0Pk != value)) {
			gfF0TempPeak = (32 * gfRuntimeIIR[0].FPEAKMOD + gsValueF0Pk) / 32;
			if (gfF0TempPeak < 0.0)  gfF0TempPeak = 0.0;
			if (gfF0TempPeak > 11.0)  gfF0TempPeak = 11.0;
		}
    }

    if (gsPhaseC1)  {            //controls FilterC1(F0M clone) frequency shift
        value = gsValueC1;
        switch (gsPhaseC1)  {
        case 1:
            gsHiValC1 += gsDeltaC1;
            if (gsHiValC1 >= gsLimitC1) intoPhaseC1_2();
            break;
        case 2:
            if (gsSignC1) {
                gsHiValC1 += gsDeltaC1;
                if (gsHiValC1 >= gsSustainHC1)  intoPhaseC1_3();
            }
            else  {
                gsHiValC1 -= gsDeltaC1;
                if (gsHiValC1 <= gsSustainHC1)  intoPhaseC1_3();
            }
            break;
//case3: handled as "default", Sustain is terminated by NOTE OFF
        case 4:
            gsHiValC1 -= gsDeltaC1;
            if (gsHiValC1 <= 0) {
                gsHiValC1 = 0;
                tempInitFilter(5,gsBaseFreqC1);
                gsPhaseC1 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityC1 == 0) gsValueC1 = (float)(gsHiValC1) / 3200.;
		else  gsValueC1 = (float)(-gsHiValC1) / 6400.;
		if  (gsValueC1 != value) {
			tempInitFilter(5,(uint32_t)(gsBaseFreqC1 * (32 + gsValueC1) / 32));
		}
    }

    if (gsPhaseC2)  {                    //controls FilterC2 frequency shift
        value = gsValueC2;
        switch (gsPhaseC2)  {
        case 1:
            gsHiValC2 += gsDeltaC2;
            if (gsHiValC2 >= gsLimitC2) intoPhaseC2_2();
            break;
        case 2:
            if (gsSignC2) {
                gsHiValC2 += gsDeltaC2;
                if (gsHiValC2 >= gsSustainHC2)  intoPhaseC2_3();
            }
            else  {
                gsHiValC2 -= gsDeltaC2;
                if (gsHiValC2 <= gsSustainHC2)  intoPhaseC2_3();
            }
            break;
        case 4:
            gsHiValC2 -= gsDeltaC2;
            if (gsHiValC2 <= 0) {
                gsHiValC2 = 0;
                tempInitFilter(6,gsBaseFreqC2);
                gsPhaseC2 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityC2 == 0) gsValueC2 = (float)(gsHiValC2) / 3200.;
		else  gsValueC2 = (float)(-gsHiValC2) / 6400.;
		if  (gsValueC2 != value) {
			tempInitFilter(6,(uint32_t)(gsBaseFreqC2* (32 + gsValueC2) / 32));
		}
    }

    if (gsPhaseC3)  {                     //controls FilterC3 frequency shift
        value = gsValueC3;
        switch (gsPhaseC3)  {
        case 1:
            gsHiValC3 += gsDeltaC3;
            if (gsHiValC3 >= gsLimitC3) intoPhaseC3_2();
            break;
        case 2:
            if (gsSignC3) {
                gsHiValC3 += gsDeltaC3;
                if (gsHiValC3 >= gsSustainHC3)  intoPhaseC3_3();
            }
            else  {
                gsHiValC3 -= gsDeltaC3;
                if (gsHiValC3 <= gsSustainHC3)  intoPhaseC3_3();
            }
            break;
        case 4:
            gsHiValC3 -= gsDeltaC3;
            if (gsHiValC3 <= 0) {
                gsHiValC3 = 0;
                tempInitFilter(7,gsBaseFreqC3);
                gsPhaseC3 = 0;
            }
            break;
        default:
            break;
        }
        if (gsPolarityC3 == 0) gsValueC3 = (float)(gsHiValC3) / 3200.;
        else  gsValueC3 = (float)(-gsHiValC3) / 6400.;
        if  (gsValueC3 != value) {
			tempInitFilter(7,(uint32_t)(gsBaseFreqC3* (32 + gsValueC3) / 32));
        }
    }

    if (gsPhaseC4)  {                    //controls FilterC4 frequency shift
        value = gsValueC4;
        switch (gsPhaseC4)  {
        case 1:
            gsHiValC4 += gsDeltaC4;
            if (gsHiValC4 >= gsLimitC4) intoPhaseC4_2();
            break;
        case 2:
            if (gsSignC4) {
                gsHiValC4 += gsDeltaC4;
                if (gsHiValC4 >= gsSustainHC4)  intoPhaseC4_3();
            }
            else  {
                gsHiValC4 -= gsDeltaC4;
                if (gsHiValC4 <= gsSustainHC4)  intoPhaseC4_3();
            }
            break;
        case 4:
            gsHiValC4 -= gsDeltaC4;
            if (gsHiValC4 <= 0) {
                gsHiValC4 = 0;
                tempInitFilter(8,gsBaseFreqC4);
                gsPhaseC4 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityC4 == 0) gsValueC4 = (float)(gsHiValC4) / 3200.;
		else  gsValueC4 = (float)(-gsHiValC4) / 6400.;
		if  (gsValueC4 != value) {
			tempInitFilter(8,(uint32_t)(gsBaseFreqC4* (32 + gsValueC4) / 32));
		}
    }

    if (gsPhaseC5)  {                    //controls FilterC5 frequency shift
        value = gsValueC5;
        switch (gsPhaseC5)  {
        case 1:
            gsHiValC5 += gsDeltaC5;
            if (gsHiValC5 >= gsLimitC5) intoPhaseC5_2();
            break;
        case 2:
            if (gsSignC5) {
                gsHiValC5 += gsDeltaC5;
                if (gsHiValC5 >= gsSustainHC5)  intoPhaseC5_3();
            }
            else  {
                gsHiValC5 -= gsDeltaC5;
                if (gsHiValC5 <= gsSustainHC5)  intoPhaseC5_3();
            }
            break;
        case 4:
            gsHiValC5 -= gsDeltaC5;
            if (gsHiValC5 <= 0) {
                gsHiValC5 = 0;
                tempInitFilter(9,gsBaseFreqC5);
                gsPhaseC5 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityC5 == 0) gsValueC5 = (float)(gsHiValC5) / 3200.;
		else  gsValueC5 = (float)(-gsHiValC5) / 6400.;
		if   (gsValueC5 != value) {
			tempInitFilter(9,(uint32_t)(gsBaseFreqC5 * (32 + gsValueC5) / 32));
		}
    }

    if (gsPhaseC6)  {                    //controls FilterC6 frequency shift
        value = gsValueC6;
        switch (gsPhaseC6)  {
        case 1:
            gsHiValC6 += gsDeltaC6;
            if (gsHiValC6 >= gsLimitC6) intoPhaseC6_2();
            break;
        case 2:
            if (gsSignC6) {
                gsHiValC6 += gsDeltaC6;
                if (gsHiValC6 >= gsSustainHC6)  intoPhaseC6_3();
            }
            else  {
                gsHiValC6 -= gsDeltaC6;
                if (gsHiValC6 <= gsSustainHC6)  intoPhaseC6_3();
            }
            break;
        case 4:
            gsHiValC6 -= gsDeltaC6;
            if (gsHiValC6 <= 0) {
                gsHiValC6 = 0;
                tempInitFilter(10,gsBaseFreqC6);
                gsPhaseC6 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityC6 == 0) gsValueC6 = (float)(gsHiValC6) / 3200.;
		else  gsValueC6 = (float)(-gsHiValC6) / 6400.;
			if  (gsValueC6 != value) {
			tempInitFilter(10,(uint32_t)(gsBaseFreqC6 * (32 + gsValueC6) / 32));
		}
    }

    if (gsPhaseC7)  {                    //controls FilterC7 frequency shift
        value = gsValueC7;
        switch (gsPhaseC7)  {
        case 1:
            gsHiValC7 += gsDeltaC7;
            if (gsHiValC7 >= gsLimitC7) intoPhaseC7_2();
            break;
        case 2:
            if (gsSignC7) {
                gsHiValC7 += gsDeltaC7;
                if (gsHiValC7 >= gsSustainHC7)  intoPhaseC7_3();
            }
            else  {
                gsHiValC7 -= gsDeltaC7;
                if (gsHiValC7 <= gsSustainHC7)  intoPhaseC7_3();
            }
            break;
        case 4:
            gsHiValC7 -= gsDeltaC7;
            if (gsHiValC7 <= 0) {
                gsHiValC7 = 0;
                tempInitFilter(11,gsBaseFreqC7);
                gsPhaseC7 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityC7 == 0) gsValueC7 = (float)(gsHiValC7) / 3200.;
		else  gsValueC7 = (float)(-gsHiValC7) / 6400.;
		if   (gsValueC7 != value) {
			tempInitFilter(11,(uint32_t)(gsBaseFreqC7 * (32 + gsValueC7) / 32));
		}
    }

    if (gsPhaseC8)  {                    //controls FilterC8 frequency shift
        value = gsValueC8;
        switch (gsPhaseC8)  {
        case 1:
            gsHiValC8 += gsDeltaC8;
            if (gsHiValC8 >= gsLimitC8) intoPhaseC8_2();
            break;
        case 2:
            if (gsSignC8) {
                gsHiValC8 += gsDeltaC8;
                if (gsHiValC8 >= gsSustainHC8)  intoPhaseC8_3();
            }
            else  {
                gsHiValC8 -= gsDeltaC8;
                if (gsHiValC8 <= gsSustainHC8)  intoPhaseC8_3();
            }
            break;
        case 4:
            gsHiValC8 -= gsDeltaC8;
            if (gsHiValC8 <= 0) {
                gsHiValC8 = 0;
                tempInitFilter(12,gsBaseFreqC8);
                gsPhaseC8 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityC8 == 0) gsValueC8 = (float)(gsHiValC8) / 3200.;
		else  gsValueC8 = (float)(-gsHiValC8) / 6400.;
		if  (gsValueC8 != value) {
			tempInitFilter(12,(uint32_t)(gsBaseFreqC8 * (32 + gsValueC8) / 32));
		}
    }

    if (gsPhaseC9)  {                    //controls FilterC9 frequency shift
        value = gsValueC9;
        switch (gsPhaseC9)  {
        case 1:
            gsHiValC9 += gsDeltaC9;
            if (gsHiValC9 >= gsLimitC9) intoPhaseC9_2();
            break;
        case 2:
            if (gsSignC9) {
                gsHiValC9 += gsDeltaC9;
                if (gsHiValC9 >= gsSustainHC9)  intoPhaseC9_3();
            }
            else  {
                gsHiValC9 -= gsDeltaC9;
                if (gsHiValC9 <= gsSustainHC9)  intoPhaseC9_3();
            }
            break;
        case 4:
            gsHiValC9 -= gsDeltaC9;
            if (gsHiValC9 <= 0) {
                gsHiValC9 = 0;
                tempInitFilter(13,gsBaseFreqC9);
                gsPhaseC9 = 0;
            }
            break;
        default:
            break;
        }
		if (gsPolarityC9 == 0) gsValueC9 = (float)(gsHiValC9) / 3200.;
		else  gsValueC9 = (float)(-gsHiValC9) / 6400.;
		if  (gsValueC9 != value) {
			tempInitFilter(13,(uint32_t)(gsBaseFreqC9 * (32 + gsValueC9) / 32));
		}
    }

//Free assigned ADSR:
    if (gsPhaseFree)  {
        if (gsFreeAdsr == 2) {					//filter#0 Bandwidth/Resonance
           value = gsValueF0QF;
            switch (gsPhaseFree)  {
            case 1:
                gsHiValFree += gsDeltaFree;
                if (gsHiValFree >= gsLimitFree) intoPhaseFree_2();
                break;
            case 2:
                if (gsSignFree) {
                    gsHiValFree += gsDeltaFree;
                    if (gsHiValFree >= gsSustainHFree)  intoPhaseFree_3();
                }
                else  {
                    gsHiValFree -= gsDeltaFree;
                    if (gsHiValFree <= gsSustainHFree)  intoPhaseFree_3();
                }
                break;
            case 4:
                gsHiValFree -= gsDeltaFree;
                if (gsHiValFree <= 0) {
                    gsHiValFree = 0;
                    gsPhaseFree = 0;
                    gsPhaseF0QF = 0;
                    gfF0TempQF = gfRuntimeIIR[0].QFMOD;
                    tempInitFilter(0,gsBaseFreqF0);
                }
                break;
            default:
                break;
            }
			if ((gfACTIV_IIR[0].TYPE == 5) || (gfACTIV_IIR[0].TYPE == 6))  {
				if (gsPolarityFree == 0) gsValueF0QF = (float)(gsHiValFree) /2.;
				else  gsValueF0QF = (float)(-gsHiValFree) / 2.;
			}
			else  {
				if (gsPolarityFree==0) gsValueF0QF = (float)(gsHiValFree)/8192.;
				else  gsValueF0QF= (float)(-gsHiValFree) / 8192.;
			}
			if (gsValueF0QF != value) {
					gfF0TempQF = (32*gfRuntimeIIR[0].QFMOD + gsValueF0QF) / 32;
			}
        }
        else if (gsFreeAdsr == 3) {					//Noise filter#2 MidFreq
           value = gsValueF2M;
            switch (gsPhaseFree)  {
            case 1:
                gsHiValFree += gsDeltaFree;
                if (gsHiValFree >= gsLimitFree) intoPhaseFree_2();
                break;
            case 2:
                if (gsSignFree) {
                    gsHiValFree += gsDeltaFree;
                    if (gsHiValFree >= gsSustainHFree)  intoPhaseFree_3();
                }
                else  {
                    gsHiValFree -= gsDeltaFree;
                    if (gsHiValFree <= gsSustainHFree)  intoPhaseFree_3();
                }
                break;
            case 4:
                gsHiValFree -= gsDeltaFree;
                if (gsHiValFree <= 0) {
                    gsHiValFree = 0;
                    gsPhaseFree = 0;
                    tempInitFilter(2,(uint32_t)gfBaseFreqF2);
                }
                break;
            default:
                break;
            }
			if (gsPolarityFree == 0) gsValueF2M = (float)(gsHiValFree) / 3200.;
			else  gsValueF2M = (float)(-gsHiValFree) / 6400.;
			if  (gsValueF2M != value) {
			tempInitFilter(2,(uint32_t)(gfBaseFreqF2* (32.0+ gsValueF2M) / 32));
			}
        }
        else  {
            switch (gsPhaseFree)  {
            case 1:
                gsHiValFree += gsDeltaFree;
                if (gsHiValFree >= gsLimitFree) intoPhaseFree_2();
                break;
            case 2:
                if (gsSignFree) {
                    gsHiValFree += gsDeltaFree;
                    if (gsHiValFree >= gsSustainHFree)  intoPhaseFree_3();
                }
                else  {
                    gsHiValFree -= gsDeltaFree;
                    if (gsHiValFree <= gsSustainHFree)  intoPhaseFree_3();
                }
                break;
            case 4:
                gsHiValFree -= gsDeltaFree;
                if (gsHiValFree <= 0) {
                    gsHiValFree = 0;
                    gsValueFree = 0;
                    gsPhaseFree = 0;
					if (gsFreeAdsr == 1)  gsValueFMO2 = 0;
                }
                break;
            default:
                break;
            }
			if (gsPolarityFree == 0)  gsValueFree = gsHiValFree >> 6;
			else  gsValueFree = -(gsHiValFree >> 6);		//restore neg.sign

			if (gsFreeAdsr == 0) gsValueFMO2 = gsValueFree;	//Osc2 additional FM
			else if (gsFreeAdsr == 1)  gsValueFLB = gsValueFree; //Lfo2 Freq.Mod
			else if (gsFreeAdsr == 4)  gsValueChor = gsValueFree;	//Chorus
        }
    }
}

//  #########################################################################

